// *****************************************************************************
//
//
// divx.cpp
// 
//
// *****************************************************************************


#ifdef ENCODERSERVER
#include "../encoderserver/es.h"
#endif

#ifdef ENCODER
#include "../encoder/en.h"
#endif

#ifdef PLAYER
#include "../player/pl.h"
#endif


char * divxErrorStrings[] = 
{
	"No Error",
	"Failed To Open File",
	"Invalid File",
	"File Not Open",
	"Failed To Open Stream",
	"Failed To Get Stream Info",
	"Invalid Stream",
	"Failed To Read Header"
	"Failed To Create File",
	"Failed To Close File",
	"Failed To Write To File"
};

extern GLOBALS * globals;


// *****************************************************************************
// divxClass::divxClass()
//
// divxClass class constructor
// *****************************************************************************
divxClass::divxClass()
{

}

// *****************************************************************************
// divxClass::~divxClass()
//
// divxClass class destructor
// *****************************************************************************
divxClass::~divxClass()
{

}

// *****************************************************************************
// divxClass::fileInit()
//
// *****************************************************************************
void divxClass::init()
{
	int i;

	// Close and open streams
	fileSize = 0;
	headerChunkSize = 0;
	headerFilePos = 0;
	movieChunkSize = 0;
	movieFilePos=0;
	indexChunkSize = 0;
	indexFilePos = 0;
	indexXChunkSize = 0;
	indexXFilePos = 0;
	streamCount = 0;
	videoStreamCount=0;
	audioStreamCount=0;
	subtitleStreamCount=0;
	currentVideoStream=0;
	currentAudioStream=0;
	currentSubtitleStream=0;
	writeIndex = NULL;
	writeIndexX = NULL;

	fileOpen = false;
	InitializeCriticalSection(&fileLock);

	for(i=0;i<DIVXMAXSTREAMS;i++)
		{
		memset(&divxStreamHeader[i],0,sizeof(DIVXSTREAMHEADER));
		memset(&divxStreamName[i],0,sizeof(DIVXSTREAMNAME));
		divxStreamFormat[i]= NULL;
		divxStreamFormatSize[i]=0;
		divxStreamChapters[i] = NULL;
		divxStreamChaptersSize[i]=0;
		divxStreamIndex[i]= NULL;
		divxStreamByteCount[i]=0;
		divxStreamIndexCount[i]=0;
		divxStreamIndexPos[i]=0;
		divxStreamIndexSize[i]=0;
		divxStreamBlockPos[i]=0;
		videoStreamList[i]=0;
		audioStreamList[i]=0;
		subtitleStreamList[i]=0;
		writeSmoothBuffer[i]=NULL;
		writeSmoothHeadPos[i]=0;
		writeSmoothTailPos[i]=0;
		divxStreamSamples[i]=NULL;
		divxStreamSamplesSize[i]=0;
		divxStreamSamplesPos[i]=0;
		}
	writeSamplesChunk=false;

}

// *****************************************************************************
// divxClass::fileInit()
//
// *****************************************************************************
void divxClass::end()
{
	int i;

	streamCount = 0;

	for(i=0;i<DIVXMAXSTREAMS;i++)
		{
		if(divxStreamFormat[i])
			{
			delete [] divxStreamFormat[i];
			divxStreamFormat[i]=NULL;
			}
		if(divxStreamIndex[i])
			{
			delete [] divxStreamIndex[i];
			divxStreamIndex[i] =NULL;
			}
		if(divxStreamChapters[i])
			{
			delete [] divxStreamChapters[i];
			divxStreamChapters[i]=NULL;
			}
		if(divxStreamSamples[i])
			{
			free(divxStreamSamples[i]);
			divxStreamSamples[i]=NULL;
			}

		}

	if(writeIndex)
		{
		free(writeIndex);
		}

	if(writeIndexX)
		{
		free(writeIndexX);
		}

	if(fileOpen)
		{
		_close(fileHandle);
		}

}

// *****************************************************************************
// divxClass::fileOpen()
//
// *****************************************************************************
int divxClass::openRead(char * file)
{
	DWORD tag;
	unsigned int chunkSize;
	DWORD tag2;
	unsigned int chunkSize2;
	bool foundMovie;
	bool foundHeader;
	bool foundIndex;
	bool foundXIndex;
	strcpy(filename,file);
	__int64 tempFilePos;
	int ret;


	if(fileOpen)
		{
		return(divxErrorFailedToOpenFile);
		}
	fileOpen = false;

	fileHandle = _open( file, _O_BINARY | _O_RDONLY ,  0);
	if(fileHandle==-1)
		{
		return(divxErrorFailedToOpenFile);
		}
	//save the filename away for use by other functions
	strcpy(filename,file);

	// Verify it's a file we understand
	if(!getNextChunk(fileHandle,&tag,&chunkSize))
		{
		_close(fileHandle);
		return(divxErrorNotAValidFile);
		}
	
	if(tag!=divxRiffHeader)
		{
		_close(fileHandle);
		return(divxErrorNotAValidFile);
		}

	// Found RIFF	
	fileSize = chunkSize;

	if(!getSubChunk(fileHandle,&tag))
		{
		_close(fileHandle);
		return(divxErrorNotAValidFile);
		}

	if(tag!=divxAviHeader)
		{
		_close(fileHandle);
		return(divxErrorNotAValidFile);
		}

	// Found AVI - now find our chunks
	foundMovie = false;
	foundHeader = false;
	foundIndex = false;
	foundXIndex = false;
	for(;;)
		{
		if(!getNextChunk(fileHandle,&tag,&chunkSize))
			{
			break;
			}
		if(tag==divxListHeader)
			{
			// It's a list Chunk so process it
			if(!getSubChunk(fileHandle,&tag))
				{
				break;				
				}
			if(tag==divxHdrlHeader)
				{
				if(!getNextChunk(fileHandle,&tag2,&chunkSize2))
					{
					break;				
					}
				if(tag2==divxAvihHeader)
					{
					// Header chunk found
					tempFilePos = _telli64(fileHandle);

					if(_read(fileHandle,&divxHeader,sizeof(DIVXHEADER))!=sizeof(DIVXHEADER))
						{
						break;
						}
					headerFilePos = _telli64(fileHandle);
					headerChunkSize = chunkSize - sizeof(DIVXHEADER)-12;
					_lseeki64(fileHandle,tempFilePos-12,SEEK_SET);
					foundHeader = true;				
					}
				else
					{
					_lseek(fileHandle,-12,SEEK_CUR);			
					}
				}
			else if(tag==divxMoviHeader)
				{
				// Movie chunk found
				movieFilePos = _telli64(fileHandle);
				movieChunkSize = chunkSize-4;

				_lseek(fileHandle,-4,SEEK_CUR);
				foundMovie = true;
				}
			else
				{
				// Unknown chunk so just skip it
				_lseek(fileHandle,-4,SEEK_CUR);
				}
			}
		else if(tag==divxIdx1Header)
			{
			// Index chunk found
			indexFilePos = _telli64(fileHandle);
			indexChunkSize = chunkSize;
			foundIndex = true;
			}
		else if(tag==divxIdxxHeader)
			{
			// IndexX chunk found
			indexXFilePos = _telli64(fileHandle);
			indexXChunkSize = chunkSize;
			foundXIndex = true;
			}
			
		if(_lseek(fileHandle,chunkSize,SEEK_CUR)==-1)
			{
			break;
			}
		tempFilePos = _telli64(fileHandle);
		if(tempFilePos>=fileSize)
			{
			break;
			}
		}

	// Read all the stream data now
	if(!foundMovie || !foundHeader)
		{
		_close(fileHandle);
		return(divxErrorNone);
		}

	if(ret = readAllStreamData())
		{
		_close(fileHandle);
		return(ret);
		}

	if(foundIndex)
		{
		foundIndex = readIndexData();
		}
	if(!foundIndex)
		{
		//Build The index

		}

	if(foundXIndex)
		{
		foundXIndex = readIndexXData();
		}

	if(!foundXIndex)
		{
		//Build The index

		}

	fileOpen = true;

	return(divxErrorNone);

}

// *****************************************************************************
// divxClass::readAllStreamData()
//
// *****************************************************************************
int divxClass::readAllStreamData()
{
	unsigned int chunkOffset;
	DWORD tag;
	unsigned int chunkSize;
	int i;

	if(_lseeki64(fileHandle,headerFilePos,SEEK_SET)==-1)
		{
		return(divxErrorFailedToReadHeader);
		}
	// Now see if we can find the stream requested

	chunkOffset =0;
	streamCount=-1;

	for(;;)
		{
		if(!getNextChunk(fileHandle,&tag,&chunkSize))
			{
			return(divxErrorFailedToReadHeader);
			}
		chunkOffset+=8;
		if(tag==divxListHeader)
			{
			streamCount++;
			if(streamCount>DIVXMAXSTREAMS)
				{
				break;
				}
			// Read the stream info
			if(!getSubChunk(fileHandle,&tag))
				{
				return(divxErrorFailedToReadHeader);
				}
			chunkOffset+=4;
			if(tag==divxStrlHeader)
				{
				if(!getNextChunk(fileHandle,&tag,&chunkSize))
					{
					return(divxErrorFailedToReadHeader);
					}
				chunkOffset+=8;
				}
			}
		if(streamCount>=0)
			{
			switch(tag)
				{
				case divxStrhHeader:
					if(_read(fileHandle,&divxStreamHeader[streamCount],sizeof(DIVXSTREAMHEADER))!=sizeof(DIVXSTREAMHEADER))
						{
						return(divxErrorFailedToReadHeader);		
						}
					break;
				case divxStrfHeader:
					if(chunkSize<4096)
						{
						divxStreamFormat[streamCount] = new char [chunkSize];
						divxStreamFormatSize[streamCount] = chunkSize;
						if(_read(fileHandle,&divxStreamFormat[streamCount][0],chunkSize)!=(int)chunkSize)
							{
							return(divxErrorFailedToReadHeader);		
							}
						}
					break;
				case divxStrnHeader:
					if(chunkSize<=64)
						{
						if(_read(fileHandle,&divxStreamName[streamCount],chunkSize)!=(int)chunkSize)
							{
							return(divxErrorFailedToReadHeader);		
							}
						}
					break;
				case divxStrcHeader:
					if(chunkSize<4096)
						{
						divxStreamChapters[streamCount] = (DIVXSTREAMCHAPTERS *) new char [chunkSize];
						divxStreamChaptersSize[streamCount] = chunkSize;
						if(_read(fileHandle,&divxStreamChapters[streamCount][0],chunkSize)!=(int)chunkSize)
							{
							return(divxErrorFailedToReadHeader);		
							}
						}
					break;

				default:
					//Unknown chunk so skip it
					break;
				}
			}

		chunkOffset+=chunkSize;
		chunkOffset = (chunkOffset+1)&0xfffffffe;

		if(chunkOffset>=headerChunkSize)
			{
			break;
			}

		if(_lseeki64(fileHandle,headerFilePos + chunkOffset,SEEK_SET)==-1)
			{
			return(divxErrorFailedToReadHeader);
			}

		}

	//Last Increment the stream count
	streamCount++;
	videoStreamCount=0;
	audioStreamCount=0;
	subtitleStreamCount=0;
	for(i=0;i<streamCount;i++)
		{
		//find the first Video Stream
		if(divxStreamHeader[i].fccType == divxStreamtypeVIDEO)
			{
			videoStreamList[videoStreamCount] = i;
			videoStreamCount++;
			}
		if(divxStreamHeader[i].fccType == divxStreamtypeAUDIO)
			{
			audioStreamList[audioStreamCount] = i;
			audioStreamCount++;
			}
		if(divxStreamHeader[i].fccType == divxStreamtypeSUBTITLE)
			{
			subtitleStreamList[subtitleStreamCount] = i;
			subtitleStreamCount++;
			}
		}

	return(divxErrorNone);	
}

// *****************************************************************************
// divxClass::readIndexData()
//
// *****************************************************************************
bool divxClass::readIndexData()
{
	unsigned int chunkOffset;
	int i;
	int index;
	DIVXINDEX * sourceIndex;
	int pos;
	DIVXCHUNK header;

	divxIndexRelative = true;

	if(_lseeki64(fileHandle,indexFilePos,SEEK_SET)==-1)
		{
		return(false);
		}

	for(i=0;i<streamCount;i++)
		{
		divxStreamIndex[i] = (DIVXINDEX *) malloc(1 * sizeof(DIVXSINGLEINDEX));
		divxStreamIndexSize[i] = 1;
		divxStreamIndexPos[i] = 0;
		}

	chunkOffset = 0;

	sourceIndex = new DIVXINDEX [indexChunkSize];
	if(_read(fileHandle,sourceIndex,indexChunkSize)!= (int) indexChunkSize)
		{
		delete [] sourceIndex;
		return(false);
		}
	pos =0;


	for(pos = 0;pos<(int)(indexChunkSize/16);pos++)
		{
		if((sourceIndex->entry[pos].tag[2] == 'd' && sourceIndex->entry[pos].tag[3]=='c') || 
			(sourceIndex->entry[pos].tag[2] == 'd' && sourceIndex->entry[pos].tag[3]=='b'))
			break;
		}

	if(_lseeki64(fileHandle,sourceIndex->entry[pos].filePos,SEEK_SET)==-1)
		{
		delete [] sourceIndex;
		return(false);
		}

	if(_read(fileHandle,&header,sizeof(DIVXCHUNK))!= sizeof(DIVXCHUNK))
		{
		delete [] sourceIndex;
		return(false);
		}

	if(!memcmp(&sourceIndex->entry[pos].tag,&header.name,4) && sourceIndex->entry[pos].size == header.size)
		{
		divxIndexRelative = false;
		}

	pos =0;

	do{
		index = sourceIndex->entry[pos].tag[1] - 0x30;			//Note: we assume less than 8 streams of any one type

		if(index<streamCount)
			{
	
			if((sourceIndex->entry[pos].tag[2] == 'd' && sourceIndex->entry[pos].tag[3]=='c') || 
				(sourceIndex->entry[pos].tag[2] == 'd' && sourceIndex->entry[pos].tag[3]=='b'))
				{
				memcpy(&divxStreamIndex[index]->entry[divxStreamIndexPos[index]],&sourceIndex->entry[pos],sizeof(DIVXSINGLEINDEX));
				divxStreamIndexPos[index]++;
				if(divxStreamIndexPos[index]>=divxStreamIndexSize[index])
					{
					divxStreamIndexSize[index]+=8192;
					divxStreamIndex[index] = (DIVXINDEX *) realloc(divxStreamIndex[index], divxStreamIndexSize[index]* sizeof(DIVXSINGLEINDEX));
					}
				}
			else if(sourceIndex->entry[pos].tag[2] == 'w' && sourceIndex->entry[pos].tag[3]=='b')
				{
				memcpy(&divxStreamIndex[index]->entry[divxStreamIndexPos[index]],&sourceIndex->entry[pos],sizeof(DIVXSINGLEINDEX));
				divxStreamIndexPos[index]++;
				divxStreamByteCount[index]+=sourceIndex->entry[pos].size;
				if(divxStreamIndexPos[index]>=divxStreamIndexSize[index])
					{
					divxStreamIndexSize[index]+=8192;
					divxStreamIndex[index] = (DIVXINDEX *) realloc(divxStreamIndex[index], divxStreamIndexSize[index]* sizeof(DIVXSINGLEINDEX));
					}
				
				}

			}
		pos++;

		chunkOffset+=sizeof(DIVXSINGLEINDEX);
		}while(chunkOffset<indexChunkSize);

	delete [] sourceIndex;

	for(i=0;i<streamCount;i++)
		{
		if(divxStreamIndexPos[i])
			{
			divxStreamIndexCount[i] = divxStreamIndexPos[i];
			}
		}

	return(true);

}

// *****************************************************************************
// divxClass::readIndexXData()
//
// *****************************************************************************
bool divxClass::readIndexXData()
{
	unsigned int chunkOffset;
	int i;
	int index;
	DIVXINDEX * sourceIndex;
	int pos;

	if(_lseeki64(fileHandle,indexXFilePos,SEEK_SET)==-1)
		{
		return(false);
		}

	chunkOffset = 0;

	for(i=0;i<streamCount;i++)
		{
		divxStreamIndexPos[i]=0;
		}

	sourceIndex = new DIVXINDEX [indexXChunkSize];
	if(_read(fileHandle,sourceIndex,indexXChunkSize)!= (int) indexXChunkSize)
		{
		delete [] sourceIndex;
		return(false);
		}
	pos =0;

	do{
		index = sourceIndex->entry[pos].tag[1] - 0x30;			//Note: we assume less than 8 streams of any one type

		if(index<streamCount)
			{
			if((sourceIndex->entry[pos].tag[2] == 's' && sourceIndex->entry[pos].tag[3]=='b'))
				{
				memcpy(&divxStreamIndex[index]->entry[divxStreamIndexPos[index]],&sourceIndex->entry[pos],sizeof(DIVXSINGLEINDEX));
				divxStreamIndexPos[index]++;
				if(divxStreamIndexPos[index]>=divxStreamIndexSize[index])
					{
					divxStreamIndexSize[index]+=8192;
					divxStreamIndex[index] = (DIVXINDEX *) realloc(divxStreamIndex[index], divxStreamIndexSize[index]* sizeof(DIVXSINGLEINDEX));
					}
				}
			}
		pos++;

		chunkOffset+=sizeof(DIVXSINGLEINDEX);
		}while(chunkOffset<indexXChunkSize);

	delete [] sourceIndex;

	for(i=0;i<streamCount;i++)
		{
		// Copy any index pos data not already covered by the idx1
		if(divxStreamIndexPos[i])
			{
			divxStreamIndexCount[i] = divxStreamIndexPos[i];
			}
		}

	return(true);

}
// *****************************************************************************
// divxClass::getErrorString()
//
// *****************************************************************************
char * divxClass::getErrorString(int error)
{
	if(error< (sizeof( divxErrorStrings)/ sizeof(int)))
		{
		return(divxErrorStrings[error]);
		}
	else
		{
		return(NULL);
		}
}

// *****************************************************************************
// divxClass::getNextChunk()
//
// *****************************************************************************
bool divxClass::getNextChunk(int fileHandle, DWORD * chunkName, unsigned int * chunkSize)
{
	if(_read(fileHandle,chunkName,4)!=4)
		{
		return(false);
		}
	if(_read(fileHandle,chunkSize,4)!=4)
		{
		return(false);
		}

	return(true);
}

// *****************************************************************************
// divxClass::getSubChunk()
//
// *****************************************************************************
bool divxClass::getSubChunk(int fileHandle, DWORD * chunkName)
{

	if(_read(fileHandle,chunkName,4)!=4)
		{
		return(false);
		}
	return(true);
}

// *****************************************************************************
// divxClass::getVideoStreamCount()
//
// *****************************************************************************
int divxClass::getVideoStreamCount()
{
	return(videoStreamCount);
}

// *****************************************************************************
// divxClass::getAudioStreamCount()
//
// *****************************************************************************
int divxClass::getAudioStreamCount()
{
	return(audioStreamCount);
}

// *****************************************************************************
// divxClass::getSubtitleStreamCount()
//
// *****************************************************************************
int divxClass::getSubtitleStreamCount()
{
	return(subtitleStreamCount);
}

// *****************************************************************************
// divxClass::setCurrentVideoStream()
//
// *****************************************************************************
void divxClass::setCurrentVideoStream(int streamNum)
{
	if(streamNum<videoStreamCount)
		currentVideoStream = streamNum;
}

// *****************************************************************************
// divxClass::setCurrentAudioStream()
//
// *****************************************************************************
void divxClass::setCurrentAudioStream(int streamNum)
{
	if(streamNum<audioStreamCount)
		currentAudioStream = streamNum;
}

// *****************************************************************************
// divxClass::setCurrentSubtitleStream()
//
// *****************************************************************************
void divxClass::setCurrentSubtitleStream(int streamNum)
{
	if(streamNum<subtitleStreamCount)
		currentSubtitleStream = streamNum;
}

// *****************************************************************************
// divxClass::getVideoStreamInfo()
//
// *****************************************************************************
void divxClass::getVideoStreamInfo(int streamNum, DIVXVIDEOINFO * info)
{
	int index;
	double frameR;
	if(streamNum<videoStreamCount)
		{
		index = videoStreamList[streamNum];
		frameR = (double )divxStreamHeader[index].dwRate;
		frameR = frameR / (double)divxStreamHeader[index].dwScale;
		info->framerate = frameR;
		info->width = divxHeader.dwWidth;
		info->height = divxHeader.dwHeight;
		memcpy(&info->szName[0],&divxStreamName[index].name,64);
		}
}

// *****************************************************************************
// divxClass::getVideoStreamFormat()
//
// *****************************************************************************
void divxClass::getVideoStreamFormat(int streamNum, BITMAPINFO * bitmap)
{
	int index;
	if(streamNum<videoStreamCount)
		{
		index = videoStreamList[streamNum];
		memcpy(bitmap,&divxStreamFormat[index][0],divxStreamFormatSize[index]);
		}
}

// *****************************************************************************
// divxClass::getVideoStreamChaptersSize()
//
// *****************************************************************************
int divxClass::getVideoStreamChaptersSize(int streamNum)
{
	int index;
	if(streamNum<videoStreamCount)
		{
		index = videoStreamList[streamNum];
		return(divxStreamChaptersSize[index]);
		}
	return(0);
}

// *****************************************************************************
// divxClass::getVideoStreamChapters()
//
// *****************************************************************************
void divxClass::getVideoStreamChapters(int streamNum, DIVXSTREAMCHAPTERS * chapter)
{
	int index;
	if(streamNum<videoStreamCount)
		{
		index = videoStreamList[streamNum];
		memcpy(chapter,divxStreamChapters[index],divxStreamChaptersSize[index]);
		}
}

// *****************************************************************************
// divxClass::getAudioStreamFormat()
//
// *****************************************************************************
void divxClass::getAudioStreamFormat(int streamNum, MPEGLAYER3WAVEFORMAT * wave)
{
	int index;
	if(streamNum<audioStreamCount)
		{
		index = audioStreamList[streamNum];
		memcpy(wave,&divxStreamFormat[index][0],divxStreamFormatSize[index]);
		}
}

// *****************************************************************************
// divxClass::getAudioStreamName()
//
// *****************************************************************************
void divxClass::getAudioStreamName(int streamNum, char * name)
{
	int index;
	if(streamNum<audioStreamCount)
		{
		index = audioStreamList[streamNum];
		strcpy(name,divxStreamName[index].name);
		}
}

// *****************************************************************************
// divxClass::getSubtitleStreamName()
//
// *****************************************************************************
void divxClass::getSubtitleStreamName(int streamNum, char * name)
{
	int index;
	if(streamNum<subtitleStreamCount)
		{
		index = subtitleStreamList[streamNum];
		strcpy(name,divxStreamName[index].name);
		}
}

// *****************************************************************************
// divxClass::getSubtitleStreamFormat()
//
// *****************************************************************************
void divxClass::getSubtitleStreamFormat(int streamNum, DIVXSUBTITLEFORMAT * format)
{
	int index;

	if(streamNum<subtitleStreamCount)
		{
		index = subtitleStreamList[streamNum];
		memcpy(format,&divxStreamFormat[index][0],divxStreamFormatSize[index]);
		}
}

// *****************************************************************************
// divxClass::getVideoStreamFrameCount()
//
// *****************************************************************************
int divxClass::getVideoStreamFrameCount(int streamNum)
{
	int index;
	if(streamNum<videoStreamCount)
		{
		index = videoStreamList[streamNum];
		return(divxStreamIndexCount[index]);
		}
	else
		{
		return(0);
		}
}

// *****************************************************************************
// divxClass::getAudioStreamByteCount()
//
// *****************************************************************************
int divxClass::getAudioStreamByteCount(int streamNum)
{
	int index;
	if(streamNum<audioStreamCount)
		{
		index = audioStreamList[streamNum];
		return(divxStreamByteCount[index]);
		}
	else
		{
		return(0);
		}
}


// *****************************************************************************
// divxClass::getAudioStreamHeader()
//
// *****************************************************************************
bool divxClass::getAudioStreamHeader(int streamNum,DIVXSTREAMHEADER * header)
{
	int index;
	if(streamNum<audioStreamCount)
		{
		index = audioStreamList[streamNum];
		memcpy(header,&divxStreamHeader[index],sizeof(DIVXSTREAMHEADER));
		return(true);
		}
	else
		{
		return(false);
		}
}

// *****************************************************************************
// divxClass::getVideoStreamHeader()
//
// *****************************************************************************
bool divxClass::getVideoStreamHeader(int streamNum,DIVXSTREAMHEADER * header)
{
	int index;

	if(streamNum<videoStreamCount)
		{
		index = videoStreamList[streamNum];
		memcpy(header,&divxStreamHeader[index],sizeof(DIVXSTREAMHEADER));
		return(true);
		}
	else
		{
		return(false);
		}	
}

// *****************************************************************************
// divxClass::getVideoStreamNextFrameSize()
//
// *****************************************************************************
unsigned int divxClass::getVideoStreamNextFrameSize(int streamNum)
{
	int index;

	if(streamNum<videoStreamCount)
		{
		index = videoStreamList[streamNum];
		if((divxStreamIndexPos[index])<divxStreamIndexCount[index])
			{
			return(divxStreamIndex[index]->entry[divxStreamIndexPos[index]].size);
			}
		}	
	return(0);

}

// *****************************************************************************
// divxClass::getSubtitleStreamNextChunkSize()
//
// *****************************************************************************
unsigned int divxClass::getSubtitleStreamNextChunkSize(int streamNum)
{
	int index;

	if(streamNum<subtitleStreamCount)
		{
		index = subtitleStreamList[streamNum];
		if((divxStreamIndexPos[index])<divxStreamIndexCount[index])
			{
			return(divxStreamIndex[index]->entry[divxStreamIndexPos[index]].size);
			}
		}	
	return(0);

}

// *****************************************************************************
// divxClass::getSubtitleStreamNextChunkFrameStart()
//
// *****************************************************************************
int divxClass::getSubtitleStreamNextChunkFrameStart(int streamNum)
{
	int index;

	if(streamNum<subtitleStreamCount)
		{
		index = subtitleStreamList[streamNum];
		if((divxStreamIndexPos[index])<divxStreamIndexCount[index])
			{
			// In subtitle streams, the flag value is the starting frame number
			return(divxStreamIndex[index]->entry[divxStreamIndexPos[index]].flags);
			}
		}	
	return(0);

}

// *****************************************************************************
// divxClass::getVideoStreamNextKeyFrame()
//
// *****************************************************************************
int divxClass::getVideoStreamNextKeyFrame(int streamNum, int startPos)
{
	int index;
	int pos;
	if(streamNum<videoStreamCount)
		{
		index = videoStreamList[streamNum];
		if(startPos < divxStreamIndexCount[index])
			{
			for(pos=startPos;pos<divxStreamIndexCount[index];pos++)
				{
				if(divxStreamIndex[index]->entry[pos].flags)
					return(pos);
				}
			return(-1);
			}
		}
	return(0);
}

// *****************************************************************************
// divxClass::seekVideoStream()
//
// *****************************************************************************
void divxClass::seekVideoStream(int streamNum, int frame)
{
	int index;

	if(streamNum<videoStreamCount)
		{
		index = videoStreamList[streamNum];
		if(frame<divxStreamIndexCount[index])
			{
			divxStreamIndexPos[index] = frame;
			}
		}
}

// *****************************************************************************
// divxClass::seekSubtitleStream()
//
// *****************************************************************************
void divxClass::seekSubtitleStream(int streamNum, int frame)
{
	int index;
	int i;

	if(streamNum<subtitleStreamCount)
		{
		index = subtitleStreamList[streamNum];
		for(i=0;i<divxStreamIndexCount[index];i++)
			{
			// With subtitle streams, the flags value is the starting frame
			if(frame<=(int)divxStreamIndex[index]->entry[i].flags)
				{
				divxStreamIndexPos[index] = i;
				return;
				}
			}
		}
}


// *****************************************************************************
// divxClass::seekAudioStreamByPos()
//
// *****************************************************************************
void divxClass::seekAudioStreamByPos(int streamNum, double pos)
{
	int index;

	if(streamNum<audioStreamCount)
		{
		index = audioStreamList[streamNum];
		divxStreamIndexPos[index] = (int)((double)divxStreamIndexCount[index] * pos);
		divxStreamBlockPos[index] = 0;
		}

}

// *****************************************************************************
// divxClass::seekAudioStreamByBlock()
//
// *****************************************************************************
void divxClass::seekAudioStreamByBlock(int streamNum, int pos)
{
	int index;

	if(streamNum<audioStreamCount)
		{
		index = audioStreamList[streamNum];
		divxStreamIndexPos[index] = pos;
		divxStreamBlockPos[index] = 0;
		}
}

// *****************************************************************************
// divxClass::seekVideoStream()
//
// *****************************************************************************
void divxClass::seekAudioStream(int streamNum, unsigned int pos)
{
	int index;
	unsigned int bytePos;
	unsigned int blockStartBytePos;

	int i;
	
	if(streamNum<audioStreamCount)
		{
		index = audioStreamList[streamNum];
		if(pos<divxStreamByteCount[index])
			{
			blockStartBytePos = 0;
			bytePos = 0;
			for(i=0;i<divxStreamIndexCount[index];i++)
				{
				bytePos += divxStreamIndex[index]->entry[i].size;

				if(pos<bytePos)
					{
					//found it
					divxStreamIndexPos[index] = i;
					divxStreamBlockPos[index] = pos - blockStartBytePos;
					return;
					}
				blockStartBytePos = bytePos;
				}			
			}
		}
}


// *****************************************************************************
// divxClass::getVideoStreamNextFrame()
//
// *****************************************************************************
bool divxClass::getVideoStreamNextFrame(int streamNum, unsigned char * data)
{
	int index;
	__int64 filePos;
	int readSize;

	if(streamNum<videoStreamCount)
		{
		index = videoStreamList[streamNum];
		if(divxStreamIndexPos[index]>=divxStreamIndexCount[index])
			return(false);

		EnterCriticalSection(&fileLock);
		//do read
		filePos = divxStreamIndex[index]->entry[divxStreamIndexPos[index]].filePos;
		if(divxIndexRelative)
			{
			filePos += movieFilePos + 4;
			}
		else
			{
			filePos += 8;
			}

		if(_lseeki64(fileHandle,filePos,SEEK_SET)==-1)
			{
			return(false);
			}
		//Early out for zero size frames
		if(divxStreamIndex[index]->entry[divxStreamIndexPos[index]].size==0)
			{
			LeaveCriticalSection(&fileLock);
			divxStreamIndexPos[index]++;
			return(true);
			}

		readSize = divxStreamIndex[index]->entry[divxStreamIndexPos[index]].size;
		if(_read(fileHandle,data,readSize)!= readSize)
			{
			return(false);
			}
		
		LeaveCriticalSection(&fileLock);
		divxStreamIndexPos[index]++;
		}
	return(true);
}

// *****************************************************************************
// divxClass::getSubtitleStreamNextChunk()
//
// *****************************************************************************
bool divxClass::getSubtitleStreamNextChunk(int streamNum, unsigned char * data)
{
	int index;
	__int64 filePos;
	int readSize;

	if(streamNum<subtitleStreamCount)
		{
		index = subtitleStreamList[streamNum];
		if(divxStreamIndexPos[index]>=divxStreamIndexCount[index])
			return(false);

		EnterCriticalSection(&fileLock);
		//do read
		filePos = divxStreamIndex[index]->entry[divxStreamIndexPos[index]].filePos;

		// index X entries are always relative to the movie start pos
		filePos += movieFilePos + 4;

		if(_lseeki64(fileHandle,filePos,SEEK_SET)==-1)
			{
			return(false);
			}

		readSize = divxStreamIndex[index]->entry[divxStreamIndexPos[index]].size;
		if(_read(fileHandle,data,readSize)!= readSize)
			{
			return(false);
			}
		
		LeaveCriticalSection(&fileLock);
		divxStreamIndexPos[index]++;
		}
	return(true);
}

// *****************************************************************************
// divxClass::getAudioStreamBytes()
//
// *****************************************************************************
int divxClass::getAudioStreamBytes(int streamNum, unsigned char * data, int numBytes)
{

	int index;
	__int64 filePos;
	int bytesRead;
	int bytesToRead;
	int bytesNeeded;
	int posIndex;

	if(streamNum<audioStreamCount)
		{
		index = audioStreamList[streamNum];
		bytesRead = 0;
		bytesNeeded = numBytes;

		//do read
		posIndex = divxStreamIndexPos[index];

		if(posIndex>=divxStreamIndexCount[index])
			return(0);

		for(;;)
			{
			filePos = divxStreamIndex[index]->entry[posIndex].filePos;

			if(divxIndexRelative)
				{
				filePos += movieFilePos + 4;
				}
			else
				{
				filePos += 8;
				}

			filePos +=divxStreamBlockPos[index]; 
			
			bytesToRead = divxStreamIndex[index]->entry[posIndex].size;
			bytesToRead -=divxStreamBlockPos[index]; 
			if(bytesToRead)
				{
				if(bytesToRead>bytesNeeded)
					{
					divxStreamBlockPos[index] +=bytesNeeded;
					bytesToRead = bytesNeeded;
					}
				else
					{
					divxStreamBlockPos[index] =0;
					}

				bytesNeeded-= bytesToRead;

				EnterCriticalSection(&fileLock);

				if(_lseeki64(fileHandle, filePos,SEEK_SET)==-1)
					{
					LeaveCriticalSection(&fileLock);
					divxStreamIndexPos[index] = posIndex;
					return(bytesRead);
					}

				if(_read(fileHandle,data+bytesRead,bytesToRead)!= bytesToRead)
					{
					LeaveCriticalSection(&fileLock);
					divxStreamIndexPos[index] = posIndex;
					return(bytesRead);
					}

				LeaveCriticalSection(&fileLock);

				bytesRead +=bytesToRead;	
				}

			if(!divxStreamBlockPos[index])
				{
				posIndex++;
				if(posIndex>=divxStreamIndexCount[index])
					{
					divxStreamIndexPos[index] = posIndex;
					break;
					}
				}

			if(!bytesNeeded)
				{
				divxStreamIndexPos[index] = posIndex;
				break;
				}
			}
		
		return(bytesRead);
		}
	return(0);
}

// *****************************************************************************
// divxClass::createFile()
//
// *****************************************************************************
int divxClass::createFile(char * file, bool mode, bool smooth, int headerReserve)
{
// Mode is used for network encode mode
	unsigned char * pad;

	if(fileOpen)
		{
		return(divxErrorFailedToCreateFile);
		}
	fileOpen = false;

	fileHandle = _open( file, _O_BINARY | _O_CREAT | _O_TRUNC | _O_WRONLY,  _S_IREAD | _S_IWRITE);
	if(fileHandle==-1)
		{
		return(divxErrorFailedToCreateFile);
		}
	//save the filename away for use by other functions
	strcpy(filename,file);

	pad = new unsigned char [headerReserve];
	memset(pad,0,headerReserve);
	
	if(_write(fileHandle,pad,headerReserve)!=headerReserve)
		{
		delete [] pad;
		return(divxErrorFailedToOpenFile);	
		}
	delete [] pad;

	headerSize = headerReserve;

	movieFilePos = _telli64(fileHandle);
	movieChunkSize = 0;

	// create index structure
	writeIndexSize = 512000;
	writeIndexPos = 0;
	writeIndex = (DIVXINDEX *) malloc(writeIndexSize * sizeof(DIVXSINGLEINDEX));

	writeIndexX = (DIVXINDEX *) malloc(1 * sizeof(DIVXSINGLEINDEX));
	writeIndexXSize = 1;
	writeIndexXPos = 0;

	writeFilePos = 4;
	writeSmooth = smooth;

	writeHeaderSetup = false;

	if(DIVXUSELAZYWRITE)
		{
		lazyWriteStart = new unsigned char [DIVXLAZYWRITEBUFFERSIZE];
		lazyWriteData = lazyWriteStart;
		lazyWriteBytesUsed = 0;
		lazyWriteSize = DIVXLAZYWRITEBUFFERSIZE;
		}
	return(false);
}

// *****************************************************************************
// divxClass::createFile()
//
// *****************************************************************************
int divxClass::closeFile()
{
	int size;
	DIVXCHUNK header;
	DWORD subChunkName;	
	__int64 headerListFilePos;
	int index;
	__int64 headerListEndFilePos;
//	int vbrSize;
//	char * fillPtr;

	if(!lazyWriteFlush(fileHandle))
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	size = writeIndexPos * sizeof(DIVXSINGLEINDEX);

	indexFilePos = _telli64(fileHandle)+8;
	indexChunkSize = size;

	// Write Index Header, Index Size, and Index Data

	header.name = divxIdx1Header;
	header.size = size;

	if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	if(_write(fileHandle,writeIndex,size)!=size)
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	// If any X index data, write it out.
	if(writeIndexXPos>0)
		{
		size = writeIndexXPos * sizeof(DIVXSINGLEINDEX);

		indexXFilePos = _telli64(fileHandle)+8;
		indexXChunkSize = size;

		// Write Index Header, Index Size, and Index Data

		header.name = divxIdxxHeader;
		header.size = size;

		if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
			{
			_close(fileHandle);
			return(divxErrorFailedToCloseFile);	
			}

		if(_write(fileHandle,writeIndexX,size)!=size)
			{
			_close(fileHandle);
			return(divxErrorFailedToCloseFile);	
			}
		}
/*
	if(writeSamplesChunk)
		{

		vbrSize = 0;

		for(index=0;index<streamCount;index++)
			{
			vbrSize +=divxStreamSamplesPos[index]*4;
			}

		header.name = divxListHeader;
		header.size = vbrSize + 4 + 8;

		if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
			{
			_close(fileHandle);
			return(divxErrorFailedToCloseFile);	
			}
		
		headerListFilePos = _telli64(fileHandle);

		subChunkName = divxVbrlHeader;

		if(_write(fileHandle,&subChunkName,sizeof(DWORD))!=sizeof(DWORD))
			{
			_close(fileHandle);
			return(divxErrorFailedToCloseFile);	
			}

		for(index=0;index<streamCount;index++)
			{
			// If stream has a sample pos that's not zero, then we need to write the data out

			if(divxStreamSamplesPos[index])
				{
				header.name = divxVbraHeader;
				header.size = divxStreamSamplesPos[index]*4;
				
				if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
					{
					_close(fileHandle);
					return(divxErrorFailedToCloseFile);	
					}

				if(_write(fileHandle,divxStreamSamples[index],divxStreamSamplesPos[index]*4)!=(divxStreamSamplesPos[index]*4))
					{
					_close(fileHandle);
					return(divxErrorFailedToCloseFile);	
					}
				}
			}
			
		}
	*/

	fileSize = (unsigned int) _telli64(fileHandle);

	// Write out RIFF header, AVI header and HDRL Header

	if(_lseeki64(fileHandle, 0 ,SEEK_SET)==-1)
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	header.name = divxRiffHeader;
	header.size = fileSize-8;

	if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	subChunkName = divxAviHeader;

	if(_write(fileHandle,&subChunkName,sizeof(DWORD))!=sizeof(DWORD))
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	header.name = divxListHeader;
	header.size = 0;

	if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	headerListFilePos = _telli64(fileHandle);

	subChunkName = divxHdrlHeader;

	if(_write(fileHandle,&subChunkName,sizeof(DWORD))!=sizeof(DWORD))
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	header.name = divxAvihHeader;
	header.size = sizeof(DIVXHEADER);
	
	if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}


	divxHeader.dwStreams = streamCount;
	divxHeader.dwTotalFrames = divxStreamIndexCount[writeHeaderVideoIndex];

	if(_write(fileHandle,&divxHeader,sizeof(DIVXHEADER))!=sizeof(DIVXHEADER))
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	// Write out stream headers and header size
	// 
	for(index=0;index<streamCount;index++)
		{
		// Only write out streams that wrote out one chunk or more. This is to eliminate 
		// subtitle streams that never wrote any subtitle data
		if(divxStreamIndexCount[index])
			{
			header.name = divxListHeader;
			header.size = 4;

			header.size +=sizeof(DIVXSTREAMHEADER)+8;
			header.size +=divxStreamFormatSize[index]+8;
			
			if(strlen(divxStreamName[index].name))
				{
				header.size +=strlen(divxStreamName[index].name)+1+8;
				if((strlen(divxStreamName[index].name)+1)&0x00000001)
					{
					header.size++;
					}
				}

			if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
				{
				_close(fileHandle);
				return(divxErrorFailedToCloseFile);	
				}

			subChunkName = divxStrlHeader;

			if(_write(fileHandle,&subChunkName,sizeof(DWORD))!=sizeof(DWORD))
				{
				_close(fileHandle);
				return(divxErrorFailedToCloseFile);	
				}

			header.name = divxStrhHeader;
			header.size = sizeof(DIVXSTREAMHEADER);
			
			if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
				{
				_close(fileHandle);
				return(divxErrorFailedToCloseFile);	
				}

			if(_write(fileHandle,&divxStreamHeader[index],sizeof(DIVXSTREAMHEADER))!=sizeof(DIVXSTREAMHEADER))
				{
				_close(fileHandle);
				return(divxErrorFailedToCloseFile);	
				}

			header.name = divxStrfHeader;
			header.size = divxStreamFormatSize[index];
			
			if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
				{
				_close(fileHandle);
				return(divxErrorFailedToCloseFile);	
				}

			if(_write(fileHandle,divxStreamFormat[index],divxStreamFormatSize[index])!=divxStreamFormatSize[index])
				{
				_close(fileHandle);
				return(divxErrorFailedToCloseFile);	
				}

			header.name = divxStrnHeader;
			header.size = strlen(divxStreamName[index].name);
			
			if(header.size)
				{
				header.size++;

				if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
					{
					_close(fileHandle);
					return(divxErrorFailedToCloseFile);	
					}

				if(_write(fileHandle,&divxStreamName[index],header.size)!=(int) header.size)
					{
					_close(fileHandle);
					return(divxErrorFailedToCloseFile);	
					}
				
				// If name was odd number of bytes, then add an extra byte to even us up.

				if(header.size&0x00000001)
					{
					// Write one more byte to even up the stream
					size = 0;
					if(_write(fileHandle,&size,1)!=1)
						{
						return(divxErrorFailedToCloseFile);	
						}
					}
				}

			header.name = divxStrcHeader;
			header.size = divxStreamChaptersSize[index];

			if(header.size)
				{			
				if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
					{
					_close(fileHandle);
					return(divxErrorFailedToCloseFile);	
					}

				if(_write(fileHandle,divxStreamChapters[index],divxStreamChaptersSize[index])!=divxStreamChaptersSize[index])
					{
					_close(fileHandle);
					return(divxErrorFailedToCloseFile);	
					}
				}
			/*
			header.name = divxJunkHeader;
			header.size = 0x100;

			if(header.size)
				{			
				if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
					{
					_close(fileHandle);
					return(divxErrorFailedToCloseFile);	
					}

				fillPtr = new char [header.size];
				memset(fillPtr,0,header.size);

				if(_write(fileHandle,fillPtr,header.size)!=(int)header.size)
					{
					delete [] fillPtr;
					_close(fileHandle);
					return(divxErrorFailedToCloseFile);	
					}
				delete [] fillPtr;
				}
			*/
			}
		}

	headerListEndFilePos = _telli64(fileHandle);

	// Write the Junk Header
	header.name = divxJunkHeader;
	header.size = (DWORD)(headerSize - (headerListEndFilePos+20));

	if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	// Correct the Header size

	if(_lseeki64(fileHandle, headerListFilePos -4,SEEK_SET)==-1)
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	size = (int) (headerListEndFilePos - headerListFilePos);

	if(_write(fileHandle,&size,4)!=4)
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	// Write out the Movie LIST ,size and movi header
	if(_lseeki64(fileHandle, headerSize -12,SEEK_SET)==-1)
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	header.name = divxListHeader;
	header.size = writeFilePos;

	if(_write(fileHandle,&header,sizeof(DIVXCHUNK))!=sizeof(DIVXCHUNK))
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}

	subChunkName = divxMoviHeader;

	if(_write(fileHandle,&subChunkName,4)!=4)
		{
		_close(fileHandle);
		return(divxErrorFailedToCloseFile);	
		}


	// Cleanup now that the encode is done.

	_close(fileHandle);
	fileHandle = NULL;

	if(DIVXUSELAZYWRITE)
		{
		delete [] lazyWriteStart;
		}
	free(writeIndex);
	free(writeIndexX);

	writeIndex = NULL;
	writeIndexX = NULL;

	return(false);


}

// *****************************************************************************
// divxClass::createAudioStream()
//
// *****************************************************************************
int divxClass::createSubtitleStream(DIVXSTREAMINFO * info)
{
	int index;

	index = streamCount;
	streamCount++;
	divxStreamIndexCount[index] = 0;
	divxStreamByteCount[index] = 0;
	divxStreamFormat[index] = NULL;
	divxStreamFormatSize[index] = 0;

	divxStreamHeader[index].fccType = info->fccType;
	divxStreamHeader[index].fccHandler = 0;
	divxStreamHeader[index].dwFlags = 0;
	divxStreamHeader[index].dwPriority = 0;
	divxStreamHeader[index].dwInitialFrames = 0;
	divxStreamHeader[index].dwScale = 0;
	divxStreamHeader[index].dwRate = 0;
	divxStreamHeader[index].dwStart = 0;
	divxStreamHeader[index].dwLength = 0;
	divxStreamHeader[index].dwSuggestedBufferSize = 0;
	divxStreamHeader[index].dwQuality = 0;
	divxStreamHeader[index].dwSampleSize = 0;
	divxStreamHeader[index].rcFrame.top = 0;
	divxStreamHeader[index].rcFrame.bottom = 0;
	divxStreamHeader[index].rcFrame.left = 0;
	divxStreamHeader[index].rcFrame.right = 0;
	
	strcpy(divxStreamName[index].name,info->szName);

	return(index);


}

// *****************************************************************************
// divxClass::setSubtitleStreamFormat()
//
// *****************************************************************************
int divxClass::setSubtitleStreamFormat(int streamNum, DIVXSUBTITLEFORMAT * format)
{
	divxStreamFormat[streamNum] = new char [sizeof(DIVXSUBTITLEFORMAT)];
	divxStreamFormatSize[streamNum] = sizeof(DIVXSUBTITLEFORMAT);
	memcpy(&divxStreamFormat[streamNum][0],format,sizeof(DIVXSUBTITLEFORMAT));

	return(true);
}
// *****************************************************************************
// divxClass::createAudioStream()
//
// *****************************************************************************
int divxClass::createAudioStream(DIVXSTREAMINFO * info, MPEGLAYER3WAVEFORMAT * wave, int formatLen)
{
	int index;

	index = streamCount;
	streamCount++;
	divxStreamIndexCount[index] = 0;
	divxStreamByteCount[index] = 0;
	divxStreamFormat[index] = new char [formatLen];
	divxStreamFormatSize[index] = formatLen;

	divxStreamSamples[index] = (int *) malloc(4);
	divxStreamSamplesSize[index] = 1;
	divxStreamSamplesPos[index] = 0;

	audioStreamFixup(index,info,wave,formatLen);

	if(writeSmooth)
		{
		writeSmoothBuffer[index] = new unsigned char [1000000];
		writeSmoothHeadPos[index] = 0;
		writeSmoothTailPos[index]=0;
		}

	return(index);
}
// *****************************************************************************
// divxClass::audioStreamFixup()
//
// *****************************************************************************
void divxClass::audioStreamFixup(int streamNum, DIVXSTREAMINFO * info, MPEGLAYER3WAVEFORMAT * wave, int formatLen)
{
	int index;

	index = streamNum;

	memcpy(&divxStreamFormat[index][0],wave,formatLen);

	divxStreamHeader[index].fccType = info->fccType;
	divxStreamHeader[index].fccHandler = info->fccHandler;
	divxStreamHeader[index].dwFlags = info->dwFlags;
	divxStreamHeader[index].dwPriority = info->dwPriority;
	divxStreamHeader[index].dwInitialFrames = info->dwInitialFrames;
	divxStreamHeader[index].dwScale = info->dwScale;
	divxStreamHeader[index].dwRate = info->dwRate;
	divxStreamHeader[index].dwStart = 0;
	divxStreamHeader[index].dwLength = info->dwLength;
	divxStreamHeader[index].dwSuggestedBufferSize = info->dwSuggestedBufferSize;
	divxStreamHeader[index].dwQuality = info->dwQuality;
	divxStreamHeader[index].dwSampleSize = info->dwSampleSize;
	divxStreamHeader[index].rcFrame.top = (unsigned short) info->rcFrame.top;
	divxStreamHeader[index].rcFrame.bottom = (unsigned short) info->rcFrame.bottom;
	divxStreamHeader[index].rcFrame.left = (unsigned short) info->rcFrame.left;
	divxStreamHeader[index].rcFrame.right = (unsigned short) info->rcFrame.right;
	
	strcpy(divxStreamName[index].name,info->szName);

}
// *****************************************************************************
// divxClass::createVideoStream()
//
// *****************************************************************************
int divxClass::createVideoStream(DIVXSTREAMINFO * info,BITMAPINFOHEADER * bitmap, int formatLen)
{
	int index;
	double framerate;

	index = streamCount;
	streamCount++;

	divxStreamIndexCount[index] = 0;
	divxStreamByteCount[index] = 0;
	divxStreamFormat[index] = new char [formatLen];
	divxStreamFormatSize[index] = formatLen;
	memcpy(&divxStreamFormat[index][0],bitmap,formatLen);

	divxStreamHeader[index].fccType = info->fccType;
	divxStreamHeader[index].fccHandler = info->fccHandler;
	divxStreamHeader[index].dwFlags = info->dwFlags;
	divxStreamHeader[index].dwPriority = info->dwPriority;
	divxStreamHeader[index].dwInitialFrames = info->dwInitialFrames;
	divxStreamHeader[index].dwScale = info->dwScale;
	divxStreamHeader[index].dwRate = info->dwRate;
	divxStreamHeader[index].dwStart = 0;
	divxStreamHeader[index].dwLength = 0;
	divxStreamHeader[index].dwSuggestedBufferSize = info->dwSuggestedBufferSize;
	divxStreamHeader[index].dwQuality = info->dwQuality;
	divxStreamHeader[index].dwSampleSize = info->dwSampleSize;
	divxStreamHeader[index].rcFrame.top = (unsigned short) info->rcFrame.top;
	divxStreamHeader[index].rcFrame.bottom = (unsigned short) info->rcFrame.bottom;
	divxStreamHeader[index].rcFrame.left = (unsigned short) info->rcFrame.left;
	divxStreamHeader[index].rcFrame.right = (unsigned short) info->rcFrame.right;

	strcpy(divxStreamName[index].name,info->szName);

	if(!writeHeaderSetup)
		{
		framerate = (double) info->dwRate;
		framerate = framerate / (double)info->dwScale;

		memset(&divxHeader,0,sizeof(DIVXHEADER));
		divxHeader.dwFlags = 0x0810;
		divxHeader.dwHeight = bitmap->biHeight;
		divxHeader.dwInitialFrames = info->dwInitialFrames;
		divxHeader.dwMaxBytesPerSec = 650000;
		divxHeader.dwMicroSecPerFrame = (DWORD) (1000000.0f / framerate);
		divxHeader.dwStreams = 0;
		divxHeader.dwSuggestedBufferSize = bitmap->biSizeImage;
		divxHeader.dwTotalFrames = 0;
		divxHeader.dwWidth = bitmap->biWidth;

		writeHeaderVideoIndex = index;
		writeHeaderSetup = true;
		}
	return(index);

}

// *****************************************************************************
// divxClass::setVideoStreamChapters()
//
// *****************************************************************************
void divxClass::setVideoStreamChapters(int streamNum, DIVXSTREAMCHAPTERS * chapterData, int size)
{
	int index;

	index = streamNum;

	if(divxStreamChapters[index])
		{
		delete [] divxStreamChapters[index];
		}

	divxStreamChaptersSize[index]=size;
	divxStreamChapters[index] = (DIVXSTREAMCHAPTERS *) new char [size];

	memcpy(divxStreamChapters[index],chapterData,size);

}
// *****************************************************************************
// divxClass::writeVideoStreamFrame()
//
// *****************************************************************************
int divxClass::writeVideoStreamFrame(int streamNum, DWORD flags, unsigned char * data, int size)
{
	char tag[4];
	int index;
	DWORD tempFlags;
	int tempSize;
	DIVXINDEX * tempIndex;

	index = streamNum;

	tag[0] = 0x30;
	tag[1] = 0x30 + streamNum;

	if(divxStreamIndexCount[index] == 0)
		{
		tag[2] = 'd';
		tag[3] = 'b';
		}
	else
		{
		tag[2] = 'd';
		tag[3] = 'c';
		}

	if(lazyWrite(fileHandle,(unsigned char *)&tag,4)!=4)
		{
		return(-1);	
		}
	
	tempSize = size;

	if(lazyWrite(fileHandle,(unsigned char *)&tempSize,4)!=4)
		{
		return(-1);	
		}
	if(size)
		{
		if(lazyWrite(fileHandle,data,size)!=size)
			{
			return(-1);	
			}
		}
	if(flags & DIVX_KEYFRAME)
		{
		tempFlags = DIVX_KEYFRAME;
		}
	else
		{
		tempFlags = 0;
		}

	writeIndex->entry[writeIndexPos].tag[0] = tag[0];
	writeIndex->entry[writeIndexPos].tag[1] = tag[1];
	writeIndex->entry[writeIndexPos].tag[2] = tag[2];
	writeIndex->entry[writeIndexPos].tag[3] = tag[3];
	writeIndex->entry[writeIndexPos].flags = tempFlags;
	writeIndex->entry[writeIndexPos].size = size;
	writeIndex->entry[writeIndexPos].filePos = writeFilePos;

	divxStreamIndexCount[index]++;
	divxStreamHeader[index].dwLength++;
	divxStreamByteCount[index]+=size;
	writeIndexPos++;
	if(writeIndexPos>=writeIndexSize)
		{
		writeIndexSize+=512000;
		tempIndex = (DIVXINDEX *) realloc(writeIndex, writeIndexSize * sizeof(DIVXSINGLEINDEX));
		if(tempIndex == NULL)
			{
			logPrintf("Failed to Realloc the index memory\n");
			tempIndex = (DIVXINDEX *) malloc(writeIndexSize * sizeof(DIVXSINGLEINDEX));
			if(tempIndex == NULL)
				{
				logPrintf("Failed to Malloc more index Memory\n");
				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage13");
				globals->translate->translate(messTitle, "VidomiMessage13Title");

				vidomiErrorBox(NULL,messText,messTitle, MB_OK);
				return(-1);
				}
			else
				{
				memcpy(tempIndex,writeIndex,writeIndexPos * sizeof(DIVXSINGLEINDEX));
				}
			}
		writeIndex = tempIndex;

		}

	writeFilePos+=size+8;
	if(writeFilePos&0x00000001)
	{
	// Write one more byte to even up the stream
	tag[0] = 0;
	if(lazyWrite(fileHandle,(unsigned char *)&tag,1)!=1)
		{
		return(-1);	
		}
	writeFilePos++;
	}

	if(writeSmooth)
		{
		// Write audio data from all the smooth buffers if it's time
		}	

	return(size);
}

// *****************************************************************************
// divxClass::writeAudioStreamBytes()
//
// *****************************************************************************
int divxClass::writeAudioStreamBytes(int streamNum, unsigned char * data, int size)
{
	char tag[4];
	int index;
	int tempSize;
	DIVXINDEX * tempIndex;


	index = streamNum;

	if(writeSmooth)
		{
		//add the bytes to the correct buffer
		//return back as if they were written to disk
		}


	tag[0] = 0x30;
	tag[1] = 0x30 + streamNum;

	tag[2] = 'w';
	tag[3] = 'b';

	if(lazyWrite(fileHandle,(unsigned char *)&tag,4)!=4)
		{
		return(false);	
		}

	tempSize = size;

	if(lazyWrite(fileHandle,(unsigned char *)&tempSize,4)!=4)
		{
		return(false);	
		}

	if(lazyWrite(fileHandle,data,size)!=size)
		{
		return(false);	
		}

	writeIndex->entry[writeIndexPos].tag[0] = tag[0];
	writeIndex->entry[writeIndexPos].tag[1] = tag[1];
	writeIndex->entry[writeIndexPos].tag[2] = tag[2];
	writeIndex->entry[writeIndexPos].tag[3] = tag[3];
	writeIndex->entry[writeIndexPos].flags = DIVX_KEYFRAME;
	writeIndex->entry[writeIndexPos].size = size;
	writeIndex->entry[writeIndexPos].filePos = writeFilePos;

	divxStreamIndexCount[index]++;
	divxStreamHeader[index].dwLength++;
	divxStreamByteCount[index]+=size;
	writeIndexPos++;
	if(writeIndexPos>=writeIndexSize)
		{
		writeIndexSize+=512000;
		tempIndex = (DIVXINDEX *) realloc(writeIndex, writeIndexSize * sizeof(DIVXSINGLEINDEX));
		if(tempIndex == NULL)
			{
			logPrintf("Failed to Realloc the index memory\n");
			tempIndex = (DIVXINDEX *) malloc(writeIndexSize * sizeof(DIVXSINGLEINDEX));
			if(tempIndex == NULL)
				{
				logPrintf("Failed to Malloc more index Memory\n");
				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage14");
				globals->translate->translate(messTitle, "VidomiMessage14Title");

				vidomiErrorBox(NULL,messText,messTitle, MB_OK);
				return(false);
				}
			else
				{
				memcpy(tempIndex,writeIndex,writeIndexPos * sizeof(DIVXSINGLEINDEX));
				}
			}
		writeIndex = tempIndex;
		}

	writeFilePos+=size+8;
	if(writeFilePos&0x00000001)
		{
		// Write one more byte to even up the stream
		tag[0] = 0;
		if(lazyWrite(fileHandle,(unsigned char *)&tag,1)!=1)
			{
			return(false);	
			}
		writeFilePos++;
		}

	return(size);


}

// *****************************************************************************
// divxClass::writeAudioStreamBytes()
//
// *****************************************************************************
int divxClass::writeAudioStreamBytesEnhanced(int index, unsigned char * data, int size, int numSamples)
{

	divxStreamSamples[index][divxStreamSamplesPos[index]] = numSamples;
	divxStreamSamplesPos[index]++;
	if(divxStreamSamplesPos[index]==divxStreamSamplesSize[index])
		{
		divxStreamSamplesSize[index]+=512000;
		divxStreamSamples[index] = (int *) realloc(divxStreamSamples[index], divxStreamSamplesSize[index] * 4);
		}

	writeSamplesChunk =true;

	return(writeAudioStreamBytes(index,data,size));
}


// *****************************************************************************
// divxClass::writeSubtitleStreamBytes()
//
// *****************************************************************************
int divxClass::writeSubtitleStreamData(int streamNum, DIVXSUBTITLEHEADER * header, unsigned char * data, int dataSize)
{
	char tag[4];
	int index;
	int tempSize;
	DIVXINDEX * tempIndex;
	int size;

	// Write out the subtitle header bytes
	// Write out the subtitle data
	// Add subtitle entry to subtitle index

	index = streamNum;
	size =  sizeof(DIVXSUBTITLEHEADER) + dataSize;

	if(writeSmooth)
		{
		//add the bytes to the correct buffer
		//return back as if they were written to disk
		}


	tag[0] = 0x30;
	tag[1] = 0x30 + streamNum;

	tag[2] = 's';
	tag[3] = 'b';

	if(lazyWrite(fileHandle,(unsigned char *)&tag,4)!=4)
		{
		return(false);	
		}

	tempSize = size;

	if(lazyWrite(fileHandle,(unsigned char *)&tempSize,4)!=4)
		{
		return(false);	
		}

	if(lazyWrite(fileHandle,(unsigned char *)header,sizeof(DIVXSUBTITLEHEADER))!=sizeof(DIVXSUBTITLEHEADER))
		{
		return(false);	
		}

	if(lazyWrite(fileHandle,data,dataSize)!=dataSize)
		{
		return(false);	
		}

	writeIndexX->entry[writeIndexXPos].tag[0] = tag[0];
	writeIndexX->entry[writeIndexXPos].tag[1] = tag[1];
	writeIndexX->entry[writeIndexXPos].tag[2] = tag[2];
	writeIndexX->entry[writeIndexXPos].tag[3] = tag[3];
	writeIndexX->entry[writeIndexXPos].flags = header->startingFrame;
	writeIndexX->entry[writeIndexXPos].size = size;
	writeIndexX->entry[writeIndexXPos].filePos = writeFilePos;

	divxStreamIndexCount[index]++;
	divxStreamHeader[index].dwLength++;
	divxStreamByteCount[index]+=size;

	writeIndexXPos++;
	if(writeIndexXPos>=writeIndexXSize)
		{
		writeIndexXSize+=1024;
		tempIndex = (DIVXINDEX *) realloc(writeIndexX, writeIndexXSize * sizeof(DIVXSINGLEINDEX));
		if(tempIndex == NULL)
			{
			logPrintf("Failed to Realloc the index memory\n");
			tempIndex = (DIVXINDEX *) malloc(writeIndexXSize * sizeof(DIVXSINGLEINDEX));
			if(tempIndex == NULL)
				{
				logPrintf("Failed to Malloc more index Memory\n");
				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage15");
				globals->translate->translate(messTitle, "VidomiMessage15Title");

				vidomiErrorBox(NULL,messText,messTitle, MB_OK);
				return(false);
				}
			else
				{
				memcpy(tempIndex,writeIndexX,writeIndexXPos * sizeof(DIVXSINGLEINDEX));
				}
			}
		writeIndexX = tempIndex;
		}

	writeFilePos+=size+8;
	if(writeFilePos&0x00000001)
		{
		// Write one more byte to even up the stream
		tag[0] = 0;
		if(lazyWrite(fileHandle,(unsigned char *)&tag,1)!=1)
			{
			return(false);	
			}
		writeFilePos++;
		}

	return(size);

}

// *****************************************************************************
// divxClass::lazyWrite()
//
// *****************************************************************************
int divxClass::lazyWrite(int fileHandle, unsigned char * data, int size)
{
	int bytesCopied;
	bool done;
	int startingSize;

	if(!DIVXUSELAZYWRITE)
		{
		if(_write(fileHandle,data,size)!=size)
			{
			return(false);	
			}
		return(size);
		}

	startingSize = size;

	done = false;
	do{
		if(size<(lazyWriteSize - lazyWriteBytesUsed))
			{
			memcpy(lazyWriteData,data,size);
			lazyWriteBytesUsed += size;
			lazyWriteData += size;
			done = true;
			}
		else
			{
			bytesCopied = lazyWriteSize - lazyWriteBytesUsed;
			memcpy(lazyWriteData,data,bytesCopied);
			if(_write(fileHandle,lazyWriteStart,lazyWriteSize)!=lazyWriteSize)
				{
				return(false);	
				}
			lazyWriteData = lazyWriteStart;
			lazyWriteBytesUsed = 0;
			data+=bytesCopied;
			size-=bytesCopied;
			}
		}while(!done);

	return(startingSize);
}

// *****************************************************************************
// divxClass::lazyWriteFlush()
//
// *****************************************************************************
int divxClass::lazyWriteFlush(int fileHandle)
{
	if(DIVXUSELAZYWRITE)
		{
		if(lazyWriteBytesUsed)
			{
			if(_write(fileHandle,lazyWriteStart,lazyWriteBytesUsed)!=lazyWriteBytesUsed)
				{
				return(false);	
				}
			}

		lazyWriteData = lazyWriteStart;
		lazyWriteBytesUsed = 0;
		}	

	return(true);

}

