// *****************************************************************************
// divx class
// *****************************************************************************
#ifndef divx_h
#define divx_h

// max streams is +1 because position 0 isn't used
#define DIVXMAXSTREAMS 16
#define DIVXLAZYWRITEBUFFERSIZE 1000000
#define DIVXUSELAZYWRITE 0
#define DIVX_KEYFRAME	0x00000010L
#define makeFOURCC( ch0, ch1, ch2, ch3 ) \
	( (DWORD)(BYTE)(ch0) | ( (DWORD)(BYTE)(ch1) << 8 ) |	\
	( (DWORD)(BYTE)(ch2) << 16 ) | ( (DWORD)(BYTE)(ch3) << 24 ) )

#define divxStreamtypeVIDEO			makeFOURCC('v', 'i', 'd', 's')
#define divxStreamtypeAUDIO			makeFOURCC('a', 'u', 'd', 's')
#define divxStreamtypeMIDI			makeFOURCC('m', 'i', 'd', 's')
#define divxStreamtypeTEXT			makeFOURCC('t', 'x', 't', 's')

#define divxRiffHeader		makeFOURCC('R','I','F','F')

#define divxAviHeader		makeFOURCC('A','V','I',' ')
#define divxListHeader		makeFOURCC('L','I','S','T')
#define divxHdrlHeader		makeFOURCC('h','d','r','l')
#define divxMoviHeader		makeFOURCC('m','o','v','i')
#define divxIdx1Header		makeFOURCC('i','d','x','1')
#define divxAvihHeader		makeFOURCC('a','v','i','h')
#define divxStrlHeader		makeFOURCC('s','t','r','l')
#define divxStrhHeader		makeFOURCC('s','t','r','h')
#define divxStrnHeader		makeFOURCC('s','t','r','n')
#define divxStrfHeader		makeFOURCC('s','t','r','f')
#define divxJunkHeader		makeFOURCC('J','U','N','K')

// Enhanced AVI file chunk types
#define divxStreamtypeSUBTITLE		makeFOURCC('s', 'u', 'b', 's')
#define divxIdxxHeader		makeFOURCC('i','d','x','x')
#define divxStrcHeader		makeFOURCC('s','t','r','c')
#define divxVbrlHeader		makeFOURCC('v','b','r','l')
#define divxVbraHeader		makeFOURCC('v','b','r','a')

#define divxSubtitleTypeGraphicFull 0
#define divxSubtitleTypeGraphicHalf 1
#define divxSubtitleTypeText 2

enum  {
	divxErrorNone=0,
	divxErrorFailedToOpenFile,	
	divxErrorNotAValidFile,
	divxErrorFileNotOpen,
	divxErrorFailedToOpenStream,
	divxErrorFailedToGetStreamInfo,
	divxErrorInvalidStream,
	divxErrorFailedToReadHeader,
	divxErrorFailedToCreateFile,
	divxErrorFailedToCloseFile,
	divxErrorFailedToWriteToFile
};

	class divxClass {

	public:
									divxClass();
									~divxClass();

			void					init();
			void					end();
			int						openRead(char * filename);
			char * 					getErrorString(int error);
			bool					getNextChunk(int fileHandle,DWORD * chunkName, unsigned int * chunkSize);
			bool					getSubChunk(int fileHandle,DWORD * chunkName);
			int						readAllStreamData();
			bool					readIndexData();
			int						getVideoStreamCount();
			int						getAudioStreamCount();
			int						getSubtitleStreamCount();
			void					setCurrentVideoStream(int streamNum);
			void					setCurrentAudioStream(int streamNum);
			void					setCurrentSubtitleStream(int streamNum);
			void					getVideoStreamInfo(int streamNum, DIVXVIDEOINFO * info);
			void					getAudioStreamName(int streamNum, char * name);
			void					getSubtitleStreamName(int streamNum, char * name);
			void					getVideoStreamFormat(int streamNum, BITMAPINFO * bitmap);
			void					getAudioStreamFormat(int streamNum, MPEGLAYER3WAVEFORMAT * wave);
			int						getVideoStreamFrameCount(int streamNum);
			int						getAudioStreamByteCount(int streamNum);
			bool					getVideoStreamNextFrame(int streamNum, unsigned char * data);
			unsigned int			getVideoStreamNextFrameSize(int streamNum);
			int						getAudioStreamBytes(int streamNum, unsigned char * data, int numBytes);
			int						getVideoStreamNextKeyFrame(int streamNum, int startPos);
			void					seekVideoStream(int streamNum, int frame);
			void					seekAudioStream(int streamNum, unsigned int pos);
			int						createFile(char * file, bool mode, bool smooth, int headerReserve);
			int						closeFile();
			int						createAudioStream(DIVXSTREAMINFO * info, MPEGLAYER3WAVEFORMAT * wave, int formatLen);
			int						createVideoStream(DIVXSTREAMINFO * info,BITMAPINFOHEADER * bitmap, int formatLen);
			int						createSubtitleStream(DIVXSTREAMINFO * info);
			int						writeVideoStreamFrame(int streamNum, DWORD flags, unsigned char * data, int size);
			int						writeAudioStreamBytes(int streamNum, unsigned char * data, int size);
			int						writeSubtitleStreamData(int streamNum, DIVXSUBTITLEHEADER * header, unsigned char * data, int dataSize);
			int						lazyWrite(int fileHandle, unsigned char * data, int size);
			int						lazyWriteFlush(int fileHandle);
			bool					readIndexXData();
			unsigned int			getSubtitleStreamNextChunkSize(int streamNum);
			int						getSubtitleStreamNextChunkFrameStart(int streamNum);
			void					seekSubtitleStream(int streamNum, int frame);
			bool					getSubtitleStreamNextChunk(int streamNum, unsigned char * data);
			void					seekAudioStreamByPos(int streamNum, double pos);
			void					seekAudioStreamByBlock(int streamNum, int pos);
			bool					getAudioStreamHeader(int streamNum,DIVXSTREAMHEADER * header);
			int						setSubtitleStreamFormat(int streamNum, DIVXSUBTITLEFORMAT * format);
			void					getSubtitleStreamFormat(int streamNum, DIVXSUBTITLEFORMAT * format);
			void					setVideoStreamChapters(int streamNum, DIVXSTREAMCHAPTERS * chapterData, int size);
			int						getVideoStreamChaptersSize(int streamNum);
			void					getVideoStreamChapters(int streamNum, DIVXSTREAMCHAPTERS * chapter);
			void					audioStreamFixup(int streamNum,DIVXSTREAMINFO * info, MPEGLAYER3WAVEFORMAT * wave, int formatLen);
			int						writeAudioStreamBytesEnhanced(int index, unsigned char * data, int size, int numSamples);
			bool					getVideoStreamHeader(int streamNum,DIVXSTREAMHEADER * header);

	protected:
			char					filename[MAX_PATH];
			int						fileHandle;
			unsigned int			fileSize;
			bool					fileOpen;
			int						fileMode;
			CRITICAL_SECTION		fileLock;
			DIVXHEADER				divxHeader;
			__int64					headerFilePos;
			unsigned int			headerChunkSize;
			__int64					movieFilePos;
			unsigned int			movieChunkSize;
			__int64					indexFilePos;
			unsigned int			indexChunkSize;
			__int64					indexXFilePos;
			unsigned int			indexXChunkSize;
			int						streamCount;
			DIVXSTREAMHEADER		divxStreamHeader[DIVXMAXSTREAMS];
			DIVXSTREAMNAME			divxStreamName[DIVXMAXSTREAMS];
			char *					divxStreamFormat[DIVXMAXSTREAMS];
			int						divxStreamFormatSize[DIVXMAXSTREAMS];
			DIVXSTREAMCHAPTERS *	divxStreamChapters[DIVXMAXSTREAMS];
			int						divxStreamChaptersSize[DIVXMAXSTREAMS];

			DIVXINDEX *				divxStreamIndex[DIVXMAXSTREAMS];
			int						divxStreamIndexSize[DIVXMAXSTREAMS];
			int						divxStreamIndexPos[DIVXMAXSTREAMS];
			int						divxStreamIndexCount[DIVXMAXSTREAMS];
			int						divxStreamBlockPos[DIVXMAXSTREAMS];
			unsigned int			divxStreamByteCount[DIVXMAXSTREAMS];
			int	*					divxStreamSamples[DIVXMAXSTREAMS];
			int						divxStreamSamplesSize[DIVXMAXSTREAMS];
			int						divxStreamSamplesPos[DIVXMAXSTREAMS];

			int						videoStreamList[DIVXMAXSTREAMS];
			int						videoStreamCount;
			int						currentVideoStream;
			int						audioStreamList[DIVXMAXSTREAMS];
			int						audioStreamCount;
			int						currentAudioStream;
			int						subtitleStreamList[DIVXMAXSTREAMS];
			int						subtitleStreamCount;
			int						currentSubtitleStream;
			unsigned int			writeFilePos;
			int						writeIndexPos;
			DIVXINDEX *				writeIndex;
			int						writeIndexSize;
			int						writeIndexXPos;
			DIVXINDEX *				writeIndexX;
			int						writeIndexXSize;
			bool					writeSmooth;
			unsigned char *			writeSmoothBuffer[DIVXMAXSTREAMS];
			int						writeSmoothHeadPos[DIVXMAXSTREAMS];
			int						writeSmoothTailPos[DIVXMAXSTREAMS];
			bool					writeHeaderSetup;
			int						writeHeaderVideoIndex;
			bool					divxIndexRelative;
			unsigned char *			lazyWriteStart;
			unsigned char *			lazyWriteData;
			int						lazyWriteBytesUsed;
			int						lazyWriteSize;
			int						headerSize;
			bool					writeSamplesChunk;

};

#endif
