// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
// This file implements the logPrintf functions 
//
// *****************************************************************************
// *****************************************************************************

#ifdef ENCODERSERVER
#include "../encoderserver/es.h"
#endif

#ifdef ENCODER
#include "../encoder/en.h"
#endif

#ifdef PLAYER
#include "../player/pl.h"
#endif

CRITICAL_SECTION logCSToken;

extern GLOBALS * globals;

void logPrintf(char *format, ...)

{
	va_list va_arg;
	FILE *logFile;

#ifdef PLAYER
	if(globals->playerConfig->sharedConfig.optionsLogOutput)
#else

	if(globals->optionsLogOutput)
#endif
		{
		EnterCriticalSection(&logCSToken);
			
		logFile = fopen(globals->optionsLogFilename, "a");

		va_start(va_arg, format);

		vfprintf(logFile, format, va_arg);

		va_end(va_arg);

		fclose(logFile);
			
		LeaveCriticalSection(&logCSToken);
		}

}

void clearLog(void)
{     
	FILE *logFile;

	InitializeCriticalSection(&logCSToken);

#ifdef PLAYER
	if(globals->playerConfig->sharedConfig.optionsLogOutput)
#else

	if(globals->optionsLogOutput)
#endif
		{
		logFile = fopen(globals->optionsLogFilename, "w");
		fclose(logFile);
		}

}

void logDumpMemoryStatus(char * title)
{
	MEMORYSTATUS memStat;

	memStat.dwLength = sizeof(MEMORYSTATUS);

	GlobalMemoryStatus(&memStat);

	logPrintf("Memory Status Dump - %s\n",title);
	logPrintf("Percent of Memory Used : %d\n",memStat.dwMemoryLoad);
	logPrintf("Bytes of Physical Memory : %d\n",memStat.dwTotalPhys);
	logPrintf("Bytes of Paging File : %d\n",memStat.dwTotalPageFile);
	logPrintf("User Bytes of Address Space : %d\n",memStat.dwTotalVirtual);
	logPrintf("---------------------------\n");
	logPrintf("Free Physical Memory Bytes : %d\n",memStat.dwAvailPhys);
	logPrintf("Free Bytes of Paging File : %d\n",memStat.dwAvailPageFile);
	logPrintf("Free User Bytes : %d\n",memStat.dwAvailVirtual);

}
//end


