// *****************************************************************************
//
//
// outputaudio.cpp
// 
//
// *****************************************************************************

#ifdef ENCODERSERVER
#include "../encoderserver/es.h"
#endif

#ifdef ENCODER
#include "../encoder/en.h"
#endif

#ifdef PLAYER
#include "../player/pl.h"
#endif

extern GLOBALS * globals;

// *****************************************************************************
// outputAudioClass::outputAudioClass()
//
// conversion class constructor
// *****************************************************************************
outputAudioClass::outputAudioClass()
{
	int i;
	outputHandle = NULL;
	for(i=0;i<MAX_AUDIO_OUTPUT_BUFFERS;i++)
		audioBuffer[i] = NULL;

	maxAudioBuffers = MAX_AUDIO_OUTPUT_BUFFERS;

}

// *****************************************************************************
// outputAudioClass::~outputAudioClass()
//
// conversion class destructor
// *****************************************************************************
outputAudioClass::~outputAudioClass()
{
	int i;

	if(outputHandle)
		{
		waveOutReset(outputHandle);
		waveOutClose(outputHandle);
		}
	for(i=0;i<MAX_AUDIO_OUTPUT_BUFFERS;i++)
		{
		if(audioBuffer[i])
			{
			delete [] audioBuffer[i]->bufferStart;
			delete [] audioBuffer[i];
			audioBuffer[i] = NULL;
			}
		}

}

// *****************************************************************************
// outputAudioClass::setMaxAudioBuffers()
// *****************************************************************************
void outputAudioClass::setMaxAudioBuffers(int max)
{
	maxAudioBuffers = max;
}

// *****************************************************************************
// outputAudioClass::setAudioInputFormat()
//
// *****************************************************************************
bool outputAudioClass::setAudioInputFormat(int channels, int sampleRate)
{
	MMRESULT ret;
	int i;

	memset(&audioFormatIn,0,sizeof(WAVEFORMATEX));	

	//note that we assume 16 bits per sample.
	audioFormatIn.wFormatTag=WAVE_FORMAT_PCM;
	audioFormatIn.nChannels = channels;
	audioFormatIn.nSamplesPerSec=sampleRate;
	audioFormatIn.nAvgBytesPerSec= sampleRate*channels*2;
	audioFormatIn.nBlockAlign=channels*2;
	audioFormatIn.wBitsPerSample=16;
	audioFormatIn.cbSize=0;

	inputSampleRate = sampleRate;
	inputChannels = channels;

	ret = waveOutOpen(&outputHandle,WAVE_MAPPER, &audioFormatIn, (DWORD) &waveOutProc, 0,CALLBACK_FUNCTION);
	if(ret!=MMSYSERR_NOERROR)
		{
		logPrintf("Error Opening wav Output device - %d\n",ret);
		return(false);
		}

	//waveOutReset(outputHandle);
	// Now create the audio output buffers
	outputBufferLen = (sampleRate*channels*2)/4;
	
	// 2 seconds as our default
	maxAudioBuffers = 5;

	for(i=0;i<maxAudioBuffers;i++)
		{
		audioBuffer[i] = new AUDIODATA [1];
		audioBuffer[i]->bufferStart = new unsigned char [outputBufferLen];
		audioBuffer[i]->bufferSize = outputBufferLen;
		audioBuffer[i]->data =audioBuffer[i]->bufferStart;
		audioBuffer[i]->bytesUsed =0;
		audioStatus[i] = 0;
		}
	currentBuffer = 0;
	return(true);
}

// *****************************************************************************
// outputAudioClass::setAudioInputFormat()
//
// *****************************************************************************
bool outputAudioClass::outputAudio(AUDIODATA * samples)
{		
	MMRESULT ret;
	int bytesToCopy;

	// If the head buffer is still being used, then just return

	if(audioStatus[currentBuffer] || !samples->bytesUsed)
		return(true);

	if((samples->bytesUsed - samples->headUsed)< (audioBuffer[currentBuffer]->bufferSize - audioBuffer[currentBuffer]->bytesUsed))
		{
		// We can fit the entire buffer
		bytesToCopy = samples->bytesUsed - samples->headUsed;
		}
	else
		{
		bytesToCopy = audioBuffer[currentBuffer]->bufferSize - audioBuffer[currentBuffer]->bytesUsed;
		}
	memcpy(audioBuffer[currentBuffer]->data,samples->bufferStart + samples->headUsed,bytesToCopy);
	audioBuffer[currentBuffer]->data += bytesToCopy;
	audioBuffer[currentBuffer]->bytesUsed += bytesToCopy;
	
	if(bytesToCopy==(samples->bytesUsed - samples->headUsed))
		{
		// We consumed the entire buffer, so reset the buffer pointers
		samples->data = samples->bufferStart;
		samples->bytesUsed = 0;
		samples->headUsed =0;
		}
	else
		{
		// Only a partial usage, so adjust the head
		samples->headUsed += bytesToCopy;
		}

	if(audioBuffer[currentBuffer]->bytesUsed <audioBuffer[currentBuffer]->bufferSize)
		return(true);

	memset(&audioBufferHeader[currentBuffer],0,sizeof(WAVEHDR));
	audioBufferHeader[currentBuffer].lpData = (char *) audioBuffer[currentBuffer]->bufferStart;
	audioBufferHeader[currentBuffer].dwBufferLength = audioBuffer[currentBuffer]->bytesUsed;
	audioBufferHeader[currentBuffer].dwUser = currentBuffer;
	ret = waveOutPrepareHeader(outputHandle,&audioBufferHeader[currentBuffer],sizeof(WAVEHDR));
	if(ret!=MMSYSERR_NOERROR)
		{
		logPrintf("Error preparing buffer for Output device - %d\n",ret);
		return(false);
		}

	ret = waveOutWrite(outputHandle,&audioBufferHeader[currentBuffer],sizeof(WAVEHDR));
	if(ret!=MMSYSERR_NOERROR)
		{
		logPrintf("Error writing to Output device - %d\n",ret);
		return(false);
		}

	audioStatus[currentBuffer] = 1;	
	currentBuffer++;
	if(currentBuffer == maxAudioBuffers)
		currentBuffer=0;

	unprepare = true;
	return(true);

}

// *****************************************************************************
// outputAudioClass::getOutputTimer()
//
// *****************************************************************************
double outputAudioClass::getOutputTimer()
{
	MMTIME mmt;
	double msPos;
	mmt.wType = TIME_SAMPLES;
	int stat;
	int i;

	stat=0;
	for(i=0;i<maxAudioBuffers;i++)
		{
		stat += audioStatus[i];
		}

	// If the buffers still have audio, then use the wavepos
	if(stat)
		{
		if(waveOutGetPosition(outputHandle,&mmt,sizeof(MMTIME))==MMSYSERR_NOERROR)
			{
			msPos = ((double)mmt.u.sample / (double)(inputSampleRate)) * 1000.0f; 
			return(msPos );
			}
		else
			{
			// false indicates that the timer should be used for sync
			return(false);
			}
		}
	else
		{
		// false indicates that the timer should be used for sync
		return(false);
		}
}

// *****************************************************************************
// outputAudioClass::bufferStatus()
//
// *****************************************************************************
bool outputAudioClass::bufferStatus()
{
	if(audioStatus[currentBuffer])
		return(false);
	else
		return(true);
}

// *****************************************************************************
// outputAudioClass::preBufferEnough()
//
// *****************************************************************************
bool outputAudioClass::preBufferEnough()
{
	int stat;
	int i;

	stat=0;
	for(i=0;i<maxAudioBuffers;i++)
		{
		stat += audioStatus[i];
		}

	if(stat>=(maxAudioBuffers/2))
		return(true);
	else
		return(false);
}

// *****************************************************************************
// outputAudioClass::pause()
//
// *****************************************************************************
void outputAudioClass::pause()
{
	waveOutPause(outputHandle);
}

// *****************************************************************************
// outputAudioClass::play()
//
// *****************************************************************************
void outputAudioClass::play()
{
	waveOutRestart(outputHandle);
}

// *****************************************************************************
// outputAudioClass::setVolume()
//
// *****************************************************************************
void outputAudioClass::setVolume(int volume )
{
	DWORD tempVol;
	if(volume==-1)
		{
		tempVol = 0x00010001;
		}
	else
		{
		tempVol = volume*256;
		tempVol = tempVol<<16;
		tempVol |= volume*256;
		}
	waveOutSetVolume(outputHandle,tempVol);
}

// *****************************************************************************
// outputAudioClass::setVolume()
//
// *****************************************************************************
int outputAudioClass::getVolume()
{
	DWORD tempVol;

	waveOutGetVolume(outputHandle,&tempVol);
	return((tempVol&0x0000ffff) / 256);
}

// *****************************************************************************
// outputAudioClass::stop()
//
// *****************************************************************************
void outputAudioClass::stop()
{
	int i;
	unprepare = false;
	waveOutReset(outputHandle);
	unprepare = true;
	
	for(i=0;i<maxAudioBuffers;i++)
		{
		waveOutUnprepareHeader(outputHandle,&audioBufferHeader[i],sizeof(WAVEHDR));
		audioBuffer[i]->data =audioBuffer[i]->bufferStart;
		audioBuffer[i]->bytesUsed =0;
		audioStatus[i] = 0;
		memset(audioBuffer[i]->data,0,audioBuffer[i]->bytesUsed);
		}

	currentBuffer = 0;	
}

// *****************************************************************************
// outputAudioClass::end()
//
// *****************************************************************************
void outputAudioClass::end()
{
	int i;
	unprepare = false;
	waveOutReset(outputHandle);

	for(i=0;i<maxAudioBuffers;i++)
		{
		waveOutUnprepareHeader(outputHandle,&audioBufferHeader[i],sizeof(WAVEHDR));

		// If the buffers exist, delete them.
		if(audioBuffer[i])
			{
			delete [] audioBuffer[i]->bufferStart;
			delete [] audioBuffer[i];
			audioBuffer[i] = NULL;
			}

		}

	waveOutClose(outputHandle); 

}

// *****************************************************************************
// outputAudioClass::waveOutProc()
//
// Callback from windows about out buffers
// *****************************************************************************
void CALLBACK outputAudioClass::waveOutProc( HWAVEOUT hwo, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
{
	globals->outputAudio->waveOutProc1(hwo,uMsg,dwInstance,dwParam1,dwParam2);
}

// *****************************************************************************
// outputAudioClass::waveOutProc()
//
// Callback from windows about out buffers
// *****************************************************************************
void outputAudioClass::waveOutProc1( HWAVEOUT hwo, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
{
	WAVEHDR * ptr;
	int index;
	switch(uMsg)
		{
		case WOM_DONE :
			// Update the buffer status to be available.
			ptr = (WAVEHDR *) dwParam1;
			if(unprepare)
				waveOutUnprepareHeader(hwo,ptr,sizeof(WAVEHDR));
			index = ptr->dwUser;
			if(audioBuffer[index])
				{
				audioBuffer[index]->data = audioBuffer[index]->bufferStart;
				audioBuffer[index]->bytesUsed = 0;
				audioStatus[index] = 0;
				}
			break;

		default : 
			break;
		}
}
