// *****************************************************************************
//
//
// skins.cpp
// 
//
// *****************************************************************************

#ifdef ENCODERSERVER
#include "../encoderserver/es.h"
#endif

#ifdef ENCODER
#include "../encoder/en.h"
#endif

#ifdef PLAYER
#include "../player/pl.h"
#endif

extern GLOBALS * globals;

char skinTextTable[] = { 
							// The first line is the counter text
							'0','1','2','3','4','5','6','7','8','9',':','.','/',
							'A','a','B','b','C','c','D','d','E','e',
							'F','f','G','g','H','h','I','i','J','j',
							'K','k','L','l','M','m','N','n','O','o',
							'P','p','Q','q','R','r','S','s','T','t',
							'U','u','V','v','W','w','X','x','Y','y',
							'Z','z',' ','~','`','!','@','#','$','%',
							'^','&','*','(',')','_','-','+','{','}',
							'[',']','\\',';',0x27,0x2c
};

SCRIPTCOMMAND skinScriptCommands[] = {
										{"scrollrate",SKIN_SCRIPT_SCROLL_RATE},
										{"transparentcolor",SKIN_SCRIPT_TRANSPARENT_COLOR},
										{"loadbaseskin",SKIN_SCRIPT_LOAD_BASE_SKIN},
										{"createbutton",SKIN_SCRIPT_CREATE_BUTTON},
										{"createtwostatebutton",SKIN_SCRIPT_CREATE_TWO_STATE_BUTTON},
										{"createhslider",SKIN_SCRIPT_CREATE_HSLIDER},
										{"createvslider",SKIN_SCRIPT_CREATE_VSLIDER},
										{"createfonttextbox",SKIN_SCRIPT_CREATE_FONT_TEXTBOX},
										{"creategraphictextbox",SKIN_SCRIPT_CREATE_GRAPHIC_TEXTBOX},
										{"createhighlightanimation",SKIN_SCRIPT_CREATE_HIGHLIGHT_ANIMATION},
										{"createanimation",SKIN_SCRIPT_CREATE_ANIMATION},
										{"loadresizebaseskin",SKIN_SCRIPT_LOAD_RESIZE_BASE_SKIN},
										{"createcombobox",SKIN_SCRIPT_CREATE_COMBOBOX},
										{"createwindowseditbox",SKIN_SCRIPT_CREATE_EDITBOX},
										};

extern GIZMONAME gizmoNameTranslationTable[];

// *****************************************************************************
// skinsClass::skinsWindowClass()
//
// skins class constructor
// *****************************************************************************
skinsClass::skinsClass()
{
	int i;
	int x;
	int spaceIndex;


	windowMoving =false;
	windowRegionListValid = false;
	windowRegionList = NULL;
	mainhWnd = NULL;
	visible = false;

	for(i=0;i<MAX_GIZMOS;i++)
		{
		memset(&gizmoList[i],0, sizeof(GIZMO));
		gizmoList[i].gizmoType = GIZMO_UNUSED;
		}

	spaceIndex=0;
	for(x=0;x<sizeof(skinTextTable);x++)
		{
		if(skinTextTable[x] == ' ')
			{
			spaceIndex = x;
			break;
			}
		}
	// Non characters are set to spaces
	for(i=0;i<256;i++)
		{
		textConversionTable[i]=spaceIndex;
		for(x=0;x<sizeof(skinTextTable);x++)
			{
			if(skinTextTable[x] == i)
				{
				textConversionTable[i]=x;
				break;
				}
			}
		}

	memset(&windowBase,0,sizeof(GRAPHICOBJECT));

	for(i=0;i<MAX_SKIN_GRAPHIC_FILES;i++)
		{
		memset(&buttonGraphic[i],0,sizeof(GRAPHICOBJECT));

		}
	lastScrollTime = timeGetTime();
	currentGizmoIndex = 0;
	currrentGraphicIndex = 0;

}

// *****************************************************************************
// skinsClass::~skinsClass()
//
// skins class destructor
// *****************************************************************************
skinsClass::~skinsClass()
{
	if(mainhWnd)
		DestroyWindow(mainhWnd);

	ReleaseDC(NULL,windowBase.hdc);
	DeleteObject(windowBase.handle);

	ReleaseDC(NULL,frontBufferHdc);
	DeleteObject(frontBufferBitmap);

	if(windowRegionListValid)
		{
		if(windowRegionList)
			{
			free(windowRegionList);
			}
		}
}

// *****************************************************************************
// skinsClass::loadSkin()
//
// *****************************************************************************
bool skinsClass::loadSkin(char * inSkinZip, char * sectionName)
{

	strcpy(skinZipName,inSkinZip);

	return(parseSkinFile("skinscript.txt", sectionName));

}

// *****************************************************************************
// skinsClass::loadBaseSkin()
//
// *****************************************************************************
bool skinsClass::loadBaseSkin(char * filename)
{
	char * memPtr;
	BITMAPINFO * bitmapInfo;
	BITMAPFILEHEADER * bitmapFileHeader;
	HDC tempHdc;
	int sizeOfFile;

	memPtr = loadFileFromZip(skinZipName, filename, &sizeOfFile);
	if(!memPtr)
		{
		return(false);
		}

	bitmapFileHeader = (BITMAPFILEHEADER *) memPtr;
	bitmapInfo = (BITMAPINFO *) (memPtr+sizeof(BITMAPFILEHEADER));

  // Create an HBITMAP from the loaded bitmap
	tempHdc = GetDC(NULL);
	windowBase.handle = CreateDIBitmap(tempHdc, &bitmapInfo->bmiHeader,CBM_INIT,(unsigned char *)bitmapInfo + bitmapInfo->bmiHeader.biSize,bitmapInfo,DIB_RGB_COLORS);
	ReleaseDC(NULL,tempHdc);
	delete [] memPtr;

	windowBase.hdc = CreateCompatibleDC(NULL); 
	GetObject(windowBase.handle,sizeof(BITMAP), (LPSTR)&windowBase.info);
	windowBase.handle1 = (HBITMAP) SelectObject(windowBase.hdc,windowBase.handle);
	windowRegionListValid = createRegionData(0,0,windowBase.info.bmWidth,windowBase.info.bmHeight);

	return(true);
}

// *****************************************************************************
// skinsClass::loadBaseSkin()
//
// *****************************************************************************
bool skinsClass::loadResizeBaseSkin(char * filename)
{
	char * memPtr;
	int sizeOfFile;
	HDC tempHdc;
	BITMAPINFO * bitmapInfo;
	BITMAPFILEHEADER * bitmapFileHeader;

	memPtr = loadFileFromZip(skinZipName, filename, &sizeOfFile);
	if(!memPtr)
		{
		return(false);
		}

	bitmapFileHeader = (BITMAPFILEHEADER *) memPtr;
	bitmapInfo = (BITMAPINFO *) (memPtr+sizeof(BITMAPFILEHEADER));

  // Create an HBITMAP from the loaded bitmap
	tempHdc = GetDC(NULL);
	windowBase.handle = CreateDIBitmap(tempHdc, &bitmapInfo->bmiHeader,CBM_INIT,(unsigned char *)bitmapInfo + bitmapInfo->bmiHeader.biSize,bitmapInfo,DIB_RGB_COLORS);
	ReleaseDC(NULL,tempHdc);
	delete [] memPtr;

	windowBase.hdc = CreateCompatibleDC(NULL); 
	GetObject(windowBase.handle,sizeof(BITMAP), (LPSTR)&windowBase.info);
	windowBase.handle1 = (HBITMAP) SelectObject(windowBase.hdc,windowBase.handle);
	windowRegionListValid = createRegionData(0,0,windowBase.info.bmWidth,windowBase.info.bmHeight);

	// Generate Top Left Region Data
	windowRegionListValid = createRegionData(resizeTilePos[RESIZE_TILE_TOPLEFT].x,
											resizeTilePos[RESIZE_TILE_TOPLEFT].y,
											resizeTilePos[RESIZE_TILE_TOPLEFT].width,
											resizeTilePos[RESIZE_TILE_TOPLEFT].height);
	if(!windowRegionListValid)
		{
		return(false);
		}
	resizeWindowRegionList[0] = windowRegionList;
	resizeWindowRegionListSize[0] = windowRegionListSize;
	resizeWindowRegionListCount[0] = windowRegionListCount;

	// Generate Top Right Region Data
	windowRegionListValid = createRegionData(resizeTilePos[RESIZE_TILE_TOPRIGHT].x,
											resizeTilePos[RESIZE_TILE_TOPRIGHT].y,
											resizeTilePos[RESIZE_TILE_TOPRIGHT].width,
											resizeTilePos[RESIZE_TILE_TOPRIGHT].height);
	if(!windowRegionListValid)
		{
		return(false);
		}
	resizeWindowRegionList[1] = windowRegionList;
	resizeWindowRegionListSize[1] = windowRegionListSize;
	resizeWindowRegionListCount[1] = windowRegionListCount;

	// Generate Bot Left Region Data
	windowRegionListValid = createRegionData(resizeTilePos[RESIZE_TILE_BOTLEFT].x,
											resizeTilePos[RESIZE_TILE_BOTLEFT].y,
											resizeTilePos[RESIZE_TILE_BOTLEFT].width,
											resizeTilePos[RESIZE_TILE_BOTLEFT].height);
	if(!windowRegionListValid)
		{
		return(false);
		}
	resizeWindowRegionList[2] = windowRegionList;
	resizeWindowRegionListSize[2] = windowRegionListSize;
	resizeWindowRegionListCount[2] = windowRegionListCount;

	// Generate Bot Right Region Data
	windowRegionListValid = createRegionData(resizeTilePos[RESIZE_TILE_BOTRIGHT].x,
											resizeTilePos[RESIZE_TILE_BOTRIGHT].y,
											resizeTilePos[RESIZE_TILE_BOTRIGHT].width,
											resizeTilePos[RESIZE_TILE_BOTRIGHT].height);
	if(!windowRegionListValid)
		{
		return(false);
		}
	resizeWindowRegionList[3] = windowRegionList;
	resizeWindowRegionListSize[3] = windowRegionListSize;
	resizeWindowRegionListCount[3] = windowRegionListCount;

	return(true);
}

// *****************************************************************************
// skinsClass::createButton()
//
// *****************************************************************************
int skinsClass::createButton(int graphicNumber, int transparency, int posX, int posY,int srcX, int srcY, 
							 int width, int height, int highlightMax, char * name)
{
	int i;
	int messageId;

	messageId = 0;
	i = 0;
	for(;;)
		{
		if(!gizmoNameTranslationTable[i].text[0])
			{
			break;
			}
		if(!_stricmp(gizmoNameTranslationTable[i].text, name))
			{
			messageId = gizmoNameTranslationTable[i].messageId1;
			break;
			}
		i++;
		}

	memset(&gizmoList[currentGizmoIndex], 0,sizeof(GIZMO));
	
	gizmoList[currentGizmoIndex].gizmoType = GIZMO_BUTTON;
	gizmoList[currentGizmoIndex].graphicNumber = graphicNumber;
	
	if(transparency)
		{
		gizmoList[currentGizmoIndex].transparentBlt = true;
		}
	else
		{
		gizmoList[currentGizmoIndex].transparentBlt = false;
		}

	gizmoList[currentGizmoIndex].width = width;
	gizmoList[currentGizmoIndex].height = height;
	gizmoList[currentGizmoIndex].posX = posX;
	gizmoList[currentGizmoIndex].posY = posY;
	gizmoList[currentGizmoIndex].srcX = srcX;
	gizmoList[currentGizmoIndex].srcY = srcY; 
	gizmoList[currentGizmoIndex].highlightMax = highlightMax;
	gizmoList[currentGizmoIndex].hitboxPosX = gizmoList[currentGizmoIndex].posX;
	gizmoList[currentGizmoIndex].hitboxPosY = gizmoList[currentGizmoIndex].posY;
	gizmoList[currentGizmoIndex].hitboxWidth = gizmoList[currentGizmoIndex].width;
	gizmoList[currentGizmoIndex].hitboxHeight = gizmoList[currentGizmoIndex].height;
	gizmoList[currentGizmoIndex].messageId[0] = messageId;
	strcpy(gizmoList[currentGizmoIndex].name,name);
	gizmoList[currentGizmoIndex].scrollText = false;
	gizmoList[currentGizmoIndex].proportionalText = false;
	gizmoList[currentGizmoIndex].canScrollText= false;
	strcpy(gizmoList[currentGizmoIndex].text,"");
	return(currentGizmoIndex++);	
}

// *****************************************************************************
// skinsClass::createTwoStateButton()
//
// *****************************************************************************
int skinsClass::createTwoStateButton(int graphicNumber, int transparency, int posX, int posY,int srcX, int srcY, 
							 int width, int height, int highlightMax, char * name)
{
	int i;
	int messageId1;
	int messageId2;

	messageId1 = 0;
	messageId2 = 0;
	i = 0;
	for(;;)
		{
		if(!gizmoNameTranslationTable[i].text[0])
			{
			break;
			}
		if(!_stricmp(gizmoNameTranslationTable[i].text, name))
			{
			messageId1 = gizmoNameTranslationTable[i].messageId1;
			messageId2 = gizmoNameTranslationTable[i].messageId2;
			break;
			}
		i++;
		}

	gizmoList[currentGizmoIndex].gizmoType = GIZMO_BUTTON;
	gizmoList[currentGizmoIndex].graphicNumber = graphicNumber;
	if(transparency)
		{
		gizmoList[currentGizmoIndex].transparentBlt = true;
		}
	else
		{
		gizmoList[currentGizmoIndex].transparentBlt = false;
		}
	gizmoList[currentGizmoIndex].width = width;
	gizmoList[currentGizmoIndex].height = height;
	gizmoList[currentGizmoIndex].posX = posX;
	gizmoList[currentGizmoIndex].posY = posY;
	gizmoList[currentGizmoIndex].srcX = srcX;
	gizmoList[currentGizmoIndex].srcY = srcY; 
	gizmoList[currentGizmoIndex].highlightMax = highlightMax;
	gizmoList[currentGizmoIndex].hitboxPosX = gizmoList[currentGizmoIndex].posX;
	gizmoList[currentGizmoIndex].hitboxPosY = gizmoList[currentGizmoIndex].posY;
	gizmoList[currentGizmoIndex].hitboxWidth = gizmoList[currentGizmoIndex].width;
	gizmoList[currentGizmoIndex].hitboxHeight = gizmoList[currentGizmoIndex].height;
	gizmoList[currentGizmoIndex].messageId[0] = messageId1;
	gizmoList[currentGizmoIndex].messageId[1] = messageId2;
	strcpy(gizmoList[currentGizmoIndex].name,name);
	gizmoList[currentGizmoIndex].scrollText = false;
	gizmoList[currentGizmoIndex].proportionalText = false;
	gizmoList[currentGizmoIndex].canScrollText= false;
	strcpy(gizmoList[currentGizmoIndex].text,"");
	gizmoList[currentGizmoIndex].multiStateGizmo = true;
	gizmoList[currentGizmoIndex].multiState = 0;
	return(currentGizmoIndex++);	
}
// *****************************************************************************
// skinsClass::createHSlider()
//
// *****************************************************************************
int skinsClass::createHSlider(int graphicNumber, int transparency, int hitboxPosX, int hitboxPosY, int hitboxWidth,int srcX, int srcY, 
							 int width, int height,  int highlightMax, char * name)
{
	int i;
	int messageId;

	messageId = 0;
	i = 0;
	for(;;)
		{
		if(!gizmoNameTranslationTable[i].text[0])
			{
			break;
			}
		if(!_stricmp(gizmoNameTranslationTable[i].text, name))
			{
			messageId = gizmoNameTranslationTable[i].messageId1;
			break;
			}
		i++;
		}

	memset(&gizmoList[currentGizmoIndex], 0,sizeof(GIZMO));

	gizmoList[currentGizmoIndex].gizmoType = GIZMO_HSLIDER;
	gizmoList[currentGizmoIndex].graphicNumber = graphicNumber;
	if(transparency)
		{
		gizmoList[currentGizmoIndex].transparentBlt = true;
		}
	else
		{
		gizmoList[currentGizmoIndex].transparentBlt = false;
		}
	gizmoList[currentGizmoIndex].width = width;
	gizmoList[currentGizmoIndex].height = height;
	gizmoList[currentGizmoIndex].posX = hitboxPosX;
	gizmoList[currentGizmoIndex].posY = hitboxPosY;
	gizmoList[currentGizmoIndex].srcX = srcX;
	gizmoList[currentGizmoIndex].srcY = srcY; 
	gizmoList[currentGizmoIndex].highlightMax = highlightMax;
	gizmoList[currentGizmoIndex].hitboxPosX = hitboxPosX;
	gizmoList[currentGizmoIndex].hitboxPosY = hitboxPosY;
	gizmoList[currentGizmoIndex].hitboxWidth = hitboxWidth + width;
	gizmoList[currentGizmoIndex].hitboxHeight = height;
	gizmoList[currentGizmoIndex].sliderPos = 0;
	gizmoList[currentGizmoIndex].messageId[0] = messageId;
	strcpy(gizmoList[currentGizmoIndex].name,name);
	gizmoList[currentGizmoIndex].scrollText = false;
	gizmoList[currentGizmoIndex].proportionalText = false;
	gizmoList[currentGizmoIndex].canScrollText= false;
	strcpy(gizmoList[currentGizmoIndex].text,"");

	return(currentGizmoIndex++);	
}

// *****************************************************************************
// skinsClass::createVSlider()
//
// *****************************************************************************
int skinsClass::createVSlider(int graphicNumber, int transparency, int hitboxPosX, int hitboxPosY, int hitboxHeight,int srcX, int srcY, 
							 int width, int height,  int highlightMax, char * name)
{
	int i;
	int messageId;

	messageId = 0;
	i = 0;
	for(;;)
		{
		if(!gizmoNameTranslationTable[i].text[0])
			{
			break;
			}
		if(!_stricmp(gizmoNameTranslationTable[i].text, name))
			{
			messageId = gizmoNameTranslationTable[i].messageId1;
			break;
			}
		i++;
		}

	memset(&gizmoList[currentGizmoIndex], 0,sizeof(GIZMO));

	gizmoList[currentGizmoIndex].gizmoType = GIZMO_VSLIDER;
	gizmoList[currentGizmoIndex].graphicNumber = graphicNumber;
	if(transparency)
		{
		gizmoList[currentGizmoIndex].transparentBlt = true;
		}
	else
		{
		gizmoList[currentGizmoIndex].transparentBlt = false;
		}
	gizmoList[currentGizmoIndex].width = width;
	gizmoList[currentGizmoIndex].height = height;
	gizmoList[currentGizmoIndex].posX = hitboxPosX;
	gizmoList[currentGizmoIndex].posY = hitboxPosY;
	gizmoList[currentGizmoIndex].srcX = srcX;
	gizmoList[currentGizmoIndex].srcY = srcY; 
	gizmoList[currentGizmoIndex].highlightMax = highlightMax;
	gizmoList[currentGizmoIndex].hitboxPosX = hitboxPosX;
	gizmoList[currentGizmoIndex].hitboxPosY = hitboxPosY;
	gizmoList[currentGizmoIndex].hitboxWidth = width;
	gizmoList[currentGizmoIndex].hitboxHeight = hitboxHeight + height;
	gizmoList[currentGizmoIndex].sliderPos = 0;
	gizmoList[currentGizmoIndex].messageId[0] = messageId;
	strcpy(gizmoList[currentGizmoIndex].name,name);
	gizmoList[currentGizmoIndex].scrollText = false;
	gizmoList[currentGizmoIndex].proportionalText = false;
	gizmoList[currentGizmoIndex].canScrollText= false;
	strcpy(gizmoList[currentGizmoIndex].text,"");

	return(currentGizmoIndex++);	
}

// *****************************************************************************
// skinsClass::createFontTextbox()
//
// *****************************************************************************
int skinsClass::createFontTextbox(char * fontName, int textboxX, int textboxY,int textboxWidth, int textboxHeight,
							 int pointSize, int bold, int ital, int underline, COLORREF color, COLORREF aaColor, int scrolling, int scrollPixels, char * name)
{
	DWORD weight;

	memset(&gizmoList[currentGizmoIndex], 0,sizeof(GIZMO));

	gizmoList[currentGizmoIndex].gizmoType = GIZMO_FONT_TEXTBOX;
	gizmoList[currentGizmoIndex].graphicNumber = -1;
	gizmoList[currentGizmoIndex].posX = textboxX;
	gizmoList[currentGizmoIndex].posY = textboxY;
	gizmoList[currentGizmoIndex].hitboxWidth = textboxWidth;
	gizmoList[currentGizmoIndex].hitboxHeight = textboxHeight;

	strcpy(gizmoList[currentGizmoIndex].name,name);

	if(scrolling)
		{
		gizmoList[currentGizmoIndex].canScrollText= true;
		}
	else
		{
		gizmoList[currentGizmoIndex].canScrollText= false;
		}
	gizmoList[currentGizmoIndex].scrollText = false;

	gizmoList[currentGizmoIndex].scrollIndex = 0;
	gizmoList[currentGizmoIndex].scrollPixelOffset = 0;
	gizmoList[currentGizmoIndex].scrollNumPixels = scrollPixels;
	strcpy(gizmoList[currentGizmoIndex].text,"");

	gizmoList[currentGizmoIndex].fontColor = color;
	gizmoList[currentGizmoIndex].fontAAColor = aaColor;

 // blah
	weight = FW_NORMAL;

	switch(bold)
		{
		case 1:
			weight = FW_BOLD;
			break;

		case 2:
			weight = FW_EXTRABOLD;
			break;

		case 3:
			weight = FW_ULTRABOLD;
			break;
		}

	gizmoList[currentGizmoIndex].font = CreateFont(pointSize, 0, 0, 0, weight , ital, underline, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, fontName);

	return(currentGizmoIndex++);
}

// *****************************************************************************
// skinsClass::createGraphicTextbox()
//
// *****************************************************************************
int skinsClass::createGraphicTextbox(int graphicNumber, int transparency, int textboxX, int textboxY,int textboxWidth, int srcX, int srcY, 
							 int width, int height, int proportional, int scrolling, int scrollPixels, char * name)
{
	memset(&gizmoList[currentGizmoIndex], 0,sizeof(GIZMO));

	gizmoList[currentGizmoIndex].gizmoType = GIZMO_GRAPHIC_TEXTBOX;
	gizmoList[currentGizmoIndex].graphicNumber = graphicNumber;
	if(transparency)
		{
		gizmoList[currentGizmoIndex].transparentBlt = true;
		}
	else
		{
		gizmoList[currentGizmoIndex].transparentBlt = false;
		}
	gizmoList[currentGizmoIndex].width = width;
	gizmoList[currentGizmoIndex].height = height;
	gizmoList[currentGizmoIndex].posX = textboxX;
	gizmoList[currentGizmoIndex].posY = textboxY;
	gizmoList[currentGizmoIndex].srcX = srcX;
	gizmoList[currentGizmoIndex].srcY = srcY; 
	gizmoList[currentGizmoIndex].hitboxWidth = textboxWidth;
	gizmoList[currentGizmoIndex].hitboxHeight = height;

	strcpy(gizmoList[currentGizmoIndex].name,name);
	 
	if(scrolling)
		{
		gizmoList[currentGizmoIndex].canScrollText= true;
		}
	else
		{
		gizmoList[currentGizmoIndex].canScrollText= false;
		}

	gizmoList[currentGizmoIndex].scrollText = false;

	if(proportional)
		{
		gizmoList[currentGizmoIndex].proportionalText = true;
		}
	else
		{
		gizmoList[currentGizmoIndex].proportionalText = false;
		}
	
	gizmoList[currentGizmoIndex].scrollIndex = 0;
	gizmoList[currentGizmoIndex].scrollPixelOffset = 0;
	gizmoList[currentGizmoIndex].scrollNumPixels = scrollPixels;
	strcpy(gizmoList[currentGizmoIndex].text,"");

	if(proportional)
		{
		//Generate the proportional data if it's not already been done
		if(buttonGraphic[gizmoList[currentGizmoIndex].graphicNumber].proportionalTextEntries==0)
			{
			createProportionalTextData(gizmoList[currentGizmoIndex].graphicNumber);
			}
		}
	return(currentGizmoIndex++);	
}

// *****************************************************************************
// skinsClass::createAnimation()
//
// *****************************************************************************
int skinsClass::createAnimation(int graphicNumber, int transparency, int posX, int posY,int srcX, int srcY, 
							 int width, int height, char * animationList, int animationMax, int animationLoopDelay, char * name)
{
	memset(&gizmoList[currentGizmoIndex], 0,sizeof(GIZMO));

	gizmoList[currentGizmoIndex].gizmoType = GIZMO_ANIMATION;
	gizmoList[currentGizmoIndex].graphicNumber = graphicNumber;
	if(transparency)
		{
		gizmoList[currentGizmoIndex].transparentBlt = true;
		}
	else
		{
		gizmoList[currentGizmoIndex].transparentBlt = false;
		}
	gizmoList[currentGizmoIndex].width = width;
	gizmoList[currentGizmoIndex].height = height;
	gizmoList[currentGizmoIndex].posX = posX;
	gizmoList[currentGizmoIndex].posY = posY;
	gizmoList[currentGizmoIndex].srcX = srcX;
	gizmoList[currentGizmoIndex].srcY = srcY; 
	strcpy(gizmoList[currentGizmoIndex].name,name);
	gizmoList[currentGizmoIndex].canScrollText= false;
	gizmoList[currentGizmoIndex].scrollText = false;
	gizmoList[currentGizmoIndex].proportionalText = false;
	gizmoList[currentGizmoIndex].animationMax = animationMax;
	gizmoList[currentGizmoIndex].animationIndex = 0;
	gizmoList[currentGizmoIndex].animationLoopDelay = animationLoopDelay;
	gizmoList[currentGizmoIndex].animationDelay = 0;
	memcpy(&gizmoList[currentGizmoIndex].animationList[0], animationList, animationMax);

	return(currentGizmoIndex++);	
}

// *****************************************************************************
// skinsClass::createAnimation()
//
// *****************************************************************************
int skinsClass::createHighlightAnimation(int graphicNumber, int transparency, int posX, int posY,int srcX, int srcY, 
							 int width, int height, char * animationList, int animationMax, char * name)
{
	memset(&gizmoList[currentGizmoIndex], 0,sizeof(GIZMO));

	gizmoList[currentGizmoIndex].gizmoType = GIZMO_HIGHLIGHT_ANIMATION;
	gizmoList[currentGizmoIndex].graphicNumber = graphicNumber;
	if(transparency)
		{
		gizmoList[currentGizmoIndex].transparentBlt = true;
		}
	else
		{
		gizmoList[currentGizmoIndex].transparentBlt = false;
		}
	gizmoList[currentGizmoIndex].width = width;
	gizmoList[currentGizmoIndex].height = height;
	gizmoList[currentGizmoIndex].posX = posX;
	gizmoList[currentGizmoIndex].posY = posY;
	gizmoList[currentGizmoIndex].srcX = srcX;
	gizmoList[currentGizmoIndex].srcY = srcY; 
	strcpy(gizmoList[currentGizmoIndex].name,name);
	gizmoList[currentGizmoIndex].canScrollText= false;
	gizmoList[currentGizmoIndex].scrollText = false;
	gizmoList[currentGizmoIndex].proportionalText = false;
	gizmoList[currentGizmoIndex].animationMax = animationMax;
	gizmoList[currentGizmoIndex].animationIndex = 0;
	memcpy(&gizmoList[currentGizmoIndex].animationList[0], animationList, animationMax);

	return(currentGizmoIndex++);	
}

// *****************************************************************************
// skinsClass::createComboBox()
//
// *****************************************************************************
int skinsClass::createComboBox(char * fontName, int textboxX, int textboxY,int textboxWidth, int textboxHeight,
							 int pointSize, int bold, int ital, int underline, COLORREF color, COLORREF aaColor, char * name)
{
	DWORD weight;

	memset(&gizmoList[currentGizmoIndex], 0,sizeof(GIZMO));

	gizmoList[currentGizmoIndex].gizmoType = GIZMO_COMBOBOX;
	gizmoList[currentGizmoIndex].graphicNumber = -1;
	gizmoList[currentGizmoIndex].posX = textboxX;
	gizmoList[currentGizmoIndex].posY = textboxY;
	gizmoList[currentGizmoIndex].hitboxWidth = textboxWidth;
	gizmoList[currentGizmoIndex].hitboxHeight = textboxHeight;

	strcpy(gizmoList[currentGizmoIndex].name,name);

	gizmoList[currentGizmoIndex].canScrollText= false;
	gizmoList[currentGizmoIndex].scrollText = false;

	strcpy(gizmoList[currentGizmoIndex].text,"");

	gizmoList[currentGizmoIndex].fontColor = color;
	gizmoList[currentGizmoIndex].fontAAColor = aaColor;

	weight = FW_NORMAL;

	switch(bold)
		{
		case 1:
			weight = FW_BOLD;
			break;

		case 2:
			weight = FW_EXTRABOLD;
			break;

		case 3:
			weight = FW_ULTRABOLD;
			break;
		}

	gizmoList[currentGizmoIndex].font = CreateFont(pointSize, 0, 0, 0, weight , ital, underline, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, fontName);

	gizmoList[currentGizmoIndex].comboBoxText.size = 2;
	gizmoList[currentGizmoIndex].comboBoxText.numEntries = 0;
	gizmoList[currentGizmoIndex].comboBoxText.entries = (COMBOBOXTEXTENTRYDATA *) malloc(gizmoList[currentGizmoIndex].comboBoxText.size * sizeof(COMBOBOXTEXTENTRYDATA));
	
	return(currentGizmoIndex++);
}

// *****************************************************************************
// skinsClass::createEditBox()
//
// *****************************************************************************
int skinsClass::createEditBox(char * fontName, int textboxX, int textboxY,int textboxWidth, int textboxHeight,
							 int pointSize, int bold, int ital, int underline, COLORREF color, COLORREF aaColor, char * name)
{
	DWORD weight;

	memset(&gizmoList[currentGizmoIndex], 0,sizeof(GIZMO));

	gizmoList[currentGizmoIndex].gizmoType = GIZMO_EDITBOX;
	gizmoList[currentGizmoIndex].graphicNumber = -1;
	gizmoList[currentGizmoIndex].posX = textboxX;
	gizmoList[currentGizmoIndex].posY = textboxY;
	gizmoList[currentGizmoIndex].hitboxWidth = textboxWidth;
	gizmoList[currentGizmoIndex].hitboxHeight = textboxHeight;

	strcpy(gizmoList[currentGizmoIndex].name,name);

	gizmoList[currentGizmoIndex].canScrollText= false;
	gizmoList[currentGizmoIndex].scrollText = false;

	strcpy(gizmoList[currentGizmoIndex].text,"");

	gizmoList[currentGizmoIndex].fontColor = color;
	gizmoList[currentGizmoIndex].fontAAColor = aaColor;

	weight = FW_NORMAL;

	switch(bold)
		{
		case 1:
			weight = FW_BOLD;
			break;

		case 2:
			weight = FW_EXTRABOLD;
			break;

		case 3:
			weight = FW_ULTRABOLD;
			break;
		}

	gizmoList[currentGizmoIndex].font = CreateFont(pointSize, 0, 0, 0, weight , ital, underline, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, fontName);
	gizmoList[currentGizmoIndex].windowsControlHwnd = NULL;
	gizmoList[currentGizmoIndex].windowsControlBackground=CreateSolidBrush(aaColor);
	return(currentGizmoIndex++);
}
// *****************************************************************************
// skinsClass::loadGraphic()
//
// *****************************************************************************
int skinsClass::loadGraphic(char * filename)
{
	int i;
	char * memPtr;
	BITMAPINFO * bitmapInfo;
	BITMAPFILEHEADER * bitmapFileHeader;
	HDC tempHdc;
	int sizeOfFile;

	if(currrentGraphicIndex!=0)
		{
		for(i=0;i<currrentGraphicIndex;i++)
			{
			if(!_stricmp(buttonGraphic[i].filename,filename))
				{
				return(i);
				}
			}
		}

	memPtr = loadFileFromZip(skinZipName, filename, &sizeOfFile);
	if(!memPtr)
		{
		return(false);
		}

	bitmapFileHeader = (BITMAPFILEHEADER *) memPtr;
	bitmapInfo = (BITMAPINFO *) (memPtr+sizeof(BITMAPFILEHEADER));
	 // Create an HBITMAP from the loaded bitmap
	tempHdc = GetDC(NULL);
	buttonGraphic[currrentGraphicIndex].handle = CreateDIBitmap(tempHdc, &bitmapInfo->bmiHeader,CBM_INIT,(unsigned char *)bitmapInfo + bitmapInfo->bmiHeader.biSize,bitmapInfo,DIB_RGB_COLORS);
	ReleaseDC(NULL,tempHdc);
	delete [] memPtr;

	buttonGraphic[currrentGraphicIndex].hdc = CreateCompatibleDC(NULL);
	GetObject(buttonGraphic[currrentGraphicIndex].handle,sizeof(BITMAP), (LPSTR)&buttonGraphic[currrentGraphicIndex].info);
	buttonGraphic[currrentGraphicIndex].handle1 = (HBITMAP) SelectObject(buttonGraphic[currrentGraphicIndex].hdc,buttonGraphic[currrentGraphicIndex].handle);
	strcpy(buttonGraphic[currrentGraphicIndex].filename,filename);
	return(currrentGraphicIndex++);
}

// *****************************************************************************
// skinsClass::setSliderPos()
//
// *****************************************************************************
bool skinsClass::setSliderPos(char * sliderName, int pos)
{
	int i;
	float offset;

	for(i=0;i<MAX_GIZMOS;i++)
		{
		switch(gizmoList[i].gizmoType)
			{
			case GIZMO_HSLIDER:
				if(!strcmp(gizmoList[i].name,sliderName))
					{
					offset = (float)((float)pos / GIZMO_SCROLL_RANGE);
					
					gizmoList[i].posX = gizmoList[i].hitboxPosX + ((int)(offset * (float)(gizmoList[i].hitboxWidth - gizmoList[i].width)));
					return(true);
					}

			case GIZMO_VSLIDER:
				if(!strcmp(gizmoList[i].name,sliderName))
					{
					offset = (float)((float)pos / GIZMO_SCROLL_RANGE);
					
					gizmoList[i].posY = gizmoList[i].hitboxPosY + ((int)(offset * (float)(gizmoList[i].hitboxHeight - gizmoList[i].height)));
					return(true);
					}
			}
		}
	return(false);

}

// *****************************************************************************
// skinsClass::setGizmoMultiState()
//
// *****************************************************************************
bool skinsClass::setGizmoMultiState(char * gizmoName, int multiState)
{
	int i;

	for(i=0;i<MAX_GIZMOS;i++)
		{	
		if(gizmoList[i].multiStateGizmo)
			{
			if(!strcmp(gizmoList[i].name,gizmoName))
				{
				gizmoList[i].multiState = multiState;
				return(true);
				}
			}
		}
	return(false);

}

// *****************************************************************************
// skinsClass::setGizmoMultiState()
//
// *****************************************************************************
bool skinsClass::getGizmoPos(char * gizmoName, RECT * buttonRect)
{
	int i;

	for(i=0;i<MAX_GIZMOS;i++)
		{	
		if(!strcmp(gizmoList[i].name,gizmoName))
			{
			buttonRect->left = gizmoList[i].posX;
			buttonRect->top = gizmoList[i].posY;
			buttonRect->right = gizmoList[i].posX + gizmoList[i].width;
			buttonRect->bottom = gizmoList[i].posY + gizmoList[i].height;
			return(true);
			}
		}
	return(false);

}

// *****************************************************************************
// skinsClass::comboBoxReset()
//
// *****************************************************************************
bool skinsClass::comboBoxReset(char * gizmoName)
{
	int i;

	for(i=0;i<MAX_GIZMOS;i++)
		{	
		if(gizmoList[i].gizmoType == GIZMO_COMBOBOX)
			{
			if(!strcmp(gizmoList[i].name,gizmoName))
				{
				gizmoList[i].comboBoxText.numEntries = 0;
				gizmoList[i].comboBoxRenderStartIndex=0;
				return(true);
				}
			}
		}
	return(false);

}

// *****************************************************************************
// skinsClass::comboBoxInsertString()
//
// *****************************************************************************
bool skinsClass::comboBoxInsertString(char * gizmoName, char * string)
{
	int i;

	for(i=0;i<MAX_GIZMOS;i++)
		{	
		if(gizmoList[i].gizmoType == GIZMO_COMBOBOX)
			{
			if(!strcmp(gizmoList[i].name,gizmoName))
				{
				if((gizmoList[i].comboBoxText.numEntries+1)>=gizmoList[i].comboBoxText.size)
					{
					gizmoList[i].comboBoxText.size +=64;
					gizmoList[i].comboBoxText.entries = (COMBOBOXTEXTENTRYDATA *) realloc(gizmoList[i].comboBoxText.entries, gizmoList[i].comboBoxText.size * sizeof(COMBOBOXTEXTENTRYDATA));	
					}
				strcpy(gizmoList[i].comboBoxText.entries->entry[gizmoList[i].comboBoxText.numEntries].text,string);
				gizmoList[i].comboBoxRenderStartIndex=0;
				gizmoList[i].comboBoxText.numEntries++;
				return(true);
				}
			}
		}
	return(false);

}

// *****************************************************************************
// skinsClass::setComboBoxScrollPos()
//
// *****************************************************************************
bool skinsClass::setComboBoxScrollPos(char * gizmoName, int pos)
{
	int i;
	float scrollPos;

	for(i=0;i<MAX_GIZMOS;i++)
		{	
		if(gizmoList[i].gizmoType == GIZMO_COMBOBOX)
			{
			if(!strcmp(gizmoList[i].name,gizmoName))
				{
				if(gizmoList[i].comboBoxText.numEntries<1)
					{
					gizmoList[i].comboBoxRenderStartIndex=0;
					}
				scrollPos = pos / GIZMO_SCROLL_RANGE;
				
				gizmoList[i].comboBoxRenderStartIndex = (int)((float)(gizmoList[i].comboBoxText.numEntries-1) * scrollPos);
				return(true);
				}
			}
		}
	return(false);

}
// *****************************************************************************
// skinsClass::setText()
//
// *****************************************************************************
bool skinsClass::setText(char * name, char * text)
{
	int i;
	char * ptr;
	int pixelWidth;
	int letterIndex;
	SIZE stringSize;
	int len;

	for(i=0;i<MAX_GIZMOS;i++)
		{
		switch (gizmoList[i].gizmoType)
			{
			case GIZMO_GRAPHIC_TEXTBOX :
				if(!_stricmp(gizmoList[i].name,name))
					{
					strcpy(gizmoList[i].text,text);
					gizmoList[i].scrollIndex = 0;
					if(gizmoList[i].canScrollText)
						{
						strcat(gizmoList[i].text,"      ");
						ptr = gizmoList[i].text;
						pixelWidth = 0;
						while(*ptr)
							{
							if(gizmoList[i].proportionalText)
								{
								letterIndex = textConversionTable[gizmoList[i].text[gizmoList[i].scrollIndex]];
								pixelWidth += buttonGraphic[gizmoList[i].graphicNumber].proportionalTextData[letterIndex].width;
								}
							else
								{
								pixelWidth += gizmoList[i].width;
								}
							ptr++;
							}
						if(pixelWidth>gizmoList[i].hitboxWidth)
							{
							gizmoList[i].scrollText=true;
							//gizmoList[i].scrollPixelOffset=0;
							}
						else
							{
							gizmoList[i].scrollText=false;
							//gizmoList[i].scrollPixelOffset=0;
							}
						}
			
					return(true);
					}
				break;

			case GIZMO_FONT_TEXTBOX:
				if(!_stricmp(gizmoList[i].name,name))
					{
					strcpy(gizmoList[i].text,text);
					gizmoList[i].scrollIndex = 0;
					if(gizmoList[i].canScrollText)
						{
						strcat(gizmoList[i].text,"      ");
						ptr = gizmoList[i].text;
						SelectObject(frontBufferHdc,gizmoList[i].font);
						GetTextExtentPoint32(frontBufferHdc,&gizmoList[i].text[0],strlen(&gizmoList[i].text[0]),&stringSize);
						pixelWidth = stringSize.cx;

						if(pixelWidth>gizmoList[i].hitboxWidth)
							{
							gizmoList[i].scrollText=true;
							}
						else
							{
							gizmoList[i].scrollText=false;
							}
						}
			
					return(true);
					}
				break;
			case GIZMO_EDITBOX:
				if(!_stricmp(gizmoList[i].name,name))
					{
					SetWindowText(gizmoList[i].windowsControlHwnd,text);
					len = strlen(text);
					SendMessage(gizmoList[i].windowsControlHwnd,EM_SETSEL,len,len);
					return(true);
					}
				break;
			}
		}
	return(false);

}
// *****************************************************************************
// skinsClass::setText()
//
// *****************************************************************************
bool skinsClass::getText(char * name, char * text, int maxCount)
{
	int i;

	for(i=0;i<MAX_GIZMOS;i++)
		{
		switch (gizmoList[i].gizmoType)
			{
			case GIZMO_EDITBOX:
				if(!_stricmp(gizmoList[i].name,name))
					{
					GetWindowText(gizmoList[i].windowsControlHwnd,text,maxCount);
					return(true);
					}
				break;
			}
		}
	return(false);
}
// *****************************************************************************
// skinsClass::setText()
//
// *****************************************************************************
void skinsClass::scrollText()
{
	int i;
	int letterWidth;
	int letterIndex;

	for(i=0;i<MAX_GIZMOS;i++)
		{	
		if(gizmoList[i].gizmoType == GIZMO_GRAPHIC_TEXTBOX)
			{			
			if(gizmoList[i].scrollText)
				{
				if(gizmoList[i].text[gizmoList[i].scrollIndex]!=0)
					{
					gizmoList[i].scrollPixelOffset += gizmoList[i].scrollNumPixels;			
					for(;;)
						{
						if(gizmoList[i].proportionalText)
							{
							letterIndex = textConversionTable[gizmoList[i].text[gizmoList[i].scrollIndex]];
							letterWidth = buttonGraphic[gizmoList[i].graphicNumber].proportionalTextData[letterIndex].width;
							}
						else
							{
							letterWidth = gizmoList[i].width;
							}
						if(gizmoList[i].scrollPixelOffset<letterWidth)
							{
							break;
							}

						gizmoList[i].scrollPixelOffset -= letterWidth;
						gizmoList[i].scrollIndex++;
						if(gizmoList[i].text[gizmoList[i].scrollIndex]==0)
							{
							gizmoList[i].scrollIndex=0;
							}
						}
					}
				}
			}
		if(gizmoList[i].gizmoType == GIZMO_FONT_TEXTBOX)
			{			
			if(gizmoList[i].scrollText)
				{
				if(gizmoList[i].text[gizmoList[i].scrollIndex]!=0)
					{
					gizmoList[i].scrollPixelOffset += gizmoList[i].scrollNumPixels;	
					SelectObject(frontBufferHdc,gizmoList[i].font);
					for(;;)
						{
						GetCharWidth32(frontBufferHdc, gizmoList[i].text[gizmoList[i].scrollIndex], gizmoList[i].text[gizmoList[i].scrollIndex], &letterWidth);
						
						if(gizmoList[i].scrollPixelOffset<letterWidth)
							{
							break;
							}

						gizmoList[i].scrollPixelOffset -= letterWidth;
						gizmoList[i].scrollIndex++;
						if(gizmoList[i].text[gizmoList[i].scrollIndex]==0)
							{
							gizmoList[i].scrollIndex=0;
							}
						}
					}
				}
			}

		}
}
// *****************************************************************************
// skinsClass::createProportionalTextData()
//
// *****************************************************************************
bool skinsClass::createProportionalTextData(int graphicIndex)
{
	unsigned int * pixelData;
	BITMAPINFO bitmapInfo;
	int x;
	int y;
	bool hasData;
	bool lookingForData;
	unsigned int * ptr;
	int propIndex;
	HDC hdc;
	int ret;

	pixelData =  new unsigned int [buttonGraphic[graphicIndex].info.bmHeight*buttonGraphic[graphicIndex].info.bmWidth];

	bitmapInfo.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bitmapInfo.bmiHeader.biWidth = buttonGraphic[graphicIndex].info.bmWidth;
	bitmapInfo.bmiHeader.biHeight = -buttonGraphic[graphicIndex].info.bmHeight;
	bitmapInfo.bmiHeader.biPlanes = 1;
	bitmapInfo.bmiHeader.biBitCount = 32;
	bitmapInfo.bmiHeader.biCompression = BI_RGB;
	bitmapInfo.bmiHeader.biSizeImage = 0;
	bitmapInfo.bmiHeader.biXPelsPerMeter = 0;
	bitmapInfo.bmiHeader.biYPelsPerMeter = 0;
	bitmapInfo.bmiHeader.biClrUsed = 0;
	bitmapInfo.bmiHeader.biClrImportant =0;

	hdc = GetDC(NULL);
	ret = GetDIBits(hdc,(HBITMAP)buttonGraphic[graphicIndex].handle,0,buttonGraphic[graphicIndex].info.bmHeight,pixelData,&bitmapInfo,DIB_RGB_COLORS);
	ReleaseDC(NULL,hdc);

	if(!ret)
		{
		delete [] pixelData;
		return(false);
		}

	// We now have the raw pixel data of the text.
	buttonGraphic[graphicIndex].proportionalTextData = new PTEXTDATA [buttonGraphic[graphicIndex].info.bmWidth];
	buttonGraphic[graphicIndex].proportionalTextEntries = 0;

	lookingForData = true;

	propIndex = 0;

	for(x=0;x<buttonGraphic[graphicIndex].info.bmWidth;x++)
		{
		hasData = false;

		for(y=0;y<buttonGraphic[graphicIndex].info.bmHeight;y++)
			{
			ptr = pixelData + ((y * 	buttonGraphic[graphicIndex].info.bmWidth) + x);
			if((*ptr&0x00ffffff)!=(unsigned int)transparentColor)
				{
				hasData = true;
				break;
				}
			}
		if(lookingForData)
			{
			if(hasData)
				{
				// mark X index away as the start of a character
				buttonGraphic[graphicIndex].proportionalTextData[propIndex].startHPixel = x;
				lookingForData=!lookingForData;
				}
			}
		else
			{
			if(!hasData)
				{
				// If we found the first line without data, then flip our flag to looking for data
				lookingForData=!lookingForData;
				// mark X index away as the end of the last character
				buttonGraphic[graphicIndex].proportionalTextData[propIndex].width = x - buttonGraphic[graphicIndex].proportionalTextData[propIndex].startHPixel;
				// Increment the counter for the number of characters found
				propIndex++;
				}
			}
		}
	if(!lookingForData)
		{
		buttonGraphic[graphicIndex].proportionalTextData[propIndex].width = buttonGraphic[graphicIndex].info.bmWidth - buttonGraphic[graphicIndex].proportionalTextData[propIndex].startHPixel;
		// Increment the counter for the number of characters found
		propIndex++;
		}
	buttonGraphic[graphicIndex].proportionalTextEntries = propIndex;

	delete [] pixelData;
	return(true);
}

// *****************************************************************************
// skinsClass::createRegionData()
//
// *****************************************************************************
bool skinsClass::createRegionData(int posX, int posY, int width, int height)
{
	unsigned int * pixelData;
	BITMAPINFO bitmapInfo;
	int y;
	int x;
	HDC hdc;
	int ret;
	unsigned int * curPos;
	bool inside;
	int numRects;

	pixelData =  new unsigned int [windowBase.info.bmHeight*windowBase.info.bmWidth];
	memset (pixelData,0,windowBase.info.bmHeight*windowBase.info.bmWidth*4);

	bitmapInfo.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bitmapInfo.bmiHeader.biWidth = windowBase.info.bmWidth;
	bitmapInfo.bmiHeader.biHeight = -windowBase.info.bmHeight;
	bitmapInfo.bmiHeader.biPlanes = 1;
	bitmapInfo.bmiHeader.biBitCount = 32;
	bitmapInfo.bmiHeader.biCompression = BI_RGB;
	bitmapInfo.bmiHeader.biSizeImage = 0;
	bitmapInfo.bmiHeader.biXPelsPerMeter = 0;
	bitmapInfo.bmiHeader.biYPelsPerMeter = 0;
	bitmapInfo.bmiHeader.biClrUsed = 0;
	bitmapInfo.bmiHeader.biClrImportant =0;

	hdc = GetDC(NULL);
	ret = GetDIBits(hdc,(HBITMAP)windowBase.handle,0,windowBase.info.bmHeight,pixelData,&bitmapInfo,DIB_RGB_COLORS);
	ReleaseDC(NULL,hdc);

	if(!ret)
		{
		delete [] pixelData;
		return(false);
		}
	windowRegionListSize = 4;
	windowRegionListCount = 0;

	windowRegionList = (WINDOWREGIONLIST *) malloc(sizeof(WINDOWREGIONLIST) + (windowRegionListSize * sizeof(RECT)));
	if(!windowRegionList)
		{
		delete [] pixelData;
		return(false);
		}

	inside = false;
	numRects = 0;

	for(y=0;y<height;y++)
		{
		curPos = pixelData + (((posY + y) * windowBase.info.bmWidth) + posX);

		for(x=0;x<width;x++)
			{
			if(inside)
				{
				if((*curPos++&0x00ffffff)==(unsigned int)transparentColor)
					{
					//Terminate the rectangle
					inside = false;
					numRects++;
					windowRegionList->rects[windowRegionListCount].right = x;
					windowRegionListCount++;
					if(windowRegionListCount>windowRegionListSize)
						{
						windowRegionListSize+=256;
						windowRegionList = (WINDOWREGIONLIST *) realloc(windowRegionList,sizeof(WINDOWREGIONLIST) + (windowRegionListSize * sizeof(RECT)));
						if(!windowRegionList)
							{
							delete [] pixelData;
							free(windowRegionList);
							return(false);
							}
						}
					}

				}
			else
				{
				if((*curPos++&0x00ffffff)!=(unsigned int)transparentColor)
					{
					// start the rectangle
					inside = true;
					windowRegionList->rects[windowRegionListCount].left = x;
					windowRegionList->rects[windowRegionListCount].top = y;
					windowRegionList->rects[windowRegionListCount].bottom = y+1;
					}

				}
			}

		if(inside)
			{
			// Terminate the rectangle since we hit the end of the line
			inside = false;
			numRects++;
			windowRegionList->rects[windowRegionListCount].right = x;
			windowRegionListCount++;
			if(windowRegionListCount>windowRegionListSize)
				{
				windowRegionListSize+=256;
				windowRegionList = (WINDOWREGIONLIST *) realloc(windowRegionList,sizeof(WINDOWREGIONLIST) + (windowRegionListSize * sizeof(RECT)));
				if(!windowRegionList)
					{
					delete [] pixelData;
					return(false);
					}
				}
			}
		}
	// Fill in the region structure

	windowRegionList->header.dwSize = sizeof(RGNDATAHEADER);
	windowRegionList->header.iType = RDH_RECTANGLES;
	windowRegionList->header.nCount= numRects;
	windowRegionList->header.nRgnSize = numRects*sizeof(RECT);
	windowRegionList->header.rcBound.top = 0;
	windowRegionList->header.rcBound.left = 0;
	windowRegionList->header.rcBound.right = windowBase.info.bmWidth;
	windowRegionList->header.rcBound.bottom = windowBase.info.bmHeight;

	return(true);

}

// *****************************************************************************
// skinsClass::createWindow()
//
// *****************************************************************************
HWND skinsClass::createWindow(HINSTANCE hInst,  char * windowName, char * className, WNDPROC parentWindowFunction)
{
	HDC hdc;
	int ret;

	resizeWidth = windowBase.info.bmWidth;
	resizeHeight = windowBase.info.bmHeight;
	resizeExtraWidth = 0;
	resizeExtraHeight = 0;
	resizeDrag = false;

	registerWindowClass(hInst,className,parentWindowFunction);
	
	mainhWnd = CreateWindowEx(0, className, windowName, WS_POPUP | WS_SYSMENU | WS_CLIPSIBLINGS,
							0, 0, 
							windowBase.info.bmWidth, windowBase.info.bmHeight, NULL, NULL, hInst, NULL );
	
	hdc = GetDC(NULL);
	frontBufferBitmap = CreateCompatibleBitmap(hdc,windowBase.info.bmWidth,windowBase.info.bmHeight);
	frontBufferHdc = CreateCompatibleDC(NULL);
	frontBufferBitmap = (HBITMAP) SelectObject(frontBufferHdc,frontBufferBitmap);
	BitBlt(frontBufferHdc,0,0,windowBase.info.bmWidth,windowBase.info.bmHeight,windowBase.hdc,0,0,SRCCOPY); //draw BG screen to front display.
	
	ReleaseDC(NULL,hdc);

	if(windowRegionListValid)
		{
		windowRegionHandle = ExtCreateRegion( NULL, sizeof(WINDOWREGIONLIST) + (windowRegionListCount * sizeof(RECT)), (RGNDATA *) windowRegionList);
		ret = SetWindowRgn( mainhWnd, windowRegionHandle, TRUE );
		}

	SetTimer(mainhWnd,0,SKIN_UPDATE_RATE,NULL);
	render();
	
	resizeWindow = false;

	return(mainhWnd);

}

// *****************************************************************************
// skinsClass::createWindow()
//
// *****************************************************************************
HWND skinsClass::createResizeWindow(HINSTANCE hInst,  char * windowName, char * className, WNDPROC parentWindowFunction)
{

	HDC hdc;
	int ret;

	resizeWidth = 640;
	resizeHeight = 480;
	resizeExtraWidth = resizeTilePos[RESIZE_TILE_LEFT].width + resizeTilePos[RESIZE_TILE_RIGHT].width;
	resizeExtraHeight = resizeTilePos[RESIZE_TILE_TOP].height + resizeTilePos[RESIZE_TILE_BOT].height;

	resizeDrag = false;

	registerWindowClass(hInst,className,parentWindowFunction);
	
	mainhWnd = CreateWindowEx(0, className, windowName, WS_POPUP | WS_SYSMENU,
							0, 0, 
							windowBase.info.bmWidth, windowBase.info.bmHeight, NULL, NULL, hInst, NULL );
	
	hdc = GetDC(NULL);
	frontBufferBitmap = CreateCompatibleBitmap(hdc,windowBase.info.bmWidth,windowBase.info.bmHeight);
	frontBufferHdc = CreateCompatibleDC(NULL);
	frontBufferBitmap = (HBITMAP) SelectObject(frontBufferHdc,frontBufferBitmap);
	BitBlt(frontBufferHdc,0,0,windowBase.info.bmWidth,windowBase.info.bmHeight,windowBase.hdc,0,0,SRCCOPY); //draw BG screen to front display.
	
	ReleaseDC(NULL,hdc);

	windowRegionListValid = createResizeRegionData(resizeWidth, resizeHeight);

	if(windowRegionListValid)
		{
		windowRegionHandle = ExtCreateRegion( NULL, sizeof(WINDOWREGIONLIST) + (windowRegionListCount * sizeof(RECT)), (RGNDATA *) windowRegionList);
		ret = SetWindowRgn( mainhWnd, windowRegionHandle, TRUE );
		}

	SetTimer(mainhWnd,0,SKIN_UPDATE_RATE,NULL);
	render();
	
	resizeWindow = true;

	return(mainhWnd);

}

// *****************************************************************************
// skinsClass::createRegionData()
//
// *****************************************************************************
bool skinsClass::createResizeRegionData(int width, int height)
{
	int i;
	int offsetLeft;
	int offsetTop;

	windowRegionListSize = 5 + resizeWindowRegionListCount[0] + resizeWindowRegionListSize[1] + 
								resizeWindowRegionListSize[2] + resizeWindowRegionListSize[3];

	windowRegionListCount = 0;

	windowRegionList = (WINDOWREGIONLIST *) malloc(sizeof(WINDOWREGIONLIST) + (windowRegionListSize * sizeof(RECT)));
	if(!windowRegionList)
		{
		return(false);
		}

	// First insert the rects for the resize areas

	// Top Center
	windowRegionList->rects[windowRegionListCount].left = resizeTilePos[RESIZE_TILE_TOPLEFT].width;
	windowRegionList->rects[windowRegionListCount].right = (resizeTilePos[RESIZE_TILE_LEFT].width + width + resizeTilePos[RESIZE_TILE_RIGHT].width) - resizeTilePos[RESIZE_TILE_TOPRIGHT].width;
	windowRegionList->rects[windowRegionListCount].top = 0;
	windowRegionList->rects[windowRegionListCount].bottom = resizeTilePos[RESIZE_TILE_TOP].height;
	windowRegionListCount++;

	// Bot Center
	windowRegionList->rects[windowRegionListCount].left = resizeTilePos[RESIZE_TILE_BOTLEFT].width;
	windowRegionList->rects[windowRegionListCount].right = (resizeTilePos[RESIZE_TILE_LEFT].width + width + resizeTilePos[RESIZE_TILE_RIGHT].width) - resizeTilePos[RESIZE_TILE_BOTRIGHT].width;
	windowRegionList->rects[windowRegionListCount].top = resizeTilePos[RESIZE_TILE_TOP].height + height;
	windowRegionList->rects[windowRegionListCount].bottom = resizeTilePos[RESIZE_TILE_TOP].height + height + resizeTilePos[RESIZE_TILE_BOT].height;
	windowRegionListCount++;

	// Left Center
	windowRegionList->rects[windowRegionListCount].left = 0;
	windowRegionList->rects[windowRegionListCount].right = resizeTilePos[RESIZE_TILE_LEFT].width;
	windowRegionList->rects[windowRegionListCount].top = resizeTilePos[RESIZE_TILE_TOPLEFT].height;
	windowRegionList->rects[windowRegionListCount].bottom = resizeTilePos[RESIZE_TILE_TOPLEFT].height + height;
	windowRegionListCount++;

	// Right Center
	windowRegionList->rects[windowRegionListCount].left = resizeTilePos[RESIZE_TILE_LEFT].width + width;
	windowRegionList->rects[windowRegionListCount].right = resizeTilePos[RESIZE_TILE_LEFT].width +  width + resizeTilePos[RESIZE_TILE_RIGHT].width;
	windowRegionList->rects[windowRegionListCount].top = resizeTilePos[RESIZE_TILE_TOPRIGHT].height;
	windowRegionList->rects[windowRegionListCount].bottom = resizeTilePos[RESIZE_TILE_TOPRIGHT].height + height;
	windowRegionListCount++;

	// Center
	windowRegionList->rects[windowRegionListCount].left = resizeTilePos[RESIZE_TILE_LEFT].width;
	windowRegionList->rects[windowRegionListCount].right = resizeTilePos[RESIZE_TILE_LEFT].width + width;
	windowRegionList->rects[windowRegionListCount].top = resizeTilePos[RESIZE_TILE_TOP].height;
	windowRegionList->rects[windowRegionListCount].bottom = resizeTilePos[RESIZE_TILE_TOP].height + height;
	windowRegionListCount++;

	// Now insert the TOP LEFT region Data

	for(i=0;i<resizeWindowRegionListCount[0];i++)
		{
		windowRegionList->rects[windowRegionListCount].left = resizeWindowRegionList[0]->rects[i].left;
		windowRegionList->rects[windowRegionListCount].right = resizeWindowRegionList[0]->rects[i].right;
		windowRegionList->rects[windowRegionListCount].top = resizeWindowRegionList[0]->rects[i].top;
		windowRegionList->rects[windowRegionListCount].bottom = resizeWindowRegionList[0]->rects[i].bottom;
		windowRegionListCount++;
		}

	// Now insert the TOP RIGHT region Data
	offsetLeft = (resizeTilePos[RESIZE_TILE_LEFT].width + resizeTilePos[RESIZE_TILE_RIGHT].width + width) - resizeTilePos[RESIZE_TILE_TOPRIGHT].width;
	
	for(i=0;i<resizeWindowRegionListCount[1];i++)
		{
		windowRegionList->rects[windowRegionListCount].left = resizeWindowRegionList[1]->rects[i].left + offsetLeft;
		windowRegionList->rects[windowRegionListCount].right = resizeWindowRegionList[1]->rects[i].right + offsetLeft;
		windowRegionList->rects[windowRegionListCount].top = resizeWindowRegionList[1]->rects[i].top;
		windowRegionList->rects[windowRegionListCount].bottom = resizeWindowRegionList[1]->rects[i].bottom;
		windowRegionListCount++;
		}

	// Now insert the BOT LEFT region Data
	offsetTop = (resizeTilePos[RESIZE_TILE_TOP].height + resizeTilePos[RESIZE_TILE_BOT].height + height) - resizeTilePos[RESIZE_TILE_BOTLEFT].height;
	for(i=0;i<resizeWindowRegionListCount[2];i++)
		{
		windowRegionList->rects[windowRegionListCount].left = resizeWindowRegionList[2]->rects[i].left;
		windowRegionList->rects[windowRegionListCount].right = resizeWindowRegionList[2]->rects[i].right;
		windowRegionList->rects[windowRegionListCount].top = resizeWindowRegionList[2]->rects[i].top + offsetTop;
		windowRegionList->rects[windowRegionListCount].bottom = resizeWindowRegionList[2]->rects[i].bottom + offsetTop;
		windowRegionListCount++;
		}

	// Now insert the BOT RIGHT region Data
	offsetLeft = (resizeTilePos[RESIZE_TILE_LEFT].width + resizeTilePos[RESIZE_TILE_RIGHT].width + width) - resizeTilePos[RESIZE_TILE_BOTRIGHT].width;
	offsetTop = (resizeTilePos[RESIZE_TILE_TOP].height + resizeTilePos[RESIZE_TILE_BOT].height + height) - resizeTilePos[RESIZE_TILE_BOTLEFT].height;
	for(i=0;i<resizeWindowRegionListCount[3];i++)
		{
		windowRegionList->rects[windowRegionListCount].left = resizeWindowRegionList[3]->rects[i].left + offsetLeft;
		windowRegionList->rects[windowRegionListCount].right = resizeWindowRegionList[3]->rects[i].right + offsetLeft;
		windowRegionList->rects[windowRegionListCount].top = resizeWindowRegionList[3]->rects[i].top + offsetTop;
		windowRegionList->rects[windowRegionListCount].bottom = resizeWindowRegionList[3]->rects[i].bottom + offsetTop;
		windowRegionListCount++;
		}

	windowRegionList->header.dwSize = sizeof(RGNDATAHEADER);
	windowRegionList->header.iType = RDH_RECTANGLES;
	windowRegionList->header.nCount= windowRegionListCount;
	windowRegionList->header.nRgnSize = windowRegionListCount*sizeof(RECT);
	windowRegionList->header.rcBound.top = 0;
	windowRegionList->header.rcBound.left = 0;
	windowRegionList->header.rcBound.right = width + resizeTilePos[RESIZE_TILE_TOPLEFT].width + resizeTilePos[RESIZE_TILE_TOPRIGHT].width;
	windowRegionList->header.rcBound.bottom = height + resizeTilePos[RESIZE_TILE_TOPLEFT].height + resizeTilePos[RESIZE_TILE_BOTLEFT].height;

	return(true);
}

// *****************************************************************************
// skinsClass::showWindow()
//
// *****************************************************************************
void skinsClass::showWindow()
{
	// Guarantee that the window has no highlight or button down stats set
	mouseOver(mainhWnd,-32767,-32767);
	mouseUp(mainhWnd,-32767,-32767);
	// Render the default state
	render();
	// Show the window
	ShowWindowAsync(mainhWnd, SW_SHOW);
	visible = true;
}

// *****************************************************************************
// skinsClass::hideWindow()
//
// *****************************************************************************
void skinsClass::hideWindow()
{
	ShowWindowAsync(mainhWnd, SW_HIDE);
	visible = false;
}


// *****************************************************************************
// skinsClass::render()
//
// *****************************************************************************
void skinsClass::render()
{
	int i;
	int srcX,srcY;
	int posX,posY;
	int buttonOffset;
	bool render;
	int letterIndex;
	char * ptr;
	int textCounter;
	int letterWidth;
	int scrollPixelOffset;
	RECT textRect;
	SIZE fontSize;
	int count;

	BitBlt(frontBufferHdc,0,0,windowBase.info.bmWidth,windowBase.info.bmHeight,windowBase.hdc,0,0,SRCCOPY); //draw BG screen to front display.	

	for(i=0;i<MAX_GIZMOS;i++)
		{
		render = false;
		switch(gizmoList[i].gizmoType)
			{
			case GIZMO_BUTTON:
				if(gizmoList[i].state == GIZMO_STATE_BUTTON_UP)
					{
					// If button up and highlighted, then render the animation
					if(gizmoList[i].highlightActive)
						{
						gizmoList[i].highlightIndex++;
						if(gizmoList[i].highlightIndex>=gizmoList[i].highlightMax)
							{
							gizmoList[i].highlightIndex=0;
							}
						if(gizmoList[i].multiStateGizmo)
							{
							buttonOffset=2;
							}
						else
							{
							buttonOffset=1;
							}
						buttonOffset+=  gizmoList[i].highlightIndex;
						srcX = gizmoList[i].srcX + (buttonOffset * gizmoList[i].width);
						srcY = gizmoList[i].srcY + (gizmoList[i].multiState * gizmoList[i].height);
						render = true;
						}
					else
						{
						// If button up and multistate button, then render it 
						if(gizmoList[i].multiStateGizmo)
							{
							buttonOffset= 0;
							srcX = gizmoList[i].srcX + (buttonOffset * gizmoList[i].width);
							srcY = gizmoList[i].srcY + (gizmoList[i].multiState * gizmoList[i].height);
							render = true;
							}
						}
					}
				else
					{
					if(gizmoList[i].multiStateGizmo)
						{
						buttonOffset=1;
						}
					else
						{
						buttonOffset=0;
						}
					srcX = gizmoList[i].srcX + (buttonOffset * gizmoList[i].width);
					srcY = gizmoList[i].srcY + (gizmoList[i].multiState * gizmoList[i].height);
					render = true;
					}
				if(render)
					{
					drawBitmap(frontBufferHdc, gizmoList[i].posX,gizmoList[i].posY, 
								buttonGraphic[gizmoList[i].graphicNumber].hdc, srcX, srcY, 
								gizmoList[i].width,  gizmoList[i].height,gizmoList[i].transparentBlt);
					}
				break;

			case GIZMO_HSLIDER:
				if(gizmoList[i].state == GIZMO_STATE_BUTTON_UP)
					{
					// If button up and highlighted, then render the animation
					if(gizmoList[i].highlightActive)
						{
						gizmoList[i].highlightIndex++;
						if(gizmoList[i].highlightIndex>=gizmoList[i].highlightMax)
							{
							gizmoList[i].highlightIndex=0;
							}
						buttonOffset= 2 +  gizmoList[i].highlightIndex;
						srcX = gizmoList[i].srcX + (buttonOffset * gizmoList[i].width);
						srcY = gizmoList[i].srcY;
						render = true;
						}
					else
						{
						buttonOffset= 0 ;
						srcX = gizmoList[i].srcX + (buttonOffset * gizmoList[i].width);
						srcY = gizmoList[i].srcY;
						render = true;
						}
					}
				else
					{
					buttonOffset= 1;
					srcX = gizmoList[i].srcX + (buttonOffset * gizmoList[i].width);
					srcY = gizmoList[i].srcY;
					render = true;
					}
				if(render)
					{
					drawBitmap(frontBufferHdc, gizmoList[i].posX,gizmoList[i].posY, 
								buttonGraphic[gizmoList[i].graphicNumber].hdc, srcX, srcY, 
								gizmoList[i].width,  gizmoList[i].height,gizmoList[i].transparentBlt);
					}
				break;

			case GIZMO_VSLIDER:
				if(gizmoList[i].state == GIZMO_STATE_BUTTON_UP)
					{
					// If button up and highlighted, then render the animation
					if(gizmoList[i].highlightActive)
						{
						gizmoList[i].highlightIndex++;
						if(gizmoList[i].highlightIndex>=gizmoList[i].highlightMax)
							{
							gizmoList[i].highlightIndex=0;
							}
						buttonOffset= 2 +  gizmoList[i].highlightIndex;
						srcX = gizmoList[i].srcX + (buttonOffset * gizmoList[i].width);
						srcY = gizmoList[i].srcY;
						render = true;
						}
					else
						{
						buttonOffset= 0 ;
						srcX = gizmoList[i].srcX + (buttonOffset * gizmoList[i].width);
						srcY = gizmoList[i].srcY;
						render = true;
						}
					}
				else
					{
					buttonOffset= 1;
					srcX = gizmoList[i].srcX + (buttonOffset * gizmoList[i].width);
					srcY = gizmoList[i].srcY;
					render = true;
					}
				if(render)
					{
					drawBitmap(frontBufferHdc, gizmoList[i].posX,gizmoList[i].posY, 
								buttonGraphic[gizmoList[i].graphicNumber].hdc, srcX, srcY, 
								gizmoList[i].width,  gizmoList[i].height,gizmoList[i].transparentBlt);
					}
				break;
			case GIZMO_FONT_TEXTBOX:				
				textCounter = strlen(gizmoList[i].text);
				if(!textCounter)
					{
					break;
					}
				SelectObject(frontBufferHdc,gizmoList[i].font);
				SetBkMode(frontBufferHdc, TRANSPARENT);

				ptr = &gizmoList[i].text[gizmoList[i].scrollIndex];
				scrollPixelOffset = gizmoList[i].scrollPixelOffset;
				posX = gizmoList[i].posX;
				posY = gizmoList[i].posY;
				textRect.left = posX;
				textRect.top = posY;
				textRect.right = gizmoList[i].posX+gizmoList[i].hitboxWidth;
				textRect.bottom = gizmoList[i].posY + gizmoList[i].hitboxHeight;

				if(gizmoList[i].fontAAColor)
					{
					SetTextColor(frontBufferHdc, gizmoList[i].fontAAColor);	
					while(textCounter--)
						{
						if(*ptr==0)
							{
							ptr = &gizmoList[i].text[0];
							}
						GetCharWidth32(frontBufferHdc, *ptr, *ptr, &letterWidth);
						posX -=scrollPixelOffset;
						scrollPixelOffset = 0;
						if((posX + letterWidth)>= (gizmoList[i].posX + gizmoList[i].hitboxWidth))
							{
							textCounter = 0;
							}
						ExtTextOut(frontBufferHdc,posX-1,posY-1,ETO_CLIPPED,&textRect,ptr,1,NULL);
						ExtTextOut(frontBufferHdc,posX+1,posY+1,ETO_CLIPPED,&textRect,ptr,1,NULL);
						ExtTextOut(frontBufferHdc,posX-1,posY+1,ETO_CLIPPED,&textRect,ptr,1,NULL);
						ExtTextOut(frontBufferHdc,posX+1,posY-1,ETO_CLIPPED,&textRect,ptr,1,NULL);
						posX +=letterWidth;
						ptr++;
						}
					}

				textCounter = strlen(gizmoList[i].text);
				ptr = &gizmoList[i].text[gizmoList[i].scrollIndex];
				scrollPixelOffset = gizmoList[i].scrollPixelOffset;
				posX = gizmoList[i].posX;
				posY = gizmoList[i].posY;

				SetTextColor(frontBufferHdc, gizmoList[i].fontColor);

				while(textCounter--)
					{
					if(*ptr==0)
						{
						ptr = &gizmoList[i].text[0];
						}
					GetCharWidth32(frontBufferHdc, *ptr, *ptr, &letterWidth);

					//DrawText(frontBufferHdc, ptr,1,&textRect,DT_LEFT);
					posX -=scrollPixelOffset;
					scrollPixelOffset = 0;
					if((posX + letterWidth)>= (gizmoList[i].posX + gizmoList[i].hitboxWidth))
						{
						textCounter = 0;
						}

					//Render The char
					ExtTextOut(frontBufferHdc,posX,posY,ETO_CLIPPED,&textRect,ptr,1,NULL);
					posX +=letterWidth;
					ptr++;
					}
				break;

			case GIZMO_COMBOBOX:
				// see if we have anything to do
				if(!gizmoList[i].comboBoxText.numEntries)
					{
					break;
					}
				posX = gizmoList[i].posX;
				posY = gizmoList[i].posY;

				textRect.left = posX;
				textRect.top = posY;
				textRect.right = gizmoList[i].posX+gizmoList[i].hitboxWidth;
				textRect.bottom = gizmoList[i].posY + gizmoList[i].hitboxHeight;

				SelectObject(frontBufferHdc,gizmoList[i].font);
				SetBkMode(frontBufferHdc, TRANSPARENT);
				GetTextExtentPoint32(frontBufferHdc,"Mg",1,&fontSize);
				//fontSize.cy+=1;

				for(count=gizmoList[i].comboBoxRenderStartIndex;count<gizmoList[i].comboBoxText.numEntries;count++)
					{
					ptr = gizmoList[i].comboBoxText.entries->entry[count].text;
					textCounter = strlen(ptr); 
					if(textCounter)
						{
						if(gizmoList[i].fontAAColor)
							{
							SetTextColor(frontBufferHdc, gizmoList[i].fontAAColor);	
							ExtTextOut(frontBufferHdc,posX-1,posY-1,ETO_CLIPPED,&textRect,ptr,textCounter,NULL);
							ExtTextOut(frontBufferHdc,posX+1,posY+1,ETO_CLIPPED,&textRect,ptr,textCounter,NULL);
							ExtTextOut(frontBufferHdc,posX-1,posY+1,ETO_CLIPPED,&textRect,ptr,textCounter,NULL);
							ExtTextOut(frontBufferHdc,posX+1,posY-1,ETO_CLIPPED,&textRect,ptr,textCounter,NULL);
							}
						SetTextColor(frontBufferHdc, gizmoList[i].fontColor);
						ExtTextOut(frontBufferHdc,posX,posY,ETO_CLIPPED,&textRect,ptr,textCounter,NULL);
						}
					posY+=fontSize.cy;
					}
				break;

			case GIZMO_GRAPHIC_TEXTBOX:

				posX = gizmoList[i].posX;
				posY = gizmoList[i].posY;
				textCounter = strlen(gizmoList[i].text);
				if(!textCounter)
					{
					break;
					}
				ptr = &gizmoList[i].text[gizmoList[i].scrollIndex];
				scrollPixelOffset = gizmoList[i].scrollPixelOffset;
		
				while(textCounter--)
					{
					if(*ptr==0)
						{
						ptr = &gizmoList[i].text[0];
						}
					letterIndex = textConversionTable[*ptr];

					srcY = gizmoList[i].srcY;

					if(gizmoList[i].proportionalText)
						{
						if(letterIndex>buttonGraphic[gizmoList[i].graphicNumber].proportionalTextEntries)
							{
							letterIndex = 0;
							}
						srcX = gizmoList[i].srcX + buttonGraphic[gizmoList[i].graphicNumber].proportionalTextData[letterIndex].startHPixel;
						letterWidth = buttonGraphic[gizmoList[i].graphicNumber].proportionalTextData[letterIndex].width;
						}
					else
						{
						srcX = gizmoList[i].srcX + (gizmoList[i].width * letterIndex);
						letterWidth = gizmoList[i].width;
						}
					
					srcX +=scrollPixelOffset;
					letterWidth-=scrollPixelOffset;
					scrollPixelOffset = 0;
					if((posX + letterWidth)>= (gizmoList[i].posX + gizmoList[i].hitboxWidth))
						{
						// The letter would go outside the text box, so adjust the src and dest rects
						letterWidth -= (posX + letterWidth) -  (gizmoList[i].posX + gizmoList[i].hitboxWidth);
						textCounter = 0;
						}
					drawBitmap(frontBufferHdc, posX,posY, 
							buttonGraphic[gizmoList[i].graphicNumber].hdc, srcX, srcY, 
							letterWidth,  gizmoList[i].height,gizmoList[i].transparentBlt);
					posX+= letterWidth;
					ptr++;
					}					
				break;

			case GIZMO_ANIMATION:
				buttonOffset=  gizmoList[i].animationList[gizmoList[i].animationIndex];
				srcX = gizmoList[i].srcX + (buttonOffset * gizmoList[i].width);
				srcY = gizmoList[i].srcY;
				if(srcX >= buttonGraphic[gizmoList[i].graphicNumber].info.bmWidth)
					{
					srcY = ((srcX / buttonGraphic[gizmoList[i].graphicNumber].info.bmWidth)*gizmoList[i].height);
					srcX = 0;
					}
				drawBitmap(frontBufferHdc, gizmoList[i].posX,gizmoList[i].posY, 
							buttonGraphic[gizmoList[i].graphicNumber].hdc, srcX, srcY, 
							gizmoList[i].width,  gizmoList[i].height,gizmoList[i].transparentBlt);
				
				gizmoList[i].animationIndex++;
				if(gizmoList[i].animationIndex>gizmoList[i].animationMax)
					{
					if(gizmoList[i].animationDelay < gizmoList[i].animationLoopDelay)
						{
						gizmoList[i].animationDelay++;
						}
					else
						{
						gizmoList[i].animationIndex = 0;
						gizmoList[i].animationDelay = 0;
						}
					}
				break;

			case GIZMO_HIGHLIGHT_ANIMATION:
				if(gizmoList[i].highlightActive)
					{
					buttonOffset=  gizmoList[i].animationList[gizmoList[i].animationIndex];
					srcX = gizmoList[i].srcX + (buttonOffset * gizmoList[i].width);
					srcY = gizmoList[i].srcY;
					if(srcX >= buttonGraphic[gizmoList[i].graphicNumber].info.bmWidth)
						{
						srcY = ((srcX / buttonGraphic[gizmoList[i].graphicNumber].info.bmWidth)*gizmoList[i].height);
						srcX = 0;
						}
					drawBitmap(frontBufferHdc, gizmoList[i].posX,gizmoList[i].posY, 
								buttonGraphic[gizmoList[i].graphicNumber].hdc, srcX, srcY, 
								gizmoList[i].width,  gizmoList[i].height,gizmoList[i].transparentBlt);
					
					if(gizmoList[i].animationIndex<(gizmoList[i].animationMax-1))
						{
						gizmoList[i].animationIndex++;
						}
					}
				break;
	
			default:
				break;
			}

		}
}

// *****************************************************************************
// skinsClass::adjustPosForResize()
//
// *****************************************************************************
void skinsClass::adjustPosForResize(int * curX, int * curY)
{
	int posX;
	int posY;

	posX = *curX;
	posY = *curY;

	// Check if mouse over lower right tile piece
	if((posX>=(resizeWidth + resizeExtraWidth) - resizeTilePos[RESIZE_TILE_BOTRIGHT].width) &&
		(posX<(resizeWidth + resizeExtraWidth)) &&
		(posY>=(resizeHeight + resizeExtraHeight) - resizeTilePos[RESIZE_TILE_BOTRIGHT].height) &&
		(posY<(resizeHeight + resizeExtraHeight)))
		{
		// Mouse is over bottom right tile so adjust the X & Y pos;
		posX -= (resizeWidth + resizeExtraWidth) - resizeTilePos[RESIZE_TILE_BOTRIGHT].width;
		posY -= (resizeHeight + resizeExtraHeight) - resizeTilePos[RESIZE_TILE_BOTRIGHT].height;
		posX += resizeTilePos[RESIZE_TILE_BOTRIGHT].x;
		posY += resizeTilePos[RESIZE_TILE_BOTRIGHT].y;
		*curX = posX;
		*curY = posY;
		return;
		}	
	// Check if mouse over upper right tile piece
	if((posX>=(resizeWidth + resizeExtraWidth) - resizeTilePos[RESIZE_TILE_TOPRIGHT].width) &&
		(posX<(resizeWidth + resizeExtraWidth)) &&
		(posY< resizeTilePos[RESIZE_TILE_TOPRIGHT].height))
		{
		// Mouse is over upper right tile so adjust the X & Y pos;
		posX -= (resizeWidth + resizeExtraWidth) - resizeTilePos[RESIZE_TILE_TOPRIGHT].width;
		posX += resizeTilePos[RESIZE_TILE_TOPRIGHT].x;
		posY += resizeTilePos[RESIZE_TILE_TOPRIGHT].y;
		*curX = posX;
		*curY = posY;
		return;
		}
	// Check if mouse over lower left tile piece
	if(	(posX< resizeTilePos[RESIZE_TILE_BOTLEFT].width) &&
		(posY>=(resizeHeight + resizeExtraHeight) - resizeTilePos[RESIZE_TILE_BOTLEFT].height) &&
		(posY<(resizeHeight + resizeExtraHeight)))
		{
		// Mouse is over lower left tile so adjust the X & Y pos;
		posY -= (resizeHeight + resizeExtraHeight) - resizeTilePos[RESIZE_TILE_BOTLEFT].height;
		posX += resizeTilePos[RESIZE_TILE_BOTLEFT].x;
		posY += resizeTilePos[RESIZE_TILE_BOTLEFT].y;
		*curX = posX;
		*curY = posY;
		return;
		}	
	// Check if mouse over top left tile piece
	if(	(posX< resizeTilePos[RESIZE_TILE_TOPLEFT].width) &&
		(posY< resizeTilePos[RESIZE_TILE_TOPLEFT].height))
		{
		// Mouse is over top left tile so adjust the X & Y pos;
		posX += resizeTilePos[RESIZE_TILE_TOPLEFT].x;
		posY += resizeTilePos[RESIZE_TILE_TOPLEFT].y;
		*curX = posX;
		*curY = posY;
		return;
		}

	// We're not over any of the sections, so set the mouse coords to out of range values

	*curX = -1;
	*curY = -1;		
}

// *****************************************************************************
// skinsClass::mouseOver()
//
// *****************************************************************************
void skinsClass::mouseOver(HWND hWnd,int posX, int posY)
{
	int i;
	float pos;
	int posInt;
	RECT newRect;

	if(resizeWindow)
		{
		if(resizeDrag)
			{
			//adjust the window rect
			newRect.top = resizeTilePos[RESIZE_TILE_TOP].height;
			newRect.left = resizeTilePos[RESIZE_TILE_LEFT].width;

			newRect.right = newRect.left + (resizeDragStartWidth + (posX - resizeDragPosX));
			if(newRect.right < newRect.left)
				newRect.right = newRect.left;

			newRect.bottom = newRect.top + (resizeDragStartHeight + (posY - resizeDragPosY));
			if(newRect.bottom < newRect.top)
				newRect.bottom = newRect.top;

			// Check Bot Width
			if((newRect.right - newRect.left) < (resizeTilePos[RESIZE_TILE_BOTLEFT].width + resizeTilePos[RESIZE_TILE_BOTRIGHT].width))
				{
				newRect.right =  newRect.left + (resizeTilePos[RESIZE_TILE_BOTLEFT].width + resizeTilePos[RESIZE_TILE_BOTRIGHT].width);
				}
			// Check Top width
			if((newRect.right - newRect.left) < (resizeTilePos[RESIZE_TILE_TOPLEFT].width + resizeTilePos[RESIZE_TILE_TOPRIGHT].width))
				{
				newRect.right =  newRect.left + (resizeTilePos[RESIZE_TILE_TOPLEFT].width + resizeTilePos[RESIZE_TILE_TOPRIGHT].width);
				}
			// Check bot height
			if((newRect.bottom - newRect.top) < (resizeTilePos[RESIZE_TILE_BOTLEFT].height + resizeTilePos[RESIZE_TILE_TOPLEFT].height))
				{
				newRect.bottom =  newRect.top + (resizeTilePos[RESIZE_TILE_BOTLEFT].height + resizeTilePos[RESIZE_TILE_TOPLEFT].height);
				}

			SendMessage(hWnd,WM_SIZING,WMSZ_BOTTOMRIGHT,(LPARAM) &newRect);

			resizeOutputWindow(newRect.right - newRect.left, newRect.bottom - newRect.top);
			UpdateWindow(hWnd);
			return;
			}
		// In resize mode, we adjust mouse events to the base skin coordinates
		adjustPosForResize(&posX,&posY);
		}

	for(i=0;i<MAX_GIZMOS;i++)
		{
		switch(gizmoList[i].gizmoType)
			{
			case GIZMO_BUTTON:
				if(	posX>=gizmoList[i].hitboxPosX && 
					posX<(gizmoList[i].hitboxPosX+gizmoList[i].hitboxWidth) &&
					posY>=gizmoList[i].hitboxPosY && 
					posY<(gizmoList[i].hitboxPosY+gizmoList[i].hitboxHeight))
					{
					// mouse is hovering over this button
					if(gizmoList[i].state == GIZMO_STATE_BUTTON_UP)
						{
						if(!gizmoList[i].highlightActive)
							{
							gizmoList[i].highlightActive = true;
							gizmoList[i].highlightIndex = 0;
							}
						}
					}
				else
					{
					gizmoList[i].highlightActive = false;
					gizmoList[i].state = GIZMO_STATE_BUTTON_UP;
					}
				break;

			case GIZMO_HSLIDER:
				if(gizmoList[i].state == GIZMO_STATE_SLIDER_DRAG)
					{
					// user is dragging the slider
					gizmoList[i].posX = posX - (gizmoList[i].width/2);
					if(gizmoList[i].posX<gizmoList[i].hitboxPosX)
						{
						gizmoList[i].posX = gizmoList[i].hitboxPosX;
						}
					if(gizmoList[i].posX>(gizmoList[i].hitboxPosX + (gizmoList[i].hitboxWidth - gizmoList[i].width)))
						{
						gizmoList[i].posX = gizmoList[i].hitboxPosX + (gizmoList[i].hitboxWidth -gizmoList[i].width);
						}

					pos = (float)(gizmoList[i].posX - gizmoList[i].hitboxPosX);
					// pos now in a range of 0.0 - 1.0
					pos = pos / (gizmoList[i].hitboxWidth - gizmoList[i].width);
					posInt = (int) (pos * GIZMO_SCROLL_RANGE);
					PostMessage(hWnd,gizmoList[i].messageId[gizmoList[i].multiState],(posInt<<16) | SB_THUMBPOSITION,0);
					}
				else
					{
					if(	posX>=gizmoList[i].posX && 
						posX<(gizmoList[i].posX+gizmoList[i].width) &&
						posY>=gizmoList[i].posY && 
						posY<(gizmoList[i].posY+gizmoList[i].height))
						{
						// Mouse is hovering over the slider
						if(gizmoList[i].state == GIZMO_STATE_BUTTON_UP)
							{
							if(!gizmoList[i].highlightActive)
								{
								gizmoList[i].highlightActive = true;
								gizmoList[i].highlightIndex = 0;
								}
							}
						}
					else
						{
						gizmoList[i].highlightActive = false;
						gizmoList[i].state = GIZMO_STATE_BUTTON_UP;
						}
					}
				break;

			case GIZMO_VSLIDER:
				if(gizmoList[i].state == GIZMO_STATE_SLIDER_DRAG)
					{
					// user is dragging the slider
					gizmoList[i].posY = posY - (gizmoList[i].width/2);
					if(gizmoList[i].posY<gizmoList[i].hitboxPosY)
						{
						gizmoList[i].posY = gizmoList[i].hitboxPosY;
						}
					if(gizmoList[i].posY>(gizmoList[i].hitboxPosY + (gizmoList[i].hitboxHeight - gizmoList[i].height)))
						{
						gizmoList[i].posY = gizmoList[i].hitboxPosY + (gizmoList[i].hitboxHeight -gizmoList[i].height);
						}

					pos = (float)(gizmoList[i].posY - gizmoList[i].hitboxPosY);
					// pos now in a range of 0.0 - 1.0
					pos = pos / (gizmoList[i].hitboxHeight - gizmoList[i].height);
					posInt = (int) (pos * GIZMO_SCROLL_RANGE);
					PostMessage(hWnd,gizmoList[i].messageId[gizmoList[i].multiState],(posInt<<16) | SB_THUMBPOSITION,0);
					}
				else
					{
					if(	posX>=gizmoList[i].posX && 
						posX<(gizmoList[i].posX+gizmoList[i].width) &&
						posY>=gizmoList[i].posY && 
						posY<(gizmoList[i].posY+gizmoList[i].height))
						{
						// Mouse is hovering over the slider
						if(gizmoList[i].state == GIZMO_STATE_BUTTON_UP)
							{
							if(!gizmoList[i].highlightActive)
								{
								gizmoList[i].highlightActive = true;
								gizmoList[i].highlightIndex = 0;
								}
							}
						}
					else
						{
						gizmoList[i].highlightActive = false;
						gizmoList[i].state = GIZMO_STATE_BUTTON_UP;
						}
					}
				break;
			case GIZMO_HIGHLIGHT_ANIMATION:
				if(	posX>=gizmoList[i].posX && 
					posX<(gizmoList[i].posX+gizmoList[i].width) &&
					posY>=gizmoList[i].posY && 
					posY<(gizmoList[i].posY+gizmoList[i].height))
					{
					if(!gizmoList[i].highlightActive)
						{
						gizmoList[i].highlightActive = true;
						gizmoList[i].animationIndex = 0;
						}
					
					}
				else
					{
					if((gizmoList[i].animationIndex<gizmoList[i].animationMax) && gizmoList[i].highlightActive)
						{
						gizmoList[i].highlightActive = false;
						gizmoList[i].animationIndex = 0;
						}
					}
			default:
				break;
			}
		}
}

// *****************************************************************************
// skinsClass::mouseDown()
//
// *****************************************************************************
bool skinsClass::mouseDown(HWND hWnd, int posX, int posY)
{
	int i;
	float pos;
	int posInt;
	int originalX;
	int originalY;

	originalX = posX;
	originalY = posY;


	if(resizeWindow)
		{
		// In resize mode, we adjust mouse events to the base skin coordinates
		adjustPosForResize(&posX,&posY);
		if((posX	>= resizeTilePos[RESIZE_TILE_BOTRIGHT].x) &&
			(posY >= resizeTilePos[RESIZE_TILE_BOTRIGHT].y) &&
			(posX < (resizeTilePos[RESIZE_TILE_BOTRIGHT].x  + resizeTilePos[RESIZE_TILE_BOTRIGHT].width)) &&
			(posY < (resizeTilePos[RESIZE_TILE_BOTRIGHT].y + resizeTilePos[RESIZE_TILE_BOTRIGHT].height)))
			{
			if(!resizeDrag)
				{
				resizeDragStartWidth = resizeWidth;
				resizeDragStartHeight = resizeHeight;
				resizeDragPosX = originalX;
				resizeDragPosY = originalY;
				SetCapture(hWnd);
				resizeDrag = true;
				}
			return(true);
			}

		}

	for(i=0;i<MAX_GIZMOS;i++)
		{
		switch(gizmoList[i].gizmoType)
			{
			case GIZMO_BUTTON:
				if(	posX>=gizmoList[i].hitboxPosX && 
					posX<(gizmoList[i].hitboxPosX+gizmoList[i].hitboxWidth) &&
					posY>=gizmoList[i].hitboxPosY && 
					posY<(gizmoList[i].hitboxPosY+gizmoList[i].hitboxHeight))
					{
					gizmoList[i].state = GIZMO_STATE_BUTTON_DOWN;
					return(true);
					}
				break;
			case GIZMO_HSLIDER:
				if(	posX>=gizmoList[i].hitboxPosX && 
					posX<(gizmoList[i].hitboxPosX+gizmoList[i].hitboxWidth) &&
					posY>=gizmoList[i].hitboxPosY && 
					posY<(gizmoList[i].hitboxPosY+gizmoList[i].hitboxHeight))
					{
					if(	posX>=gizmoList[i].posX && 
						posX<(gizmoList[i].posX+gizmoList[i].width) &&
						posY>=gizmoList[i].posY && 
						posY<(gizmoList[i].posY+gizmoList[i].height))
						{
						// Button down on Slider
						gizmoList[i].state = GIZMO_STATE_SLIDER_DRAG;
						// Capture the mouse ?
						SetCapture(hWnd);
						}
					else
						{
						pos = (float)(gizmoList[i].posX - gizmoList[i].hitboxPosX);
						// pos now in a range of 0.0 - 1.0
						pos = pos / (gizmoList[i].hitboxWidth - gizmoList[i].width);

						// Button down over the slider bar, but not the slider itself
						// Send message of move left/right of current pos
						if(posX<(gizmoList[i].posX + (gizmoList[i].width/2)))
							{
							pos-= 0.20f;
							if(pos<0.0f)
								{
								pos = 0.0f;
								}
							posInt = (int) (pos * GIZMO_SCROLL_RANGE);
							PostMessage(hWnd,gizmoList[i].messageId[gizmoList[i].multiState],(posInt<<16) | SB_PAGELEFT,0);
							gizmoList[i].posX = gizmoList[i].hitboxPosX + ((int)(pos * (float)gizmoList[i].hitboxWidth));
							}
						else
							{
							pos+= 0.20f;
							if(pos>1.0f)
								{
								pos = 1.0f;
								}
							posInt = (int) (pos * GIZMO_SCROLL_RANGE);
							PostMessage(hWnd,gizmoList[i].messageId[gizmoList[i].multiState],(posInt<<16) | SB_PAGERIGHT,0);	
							gizmoList[i].posX = gizmoList[i].hitboxPosX + ((int)(pos * (float)gizmoList[i].hitboxWidth));
							}
						}
					return(true);
					}
				break;
			case GIZMO_VSLIDER:
				if(	posX>=gizmoList[i].hitboxPosX && 
					posX<(gizmoList[i].hitboxPosX+gizmoList[i].hitboxWidth) &&
					posY>=gizmoList[i].hitboxPosY && 
					posY<(gizmoList[i].hitboxPosY+gizmoList[i].hitboxHeight))
					{
					if(	posX>=gizmoList[i].posX && 
						posX<(gizmoList[i].posX+gizmoList[i].width) &&
						posY>=gizmoList[i].posY && 
						posY<(gizmoList[i].posY+gizmoList[i].height))
						{
						// Button down on Slider
						gizmoList[i].state = GIZMO_STATE_SLIDER_DRAG;
						// Capture the mouse ?
						SetCapture(hWnd);
						}
					else
						{
						pos = (float)(gizmoList[i].posY - gizmoList[i].hitboxPosY);
						// pos now in a range of 0.0 - 1.0
						pos = pos / (gizmoList[i].hitboxHeight - gizmoList[i].height);

						// Button down over the slider bar, but not the slider itself
						// Send message of move left/right of current pos
						if(posY<(gizmoList[i].posY + (gizmoList[i].height/2)))
							{
							pos-= 0.20f;
							if(pos<0.0f)
								{
								pos = 0.0f;
								}
							posInt = (int) (pos * GIZMO_SCROLL_RANGE);
							PostMessage(hWnd,gizmoList[i].messageId[gizmoList[i].multiState],(posInt<<16) | SB_PAGEUP,0);
							gizmoList[i].posY = gizmoList[i].hitboxPosY + ((int)(pos * (float)gizmoList[i].hitboxHeight));
							}
						else
							{
							pos+= 0.20f;
							if(pos>1.0f)
								{
								pos = 1.0f;
								}
							posInt = (int) (pos * GIZMO_SCROLL_RANGE);
							PostMessage(hWnd,gizmoList[i].messageId[gizmoList[i].multiState],(posInt<<16) | SB_PAGEDOWN,0);	
							gizmoList[i].posY = gizmoList[i].hitboxPosY + ((int)(pos * (float)gizmoList[i].hitboxHeight));
							}
						}
					return(true);
					}
				break;
			default:
				break;
			}
		}
	return(false);
}

// *****************************************************************************
// skinsClass::mouseDown()
//
// *****************************************************************************
void skinsClass::mouseUp(HWND hWnd, int posX, int posY)
{
	int i;

	if(resizeWindow)
		{
		if(resizeDrag)
			{
			ReleaseCapture();
			resizeDrag = false;
			return;
			}
		// In resize mode, we adjust mouse events to the base skin coordinates
		adjustPosForResize(&posX,&posY);
		}

	for(i=0;i<MAX_GIZMOS;i++)
		{
		switch(gizmoList[i].gizmoType)
			{
			case GIZMO_BUTTON:
				if(	posX>=gizmoList[i].hitboxPosX && 
					posX<(gizmoList[i].hitboxPosX+gizmoList[i].hitboxWidth) &&
					posY>=gizmoList[i].hitboxPosY && 
					posY<(gizmoList[i].hitboxPosY+gizmoList[i].hitboxHeight))
					{
					// Mouse up over a button
					// Send windows message for the command to happen
					PostMessage(hWnd,gizmoList[i].messageId[gizmoList[i].multiState],0,0);
					}
				gizmoList[i].state = GIZMO_STATE_BUTTON_UP;
				break;
			case GIZMO_HSLIDER:
				if(gizmoList[i].state == GIZMO_STATE_SLIDER_DRAG)
					{
					ReleaseCapture();
					}
				gizmoList[i].state = GIZMO_STATE_SLIDER_UP;
				break;
			case GIZMO_VSLIDER:
				if(gizmoList[i].state == GIZMO_STATE_SLIDER_DRAG)
					{
					ReleaseCapture();
					}
				gizmoList[i].state = GIZMO_STATE_SLIDER_UP;
				break;
			default:
				break;
			}
		}
}

// *****************************************************************************
// skinsClass::paintResizeWindow()
//
// *****************************************************************************
void skinsClass::paintResizeWindow(HDC hdc)
{
	int posX;
	int	posY;
	int fillSize;

	// TOP LEFT
	BitBlt(hdc,0,0,resizeTilePos[RESIZE_TILE_TOPLEFT].width,resizeTilePos[RESIZE_TILE_TOPLEFT].height,frontBufferHdc,resizeTilePos[RESIZE_TILE_TOPLEFT].x,resizeTilePos[RESIZE_TILE_TOPLEFT].y,SRCCOPY); //draw BG screen to front display.				

	//StretchBlt(hdc,0,0, resizeTilePos[RESIZE_TILE_TOPLEFT].width,  resizeTilePos[RESIZE_TILE_TOPLEFT].height,
	//				frontBufferHdc,resizeTilePos[RESIZE_TILE_TOPLEFT].x,resizeTilePos[RESIZE_TILE_TOPLEFT].y,resizeTilePos[RESIZE_TILE_TOPLEFT].width,  resizeTilePos[RESIZE_TILE_TOPLEFT].height,SRCCOPY);

	// TOP CENTER FILL
	fillSize = (resizeTilePos[RESIZE_TILE_LEFT].width + resizeTilePos[RESIZE_TILE_RIGHT].width + resizeWidth) - (resizeTilePos[RESIZE_TILE_TOPLEFT].width + resizeTilePos[RESIZE_TILE_TOPRIGHT].width);
	posX = resizeTilePos[RESIZE_TILE_TOPLEFT].width;
	posY = 0;
	while(fillSize>=0)
		{
		BitBlt(hdc,posX,posY,resizeTilePos[RESIZE_TILE_TOP].width,resizeTilePos[RESIZE_TILE_TOP].height,frontBufferHdc,resizeTilePos[RESIZE_TILE_TOP].x,resizeTilePos[RESIZE_TILE_TOP].y,SRCCOPY); //draw BG screen to front display.				
		posX+=resizeTilePos[RESIZE_TILE_TOP].width;
		fillSize-=resizeTilePos[RESIZE_TILE_TOP].width;
		}

	// TOP RIGHT
	posX = (resizeTilePos[RESIZE_TILE_LEFT].width + resizeTilePos[RESIZE_TILE_RIGHT].width + resizeWidth) - resizeTilePos[RESIZE_TILE_TOPRIGHT].width;
	posY = 0;
	BitBlt(hdc,posX,posY,resizeTilePos[RESIZE_TILE_TOPRIGHT].width,resizeTilePos[RESIZE_TILE_TOPRIGHT].height,frontBufferHdc,resizeTilePos[RESIZE_TILE_TOPRIGHT].x,resizeTilePos[RESIZE_TILE_TOPRIGHT].y,SRCCOPY); //draw BG screen to front display.				
	
	// LEFT CENTER FILL
	fillSize = (resizeTilePos[RESIZE_TILE_TOP].height + resizeTilePos[RESIZE_TILE_BOT].height + resizeHeight) - (resizeTilePos[RESIZE_TILE_TOPLEFT].height + resizeTilePos[RESIZE_TILE_BOTLEFT].height);
	posX = 0;
	posY = resizeTilePos[RESIZE_TILE_TOPLEFT].height;
	while(fillSize>=0)
		{
		BitBlt(hdc,posX,posY,resizeTilePos[RESIZE_TILE_LEFT].width,resizeTilePos[RESIZE_TILE_LEFT].height,frontBufferHdc,resizeTilePos[RESIZE_TILE_LEFT].x,resizeTilePos[RESIZE_TILE_LEFT].y,SRCCOPY); //draw BG screen to front display.				
		posY+=resizeTilePos[RESIZE_TILE_LEFT].height;
		fillSize-=resizeTilePos[RESIZE_TILE_LEFT].height;
		}

	// RIGHT CENTER FILL
	fillSize = (resizeTilePos[RESIZE_TILE_TOP].height + resizeTilePos[RESIZE_TILE_BOT].height + resizeHeight) - (resizeTilePos[RESIZE_TILE_TOPLEFT].height + resizeTilePos[RESIZE_TILE_BOTLEFT].height);
	posX = (resizeTilePos[RESIZE_TILE_LEFT].width + resizeTilePos[RESIZE_TILE_RIGHT].width + resizeWidth) - resizeTilePos[RESIZE_TILE_RIGHT].width;
	posY = resizeTilePos[RESIZE_TILE_TOPRIGHT].height;
	while(fillSize>=0)
		{
		BitBlt(hdc,posX,posY,resizeTilePos[RESIZE_TILE_RIGHT].width,resizeTilePos[RESIZE_TILE_RIGHT].height,frontBufferHdc,resizeTilePos[RESIZE_TILE_RIGHT].x,resizeTilePos[RESIZE_TILE_RIGHT].y,SRCCOPY); //draw BG screen to front display.				
		posY+=resizeTilePos[RESIZE_TILE_RIGHT].height;
		fillSize-=resizeTilePos[RESIZE_TILE_RIGHT].height;
		}


	// BOT LEFT
	posX = 0;
	posY = resizeTilePos[RESIZE_TILE_TOP].height + resizeHeight;
	BitBlt(hdc,posX,posY,resizeTilePos[RESIZE_TILE_BOTLEFT].width,resizeTilePos[RESIZE_TILE_BOTLEFT].height,frontBufferHdc,resizeTilePos[RESIZE_TILE_BOTLEFT].x,resizeTilePos[RESIZE_TILE_BOTLEFT].y,SRCCOPY); //draw BG screen to front display.				

	// BOT CENTER FILL
	fillSize = (resizeTilePos[RESIZE_TILE_LEFT].width + resizeTilePos[RESIZE_TILE_RIGHT].width + resizeWidth) - (resizeTilePos[RESIZE_TILE_BOTLEFT].width + resizeTilePos[RESIZE_TILE_BOTRIGHT].width);
	posX = resizeTilePos[RESIZE_TILE_BOTLEFT].width;
	posY = resizeTilePos[RESIZE_TILE_TOP].height + resizeHeight;
	while(fillSize>=0)
		{
		BitBlt(hdc,posX,posY,resizeTilePos[RESIZE_TILE_BOT].width,resizeTilePos[RESIZE_TILE_BOT].height,frontBufferHdc,resizeTilePos[RESIZE_TILE_BOT].x,resizeTilePos[RESIZE_TILE_BOT].y,SRCCOPY); //draw BG screen to front display.				
		posX+=resizeTilePos[RESIZE_TILE_BOT].width;
		fillSize-=resizeTilePos[RESIZE_TILE_BOT].width;
		}

	// BOT RIGHT
	posX = (resizeTilePos[RESIZE_TILE_LEFT].width + resizeTilePos[RESIZE_TILE_RIGHT].width + resizeWidth) - resizeTilePos[RESIZE_TILE_BOTRIGHT].width;
	posY = resizeTilePos[RESIZE_TILE_TOP].height + resizeHeight;
	BitBlt(hdc,posX,posY,resizeTilePos[RESIZE_TILE_BOTRIGHT].width,resizeTilePos[RESIZE_TILE_BOTRIGHT].height,frontBufferHdc,resizeTilePos[RESIZE_TILE_BOTRIGHT].x,resizeTilePos[RESIZE_TILE_BOTRIGHT].y,SRCCOPY); //draw BG screen to front display.				

}

// *****************************************************************************
// skinsClass::getInnerWindowOffsets()
//
// *****************************************************************************
void skinsClass::getInnerWindowOffsets(int * offsetX, int * offsetY)
{
	*offsetX = resizeTilePos[RESIZE_TILE_LEFT].width;
	*offsetY = resizeTilePos[RESIZE_TILE_TOP].height;
}

// *****************************************************************************
// skinsClass::getInnerRect()
//
// *****************************************************************************
void skinsClass::getInnerRect(RECT * innerRect)
{
	innerRect->top = resizeTilePos[RESIZE_TILE_TOP].height;
	innerRect->left = resizeTilePos[RESIZE_TILE_LEFT].width;
	innerRect->bottom = innerRect->top + resizeHeight;
	innerRect->right = innerRect->left + resizeWidth;

}

// *****************************************************************************
// skinsClass::resizeOutputWindow()
//
// *****************************************************************************
void skinsClass::resizeOutputWindow(int innerWidth, int innerHeight)
{
	int ret;
	WINDOWREGIONLIST * oldWindowRegionList;
	RECT win;

	if(resizeWindow)
		{
		oldWindowRegionList = windowRegionList;
		windowRegionListValid = createResizeRegionData(innerWidth, innerHeight);

		if(windowRegionListValid)
			{
			windowRegionHandle = ExtCreateRegion( NULL, sizeof(WINDOWREGIONLIST) + (windowRegionListCount * sizeof(RECT)), (RGNDATA *) windowRegionList);
			ret = SetWindowRgn( mainhWnd, windowRegionHandle, TRUE );
			free(oldWindowRegionList);
			resizeWidth = innerWidth;
			resizeHeight = innerHeight;
			GetWindowRect(mainhWnd, &win);
			MoveWindow(mainhWnd, win.left, win.top ,resizeWidth + resizeExtraWidth, resizeHeight + resizeExtraHeight, TRUE );
			}
		}
}

// *****************************************************************************
// skinsClass::registerWindowClass()
//
//
// *****************************************************************************
bool skinsClass::registerWindowClass(HINSTANCE hInst,char * className, WNDPROC parentWindowFunction)
{
	//Main Window
	windCls.style         = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
    windCls.lpfnWndProc   = parentWindowFunction;
    windCls.cbClsExtra    = 0;
    windCls.cbWndExtra    = 0;
    windCls.hInstance     = hInst;          
    windCls.hIcon         = LoadIcon(hInst, MAKEINTRESOURCE(VidomiIcon));
    windCls.hCursor       = LoadCursor(NULL, IDC_ARROW);
    windCls.hbrBackground = HBRUSH(0);
    windCls.lpszMenuName  = NULL; 
	windCls.lpszClassName = className;
	
    if (!RegisterClass( &windCls))
		{
		logPrintf("Failed To create window Class\n");
        return FALSE;
		}

	return TRUE;
}

// *****************************************************************************
// skinsClass::skinMessageHandler()
//
//
// *****************************************************************************
BOOL skinsClass::skinMessageHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	int posX, posY;
	DWORD curTime;
	RECT tempRect;
	int i;

	if(!visible)
		{
		return(FALSE);
		}
	switch(message)
		{
		case WM_SHOWWINDOW :
			if (wParam)
				{
				for(i=0;i<MAX_GIZMOS;i++)
					{	
					switch(gizmoList[i].gizmoType)
						{
						case GIZMO_EDITBOX:
							if(!gizmoList[i].windowsControlHwnd)
								{
								gizmoList[i].windowsControlHwnd= CreateWindowEx(0, "EDIT", "", WS_CHILD | WS_VISIBLE | ES_AUTOHSCROLL, 
												gizmoList[i].posX, gizmoList[i].posY, 
												gizmoList[i].hitboxWidth, gizmoList[i].hitboxHeight, hWnd, (HMENU)SKIN_EDIT_CONTROL_MESSAGE, globals->hInst, NULL );

								SendMessage(gizmoList[i].windowsControlHwnd, WM_SETFONT, (WPARAM)gizmoList[i].font, (LPARAM)0 );											 
								}
							break;
						}
					}

				}
			break;
		case WM_CTLCOLOREDIT:
			for(i=0;i<MAX_GIZMOS;i++)
				{	
				switch(gizmoList[i].gizmoType)
					{
					case GIZMO_EDITBOX:
						if(gizmoList[i].windowsControlHwnd == (HWND) lParam)
							{
							//return the brush for the background of the edit box
							//SelectObject((HDC)wParam,gizmoList[i].windowsControlBackground);
							SetTextColor((HDC)wParam, gizmoList[i].fontColor);	
							SetBkMode((HDC)wParam, OPAQUE);
							SetBkColor((HDC)wParam,gizmoList[i].fontAAColor);
							return((int)gizmoList[i].windowsControlBackground);
							}	
					}
				}
			break;

		case WM_TIMER:
			// Rerender a max of 15 times per second
			//SetTimer(mainhWnd,0,SKIN_UPDATE_RATE,NULL);
			if(!windowMoving)
				{
				curTime = timeGetTime();
				if(curTime>(lastScrollTime+scrollRate))
					{
					lastScrollTime = curTime;
					scrollText();
					}
				render();

				if(resizeWindow)
					{
					//InvalidateRect( hWnd, NULL, FALSE );  
					
					//Top
					tempRect.left = 0;
					tempRect.top = 0;
					tempRect.right = resizeWidth + resizeExtraWidth;
					tempRect.bottom = resizeTilePos[RESIZE_TILE_TOP].height;
					InvalidateRect( hWnd, &tempRect, FALSE );  
					//Bottom
					tempRect.left = 0;
					tempRect.top = resizeTilePos[RESIZE_TILE_TOP].height + resizeHeight;
					tempRect.right = resizeWidth + resizeExtraWidth;
					tempRect.bottom = resizeHeight + resizeExtraHeight;
					InvalidateRect( hWnd, &tempRect, FALSE );  
					//Left
					tempRect.left = 0;
					tempRect.top = 0;
					tempRect.right = resizeTilePos[RESIZE_TILE_LEFT].width;
					tempRect.bottom = resizeHeight + resizeExtraHeight;
					InvalidateRect( hWnd, &tempRect, FALSE );  
					//Right
					tempRect.left = resizeTilePos[RESIZE_TILE_LEFT].width + resizeWidth;
					tempRect.top = 0;
					tempRect.right = resizeWidth + resizeExtraWidth;
					tempRect.bottom = resizeHeight + resizeExtraHeight;
					InvalidateRect( hWnd, &tempRect, FALSE );  
					
					}
				else
					{
					InvalidateRect( hWnd, NULL, FALSE );
					for(i=0;i<MAX_GIZMOS;i++)
						{	
						switch(gizmoList[i].gizmoType)
							{
							case GIZMO_EDITBOX:
								tempRect.left = gizmoList[i].posX;
								tempRect.top = gizmoList[i].posY;
								tempRect.right = gizmoList[i].posX + gizmoList[i].hitboxWidth;
								tempRect.bottom = gizmoList[i].posY + gizmoList[i].hitboxHeight;
								ValidateRect(hWnd,&tempRect);
								break;
							}
						}
					}
				}
			return(TRUE);

		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			if(resizeWindow)
				{
				paintResizeWindow(hdc);
				}
			else
				{
				BitBlt(hdc,0,0,windowBase.info.bmWidth,windowBase.info.bmHeight,frontBufferHdc,0,0,SRCCOPY); //draw BG screen to front display.				
				}
			EndPaint (hWnd, &ps);

			for(i=0;i<MAX_GIZMOS;i++)
				{	
				switch(gizmoList[i].gizmoType)
					{
					case GIZMO_EDITBOX:
						UpdateWindow(gizmoList[i].windowsControlHwnd);
						break;
					}
				}

			return(TRUE);

		case WM_LBUTTONDOWN:
			posX = (short) LOWORD(lParam);
			posY = (short) HIWORD(lParam);
			if(!mouseDown(hWnd,posX,posY))
				{
				// If the user didn't hit a button, then assume it's a move
				if (!windowMoving)
					{
					startMovePos.x = (short)LOWORD(lParam);
					startMovePos.y = (short)HIWORD(lParam);
					SetCapture(hWnd);
					windowMoving = true;
					}
				}
			return(TRUE);

		case WM_MOUSEMOVE:
			if (windowMoving)
				{
				endMovePos.x = (short)LOWORD( lParam );
				endMovePos.y = (short)HIWORD( lParam );
				ClientToScreen(hWnd,&endMovePos);
				MoveWindow( hWnd, endMovePos.x - startMovePos.x, endMovePos.y - startMovePos.y,resizeWidth + resizeExtraWidth, resizeHeight + resizeExtraHeight, TRUE );
				}
			else
				{
				posX = (short) LOWORD(lParam);
				posY = (short) HIWORD(lParam);
				mouseOver(hWnd,posX,posY);
				if(resizeDrag)
					{
					return(FALSE);
					}
				}
			return(TRUE);

		case WM_LBUTTONUP:
			posX = (short) LOWORD(lParam);
			posY = (short) HIWORD(lParam);

			if (windowMoving)
				{
				ReleaseCapture();
				}
			windowMoving = false;
			mouseUp(hWnd,posX,posY);
			return (TRUE); 

		}
	return(FALSE);
}

// *****************************************************************************
// skinsClass::drawBitmap()
//
// Draw a bitmap to the dest location. Draw with tranparency if set.
// *****************************************************************************
void skinsClass::drawBitmap(HDC destHdc, int destX, int destY, HDC srcHdc, int srcX,int srcY, int srcWidth, int srcHeight, bool transp)
{
	if(transp)
		{
		drawTransparentBitmap(destHdc, destX,destY, 
					srcHdc, srcX, srcY, srcWidth,  srcHeight);
		}
	else
		{
		StretchBlt(destHdc,destX,destY, srcWidth,  srcHeight,
					srcHdc,srcX,srcY,srcWidth, srcHeight,SRCCOPY);
		}
}

// *****************************************************************************
// skinsClass::drawTransparentBitmap()
//
// Code taken from MS website. Modified to fit how I do things and to be a bit faster.
// *****************************************************************************
void skinsClass::drawTransparentBitmap(HDC destHdc, int destX, int destY, HDC srcHdc, int srcX,int srcY, int srcWidth, int srcHeight)
{
	COLORREF   cColor;
	HBITMAP    bmAndBack, bmAndObject, bmAndMem, bmTemp;
	HBITMAP    bmBackOld, bmObjectOld, bmMemOld, bmTempOld;
	HDC        hdcMem, hdcBack, hdcObject, hdcTemp;
	POINT      ptSize;

	ptSize.x = srcWidth;            // Get width of bitmap
	ptSize.y = srcHeight;           // Get height of bitmap

	// Create some DCs to hold temporary data.
	hdcBack   = CreateCompatibleDC(destHdc);
	hdcObject = CreateCompatibleDC(destHdc);
	hdcMem    = CreateCompatibleDC(destHdc);
	hdcTemp   = CreateCompatibleDC(destHdc);

	// Create a bitmap for each DC. DCs are required for a number of
	// GDI functions.

	// Monochrome DC
	bmAndBack   = CreateBitmap(ptSize.x, ptSize.y, 1, 1, NULL);

	// Monochrome DC
	bmAndObject = CreateBitmap(ptSize.x, ptSize.y, 1, 1, NULL);

	bmAndMem    = CreateCompatibleBitmap(destHdc, ptSize.x, ptSize.y);
	bmTemp      = CreateCompatibleBitmap(destHdc, ptSize.x, ptSize.y);

	// Each DC must select a bitmap object to store pixel data.
	bmBackOld   = (HBITMAP) SelectObject(hdcBack, bmAndBack);
	bmObjectOld = (HBITMAP) SelectObject(hdcObject, bmAndObject);
	bmMemOld    = (HBITMAP) SelectObject(hdcMem, bmAndMem);
	bmTempOld   = (HBITMAP) SelectObject(hdcTemp, bmTemp);

	// Set proper mapping mode.
	// SetMapMode(hdcTemp, GetMapMode(hdc));

	// Save the bitmap sent here, because it will be overwritten.
	//BitBlt(hdcSave, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);
	StretchBlt(hdcTemp,0,0, ptSize.x, ptSize.y,srcHdc,srcX,srcY,ptSize.x, ptSize.y,SRCCOPY);

	// SetMapMode(hdcSave, GetMapMode(hdc));

	// Set the background color of the source DC to the color.
	// contained in the parts of the bitmap that should be transparent
	cColor = SetBkColor(hdcTemp, transparentColor);

	// Create the object mask for the bitmap by performing a BitBlt
	// from the source bitmap to a monochrome bitmap.
	BitBlt(hdcObject, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0,
		  SRCCOPY);

	// Set the background color of the source DC back to the original
	// color.
	SetBkColor(hdcTemp, cColor);

	// Create the inverse of the object mask.
	BitBlt(hdcBack, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0,
		  NOTSRCCOPY);

	// Copy the background of the main DC to the destination.
	BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, destHdc, destX, destY,
		  SRCCOPY);

	// Mask out the places where the bitmap will be placed.
	BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, SRCAND);

	// Mask out the transparent colored pixels on the bitmap.
	BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcBack, 0, 0, SRCAND);

	// XOR the bitmap with the background on the destination DC.
	BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCPAINT);

	// Copy the destination to the screen.
	BitBlt(destHdc, destX, destY, ptSize.x, ptSize.y, hdcMem, 0, 0,
		  SRCCOPY);

	// Place the original bitmap back into the bitmap sent here.
	//BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcSave, 0, 0, SRCCOPY);

	// Delete the memory bitmaps.
	DeleteObject(SelectObject(hdcBack, bmBackOld));
	DeleteObject(SelectObject(hdcObject, bmObjectOld));
	DeleteObject(SelectObject(hdcMem, bmMemOld));
	DeleteObject(SelectObject(hdcTemp, bmTempOld));

	// Delete the memory DCs.
	DeleteDC(hdcMem);
	DeleteDC(hdcBack);
	DeleteDC(hdcObject);
	DeleteDC(hdcTemp);
}

// *****************************************************************************
// skinsClass::parseSkinFile()
//
// *****************************************************************************
bool skinsClass::parseSkinFile(char * filename, char * sectionName)
{
	char line[4096];
	int maxLineSize;
	char * ptr;
	bool foundSection;
	char * parameters[256];
	char parameterCount;
	char * commandText;
	int commandNumber;
	int i;
	char oldChar;
	int graphicIndex;
	int transparency;
	int posX;
	int posY;
	int srcX;
	int srcY;
	int width;
	int height;
	int highlightCount;
	int hitboxX;
	int hitboxY;
	int hitboxWidth;
	int hitboxHeight;
	int textboxX;
	int textboxY;
	int textboxWidth;
	int textboxHeight;
	int proportional;
	int scrolling;
	int scrollPixels;
	char animFrames[256];
	int animCount;
	int animDelay;
	int tileIndex;
	unsigned int colorValue;
	unsigned int aaColorValue;
	int boldFlag;
	int italFlag;
	int underlineFlag;
	int pointSize;
	int sizeOfFile;
	char * scriptFile;
	char * scriptFilePtr;

	scriptFile = loadFileFromZip(skinZipName, filename, &sizeOfFile);
	scriptFilePtr = scriptFile;

	if(!scriptFile)
		{
		return(false);
		}

	foundSection = false;
	for(;;)
		{
		ptr = &line[0];
		maxLineSize = 4095;
		if(sizeOfFile <maxLineSize)
			{
			maxLineSize = sizeOfFile;
			}
		do{
			*ptr = *scriptFilePtr++;
			maxLineSize--;
			sizeOfFile--;
			if(*ptr++==0x0a)
				{
				break;
				}
			}while(maxLineSize);
		*ptr = 0;
		if(!sizeOfFile)
			{
			break;
			}

		ptr = &line[0];

		if(*ptr!='#')
			{
			// Remove CR/LF from the source line
			ptr = &line[0];

			while(*ptr)
				{
				if((*ptr==0x0a) || (*ptr==0x0d))
					{
					*ptr=0;
					}
				ptr++;
				}

			// Skip over leading spaces
			ptr = &line[0];

			while(*ptr)
				{
				if(*ptr==' ')
					{
					ptr++;
					}
				else
					{
					break;
					}
				}
			// If first character if a null, then it's an empty line
			if(*ptr!=0)
				{
				// If we're at the beginning of a section, force our state to not found
				// This will cause us to skip over all sections after the one we want
				if(*ptr=='[')
					{
					foundSection = false;
					}

				if(!foundSection)
					{
					if(!_stricmp(ptr,sectionName))
						{
						foundSection = true;
						}
					}
				else
					{
					// Found the start of a command, so break it into parameters
					parameterCount=0;
					commandText = ptr;
					while(*ptr)
						{
						if((*ptr == '(') || (*ptr == ')') || (*ptr==','))
							{
							oldChar = *ptr;
							*ptr=0;
							ptr++;
							parameters[parameterCount]=ptr;
							if(oldChar==')')
								{
								break;
								}
							parameterCount++;						
							}
						else
							{
							ptr++;
							}
						}
					// Remove white space and quotes from the beginning of the parameters
					for(i=0;i<parameterCount;i++)
						{
						while(*parameters[i])
							{
							if(*parameters[i]==' ')
								{
								parameters[i]++;
								}
							else
								{
								break;
								}
							}
						parameters[i] = stripQuotes(parameters[i]);
						}
					// Determine the command number
					commandNumber = -1;
					for(i=0;i<SKIN_SCRIPT_NUM_COMMANDS;i++)
						{
						if(!_stricmp(skinScriptCommands[i].text,commandText))
							{
							commandNumber = skinScriptCommands[i].value;
							break;
							}
						}
					switch(commandNumber)
						{
						case SKIN_SCRIPT_SCROLL_RATE:
							if(parameterCount!=1)
								{
								logPrintf("Skin Script scrollrate() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}
							scrollRate = atoi(parameters[0]);
							break;
						case SKIN_SCRIPT_TRANSPARENT_COLOR:
							if(parameterCount!=4)
								{
								logPrintf("Skin Script transparentcolor() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}
							colorValue = atoi(parameters[0])<<24;
							colorValue |= atoi(parameters[1])<<16;
							colorValue |= atoi(parameters[2])<<8;
							colorValue |= atoi(parameters[3]);
							transparentColor = (COLORREF) colorValue;
							break;

						case SKIN_SCRIPT_LOAD_BASE_SKIN:
							if(parameterCount!=1)
								{
								logPrintf("Skin Script loadbaseskin() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}

							if(!loadBaseSkin(parameters[0]))
								{
								logPrintf("Skin Script loadbaseskin() failed\n");							
								delete [] scriptFile;
								return(false);
								}
							break;

						case SKIN_SCRIPT_LOAD_RESIZE_BASE_SKIN:
							if(parameterCount!=33)
								{
								logPrintf("Skin Script loadresizebaseskin() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}

							tileIndex =0;
							i=1;
							for(tileIndex=0;tileIndex<8;tileIndex++)
								{
								resizeTilePos[tileIndex].x = atoi(parameters[i++]);
								resizeTilePos[tileIndex].y = atoi(parameters[i++]);
								resizeTilePos[tileIndex].width = atoi(parameters[i++]);
								resizeTilePos[tileIndex].height = atoi(parameters[i++]);
								}

							if(!loadResizeBaseSkin(parameters[0]))
								{
								logPrintf("Skin Script loadresizebaseskin() failed\n");							
								delete [] scriptFile;
								return(false);
								}

							break;

						case SKIN_SCRIPT_CREATE_BUTTON:
							if(parameterCount!=10)
								{
								logPrintf("Skin Script createbutton() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}

							transparency = atoi(parameters[2]);
							posX = atoi(parameters[3]);
							posY = atoi(parameters[4]);
							srcX = atoi(parameters[5]);
							srcY = atoi(parameters[6]);
							width = atoi(parameters[7]);
							height = atoi(parameters[8]);
							highlightCount = atoi(parameters[9]);

							graphicIndex = loadGraphic(parameters[0]);
							createButton(graphicIndex, transparency, posX, posY, srcX,srcY, width, height, highlightCount,parameters[1]);
							break;
						case SKIN_SCRIPT_CREATE_TWO_STATE_BUTTON:
							if(parameterCount!=10)
								{
								logPrintf("Skin Script createtwostatebutton() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}

							transparency = atoi(parameters[2]);
							posX = atoi(parameters[3]);
							posY = atoi(parameters[4]);
							srcX = atoi(parameters[5]);
							srcY = atoi(parameters[6]);
							width = atoi(parameters[7]);
							height = atoi(parameters[8]);
							highlightCount = atoi(parameters[9]);

							graphicIndex = loadGraphic(parameters[0]);
							createTwoStateButton(graphicIndex, transparency, posX, posY, srcX,srcY, width, height, highlightCount,parameters[1]);
							break;

						case SKIN_SCRIPT_CREATE_HSLIDER:
							if(parameterCount!=11)
								{
								logPrintf("Skin Script createhslider() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}

							transparency = atoi(parameters[2]);
							hitboxX = atoi(parameters[3]);
							hitboxY = atoi(parameters[4]);
							hitboxWidth = atoi(parameters[5]);
							srcX = atoi(parameters[6]);
							srcY = atoi(parameters[7]);
							width = atoi(parameters[8]);
							height = atoi(parameters[9]);
							highlightCount = atoi(parameters[10]);

							graphicIndex = loadGraphic(parameters[0]);
							createHSlider(graphicIndex, transparency, hitboxX, hitboxY, hitboxWidth, srcX, srcY, width, height, highlightCount,parameters[1]);
							break;

						case SKIN_SCRIPT_CREATE_VSLIDER:
							if(parameterCount!=11)
								{
								logPrintf("Skin Script createvslider() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}

							transparency = atoi(parameters[2]);
							hitboxX = atoi(parameters[3]);
							hitboxY = atoi(parameters[4]);
							hitboxHeight = atoi(parameters[5]);
							srcX = atoi(parameters[6]);
							srcY = atoi(parameters[7]);
							width = atoi(parameters[8]);
							height = atoi(parameters[9]);
							highlightCount = atoi(parameters[10]);

							graphicIndex = loadGraphic(parameters[0]);
							createVSlider(graphicIndex, transparency, hitboxX, hitboxY, hitboxHeight, srcX, srcY, width, height, highlightCount,parameters[1]);
							break;

						case SKIN_SCRIPT_CREATE_GRAPHIC_TEXTBOX:
							if(parameterCount!=13)
								{
								logPrintf("Skin Script creategraphictextbox() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}

							transparency = atoi(parameters[2]);
							textboxX = atoi(parameters[3]);
							textboxY = atoi(parameters[4]);
							textboxWidth = atoi(parameters[5]);
							srcX = atoi(parameters[6]);
							srcY = atoi(parameters[7]);
							width = atoi(parameters[8]);
							height = atoi(parameters[9]);
							proportional = atoi(parameters[10]);
							scrolling = atoi(parameters[11]);
							scrollPixels = atoi(parameters[12]);

							graphicIndex = loadGraphic(parameters[0]);
							createGraphicTextbox(graphicIndex, transparency, textboxX, textboxY, textboxWidth, srcX, srcY, width, height, proportional,scrolling, scrollPixels,parameters[1]);
							break;

						case SKIN_SCRIPT_CREATE_FONT_TEXTBOX:
							if(parameterCount!=18)
								{
								logPrintf("Skin Script createfonttextbox() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}

							textboxX = atoi(parameters[2]);
							textboxY = atoi(parameters[3]);
							textboxWidth = atoi(parameters[4]);
							textboxHeight = atoi(parameters[5]);
							pointSize = atoi(parameters[6]);
							boldFlag = atoi(parameters[7]);
							italFlag = atoi(parameters[8]);
							underlineFlag = atoi(parameters[9]);
							colorValue = atoi(parameters[10])<<16;
							colorValue |= atoi(parameters[11])<<8;
							colorValue |= atoi(parameters[12]);
							aaColorValue = atoi(parameters[13])<<16;
							aaColorValue |= atoi(parameters[14])<<8;
							aaColorValue |= atoi(parameters[15]);
							scrolling = atoi(parameters[16]);
							scrollPixels = atoi(parameters[17]);

							createFontTextbox(parameters[0], textboxX, textboxY,textboxWidth, textboxHeight,
												pointSize, boldFlag, italFlag, underlineFlag, colorValue, aaColorValue, scrolling, scrollPixels, parameters[1]);
							break;

						case SKIN_SCRIPT_CREATE_COMBOBOX:
							if(parameterCount!=16)
								{
								logPrintf("Skin Script createcombobox() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}

							textboxX = atoi(parameters[2]);
							textboxY = atoi(parameters[3]);
							textboxWidth = atoi(parameters[4]);
							textboxHeight = atoi(parameters[5]);
							pointSize = atoi(parameters[6]);
							boldFlag = atoi(parameters[7]);
							italFlag = atoi(parameters[8]);
							underlineFlag = atoi(parameters[9]);
							colorValue = atoi(parameters[10])<<16;
							colorValue |= atoi(parameters[11])<<8;
							colorValue |= atoi(parameters[12]);
							aaColorValue = atoi(parameters[13])<<16;
							aaColorValue |= atoi(parameters[14])<<8;
							aaColorValue |= atoi(parameters[15]);

							createComboBox(parameters[0], textboxX, textboxY,textboxWidth, textboxHeight,
												pointSize, boldFlag, italFlag, underlineFlag, colorValue, aaColorValue, parameters[1]);
							break;

						case SKIN_SCRIPT_CREATE_EDITBOX:
							if(parameterCount!=16)
								{
								logPrintf("Skin Script createeditox() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}

							textboxX = atoi(parameters[2]);
							textboxY = atoi(parameters[3]);
							textboxWidth = atoi(parameters[4]);
							textboxHeight = atoi(parameters[5]);
							pointSize = atoi(parameters[6]);
							boldFlag = atoi(parameters[7]);
							italFlag = atoi(parameters[8]);
							underlineFlag = atoi(parameters[9]);
							colorValue = atoi(parameters[10])<<16;
							colorValue |= atoi(parameters[11])<<8;
							colorValue |= atoi(parameters[12]);
							aaColorValue = atoi(parameters[13])<<16;
							aaColorValue |= atoi(parameters[14])<<8;
							aaColorValue |= atoi(parameters[15]);

							createEditBox(parameters[0], textboxX, textboxY,textboxWidth, textboxHeight,
												pointSize, boldFlag, italFlag, underlineFlag, colorValue, aaColorValue, parameters[1]);
							break;


						case SKIN_SCRIPT_CREATE_HIGHLIGHT_ANIMATION:
							if(parameterCount<9)
								{
								logPrintf("Skin Script createhighlightanimation() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}

							transparency = atoi(parameters[2]);
							posX = atoi(parameters[3]);
							posY = atoi(parameters[4]);
							srcX = atoi(parameters[5]);
							srcY = atoi(parameters[6]);
							width = atoi(parameters[7]);
							height = atoi(parameters[8]);
							
							animCount=0;
							for(i=9;i<parameterCount;i++)
								{
								animFrames[animCount] = atoi(parameters[i]);
								animCount++;
								}
							graphicIndex = loadGraphic(parameters[0]);
							createHighlightAnimation(graphicIndex, transparency, posX, posY, srcX, srcY, width, height, &animFrames[0], animCount,parameters[1]);
							break;
						case SKIN_SCRIPT_CREATE_ANIMATION:
							if(parameterCount<10)
								{
								logPrintf("Skin Script createanimation() had an incorrect number of parameters\n");							
								delete [] scriptFile;
								return(false);
								}

							transparency = atoi(parameters[2]);
							posX = atoi(parameters[3]);
							posY = atoi(parameters[4]);
							srcX = atoi(parameters[5]);
							srcY = atoi(parameters[6]);
							width = atoi(parameters[7]);
							height = atoi(parameters[8]);
							animDelay = atoi(parameters[9]);
							
							animCount=0;
							for(i=10;i<parameterCount;i++)
								{
								animFrames[animCount] = atoi(parameters[i]);
								animCount++;
								}
							graphicIndex = loadGraphic(parameters[0]);
							createAnimation(graphicIndex, transparency, posX, posY, srcX, srcY, width, height, &animFrames[0], animCount,animDelay,parameters[1]);
							break;


						default:
							break;
						}
					}
				}
			}
		}
	delete [] scriptFile;

	return(true);
}

// *****************************************************************************
// skinsClass::stripQuotes()
//
// *****************************************************************************
char * skinsClass::stripQuotes(char * source)
{
	char *ptr;

	// If it's a null scring, then just bail out
	if(!*source)
		{
		return(source);
		}


	// Leading quote
	if(*source=='"')
		{
		source++;
		}

	ptr = source;

	while(*ptr)
		{
		if(*ptr=='"')
			{
			*ptr = 0;
			}
		ptr++;
		}
	return(source);
}



// *****************************************************************************
// skinsClass::loadFileFromZip()
//
// *****************************************************************************
char * skinsClass::loadFileFromZip(char * zipFile, char * filename, int * sizeOfFile)
{
	unzFile uf;
	unz_file_info info;
	char * memPtr;

	*sizeOfFile = 0;

	uf = unzOpen(zipFile);

	if(uf==NULL)
		{
		return(false);
		}

	if(unzLocateFile(uf,filename,0)!=UNZ_OK)
		{
		unzClose(uf);
		return(false);
		}

	memset(&info, 0, sizeof(unz_file_info));

	if(unzGetCurrentFileInfo(uf,&info,NULL,0,NULL,0,NULL,0)!=UNZ_OK)
		{
		unzClose(uf);
		return(false);
		}


	if(unzOpenCurrentFile(uf)!=UNZ_OK)
		{
		unzCloseCurrentFile(uf);
		return(false);		
		}

	*sizeOfFile = info.uncompressed_size;

	memPtr = new char [info.uncompressed_size];

	if(unzReadCurrentFile(uf,memPtr,info.uncompressed_size)!=(long)info.uncompressed_size)
		{
		delete [] memPtr;
		unzCloseCurrentFile(uf);
		unzClose(uf);
		return(false);
		}

	unzCloseCurrentFile(uf);
	unzClose(uf);
	return(memPtr);
}