// *****************************************************************************
// playerwindow class
// *****************************************************************************
#ifndef skins_h
#define skins_h

#define MAX_GIZMOS 32
#define MAX_SKIN_GRAPHIC_FILES 8

#define SKIN_EDIT_CONTROL_MESSAGE 500

#define GIZMO_SCROLL_RANGE 1000.0f

enum{
		RESIZE_TILE_TOPLEFT=0,
		RESIZE_TILE_TOP,
		RESIZE_TILE_TOPRIGHT,
		RESIZE_TILE_LEFT,
		RESIZE_TILE_RIGHT,
		RESIZE_TILE_BOTLEFT,
		RESIZE_TILE_BOT,
		RESIZE_TILE_BOTRIGHT,
		};

enum{
		SKIN_SCRIPT_SCROLL_RATE=0,
		SKIN_SCRIPT_TRANSPARENT_COLOR,
		SKIN_SCRIPT_LOAD_BASE_SKIN,
		SKIN_SCRIPT_CREATE_BUTTON,
		SKIN_SCRIPT_CREATE_TWO_STATE_BUTTON,
		SKIN_SCRIPT_CREATE_HSLIDER,
		SKIN_SCRIPT_CREATE_GRAPHIC_TEXTBOX,
		SKIN_SCRIPT_CREATE_FONT_TEXTBOX,
		SKIN_SCRIPT_CREATE_HIGHLIGHT_ANIMATION,
		SKIN_SCRIPT_CREATE_ANIMATION,
		SKIN_SCRIPT_LOAD_RESIZE_BASE_SKIN,
		SKIN_SCRIPT_CREATE_COMBOBOX,
		SKIN_SCRIPT_CREATE_EDITBOX,
		SKIN_SCRIPT_CREATE_VSLIDER,
		SKIN_SCRIPT_NUM_COMMANDS
		};

// 30 FPS max update rate
#define SKIN_UPDATE_RATE 33

enum {
		GIZMO_UNUSED = 0,
		GIZMO_BUTTON,
		GIZMO_ANIMATION,
		GIZMO_HIGHLIGHT_ANIMATION,
		GIZMO_HSLIDER,
		GIZMO_VSLIDER,
		GIZMO_GRAPHIC_TEXTBOX,
		GIZMO_FONT_TEXTBOX,
		GIZMO_COMBOBOX,
		GIZMO_EDITBOX,
};

enum {
		GIZMO_STATE_BUTTON_UP,
		GIZMO_STATE_BUTTON_DOWN,
	};

enum {
		GIZMO_STATE_SLIDER_UP,
		GIZMO_STATE_SLIDER_DRAG,
	};

typedef struct
	{
	RGNDATAHEADER	header;
	RECT			rects[1];
	} WINDOWREGIONLIST; 

typedef struct
	{
	int				startHPixel;
	int				width;
	} PTEXTDATA; 

typedef struct
	{
	HANDLE			handle;
	HDC				hdc;
	BITMAP			info;
	HBITMAP			handle1;
	PTEXTDATA *		proportionalTextData;
	int				proportionalTextEntries;
	char			filename[MAX_PATH];
	} GRAPHICOBJECT; 

typedef struct 
	{
	char		text[32];
	int			value;
	} SCRIPTCOMMAND;

typedef struct 
	{
	char		text[32];
	int			messageId1;
	int			messageId2;
	} GIZMONAME;

typedef struct 
	{
	int			x;
	int			y;
	int			width;
	int			height;
	} TILERECT;

typedef struct
	{
	char		text[256];
	int			data;
	} COMBOBOXTEXTENTRY; 

typedef struct
	{
	COMBOBOXTEXTENTRY	entry[1];
	} COMBOBOXTEXTENTRYDATA; 
typedef struct
	{
	int						size;
	int						numEntries;
	COMBOBOXTEXTENTRYDATA *	entries;
	} COMBOBOXTEXT; 

typedef struct
	{
	int				gizmoType;
	int				graphicNumber;
	bool			transparentBlt;
	int				state;
	bool			multiStateGizmo;
	int				multiState;
	int				highlightActive;
	int				highlightIndex;
	int				highlightMax;
	int				hitboxPosX;
	int				hitboxPosY;
	int				hitboxWidth;
	int				hitboxHeight;
	int				posX;
	int				posY;
	int				width;
	int				height;
	int				srcX;
	int				srcY;
	int				messageId[2];
	char			name[32];
	int				sliderPos;
	bool			canScrollText;
	bool			scrollText;
	int				scrollIndex;
	int				scrollPixelOffset;
	int				scrollNumPixels;
	bool			proportionalText;
	char			text[256];
	int				animationIndex;
	int				animationMax;
	char			animationList[256];
	int				animationDelay;
	int				animationLoopDelay;
	HFONT			font;
	COLORREF		fontColor;
	COLORREF		fontAAColor;
	COMBOBOXTEXT	comboBoxText;
	int				comboBoxRenderStartIndex;
	HWND			windowsControlHwnd;
	HBRUSH			windowsControlBackground;
	} GIZMO; 

   class skinsClass {

	public:
									skinsClass();
									~skinsClass();
			bool					loadSkin(char * skinName, char * sectionName);

			HWND					createWindow(HINSTANCE hInst, char * windowName, char * className, WNDPROC parentWindowFunction);
			HWND					createResizeWindow(HINSTANCE hInst,  char * windowName, char * className, WNDPROC parentWindowFunction);
			BOOL					skinMessageHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
			void					render();
			void					mouseOver(HWND hWnd,int posX, int posY);
			bool					mouseDown(HWND hWnd,int posX, int posY);
			void					mouseUp(HWND hWnd,int posX, int posY);
			bool					setSliderPos(char * sliderName, int pos);
			void					showWindow();
			void					hideWindow();
			bool					setText(char * counterName, char * text);
			bool					getText(char * name, char * text, int maxCount);
			bool					setGizmoMultiState(char * gizmoName, int multiState);
			void					resizeOutputWindow(int innerWidth, int innerHeight);
			void					getInnerWindowOffsets(int * offsetX, int * offsetY);
			void					getInnerRect(RECT * innerRect);
			bool					getGizmoPos(char * gizmoName, RECT * buttonRect);
			bool					comboBoxReset(char * gizmoName);
			bool					comboBoxInsertString(char * gizmoName, char * string);
			bool					setComboBoxScrollPos(char * gizmoName, int pos);

	protected:
			bool					registerWindowClass(HINSTANCE hInst,char * className,WNDPROC parentWindowFunction);
			bool					createRegionData(int posX, int posY, int width, int height);
			bool					createResizeRegionData(int width, int height);
			void					drawBitmap(HDC destHdc, int destX, int destY, HDC srcHdc, int srcX,int srcY, int srcWidth, int srcHeight, bool transp);
			void					drawTransparentBitmap(HDC hdc, int destX, int destY, HDC srcHdc, int srcX,int srcY, int srcWidth, int srcHeight);
			void					scrollText();
			bool					createProportionalTextData(int graphicIndex);
			int						loadGraphic(char * filename);
			int						createButton(int graphicNumber, int transparency, int posX, int posY, int srcX, int srcY, int width, int height, int highlightMax, char * name);
			int						createHSlider(int graphicNumber, int transparency, int hitboxPosX, int hitboxPosY, int hitboxWidth, int srcX, int srcY, int width, int height, int highlightMax, char * name);
			int						createGraphicTextbox(int graphicNumber, int transparency, int textboxX, int textboxY,int textboxWidth, int srcX, int srcY, int width, int height, int proportional, int scrolling, int scrollPixels, char * name);
			int						createAnimation(int graphicNumber, int transparency, int posX, int posY,int srcX, int srcY, int width, int height, char * animationList, int animationMax, int animationLoopDelay, char * name);
			int						createHighlightAnimation(int graphicNumber, int transparency, int posX, int posY,int srcX, int srcY, int width, int height, char * animationList, int animationMax, char * name);
			int						createTwoStateButton(int graphicNumber, int transparency, int posX, int posY,int srcX, int srcY, int width, int height, int highlightMax, char * name);
									createFontTextbox(char * fontName, int textboxX, int textboxY,int textboxWidth, int textboxHeight, int pointSize, int bold, int ital, int underline, COLORREF color, COLORREF aaColor, int scrolling, int scrollPixels, char * name);
			int						createComboBox(char * fontName, int textboxX, int textboxY,int textboxWidth, int textboxHeight, int pointSize, int bold, int ital, int underline, COLORREF color, COLORREF aaColor, char * name);
			int						createEditBox(char * fontName, int textboxX, int textboxY,int textboxWidth, int textboxHeight, int pointSize, int bold, int ital, int underline, COLORREF color, COLORREF aaColor, char * name);
			int						createVSlider(int graphicNumber, int transparency, int hitboxPosX, int hitboxPosY, int hitboxHeight,int srcX, int srcY, int width, int height,  int highlightMax, char * name);

			void					adjustPosForResize(int * curX, int * curY);

			bool					parseSkinFile(char * filename, char * sectionName);
			char *					stripQuotes(char * source);
			bool					loadBaseSkin(char * filename);
			bool					loadResizeBaseSkin(char * filename);
			void					paintResizeWindow(HDC hdc);
			char *					loadFileFromZip(char * zipFile, char * filename, int * sizeOfFile);

			char					skinZipName[MAX_PATH];
			WNDCLASS				windCls;
			HWND					mainhWnd;
			GRAPHICOBJECT			windowBase;
			GRAPHICOBJECT			buttonGraphic[MAX_SKIN_GRAPHIC_FILES];
			GIZMO					gizmoList[MAX_GIZMOS];
			bool					windowRegionListValid;
			WINDOWREGIONLIST *		windowRegionList;
			int						windowRegionListSize;
			int						windowRegionListCount;
			HRGN					windowRegionHandle;
			bool					windowMoving;
			POINT					startMovePos;
			POINT					endMovePos;
			HBITMAP					frontBufferBitmap;
			HDC						frontBufferHdc;
			unsigned char			textConversionTable[256];
			DWORD					lastScrollTime;
			int						scrollRate;
			COLORREF				transparentColor;
			int						currentGizmoIndex;
			int						currrentGraphicIndex;
			bool					visible;
			bool					resizeWindow;
			int						resizeWidth;
			int						resizeHeight;
			int						resizeExtraWidth;
			int						resizeExtraHeight;
			bool					resizeDrag;
			int						resizeDragPosX;
			int						resizeDragPosY;
			int						resizeDragStartWidth;
			int						resizeDragStartHeight;
			TILERECT				resizeTilePos[8];		
			WINDOWREGIONLIST *		resizeWindowRegionList[4];
			int						resizeWindowRegionListSize[4];
			int						resizeWindowRegionListCount[4];
};

#endif