// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// translate.cpp
// 
//
// *****************************************************************************

#ifdef ENCODERSERVER
#include "../encoderserver/es.h"
#endif

#ifdef ENCODER
#include "../encoder/en.h"
#endif

#ifdef PLAYER
#include "../player/pl.h"
#endif

extern GLOBALS * globals;

// *****************************************************************************
// translateClass::translateClass()
//
// translate class constructor
// *****************************************************************************
translateClass::translateClass()
{
	int i;

	for(i=0;i<MAX_TRANSTABLE;i++)
		{
		transTable[i] = new TRANSLATEFROMTO [1];
		transTable[i]->text = NULL;
		transTable[i]->textId = NULL;
		}

	numEntries = 0;
	transTableTail = 0;

}

// *****************************************************************************
// translateClass::~translateClass()
//
// translate class destructor
// *****************************************************************************
translateClass::~translateClass()
{
	int i;

	for(i=0;i<numEntries;i++)
		{
		if(transTable[i]->text)
			delete [] transTable[i]->text;
		if(transTable[i]->textId)
			delete [] transTable[i]->textId;
		delete [] transTable[i];
		}

}

// *****************************************************************************
// translateClass::setDefaultLanguageFile()
//
// Set the translation file
// *****************************************************************************
bool translateClass::setDefaultLanguageFile(char * filename)
{
	bool val;

	val = setLanguageFile(filename);
	if(!val)
		return(false);

	numEntries = transTableTail;

	return(true);
}

// *****************************************************************************
// translateClass::setLanguageFile()
//
// Set the translation file
// *****************************************************************************
bool translateClass::setLanguageFile(char * filename)
{

	FILE * inFile;
	char line[4096];
	char * ptr;
	char * textIdStart;
	char * textStart;
	int index;
	bool foundIt;

	// Convert from normal to wide
	
	// Load the english translation file
	
	inFile = fopen(filename,"r");

	if(!inFile)
		{
		return(false);
		}
	
	for(;;)
		{
		fgets(line,4096,inFile);

		if(feof(inFile))
			break;

		ptr = &line[0];	
		
		textIdStart = NULL;
		textStart = NULL;

		if(*ptr!='#')
			{
			// Find the First Quote (This marks the start of the ID
			for(;;)
				{
				if(*ptr=='"')
					{
					break;
					}

				if(!*ptr)
					{
					break;
					}
				ptr++;
				}
			
			if(*ptr)
				{
				ptr++;

				// Found start of first String
				// Find the End Quote (This marks the end of the ID)

				textIdStart = ptr;

				for(;;)
					{
					if(*ptr=='"')
						{
						break;
						}

					if(!*ptr)
						{
						break;
						}
					ptr++;
					}
				
				if(*ptr)
					{
					*ptr++=0;

					// Find the Start Quote (This marks the start of the text)

					for(;;)
						{
						if(*ptr=='"')
							{
							break;
							}

						if(!*ptr)
							{
							break;
							}
						ptr++;
						}
					
					if(*ptr)
						{
						ptr++;

						// Found start of second String
						// Find the End Quote (This marks the end of the text)
						textStart = ptr;

						for(;;)
							{
							if(*ptr=='"')
								{
								break;
								}

							if(!*ptr)
								{
								break;
								}
							ptr++;
							}
						
						*ptr=0;
						}
					}
				}	

			}

		if(textIdStart && textStart)
			{
			// We now have two pointers in the text file. One to the ID and one to the text
			foundIt = false;

			for(index=0;index<transTableTail;index++)
				{
				if(transTable[index]->textId)
					{
					if(!strcmp(transTable[index]->textId,textIdStart))
						{
						foundIt = true;
						break;
						}
					}
				}

			if(foundIt)
				{
				if(transTable[index]->text)
					delete [] transTable[index]->text;
				transTable[index]->text = new char [strlen(textStart)+1];
				strcpy(transTable[index]->text,textStart);
				}
			else
				{
				transTable[transTableTail]->textId = new char [strlen(textIdStart)+1];
				strcpy(transTable[transTableTail]->textId,textIdStart);
				transTable[transTableTail]->text = new char [strlen(textStart)+1];
				strcpy(transTable[transTableTail]->text,textStart);

				transTableTail++;

				}
			}

		}

	fclose(inFile);	
	return(true);

}

// *****************************************************************************
// translateClass::translate()
//
// Lookup a translation using an ID
// *****************************************************************************
void translateClass::translate(char * text, char * textId)
{
	int index;

	for(index=0;index<numEntries;index++)
		{
		if(transTable[index]->textId)
			{
			if(!strcmp(transTable[index]->textId,textId))
				{
				strcpy(text, transTable[index]->text);
				return;
				}
			}
		}
	strcpy(text, textId);
	return;
}

// *****************************************************************************
// translateClass::translateMenu()
//
// Lookup a translation using an ID
// *****************************************************************************
void translateClass::translateMenu(HMENU menu)
{
	MENUITEMINFO item;
 	int menuItems;
	int i;
	char text[4096];
	char translatedText[4096];
	menuItems = GetMenuItemCount(menu);

	for(i=0;i<menuItems;i++)
		{
		memset(&item,0,sizeof(MENUITEMINFO));
		item.cbSize = sizeof(MENUITEMINFO);
		item.fMask  = MIIM_SUBMENU | MIIM_TYPE;
		item.dwTypeData = text;
		item.cch = 4096;

		GetMenuItemInfo(menu,i,TRUE,&item);

		if(!item.fType)
			{
			item.fType = MFT_STRING;
			translate(translatedText,text);
			item.dwTypeData = translatedText;
			item.cch = 4096;
			SetMenuItemInfo(menu,i,TRUE,&item);
			}
	
		if(item.hSubMenu)
			{
			translateMenu(item.hSubMenu);
			}
		}

}
