// *****************************************************************************
//
//
// jobHelper.cpp
// 
//
// *****************************************************************************

#ifdef ENCODERSERVER
#include "../encoderserver/es.h"
#endif

#ifdef ENCODER
#include "../encoder/en.h"
#endif

extern GLOBALS * globals;

// *****************************************************************************
// jobHelperClass::jobHelperClass()
//
// jobHelper class constructor
// *****************************************************************************
jobHelperClass::jobHelperClass()
{

}

// *****************************************************************************
// jobHelperClass::~jobHelperClass()
//
// jobHelper class destructor
// *****************************************************************************
jobHelperClass::~jobHelperClass()
{

}

// *****************************************************************************
// jobHelperClass::flattenJobEntry()
//
// Flattens a job (to be sent over TCP/IP or saved)
// *****************************************************************************
void jobHelperClass::flattenJobEntry(JOBLISTENTRY * jobList, FLATJOBENTRY * flatJobEntry)
{

	memset(flatJobEntry,0,sizeof(FLATJOBENTRY));

	memcpy(&flatJobEntry->convertConfig,jobList->convertConfig,sizeof(CONVERTCONFIG));

	memcpy(&flatJobEntry->gatherResults,jobList->gatherResults,sizeof(GATHERRESULTS));

	// If Codec is open - Close it
	if(jobList->convertConfig->videoFormatOut->hic)
		{
		ICClose(jobList->convertConfig->videoFormatOut->hic);
		jobList->convertConfig->videoFormatOut->hic = NULL;
		}

	if(jobList->convertConfig->videoFormatOutP2->hic)
		{
		ICClose(jobList->convertConfig->videoFormatOutP2->hic);
		jobList->convertConfig->videoFormatOutP2->hic = NULL;
		}

	// Now flatten the video codec settings
	memcpy(&flatJobEntry->settingsVideoFormatOut,jobList->convertConfig->videoFormatOut,sizeof(COMPVARS));
	memcpy(&flatJobEntry->settingsVideoFormatOutLpbiOut,jobList->convertConfig->videoFormatOut->lpbiOut, jobList->convertConfig->videoFormatOut->lpbiOut->bmiHeader.biSize);
	memcpy(&flatJobEntry->settingsVideoFormatOutLpState,jobList->convertConfig->videoFormatOut->lpState, jobList->convertConfig->videoFormatOut->cbState);	

	// Now flatten the second pass video codec settings
	memcpy(&flatJobEntry->settingsVideoFormatOutP2,jobList->convertConfig->videoFormatOutP2,sizeof(COMPVARS));
	memcpy(&flatJobEntry->settingsVideoFormatOutP2LpbiOut,jobList->convertConfig->videoFormatOutP2->lpbiOut, jobList->convertConfig->videoFormatOut->lpbiOut->bmiHeader.biSize);
	memcpy(&flatJobEntry->settingsVideoFormatOutP2LpState,jobList->convertConfig->videoFormatOutP2->lpState, jobList->convertConfig->videoFormatOut->cbState);	
	
	// Now flatten the audio codec settings
	memcpy(&flatJobEntry->settingsAudioFormatOut,jobList->convertConfig->audioFormatOut,AUDIO_STATE_DEFAULT_SIZE);

	// And last transfer the misc state vars over
	flatJobEntry->inputValidated = jobList->inputValidated;
	flatJobEntry->outputSet = jobList->outputSet;
	flatJobEntry->cropScanDone = jobList->cropScanDone;
	strcpy(flatJobEntry->jobName,jobList->jobName);

}

// *****************************************************************************
// jobHelperClass::unflattenJobEntry()
//
// unflattens a job
// *****************************************************************************
void jobHelperClass::unflattenJobEntry(JOBLISTENTRY * jobList, FLATJOBENTRY * flatJobEntry)
{

	// Now unflatten the job and add it to the job list

	// First transfer the current memory pointers over so they don't get overwritten
	flatJobEntry->convertConfig.videoFormatOut = jobList->convertConfig->videoFormatOut;
	flatJobEntry->convertConfig.audioFormatOut = jobList->convertConfig->audioFormatOut;

	flatJobEntry->convertConfig.videoFormatOutP2 = jobList->convertConfig->videoFormatOutP2;

	//now copy the structs over the current data
	memcpy(jobList->convertConfig,&flatJobEntry->convertConfig,sizeof(CONVERTCONFIG));
	memcpy(jobList->gatherResults,&flatJobEntry->gatherResults,sizeof(GATHERRESULTS));

	// And last, overwrite the video codec data with the file based data
	flatJobEntry->settingsVideoFormatOut.lpbiOut = jobList->convertConfig->videoFormatOut->lpbiOut;
	flatJobEntry->settingsVideoFormatOut.lpState = jobList->convertConfig->videoFormatOut->lpState;
	memcpy(jobList->convertConfig->videoFormatOut,&flatJobEntry->settingsVideoFormatOut,sizeof(COMPVARS));

	memcpy(jobList->convertConfig->videoFormatOut->lpbiOut,&flatJobEntry->settingsVideoFormatOutLpbiOut, VIDOMI_BITMAP_INFO_SIZE);
	memcpy(jobList->convertConfig->videoFormatOut->lpState, &flatJobEntry->settingsVideoFormatOutLpState,jobList->convertConfig->videoFormatOut->cbState);	

	flatJobEntry->settingsVideoFormatOutP2.lpbiOut = jobList->convertConfig->videoFormatOutP2->lpbiOut;
	flatJobEntry->settingsVideoFormatOutP2.lpState = jobList->convertConfig->videoFormatOutP2->lpState;
	memcpy(jobList->convertConfig->videoFormatOutP2,&flatJobEntry->settingsVideoFormatOutP2,sizeof(COMPVARS));

	memcpy(jobList->convertConfig->videoFormatOutP2->lpbiOut,&flatJobEntry->settingsVideoFormatOutP2LpbiOut, VIDOMI_BITMAP_INFO_SIZE);
	memcpy(jobList->convertConfig->videoFormatOutP2->lpState, &flatJobEntry->settingsVideoFormatOutP2LpState,jobList->convertConfig->videoFormatOutP2->cbState);	
	
	memcpy(jobList->convertConfig->audioFormatOut,&flatJobEntry->settingsAudioFormatOut,AUDIO_STATE_DEFAULT_SIZE);

	jobList->inputValidated = flatJobEntry->inputValidated;
	jobList->outputSet = flatJobEntry->outputSet;
	jobList->cropScanDone = flatJobEntry->cropScanDone;
	strcpy(jobList->jobName,flatJobEntry->jobName);

}

// *****************************************************************************
// jobHelperClass::clearJob()
//
// Clears one entry in the job List
// *****************************************************************************
void jobHelperClass::deleteJobEntry(JOBLISTENTRY * jobList)
{
	if(jobList->convertConfig)
		{
		logPrintf("Freeing jobList->convertConfig\n");
		
		if(jobList->convertConfig->videoFormatOut)
			{
			logPrintf("Freeing jobList->convertConfig->videoFormatOut\n");

			if(jobList->convertConfig->videoFormatOut->lpState)
				delete [] jobList->convertConfig->videoFormatOut->lpState;
			if(jobList->convertConfig->videoFormatOut->lpbiOut)
				delete [] jobList->convertConfig->videoFormatOut->lpbiOut;

			delete [] jobList->convertConfig->videoFormatOut;
			}
		if(jobList->convertConfig->videoFormatOutP2)
			{
			logPrintf("Freeing jobList->convertConfig->videoFormatOutP2\n");

			if(jobList->convertConfig->videoFormatOutP2->lpState)
				delete [] jobList->convertConfig->videoFormatOutP2->lpState;
			if(jobList->convertConfig->videoFormatOutP2->lpbiOut)
				delete [] jobList->convertConfig->videoFormatOutP2->lpbiOut;

			delete [] jobList->convertConfig->videoFormatOutP2;
			}

		if(jobList->convertConfig->audioFormatOut)
			{
			logPrintf("Freeing jobList->convertConfig->audioFormatOut\n");
			delete [] jobList->convertConfig->audioFormatOut;
			}
		delete [] jobList->convertConfig;
		jobList->convertConfig = NULL;
		}

	if(jobList->gatherResults)
		{	
		logPrintf("Freeing jobList->gatherResults\n");
		delete [] jobList->gatherResults;
		jobList->gatherResults = NULL;
		}

	jobList->cropScanDone = false;
	jobList->inputValidated = false;
	jobList->outputSet = false;
}

// *****************************************************************************
// jobHelperClass::createJobEntry()
//
// Allocate a clean job struture at the head of the job list
// *****************************************************************************
void jobHelperClass::createJobEntry(JOBLISTENTRY * jobList)
{
	memset(jobList,0,sizeof(JOBLISTENTRY));

	jobList->convertConfig = new CONVERTCONFIG [1];
	memset(jobList->convertConfig,0,sizeof(CONVERTCONFIG));

	jobList->convertConfig->videoFormatOut = new COMPVARS [1];
	memset(jobList->convertConfig->videoFormatOut,0,sizeof(COMPVARS));
	jobList->convertConfig->videoFormatOut->lpbiOut = (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
	memset(jobList->convertConfig->videoFormatOut->lpbiOut,0,VIDOMI_BITMAP_INFO_SIZE);
	jobList->convertConfig->videoFormatOut->lpState = new char [VIDEO_STATE_DEFAULT_SIZE];
	memset(jobList->convertConfig->videoFormatOut->lpState,0,sizeof(VIDEO_STATE_DEFAULT_SIZE));
	jobList->convertConfig->videoFormatOut->hic = NULL;

	jobList->convertConfig->videoFormatOutP2 = new COMPVARS [1];
	memset(jobList->convertConfig->videoFormatOutP2,0,sizeof(COMPVARS));
	jobList->convertConfig->videoFormatOutP2->lpbiOut =  (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
	memset(jobList->convertConfig->videoFormatOutP2->lpbiOut,0,VIDOMI_BITMAP_INFO_SIZE);
	jobList->convertConfig->videoFormatOutP2->lpState = new char [VIDEO_STATE_DEFAULT_SIZE];
	memset(jobList->convertConfig->videoFormatOutP2->lpState,0,sizeof(VIDEO_STATE_DEFAULT_SIZE));
	jobList->convertConfig->videoFormatOutP2->hic = NULL;

	jobList->convertConfig->audioFormatOut  = (MPEGLAYER3WAVEFORMAT *) new char [AUDIO_STATE_DEFAULT_SIZE];
	memset(jobList->convertConfig->audioFormatOut,0,sizeof(AUDIO_STATE_DEFAULT_SIZE));

	jobList->gatherResults = new GATHERRESULTS [1];

}