// *****************************************************************************
//
//
// jobServer.cpp
// 
//
// *****************************************************************************

#ifdef ENCODERSERVER
#include "../encoderserver/es.h"
#endif

#ifdef ENCODER
#include "../encoder/en.h"
#endif

extern GLOBALS * globals;

// *****************************************************************************
// jobServerClass::jobServerClass()
//
// jobServer class constructor
// *****************************************************************************
jobServerClass::jobServerClass()
{
	connectionStatus = NET_NOT_CONNECTED;
	tcp = NULL;

}

// *****************************************************************************
// jobServerClass::~jobSubmitClass()
//
// jobServer class destructor
// *****************************************************************************
jobServerClass::~jobServerClass()
{

}
#ifdef ENCODERSERVER

// *****************************************************************************
// jobServerClass::processCommand()
//
// Called by the listen thread that owns us 
// *****************************************************************************
int jobServerClass::processCommand(tcpClass * myTcp, SOCKET myRSocket)
{
	JOBSERVERHEADER header;

	if(!myTcp->safeReceiveBytes(myRSocket,(char *)&header,sizeof(JOBSERVERHEADER)))
		{
		return(false);
		}

	switch(header.command)
		{
		case jobServerStatus:
			return(sendStatus(myTcp, myRSocket, header.size));

		case jobServerSubmit:
			return(receiveJob(myTcp, myRSocket, header.size));

		case jobServerClose:
			globals->visualLog->logPrintf("Closed Connection\n");
			return(false);
		}

	return(true);
}
// *****************************************************************************
// jobServerClass::receiveJob()
//
// *****************************************************************************
bool jobServerClass::sendStatus(tcpClass * myTcp, SOCKET myRSocket, int size)
{
	JOBSERVERSTATS * jobStats;
	JOBSERVERHEADER header;
	int returnSize;
	int nextJob;
	int nextJobJobNumber;
	int jobCount;
	int jobNumber;
	int oldJobJobNumber;
	int i;

	if(size!=0)
		{	
		return(false);
		}

	returnSize = sizeof(JOBSERVERSTATS) + ((globals->numSlaveEncodeMachines-1) * sizeof(CONVERSIONSTATS));
	jobStats = (JOBSERVERSTATS *) new char [returnSize];
	
	memset(jobStats,0,returnSize);
	
	EnterCriticalSection(&globals->slaveEncodeStatsLock);
	jobStats->numJobsBeingProcessed = globals->jobsBeingProcessed;
	jobStats->numJobsInQueue = globals->jobsInQueue;
	jobStats->numSlavesAvailable = globals->slavesAvailable;
	jobStats->numStatsThatFollow = globals->numSlaveEncodeMachines;

	// Clear the returned mini job Queue list
	for(i=0;i<16;i++)
		{
		jobStats->jobQueue[i].jobNumber = -1;
		strcpy(jobStats->jobQueue[i].jobName,"");
		}

	nextJob = -1;
	nextJobJobNumber = 0x7fffffff;
	for(jobNumber=0;jobNumber<globals->slaveJobListSize;jobNumber++)
		{
		if(!globals->slaveJobList[jobNumber].job->encodeDone && globals->slaveJobList[jobNumber].job->slaveAllocated==-1)
			{
			if(globals->slaveJobList[jobNumber].job->jobNumber < nextJobJobNumber)
				{
				// Is this the lowest ? 
				nextJob = jobNumber;
				nextJobJobNumber = globals->slaveJobList[jobNumber].job->jobNumber;
				}
			}
		}
	// Do we have a next Job ? 
	if(nextJob!=-1)
		{
		// If we have more jobs, then find the next 16 jobs in the queue
		// Note that we're looking at the job numbers to determine which 
		// job numbers to determine which job will be next. Gaps are allowed in the
		// job numbers, so we have to do some extra checks to find the next 16 jobs
		jobStats->jobQueue[0].jobNumber = nextJobJobNumber;
		strcpy(jobStats->jobQueue[0].jobName,globals->slaveJobList[nextJob].job->flatJob.jobName);

		for(jobCount=1;jobCount<16;jobCount++)
			{
			oldJobJobNumber = nextJobJobNumber;
			nextJob = -1;
			nextJobJobNumber = 0x7fffffff;
			for(jobNumber=0;jobNumber<globals->slaveJobListSize;jobNumber++)
				{
				if(!globals->slaveJobList[jobNumber].job->encodeDone && globals->slaveJobList[jobNumber].job->slaveAllocated==-1)
					{
					if((globals->slaveJobList[jobNumber].job->jobNumber < nextJobJobNumber) &&
						(globals->slaveJobList[jobNumber].job->jobNumber > oldJobJobNumber))
						{
						// Is this the lowest ? 
						nextJob = jobNumber;
						nextJobJobNumber = globals->slaveJobList[jobNumber].job->jobNumber;
						}
					}
				}
			if(nextJob==-1)
				{
				break;
				}
			jobStats->jobQueue[jobCount].jobNumber = nextJobJobNumber;
			strcpy(jobStats->jobQueue[jobCount].jobName,globals->slaveJobList[nextJob].job->flatJob.jobName);
			}
		}
	
	memcpy(jobStats->stats,globals->slaveEncodeStats,(sizeof(CONVERSIONSTATS)*globals->numSlaveEncodeMachines));
	LeaveCriticalSection(&globals->slaveEncodeStatsLock);

	header.command = jobServerStatusReturn;
	header.size = returnSize;

	// Send the stats return header
	if(!myTcp->safeSendBytes(myRSocket,(char *)&header,sizeof(JOBSERVERHEADER)))
		{
		delete [] jobStats;
		return(false);
		}
	
	// Send the stats return bytes
	if(!myTcp->safeSendBytes(myRSocket,(char *)jobStats,returnSize))
		{
		delete [] jobStats;
		return(false);
		}
	globals->visualLog->logPrintf("Sent Statistics\n");

	return(true);
}

// *****************************************************************************
// jobServerClass::receiveJob()
//
// *****************************************************************************
bool jobServerClass::receiveJob(tcpClass * myTcp, SOCKET myRSocket, int size)
{
	FLATJOBENTRY * flatJobEntry;
	int jobIndex;
	bool jobAdded;
	SLAVEJOBLIST * subSlaveJobList;
	int subJobCount;
	int i;

	flatJobEntry = (FLATJOBENTRY *) new char [size];

	memset(flatJobEntry,0,size);

	if(!myTcp->safeReceiveBytes(myRSocket,(char *)flatJobEntry,size))
		{
		delete [] flatJobEntry;
		return(false);
		}

	if(size!=sizeof(FLATJOBENTRY))
		{
		// Ignore the command. The version being used doesn't match the server version
		delete [] flatJobEntry;
		return(true);		
		}

	subSlaveJobList = globals->netHelper->createSlaveJobList(flatJobEntry,&subJobCount,false);

	for(jobIndex = 0;jobIndex<subJobCount;jobIndex++)
		{
		jobAdded = false;
		EnterCriticalSection(&globals->slaveJobListLock);\

		for(i=0;i<globals->slaveJobListSize;i++)
			{
			if(globals->slaveJobList[i].job->encodeDone)
				{	
				// Add the job to the job list
				memcpy(&globals->slaveJobList[i].job,&subSlaveJobList[jobIndex].job,sizeof(SLAVEJOBLISTENTRY));
				globals->slaveJobList[i].job->jobNumber = globals->slaveJobCounter;
				globals->slaveJobCounter++;
				jobAdded = true;
				break;
				}
			}
		LeaveCriticalSection(&globals->slaveJobListLock);

		if(jobAdded)
			{
			globals->visualLog->logPrintf("Sucessfully added job %d \"%s\" to the joblist\n",globals->slaveJobList[jobIndex].job->jobNumber,subSlaveJobList[jobIndex].job->flatJob.jobName);
			}
		else
			{
			globals->visualLog->logPrintf("Failed to added job \"%s\" to the joblist. The Job List is full\n",subSlaveJobList[jobIndex].job->flatJob.jobName);
			break;
			}
		}

	globals->netHelper->freeSlaveJobList(subSlaveJobList);

	delete [] flatJobEntry;
	return(true);
}
#endif ENCODERSERVER

#ifdef ENCODER
// *****************************************************************************
// jobServerClass::connectJobServer()
//
// *****************************************************************************
bool jobServerClass::connectJobServer(char * networkName)
{
	int status;
	fd_set writeFds;
	fd_set readFds;
	fd_set errorFds;
	timeval statusTimer;
	DWORD curTime;
	DWORD startTime;

	startTime = timeGetTime();

	connectionStatus = NET_NOT_CONNECTED;

	tcp = new tcpClass();

	do{
		switch(connectionStatus)
			{
			case NET_NOT_CONNECTED:
				commSocket = tcp->createSocket();

				if(commSocket<0)
					{
					connectionStatus = JOB_SERVER_FAILED;
					delete tcp;
					return(false);
					}
				status = tcp->connectAddress(commSocket,
											 JOB_SERVER_PORT_NUMBER,
											 networkName);

				if(!status)
					{
					connectionStatus = JOB_SERVER_CONNECTED;
					return(true);
					}

				connectionStatus = JOB_SERVER_CONNECTING;
				// continue looping
				break;

			case NET_CONNECTING:
				memset(&statusTimer,0,sizeof(timeval));
				FD_ZERO(&readFds);
				FD_ZERO(&writeFds);
				FD_ZERO(&errorFds);
		        FD_SET(commSocket,&writeFds);
		        FD_SET(commSocket,&readFds);
		        FD_SET(commSocket,&errorFds);

				if(select(0,&readFds,&writeFds,&errorFds,&statusTimer)<0)
					{
					connectionStatus = JOB_SERVER_FAILED;
					tcp->closeSocket(commSocket);
					delete tcp;
					return(false);
					}

				if(FD_ISSET(commSocket,&writeFds))
					{
					connectionStatus = JOB_SERVER_CONNECTED;
					return(true);
					}
				// No status change, so we continue looping
				break;

			}
		curTime = timeGetTime();
		}while(1 /*(curTime - startTime)<5000*/);

	connectionStatus = JOB_SERVER_FAILED;
	tcp->closeSocket(commSocket);
	delete tcp;
	return(false);
}

// *****************************************************************************
// jobServerClass::disconnectJobServer()
//
// *****************************************************************************
void jobServerClass::disconnectJobServer()
{
	if(connectionStatus == NET_CONNECTED)
		{
		connectionStatus = NET_NOT_CONNECTED;
		tcp->closeSocket(commSocket);
		delete tcp;
		}
}

// *****************************************************************************
// jobServerClass::disconnectJobServer()
//
// *****************************************************************************
bool jobServerClass::sendJob(FLATJOBENTRY * flatJobEntry)
{
	JOBSERVERHEADER header;
	
	header.command = jobServerSubmit;
	header.size = sizeof(FLATJOBENTRY);

	if(!tcp->safeSendBytes(commSocket,(char *)&header,sizeof(JOBSERVERHEADER)))
		{
		return(false);
		}

	if(!tcp->safeSendBytes(commSocket,(char *)flatJobEntry,sizeof(FLATJOBENTRY)))
		{
		return(false);
		}

	return(true);
}

// *****************************************************************************
// jobServerClass::sendClose()
//
// *****************************************************************************
bool jobServerClass::sendClose()
{
	JOBSERVERHEADER header;

	header.command = jobServerClose;
	header.size = 0;

	if(!tcp->safeSendBytes(commSocket,(char *)&header,sizeof(JOBSERVERHEADER)))
		{
		return(false);
		}

	return(true);
}

// *****************************************************************************
// jobServerClass::getStatus()
//
// *****************************************************************************
JOBSERVERSTATS * jobServerClass::getStatus()
{
	JOBSERVERHEADER header;
	JOBSERVERSTATS * jobStats;

	header.command = jobServerStatus;
	header.size = 0;

	if(!tcp->safeSendBytes(commSocket,(char *)&header,sizeof(JOBSERVERHEADER)))
		{
		return(false);
		}

	header.command = 0;
	header.size = 0;

	// Get the return header
	if(!tcp->safeReceiveBytes(commSocket,(char *)&header,sizeof(JOBSERVERHEADER)))
		{
		return(false);
		}

	// return header should have a signature
	if(header.command == jobServerStatusReturn)
		{
		jobStats = (JOBSERVERSTATS *)new char [header.size];

		if(!tcp->safeReceiveBytes(commSocket,(char *)jobStats,header.size))
			{
			return(false);
			}
		}

	return(jobStats);
}
#endif ENCODER
