// *****************************************************************************
//
//
// netHelper.cpp
// 
//
// *****************************************************************************

#ifdef ENCODERSERVER
#include "../encoderserver/es.h"
#endif

#ifdef ENCODER
#include "../encoder/en.h"
#endif

extern GLOBALS * globals;

// *****************************************************************************
// netHelperClass::netHelperClass()
//
// netHelper class constructor
// *****************************************************************************
netHelperClass::netHelperClass()
{

}

// *****************************************************************************
// netHelperClass::~netHelperClass()
//
// netHelper class destructor
// *****************************************************************************
netHelperClass::~netHelperClass()
{

}

// *****************************************************************************
// netHelperClass::loadSlaveList()
//
// *****************************************************************************
bool netHelperClass::loadSlaveList(char * filename)
{
	FILE * inFile;
	char line[4096];
	char * ptr;
	char * machineNameStart;
	char * machineIpStart;
	SLAVEENCODEMACHINES * machineList;
	int numSlaves;

	
	inFile = fopen(filename,"r");

	numSlaves=0;
	machineList = (SLAVEENCODEMACHINES *) malloc(sizeof(SLAVEENCODEMACHINES)*(numSlaves+1));

	if(!inFile)
		{
		logPrintf("Failed to Find an encodeslave.txt file\n");
		}
	else
		{

		for(;;)
			{
			fgets(line,4096,inFile);

			ptr = &line[0];	

			if(*ptr!='#')
				{
				for(;;)
					{
					//Find first quote
					if(*ptr=='"')
						{
						break;
						}

					if(!*ptr)
						{
						break;
						}
					ptr++;
					}
				if(*ptr)
					{
					ptr++;
					machineNameStart = ptr;
					for(;;)
						{
						//Find second quote
						if(*ptr=='"')
							{
							break;
							}

						if(!*ptr)
							{
							break;
							}
						ptr++;
						}
					
					if(*ptr)
						{
						*ptr++=0;

						for(;;)
							{
							//Find equal sign
							if(*ptr=='=')
								{
								break;
								}

							if(!*ptr)
								{
								break;
								}
							ptr++;
							}
						
						if(*ptr)
							{
							ptr++;

							for(;;)
								{
								//Find the first non white space character
								if(*ptr!=' ')
									{
									break;
									}
								if(!*ptr)
									{
									break;
									}
								ptr++;
								}
							
							if(*ptr)
								{
								machineIpStart = ptr;

								for(;;)
									{
									//Delete the ending CR/LF
									if((*ptr==0x0a) || (*ptr==0x20) || (*ptr==0x0d) || (*ptr==0))
										{
										*ptr = 0;
										break;
										}
									ptr++;
									}
								logPrintf("Slave %d - \"%s\" on \"%s\"\n",numSlaves, machineNameStart, machineIpStart);
								// Now add the strings to the list
								strcpy(machineList->slave[numSlaves].machineName,machineNameStart);
								strcpy(machineList->slave[numSlaves].machineIp,machineIpStart);
								numSlaves++;
								machineList = (SLAVEENCODEMACHINES *) realloc(machineList, sizeof(SLAVEENCODEMACHINES)*(numSlaves+1));
								}
							}
						}
					}

				}

			if(feof(inFile))
				break;

			}

		// And last set the global slave machine list pointers
		fclose(inFile);	
		}

	if(numSlaves==0)
		{
		// If input file is bogus, then create a local host entry.
		logPrintf("Creating Default Slave entry on 0.0.0.0\n");

		strcpy(machineList->slave[0].machineName,"Local Machine");
		strcpy(machineList->slave[0].machineIp,"0.0.0.0");
		numSlaves++;
		}

	globals->slaveEncodeMachines = machineList;
	globals->numSlaveEncodeMachines = numSlaves;

	logPrintf("Found %d Slaves\n",globals->numSlaveEncodeMachines);

	return(true);

}

// *****************************************************************************
// netHelperClass::freeSlaveList()
//
// *****************************************************************************
void netHelperClass::freeSlaveList(SLAVEENCODEMACHINES * machineList)
{
	free(machineList);
}

// *****************************************************************************
// netHelperClass::createSlaveJobList()
//
// *****************************************************************************
SLAVEJOBLIST * netHelperClass::createSlaveJobList(FLATJOBENTRY * flatJob, int * subJobsOutput, bool useBuiltinSlave)
{
	char * ptr;
	int subJobNumber;
	int subJobSection;
	__int64 subSectionSize;
	char subJobText[255];
	SLAVEJOBLIST * slaveJobList;
	subJobNumber = 0;

	slaveJobList =  (SLAVEJOBLIST *) malloc(sizeof(SLAVEJOBLIST)*2);

	subJobSection = 0;
		
	for(;;)
		{
		if(subJobNumber!=0)
			{
			slaveJobList =  (SLAVEJOBLIST *) realloc(slaveJobList, sizeof(SLAVEJOBLIST)*(subJobNumber+2));
			}

		memcpy(&slaveJobList[subJobNumber].job->flatJob,flatJob,sizeof(FLATJOBENTRY));

		slaveJobList[subJobNumber].job->flatJob.convertConfig.subJobSection = subJobSection;

		// Extra parameters
		slaveJobList[subJobNumber].job->slaveAllocated = -1;
		slaveJobList[subJobNumber].job->encodeDone = false;
		slaveJobList[subJobNumber].job->errorRetryCounter = 0;
		strcpy(slaveJobList[subJobNumber].job->errorText,"");
		
		// Remove the tail on the output filename
		ptr = strrchr(slaveJobList[subJobNumber].job->flatJob.convertConfig.outputFilename,'.');
		if(ptr)
			{
			*ptr = 0;
			}


		if(!useBuiltinSlave && slaveJobList[subJobNumber].job->flatJob.convertConfig.hyperEncode &&
			!slaveJobList[subJobNumber].job->flatJob.convertConfig.useSpecifiedTarget && 
			(slaveJobList[subJobNumber].job->flatJob.convertConfig.vbrNumberOfDiscs>1) && 
			slaveJobList[subJobNumber].job->flatJob.convertConfig.numRanges==1)
			{
			if(subJobSection==0)
				{
				//logPrintf("Performing Hyper Encode on Job %d. Splitting Job into %d subjobs\n",jobNumber,
				//			slaveJobList[subJobNumber].job->flatJob.convertConfig.vbrNumberOfDiscs);

				subSectionSize = slaveJobList[subJobNumber].job->flatJob.convertConfig.rangelist[0].endingFilePos -
									slaveJobList[subJobNumber].job->flatJob.convertConfig.rangelist[0].startingFilePos;

				subSectionSize /= slaveJobList[subJobNumber].job->flatJob.convertConfig.vbrNumberOfDiscs;
				}

			// The job needs to be broken UP
			slaveJobList[subJobNumber].job->flatJob.convertConfig.rangelist[0].endingFilePos = slaveJobList[subJobNumber].job->flatJob.convertConfig.rangelist[0].startingFilePos + (subSectionSize * (subJobSection+1)) + 500000;
			slaveJobList[subJobNumber].job->flatJob.convertConfig.rangelist[0].endingFileFrame =0;
			slaveJobList[subJobNumber].job->flatJob.convertConfig.rangelist[0].startingFilePos = slaveJobList[subJobNumber].job->flatJob.convertConfig.rangelist[0].startingFilePos + (subSectionSize * subJobSection);
			slaveJobList[subJobNumber].job->flatJob.convertConfig.rangelist[0].startingFileFrame =0;
			slaveJobList[subJobNumber].job->flatJob.convertConfig.rangelist[0].valid = true;
			
			// Disable chapter support when hyper encode is enabled.
			slaveJobList[subJobNumber].job->flatJob.convertConfig.insertChapters = false;

			slaveJobList[subJobNumber].job->flatJob.convertConfig.vbrNumberOfDiscs = 1;
			sprintf(subJobText," - Disc %d",subJobSection+1);
			strcat(slaveJobList[subJobNumber].job->flatJob.convertConfig.outputFilename,subJobText);
			strcat(slaveJobList[subJobNumber].job->flatJob.jobName,subJobText);

			subJobNumber++;
			subJobSection++;

			if(subJobSection == flatJob->convertConfig.vbrNumberOfDiscs)
				{
				break;
				}
			}
		else
			{
			subJobNumber++;
			break;			
			}
		}

	*subJobsOutput = subJobNumber;

	return(slaveJobList);

}

// *****************************************************************************
// netHelperClass::freeSlaveJobList()
//
// *****************************************************************************
void netHelperClass::freeSlaveJobList(SLAVEJOBLIST * slaveJobList)
{
	free(slaveJobList);
}
