// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
// netmaster class
// *****************************************************************************
#ifndef netmaster_h
#define netmaster_h

//Slave's network status
enum {
	NET_NOT_CONNECTED =0,
	NET_CONNECTING,
	NET_CONNECTED,
	NET_FAILED,
	NET_DOWN,
	NET_SHORTDELAYRETRY,
	NET_LONGDELAYRETRY,
	NET_VERSIONMISMATCH,
	NET_PERMANENTLYDOWN,

};

//
// netMaster : This sends commands to slaves to do the encode work
//
typedef struct {
	char 				name[256];	//Human Name
	char				networkName[MAX_PATH];	// network Name / IP of machine
	int					port;					// Port on machine
	DWORD				connectionStartTime;	// Used for socket comm failure
	int					connectionStatus;		// Status of the connection UP / DOWN / ETC
	DWORD				connectionErrorTime;	// Used for socket comm failure
	int					attemptedLongRetrys;
	int					encodeStatus;			// Status of the encode operation
	SOCKET				commSocket;				// Communication Socket
	CONVERSIONSTATS		conversionStats;		// Last update of conversion Statistics from Slave
} ENCODESLAVESINGLE;

typedef struct {
	ENCODESLAVESINGLE	slave[1];
} ENCODESLAVES;


typedef struct {
	netMasterClass *		master;
	FLATJOBENTRY *			flatJob;

} FAKESLAVECONVERSIONTHREAD;

class netMasterClass {

	public:
									netMasterClass();
									~netMasterClass();
				void				begin(int numSlaves,ENCODESLAVES * slaves, bool myUseBuiltinSlave);
				void				end();
				bool				testConnection(int slaveNumber);
				bool				sendJob(int slaveNumber, FLATJOBENTRY * flatJob);
				bool				safeSendBytes(int slaveNumber, char * ptr, int bytesRemaining);
				bool				safeReceiveBytes(int slaveNumber, char * ptr, int bytesRemaining);
				int					getStatus(int slaveNumber);
				CONVERSIONSTATS *	getStats(int slaveNumber);
				int					findFreeEncodeResource();
				int					getEncodeStatus(int slaveNumber);
				int					sendClose(int slaveNumber);
				CONVERSIONSTATS *	getEncodeStats(int slaveNumber);
				int					getError(int slaveNumber, char * errorBuffer);
				int					sendAbort(int slaveNumber);
				int					sendPause(int slaveNumber);
				int					sendResume(int slaveNumber);
				int					sendSleep(int slaveNumber, int sleep);
				int					sendThreadPriority(int slaveNumber, int threadPriority);
				int					sendShutdown(int slaveNumber);
				void				checkConnections(int numChances);
				int					getConnectionStatus(int slaveNumber);
#ifdef USEBUILTINSLAVE
				bool				startFakeSlaveConversion(FLATJOBENTRY * flatJob);
				static DWORD WINAPI	fakeSlaveConversionThread(LPVOID lParam);
				bool 				fakeSlaveConversion(JOBLISTENTRY * jobListEntry);
#endif

	protected:
				int					numEncodeSlaves;
				ENCODESLAVES *		encodeSlaves;
				tcpClass *			tcp;
				bool				useBuiltinSlave;
				DWORD				fakeSlaveConversionThreadId;
				HANDLE				fakeSlaveConversionThreadHandle;
				CRITICAL_SECTION	conversionStatusLock;
				volatile int		conversionStatus;
				volatile bool		fakeSlaveThreadTerminated;

};




#endif