// *****************************************************************************
//
//
// renderSubtitles.cpp
// 
//
// *****************************************************************************

#ifdef ENCODERSERVER
#include "../encoderserver/es.h"
#endif

#ifdef ENCODER
#include "../encoder/en.h"
#endif

extern GLOBALS * globals;

// *****************************************************************************
// renderSubtitlesClass::renderSubtitlesClass()
//
// subtitles class constructor
// *****************************************************************************
renderSubtitlesClass::renderSubtitlesClass()
{
}

// *****************************************************************************
// renderSubtitlesClass::~renderSubtitlesClass()
//
// renderSubtitles class destructor
// *****************************************************************************
renderSubtitlesClass::~renderSubtitlesClass()
{
}

// *****************************************************************************
// renderSubtitlesClass::init()
//
// *****************************************************************************
void renderSubtitlesClass::init()
{
	startingFrame =0x7fffffff;
	
	endingFrame = 0x7fffffff;

	colorTable[0] = 0x00000000;
	colorTable[1] = 0x00000000;
	colorTable[2] = 0x00000000;
	colorTable[3] = 0x00000000;

	firstEver = true;

}

// *****************************************************************************
// renderSubtitlesClass::end()
//
// *****************************************************************************
void renderSubtitlesClass::end()
{

}

// *****************************************************************************
// renderSubtitlesClass::renderSubtitle()
//
// *****************************************************************************
void renderSubtitlesClass::postSubtitleData(int curFrame, SUBTITLEDATA * sub, FRAMEDATA * subtitleFrame)
{
	// Render the subtitle to the subtitle frame buffer

	startingFrame = curFrame;
	endingFrame = startingFrame + sub->header.duration;

	switch(sub->format.mode)
		{
		case divxSubtitleTypeGraphicFull:
		case divxSubtitleTypeGraphicHalf:

			renderSubtitle(curFrame, sub, subtitleFrame);
			subtitleFrameDisplay = true;
			break;

		case divxSubtitleTypeText:
			subtitleFrameDisplay = false;
			break;
		}

}

// *****************************************************************************
// renderSubtitlesClass::updateSubtitleCounters()
//
// *****************************************************************************
void renderSubtitlesClass::updateSubtitleCounters(int curFrame, FRAMEDATA * subtitleFrame)
{
	if((curFrame<startingFrame) || (curFrame > endingFrame))
		{
		subtitleFrameDisplay = false;
		}
}

// *****************************************************************************
// renderSubtitlesClass::renderSubtitle()
//
// *****************************************************************************
bool renderSubtitlesClass::renderSubtitle(int curFrame, SUBTITLEDATA * sub, FRAMEDATA * subtitleFrame)
{
	unsigned char * ptr;
	int size;
	bool renderOdd;
	unsigned char * tempPtr;
	int tempSize;
/*
	int i;
	int a;
	int r;
	int g;
	int b;

	if(globals->playerConfig->sharedConfig.subtitleColorOverride)
		{
		for(i=0;i<4;i++)
			{
			a = (globals->playerConfig->sharedConfig.subtitleColorData[i] &0xff000000)>>24;
			b = (globals->playerConfig->sharedConfig.subtitleColorData[i] &0x00ff0000)>>16;
			g = (globals->playerConfig->sharedConfig.subtitleColorData[i] &0x0000ff00)>>8;
			r = globals->playerConfig->sharedConfig.subtitleColorData[i] &0x000000ff;

			colorTable[i] = (a<<24) | (r<<16) | (g<<8) | b;
			}
		}
	else
		{
		*/

		// Color 0 (The BG Color) is always transparent
		colorTable[0] = sub->format.palette[0] | 0xff000000;
		colorTable[1] = sub->format.palette[1];
		colorTable[2] = sub->format.palette[2];
		colorTable[3] = sub->format.palette[3];
		/*
		}
*/
	subtitlePalette[sub->format.remap[0]] = 0;
	subtitlePalette[sub->format.remap[1]] = 1;
	subtitlePalette[sub->format.remap[2]] = 2;
	subtitlePalette[sub->format.remap[3]] = 3;

	ptr = subtitleFrame->bufferStart;
	current = (unsigned int *) ptr;

	subtitleWidth = sub->header.right - sub->header.left;
	subtitleHeight =sub->header.bot - sub->header.top; 

	dataPtr[0] = sub->data;
	dataPtr[0] += sub->header.evenStartPos;
	dataNibble[0] = false;
	dataPtr[1] = sub->data;
	dataPtr[1] +=sub->header.oddStartPos;
	dataNibble[1] = false;

	size = (*dataPtr[0]++)<<8;
	size |= *dataPtr[0]++;

	subtitleField = 0;
	
	stride = subtitleWidth;

	outputPtr = current;
	if(sub->format.mode == divxSubtitleTypeGraphicHalf)
		{
		dataSize[0] = (sub->bytesUsed - sub->header.evenStartPos)-2;
		dataSize[1] = 0;
		renderOdd = false;
		}
	else
		{
		dataSize[0] = dataPtr[1] - dataPtr[0];
		dataSize[1] = (dataPtr[0] + size) - dataPtr[1];
		renderOdd = true;
		}

	//Hack
	//sub->format->mode = divxSubtitleTypeGraphicHalf;
	//dataSize[0] = dataPtr[1] - dataPtr[0];
	//dataSize[1] = 0;
	//renderOdd=false;

	lineCounter = 0;

	for(;;)
		{
		tempPtr = dataPtr[0];
		tempSize = dataSize[0];
		renderLineRgb32();
		if(renderOdd)
			{
			subtitleField ^= 0x01;
			renderLineRgb32();
			subtitleField ^= 0x01;
			}
		else
			{
			dataPtr[0] = tempPtr;
			dataSize[0] = tempSize;
			renderLineRgb32();
			}

		if(dataSize[0]<=0)
			break;
		}

	subtitleFrame->width = subtitleWidth;
	subtitleFrame->height = lineCounter;
		
		//lineCounter;
	return(true);
}

// *****************************************************************************
// renderSubtitlesClass::renderLineRgb32()
// 
// *****************************************************************************
void renderSubtitlesClass::renderLineRgb32()
{
	int	token;
	int run;
	int color;
//	int	alpha;
	int xPos;
	bool outputData;
	unsigned int outputColor;

	xPos = 0;

	outputData =false;

	do{
		token = getBits();
		if(token<0x04)
			{
			token = (token<<4) | getBits();
			if(token<0x10) 
				{
				token = (token<<4) | getBits();
				if(token<0x40)	
					{
					token = (token<<4) | getBits();
					if(!(token&0xfff0))
						{
						color = subtitlePalette[token&0x03];
						token = -1;
						}
					}
				}
			}
		if(token>=0)
			{
			run = token>>2;
			color = subtitlePalette[token&0x03];
			}
		else
			{
			// run length is the remainder of the current line.
			run = subtitleWidth - xPos;

			// Not sure if this is correct, but it prevents a crash
			if(run<0)
				run = 0;
			if(run>subtitleWidth)
				run = subtitleWidth;
			}

		// Render..
		xPos +=run;

		outputColor = colorTable[color];

		if(!(outputColor&0xff000000))
			{
			// Color is a value 0 - f
			// Run is the output color run length
			while(run--)
				{
				*outputPtr++ = outputColor;  
				}
			outputData = true;
			}
		else
			{
			while(run--)
				{
				*outputPtr++ = 0xff000000;
				}
			}

		if(xPos > subtitleWidth)
			token = -1;

		}while(token>=0);

	if(outputData)
		{
		lineCounter++;
		current+=stride;
		}

	outputPtr = current;
	if(dataNibble[subtitleField])
		{
		getBits();
		}

}
// *****************************************************************************
// renderSubtitlesClass::getBits()
// 
// *****************************************************************************
int renderSubtitlesClass::getBits()
{
	if(!dataSize[subtitleField])
		return(0);

	if(!dataNibble[subtitleField])
		{
		dataNibble[subtitleField] = !dataNibble[subtitleField];
		return(*dataPtr[subtitleField]>>4);
		}
	else
		{
		dataSize[subtitleField]--;
		dataNibble[subtitleField] = !dataNibble[subtitleField];
		return((*dataPtr[subtitleField]++)&0x0f);
		}
}

