// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// tcp.cpp
// 
//
// *****************************************************************************

#ifdef ENCODERSERVER
#include "../encoderserver/es.h"
#endif

#ifdef ENCODER
#include "../encoder/en.h"
#endif

extern GLOBALS * globals;

// *****************************************************************************
// tcpClass::tcpClass()
//
// tcp class constructor
// *****************************************************************************
tcpClass::tcpClass()
{
}

// *****************************************************************************
// tcpClass::~tcpClass()
//
// tcp class destructor
// *****************************************************************************
tcpClass::~tcpClass()
{
}


// *****************************************************************************
// tcpClass::createSocket()
//
// Create the Socket
// *****************************************************************************
SOCKET tcpClass::createSocket()
{
    unsigned long flag;
	SOCKET openSock;

	if (WSAStartup(MAKEWORD( 2, 0 ), &wsaData))
		{
		return(-1);
		}

	openSock = socket(PF_INET, SOCK_STREAM, 0);

	if(openSock<0)
		{
		return(-1);
		}


	// Set Socket to non blocking mode
	flag = 1;
	ioctlsocket(openSock,FIONBIO,&flag);

	// Allow Socket to be reused
	flag = 1;
    setsockopt(openSock, SOL_SOCKET, SO_REUSEADDR, (char *) &flag, sizeof(flag));
 
	return(openSock);
}

// *****************************************************************************
// tcpClass::closeSocket()
//
// Close the Socket
// *****************************************************************************
void tcpClass::closeSocket(SOCKET openSock)
{
	if(openSock>=0)
		{
		closesocket(openSock);
		}
}

// *****************************************************************************
// tcpClass::bindSocket(int port)
//
// Close the Socket
// *****************************************************************************
bool tcpClass::bindSocket(SOCKET openSock, int portNumber, unsigned long useInterface)
{
	sockaddr_in  sockAddr;

	memset(&sockAddr, 0, sizeof(sockAddr));
	sockAddr.sin_family = AF_INET;

	if(useInterface)
		{
		sockAddr.sin_addr.s_addr = useInterface;
		}
	else
		{
		sockAddr.sin_addr.s_addr = INADDR_ANY;
		}

	sockAddr.sin_port = htons(portNumber);

	if(bind(openSock,(SOCKADDR *)&sockAddr,sizeof(sockAddr))!=0)
		{
		closesocket(openSock);
		return(false);
		}

	return(true);
}

// *****************************************************************************
// tcpClass::connectAddress()
//
// connect to a target
// *****************************************************************************
int tcpClass::connectAddress(SOCKET openSock, int portNumber, char * nameOfMachine)
{
	sockaddr_in  sockAddr;
	hostent * hostAddr;
	DWORD address;
	
	hostAddr = gethostbyname(nameOfMachine);
	if(!hostAddr)
		{
		return(-1);
		}
	memset(&sockAddr, 0, sizeof(sockAddr));
	sockAddr.sin_family = AF_INET;
	memcpy(&address, hostAddr->h_addr_list[0], sizeof(DWORD));
	sockAddr.sin_addr.s_addr = address;
	sockAddr.sin_port = htons(portNumber);

	return(connect(openSock,(SOCKADDR *)&sockAddr,sizeof(sockAddr)));

}
// *****************************************************************************
// tcpClass::setSocketListen()
//
// set the socket to listen mode
// *****************************************************************************
bool tcpClass::setSocketListen(SOCKET openSock)
{
	listen(openSock,SOMAXCONN);

	return(true);
}


// *****************************************************************************
// tcpClass::acceptConnection()
//
// 
// *****************************************************************************
SOCKET tcpClass::acceptConnection(SOCKET openSock)
{
	// Accept the connection if one exists

	return (accept(openSock,NULL,NULL));
	
}

// *****************************************************************************
// tcpClass::peekBytes()
//
// 
// *****************************************************************************
int tcpClass::peekBytes(SOCKET openSocket,char * inBuffer, int inBufferSize)
{

	// Peek at how many bytes are available
	return(recv(openSocket,inBuffer,inBufferSize,MSG_PEEK));
}

// *****************************************************************************
// tcpClass::receiveBytes()
//
// 
// *****************************************************************************
int tcpClass::receiveBytes(SOCKET openSocket,char * inBuffer, int inBufferSize)
{
	// get the bytes pending
	return(recv(openSocket,inBuffer,inBufferSize,0));
}

// *****************************************************************************
// tcpClass::sendBytes()
//
// 
// *****************************************************************************
int tcpClass::sendBytes(SOCKET openSocket,char * outBuffer, int outBufferSize)
{
	// send the bytes 
	return(send(openSocket,outBuffer,outBufferSize,0));
}

// *****************************************************************************
// tcpClass::closeConnection()
//
// 
// *****************************************************************************
void tcpClass::closeConnection(SOCKET openSocket)
{
	closesocket(openSocket);
}

// *****************************************************************************
// tcpClass::safeReceiveBytes()
//
// *****************************************************************************
bool tcpClass::safeReceiveBytes(SOCKET openSocket, char * ptr, int bytesRemaining)
{
	int bytesReceived;
	int err;
	DWORD startTime;
	DWORD endTime;

	startTime = timeGetTime();

	while(bytesRemaining)
		{
		bytesReceived = receiveBytes(openSocket, ptr, bytesRemaining);
		err = WSAGetLastError();
		if((bytesReceived==0) || (bytesReceived<0 && err!=WSAEWOULDBLOCK))
			{
			return(false);
			}

		if(bytesReceived>0)
			{
			ptr+=bytesReceived;
			bytesRemaining-=bytesReceived;
			}
		else
			{
			Sleep(20);
			endTime = timeGetTime();
			if((endTime - startTime)>5000)
				{
				return(false);
				}
			}

		}

	return(true);
}

// *****************************************************************************
// tcpClass::safeSendBytes()
//
// *****************************************************************************
bool tcpClass::safeSendBytes(SOCKET openSocket, char * ptr, int bytesRemaining)
{
	int bytesSent;
	int err;
	DWORD startTime;
	DWORD endTime;

	startTime = timeGetTime();

	while(bytesRemaining)
		{
		bytesSent = sendBytes(openSocket, ptr, bytesRemaining);
		err = WSAGetLastError();
		if((bytesSent==0) || (bytesSent<0 && err!=WSAEWOULDBLOCK))
			{
			return(false);
			}

		if(bytesSent>0)
			{
			ptr+=bytesSent;
			bytesRemaining-=bytesSent;
			}
		else
			{
			Sleep(20);
			endTime = timeGetTime();
			if((endTime - startTime)>5000)
				{
				return(false);
				}
			}
		}

	return(true);
}