// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// audiopostprocess.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

// *****************************************************************************
// audioPostProcessClass::audioPostProcessClass()
//
// conversion class constructor
// *****************************************************************************
audioPostProcessClass::audioPostProcessClass()
{
	pathToDll = new char [MAX_PATH];

	resample = false;
	streamCopy = false;
	d2aDll = NULL;
	dvd2AviResampleInit = NULL;
	dvd2AviResample = NULL;
	processBuffer[0] = NULL;
	processBuffer[1] = NULL;

}

// *****************************************************************************
// audioPostProcessClass::~audioPostProcessClass()
//
// conversion class destructor
// *****************************************************************************
audioPostProcessClass::~audioPostProcessClass()
{
	end();
	delete [] pathToDll;

}

// *****************************************************************************
// audioPostProcessClass::init()
//
// *****************************************************************************
bool audioPostProcessClass::init()
{
	strcpy(pathToDll,globals->pathToExeDir);
	strcat(pathToDll,"\\ncoded2a.dll");

	d2aDll = LoadLibrary(pathToDll);

	if(d2aDll==NULL)
		{
		// If failed, make a second attempt at debug env location
		#ifdef _DEBUG
		strcpy(pathToDll,globals->pathToExeDir);
		strcat(pathToDll,"\\..\\ncoded2a\\Debug\\ncoded2a.dll");
		#else
		strcpy(pathToDll,globals->pathToExeDir);
		strcat(pathToDll,"\\..\\ncoded2a\\Release\\ncoded2a.dll");
		#endif
		d2aDll=LoadLibrary(pathToDll);
		if(d2aDll==NULL)
			return(false);
		}

	dvd2AviResampleInit	= (DVD2AVIRESAMPLEINIT) GetProcAddress(d2aDll, TEXT_DVD2AVIRESAMPLEINIT);
	dvd2AviResample	= (DVD2AVIRESAMPLE) GetProcAddress(d2aDll, TEXT_DVD2AVIRESAMPLE);

	if(!dvd2AviResampleInit || !dvd2AviResample)
		{
		FreeLibrary(d2aDll);
		d2aDll=NULL;
		return(false);
		}
	return(true);
}

// *****************************************************************************
// audioPostProcessClass::end()
//
// *****************************************************************************
void audioPostProcessClass::end()
{
	if(d2aDll)
		{
		FreeLibrary(d2aDll);
		}

	if(processBuffer[0])
		{
		delete [] processBuffer[0]->bufferStart;
		delete [] processBuffer[0];
		}

	if(processBuffer[1])
		{
		delete [] processBuffer[1]->bufferStart;
		delete [] processBuffer[1];
		}

}

// *****************************************************************************
// audioPostProcessClass::setResample()
//
// on = for resample
// *****************************************************************************
void audioPostProcessClass::setResample(bool setOn)
{
	resample = setOn;

}

// *****************************************************************************
// audioPostProcessClass::setResample()
//
// on = for resample
// *****************************************************************************
void audioPostProcessClass::setStreamCopy(bool setOn)
{
	streamCopy = setOn;

}
// *****************************************************************************
// audioPostProcessClass::setNormalize()
//
// on = for normalize
// level is % range from 0 -100
// min = min sample rate found during scan
// max = max sample rate found during scan
// *****************************************************************************
void audioPostProcessClass::setNormalize(int normType, int level, short min, short max)
{
	normalize = normType;
	normalizeLevel = level;
	normalizeMin = min;
	normalizeMax = max;

}

// *****************************************************************************
// audioPostProcessClass::getOutputRate()
//
// on = TRUE for scale is enabled
// width = output width
// height = output height
// *****************************************************************************
void audioPostProcessClass::getOutputRate(int * sampleRate, int * channels)
{

	*sampleRate = outputSampleRate;
	*channels = outputChannels;
}

// *****************************************************************************
// audioPostProcessClass::getNormalizeMinMax()
//
// min = min sample rate found during minmax processing
// max = max sample rate found during minmax processing
// *****************************************************************************
void audioPostProcessClass::getNormalizeMinMax(short * min, short * max)
{

	*min = normalizeMinFound;
	*max = normalizeMaxFound;
}

// *****************************************************************************
// audioPostProcessClass::begin()
//
// sampleRate = input Samplerate
// channels = input number of channels
// *****************************************************************************
bool audioPostProcessClass::begin(int sampleRate, int channels)
{
	resampleWork = false;
	firstTime = true;
	outputSampleRate = sampleRate;
	outputChannels = channels;

	// If Stream copy, then make audio processing a NOOP.
	if(streamCopy)
		{
		normalize = NORMALIZE_NONE;
		resample = false;
		}

	if(normalize == NORMALIZE_ONE_PASS)
		{
		normalizeMinFound = -4096;
		normalizeMaxFound = 4096;
		}
	else
		{
		normalizeMinFound = 32767;
		normalizeMaxFound = -32767;
		}
	computeNormalizeRatio(normalizeMin,normalizeMax);


	if(resample && sampleRate==48000)
		{
		outputSampleRate = 44100;
		dvd2AviResampleInit();
		resampleWork=true;
		}

	if(resampleWork)
		{
		// These buffers are only used when Vidomi is resampling 48Khz Audio to 44.1Khz
		processBuffer[0] = new AUDIODATA [1];
		processBuffer[1] = new AUDIODATA [1];

		processBuffer[0]->bufferStart = new unsigned char [AUDIO_BUFFER_SIZE];
		processBuffer[0]->bufferSize = AUDIO_BUFFER_SIZE;
		processBuffer[0]->data = processBuffer[0]->bufferStart;
		processBuffer[0]->bytesUsed = 0;
		processBuffer[0]->pts = 0;
		processBuffer[0]->type = 0;

		processBuffer[1]->bufferStart = new unsigned char [AUDIO_BUFFER_SIZE];
		processBuffer[1]->bufferSize = AUDIO_BUFFER_SIZE;
		processBuffer[1]->data = processBuffer[1]->bufferStart;
		processBuffer[1]->bytesUsed = 0;
		processBuffer[1]->pts = 0;
		processBuffer[0]->type = 0;
		}
	else
		{
		processBuffer[0] = NULL;
		processBuffer[1] = NULL;
		}

	return(true);
}


// *****************************************************************************
// audioPostProcessClass::processSamples()
//
// Returns pointer to Samples to Write to Disk
// *****************************************************************************
AUDIODATA * audioPostProcessClass::processSamples(AUDIODATA * inputSamples)
{
	int samplesOut;
	int samplesIn;
	int sampleBytesDone;
	int sampleWork;
	int	sampleBytes;
	samplesOut = 0;
	samplesIn = 1;
	int bytesUsed;
	float sampleMult;
	float byteRate;
	int	ptsToAdd;
	int temp;


	if(firstTime)
		{
		audioPts = inputSamples->pts;
		firstTime = false;
		errorAccum =0.0f;
		}

	if(resampleWork)
		{
		sampleWork = inputSamples->bytesUsed;
		sampleBytesDone=0;
		
		while(sampleBytesDone!=sampleWork)
			{
			sampleBytes = sampleWork - sampleBytesDone;
			if(sampleBytes>(1024*4))
				sampleBytes=(1024*4);

			bytesUsed = dvd2AviResample(inputSamples->bufferStart+sampleBytesDone, processBuffer[samplesOut]->data, sampleBytes);
			processBuffer[samplesOut]->bytesUsed += bytesUsed;
			processBuffer[samplesOut]->data +=bytesUsed;
			sampleBytesDone+=sampleBytes;
			}

		//Setup the input buffer for the next run
		inputSamples->data = inputSamples->bufferStart;
		inputSamples->bytesUsed=0;
		
		if(normalize)
			{
			// Normalize the output Buffer
			normalizeSamples(processBuffer[samplesOut]);
			}
		//give the data a timecode since the input timecode is inaccurate now
		processBuffer[samplesOut]->pts = audioPts;
		sampleMult = 90000.0f / 44100.0f;
		byteRate =  ((float) processBuffer[samplesOut]->bytesUsed / (float)(2 * outputChannels));

		// calculate our position in time for the next buffer output
		// Accumulate error into byteRateError
		ptsToAdd = (int) (sampleMult * byteRate); 
		errorAccum += (sampleMult * byteRate) - ptsToAdd;
		if(errorAccum>1)
			{
			temp = (int) errorAccum;
			errorAccum -= temp;
			ptsToAdd += temp;
			}
		audioPts += ptsToAdd;
		samplesOut=!samplesOut;
		samplesIn=!samplesIn;
		
		// Set the output buffer type to be the same as the input buffer type
		processBuffer[samplesIn]->type = inputSamples->type; 
		return(processBuffer[samplesIn]);
		}
	else
		{
		if(normalize)
			{
			// Normalize the output Buffer
			normalizeSamples(inputSamples);
			}

		//give the data a timecode since the input timecode is inaccurate

		inputSamples->pts = audioPts;

		sampleMult = 90000.0f / (float)outputSampleRate;
		byteRate =  ((float) inputSamples->bytesUsed / (float)(2 * outputChannels));

		// calculate our position in time for the next buffer output
		// Accumulate error into byteRateError
		ptsToAdd = (int) (sampleMult * byteRate); 
		errorAccum += (sampleMult * byteRate) - ptsToAdd;
		if(errorAccum>1)
			{
			temp = (int) errorAccum;
			errorAccum -= temp;
			ptsToAdd += temp;
			}
		audioPts += ptsToAdd;

		return(inputSamples);
		}
}

// *****************************************************************************
// audioPostProcessClass::resetAudioPts()
//
// Returns pointer to Samples to Write to Disk
// *****************************************************************************
void audioPostProcessClass::resetAudioPts()
{
	// Reset the first Time flag to force a regrab of the PTS

	firstTime = true;
}

// *****************************************************************************
// audioPostProcessClass::processSamples()
//
// Returns pointer to Samples to Write to Disk
// *****************************************************************************
void audioPostProcessClass::processForMinMax(AUDIODATA * inputSamples)
{
	short * ptr;
	short sample;
	int count;

	ptr = (short * )inputSamples->bufferStart;

	count = inputSamples->bytesUsed>>1;
	
	while(count--)
		{
		sample = *ptr++;
		if(sample<normalizeMinFound)
			normalizeMinFound = sample;
		if(sample>normalizeMaxFound)
			normalizeMaxFound = sample;
		}
}

// *****************************************************************************
// audioPostProcessClass::normalize()
//
// Returns pointer to the normalized Samples 
// *****************************************************************************
AUDIODATA * audioPostProcessClass::normalizeSamples(AUDIODATA * inputSamples)
{
	short * ptr;
	double sample;
	int count;

	if(normalize == NORMALIZE_ONE_PASS)
		{
		processForMinMax(inputSamples);
		computeNormalizeRatio(normalizeMinFound,normalizeMaxFound);
		}
	ptr = (short * )inputSamples->bufferStart;

	count = inputSamples->bytesUsed>>1;
	
	while(count--)
		{
		sample = (double)*ptr;
		sample = sample * normalizeRatio;
		*ptr++ = (short) sample;
		}

	return(inputSamples);
}

// *****************************************************************************
// audioPostProcessClass::normalize()
//
// Returns pointer to the normalized Samples 
// *****************************************************************************
void audioPostProcessClass::computeNormalizeRatio(short minSamp, short maxSamp)
{
	short highestFound;
	
	highestFound = abs(minSamp);
	if(highestFound<abs(maxSamp))
		{
		highestFound = abs(maxSamp);
		}

	normalizeRatio = 32767.0f;
	normalizeRatio = normalizeRatio * ((double)normalizeLevel / 100.0f);
	normalizeRatio = normalizeRatio / highestFound;

}
