// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// conversion.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

// *****************************************************************************
// conversionClass::conversionClass()
//
// conversion class constructor
// *****************************************************************************
conversionClass::conversionClass()
{


	abortConversion=false;
	pauseConversion=false;
	sleepPerFrame = 0;
	auxStats = new AUXSTATS [1];
	memset(auxStats,0,sizeof(AUXSTATS));

	conversionStats = new CONVERSIONSTATS [1];
	memset(conversionStats,0,sizeof(CONVERSIONSTATS));
	conversionThreadTerminated = true;

}

// *****************************************************************************
// conversionClass::~conversionClass()
//
// conversion class destructor
// *****************************************************************************
conversionClass::~conversionClass()
{

	int i;
	// Terminate the conversion thread
	for(i=0;i<100;i++)
		{
		if(conversionThreadTerminated)
			break;
		Sleep(100);
		}

	delete [] auxStats;
	delete [] conversionStats;

}
// *****************************************************************************
// conversionClass::getConversionStatus()
//
// *****************************************************************************
void conversionClass::getConversionStats(CONVERSIONSTATS * stats)
{
	memcpy(stats,conversionStats,sizeof(CONVERSIONSTATS));
}

// *****************************************************************************
// conversionClass::startSingleConversion()
//
// Returns TRUE if success
// *****************************************************************************
bool conversionClass::startSingleConversion(JOBLISTENTRY * myJobEntry)
{
	int i;
	SYSTEM_INFO sysInfo;
	char * ptr;
	char filename[MAX_PATH];
	int ending;
	int channel;
	int channelNum;
	bool ret;
	char subJobName[255];
	conversionMpegClass * conversionMpeg;


	jobEntry = myJobEntry;
	settings = jobEntry->convertConfig;
	inputInfo = jobEntry->gatherResults;


	// Special case for the MPEG editing hack
	if(settings->mpegConversion)
		{
		conversionMpeg = new conversionMpegClass();
		if(conversionMpeg)
			{
			ret = conversionMpeg->startSingleConversion(settings,inputInfo);
			delete conversionMpeg;
			return(ret);
			}
		else
			{
			return(false);
			}
		}

	displayOutputWindow = true;
	abortConversion=false;
	pauseConversion=false;
	encodeMode = settings->encodeMode;
	dualMode = settings->multiThread;
	writeAviIntro = settings->introAvi;
	colorSpace = settings->encodeColorSpace;
	normalizeAudio = settings->normalizeAudio;

	maxWriteBuffers = globals->dualCpuEncodeBuffers;

	// Configure the ststs job name being worked on
	strcpy(conversionStats->jobName,jobEntry->jobName);

	// The range list should always be used. The start and end file position should 
	// be localized to this file.

	xtempPos = 0;
	lasttpos = 0;
	rangeIndex = 0;

	totalRanges = getTotalNumRanges();

	if (!settings->rangelist[0].valid) 
		{
		settings->rangelist[0].startingFilePos=0;
		settings->rangelist[0].startingFileFrame=0;
		settings->rangelist[0].endingFilePos=0x7fffffffffffffff;
		settings->rangelist[0].endingFileFrame=0;
		}

	// set 1. segment start
	settings->startingFilePos=settings->rangelist[0].startingFilePos;
	settings->startingFileFrame=settings->rangelist[0].startingFileFrame;
	settings->endingFilePos=settings->rangelist[0].endingFilePos;
	settings->endingFileFrame=settings->rangelist[0].endingFileFrame;
	
	switch(settings->encodeMode)
		{
		case ENCODE_MODE_DIVXVBR:
		case ENCODE_MODE_XVIDVBR:
			settings->splitAtKeyframe=true;
			break;

		case ENCODE_MODE_HYBRID2TIME:
		case ENCODE_MODE_VBRTIME:
		case ENCODE_MODE_CBR:
		case ENCODE_MODE_DIVXCBR:
		case ENCODE_MODE_DIVXCBRQUALITY:
			settings->useSpecifiedTarget=false;
			settings->splitAtKeyframe=false;
			break;
		case ENCODE_MODE_CBR2PASS:
			settings->useSpecifiedTarget=false;
			settings->splitAtKeyframe=true;
			break;

		case ENCODE_MODE_HYBRID1:
		case ENCODE_MODE_HYBRID2:
		case ENCODE_MODE_VBR:
		case ENCODE_MODE_DIVXVBRPLUS:
		case ENCODE_MODE_XVIDHYBRID:
		default:
			settings->splitAtKeyframe=false;
			break;
		}


	logPrintf("Source has been identified as :\n");															
	logPrintf("Source Width : %d\n",inputInfo->videoWidth);
	logPrintf("Source Height : %d\n",inputInfo->videoHeight);
	logPrintf("Source Framerate : %f\n",inputInfo->videoFramerate);
	logPrintf("Source Aspect Ratio : %f\n",inputInfo->videoAspectRatio);

	if(settings->processAudio)
		{
		channelNum=0;
		for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
			{
			if(settings->processAudioSelected[channel]!=-1)
				{
				logPrintf("Channel[%d] Samplerate : %d\n",channel+1,inputInfo->audioRate[settings->processAudioSelected[channel]]);
				logPrintf("Channel[%d] Channels : %d\n",channel+1,inputInfo->audioChannels[settings->processAudioSelected[channel]]);
				logPrintf("Channel[%d] Source Audio Name : %s\n",channel+1,&inputInfo->audioName[settings->processAudioSelected[channel]][0]);
				logPrintf("Channel[%d] mp3MinBitrate=%d\n",channel+1,settings->mp3MinBitrate[channel]);				
				logPrintf("Channel[%d] mp3MaxBitrate=%d\n",channel+1,settings->mp3MaxBitrate[channel]);				
				logPrintf("Channel[%d] mp3EncodeMode=%d\n",channel+1,settings->mp3EncodeMode[channel]);				
				logPrintf("Channel[%d] mp3ChannelType=%d\n",channel+1,settings->mp3ChannelType[channel]);				
				logPrintf("Channel[%d] mp3Quality=%d\n",channel+1,settings->mp3Quality[channel]);				
				logPrintf("Channel[%d] mp3OutputFrequency=%d\n",channel+1,settings->mp3OutputFrequency[channel]);				
				logPrintf("Channel[%d] mp3VbrQuality=%d\n",channel+1,settings->mp3VbrQuality[channel]);				
				channelNum++;
				}
			else
				{
				break;
				}
			}
		numAudioStreams = channelNum;
		}
	else
		{
		numAudioStreams = 0;
		}

	// Determine if VBR audio is being used. This determines if the scan pass
	// needs to perform an encode of the audio to determine the final size
	usingVbrAudio = false;
	if(settings->processAudio && settings->mp3)
		{
		for(channel=0;channel<channelNum;channel++)
			{
			if(settings->mp3EncodeMode[channel] == LAME_VBR)
				{
				usingVbrAudio = true;
				break;
				}
			}
		}

	if(!settings->enhancedAvi)
		{
		// Disable all features that require enhanced AVI support
		settings->insertChapters = false;
		settings->processSubtitles = false;
		}
	
	channelNum=0;
	if(settings->enhancedAvi)
		{
		settings->processSubtitles=false;

		for(channel=0;channel<MAX_SUBTITLE_CHANNELS;channel++)
			{
			if(settings->processSubtitleMode[channel]!=-1)
				{
				switch(settings->processSubtitleMode[channel])
					{
					case SUBTITLE_MODE_GRAPHIC:
						if(settings->processSubtitleOnlyEven[channel])
							{
							logPrintf("Subtitle Channel[%d] - Graphic Half\n",channel+1);
							}
						else
							{
							logPrintf("Subtitle Channel[%d] - Graphic Full\n",channel+1);
							}
						break;
					case SUBTITLE_MODE_TEXT:
						logPrintf("Subtitle Channel[%d] - Text from File : %s\n",channel+1, settings->processSubtitleTextFile[channel]);
						break;
					}
				channelNum++;
				settings->processSubtitles=true;
				}
			else
				{
				break;
				}
			}
		}
	numSubtitleStreams = channelNum;

	// Configure the stream IVTC and film mode vars

	switch(settings->ivtcFilmMode)
		{
		case 0:
			settings->film=false;
			settings->ivtc=false;
			break;
		case 1:
			settings->film=true;
			settings->ivtc=false;
			break;
		case 2:
			settings->deinterlace=false;
			settings->film=true;
			settings->ivtc=false;
			settings->ivtcOffset=-1;
			settings->ivtcDeComb=true;
			break;
		case 3:
			settings->film=false;
			settings->ivtc=true;
			settings->ivtcDeComb=true;
			break;
		case 4:
			settings->film=false;
			settings->ivtc=true;
			settings->ivtcDeComb=false;
			break;
		}

	if((settings->encodeMode == ENCODE_MODE_CBR) || 
		(settings->encodeMode == ENCODE_MODE_CBR2PASS) ||
		(settings->encodeMode == ENCODE_MODE_DIVXCBR) ||
		(settings->encodeMode == ENCODE_MODE_DIVXCBRQUALITY) ||
		(settings->encodeMode == ENCODE_MODE_DIVXVBR) ||
		(settings->encodeMode == ENCODE_MODE_DIVXVBRPLUS) ||
		(settings->encodeMode == ENCODE_MODE_XVIDVBR) || 
		(settings->encodeMode == ENCODE_MODE_XVIDHYBRID))
		{
		settings->lumaProcessing = false;
		}

	chapterIndex = 0;
	
	// Compute a rough number of header bytes required.. NOTE THIS IS ROUGH!!

	// Subtitle bytes needed
	headerSize = numSubtitleStreams * (sizeof(DIVXSTREAMHEADER) + sizeof(DIVXSUBTITLEHEADER) + sizeof(DIVXSUBTITLEFORMAT) + 65 );
	// Audio Bytes Needed
	headerSize += numAudioStreams * (sizeof(DIVXSTREAMHEADER) + sizeof(WAVEFORMATEX) + 65 );
	// Video Bytes Needed
	headerSize += sizeof(DIVXSTREAMHEADER) + sizeof(BITMAPINFOHEADER) ;

	// Chapter Bytes Needed
	for(i=0;i< settings->numChapters;i++)
		{
		headerSize += sizeof(DIVXSTREAMCHAPTERENTRY) + strlen(settings->chapterName[i]);
		}
	headerSize += sizeof(DIVXSTREAMCHAPTERS);

	// Last we add in 1K of breathing room for misc header data / etc.
	
	headerSize += 0x400;

	// align us to next 0x100 byte bound 
	headerSize = (headerSize+0xff)&0xffffff00;

	// Give us a minimum size of 0x2800
	// This was 0x800 until winXP was released. WinXP seems to not like *some* headers that are 0x800 in length.
	// Making the minimum be 0x2800 (like virtualdub) fixes the problem. odd.

	if(headerSize<0x2800)
		{
		headerSize = 0x2800;
		}

	logPrintf("Starting Conversion with following parameters:\n");															
	logPrintf("encodeMode=%d\n",settings->encodeMode);				
	logPrintf("deinterlace=%d\n",settings->deinterlace);				
	logPrintf("blend=%d\n",settings->blend);				
	logPrintf("blendThreshold=%d\n",settings->blendThreshold);
	logPrintf("hyperScale=%d\n",settings->hyperScale);				
	logPrintf("film=%d\n",settings->film);				
	logPrintf("forceMultiple16=%d\n",settings->forceMultiple16);				
	logPrintf("scale=%d\n",settings->scale);				
	logPrintf("scaleby=%d\n",settings->scaleBy);				
	logPrintf("scaleWidth=%d\n",settings->scaleWidth);				
	logPrintf("scaleHeight=%d\n",settings->scaleHeight);				
	logPrintf("scaleType=%d\n",settings->scaleType);				
	logPrintf("hyperCrop=%d\n",settings->hyperCrop);				
	logPrintf("hyperCropThreshold=%d\n",settings->hyperCropThreshold);				
	logPrintf("cropTop=%d\n",settings->cropTop);				
	logPrintf("cropBottom=%d\n",settings->cropBottom);				
	logPrintf("cropLeft=%d\n",settings->cropLeft);				
	logPrintf("cropRight=%d\n",settings->cropRight);				
	logPrintf("processAudio=%d\n",settings->processAudio);				
	logPrintf("maxOutputSize=%d\n",settings->maxOutputSize);				
	logPrintf("Resample48KhzAudio=%d\n",settings->resample48KhzAudio);				
	logPrintf("mp3=%d\n",settings->mp3);				
	logPrintf("normalizeAudio=%d\n",normalizeAudio);				
	logPrintf("normalizeLevel=%d\n",settings->normalizeLevel);				
	logPrintf("vbrReserve=%d\n",settings->vbrReserve);				
	logPrintf("vbrAudioBitrate=%d\n",settings->vbrAudioBitrate);				
	logPrintf("vbrDeletePassOneFiles=%d\n",settings->vbrDeletePassOneFiles);				
	logPrintf("vbrCrispness=%d\n",settings->vbrCrispness);				
	logPrintf("vbrMaxBitrate=%d\n",settings->vbrMaxBitrate);				
	logPrintf("vbrMinBitrate=%d\n",settings->vbrMinBitrate);				
	logPrintf("vbrMaxKeyFrame=%d\n",settings->vbrMaxKeyFrame);				
	logPrintf("vbrMinKeyFrame=%d\n",settings->vbrMinKeyFrame);				
	logPrintf("vbrMaxReservoir=%d\n",settings->vbrMaxReservoir);				
	logPrintf("vbrMinReservoir=%d\n",settings->vbrMinReservoir);				
	logPrintf("vbrNumberOfDiscs=%d\n",settings->vbrNumberOfDiscs);				
	logPrintf("introAvi=%d\n",settings->introAvi);				
	logPrintf("lumaProcessing=%d\n",settings->lumaProcessing);				
	logPrintf("lumaLowThresh=%f\n",settings->lumaLowThresh);				
	logPrintf("lumaLowBoost=%d\n",settings->lumaLowBoost);				
	logPrintf("lumaDeltaThresh=%f\n",settings->lumaDeltaThresh);				
	logPrintf("lumaDeltaBoost=%d\n",settings->lumaDeltaBoost);				
	logPrintf("startingFilePos=%I64x\n",settings->startingFilePos);				
	logPrintf("startingFileFrame=%d\n",settings->startingFileFrame);				
	logPrintf("endingFilePos=%I64x\n",settings->endingFilePos);				
	logPrintf("endingFileFrame=%d\n",settings->endingFileFrame);				
	logPrintf("ivtc=%d\n",settings->ivtc);				
	logPrintf("ivtcDeComb=%d\n",settings->ivtcDeComb);				
	logPrintf("ivtcOffset=%d\n",settings->ivtcOffset);				
	logPrintf("ivtcPolarity=%d\n",settings->ivtcPolarity);				
	logPrintf("enahcedAvi=%d\n",settings->enhancedAvi);				
	logPrintf("insertChapters=%d\n",settings->insertChapters);				
	logPrintf("numChapters=%d\n",settings->numChapters);
	logPrintf("Divx headerSize=%d\n",headerSize);
	logPrintf("hyperEncode=%d\n",settings->hyperEncode);
	logPrintf("totalRanges=%d\n",totalRanges);
	logPrintf("useSpecifiedTarget=%d\n",settings->useSpecifiedTarget);

	// Set the last chapter index to a number that will never be reached.
	settings->chapterStartFramePos[settings->numChapters]=0x7fffffff;

	for(i=0;i<settings->numChapters;i++)
		{
		logPrintf("chapter%d - StartFramePos=%d\n",i,settings->chapterStartFramePos[i]);
		}		


	logPrintf("outputFilename=%s\n",settings->outputFilename);
	for(i=0;i<settings->inputFilenameIndex;i++)
		{
		logPrintf("inputFilename%d=%s\n",i,settings->inputFilenames[i]);
		}		

	//Create the .stats filename and the .aux filename based on the source mpeg filename
	strcpy(statsFilename,settings->inputFilenames[0]);
	ptr = strrchr(statsFilename,'.');
	if(ptr)
		{
		*ptr = 0;
		}
	sprintf(subJobName,"%d",settings->subJobSection+1);
	strcat(statsFilename,subJobName);

	// Create the various log filenames

	strcpy(auxFilename,statsFilename);
	strcpy(divxNetworksLogFilename,statsFilename);
	strcpy(xvidLogFilename,statsFilename);
	strcat(statsFilename,".stats");
	strcat(auxFilename,".aux");
	strcat(xvidLogFilename,".stats");
	strcat(divxNetworksLogFilename,".log");
	
	if(strlen(divxNetworksLogFilename)>254)
		{
		ptr = strrchr(divxNetworksLogFilename,'\\');
		if(ptr)
			{
			strcpy(filename, ptr);
			strcpy(divxNetworksLogFilename,"c:");
			strcat(divxNetworksLogFilename,filename);
			}
		else
			{
			strcpy(divxNetworksLogFilename,"c:\\divx.log");
			}
		}
	// Now create the filtered output filename
	strcpy(filename,settings->outputFilename);
	ending=strlen(filename);
	if(ending>3)
		{
		if(!strcmp(&filename[ending-4],".avi"))
			filename[strlen(filename)-4]=0;
		if(!strcmp(&filename[ending-4],".vid"))
			filename[strlen(filename)-4]=0;
		if(ending>4)
			{
			if(!strcmp(&filename[ending-5],".divx"))
				filename[strlen(filename)-5]=0;
			if(!strcmp(&filename[ending-5],".svid"))
				filename[strlen(filename)-5]=0;
			if(!strcmp(&filename[ending-5],".xvid"))
				filename[strlen(filename)-5]=0;
			}
		}

	strcpy(outputFilename,filename);

	// If using VBR mode, force audio to be two pass normalize
	/*
	if(((encodeMode ==ENCODE_MODE_VBR) || (encodeMode ==ENCODE_MODE_DIVXVBR) || 
		(encodeMode ==ENCODE_MODE_DIVXVBRPLUS) || (encodeMode ==ENCODE_MODE_VBRTIME)) 
		&& settings->mp3 && settings->processAudio)
		{
		logPrintf("Forcing Audio to two pass normalize\n");
		normalizeAudio = NORMALIZE_TWO_PASS;
		}
	*/

	InitializeCriticalSection(&writeLock);
	writeCommand = writeCommandNoOp;
	writeState = writeStateIdle;

	GetSystemInfo(&sysInfo);

	if(sysInfo.dwNumberOfProcessors==1)
		{
		// If only one CPU is in the system, then force a single thread for conversion
		logPrintf("One CPU found - Forcing Single Thread\n");
		dualMode = false;
		}
	
	// Init the cut Change Flags
	for(channel=0;channel<numAudioStreams;channel++)
		{
		cutChangeAudioFlag[channel]=false;
		}

	cutChangeVideoFlag=false;

	if(dualMode)
		{
		// Do thread Init
		for(channel=0;channel<numAudioStreams;channel++)
			{
			for(i=0;i<maxWriteBuffers;i++)
				{
				processedSamples[channel][i] = new AUDIODATA [1];
				memset(processedSamples[channel][i],0,sizeof(AUDIODATA));
				processedSamplesReady[channel][i]=false;
				processedSamplesCutChange[channel][i]=false;
				}
			}

		for(channel=0;channel<numSubtitleStreams;channel++)
			{
			for(i=0;i<maxWriteBuffers;i++)
				{
				processedSubtitles[channel][i] = new SUBTITLEDATA [1];
				memset(processedSubtitles[channel][i],0,sizeof(SUBTITLEDATA));
				processedSubtitlesReady[channel][i]=false;
				}
			}

		for(i=0;i<maxWriteBuffers;i++)
			{
			processedFrames[i] = new FRAMEDATA [1];
			memset(processedFrames[i],0,sizeof(FRAMEDATA));
			processedFramesReady[i]=false;
			processedFramesCutChange[i]=false;
			}

		conversionThreadTerminated = false;
		writeThreadHandle = CreateThread(NULL,0,	conversionWriteThread ,(LPVOID)this , 0 ,&writeThreadId);	

#ifndef VIDOMISLAVE
		SetThreadPriority(writeThreadHandle,THREAD_PRIORITY_NORMAL);
#else
		SetThreadPriority(writeThreadHandle,THREAD_PRIORITY_IDLE);
#endif
		logPrintf("Created Conversion Write Thread. WriteThreadID = %x\n",writeThreadId);
		}
	else
		{
		// Do thread Init
		for(i=0;i<maxWriteBuffers;i++)
			{
			processedFramesReady[i]=false;
			}

		for(channel=0;channel<numAudioStreams;channel++)
			{

			for(i=0;i<maxWriteBuffers;i++)
				{
				processedSamplesReady[channel][i]=false;
				}
			}

		for(channel=0;channel<numSubtitleStreams;channel++)
			{

			for(i=0;i<maxWriteBuffers;i++)
				{
				processedSubtitlesReady[channel][i]=false;
				}
			}

		}

	ret = conversionPrepare();

	// Force stats window to 100% / 0 time remaining
	conversionStats->percentOfPassComplete = 100.0f;
	conversionStats->estimatedRemainingPassTime = 0;

	return(ret);
}

// *****************************************************************************
// conversionClass::conversionPrepare()
//
// Returns TRUE if success
// *****************************************************************************

bool conversionClass::conversionPrepare(void)
{
	FRAMEDATA * endingFrame;
	FRAMEDATA *	frame;
	SUBTITLEDATA * subtitle;
	AUDIODATA *	samples;
	bool prepareFirstPass;
	bool finished;
	int i;
	int	tempState;
	volatile bool readyVideo;
	int currentFrame;
	int lastFrame;
	int channel;
	int ret;
	bool returnStatus;

	returnStatus = true;

	switch(encodeMode)
		{

		case ENCODE_MODE_CBR:
			logPrintf("Starting CBR Conversion\n");
			break;
		case ENCODE_MODE_CBR2PASS:
			logPrintf("Starting CBR 2 Pass Conversion\n");
			break;
		case ENCODE_MODE_HYBRID1:
			logPrintf("Starting HYBRID1 Conversion\n");
			break;
		case ENCODE_MODE_HYBRID2:
			logPrintf("Starting HYBRID2 Conversion\n");
			break;
		case ENCODE_MODE_VBR:
			logPrintf("Starting VBR Conversion\n");
			break;
		case ENCODE_MODE_DIVXCBR:
			logPrintf("Starting DIVX CBR Conversion\n");
			break;
		case ENCODE_MODE_DIVXCBRQUALITY:
			logPrintf("Starting DIVX CBR Quality Conversion\n");
			break;
		case ENCODE_MODE_DIVXVBR:
			logPrintf("Starting DIVX VBR Conversion\n");
			break;
		case ENCODE_MODE_DIVXVBRPLUS:
			logPrintf("Starting DIVX VBRPLUS Conversion\n");
			break;
		case ENCODE_MODE_HYBRID2TIME:
			logPrintf("Starting HYBRID2TIME Conversion\n");
			break;
		case ENCODE_MODE_VBRTIME:
			logPrintf("Starting VBRTIME Conversion\n");
			break;
		case ENCODE_MODE_XVIDVBR:
			logPrintf("Starting XVID VBR Conversion\n");
			break;
		case ENCODE_MODE_XVIDHYBRID:
			logPrintf("Starting XVID Hybrid Conversion\n");
			break;

		default:
			break;
		}

	currentPass = 1;
	totalPasses = 1;
	startTime = timeGetTime(); 

	// First scan the movie to create the datarate map and obtain the number of frames
	// If user want us to normalize, then we need to do a scan
	if((settings->processAudio && (normalizeAudio == NORMALIZE_TWO_PASS)) || 
		(encodeMode == ENCODE_MODE_VBR) || 
		(encodeMode == ENCODE_MODE_VBRTIME) ||
		(encodeMode == ENCODE_MODE_DIVXVBR) ||
		(encodeMode == ENCODE_MODE_DIVXVBRPLUS) ||
		(encodeMode == ENCODE_MODE_XVIDVBR) ||
		(encodeMode == ENCODE_MODE_CBR2PASS))
		{
		logPrintf("Starting Scan of Source\n");
		ret = scanMovie();
		if(ret==SCAN_FINISHED_OK || ret ==SCAN_NOT_NEEDED)
			{
			// Wait 1 seconds before starting the second encode
			Sleep(1000);
			logPrintf("Scan of Source Completed Successfully\n");
			}
		else
			{
			logPrintf("Scan of Source Failed or was aborted\n");
			}
		if(ret!=SCAN_NOT_NEEDED)
			{
			if(ret==SCAN_FAILED || settings->stopAfterPassOne)
				{
				if(dualMode)
					{
					EnterCriticalSection(&writeLock);
					writeCommand = writeCommandDie;
					LeaveCriticalSection(&writeLock);
					while(1)
						{
						EnterCriticalSection(&writeLock);					
						tempState = writeState;
						LeaveCriticalSection(&writeLock);
						if(tempState==writeStateDead)
							break;
						Sleep(0);
						}
					}
				return(false);
				}
			}
		}

	currentPass = totalPasses;

	if(!prepareSetup(3))
		{
		if(dualMode)
			{
			EnterCriticalSection(&writeLock);
			writeCommand = writeCommandDie;
			LeaveCriticalSection(&writeLock);
			while(1)
				{
				EnterCriticalSection(&writeLock);					
				tempState = writeState;
				LeaveCriticalSection(&writeLock);
				if(tempState==writeStateDead)
					break;
				Sleep(0);
				}

			}
		return(false);
		}

	prepareFirstPass = true;



	// Skip frames before starting encode loop
	
	currentFrame = 0;
	lastFrame = 0;

    // reset pts diagnostic vars
    //checkPts(0,0,0,0);

	// Process each frame in the source video

	logPrintf("Entering Decode Conversion Loop\n");

	while (avRead->decodeFrame())
		{
		frame = avRead->getFrameData();
		vhardPts = frame->hardPts;
		vpts = frame->pts;

		// Set the PTS if this is the first frame
		if(currentFrame==settings->startingFileFrame)
			{
			//frame->pts = frame->hardPts;
			 if (!rangeIndex) 
				{
				frame->pts = frame->hardPts;
				}

			}

		// Do the generic per frame work and the first pass setup work
		if(!prepareFrameWork(prepareFirstPass,3))
			{
			logPrintf("prepareFrameWork(...) Failed\n");
			prepareCleanup(prepareFirstPass,3);
			if(dualMode)
				{
				EnterCriticalSection(&writeLock);
				writeCommand = writeCommandDie;
				LeaveCriticalSection(&writeLock);
				while(1)
					{
					EnterCriticalSection(&writeLock);					
					tempState = writeState;
					LeaveCriticalSection(&writeLock);
					if(tempState==writeStateDead)
						break;
					Sleep(0);
					}
				}
			return(false);
			}
		if(prepareFirstPass)
			{
			// Send Write Thread Command To Start
			writeBufferHead = 0;
			writeBufferTail = 0;
			if(dualMode)
				{
				EnterCriticalSection(&writeLock);
				writeCommand = writeCommandStart;
				LeaveCriticalSection(&writeLock);
				}
			else
				{
				if(!writeSetup())
					{
					returnStatus = false;
					break;
					}
				}
			prepareFirstPass = false;
			}

		endingFrame = postProcess->processFrame(frame);

		if(audio)
			{
			for(channel=0;channel<numAudioStreams;channel++)
				{
				samples = avRead->getAudioData(channel);

				if(samples->bytesUsed)
					{
					// If were not at the starting frame, then eat the audio
					if(currentFrame<settings->startingFileFrame)
						{
						samples->data = samples->bufferStart;
						samples->bytesUsed = 0;
						}
					else
						{
						processedSamplesCutChange[channel][writeBufferHead] = cutChangeAudioFlag[channel];
						cutChangeAudioFlag[channel] = false;

						if(dualMode)
							{
							// hand the buffer to the other thread
							memcpy(processedSamples[channel][writeBufferHead],samples,sizeof(AUDIODATA));
							EnterCriticalSection(&writeLock);
							processedSamplesReady[channel][writeBufferHead] = true;
							LeaveCriticalSection(&writeLock);
							// Setup a new clean buffer for this thread
							samples->bufferStart = new unsigned char [samples->bufferSize];
							samples->data = samples->bufferStart;
							samples->bytesUsed = 0;
							}
						else
							{
							processedSamples[channel][writeBufferHead] = samples;
							processedSamplesReady[channel][writeBufferHead] = true;
							}
						}				
					}
				}
			}

		if(settings->processSubtitles)
			{
			for(channel=0;channel<numSubtitleStreams;channel++)
				{
				if(settings->processSubtitleMode[channel] == SUBTITLE_MODE_GRAPHIC)
					{
					subtitle = avRead->getSubtitleData(channel);
					if(subtitle->bytesUsed && subtitle->complete)
						{
						// If were not at the starting frame, then eat the subtitle
						if(currentFrame<settings->startingFileFrame)
							{
							subtitle->data = subtitle->bufferStart;
							subtitle->bytesUsed = 0;
							subtitle->complete=0;
							}
						else
							{
							if(dualMode)
								{
								// hand the buffer to the other thread
								memcpy(processedSubtitles[channel][writeBufferHead],subtitle,sizeof(SUBTITLEDATA));
								EnterCriticalSection(&writeLock);
								processedSubtitlesReady[channel][writeBufferHead] = true;
								LeaveCriticalSection(&writeLock);
								// Setup a new clean buffer for this thread
								subtitle->bufferStart = new unsigned char [subtitle->bufferSize];
								subtitle->data = subtitle->bufferStart;
								subtitle->bytesUsed = 0;
								subtitle->complete=0;
								}
							else
								{
								processedSubtitles[channel][writeBufferHead] = subtitle;
								processedSubtitlesReady[channel][writeBufferHead] = true;
								}
							
							}				
						}
					}
				}

			}

		if(currentFrame<settings->startingFileFrame)
			{
			// If we're not at the starting frame, then display the just decoded frame
			// note that the display frame is usually done from the other thread
			#ifndef VIDOMISLAVE
			if(endingFrame)
				{
				globals->outputVideo->displayFrame(endingFrame);
				}
			#endif
			}
		else
			{
			processedFramesFilePos[writeBufferHead] = currentFilePos;
			processedFramesCutChange[writeBufferHead] = cutChangeVideoFlag;
			cutChangeVideoFlag=false;

			if(settings->film)
				{
				processedFramesFramePos[writeBufferHead] = (int)((float)(currentFrame)* 1.25f);
				}
			else
				{
				processedFramesFramePos[writeBufferHead] = currentFrame;
				}
		
			if(dualMode)
				{

				// hand the buffer to the other thread
				if(endingFrame)
					{
					memcpy(	processedFrames[writeBufferHead],endingFrame,sizeof(FRAMEDATA));
					EnterCriticalSection(&writeLock);
					processedFramesReady[writeBufferHead] = true;
					LeaveCriticalSection(&writeLock);
					// Setup a new clean buffer for this thread
					endingFrame->allocStart = new unsigned char [endingFrame->bufferSize+16];
					endingFrame->bufferStart = (unsigned char *)((unsigned long)endingFrame->allocStart+15 &0xfffffff0);
					endingFrame->data = endingFrame->bufferStart;
					}
				else
					{
					processedFrames[writeBufferHead]->bufferStart = NULL;
					EnterCriticalSection(&writeLock);
					processedFramesReady[writeBufferHead] = true;
					LeaveCriticalSection(&writeLock);
					}
				}
			else
				{
				processedFrames[writeBufferHead] = endingFrame;
				processedFramesReady[writeBufferHead] = true;
				if(!writeFrameWork())
					{
					returnStatus = false;
					break;
					}
				}

			EnterCriticalSection(&writeLock);					
			tempState = writeState;
			LeaveCriticalSection(&writeLock);

			// Did the other process log an error
			if(tempState == writeStateError)
				{
				logPrintf("Work Thread reported a writeStateError\n");
				returnStatus = false;
				break;
				}

			writeBufferHead++;
			if(writeBufferHead==maxWriteBuffers)
				writeBufferHead=0;

			// Check that the next buffer is available. (Ideally it always is)
			do{
				EnterCriticalSection(&writeLock);					
				readyVideo = processedFramesReady[writeBufferHead];
				LeaveCriticalSection(&writeLock);
				if(readyVideo)
					Sleep(0);
				}while(readyVideo);
			}

		currentFrame++;

		// check for end of range
		if((currentFilePos + settings->startingFilePos) >= settings->endingFilePos) 
			{
			if(lastFrame++ >= settings->endingFileFrame) 
				{ 
				// end here or take next segment
				if (checkSegmentedFileEnd(settings))
					{
					break;
					}
	            logPrintf("Encoded %d Cut Frames \n",currentFrame);
                currentFrame = 0;
                lastFrame = 0;
               }   
			}

		if(pauseConversion)
			{
			while(pauseConversion && !abortConversion)
				{
				Sleep(1000);
				}
			}
		
		if(sleepPerFrame)
			{
			Sleep(sleepPerFrame);
			}

		if(avRead->checkEnd() || abortConversion)
			{
			
			logPrintf("End of Mpeg hit, or abort pressed\n");
			break;
			}

		}
	logPrintf("Encoded %d Frames \n",currentFrame);

	logPrintf("Finishing encode. Waiting for buffers to empty and Terminating threads\n");

	if(dualMode)
		{
		do	{
			finished = true;
			EnterCriticalSection(&writeLock);
			for(i=0;i<maxWriteBuffers;i++)
				{
				if(processedFramesReady[i])
					{
					finished=false;
					break;
					}
				}
			LeaveCriticalSection(&writeLock);
			if(!finished)
				{
				Sleep(0);
				}	
			}while(!finished);

		EnterCriticalSection(&writeLock);
		writeCommand = writeCommandEnd;
		LeaveCriticalSection(&writeLock);

		while(1)
			{
			EnterCriticalSection(&writeLock);					
			tempState = writeState;
			LeaveCriticalSection(&writeLock);
			if(tempState==writeStateIdle)
				break;
			Sleep(0);
			}

		EnterCriticalSection(&writeLock);
		writeCommand = writeCommandDie;
		LeaveCriticalSection(&writeLock);

		while(1)
			{
			EnterCriticalSection(&writeLock);					
			tempState = writeState;
			LeaveCriticalSection(&writeLock);
			if(tempState==writeStateDead)
				break;
			Sleep(0);
			}
		}
	else
		{
		writeCleanup();
		}

	logPrintf("calling prepareCleanup(...)\n");

	prepareCleanup(prepareFirstPass,3);
	
	if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_DIVXVBR) ||
		(encodeMode == ENCODE_MODE_DIVXVBRPLUS) || (encodeMode == ENCODE_MODE_VBRTIME)||
		(encodeMode == ENCODE_MODE_XVIDVBR))
		{
		logPrintf("Deleting the .stats and .aux files\n");
		//Delete the stat and aux files created on the first pass
		if(settings->vbrDeletePassOneFiles)
			{
			DeleteFile(statsFilename);
			DeleteFile(auxFilename);
			DeleteFile(divxNetworksLogFilename);
			DeleteFile(xvidLogFilename);
			}
		}

	return(returnStatus);
}

// *****************************************************************************
// conversionClass::conversionWriteThread()
//
// Returns TRUE if success
// *****************************************************************************

DWORD WINAPI conversionClass::conversionWriteThread(LPVOID lParam)
{
	conversionClass * conv;
	int ret;

	conv = (conversionClass *) lParam;
	
	ret = conv->conversionWrite();

	logPrintf("Terminating Conversion Write Thread.\n");	

	conv->writeThreadId=0;
	conv->conversionThreadTerminated=true;
	ExitThread(ret);   
	return(ret);
}

// *****************************************************************************
// conversionClass::conversionWrite()
//
// Returns TRUE if success
// *****************************************************************************

bool conversionClass::conversionWrite(void)
{
	bool writing;
	bool scanning;
	volatile int wCmd;
	int i;
	int channel;

	writing = false;
	scanning = false;
	statsFile = NULL;
	auxFile = NULL;

	for(;;)
		{
		EnterCriticalSection(&writeLock);
		wCmd = writeCommand;
		LeaveCriticalSection(&writeLock);

		switch(wCmd)
			{
			case writeCommandStart :
				if(writeSetup())
					{
					EnterCriticalSection(&writeLock);
					writeState = writeStateWriting;
					writeCommand = writeCommandNoOp;
					LeaveCriticalSection(&writeLock);
					writing=true;
					}
				else
					{
					EnterCriticalSection(&writeLock);
					writeState = writeStateError;
					writeCommand = writeCommandNoOp;
					LeaveCriticalSection(&writeLock);
					writing=false;
					}
				break;

			case writeCommandScanStart :
				if(scanSetup())
					{
					EnterCriticalSection(&writeLock);
					writeState = writeStateScanning;
					writeCommand = writeCommandNoOp;
					LeaveCriticalSection(&writeLock);
					scanning=true;				
					}
				else
					{
					EnterCriticalSection(&writeLock);
					writeState = writeStateError;
					writeCommand = writeCommandNoOp;
					LeaveCriticalSection(&writeLock);
					scanning=false;
					}
				break;

			case writeCommandEnd :
				EnterCriticalSection(&writeLock);
				writeState = writeStateIdle;
				writeCommand = writeCommandNoOp;
				LeaveCriticalSection(&writeLock);
				writeCleanup();
				writing=false;
				break;

			case writeCommandScanEnd :
				EnterCriticalSection(&writeLock);
				writeState = writeStateIdle;
				writeCommand = writeCommandNoOp;
				LeaveCriticalSection(&writeLock);
				if(!scanCleanup())
					{
					EnterCriticalSection(&writeLock);
					writeState = writeStateError;
					writeCommand = writeCommandNoOp;
					LeaveCriticalSection(&writeLock);
					}
				scanning=false;
				break;

			case writeCommandDie :
				for(channel=0;channel<numAudioStreams;channel++)
					{
					for(i=0;i<maxWriteBuffers;i++)
						{
						if(processedSamples[channel][i])
							{
							delete [] processedSamples[channel][i];
							processedSamples[channel][i] = NULL;
							}
						}
					}
				for(channel=0;channel<numSubtitleStreams;channel++)
					{
					for(i=0;i<maxWriteBuffers;i++)
						{
						if(processedSubtitles[channel][i])
							{
							delete [] processedSubtitles[channel][i];
							processedSubtitles[channel][i] = NULL;
							}
						}
					}
				for(i=0;i<maxWriteBuffers;i++)
					{
					if(processedFrames[i])
						{
						delete [] processedFrames[i];
						processedFrames[i] = NULL;
						}
					}
				EnterCriticalSection(&writeLock);
				writeState = writeStateDead;
				writeCommand = writeCommandNoOp;
				LeaveCriticalSection(&writeLock);
				return(false);

			case writeCommandNoOp :
			default : 
				if(!writing && !scanning)
					{
					Sleep(100);
					}
				break;
			}
		
		if(writing)
			{
			if(!writeFrameWork())
				{
				writing = false;
				EnterCriticalSection(&writeLock);
				writeState = writeStateError;
				writeCommand = writeCommandNoOp;
				LeaveCriticalSection(&writeLock);
				}
			}

		if(scanning)
			{
			if(!scanFrameWork())
				{
				scanning = false;
				EnterCriticalSection(&writeLock);
				writeState = writeStateError;
				writeCommand = writeCommandNoOp;
				LeaveCriticalSection(&writeLock);
				}
			}

		if(pauseConversion)
			{
			while(pauseConversion && !abortConversion)
				{
				Sleep(1000);
				}
			}

		if(sleepPerFrame)
			{
			Sleep(sleepPerFrame);
			}

		}
	return(true);
}

// *****************************************************************************
// conversionClass::writeFrameWork()
//
// Returns TRUE if success
// *****************************************************************************

bool conversionClass::writeFrameWork(void)
{
	int keyFrame;
	int bitsRem;
	int currentBps;
	volatile bool readyAudio;
	volatile bool readyVideo;
	volatile bool readySubtitle;
	BITMAPINFOHEADER * outHeader;
	int bitrateUsed;
	int frameBytesAllocated;
	int audioFramesOutput;
	double bitrateMultiplier;
	FRAMEDATA * encodedFrame;
	int ret;
	DWORD encodeFlags;
	AUDIODATA * endingSamples;
	__int64 tempCurPos;
	int tempFramesDone;
	int bytesPerFrame;
	int aviOverhead;
	int remainingMovieSeconds;
	__int64 remainingBytes;
	int remainingFrames;
	double luma;
	float boost;
	int channel;
	bool performSplit;
	int averageBr;
	int savedFrameCounter;

	EnterCriticalSection(&writeLock);
	readyVideo = processedFramesReady[writeBufferTail];
	LeaveCriticalSection(&writeLock);

	if(readyVideo)
		{
		if(writeFirstPass)
			{
			splitAvi = false;

			if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_HYBRID1) || 
				(encodeMode == ENCODE_MODE_HYBRID2) || (encodeMode == ENCODE_MODE_HYBRID2TIME) ||
				(encodeMode == ENCODE_MODE_VBRTIME))
				{
				// Configure DIVX to our starting values

				encodeVideo1->setCrispness(settings->vbrCrispness);
				// Set to low motion
				encodeVideo1->setCompressionMinMax(2,16);
				if((encodeMode == ENCODE_MODE_HYBRID2) || (encodeMode == ENCODE_MODE_HYBRID2TIME))
					{
					// Do the same for the second encoder
					encodeVideo2->setCrispness(settings->vbrCrispness);
					encodeVideo2->setCompressionMinMax(2,16);
					encodeVideo2->setDataRate(6000*1024);
					}
				}
			writeFirstPass=false;
			}

		if(writeAviIntro && ((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_HYBRID1) ||
			(encodeMode == ENCODE_MODE_HYBRID2) || (encodeMode == ENCODE_MODE_HYBRID2TIME) || 
			(encodeMode == ENCODE_MODE_VBRTIME)))
			{
			writeIntro(false);
			}

		savedFrameCounter = writeFrameCounter;

		///////////////////////////
		// Video Buffer Processing
		///////////////////////////
		if(processedFrames[writeBufferTail]->bufferStart)
			{
			if(encodeMode == ENCODE_MODE_DIVXVBRPLUS)
				{
				if(!fread(frameStats,sizeof(FRAMESTATS),1,statsFile))
					{
					logPrintf("The .stats file ended early. Using the Average for this frame!!!\n");
					frameStats->frameSize = inputVideoAverageBytes;
					frameStats->keyFrame = false;
					}

				//bitrateMultiplier = 1.0f;
				bitrateMultiplier =  (double)frameStats->frameSize / inputVideoAverageBytes;
				currentBps = (int)(outputVideoAverageBitrate * bitrateMultiplier);
				keyFrame = frameStats->keyFrame;
				frameBytesAllocated = (int)(bitrateMultiplier * outputVideoAverageBytes);

				// Give the current frame up to 10% of the banks bitrate available (or debt as the case may be)
				bitrateUsed = (bitrateDeltaAccum/10)*8;
				bitrateUsed += outputVideoAverageBitrate;
				
				// Verify that bitrate is reasonable
				if(bitrateUsed<(settings->vbrMinBitrate*1024))	//300Kbps lowest
					bitrateUsed = settings->vbrMinBitrate*1024;

				if(bitrateUsed>(settings->vbrMaxBitrate*1024))	//6000Kbps Highest
					bitrateUsed = settings->vbrMaxBitrate*1024;
				}

			if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_VBRTIME))
				{
				if(!fread(frameStats,sizeof(FRAMESTATS),1,statsFile))
					{
					logPrintf("The .stats file ended early. Using the Average for this frame!!!\n");
					frameStats->frameSize = inputVideoAverageBytes;
					frameStats->keyFrame = false;
					}

				bitrateMultiplier =  (double)frameStats->frameSize / inputVideoAverageBytes;
				currentBps = (int)(outputVideoAverageBitrate * bitrateMultiplier);
				
				keyFrame = frameStats->keyFrame;
				frameBytesAllocated = (int)(bitrateMultiplier * outputVideoAverageBytes);
				
				// Give the current frame up to 10% of the banks bitrate available (or debt as the case may be)
				bitrateUsed = (bitrateDeltaAccum/10)*8;
				bitrateUsed += currentBps;
				
				// Verify that bitrate is reasonable
				if(bitrateUsed<(settings->vbrMinBitrate*1024))	//300Kbps lowest
					bitrateUsed = settings->vbrMinBitrate*1024;

				if(bitrateUsed>(settings->vbrMaxBitrate*1024))	//6000Kbps Highest
					bitrateUsed = settings->vbrMaxBitrate*1024;
				}

			if((encodeMode == ENCODE_MODE_HYBRID1) || (encodeMode == ENCODE_MODE_HYBRID2) ||
				(encodeMode == ENCODE_MODE_HYBRID2TIME) || (encodeMode == ENCODE_MODE_XVIDHYBRID))
				{

				// First compute the number of remaining frames
				EnterCriticalSection(&writeLock);
				tempCurPos = currentFilePos;
				tempFramesDone = framesDone;
				LeaveCriticalSection(&writeLock);
				
				// If we've done over 100 frames, then start recomputing the bitrate based on what we know
				if(tempFramesDone > 100)
					{
					// use the statisitcs
					bytesPerFrame = (int)(tempCurPos  / tempFramesDone);
					auxStats->scanNumFrames = (int)(totalFileLength / bytesPerFrame);
					}
				else
					{
					// Guess based on an average mpeg 2 data rate
					auxStats->scanNumFrames = (int)(((double)totalFileLength * 0.750f) * outputFramerate);
					}

				// Now calculate the average bitrate for all remaining frames 
				remainingFrames = auxStats->scanNumFrames - tempFramesDone;
				
				// To avaid divide by zero condition
				if(remainingFrames==0)
					remainingFrames=1;

				// Next Compress the current frame @ 6MBPS using the second encoder
				if((encodeMode == ENCODE_MODE_HYBRID2) ||
					encodeMode == ENCODE_MODE_HYBRID2TIME)
					{
					if(writeFrameFirstPass)
						{
						encodeVideo2->setDeltaFrame(0);
						writeFrameFirstPass = false;
						}
					else
						{
						encodeVideo2->setDeltaFrame(1);
						}
					encodeVideo2->setBitsRemaining(6000*1024*5);

					encodedFrame = encodeVideo2->encode(processedFrames[writeBufferTail],&encodeFlags);
					outHeader = encodeVideo2->getFormatOut();

					frameStats->frameSize = outHeader->biSizeImage;
					auxStats->scanBytes += frameStats->frameSize;
					
					if(tempFramesDone)
						{
						inputVideoAverageBytes = (int)(auxStats->scanBytes / tempFramesDone);
						bitrateMultiplier =  (double)frameStats->frameSize / inputVideoAverageBytes;
						}
					else
						{
						bitrateMultiplier = 1.0f;
						}
					}
				else
					{
					bitrateMultiplier = 1.0f;
					}

				remainingMovieSeconds = (int)(((double)remainingFrames) / outputFramerate);

				outputAudioBytesRequired = 0;
				if(settings->processAudio)
					{
					for(channel=0;channel<numAudioStreams;channel++)
						{
						if(settings->mp3)
							{
							if(settings->mp3EncodeMode[channel] == LAME_CBR || settings->mp3EncodeMode[channel] == LAME_ABR)
								{
								outputAudioBytesRequired += remainingMovieSeconds * ((settings->mp3MinBitrate[channel] * 1000)/8);
								}
							else
								{
								averageBr = (settings->mp3MinBitrate[channel] + settings->mp3MaxBitrate[channel]) / 2;
								outputAudioBytesRequired += remainingMovieSeconds * ((averageBr * 1000)/8);
								}
							}
						else
							{
							// Not using Lame, so assume it's a bitrate of 128Kbps
							outputAudioBytesRequired += remainingMovieSeconds * ((settings->vbrAudioBitrate * 1000) / 8); 			
							}
						}			
					}
				else
					{
					// Not using Lame, so assume it's a bitrate of 128Kbps
					outputAudioBytesRequired += remainingMovieSeconds * ((settings->vbrAudioBitrate * 1000) / 8); 			
					}
				
				aviOverhead = 24 * (auxStats->scanNumFrames - tempFramesDone);

				if(encodeMode == ENCODE_MODE_HYBRID2TIME)
					{
					if(tempFramesDone == 120)
						{
						logPrintf("Hybrid 2 Time - Remaining Movie Seconds %d\n",remainingMovieSeconds);
						maxMovieSize = remainingMovieSeconds;
						maxMovieSize *= ((settings->vbrQualityBitrate * 1024)/ 8);
						remainingBytes = maxMovieSize - writeAvi->getTotalBytesWritten();
						outputVideoAverageBytes = (int)((remainingBytes - (outputAudioBytesRequired+aviOverhead)) / remainingFrames);
						// And last we set the mode to Hybrid 2
						encodeMode = ENCODE_MODE_HYBRID2;
						}
					else
						{
						outputVideoAverageBytes = (int)(((settings->vbrQualityBitrate * 1024) / 8.0f) / outputFramerate);	
						}
					}	
				else
					{
					remainingBytes = maxMovieSize - writeAvi->getTotalBytesWritten();
					outputVideoAverageBytes = (int)((remainingBytes - (outputAudioBytesRequired+aviOverhead)) / remainingFrames);
					}

				currentBps = (int)(((double)outputVideoAverageBytes * (double)outputFramerate) * 8.0f);

				// Now we multiply the average bitrate by the amount this frame deserves.
		
				bitrateUsed = (int) (currentBps * bitrateMultiplier);
				

				if(settings->lumaProcessing)
					{
					luma = decodeVideo->computeLumananceValue(processedFrames[writeBufferTail]);

					// Low luma situations need to be compensated for
					if(settings->lumaLowProcessing)
						{
						if(luma<settings->lumaLowThresh)
							{
							boost = settings->lumaLowBoost / 100.0f;
							bitrateUsed += (int) (bitrateUsed * boost);
							}
						}

					//fast luma changes need to be compensated for 
					if(settings->lumaDeltaProcessing)
						{
						if(fabs(luma - previousLuma)>settings->lumaDeltaThresh)
							{
							boost = settings->lumaDeltaBoost / 100.0f;
							bitrateUsed += (int) (bitrateUsed * boost);
							}
						}
					previousLuma = luma;

					}

				// Verify that bitrate is reasonable
				if(bitrateUsed<(settings->vbrMinBitrate*1024))	//300Kbps lowest
					bitrateUsed = settings->vbrMinBitrate*1024;

				if(bitrateUsed>(settings->vbrMaxBitrate*1024))	//6000Kbps Highest
					bitrateUsed = settings->vbrMaxBitrate*1024;
				
				if(!keyFrameCounter--)
					{
					keyFrameCounter = settings->vbrMaxKeyFrame;
					keyFrame = true;
					}
				else
					{
					keyFrame = false;
					}
				}

			if(encodeMode == ENCODE_MODE_DIVXVBRPLUS)
				{
				encodeVideo1->setDivxNetworksBitrate(bitrateUsed);
				}

			if(encodeMode == ENCODE_MODE_XVIDHYBRID)
				{
				encodeVideo1->updateXvidBitrate(bitrateUsed);
				}

			if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_HYBRID1) || 
				(encodeMode == ENCODE_MODE_HYBRID2) || (encodeMode == ENCODE_MODE_HYBRID2TIME) ||
				(encodeMode == ENCODE_MODE_VBRTIME))
				{
				
				if(writeFrameCounter==0)
					keyFrame=true;

				// If the encode is Divx 3.11a, then create a keyframe at the chapter break
				if(settings->insertChapters)
					{
					if(settings->chapterStartFramePos[chapterIndex] <= processedFramesFramePos[writeBufferTail])
						{
						keyFrame=true;
						}		
					}

				encodeVideo1->setDeltaFrame(!keyFrame);
				encodeVideo1->setDataRate(bitrateUsed);

				bitsRem = encodeVideo1->getBitsRemaining();

				// If we're going to start dropping frames, then take action
				if(bitsRem< (int)((double)bitrateUsed * 5.0f * ((double)settings->vbrMinReservoir / 100.0f) ))
					{
					encodeVideo1->setBitsRemaining((int)((double)bitrateUsed * 5.0f * settings->vbrMinReservoir));
					}

				// If we're using too much bitrate, then set prevent over use.
				if(bitsRem > (int)((double)bitrateUsed * 5.0f * ((double)settings->vbrMaxReservoir / 100.0f)))
					{
					encodeVideo1->setBitsRemaining((int)((double)bitrateUsed * 5.0f * settings->vbrMaxReservoir));
					}
				}


			encodedFrame = encodeVideo1->encode(processedFrames[writeBufferTail],&encodeFlags);

			if(encodedFrame)
				{
				#ifndef VIDOMISLAVE
				globals->outputVideo->displayFrame(processedFrames[writeBufferTail]);
				#endif

				outHeader = encodeVideo1->getFormatOut();

				if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_VBRTIME) || (encodeMode == ENCODE_MODE_DIVXVBRPLUS))
					{
					bytesPredicted +=frameBytesAllocated;
					bytesOutput += outHeader->biSizeImage;
					
					bitrateDeltaAccum += frameBytesAllocated;
					bitrateDeltaAccum -= outHeader->biSizeImage;
					//logPrintf("Bytes Allocated : %d / Bytes Used : %d\n", frameBytesAllocated , outHeader->biSizeImage);
					}

				if((encodeMode == ENCODE_MODE_HYBRID1) || (encodeMode == ENCODE_MODE_HYBRID2) ||
					(encodeMode == ENCODE_MODE_HYBRID2TIME) || (encodeMode == ENCODE_MODE_XVIDHYBRID))
					{
					bytesOutput += outHeader->biSizeImage;
					}
			
				///////////////////////////
				// Keyframe Based File Split
				///////////////////////////
				if(settings->splitAtKeyframe && splitAvi && (encodeFlags&DIVX_KEYFRAME))
					{
					ret = writeAvi->split();
					if(ret==false)
						{
						char messTitle[4096];
						char messText[4096];
						globals->translate->translate(messText, "VidomiMessage2");
						globals->translate->translate(messTitle, "VidomiMessage2Title");
						vidomiErrorBox(globals->hWnd, messText,messTitle,MB_OK | MB_ICONERROR);
						return(false);
						}
					splitAvi = false;
					writeFrameCounter = 0;
					keyFrameCounter = 1;
					}

				if(settings->insertChapters)
					{
					// We only insert chapter starts on keyframes
					if(encodeFlags&DIVX_KEYFRAME)
						{
						// Make work on non divx 3.11a enocdes
						if(settings->chapterStartFramePos[chapterIndex] <= processedFramesFramePos[writeBufferTail])
							{
							writeAvi->insertChapterStart(settings->chapterName[chapterIndex]);
							logPrintf("Inserted Chapter %d at frame %d\n",chapterIndex,processedFramesFramePos[writeBufferTail]);
							chapterIndex++;
							logPrintf("NextChapter at frame %d\n",settings->chapterStartFramePos[chapterIndex]);
							}		
						}
					}

				ret = writeAvi->writeVideoData(encodedFrame,outHeader,encodeFlags,&splitAvi,!settings->splitAtKeyframe, 
												processedFramesCutChange[writeBufferTail]);
				if(ret==false)
					{
					char messTitle[4096];
					char messText[4096];
					globals->translate->translate(messText, "VidomiMessage2");
					globals->translate->translate(messTitle, "VidomiMessage2Title");

					vidomiErrorBox(globals->hWnd, messText,messTitle,MB_OK | MB_ICONERROR);
					return(false);
					}
				}			
			else
				{
				logPrintf("encodeVideo1->encode returned a NULL encodedFrame (Frame Being dropped by Codec)\n");
				}

			if(dualMode)
				{
				// Now free the buffer
				delete [] processedFrames[writeBufferTail]->allocStart;
				processedFrames[writeBufferTail]->allocStart = NULL;
				processedFrames[writeBufferTail]->bufferStart = NULL;
				processedFrames[writeBufferTail]->data = NULL;
				}	
			
			writeFrameCounter++;
			}


		//////////////////////////////
		// Subtitle Buffer Processing
		//////////////////////////////
		for(channel=0;channel<numSubtitleStreams;channel++)
			{
			EnterCriticalSection(&writeLock);
			readySubtitle = processedSubtitlesReady[channel][writeBufferTail];
			LeaveCriticalSection(&writeLock);
			if(readySubtitle)
				{
				if(subtitles->processSubtitleData(channel,settings, processedSubtitles[channel][writeBufferTail]))
					{
					// The subtitle data is valid and ready to be written to disk
					
					// Set the starting Frame to the saved frame counter
					processedSubtitles[channel][writeBufferTail]->header.startingFrame = savedFrameCounter;
					writeAvi->writeSubtitleData(channel,processedSubtitles[channel][writeBufferTail]);
					}

				// Do the processing and write out the final data
				if(dualMode)
					{
					// Now free the buffer
					delete [] processedSubtitles[channel][writeBufferTail]->bufferStart;
					processedSubtitles[channel][writeBufferTail]->bufferStart = NULL;
					processedSubtitles[channel][writeBufferTail]->data = NULL;
					}
				else
					{
					processedSubtitles[channel][writeBufferTail]->data = processedSubtitles[channel][writeBufferTail]->bufferStart;
					processedSubtitles[channel][writeBufferTail]->bytesUsed = 0;
					processedSubtitles[channel][writeBufferTail]->complete=0;
					}
				// And mark the structure as being available
				EnterCriticalSection(&writeLock);
				processedSubtitlesReady[channel][writeBufferTail] = false;
				LeaveCriticalSection(&writeLock);
				// Free the buffer we were handed
				}
			}

		////////////////////////////
		// Audio Buffer Processing 
		////////////////////////////

		for(channel=0;channel<numAudioStreams;channel++)
			{
			// Only allow split if this is the last audio channel
			if(channel+1 >= numAudioStreams)
				{
				performSplit = !settings->splitAtKeyframe;
				}
			else
				{
				performSplit = false;
				}

			EnterCriticalSection(&writeLock);
			readyAudio = processedSamplesReady[channel][writeBufferTail];
			
			LeaveCriticalSection(&writeLock);
			if(readyAudio)
				{
				// Do resample
				if(processedSamplesCutChange[channel][writeBufferTail])
					{
					audioPostProcess[channel]->resetAudioPts();
					}
				endingSamples = audioPostProcess[channel]->processSamples(processedSamples[channel][writeBufferTail]);
				if(endingSamples->bytesUsed)
					{
					audioFramesOutput =  writeAvi->writeAudioData(channel,endingSamples,processedFrames[writeBufferTail],&splitAvi,
																performSplit,processedSamplesCutChange[channel][writeBufferTail]);
					if(audioFramesOutput==false)
						{
						logPrintf("writeAvi->writeAudioData failed to write any frames\n");
						char messTitle[4096];
						char messText[4096];
						globals->translate->translate(messText, "VidomiMessage3");
						globals->translate->translate(messTitle, "VidomiMessage3Title");

						vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
						return(false);
						}
					else
						{
						bitrateDeltaAccum += -((audioFramesOutput-1) * 24);
						}
					endingSamples->bytesUsed = 0;
					endingSamples->data = endingSamples->bufferStart;
					}

				if(dualMode)
					{
					// Now free the buffer
					delete [] processedSamples[channel][writeBufferTail]->bufferStart;
					processedSamples[channel][writeBufferTail]->bufferStart = NULL;
					processedSamples[channel][writeBufferTail]->data = NULL;
					}
				else
					{
					processedSamples[channel][writeBufferTail]->data = processedSamples[channel][writeBufferTail]->bufferStart;
					processedSamples[channel][writeBufferTail]->bytesUsed = 0;
					}
				// And mark the structure as being available
				EnterCriticalSection(&writeLock);
				processedSamplesReady[channel][writeBufferTail] = false;
				LeaveCriticalSection(&writeLock);
				// Free the buffer we were handed
				}
			}

		////////////////////////////
		// Split AVI work to be done
		////////////////////////////

		if(splitAvi && !settings->splitAtKeyframe)
			{

			logPrintf("AVI was split in the audio write routines. Stopping and restarting the video codec\n");
			// If the AVI was split, then we need to close the video encode and restart it
			encodeVideo1->endEncoder();

			logPrintf("Ended encoder1\n");
			// Give windows Time to Catch up
			encodeVideo1->setVideoInputFormat(outputWidth,outputHeight,outputFramerate,colorSpace);
			logPrintf("Set Encoder 1's Video Input\n");
			
			if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_HYBRID1) || 
				(encodeMode == ENCODE_MODE_HYBRID2) || (encodeMode == ENCODE_MODE_HYBRID2TIME) || 
				(encodeMode == ENCODE_MODE_VBRTIME))
				{
				encodeVideo1->setVideoOutputFormatVbr(encoder1FourCC);
				}

			if((encodeMode == ENCODE_MODE_DIVXCBR) || (encodeMode == ENCODE_MODE_DIVXCBRQUALITY) ||
				(encodeMode == ENCODE_MODE_DIVXVBR) || (encodeMode == ENCODE_MODE_DIVXVBRPLUS))
				{
				encodeVideo1->setVideoOutputFormatDivxNetworks(videoFormatOut);
				encodeVideo1->setDivxNetworksLogFile(divxNetworksLogFilename);
				encodeVideo1->setDivxNetworksEncodeMode(encodeMode,1);
				if((encodeMode == ENCODE_MODE_DIVXVBR) || (encodeMode == ENCODE_MODE_DIVXVBRPLUS))
					{
					encodeVideo1->setDivxNetworksBitrate(outputVideoDivxNetworksAverageBitrate);
					}
				}
			
			if((encodeMode == ENCODE_MODE_XVIDVBR) || (encodeMode == ENCODE_MODE_XVIDHYBRID))
				{
				encodeVideo1->setVideoOutputFormatXvid(videoFormatOut);
				encodeVideo1->setXvidLogFile(xvidLogFilename);
				encodeVideo1->setXvidEncodeMode(encodeMode, 1);
				encodeVideo1->setXvidFilesize((outputVideoBytes-(outputVideoBytes/10)));
				//encodeVideo1->setXvidBitrate(outputVideoXvidAverageBitrate);
				}
			

			if((encodeMode == ENCODE_MODE_CBR) || (encodeMode == ENCODE_MODE_CBR2PASS))
				{
				encodeVideo1->setVideoOutputFormatCbr(videoFormatOut);
				}

			logPrintf("Set Encoder 1's Video Output\n");
			encodeVideo1->startEncoder(true);
			
			logPrintf("Restarted Encoder 1\n");
			if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_HYBRID1) || 
				(encodeMode == ENCODE_MODE_HYBRID2) || (encodeMode == ENCODE_MODE_HYBRID2TIME) ||
				(encodeMode == ENCODE_MODE_VBRTIME))
				{
				keyFrame = true;
				encodeVideo1->setCrispness(settings->vbrCrispness);
				// Set to low motion
				encodeVideo1->setCompressionMinMax(2,16);
				//if(settings->introAvi)
				//	{
				//	writeIntro(true);
				//	}
				}

			splitAvi = false;
			writeFrameCounter = 0;
			keyFrameCounter = 1;
			}


		// Mark that we processed this frame and the structure is available
		EnterCriticalSection(&writeLock);
		processedFramesReady[writeBufferTail] = false;
		LeaveCriticalSection(&writeLock);

		writeBufferTail++;
		if(writeBufferTail==maxWriteBuffers)
			writeBufferTail=0;
		}
	else
		{
		// If no frame is ready then suspend processing
		Sleep(0);
		}

	return(true);
}

// *****************************************************************************
// conversionClass::writeIntro()
//
// *****************************************************************************

void conversionClass::writeIntro(bool atSplit)
{
	avReadClass * introRead;
	FRAMEDATA * introFrame;
	AUDIODATA * introSamples;
	int introSamplesPerFrame;
	char introPath[MAX_PATH];
	char introRootPath[MAX_PATH];
	int introCount;
	FILE * introFile;
	FRAMEDATA * encodedFrame;
	DWORD encodeFlags;
	BITMAPINFOHEADER * outHeader;

	writeAviIntro = false;

	// Intro's only supported in HYBRID or VBR mode.
	if((encodeMode == ENCODE_MODE_CBR) || (encodeMode == ENCODE_MODE_CBR2PASS))
		{
		return;
		}

	for(introCount=1;introCount<10;introCount++)
		{	
		
		strcpy(introRootPath,settings->introAviPrimaryPath);
		sprintf(introPath,"%s\\intro%d.avi",introRootPath,introCount);
		introFile = fopen(introPath,"rb");
		if(!introFile)
			{
			strcpy(introRootPath,settings->introAviSecondaryPath);
			sprintf(introPath,"%s\\intro%d.avi",introRootPath,introCount);
			introFile = fopen(introPath,"rb");
			if(!introFile)
				return;
			}
		fclose(introFile);

		introRead = new readAviClass();

		if(introRead->init())
			{
			introRead->clearInput();
			introRead->addInput(introPath);
			introRead->setFilm(settings->film);
			introRead->setAudio(false,0,0,0,false);
			if(introRead->begin(colorSpace))
				{
				introSamples = new AUDIODATA [1];
				introSamplesPerFrame = (int)(((double)(outputSampleRate[0]*outputChannels[0])) / outputFramerate);
				introSamplesPerFrame*=2;
				introSamples->bufferStart = new unsigned char [introSamplesPerFrame];
				introSamples->bufferSize = introSamplesPerFrame;
				introSamples->data = introSamples->bufferStart;
				introSamples->bytesUsed = introSamplesPerFrame;
				introSamples->pts = 0;
				introSamples->type = audioBufferMpegDecoded;
				memset(introSamples->bufferStart,0,introSamplesPerFrame);

				// Force first frame to be a keyframe
				encodeVideo1->setDeltaFrame(false);
				encodeVideo1->setDataRate(1200*1024);
				encodeVideo1->setBitsRemaining(1200*1024*5);
				while (introRead->decodeFrame())
					{
					introFrame = introRead->getFrameData();
					encodedFrame = encodeVideo1->encode(introFrame,&encodeFlags);
					encodedFrame->pts = 0;
					
					#ifndef VIDOMISLAVE
					globals->outputVideo->displayFrame(introFrame);
					#endif

					outHeader = encodeVideo1->getFormatOut();
					writeAvi->writeVideoData(encodedFrame,outHeader,encodeFlags,&splitAvi,true,false);
					introSamples->data = introSamples->bufferStart;
					introSamples->bytesUsed = introSamplesPerFrame;
					writeAvi->writeAudioData(0,introSamples,encodedFrame,&splitAvi,true, false);
					encodeVideo1->setDeltaFrame(true);
					encodeVideo1->setDataRate(1200*1024);
					}
				introRead->end();
				delete [] introSamples->bufferStart;
				delete [] introSamples;
				}
			}
		
		delete introRead;
		writeAvi->resetPtsFlags(0,atSplit);
		startTime = timeGetTime(); 
		}

}

// *****************************************************************************
// conversionClass::writeSetup()
//
// Returns TRUE if success
// *****************************************************************************

bool conversionClass::writeSetup(void)
{
	int channel;
	int i;
	if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_VBRTIME) ||(encodeMode == ENCODE_MODE_DIVXVBRPLUS))
		{
		statsFile = fopen(statsFilename,"rb");
		if(!statsFile)
			{
			return(false);
			}
		}

	// If it's just the first pass, then don't setup the writer
	writeAvi->setVideoInputFormat(outputWidth,outputHeight,outputFramerate,settings->film);
	writeAvi->setVideoOutputFormat(encodeVideo1->getFccHandler());
	if(audio)
		{
		for(channel=0;channel<numAudioStreams;channel++)
			{
			logPrintf("Final Output Samplerate : %d\n",outputSampleRate[channel]);
			logPrintf("Final Output Channels : %d\n",outputChannels[channel]);
			writeAvi->setUseMp3(channel,settings->mp3,settings->mp3OutputFrequency[channel],
								settings->mp3EncodeMode[channel], settings->mp3ChannelType[channel],
								settings->mp3Quality[channel], settings->mp3MinBitrate[channel],
								settings->mp3MaxBitrate[channel],settings->mp3VbrQuality[channel]);
			writeAvi->setAudioOutputFormat(channel,outputSampleRate[channel],outputChannels[channel],audioFormatOut);
			writeAvi->setAudioInputFormat(channel,outputSampleRate[channel],outputChannels[channel]);
			}
		}

	//Create the AVI file
	writeAvi->createFile(outputFilename,&settings->outputTail[0],headerSize);

	//Create AVI video stream in AVI file 
	if(!writeAvi->createVideoStream(encodeVideo1->getFormatOut(),settings->videoStreamName))
		{
		logPrintf("writeAvi->createVideoStream failed\n");
		char messTitle[4096];
		char messText[4096];
		globals->translate->translate(messText, "VidomiMessage4");
		globals->translate->translate(messTitle, "VidomiMessage4Title");

		vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
		writeAvi->closeVideoStream();
		writeAvi->closeFile();
		return(false);	
		}
	if(audio)
		{
		for(channel=0;channel<numAudioStreams;channel++)
			{
			if(!writeAvi->createAudioStream(channel,settings->audioStreamName[channel]))
				{
				logPrintf("writeAvi->createAudioStream failed\n");
				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage5");
				globals->translate->translate(messTitle, "VidomiMessage5Title");

				vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
				writeAvi->closeVideoStream();
				for(i=0;i<channel;i++)
					{
					writeAvi->closeAudioStream(i);
					}
				writeAvi->closeFile();
				return(false);
				}
			}
		}

	if(settings->processSubtitles)
		{
		for(channel=0;channel<numSubtitleStreams;channel++)
			{
			if(!writeAvi->createSubtitleStream(channel,settings->subtitleStreamName[channel]))
				{
				logPrintf("writeAvi->createSubtitleStream failed\n");
				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage6");
				globals->translate->translate(messTitle, "VidomiMessage6Title");

				vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
				writeAvi->closeVideoStream();
				
				for(i=0;i<numAudioStreams;i++)
					{
					writeAvi->closeAudioStream(i);
					}

				for(i=0;i<channel;i++)
					{
					writeAvi->closeSubtitleStream(i);
					}
				writeAvi->closeFile();
				return(false);
				}
			}
		}

	logPrintf("AVIWriter Fully setup\n");

	keyFrameCounter = 0;
	previousLuma = 0.50f;
	writeFirstPass = true;
	writeFrameFirstPass = true;
	writeFrameCounter=0;
	bitrateDeltaAccum = 0;	
	bytesPredicted = 0;
	bytesOutput=0;
	compressionLevel =2;
	return(true);
}


// *****************************************************************************
// conversionClass::writeCleanup()
//
// Returns TRUE if success
// *****************************************************************************

void conversionClass::writeCleanup(void)
{
	int channel;

	if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_VBRTIME) || (encodeMode == ENCODE_MODE_DIVXVBRPLUS))
		{
		if(statsFile)
			{
			fclose(statsFile);
			}
		}
	writeAvi->closeVideoStream();

	if(audio)
		{
		for(channel=0;channel<numAudioStreams;channel++)
			{
			writeAvi->closeAudioStream(channel);
			}
		}

	if(settings->processSubtitles)
		{
		for(channel=0;channel<numSubtitleStreams;channel++)
			{
			writeAvi->closeSubtitleStream(channel);
			}
		}

	writeAvi->closeFile();
}

// *****************************************************************************
// conversionClass::prepareSetup()
//
// Returns TRUE if success
// *****************************************************************************

bool conversionClass::prepareSetup(int passNumber)
{
	int fileNum;
	char * tempPtr;
	FILE * tempFile;
	int channelNum;
	int channel;
	char tempFilename[MAX_PATH];


	if(inputInfo->audioFound && ((passNumber==3) || normalizeAudio || usingVbrAudio) && passNumber!=1)
		{
		audio = settings->processAudio;
		if(audio)
			{
			channelNum =0;
			for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
				{
				if(settings->processAudioSelected[channel]!=-1)
					{
					audioTrack[channel] = inputInfo->audioNumber[settings->processAudioSelected[channel]];
					audioType[channel] = inputInfo->audioType[settings->processAudioSelected[channel]];
					audioSampleRate[channel] = inputInfo->audioRate[settings->processAudioSelected[channel]];
					audioChannels[channel] = inputInfo->audioChannels[settings->processAudioSelected[channel]];
					audioDirectStreamCopy[channel] = settings->audioDirectStreamCopy[channel];
					channelNum++;
					}
				else
					{
					break;
					}
				}
			numAudioStreams = channelNum;
			}
		else
			{	
			numAudioStreams = 0;
			}
		}
	else
		{
		audio = false;
		for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
			{
			audioTrack[channel] = 0;
			audioType[channel] = 0;
			audioSampleRate[channel] = 0;
			audioChannels[channel] = 0;
			audioDirectStreamCopy[channel] = false;
			}
		numAudioStreams = 0;
		}

	maxOutputFileSize = settings->maxOutputSize;
	if(maxOutputFileSize>3980)
		maxOutputFileSize=3980;

	if(maxOutputFileSize<1)
		maxOutputFileSize=1;

	audioFormatOut = settings->audioFormatOut;
	videoFormatOut = settings->videoFormatOut;
	videoFormatOutP2 = settings->videoFormatOutP2;

	// We're now ready to start the encode process


	strcpy(tempFilename,outputFilename);
	strcat(tempFilename,&settings->outputTail[0]);
	tempFile = fopen(tempFilename,"r");
	if(tempFile)
		{
		fclose(tempFile);
		if(!settings->overwriteExistingFiles)
			{
			char messTempText[4096];
			char messTitle[4096];
			char messText[4096];
			globals->translate->translate(messTempText, "VidomiMessage35");
			globals->translate->translate(messTitle, "VidomiMessage35Title");

			sprintf(messText,messTempText,tempFilename);

			// Vidomi slave always overwrites
			#ifndef VIDOMISLAVE
			if(vidomiErrorBox(globals->hWnd,messText,messTitle,MB_YESNO | MB_ICONWARNING)==IDNO)
				{
				return(false);	
				}
			#endif
			}
		DeleteFile(tempFilename);
		}

	postProcess = new postProcessClass();
	if(!postProcess->init())
		{
		settings->scale = false;
		settings->deinterlace = false;
		settings->cropTop = 0;
		settings->cropBottom = 0;
		settings->cropLeft = 0;
		settings->cropRight = 0;
		}
	logPrintf("Video Post Process Class created\n");

	for(channel=0;channel<numAudioStreams;channel++)
		{
		audioPostProcess[channel] = new audioPostProcessClass();
		if(!audioPostProcess[channel]->init())
			{
			settings->resample48KhzAudio = false;
			}
		logPrintf("Audio Post Process Class[%d] created\n",channel);
		}

	// Check file extension, then call apropriate routine
	tempPtr = strrchr(settings->inputFilenames[0],'.');
	if(!_strnicmp(tempPtr+1,"avi",3))
		{
		aviConversion = true;
		avRead = new readAviClass();
		}
	else
		{
		aviConversion=false;
		avRead = new readMpegClass();	
		}

	if(!avRead->init())
		{
		delete postProcess;
		for(channel=0;channel<numAudioStreams;channel++)
			{
			delete audioPostProcess[channel];
			}
		avRead->end();
		delete avRead;
		return(false);
		}
	logPrintf("avRead Class Created\n");


	writeAvi = new writeAviClass();
	if(!writeAvi->init())
		{
		delete postProcess;
		for(channel=0;channel<numAudioStreams;channel++)
			{
			delete audioPostProcess[channel];
			}
		avRead->end();
		delete avRead;
		delete writeAvi;
		return(false);
		}
	logPrintf("writeAvi Class Created\n");

	encodeVideo1 = new encodeVideoClass();
	if(!encodeVideo1->init(settings))
		{
		delete postProcess;
		for(channel=0;channel<numAudioStreams;channel++)
			{
			delete audioPostProcess[channel];
			}
		avRead->end();
		delete avRead;
		delete writeAvi;
		delete encodeVideo1;
		return(false);
		}
	logPrintf("encodeVideo1 Class Created\n");

	encodeVideo2 = new encodeVideoClass();
	if(!encodeVideo2->init(settings))
		{
		delete postProcess;
		for(channel=0;channel<numAudioStreams;channel++)
			{
			delete audioPostProcess[channel];
			}
		avRead->end();
		delete avRead;
		delete writeAvi;
		delete encodeVideo1;
		delete encodeVideo2;
		return(false);
		}
	logPrintf("encodeVideo2 Class Created\n");


	decodeVideo = new decodeVideoClass();
	if(!decodeVideo->init())
		{
		delete postProcess;
		for(channel=0;channel<numAudioStreams;channel++)
			{
			delete audioPostProcess[channel];
			}
		avRead->end();
		delete avRead;
		delete writeAvi;
		delete encodeVideo1;
		delete encodeVideo2;
		delete decodeVideo;
		return(false);
		}
	logPrintf("decodeVideo Class Created\n");

	subtitles = new subtitlesClass();
	logPrintf("subtitles Class Created\n");

	decodeVideo->detectSceneChangeInit(settings->vbrMinKeyFrame,settings->vbrMaxKeyFrame);

	// First tell the mpeg decoder the files we are operating on
	avRead->clearInput();
	for(fileNum=0;fileNum<settings->inputFilenameIndex;fileNum++)
		avRead->addInput(settings->inputFilenames[fileNum]);
	//Note that we use the default idct setting with mpeg files and avi doesn't use such a setting currently.
	avRead->setFilm(settings->film);

	// then tell the decoder we're ready to begin
	if(!avRead->begin(colorSpace))
		{
		logPrintf("avRead->begin failed");
		char messTitle[4096];
		char messText[4096];
		globals->translate->translate(messText, "VidomiMessage7");
		globals->translate->translate(messTitle, "VidomiMessage7Title");

		vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
		delete postProcess;
		for(channel=0;channel<numAudioStreams;channel++)
			{
			delete audioPostProcess[channel];
			}
		avRead->end();
		delete avRead;
		delete writeAvi;
		delete encodeVideo1;
		delete encodeVideo2;
		delete decodeVideo;
		return (false);
		}
	
	// see if we failed on the open of the mpeg file. 
	if(avRead->checkEnd())
		{
		logPrintf("avRead->checkEnd failed");
		char messTitle[4096];
		char messText[4096];
		globals->translate->translate(messText, "VidomiMessage8");
		globals->translate->translate(messTitle, "VidomiMessage8Title");

		vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
		delete postProcess;
		for(channel=0;channel<numAudioStreams;channel++)
			{
			delete audioPostProcess[channel];
			}
		avRead->end();
		delete avRead;
		delete writeAvi;
		delete encodeVideo1;
		delete encodeVideo2;
		delete decodeVideo;
		return (false);
		}

	// Audio Streams
	for(channel=0;channel<numAudioStreams;channel++)
		{
		avRead->setAudio(audio,channel,audioType[channel],audioTrack[channel],audioDirectStreamCopy[channel]);
		}

	// Subtitle Streams
	if(passNumber>=2)
		{
		for(channel=0;channel<numSubtitleStreams;channel++)
			{
			if(settings->processSubtitleMode[channel] == SUBTITLE_MODE_GRAPHIC)
				{
				avRead->setSubtitle(settings->processSubtitles,channel,settings->processSubtitleSelected[channel]);
				}
			}
		}
	else
		{
		avRead->setSubtitle(false,0,0);
		}

	// get segmented file lenght
	totalFileLength=getSegmentedFileLength(settings);

	if(passNumber==1)
		{
		avRead->seekPos(settings->startingFilePos + (totalFileLength/3));
		}
	else
		{
		avRead->seekPos(settings->startingFilePos);
		}
	frameStats = new FRAMESTATS [1];
	if(currentPass==1)
		{
		passTime = startTime; 
		}
	else
		{
		passTime = timeGetTime();
		}
	endTime = passTime; 
	framesDone = 0;

	return(true);
}

// *****************************************************************************
// conversionClass::conversionFrameWork()
//
// Returns TRUE if success
// *****************************************************************************

bool conversionClass::prepareFrameWork(bool firstPass, int passNumber)
{
	int	inputWidth;
	int	inputHeight;
	double inputAspectRatio;
	int aviOverhead;
	double percent;
	int remTime;
	int elapsedMs;
	__int64 tempPos;
	double outputFactor;
	double finalAspectRatio;
	int channel;
	int ret;
	unsigned long tempSize;

	if(firstPass)
		{
		//fetch the mpeg frame size
		avRead->getDimentions(&inputWidth,&inputHeight,&inputAspectRatio);

		// Framerate doesn't get modified, so the input and output are the same.
		inputFramerate = avRead->getFramerate();
		outputFramerate = inputFramerate;

		if(settings->film)
			{
			// If film mode is set, the input framerate is a lie. Adjust for the lie
			inputFramerate = inputFramerate *= 1.25;
			}

		//set PostProcess options
		for(channel=0;channel<numAudioStreams;channel++)
			{
			audioPostProcess[channel]->setResample(settings->resample48KhzAudio);
			if(passNumber==3 && normalizeAudio)
				{
				audioPostProcess[channel]->setNormalize(normalizeAudio,settings->normalizeLevel, auxStats->scanNormalizeMin[channel], auxStats->scanNormalizeMax[channel]);
				}
			else
				{
				audioPostProcess[channel]->setNormalize(false,0, 0, 0);
				}
			audioPostProcess[channel]->setStreamCopy(audioDirectStreamCopy[channel]);
			audioPostProcess[channel]->begin(audioSampleRate[channel],audioChannels[channel]);
			audioPostProcess[channel]->getOutputRate(&outputSampleRate[channel],&outputChannels[channel]);
			}

		postProcess->setCrop(settings->cropTop,settings->cropBottom,settings->cropLeft,settings->cropRight);
		postProcess->setForceMultiple16(settings->forceMultiple16);
		postProcess->setDeinterlace(settings->deinterlace,settings->blend,settings->blendThreshold);
		postProcess->setIvtc(settings->ivtc,settings->ivtcDeComb,settings->ivtcOffset,settings->ivtcPolarity);

		if(passNumber==3 || settings->scanPassScale)
			{
			finalAspectRatio = inputAspectRatio;
			if(settings->aspectRatioOverride)
				{
				finalAspectRatio = settings->aspectRatioOverrideValue;
				}
			postProcess->setHyperScale(settings->hyperScale,finalAspectRatio, settings->aspectRatioOverride);
			postProcess->setOutputDimentions(settings->scale, settings->scaleType,settings->scaleBy, settings->scaleWidth,settings->scaleHeight);
			}
		else
			{
			postProcess->setHyperScale(false,inputInfo->videoAspectRatio, false);
			postProcess->setOutputDimentions(false, settings->scaleType,settings->scaleBy, settings->scaleWidth,settings->scaleHeight);
			}

		// Then init postprocess by passing in the input frame width / height
		if(!postProcess->begin(inputWidth,inputHeight,outputFramerate,colorSpace, dualMode,settings))
			{
			logPrintf("postProcess->begin Failed to Initialize Properly.\n");
			return(false);
			}
		outputFramerate = postProcess->getOutputFramerate();

		// Now Fetch the final resulting frame output dimentions
		postProcess->getOutputDimentions(&outputWidth,&outputHeight);
		logPrintf("Final Output Width : %d\n",outputWidth);
		logPrintf("Final Output Height : %d\n",outputHeight);
		logPrintf("Final Output Framerate : %f\n",outputFramerate);

		tempSize =maxOutputFileSize;
		tempSize*=1024*1024;

		writeAvi->setOutputSize(tempSize);
		// Reset the video input now that we know for certain what the user wants done

		//Decode using the fourCC that encoder 2 uses. (decode is tied to encode2)
		//decodeVideo->setVideoInputFormat(encodeVideo2->getFormatOut());
		//decodeVideo->setVideoOutputFormat(outputWidth,outputHeight);
		//decodeVideo->startDecoder();

		// Init the subtitle process
		subtitles->init(inputFramerate, outputFramerate);		

		// Configure each channels mode
		if(passNumber>=2)
			{
			if(settings->processSubtitles)
				{
				for(channel=0;channel<numSubtitleStreams;channel++)
					{
					subtitles->channelConfigure(channel,settings->processSubtitleMode[channel], settings->processSubtitleOnlyEven[channel],settings->processSubtitleTextFile[channel]);
					}
				}
			}

		//Set input frame size
		#ifndef VIDOMISLAVE
		globals->outputVideo->setVideoInputFormat(outputWidth,outputHeight,settings->encodeColorSpace);
		//set output window size
		globals->outputVideo->setOutputWindowSize(outputWidth,outputHeight,0,0);
		globals->outputVideo->setResize(false);
		//display the output window
		if(displayOutputWindow)
			{
			// Output windows are only displayed the very first time
			// And are only available on the master computer
			if(settings->showOutput)
				{
				globals->outputVideo->showOutputWindow();
				}

			displayOutputWindow = false;
			}
		#endif
		
		//Create the necessary vars for VBR to work
		outputVideoDivxNetworksAverageBitrate = 6000000;
		outputVideoXvidAverageBitrate = 6000000;

		if(passNumber==3 || ((encodeMode == ENCODE_MODE_DIVXVBR) && passNumber==2))
			{
			if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_DIVXVBR) ||
				(encodeMode == ENCODE_MODE_VBRTIME) || (encodeMode == ENCODE_MODE_DIVXVBRPLUS) ||
				(encodeMode == ENCODE_MODE_XVIDVBR))
				{

				//Determine the number of seconds long the input movie is
				inputMovieSeconds = (int)((double)auxStats->scanNumFrames / outputFramerate); 

				outputAudioBytesRequired = 0;
				if(settings->processAudio)
					{
					for(channel=0;channel<numAudioStreams;channel++)
						{
						if(settings->mp3)
							{
							if(settings->mp3EncodeMode[channel] == LAME_CBR || settings->mp3EncodeMode[channel] == LAME_ABR)
								{
								outputAudioBytesRequired += inputMovieSeconds * ((settings->mp3MinBitrate[channel] * 1000)/8);
								}
							else
								{
								outputAudioBytesRequired += auxStats->scanVbrAudioBytesUsed[channel];
								}

							}
						else
							{
							// Not using Lame, so assume it's a bitrate of 128Kbps
							outputAudioBytesRequired += inputMovieSeconds * ((settings->vbrAudioBitrate * 1000) / 8); 			
							}
						}
					}
				else
					{
					outputAudioBytesRequired += inputMovieSeconds * ((settings->vbrAudioBitrate * 1000) / 8);
					}
				//Compute AVI file overhead for the video


				aviOverhead = headerSize + 0x98; //overhead for any AVI
				aviOverhead += 24 * auxStats->scanNumFrames;

				if(settings->useSpecifiedTarget)
					{
					maxMovieSize = 1024 * 1024;
					maxMovieSize *= settings->specifiedTarget;
					maxMovieSize -= settings->vbrReserve * (1024*1024);
					}
				else
					{
					// Max movie size (less one MB for safety)
					if(encodeMode == ENCODE_MODE_VBRTIME)
						{
						logPrintf("VBR Time - Input Movie Seconds %d\n",inputMovieSeconds);
						maxMovieSize = inputMovieSeconds;
						maxMovieSize *= ((settings->vbrQualityBitrate * 1024)/ 8);
						}
					else
						{
						maxMovieSize = (settings->vbrNumberOfDiscs * settings->maxOutputSize) - settings->vbrReserve;
						maxMovieSize *=(1024*1024);
						}
					}

				logPrintf("Target Movie Size %I64d\n",maxMovieSize);

				outputVideoBytes = maxMovieSize - (outputAudioBytesRequired + aviOverhead + auxStats->scanSubtitleBytes);  

				logPrintf("Target Video Stream Size %I64d\n",outputVideoBytes);

				if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_VBRTIME) || (encodeMode == ENCODE_MODE_DIVXVBRPLUS))
					{
					outputFactor = (double)outputVideoBytes / (double)auxStats->scanBytes;

					inputVideoAverageBytes = (int)(auxStats->scanBytes / auxStats->scanNumFrames);
					inputVideoAverageBitrate = (int)(((double)inputVideoAverageBytes * (double)outputFramerate) * 8.0f);

					outputVideoAverageBytes = (int)((double)inputVideoAverageBytes * outputFactor);

					outputVideoAverageBitrate = (int)((double)inputVideoAverageBitrate * outputFactor);
					outputVideoDivxNetworksAverageBitrate = outputVideoAverageBitrate;
					}
				else
					{
					outputVideoDivxNetworksAverageBitrate = (int)(outputVideoBytes / inputMovieSeconds);
					outputVideoDivxNetworksAverageBitrate *=8;
					outputVideoXvidAverageBitrate = (int)(outputVideoBytes / inputMovieSeconds);
					outputVideoXvidAverageBitrate *=8;
					}
				
				}

			if((encodeMode == ENCODE_MODE_HYBRID1) || (encodeMode == ENCODE_MODE_HYBRID2) ||
				(encodeMode == ENCODE_MODE_HYBRID2TIME)|| (encodeMode == ENCODE_MODE_XVIDHYBRID))
				{
				// Max movie size (less one MB for safety)
				if(settings->useSpecifiedTarget)
					{
					maxMovieSize = 1024 * 1024;
					maxMovieSize *= settings->specifiedTarget;
					maxMovieSize -= settings->vbrReserve * (1024*1024);
					}
				else
					{
					maxMovieSize = (settings->vbrNumberOfDiscs * settings->maxOutputSize) - settings->vbrReserve;
					maxMovieSize *=(1024*1024);
					}

				auxStats->scanBytes = 0;
				}
			}

		if((encodeMode == ENCODE_MODE_DIVXCBR) || (encodeMode == ENCODE_MODE_DIVXCBRQUALITY) || 
			(encodeMode == ENCODE_MODE_DIVXVBR) || (encodeMode == ENCODE_MODE_DIVXVBRPLUS))
			{
			//Divx Codec Setup

			encodeVideo1->setVideoInputFormat(outputWidth,outputHeight,outputFramerate,colorSpace);
			encodeVideo1->setVideoOutputFormatDivxNetworks(videoFormatOut);
			encodeVideo1->setDivxNetworksLogFile(divxNetworksLogFilename);
			if(passNumber==3)
				{
				encodeVideo1->setDivxNetworksEncodeMode(encodeMode,1);		
				}
			else
				{
				encodeVideo1->setDivxNetworksEncodeMode(encodeMode,0);		
				}

			if((encodeMode == ENCODE_MODE_DIVXVBR)||(encodeMode == ENCODE_MODE_DIVXVBRPLUS))
				{
				logPrintf("DivxNetworks Bitrate set to %d\n",outputVideoDivxNetworksAverageBitrate);
				encodeVideo1->setDivxNetworksBitrate(outputVideoDivxNetworksAverageBitrate);
				}

			if(!encodeVideo1->startEncoder(true))
				{
				logPrintf("encodeVideo1->startEncoder() Failed to Initialize Properly.\n");
				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage9");
				globals->translate->translate(messTitle, "VidomiMessage9Title");

				vidomiErrorBox(NULL,messText,messTitle,MB_OK);
				return(false);
				}
			}

		if(encodeMode == ENCODE_MODE_XVIDVBR)				//XVID Changes here
			{
			//Xvid Codec Setup

			encodeVideo1->setVideoInputFormat(outputWidth,outputHeight,outputFramerate,colorSpace);
			encodeVideo1->setVideoOutputFormatXvid(videoFormatOut);
			encodeVideo1->setXvidLogFile(xvidLogFilename);

			if(passNumber == 3)
				{
				encodeVideo1->setXvidEncodeMode(encodeMode,1);		
				encodeVideo1->setXvidFilesize(outputVideoBytes);
				}
			else
				{
				encodeVideo1->setXvidEncodeMode(encodeMode,0);	
				// This is ignored on all passes except the last
				encodeVideo1->setXvidFilesize(0);
				}

			
			if(!encodeVideo1->startEncoder(true))
				{
				logPrintf("encodeVideo1->startEncoder() Failed to Initialize Properly.\n");
				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage9");
				globals->translate->translate(messTitle, "VidomiMessage9Title");

				vidomiErrorBox(NULL,messText,messTitle,MB_OK);
				return(false);
				}
			}

		if(encodeMode == ENCODE_MODE_XVIDHYBRID)				//XVID Changes here
			{
			//Xvid Codec Setup

			encodeVideo1->setVideoInputFormat(outputWidth,outputHeight,outputFramerate,colorSpace);
			encodeVideo1->setVideoOutputFormatXvid(videoFormatOut);
			encodeVideo1->setXvidLogFile(xvidLogFilename);

			encodeVideo1->setXvidEncodeMode(encodeMode,0);		
			encodeVideo1->setXvidBitrate(900000);		
			
			if(!encodeVideo1->startEncoder(true))
				{
				logPrintf("encodeVideo1->startEncoder() Failed to Initialize Properly.\n");
				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage9");
				globals->translate->translate(messTitle, "VidomiMessage9Title");

				vidomiErrorBox(NULL,messText,messTitle,MB_OK);
				return(false);
				}
			}


		if(encodeMode == ENCODE_MODE_CBR)
			{
			//CBR Setup
			encodeVideo1->setVideoInputFormat(outputWidth,outputHeight,outputFramerate,colorSpace);
			encodeVideo1->setVideoOutputFormatCbr(videoFormatOut);
			if(!encodeVideo1->startEncoder(true))
				{
				logPrintf("encodeVideo1->startEncoder() Failed to Initialize Properly.\n");
				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage10");
				globals->translate->translate(messTitle, "VidomiMessage10Title");

				vidomiErrorBox(NULL,messText,messTitle,MB_OK);
				return(false);
				}
			}

		if(encodeMode == ENCODE_MODE_CBR2PASS)
			{
			//CBR Setup
			encodeVideo1->setVideoInputFormat(outputWidth,outputHeight,outputFramerate,colorSpace);
	
			// On the last pass we switch to the second codec setting
			if(passNumber == 3)
				{
				videoFormatOut = videoFormatOutP2;
				}
			encodeVideo1->setVideoOutputFormatCbr(videoFormatOut);
			if(!encodeVideo1->startEncoder(true))
				{
				logPrintf("encodeVideo1->startEncoder() Failed to Initialize Properly.\n");
				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage10");
				globals->translate->translate(messTitle, "VidomiMessage10Title");

				vidomiErrorBox(NULL,messText,messTitle,MB_OK);
				return(false);
				}
			}

		if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_HYBRID1) || 
			(encodeMode == ENCODE_MODE_HYBRID2) || (encodeMode == ENCODE_MODE_HYBRID2TIME) ||
			(encodeMode == ENCODE_MODE_VBRTIME))
			{
			encoder1FourCC = mmioFOURCC('d', 'i', 'v', '3');
			encoder2FourCC = mmioFOURCC('d', 'i', 'v', '4');

			// Start Encoder 1
			encodeVideo1->setVideoInputFormat(outputWidth,outputHeight,outputFramerate,colorSpace);
			encodeVideo1->setVideoOutputFormatVbr(encoder1FourCC);
			ret = encodeVideo1->startEncoder(true); 
			if(!ret)
				{
				logPrintf("encodeVideo1->startEncoder() Failed to Initialize Properly.\n");
				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage11");
				globals->translate->translate(messTitle, "VidomiMessage11Title");

				vidomiErrorBox(NULL,messText,messTitle,MB_OK);
				return(false);
				}

			// Start Encoder 2
			if((encodeMode == ENCODE_MODE_HYBRID2) || (encodeMode == ENCODE_MODE_HYBRID2TIME))
				{
				encodeVideo2->setVideoInputFormat(outputWidth,outputHeight,outputFramerate,colorSpace);
				encodeVideo2->setVideoOutputFormatVbr(encoder2FourCC);
				if(!encodeVideo2->startEncoder(true))
					{
					logPrintf("encodeVideo2->startEncoder() Failed to Initialize Properly.\n");
					char messTitle[4096];
					char messText[4096];
					globals->translate->translate(messText, "VidomiMessage12");
					globals->translate->translate(messTitle, "VidomiMessage12Title");

					vidomiErrorBox(NULL,messText,messTitle,MB_OK);
					return(false);
					}
				}
			}

		EnterCriticalSection(&writeLock);
		currentFilePos = 0;
		LeaveCriticalSection(&writeLock);

		}
	else
		{
		endTime = timeGetTime();
		elapsedMs = endTime - passTime;
		cumMs = (double) elapsedMs;
		cumMs /= 1000;

		conversionStats->framesPerSecond = (float)(framesDone / cumMs);
		conversionStats->passNumber = currentPass;
		conversionStats->totalPasses = totalPasses;

		if(passNumber==1)
			{
			percent = (100.0f*((double)(framesDone) / (double)(PRESCAN_FRAMES)));
			}
		else
			{
			tempPos = avRead->getCurPos();
			if(tempPos>settings->endingFilePos)
				{
				tempPos = settings->endingFilePos;
				}
			tempPos -= settings->startingFilePos;
			if(!settings->rangelist[0].valid)
				{
				percent = (100.0f*((double)(tempPos) / (double)(totalFileLength)));
				}
			else
				{
				// range files
				xtempPos+=tempPos-lasttpos;
				lasttpos=tempPos;
				percent = (100.0f*((double)(xtempPos) / (double)(totalFileLength)));
				}
			}
		remTime = (int)(((double)elapsedMs / percent)*100.0f) - elapsedMs;

		conversionStats->percentOfPassComplete = (float) percent;
		conversionStats->estimatedRemainingPassTime = remTime/1000;
		conversionStats->elapsedPassTime = elapsedMs/1000;
		conversionStats->elapsedJobTime = (endTime - startTime)/1000;

		conversionStats->rangeNumber = rangeIndex+1;
		conversionStats->totalRanges = totalRanges;

		EnterCriticalSection(&writeLock);
		currentFilePos = tempPos;
		framesDone++;
		LeaveCriticalSection(&writeLock);
		/*
		if((globals->outputStats->windowVisible() == false) || IsIconic(globals->hWnd))
			{
			sprintf(tempStr,"%.2f%% of Pass %d Complete - Vidomi Encoder",percent, (passNumber-1));
			//globals->outputVideo->setOutputWindowText(tempStr);
		
			SetWindowText(globals->hWnd, tempStr);
			}
		*/
		}

	return(true);

}

// *****************************************************************************
// conversionClass::conversionCleanup()
//
// Returns TRUE if success
// *****************************************************************************

void conversionClass::prepareCleanup(bool firstPass, int passNumber)
{
	int channel;

	// Close the AVI file
	Sleep(1000);
	if(!firstPass)
		{
		//decodeVideo->endDecoder();
		logPrintf("Calling encodeVideo1->endEncoder()\n");
		encodeVideo1->endEncoder();

		if((encodeMode == ENCODE_MODE_HYBRID2) || (encodeMode == ENCODE_MODE_HYBRID2TIME))
			{
			logPrintf("Calling encodeVideo2->endEncoder()\n");
			encodeVideo2->endEncoder();
			}

		if(passNumber==3)
			{
			// Set the players file to be the freshly encoded file
			logPrintf("posting the new filename from encode\n");

			#ifndef VIDOMISLAVE
			strcpy(globals->playFilename,outputFilename);
			strcat(globals->playFilename,&settings->outputTail[0]);
			#endif

			}
		}

	logPrintf("Deleting writeAvi\n");
	delete writeAvi;
	logPrintf("Deleting encodeVideo1\n");
	delete encodeVideo1;
	if((encodeMode == ENCODE_MODE_HYBRID2) || (encodeMode == ENCODE_MODE_HYBRID2TIME))
		{
		logPrintf("Deleting encodeVideo2\n");
		delete encodeVideo2;
		}
	logPrintf("Deleting decodeVideo\n");
	delete decodeVideo;
	logPrintf("Deleting postProcess\n");
	delete postProcess;

	logPrintf("Deleting subtitles\n");
	subtitles->end();
	delete subtitles;

	for(channel=0;channel<numAudioStreams;channel++)
		{
		logPrintf("Deleting audioPostProcess\n");
		delete audioPostProcess[channel];
		}
	logPrintf("Calling avRead->end()\n");
	avRead->end();
	logPrintf("Deleting avRead\n");
	delete avRead;
	logPrintf("Deleting frameStats\n");
	delete [] frameStats;

	/*
	SetWindowText(globals->hWnd, "Vidomi Encoder");
	*/
}

// *****************************************************************************
// conversionClass::scanSetup()
//
// Returns TRUE if success
// *****************************************************************************

bool conversionClass::scanSetup(void)
{
	unsigned long mp3BufferSize;
	unsigned long wavBufferSize;
	int channel;

	if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_HYBRID1) ||
		(encodeMode == ENCODE_MODE_HYBRID2) || (encodeMode == ENCODE_MODE_HYBRID2TIME) ||
		(encodeMode == ENCODE_MODE_VBRTIME) || (encodeMode == ENCODE_MODE_DIVXVBRPLUS)) 
		{
		statsFile = fopen(statsFilename,"wb");
		if(!statsFile)
			return(false);
		}

	auxStats->scanBytes = 0;
	auxStats->scanDeltaBytes = 0;
	auxStats->scanKeyBytes = 0;
	auxStats->scanSubtitleBytes = 0;
	
	auxStats->scanNumFrames = 0;
	auxStats->scanNumKeyFrames = 0;
	auxStats->scanNumDeltaFrames = 0;

	auxStats->scanDataRate = 6000*1024;

	auxStats->scanSubtitleBytes = 0;

	for(channel=0;channel<numAudioStreams;channel++)
		{
		auxStats->scanVbrAudioBytesUsed[channel] = 0;
		auxStats->scanNormalizeMin[channel] = 0;
		auxStats->scanNormalizeMax[channel] = 0;
		}

	// If using lame, setup the lame encode for audio VBR encode during the scan pass
	// NOTE: If this code grows at all, it should be moved into it's own file. 

	if(settings->mp3)
		{
		for(channel=0;channel<numAudioStreams;channel++)
			{
			mp3Encode[channel] = new mp3EncodeClass();
			mp3Encode[channel]->init(channel);
			mp3Encode[channel]->config(outputSampleRate[channel],settings->mp3OutputFrequency[channel], 
											outputChannels[channel], settings->mp3EncodeMode[channel],
											settings->mp3ChannelType[channel],settings->mp3Quality[channel],
											settings->mp3MinBitrate[channel] , settings->mp3MaxBitrate[channel],
											settings->mp3VbrQuality[channel],
											&mp3BufferSize, &wavBufferSize);

			mp3EncodeBufferIn[channel] = new AUDIODATA [1];
			memset(mp3EncodeBufferIn[channel],0,sizeof(AUDIODATA));
			mp3EncodeBufferIn[channel]->bufferStart = new unsigned char [wavBufferSize];
			mp3EncodeBufferIn[channel]->data = 	mp3EncodeBufferIn[channel]->bufferStart;
			mp3EncodeBufferIn[channel]->bufferSize = wavBufferSize;
			mp3EncodeBufferOut[channel] = new AUDIODATA [1];
			memset(mp3EncodeBufferOut[channel],0,sizeof(AUDIODATA));
			mp3EncodeBufferOut[channel]->bufferStart = new unsigned char [mp3BufferSize];
			mp3EncodeBufferOut[channel]->data = 	mp3EncodeBufferOut[channel]->bufferStart;
			mp3EncodeBufferOut[channel]->bufferSize = mp3BufferSize;

			}
		}

	scanFirstPass = true;
	runningAverage = 0;
	previousLuma = 0.50f;

	return(true);
}

// *****************************************************************************
// conversionClass::scanCleanup()
//
// Returns TRUE if success
// *****************************************************************************

bool conversionClass::scanCleanup(void)
{
	int channel;
	bool returnStatus;
	int bytesReturned;

	returnStatus = true;

	if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_HYBRID1) ||
		(encodeMode == ENCODE_MODE_HYBRID2) || (encodeMode == ENCODE_MODE_HYBRID2TIME) ||
		(encodeMode == ENCODE_MODE_VBRTIME) || (encodeMode == ENCODE_MODE_DIVXVBRPLUS))
		{
		if(statsFile)
			{
			fclose(statsFile);
			}
		}

	for(channel=0;channel<numAudioStreams;channel++)
		{
		// Fetch the nortmalize min/max values
		audioPostProcess[channel]->getNormalizeMinMax(&auxStats->scanNormalizeMin[channel], &auxStats->scanNormalizeMax[channel]);

		// If we're using lame, then close off lame
		if(settings->mp3)
			{
			bytesReturned = mp3Encode[channel]->encodeEnd(mp3EncodeBufferOut[channel]->bufferStart);
			auxStats->scanVbrAudioBytesUsed[channel] +=bytesReturned;

			mp3Encode[channel]->end();
			delete mp3Encode[channel];
			delete [] mp3EncodeBufferIn[channel]->bufferStart;
			delete [] mp3EncodeBufferIn[channel];
			delete [] mp3EncodeBufferOut[channel]->bufferStart;
			delete [] mp3EncodeBufferOut[channel];
			}
		}

	if(!abortConversion)
		{
		// Write out the aux file for error recovery.
		auxFile = fopen(auxFilename,"wb");
		if(auxFile)
			{
			if(fwrite(auxStats,sizeof(AUXSTATS),1,auxFile)!=1)
				{
				returnStatus=false;
				}
			fclose(auxFile);
			}
		else
			{
			logPrintf("Failed to write auxstats file. Perhaps out of Disk space ?\n");
			returnStatus = false;
			}
		}
	return(returnStatus);
}

// *****************************************************************************
// conversionClass::scanFrameWork()
//
// Returns TRUE if success
// *****************************************************************************

bool conversionClass::scanFrameWork(void)
{
	bool keyFrame;
	BITMAPINFOHEADER * outHeader;
	DWORD encodeFlags;
	FRAMEDATA * encodedFrame;
	volatile bool readyVideo;
	volatile bool readyAudio;
	AUDIODATA * endingSamples;
	double luma;
	float boost;
	int channel;
	int bytesToCopy;
	int totalBytesToCopy;
	int bytesReturned;

	EnterCriticalSection(&writeLock);
	readyVideo = processedFramesReady[writeBufferTail];
	LeaveCriticalSection(&writeLock);

	if(readyVideo)
		{
		for(channel=0;channel<numAudioStreams;channel++)
			{
			EnterCriticalSection(&writeLock);
			readyAudio = processedSamplesReady[channel][writeBufferTail];
			LeaveCriticalSection(&writeLock);

			if(readyAudio)
				{

				// Do resample
				endingSamples = audioPostProcess[channel]->processSamples(processedSamples[channel][writeBufferTail]);
					
				if(endingSamples->bytesUsed)
					{
					audioPostProcess[channel]->processForMinMax(endingSamples);


					// Do the MP3 VBR Audio Processing. 
					if(settings->mp3EncodeMode[channel] == LAME_VBR)
						{
						totalBytesToCopy = endingSamples->bytesUsed;
						do{
							bytesToCopy = totalBytesToCopy;
							// Copy audio bytes to MP3 encode buffer
							if(bytesToCopy > (mp3EncodeBufferIn[channel]->bufferSize - mp3EncodeBufferIn[channel]->bytesUsed))
								{
								bytesToCopy = (mp3EncodeBufferIn[channel]->bufferSize - mp3EncodeBufferIn[channel]->bytesUsed);
								}
							memcpy(mp3EncodeBufferIn[channel]->data,endingSamples->bufferStart,bytesToCopy);
							mp3EncodeBufferIn[channel]->data +=bytesToCopy;
							mp3EncodeBufferIn[channel]->bytesUsed +=bytesToCopy;
							if(mp3EncodeBufferIn[channel]->bytesUsed == mp3EncodeBufferIn[channel]->bufferSize)
								{
							// Encode Bytes if we have enough. Save the count of the bytes returned.
								bytesReturned = mp3Encode[channel]->encode(mp3EncodeBufferIn[channel]->bufferStart,
																		mp3EncodeBufferOut[channel]->bufferStart,
																		mp3EncodeBufferIn[channel]->bufferSize>>1);
								auxStats->scanVbrAudioBytesUsed[channel] += bytesReturned;
								mp3EncodeBufferIn[channel]->data = mp3EncodeBufferIn[channel]->bufferStart;
								mp3EncodeBufferIn[channel]->bytesUsed = 0;
								}
							totalBytesToCopy -=bytesToCopy;
							}while(totalBytesToCopy);
						}

					// Setup for the next buffer
					endingSamples->bytesUsed = 0;
					endingSamples->data = endingSamples->bufferStart;
					}

				if(dualMode)
					{
					// Now free the buffer
					delete [] processedSamples[channel][writeBufferTail]->bufferStart;
					processedSamples[channel][writeBufferTail]->bufferStart = NULL;
					processedSamples[channel][writeBufferTail]->data = NULL;
					}
				else
					{
					processedSamples[channel][writeBufferTail]->data = processedSamples[channel][writeBufferTail]->bufferStart;
					processedSamples[channel][writeBufferTail]->bytesUsed = 0;
					}
					
				// And mark the structure as being available
				EnterCriticalSection(&writeLock);
				processedSamplesReady[channel][writeBufferTail] = false;
				LeaveCriticalSection(&writeLock);
				// Free the buffer we were handed
				}
			}

		if((encodeMode == ENCODE_MODE_CBR) || (encodeMode == ENCODE_MODE_CBR2PASS) ||
			(encodeMode == ENCODE_MODE_DIVXCBR) ||
			(encodeMode == ENCODE_MODE_DIVXCBRQUALITY) || (encodeMode == ENCODE_MODE_DIVXVBR) ||
			 (encodeMode == ENCODE_MODE_XVIDVBR) || (encodeMode == ENCODE_MODE_XVIDHYBRID))
			{
			frameStats->frameSize = 0;
			frameStats->keyFrame = false;
			encodedFrame = encodeVideo1->encode(processedFrames[writeBufferTail],&encodeFlags);
			//frameStats->frameSize = outHeader->biSizeImage;
			//frameStats->keyFrame = keyFrame;

			}
		else
			{
			// Do the scene detect
			if(settings->lumaProcessing)
				{
				luma = decodeVideo->computeLumananceValue(processedFrames[writeBufferTail]);
				}

			if((encodeMode == ENCODE_MODE_DIVXVBRPLUS))
				{
				if(scanFirstPass)
					{
					//encodeVideo1->setDivxNetworksBitrate(auxStats->scanDataRate);
					keyFrame = true;
					encodeVideo1->setDivxNetworksKeyframe(true);
					}
				else
					{
					encodeVideo1->setDivxNetworksKeyframe(false);
					}
				}
			else
				{
				keyFrame = decodeVideo->detectSceneChange(processedFrames[writeBufferTail]);

				if(scanFirstPass)
					{
					encodeVideo1->setCrispness(100);
					// Set to low motion
					encodeVideo1->setCompressionMinMax(2,16);
					//Set our Datarate
					encodeVideo1->setDataRate(auxStats->scanDataRate);

					//Guarantee the first frame is a keyframe
					keyFrame = true;
					}

				encodeVideo1->setBitsRemaining(auxStats->scanDataRate*5);

				//encodeVideo1->setDeltaFrame(!keyFrame);


				//Only the first frame is a keyframes on first pass
				
				if(scanFirstPass)
					{
					encodeVideo1->setDeltaFrame(0);
					}
				else
					{
					encodeVideo1->setDeltaFrame(1);
					}
				}
			encodedFrame = encodeVideo1->encode(processedFrames[writeBufferTail],&encodeFlags);

			if((encodeMode == ENCODE_MODE_DIVXVBRPLUS))
				{
				if(encodeFlags==DIVX_KEYFRAME)
					{
					keyFrame = true;
					}
				else
					{
					keyFrame = false;
					}
				}
			outHeader = encodeVideo1->getFormatOut();
			
			frameStats->frameSize = outHeader->biSizeImage;
			frameStats->keyFrame = keyFrame;

			if(settings->lumaProcessing)
				{
				// Low luma situations need to be compensated for 
				if(settings->lumaLowProcessing)
					{
					if(luma<settings->lumaLowThresh)
						{
						boost = settings->lumaLowBoost / 100.0f;
						frameStats->frameSize += (int) (frameStats->frameSize * boost);
						}
					}
				if(settings->lumaDeltaProcessing)
					{
					//fast luma changes need to be compensated for 
					if(fabs(luma - previousLuma)>settings->lumaDeltaThresh)
						{
						boost = settings->lumaDeltaBoost / 100.0f;
						frameStats->frameSize += (int) (frameStats->frameSize * boost);
						}
					}
				}
			previousLuma = luma;
			auxStats->scanBytes+=frameStats->frameSize;
			}
		
		if(!keyFrame)
			{
			auxStats->scanNumDeltaFrames++;
			auxStats->scanDeltaBytes+=frameStats->frameSize;
			}		
		else
			{
			auxStats->scanNumKeyFrames++;
			auxStats->scanKeyBytes+=frameStats->frameSize;
			}

		if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_HYBRID1) ||
			(encodeMode == ENCODE_MODE_HYBRID2) || (encodeMode == ENCODE_MODE_HYBRID2TIME) ||
			(encodeMode == ENCODE_MODE_VBRTIME) || (encodeMode == ENCODE_MODE_DIVXVBRPLUS))
			{
			if(fwrite(frameStats,sizeof(FRAMESTATS),1,statsFile)!=1)
				{
				// Looks like we ran out of disk space
				logPrintf("Scanned %d frames\n",auxStats->scanNumFrames);
				logPrintf("Write to .stats file failed. Probably ran out of disk space.\n");
				fclose(statsFile);
				return(false);
				}
			}

		#ifndef VIDOMISLAVE
		globals->outputVideo->displayFrame(processedFrames[writeBufferTail]);
		#endif

		auxStats->scanNumFrames++;
		runningAverage = (int)(auxStats->scanBytes / auxStats->scanNumFrames);

		scanFirstPass = false;

		if(dualMode)
			{
			// Now free the buffer
			delete [] processedFrames[writeBufferTail]->allocStart;
			processedFrames[writeBufferTail]->allocStart = NULL;
			processedFrames[writeBufferTail]->bufferStart = NULL;
			processedFrames[writeBufferTail]->data = NULL;
			}			

		// Mark that we processed this frame and the structure is available
		EnterCriticalSection(&writeLock);
		processedFramesReady[writeBufferTail] = false;
		LeaveCriticalSection(&writeLock);

		writeBufferTail++;
		if(writeBufferTail==maxWriteBuffers)
			writeBufferTail=0;
		}
	else
		{
		Sleep(0);
		}

	return(true);
}

// *****************************************************************************
// conversionClass::scanMovie()
//
// Returns TRUE if success
// *****************************************************************************

int conversionClass::scanMovie(void)
{
	bool firstPass;
	FRAMEDATA *	frame;
	FRAMEDATA * endingFrame;
	int size;
	AUDIODATA * samples;
	SUBTITLEDATA * subtitle;
	int	tempState;
	bool finished;
	int i;
	volatile bool readyVideo;
	int currentFrame;
	int lastFrame;
	int channel;
	int returnStatus;

	rangeIndex = 0;
	returnStatus = SCAN_FINISHED_OK;

	if(encodeMode == ENCODE_MODE_XVIDVBR)
		{
		statsFile = fopen(xvidLogFilename,"rb");
		if(statsFile)
			{
			//Previous results exist.. Use them
			fclose(statsFile);
			auxFile = fopen(auxFilename,"rb");
			if(auxFile)
				{
				size = fread(auxStats,sizeof(AUXSTATS),1,auxFile);
				fclose(auxFile);
				// The stats file is valid, so use it.
				if(size == 1)
					{
					logPrintf("Found valid .stats and .aux files. Using them\n");
					return(SCAN_NOT_NEEDED);
					}
				}
			}
		}

	if((encodeMode == ENCODE_MODE_VBR) || (encodeMode == ENCODE_MODE_HYBRID1) ||
		(encodeMode == ENCODE_MODE_VBRTIME) || (encodeMode == ENCODE_MODE_HYBRID2) 
		|| (encodeMode == ENCODE_MODE_HYBRID2TIME) || (encodeMode == ENCODE_MODE_DIVXVBRPLUS))
		{
		logPrintf("Opening Stats Files in Scan Movie..\n");

		statsFile = fopen(statsFilename,"rb");
		if(statsFile)
			{
			//Previous results exist.. Use them
			fclose(statsFile);
			auxFile = fopen(auxFilename,"rb");
			if(auxFile)
				{
				size = fread(auxStats,sizeof(AUXSTATS),1,auxFile);
				fclose(auxFile);
				// The stats file is valid, so use it.
				if(size == 1)
					{
					logPrintf("Found valid .stats and .aux files. Using them\n");
					return(SCAN_NOT_NEEDED);
					}
				}
			}
		}


/*
	// Scan of DivX is always needed now

	if((encodeMode == ENCODE_MODE_DIVXVBR)) 
		{
		statsFile = fopen(divxNetworksLogFilename,"rb");
		if(statsFile)
			{
			//Previous results exist.. Use them
			fclose(statsFile);
			auxFile = fopen(auxFilename,"rb");
			if(auxFile)
				{
				size = fread(auxStats,sizeof(AUXSTATS),1,auxFile);
				fclose(auxFile);
				// The stats file is valid, so use it.
				if(size == 1)
					{
					logPrintf("Found valid .stats and .aux files. Using them\n");
					return(SCAN_NOT_NEEDED);
					}
				}
			}
		}
*/
	// We need to do a scan, so up our pass count if needed
	totalPasses++;


	if((encodeMode == ENCODE_MODE_DIVXVBR)) 
		{
		// If divx 4 or 5, we need to prescan
		totalPasses++;
		if(preScanMovie()==SCAN_FAILED)
			return(SCAN_FAILED);
		currentPass++;
		Sleep(1000);
		rangeIndex = 0;
		}

	logPrintf("Calling PrepareSetup(2)..\n");

	if(!prepareSetup(2))
		return(SCAN_FAILED);


	firstPass = true;
	
	currentFrame = 0;
	lastFrame = 0;

	logPrintf("Entering Scan Main Loop\n");

	// Process each frame in the source video
	while (avRead->decodeFrame())
		{
		// Do the generic per frame work and the first pass setup work
		if(!prepareFrameWork(firstPass,2))
			{
			logPrintf("prepareFrameWork Failed\n");
			prepareCleanup(firstPass,2);
			return(SCAN_FAILED);
			}

		if(firstPass)
			{
			// Send Write Thread Command To Start
			writeBufferHead = 0;
			writeBufferTail = 0;
			if(dualMode)
				{
				EnterCriticalSection(&writeLock);
				writeCommand = writeCommandScanStart;
				LeaveCriticalSection(&writeLock);
				}
			else
				{
				if(!scanSetup())
					break;
				}
			}
		
		frame = avRead->getFrameData();
		endingFrame = postProcess->processFrame(frame);

		if(endingFrame)
			{			
			if(settings->processSubtitles)
				{
				// Track the number of bytes needed for the subtitle tracks
				for(channel=0;channel<numSubtitleStreams;channel++)
					{
					if(settings->processSubtitleMode[channel] == SUBTITLE_MODE_GRAPHIC)
						{
						subtitle = avRead->getSubtitleData(channel);
						if(subtitle->bytesUsed && subtitle->complete)
							{
							if(settings->processSubtitleOnlyEven[channel])
								auxStats->scanSubtitleBytes += (subtitle->bytesUsed/2) + sizeof(DIVXSUBTITLEHEADER);
							else
								auxStats->scanSubtitleBytes += subtitle->bytesUsed + sizeof(DIVXSUBTITLEHEADER);
							
							subtitle->data = subtitle->bufferStart;
							subtitle->bytesUsed = 0;
							subtitle->complete=0;
							}
						}
					}
				}

			if(audio)
				{
				for(channel=0;channel<numAudioStreams;channel++)
					{
					samples = avRead->getAudioData(channel);
					if(samples->bytesUsed)
						{
						if(dualMode)
							{
							// hand the buffer to the other thread
							memcpy(processedSamples[channel][writeBufferHead],samples,sizeof(AUDIODATA));
							EnterCriticalSection(&writeLock);
							processedSamplesReady[channel][writeBufferHead] = true;
							LeaveCriticalSection(&writeLock);
							// Setup a new clean buffer for this thread
							samples->bufferStart = new unsigned char [samples->bufferSize];
							samples->data = samples->bufferStart;
							samples->bytesUsed = 0;
							}
						else
							{
							processedSamples[channel][writeBufferHead] = samples;
							processedSamplesReady[channel][writeBufferHead] = true;
							}
						}
					}
				}

			if(dualMode)
				{
				// hand the buffer to the other thread
				memcpy(	processedFrames[writeBufferHead],endingFrame,sizeof(FRAMEDATA));
				EnterCriticalSection(&writeLock);
				processedFramesReady[writeBufferHead] = true;
				LeaveCriticalSection(&writeLock);
				// Setup a new clean buffer for this thread
				endingFrame->allocStart = new unsigned char [endingFrame->bufferSize+16];
				endingFrame->bufferStart = (unsigned char *)((unsigned long)endingFrame->allocStart+15 &0xfffffff0);
				endingFrame->data = endingFrame->bufferStart;
				}
			else
				{
				processedFrames[writeBufferHead] = endingFrame;
				processedFramesReady[writeBufferHead] = true;
				if(!scanFrameWork())
					{
					break;
					}
				}
			}

		EnterCriticalSection(&writeLock);					
		tempState = writeState;
		LeaveCriticalSection(&writeLock);

		// Did the other process log an error
		if(tempState == writeStateError)
			{
			logPrintf("ScanFrameWork reported a writeStateError\n");
			returnStatus = SCAN_FAILED;
			break;
			}
		
		if(pauseConversion)
			{
			while(pauseConversion && !abortConversion)
				{
				Sleep(1000);
				}
			}

		if(sleepPerFrame)
			{
			Sleep(sleepPerFrame);
			}

		if(avRead->checkEnd() || abortConversion)
			{
			logPrintf("End of Mpeg hit, or user pressed abort\n");
			break;
			}

		firstPass = false;
		if(endingFrame)
			{
			writeBufferHead++;
			if(writeBufferHead==maxWriteBuffers)
				writeBufferHead=0;

			// Check that the next buffer is available. (Ideally it always is)
			do{

				EnterCriticalSection(&writeLock);					
				readyVideo = processedFramesReady[writeBufferHead];
				LeaveCriticalSection(&writeLock);
				if(readyVideo)
					Sleep(0);
				}while(readyVideo);
			}
		currentFrame++;

		if ((currentFilePos + settings->startingFilePos) >= settings->endingFilePos)
			{
			if(lastFrame++ >=settings->endingFileFrame) 
				{	
				// end here or take next segment
				if (checkSegmentedFileEnd(settings)) 
					{
					break;
					}
				logPrintf("Encoded %d Cut Frames \n",currentFrame);
				currentFrame = 0;
				lastFrame = 0;
				}
			}
		}

	logPrintf("Scanned %d frames\n",auxStats->scanNumFrames);

	if(dualMode)
		{
		if(returnStatus != SCAN_FAILED)
			{
			do	{
				finished = true;
				EnterCriticalSection(&writeLock);
				for(i=0;i<maxWriteBuffers;i++)
					{
					if(processedFramesReady[i])
						{
						finished=false;
						break;
						}
					}
				LeaveCriticalSection(&writeLock);
				if(!finished)
					{
					Sleep(0);
					}	
				}while(!finished);
			}
		EnterCriticalSection(&writeLock);
		writeCommand = writeCommandScanEnd;
		LeaveCriticalSection(&writeLock);

		while(1)
			{
			EnterCriticalSection(&writeLock);					
			tempState = writeState;
			LeaveCriticalSection(&writeLock);
			if(tempState==writeStateIdle)
				break;
			Sleep(0);
			}
		}
	else
		{
		if(!scanCleanup())
			{
			return(SCAN_FAILED);
			}
		}


	prepareCleanup(firstPass,2);

	// If we aborted
	if(!abortConversion)
		{
		return(returnStatus);
		}
	else
		{
		logPrintf("Scan was aborted\n");
		return(SCAN_FAILED);
		}
}

// *****************************************************************************
// conversionClass::scanMovie()
//
// Returns TRUE if success
// *****************************************************************************

int conversionClass::preScanMovie(void)
{
	bool firstPass;
	int currentFrame;
	__int64 tempCurPos;
	__int64 tempStartPos;
	__int64 bytesPerFrame;


	if(!prepareSetup(1))
		return(SCAN_FAILED);

	firstPass = true;	
	currentFrame = 0;

	logPrintf("Entering PreScan Main Loop\n");

	tempStartPos = avRead->getCurPos();

	// Process each frame in the source video
	while (avRead->decodeFrame())
		{
		// Do the generic per frame work and the first pass setup work
		if(!prepareFrameWork(firstPass,1))
			{
			logPrintf("prepareFrameWork Failed\n");
			prepareCleanup(firstPass,1);
			return(SCAN_FAILED);
			}
		firstPass = false;

		// Early out if the average bitrate is known
		if(inputInfo->averageDatarate)
			{
			bytesPerFrame = (__int64)((double)inputInfo->averageDatarate / inputFramerate);
			auxStats->scanNumFrames = (int)(totalFileLength / bytesPerFrame);
			prepareCleanup(firstPass,1);
			return(SCAN_FINISHED_OK);
			}

		if(pauseConversion)
			{
			while(pauseConversion && !abortConversion)
				{
				Sleep(1000);
				}
			}

		if(sleepPerFrame)
			{
			Sleep(sleepPerFrame);
			}

		if(abortConversion)
			{
			logPrintf("Prescan Aborted\n");
			prepareCleanup(firstPass,1);
			return(SCAN_FAILED);
			}

		if(avRead->checkEnd())
			{
			break;
			}
		
		currentFrame++;
		if(currentFrame==PRESCAN_FRAMES)
			break;
		}

	// Compute the rough length of the file

	tempCurPos = avRead->getCurPos() - tempStartPos;

	bytesPerFrame = (tempCurPos  / (__int64)currentFrame);
	auxStats->scanNumFrames = (int)(totalFileLength / bytesPerFrame);

	logPrintf("Finished PreScan - Source is Estimated to be %d Frames.\n",auxStats->scanNumFrames);

	prepareCleanup(firstPass,1);

	return(SCAN_FINISHED_OK);

}

// *****************************************************************************
// conversionClass::getSegmentedFileLength()
//
// Get file length of all segments
// *****************************************************************************
__int64 conversionClass::getSegmentedFileLength(CONVERTCONFIG * settings) 
{
	short count;
	__int64 totalFileLength;
	
	if (settings->endingFilePos == 0x7fffffffffffffff)
		{	
		totalFileLength = avRead->getLength();
		}
	else
		{
		if(!settings->rangelist[0].valid) 
			{
			totalFileLength = settings->endingFilePos - settings->startingFilePos;
			}
		 else 
			{
			 // find accumulated file lenght
			xtempPos=0;
			lasttpos=0;
			totalFileLength=0;

			for (count=0;count<MAX_RANGES;count++)
				{
				if(settings->rangelist[count].valid) 
					{
					totalFileLength+=settings->rangelist[count].endingFilePos - settings->rangelist[count].startingFilePos;
					}
				}

			logPrintf("Accumulated segment file length: %I64d\n",totalFileLength);
			}
		}

	return(totalFileLength);
}

// *****************************************************************************
// conversionClass::getTotalNumRanges()
//
// Get total number of Ranges
// *****************************************************************************
int conversionClass::getTotalNumRanges(void) 
{
	int count;
	int segments;

	segments = 0;

	for(count=0;count<MAX_RANGES;count++)
		{
		if(settings->rangelist[count].valid)
			{
			segments++;
			}
		}

	if(segments==0)
		{
		segments=1;
		}

	return(segments);
}


// *****************************************************************************
// conversionClass::checkSegmentedFileEnd()
//
// Check for next segment, return true if end
// *****************************************************************************
bool conversionClass::checkSegmentedFileEnd(CONVERTCONFIG *settings) 
{
	int channel;
	AUDIODATA *	samples;

// end here or take next segment
	for(channel=0;channel<numAudioStreams;channel++)
		{
		cutChangeAudioFlag[channel]=false;
		}
	cutChangeVideoFlag=false;
	
	rangeIndex++;

	if(rangeIndex>=MAX_RANGES) 
		{
		return(true);
		}

	for(;rangeIndex<MAX_RANGES;rangeIndex++) 
		{
		if(settings->rangelist[rangeIndex].valid) 
			{
			// I *hate* gotos. I need to rewrite this to get rid of it.
			goto getnext;
			}
		}
 
	rangeIndex=0;
 
	settings->startingFilePos=settings->rangelist[rangeIndex].startingFilePos;
	settings->startingFileFrame=settings->rangelist[rangeIndex].startingFileFrame;
	settings->endingFilePos=settings->rangelist[rangeIndex].endingFilePos;
	settings->endingFileFrame=settings->rangelist[rangeIndex].endingFileFrame;
	return (true);
 
	// find next segment
	getnext:

	settings->startingFilePos=settings->rangelist[rangeIndex].startingFilePos;
	settings->startingFileFrame=settings->rangelist[rangeIndex].startingFileFrame;
	settings->endingFilePos=settings->rangelist[rangeIndex].endingFilePos;
	settings->endingFileFrame=settings->rangelist[rangeIndex].endingFileFrame;

	// now seek to position
	avRead->seekPos(settings->startingFilePos);
	lasttpos=0;
	logPrintf("Segment n Start: %I64d\n",settings->startingFilePos);
	logPrintf("Segment n End: %I64d\n",settings->endingFilePos);
	logPrintf("Segment n Percent: %f\n",(100.0f*((double)(xtempPos) / (double)(totalFileLength))));

	for(channel=0;channel<numAudioStreams;channel++)
		{
		cutChangeAudioFlag[channel]=true;
		// Clear contents of audio buffers
		samples = avRead->getAudioData(channel);
		if(samples->bytesUsed)
			{
			samples->bytesUsed=0;
			samples->data = samples->bufferStart;
			}
		}
    cutChangeVideoFlag=true;               
	return(false);
}