// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
// conversion class
// *****************************************************************************
#ifndef conversion_h
#define conversion_h

typedef struct {
	int		frameSize;
	int		keyFrame;
} FRAMESTATS;

typedef struct {
	int						scanDataRate;
	int						scanNumFrames;
	int						scanNumKeyFrames;
	int						scanNumDeltaFrames;
	__int64					scanBytes;
	__int64					scanDeltaBytes;
	__int64					scanKeyBytes;
	short					scanNormalizeMin[MAX_AUDIO_CHANNELS];
	short					scanNormalizeMax[MAX_AUDIO_CHANNELS];
	int						scanVbrAudioBytesUsed[MAX_AUDIO_CHANNELS];
	__int64					scanSubtitleBytes;

} AUXSTATS;

#define PRESCAN_FRAMES 1000

// Write Thread Commands
enum {

	writeCommandNoOp=0,
	writeCommandStart,
	writeCommandEnd,
	writeCommandScanStart,
	writeCommandScanEnd,
	writeCommandDie
} ;

enum {

	writeStateIdle=0,
	writeStateWriting,
	writeStateScanning,
	writeStateError,
	writeStateDead
};

#define SCAN_FINISHED_OK 1
#define SCAN_NOT_NEEDED 2
#define SCAN_FAILED 0

#define MAX_WRITE_THREAD_BUFFERS 64

class conversionClass {

	public:
									conversionClass();
									~conversionClass();
			bool					startSingleConversion(JOBLISTENTRY * jobEntry);
			bool					conversionPrepare(void);
			static DWORD WINAPI		conversionWriteThread(LPVOID lParam);
			bool					conversionWrite(void);
			bool					prepareSetup(int passNumber);
			bool					prepareFrameWork(bool firstPass, int passNumber);
			void					prepareCleanup(bool firstPass, int passNumber);
			bool					writeSetup(void);
			bool					writeFrameWork(void);
			void					writeCleanup(void);
			bool					scanSetup(void);
			bool					scanFrameWork(void);
			bool					scanCleanup(void);
			int						scanMovie(void);
			void					writeIntro(bool atSplit);
			int						preScanMovie(void);
			void					getConversionStats(CONVERSIONSTATS * stats);
			__int64 				getSegmentedFileLength(CONVERTCONFIG *settings);
			bool 					checkSegmentedFileEnd(CONVERTCONFIG *settings);
			int						getTotalNumRanges(void);

			volatile bool			abortConversion;
			volatile bool			pauseConversion;
			volatile int			sleepPerFrame;

			DWORD					writeThreadId;
			HANDLE					writeThreadHandle;

	protected:
			int						outputWidth;
			int						outputHeight;
			float					inputFramerate;
			float					outputFramerate;
			char					outputFilename[MAX_PATH];
			char *					inputFilenames[MAX_INPUT_FILES];
			int						fileIndex;
			bool					audio;
			int						audioType[MAX_AUDIO_CHANNELS];
			int						audioTrack[MAX_AUDIO_CHANNELS];
			bool					audioResample[MAX_AUDIO_CHANNELS];
			int						audioSampleRate[MAX_AUDIO_CHANNELS];
			int						audioChannels[MAX_AUDIO_CHANNELS];
			bool					audioDirectStreamCopy[MAX_AUDIO_CHANNELS];
			int						outputSampleRate[MAX_AUDIO_CHANNELS];
			int						outputChannels[MAX_AUDIO_CHANNELS];
			unsigned long			maxOutputFileSize;
			MPEGLAYER3WAVEFORMAT *	audioFormatOut;
			COMPVARS *  			videoFormatOut;
			COMPVARS *  			videoFormatOutP2;
			CONVERTCONFIG *			settings;
			GATHERRESULTS *			inputInfo;
			JOBLISTENTRY *			jobEntry;
			avReadClass *			avRead;
			postProcessClass *		postProcess;
			audioPostProcessClass * audioPostProcess[MAX_AUDIO_CHANNELS];
			writeAviClass *			writeAvi;
			encodeVideoClass *		encodeVideo1;
			encodeVideoClass *		encodeVideo2;
			decodeVideoClass *		decodeVideo;
			subtitlesClass *		subtitles;
			double					cumMs;
			DWORD					startTime;
			DWORD					passTime;
			DWORD					endTime;
			int						encodeMode;
			bool					dualMode;
			char 					statsFilename[MAX_PATH];
			char 					auxFilename[MAX_PATH];
			char 					divxNetworksLogFilename[MAX_PATH];
			char					xvidLogFilename[MAX_PATH];

			FRAMESTATS *			frameStats;
			AUXSTATS *				auxStats;
			int						inputVideoAverageBitrate;
			int						inputVideoAverageBytes;
			int						inputMovieSeconds;
			int						outputAudioBytesRequired;
			int						outputVideoAverageBytes;
			__int64					outputVideoBytes;
			int						outputVideoAverageBitrate;
			DWORD					encoder1FourCC;
			DWORD					encoder2FourCC;
			__int64					maxMovieSize;
			float					variance;
			int						currentPass;
			int						totalPasses;
			__int64					totalFileLength;
			bool					aviConversion;
			bool					writeAviIntro;
	
			// Multithreaded vars / write thread vars
			volatile __int64		currentFilePos;
			volatile int			framesDone;
			FRAMEDATA *	volatile	processedFrames[MAX_WRITE_THREAD_BUFFERS];
			volatile __int64		processedFramesFilePos[MAX_WRITE_THREAD_BUFFERS];
			volatile int			processedFramesFramePos[MAX_WRITE_THREAD_BUFFERS];
			volatile bool			processedFramesCutChange[MAX_WRITE_THREAD_BUFFERS];
			volatile bool			processedFramesReady[MAX_WRITE_THREAD_BUFFERS];
			AUDIODATA *	volatile	processedSamples[MAX_AUDIO_CHANNELS][MAX_WRITE_THREAD_BUFFERS];
			volatile bool			processedSamplesCutChange[MAX_AUDIO_CHANNELS][MAX_WRITE_THREAD_BUFFERS];
			volatile bool			processedSamplesReady[MAX_AUDIO_CHANNELS][MAX_WRITE_THREAD_BUFFERS];
			SUBTITLEDATA * volatile	processedSubtitles[MAX_SUBTITLE_CHANNELS][MAX_WRITE_THREAD_BUFFERS];
			volatile bool			processedSubtitlesReady[MAX_SUBTITLE_CHANNELS][MAX_WRITE_THREAD_BUFFERS];
			bool					cutChangeAudioFlag[MAX_AUDIO_CHANNELS];
			bool					cutChangeVideoFlag;
			volatile int			writeCommand;
			volatile int			writeState;
			int						writeBufferHead;
			int						writeBufferTail;
			CRITICAL_SECTION		writeLock;
			int						bitrateDeltaAccum;
			int						bytesPredicted;
			int						bytesOutput;
			int						compressionLevel;
			bool					writeFirstPass;
			bool					writeFrameFirstPass;
			bool					splitAvi;	
			FILE *					statsFile;
			FILE *					auxFile;
			bool					scanFirstPass;
			int						keyFrameCounter;
			int						runningAverage;
			double					previousLuma;
			int						numAudioStreams;
			int						colorSpace;
			int						maxWriteBuffers;
			int						outputVideoDivxNetworksAverageBitrate;
			int						outputVideoXvidAverageBitrate;
			int						normalizeAudio;
			bool					displayOutputWindow;
			int						preScanNumFrames;
			int						numSubtitleStreams;
			int						writeFrameCounter;
			int						chapterIndex;
			int						headerSize;
			CONVERSIONSTATS *		conversionStats;
			bool					cutChange;
			long					ahardPts;
			long					vpts;
			long					vhardPts;
			__int64					xtempPos;
			__int64					lasttpos;
			int						rangeIndex;
			int						totalRanges;
			volatile bool			conversionThreadTerminated;
			mp3EncodeClass *		mp3Encode[MAX_AUDIO_CHANNELS];
			AUDIODATA *				mp3EncodeBufferIn[MAX_AUDIO_CHANNELS];
			AUDIODATA *				mp3EncodeBufferOut[MAX_AUDIO_CHANNELS];
			bool					usingVbrAudio;
};




#endif