// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
// This is convertconfig.h
//
// *****************************************************************************

#ifndef convertconfig_h
#define convertconfig_h


#define AUDIO_STATE_DEFAULT_SIZE 1024
#define VIDEO_STATE_DEFAULT_SIZE 2048

#define MAX_JOBS 256
#define MAX_RANGES 32
#define MAX_INPUT_FILES 64
#define MAX_CHAPTERS 128

typedef struct{
	
	int		fileType;			
	int		videoWidth;
	int		videoHeight;
	float	videoFramerate;
	double	videoAspectRatio;
	int		videoChromaFormat;
	int		audioFound;
	int		audioRate[48];
	int		audioChannels[48];
	int		audioType[48];
	char	audioName[48][256];
	char	audioLanguage[48][20];
	int		audioNumber[48];
	int		averageDatarate;
	int		subtitleFound;
	char	subtitleName[32][256];
	int		subtitleNumber[32];
	char	subtitleLanguage[32][20];
	int		chaptersFound;
	int		chapterStartFramePos[256];
} GATHERRESULTS;

typedef struct {
	__int64		startingFilePos;
	int			startingFileFrame;
	__int64		endingFilePos;
	int			endingFileFrame;
	int			valid;
} RANGE;

typedef struct 
	{
	bool					hyperCrop;
	int						hyperCropThreshold;
	int						cropTop;
	int						cropBottom;
	int						cropLeft;
	int						cropRight;
	int						scaleWidth;
	int						scaleHeight;
	bool					deinterlace;
	bool					hyperScale;
	bool					scale;
	int						scaleBy;
	int						scaleType;
	bool					blend;
	int						blendThreshold;
	bool					processAudio;
	int						processAudioSelected[MAX_AUDIO_CHANNELS];
	bool					audioDirectStreamCopy[MAX_AUDIO_CHANNELS];
	int						mp3MinBitrate[MAX_AUDIO_CHANNELS];
	int						mp3MaxBitrate[MAX_AUDIO_CHANNELS];
	int						mp3Quality[MAX_AUDIO_CHANNELS];
	int						mp3EncodeMode[MAX_AUDIO_CHANNELS];
	int						mp3ChannelType[MAX_AUDIO_CHANNELS];
	int						mp3OutputFrequency[MAX_AUDIO_CHANNELS];
	int						mp3VbrQuality[MAX_AUDIO_CHANNELS];
	bool					resample48KhzAudio;
	int						maxOutputSize;
	bool					film;
	bool					forceMultiple16;
	bool					mp3;
	bool					lameFound;
	MPEGLAYER3WAVEFORMAT *	audioFormatOut;
	COMPVARS *				videoFormatOut;
	COMPVARS *				videoFormatOutP2;
	char					inputFilenames[MAX_INPUT_FILES][MAX_PATH];
	int						inputFilenameIndex;
	char		 			outputFilename[MAX_PATH];
	int						vbrNumberOfDiscs;
	int						encodeMode;
	int						vbrReserve;
	int						vbrAudioBitrate;
	bool					vbrDeletePassOneFiles;
	int						normalizeAudio;
	int						normalizeLevel;
	bool					showOutput;
	bool					showStats;
	int						vbrCrispness;
	int						vbrMinKeyFrame;
	int						vbrMaxKeyFrame;
	int						vbrMinBitrate;
	int						vbrMaxBitrate;
	int						vbrMinReservoir;
	int						vbrMaxReservoir;
	int						vbrQualityBitrate;
	bool					multiThread;
	bool					introAvi;
	char					outputTail[6];
	int						scanLength;
	bool					useInfoTxtFile;
	bool					lumaProcessing;
	bool					lumaLowProcessing;
	float					lumaLowThresh;
	int						lumaLowBoost;
	bool					lumaDeltaProcessing;
	float					lumaDeltaThresh;
	int						lumaDeltaBoost;
	__int64					startingFilePos;
	int						startingFileFrame;
	__int64					endingFilePos;
	int						endingFileFrame;
	bool					useSpecifiedTarget;
	int						specifiedTarget;
	char					introAviPrimaryPath[MAX_PATH];
	char					introAviSecondaryPath[MAX_PATH];
	bool					aspectRatioOverride;
	double					aspectRatioOverrideValue;
	bool					ivtc;
	bool					ivtcDeComb;
	int						ivtcOffset;
	bool					ivtcPolarity;
	int						ivtcFilmMode;
	bool					overwriteExistingFiles;
	bool					scanPassScale;
	int						defaultOutputSize;
	bool					stopAfterPassOne;
	bool					mp3LimitToCbr;
	bool					mp3LimitToMpeg1;
	char					audioStreamName[MAX_AUDIO_CHANNELS][65];
	char					videoStreamName[65];
	bool					aspectRatioDontCorrect43;
	int						encodeColorSpace;
	bool					processSubtitles;
	int						processSubtitleMode[MAX_SUBTITLE_CHANNELS];
	bool					processSubtitleOnlyEven[MAX_SUBTITLE_CHANNELS];
	int						processSubtitleSelected[MAX_SUBTITLE_CHANNELS];
	char					processSubtitleTextFile[MAX_SUBTITLE_CHANNELS][MAX_PATH];
	char					subtitleStreamName[MAX_SUBTITLE_CHANNELS][65];
	unsigned int			subtitleColorData[4];
	bool					enhancedAvi;
	bool					insertChapters;
	int						numChapters;
	char 					chapterName[MAX_CHAPTERS][65];
	int						chapterStartFramePos[MAX_CHAPTERS];
	RANGE					rangelist[MAX_RANGES];
	bool					splitAtKeyframe;
	bool					hyperEncode;
	int						subJobSection;
	int						xvidEncodeModeOffset;
	int						xvidBitrateOffset;
	int						xvidDesiredSizeOffset;
	int						xvidStatsFilenameOffset;
	int						divx4VbrModeOffset;
	int						divx4BitrateOffset;
	int						divx5BitrateOffset;
	int						divx5VbrModeOffset;
	bool					mpegConversion;
	} CONVERTCONFIG;

typedef struct
	{
	CONVERTCONFIG			convertConfig;
	GATHERRESULTS			gatherResults;
	bool					inputValidated;
	bool					outputSet;
	bool					cropScanDone;
	char					jobName[MAX_PATH];
	COMPVARS				settingsVideoFormatOut;
	BITMAPINFO				settingsVideoFormatOutLpbiOut;
	char					settingsVideoFormatOutLpState[VIDEO_STATE_DEFAULT_SIZE];
	char					settingsAudioFormatOut[AUDIO_STATE_DEFAULT_SIZE];
	COMPVARS				settingsVideoFormatOutP2;
	BITMAPINFO				settingsVideoFormatOutP2LpbiOut;
	char					settingsVideoFormatOutP2LpState[VIDEO_STATE_DEFAULT_SIZE];
	} FLATJOBENTRY;

typedef struct {
	int		elapsedJobTime;
	int		passNumber;
	int		totalPasses;
	float	percentOfPassComplete;
	int		elapsedPassTime;
	int		estimatedRemainingPassTime;
	float	framesPerSecond;
	int		rangeNumber;
	int		totalRanges;
} CONVERSIONSTATS;

#endif
