// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// decodeVideo.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

// *****************************************************************************
// decodeVideoClass::decodeVideoClass()
//
// conversion class constructor
// *****************************************************************************
decodeVideoClass::decodeVideoClass()
{
	frameOut = new FRAMEDATA [1];
	memset(frameOut,0,sizeof(FRAMEDATA));

}

// *****************************************************************************
// decodeVideoClass::~decodeVideoClass()
//
// conversion class destructor
// *****************************************************************************
decodeVideoClass::~decodeVideoClass()
{
	delete [] frameOut;
}

// *****************************************************************************
// decodeVideoClass::init()
//
// *****************************************************************************
bool decodeVideoClass::init()
{
	detectSceneChangeInit(100,300);
	return(true);
}

// *****************************************************************************
// decodeVideoClass::setVideoOutputFormat()
// *****************************************************************************
bool decodeVideoClass::setVideoOutputFormat(int width, int height)
{
	videoWidth = width;
	videoHeight = height;

	memset(&bitmapFormatOut,0,sizeof(BITMAPINFOHEADER));

	bitmapFormatOut.biSize=sizeof(BITMAPINFOHEADER);
	bitmapFormatOut.biWidth=width;
	bitmapFormatOut.biHeight=height;
	bitmapFormatOut.biPlanes=1;
	bitmapFormatOut.biCompression=BI_RGB;
	bitmapFormatOut.biXPelsPerMeter=0;
	bitmapFormatOut.biYPelsPerMeter=0;
	bitmapFormatOut.biClrUsed=0;
	bitmapFormatOut.biClrImportant=0;
	bitmapFormatOut.biSizeImage = width * height * 32;
	bitmapFormatOut.biBitCount = 32;
	return(true);
}

// *****************************************************************************
// decodeVideoClass::setVideoInputFormat()
// *****************************************************************************
bool decodeVideoClass::setVideoInputFormat(BITMAPINFOHEADER * formatIn)
{

	memcpy(&bitmapFormatIn,formatIn,sizeof(BITMAPINFOHEADER));

	videoHic = ICOpen(mmioFOURCC('v', 'i', 'd', 'c'), bitmapFormatIn.biCompression ,ICMODE_DECOMPRESS);
	
	if(!videoHic)
		{
		return(false);
		}

	return(true);

}

// *****************************************************************************
// decodeVideoClass::startDecoder()
//
// *****************************************************************************
bool decodeVideoClass::startDecoder()
{
	LRESULT lres;
	DWORD size;

	lres = ICDecompressBegin(videoHic, &bitmapFormatIn, &bitmapFormatOut);
	if(lres != ICERR_OK)
		return(false);

	// Fetch max size of a compressed frame
	size = videoWidth * videoHeight * 4;

	frameOut->allocStart = new unsigned char [size]; 
	frameOut->bufferStart = frameOut->allocStart;
	frameOut->data = frameOut->allocStart;
	frameOut->width = videoWidth;
	frameOut->height = videoHeight;
	frameOut->stride = videoWidth*4;
	return(true);
}

// *****************************************************************************
// decodeVideoClass::decode()
//
// *****************************************************************************
FRAMEDATA * decodeVideoClass::decode(FRAMEDATA * frameIn)
{
	
	ICDecompress(videoHic,ICDECOMPRESS_NOTKEYFRAME ,&bitmapFormatIn,frameIn->data,&bitmapFormatOut,frameOut->data);

	return(frameOut);
}

// *****************************************************************************
// decodeVideoClass::endDecoder()
//
// *****************************************************************************
bool decodeVideoClass::endDecoder()
{

	if(frameOut->allocStart)
		{
		delete [] frameOut->allocStart;
		memset(frameOut,0,sizeof(FRAMEDATA));
		}

	if(videoHic)
		{
		//videoHic->unused = 0;
		ICDecompressEnd(videoHic);
		ICClose(videoHic);
		}

	return(true);
}

// *****************************************************************************
// decodeVideoClass::delta()
//
// *****************************************************************************
int decodeVideoClass::delta(FRAMEDATA * image1, FRAMEDATA * image2)
{
	int w;
	int h;
	unsigned char * ptr1; 
	unsigned char * ptr2; 
	unsigned char * ptr1s; 
	unsigned char * ptr2s; 
	int diff;

	diff = 0;
	ptr1s = image1->data;
	ptr2s = image2->data;

	for(h=0;h<image1->height;h++)
		{
		ptr1 = ptr1s;
		ptr2 = ptr2s;
		for(w=0;w<(image1->width*4);w++)
			{
			if(*ptr1 < *ptr2)
				{
				diff += (*ptr2++ - *ptr1++);
				}
			else
				{
				diff += (*ptr1++ - *ptr2++);
				}
			}
		ptr1s += image1->stride;
		ptr2s += image2->stride;
		}

	return(diff);
}

// *****************************************************************************
// decodeVideoClass::detectSceneChange()
//
// *****************************************************************************
bool decodeVideoClass::detectSceneChange(FRAMEDATA * image1)
{
	switch(image1->type)
		{
		case rgb32Frame:
			return(detectSceneChangeRgb32(image1));
		case yv12Frame:
			return(detectSceneChangeYv12(image1));
		default:
			return(0);
		}
}

// *****************************************************************************
// decodeVideoClass::detectSceneChange()
//
// *****************************************************************************
bool decodeVideoClass::detectSceneChangeYv12(FRAMEDATA * image1)
{
	int i;
	int totalBlocksDifference;
	int blockX;
	int blockY;
	unsigned char * ptr;
	unsigned char * ptrs;
	unsigned char * ptrPos;

	int blockWidth;
	int blockHeight;
	int x;
	int y;
	int index;
	int stride;
	int histTemp;
	int blockNumber;
	int blockSum;
	int val;
	unsigned char * vStart;
	unsigned char * uStart;
	int vuDelta; 
	sceneDetectFrame++;

	// If it's been 300 frames without a keyframe, then force one
	if(sceneDetectFrame>maxKeyFrame)
		{
		sceneDetectFrame=0;
		return(true);
		}
		
	totalBlocksDifference =0;
	
	blockNumber = 0;

	blockWidth = (image1->width/2)/5;
	blockHeight = (image1->height/2)/5;
	stride = (image1->width/2);

	vStart = image1->data + (image1->width * image1->height);
	uStart = vStart + ((image1->width/2) * (image1->height/2));
	vuDelta = uStart - vStart;

	for(blockY=0;blockY<5;blockY++)
		{
		ptrPos = vStart + (blockY * (blockHeight * stride));

		for(blockX=0;blockX<5;blockX++)
			{
			for(i=0;i<64;i++)
				{
				histogram[histCur+blockNumber][i] = 1;
				}
			ptrs = ptrPos; 
			for(y=0;y<blockHeight;y++)
				{
				ptr = ptrs;
				for(x=0;x<blockWidth;x++)
					{
					index = (*ptr&0xe0)>>5;
					index |= ((*ptr+vuDelta)&0xe0)>>2;
					*ptr++;
					histogram[histCur+blockNumber][index]++;
					}
				ptrs+=stride;
				}

			// Now Compare the current frames histogram to the previous frames

			blockSum =0;
			for(i=0;i<64;i++)
				{
				val = ((histogram[histCur+blockNumber][i] - histogram[histPrev+blockNumber][i]) / histogram[histPrev+blockNumber][i])/2;
				blockSum += val;
				//dif = abs(histogram[histCur+blockNumber][i] - histogram[histPrev+blockNumber][i]);
				//if(dif>largestDif)
				//	largestDif =dif;
				// Accumulate any dramatic changes
				//if(dif>10)
				//	blockSum+= dif;
				}

			totalBlocksDifference += blockSum;

			ptrPos+= blockWidth;
			blockNumber++;
			}
		}

	// Swap the histogram buffers
	histTemp = histCur;
	histCur = histPrev;
	histPrev = histTemp;

	if(totalBlocksDifference >4096)
		{
		// If we found the next scene change too quick, dump the results
		if(sceneDetectFrame<minKeyFrame)
			return(false);
		
		// otherwise call it valid and clear the frame count
		sceneDetectFrame=0;
		return(true);
		}
	return(false);
}


// *****************************************************************************
// decodeVideoClass::detectSceneChange()
//
// *****************************************************************************
bool decodeVideoClass::detectSceneChangeRgb32(FRAMEDATA * image1)
{
	int i;
	int totalBlocksDifference;
	int blockX;
	int blockY;
	unsigned char * ptr;
	unsigned char * ptrs;
	unsigned char * ptrPos;
	int blockWidth;
	int blockHeight;
	int x;
	int y;
	int index;
	int stride;
	int histTemp;
	int blockNumber;
	int blockSum;
	int val;

	blockWidth = image1->width /5;
	blockHeight = image1->height/5;
	stride = image1->stride;

	sceneDetectFrame++;

	// If it's been 300 frames without a keyframe, then force one
	if(sceneDetectFrame>maxKeyFrame)
		{
		sceneDetectFrame=0;
		return(true);
		}
		
	totalBlocksDifference =0;
	
	blockNumber = 0;


	for(blockY=0;blockY<5;blockY++)
		{
		ptrPos = image1->data + (blockY * (blockHeight * stride));

		for(blockX=0;blockX<5;blockX++)
			{
			for(i=0;i<64;i++)
				{
				histogram[histCur+blockNumber][i] = 1;
				}
			ptrs = ptrPos; 
			for(y=0;y<blockHeight;y++)
				{
				ptr = ptrs;
				for(x=0;x<blockWidth;x++)
					{
					index = (*ptr++&0xc0)>>2;
					index |=(*ptr++&0xc0)>>4;
					index |=(*ptr++&0xc0)>>6;
					*ptr++;

					histogram[histCur+blockNumber][index]++;
					}
				ptrs+=stride;
				}

			// Now Compare the current frames histogram to the previous frames

			blockSum =0;
			for(i=0;i<64;i++)
				{
				val = ((histogram[histCur+blockNumber][i] - histogram[histPrev+blockNumber][i]) / histogram[histPrev+blockNumber][i])/2;
				blockSum += val;
				//dif = abs(histogram[histCur+blockNumber][i] - histogram[histPrev+blockNumber][i]);
				//if(dif>largestDif)
				//	largestDif =dif;
				// Accumulate any dramatic changes
				//if(dif>10)
				//	blockSum+= dif;
				}

			totalBlocksDifference += blockSum;

			ptrPos+= blockWidth*4;
			blockNumber++;
			}
		}

	// Swap the histogram buffers
	histTemp = histCur;
	histCur = histPrev;
	histPrev = histTemp;

	if(totalBlocksDifference >4096)
		{
		// If we found the next scene change too quick, dump the results
		if(sceneDetectFrame<minKeyFrame)
			return(false);
		
		// otherwise call it valid and clear the frame count
		sceneDetectFrame=0;
		return(true);
		}
	return(false);
}

// *****************************************************************************
// decodeVideoClass::detectSceneChange()
//
// *****************************************************************************
bool decodeVideoClass::detectSceneChangeInit(int min, int max)
{
	int b;
	int i;

	// Init the double buffered histogram offsets
	histCur = 0;
	histPrev = 25;
	sceneDetectFrame = 0;
	minKeyFrame = min;
	maxKeyFrame = max;

	for(b=0;b<50;b++)
		{
		for(i=0;i<64;i++)
			{
			histogram[b][i]=1;
			}
		}
	return(true);

}

// *****************************************************************************
// decodeVideoClass::detectSceneChange()
//
// *****************************************************************************
double decodeVideoClass::computeLumananceValue(FRAMEDATA * image)
{

	switch(image->type)
		{
		case rgb32Frame:
			return(computeLumananceValueRgb32(image));
		case yv12Frame:
			return(computeLumananceValueYv12(image));
		default:
			return(0);
		}

}
// *****************************************************************************
// decodeVideoClass::detectSceneChangeRgb32()
//
// *****************************************************************************
double decodeVideoClass::computeLumananceValueRgb32(FRAMEDATA * image)
{
	int width;
	int height;
	int stride;
	unsigned char * ptrPos;
	unsigned char * ptr;
	int x;
	int y;
	unsigned char c1;
	unsigned char c2;
	unsigned char c3;
	unsigned int overallLuma;
	double lumaSq;
	width = image->width;
	height = image->height;
	stride = image->stride;
	
	ptrPos = image->data;
	overallLuma = 0;
	lumaSq = 0.0f;
	for(y=0;y<height;y++)
		{
		ptr = ptrPos;
		for(x=0;x<width;x++)
			{
			c1 = *ptr++;
			c2 = *ptr++;
			c3 = *ptr++;
			*ptr++;
			overallLuma += (int) ((c1 * 0.299f) + (c2 * 0.587f) + (0.114 * c3));
			//tempLuma = (int) ((c1 * 0.299f) + (c2 * 0.587f) + (0.114 * c3));
			//lumaSq += tempLuma * tempLuma;
			//overallLuma +=tempLuma;
			}
		ptrPos+=stride;
		}

	//lumaSq *= (width*height);
	
	//return(fabs(sqrt(lumaSq) - (double)overallLuma) / (width*height));
	overallLuma = overallLuma / (width*height);

	return((double)overallLuma / (double)255.0f);
}

// *****************************************************************************
// decodeVideoClass::detectSceneChangeYv12()
//
// *****************************************************************************
double decodeVideoClass::computeLumananceValueYv12(FRAMEDATA * image)
{
	int width;
	int height;
	int stride;
	unsigned char * ptrPos;
	unsigned char * ptr;
	int x;
	int y;
	unsigned int overallLuma;
	double lumaSq;
	width = image->width;
	height = image->height;
	stride = image->width;
	
	ptrPos = image->data;
	overallLuma = 0;
	lumaSq = 0.0f;
	for(y=0;y<height;y++)
		{
		ptr = ptrPos;
		for(x=0;x<width;x++)
			{
			overallLuma += *ptr;
			}
		ptrPos+=stride;
		}

	//lumaSq *= (width*height);
	
	//return(fabs(sqrt(lumaSq) - (double)overallLuma) / (width*height));
	overallLuma = overallLuma / (width*height);

	return((double)overallLuma / (double)255.0f);
}
