// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// deepScan.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;


// *****************************************************************************
// deepScanClass::deepScanClass()
//
// deepScan class constructor
// *****************************************************************************
deepScanClass::deepScanClass()
{
}

// *****************************************************************************
// deepScanClass::~deepScanClass()
//
// deepScan class destructor
// *****************************************************************************
deepScanClass::~deepScanClass()
{
}

// *****************************************************************************
// deepScanClass::deepScan()
//
// scans an mpeg file for detailed and accurate audio and video info
// *****************************************************************************
bool deepScanClass::deepScan(CONVERTCONFIG * settings, GATHERRESULTS * fileInfo)
{
	int i;
	bool status;
	__int64 pos;

	// Create the needed buffers for the scan
	readBuffer = new unsigned char [DEEPSCAN_BUFFER_SIZE];
	scanFramesSize = 10000;
	scanFrames = (DEEPSCANFRAMES *) malloc(sizeof(DEEPSCANFRAMES) * scanFramesSize);
	scanFramesPos = 0;

	status = false;
	pos =0;
	totalSize = 0;

	// Open the files for the scan
	for(i=0;i<settings->inputFilenameIndex;i++)
		{
		inFiles[i] = _open(settings->inputFilenames[i],_O_SEQUENTIAL | _O_BINARY );
		if(inFiles[i]==-1)
			{
			logPrintf("Failed to open Mpeg File %s for Deep Scanning\n", settings->inputFilenames[i]);
			status = true;
			break;
			}

		if(_lseeki64( inFiles[i], 0, SEEK_END )==-1)
			{
			logPrintf("Failed to seek to end of Mpeg File for Deep Scanning\n");
			status = true;
			break;
			}
		inSize[i] = _telli64(inFiles[i]);
		inStart[i] = pos;
		pos += inSize[i];
		inEnd[i] = pos;
		totalSize += inSize[i];
		}

	numFiles = settings->inputFilenameIndex;
	scanSize = totalSize;
	//scanSize = 100000000;

	if(!status)
		{
		setPos(0);
		// perform scan
		status = scan();
		}

	// Close all files
	for(i=0;i<settings->inputFilenameIndex;i++)
		{
		_close(inFiles[i]);
		}

	for(i=0;i<1000;i++)
		{
		switch(scanFrames[i].frame->type)
			{
			case 0:
				logPrintf("Frame %d - %I64x - Type 0\n",i,scanFrames[i].frame->pos);	
				break;
			case IFRAME:
				logPrintf("Frame %d - %I64x - Type I Frame\n",i,scanFrames[i].frame->pos);	
				break;
			case PFRAME:
				logPrintf("Frame %d - %I64x - Type P Frame\n",i,scanFrames[i].frame->pos);	
				break;
			case BFRAME:
				logPrintf("Frame %d - %I64x - Type B Frame\n",i,scanFrames[i].frame->pos);	
				break;
			case 4:
				logPrintf("Frame %d - %I64x - Type 4\n",i,scanFrames[i].frame->pos);	
				break;
			case 5:
				logPrintf("Frame %d - %I64x - Type 5\n",i,scanFrames[i].frame->pos);	
				break;
			case 6:
				logPrintf("Frame %d - %I64x - Type 6\n",i,scanFrames[i].frame->pos);	
				break;
			case 7:
				logPrintf("Frame %d - %I64x - Type 7\n",i,scanFrames[i].frame->pos);	
				break;
			}
		}	
	free(scanFrames);
	delete [] readBuffer;

	return(status);
}

// *****************************************************************************
// deepScanClass::scan()
//
// Perform Scan
// ****************************************************************************
bool deepScanClass::scan()
{

	unsigned long code;
	int horizontalSizeExtension;
	int verticalSizeExtension;
	int aspectRatioCode;
	int frameRateCode;
	int width;
	int height;
	int frameRateExtensionN;
	int frameRateExtensionD;
	int size;
	int frameCount;
	// Start Getting bits and walk through file 
	bool sequenceHeader;
	bool sequenceExtension;
	int lastPictureValue;
	int curPictureValue;
	int frameType;

	frameCount = 0;
	lastPictureValue = -1;

	sequenceHeader = false;
	sequenceExtension = false;

	while(!reachedEnd)
		{
		alignFrame();
		code = getBits(32);
		switch(code)
			{
			case PACK_START:
				setPos(bufferPos + bufferStart + 10);
				break;

			case PRIVATE_STREAM:
				size = (getByte() <<8) | getByte();
				setPos(bufferPos + bufferStart + size);
				break;

			case PADDING_STREAM:
				size = (getByte() <<8) | getByte();
				setPos(bufferPos + bufferStart + size);
				break;

			case PRIVATE_STREAM2:
				size = (getByte() <<8) | getByte();
				setPos(bufferPos + bufferStart + size);
				break;
	
			case GOP_HEADER:
				setPos(bufferPos + bufferStart + 4);
				break;

			case SEQUENCE_HEADER :
				if(!sequenceHeader)
					{
					width = getBits(12);
					height = getBits(12);
					aspectRatioCode = getBits(4);
					//aspectRatio = aspectTable[aspectRatioCode];
					frameRateCode = getBits(4);
	//				frameRate = (float) framerateTable[frameRateCode];
					sequenceHeader = true;
					}
				else
					{
					setPos(bufferPos + bufferStart + 8);
					}
				break;

			case SEQUENCE_EXTENSION :
				if(!sequenceExtension)
					{
					flushBits(8);
					flushBits(1);
					flushBits(2);			//chromaFormat = getBits(2);
					horizontalSizeExtension = getBits(2);
					verticalSizeExtension = getBits(2);
					flushBits(12);
					flushBits(1);
					flushBits(8);
					flushBits(1);
					frameRateExtensionN = getBits(2);
					frameRateExtensionD = getBits(5);

					//frameRate = (float) framerateTable[frameRateCode] * (frameRateExtensionN + 1) / (frameRateExtensionD +1 );
					width = (horizontalSizeExtension<<12) | (width&0x0fff);
					height = (verticalSizeExtension<<12) | (height&0x0fff);
					sequenceExtension = true;
					}
				break;

			case PICTURE_HEADER:
				curPictureValue = getBits(10);
				frameType = getBits(3);
				if(frameType == IFRAME || frameType == BFRAME || frameType == PFRAME)
					{
					size = getBits(16);
					size/=8;
					if(curPictureValue!=lastPictureValue)
						{
						scanFrames[scanFramesPos].frame->pos = (bufferPos + bufferStart)-8;
						scanFrames[scanFramesPos].frame->type = frameType;
						scanFramesPos++;
						if(scanFramesPos==scanFramesSize)
							{
							// Grow the scan frames buffer as needed
							scanFramesSize+=10000;
							scanFrames = (DEEPSCANFRAMES *) realloc(scanFrames,sizeof(DEEPSCANFRAMES) * scanFramesSize);
							}
						frameCount++;
						lastPictureValue = curPictureValue;
						}
					//setPos(bufferPos + bufferStart + size);
					}
				break;

			default:
				break;
			}
		}
	
	logPrintf("Frame Count = %d\n",frameCount);

	return(true);
}

// *****************************************************************************
// deepScanClass::setPos()
//
// Set position in File List
// *****************************************************************************
bool deepScanClass::setPos(__int64 pos)
{
	int i;
	__int64 localPosStart;
	__int64 readSize;
	unsigned char * ptr;
	int bytesRead;
	__int64 maxReadSize;

	if((pos < bufferStart) || (pos > bufferEnd))
		{
		if(pos>=scanSize)
			{
			reachedEnd = true;
			return(false);
			}
		

		bytesRead =0;
		bufferStart = pos;
		ptr = readBuffer;
		reachedEnd = false;

		do{
			// Find the starting File
			for(i=0;i<numFiles;i++)
				{
				if(pos>=inStart[i] && pos<inEnd[i])
					{
					curFile = i;
					break;
					}
				}

			if(i==numFiles)
				{
				break;
				}

			localPosStart = pos - inStart[curFile];

			// we need to fill the buffer with new data
			_lseeki64(inFiles[curFile],localPosStart,SEEK_SET);
			
			maxReadSize = inSize[curFile] - localPosStart;

			if(maxReadSize > DEEPSCAN_BUFFER_SIZE)
				maxReadSize = DEEPSCAN_BUFFER_SIZE;

			readSize = DEEPSCAN_BUFFER_SIZE - bytesRead;

			if(readSize>maxReadSize)
				readSize = maxReadSize;

			_read(inFiles[curFile],ptr,(int) readSize);
			
			bytesRead += (int)readSize;
			ptr+=(int)readSize;
			pos+=readSize;
			}while(bytesRead!=DEEPSCAN_BUFFER_SIZE);
		
		bufferEnd = bufferStart + (bytesRead - 8);	
		bitsRemaining = 8;
		bufferPos = 0;
		}
	else
		{
		// move the pointer in memory
		bufferPos = (int)(pos - bufferStart);
		bitsRemaining = 8;
		reachedEnd = false;
		}
	return(true);
}

// *****************************************************************************
// deepScanClass::getBits()
//
// get up to 32 Bits from the file
// *****************************************************************************
unsigned long deepScanClass::getBits(int numBits)
{
	int giveBits;
	unsigned long outputValue;

	outputValue=0;
	if(!reachedEnd)
		{
		while(numBits)
			{
			giveBits = numBits;
			if(numBits >= bitsRemaining)
				giveBits = bitsRemaining;
			outputValue = (outputValue << giveBits) | 
				(readBuffer[bufferPos]&(0xff>>(8-bitsRemaining)))>>(bitsRemaining-giveBits);
			bitsRemaining = bitsRemaining - giveBits;
			numBits = numBits - giveBits;
			if(!bitsRemaining)
				{
				bufferPos++;
				bitsRemaining=8;
				if((bufferPos + bufferStart)>bufferEnd)
					setPos(bufferPos + bufferStart);
				if(reachedEnd)
					return(outputValue);
				}
			}
		}
	return(outputValue);

}

// *****************************************************************************
// deepScanClass::alignFrame()
//
// align to the beginning of a frame header
// *****************************************************************************
void deepScanClass::alignFrame(void)
{
	long nextBits;

	if(bitsRemaining!=8)
		getBits(bitsRemaining);

	nextBits =0;

	while(!reachedEnd)
		{
		nextBits = peekBits(24);
		if(nextBits!=0x000001)
			{
			getByte();					//keep moving forward in stream until we find a valid frame start
			}
		else
			{
			break;
			}
		}
}

// *****************************************************************************
// deepScanClass::peekBits()
//
// get up to 32 Bits from the file
// *****************************************************************************
unsigned long deepScanClass::peekBits(int numBits)
{
	int giveBits;
	unsigned long outputValue;
	int tempBufferPos;
	long tempBitsRemaining;

	outputValue=0;
	
	tempBufferPos = bufferPos;
	tempBitsRemaining = bitsRemaining;

	if(!reachedEnd)
		{
		while(numBits)
			{
			giveBits = numBits;
			if(numBits >= tempBitsRemaining)
				giveBits = tempBitsRemaining;
			outputValue = outputValue << giveBits | 
				(readBuffer[tempBufferPos]&(0xffffffff>>(32-tempBitsRemaining)))>>(tempBitsRemaining>>giveBits);
			tempBitsRemaining = tempBitsRemaining - giveBits;
			numBits = numBits - giveBits;
			if(!tempBitsRemaining)
				{
				// Peek is allowed to go past the end for up to 32 bits (it's max peek size). 
				// this is to prevent thrashing of the buffer
				tempBufferPos++;
				tempBitsRemaining=8;
				}
			}
		}
	return(outputValue);

}

// *****************************************************************************
// deepScanClass::flushBits()
//
// flush N bits
// *****************************************************************************
void deepScanClass::flushBits(int numBits)
{
	getBits(numBits);
}

// *****************************************************************************
// deepScanClass::getByte()
//
// *****************************************************************************
unsigned char deepScanClass::getByte()
{
	unsigned char val;

	if(bitsRemaining != 8)
		return((unsigned char) getBits(8));

	val = readBuffer[bufferPos];

	bufferPos++;

	if((bufferPos + bufferStart)>bufferEnd)
		setPos(bufferPos + bufferStart);
	return(val);
}

