// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// job.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

char * encodeThreadPriorityType[] = {"VidomiEncodeControlLowest","VidomiEncodeControlNormal","VidomiEncodeControlHighest"};
int encodeThreadPriorityValue[] = {THREAD_PRIORITY_LOWEST,THREAD_PRIORITY_NORMAL,THREAD_PRIORITY_HIGHEST};
int encodeSleepPerFrame[] = {0,1,3,5,8,10,20,25,50,75,100,150,200,300,400,500,600,700,800,900,1000,2000,3000,5000};

static TRANSLATEIDTABLE encodeControlTID[] = {
										{"VidomiEncodeControlText1",VidomiEncodeControlText1},
										{"VidomiEncodeControlPause",VidomiEncodeControlPause},
										{"VidomiEncodeControlStop",VidomiEncodeControlStop},
										{"VidomiEncodeControlText2",VidomiEncodeControlText2},
										{"VidomiEncodeControlText3",VidomiEncodeControlText3},
										{"VidomiEncodeControlText4",VidomiEncodeControlText4},
										{"VidomiEncodeControlText5",VidomiEncodeControlText5},

										{"",0}																			
										};
// *****************************************************************************
// encodeControlClass::encodeControlClass()
//
// conversion class constructor
// *****************************************************************************
encodeControlClass::encodeControlClass()
{
	currentPriority = 1;
	currentSleepPerFrame = 0;
	windowsCreated = false;
	shutdown = false;
	visible = false;
}

// *****************************************************************************
// encodeControlClass::~encodeControlClass()
//
// conversion class destructor
// *****************************************************************************
encodeControlClass::~encodeControlClass()
{
	if(encodeControlhWnd && windowsCreated)
		DestroyWindow(encodeControlhWnd);

}

// *****************************************************************************
// encodeControlClass::createOutputWindow()
//
// conversion class constructor
// *****************************************************************************
bool encodeControlClass::createWindow(HINSTANCE hInst)
{
	encodeControlhInst = hInst;

	return(true);
}

// *****************************************************************************
// encodeControlClass::showJobWindow()
//
// 
// *****************************************************************************
bool encodeControlClass::showWindow()
{
	RECT client;
	int fullHeight;
	int fullWidth;
	int i;
	int ret;
	char sleepText[255];
	char text[4096];


	if(!windowsCreated)
		{
		fullHeight = GetSystemMetrics(SM_CYSCREEN);
		fullWidth = GetSystemMetrics(SM_CXSCREEN);

		if((globals->encodeControlLeft > fullWidth) || globals->encodeControlLeft < 0)
			globals->encodeControlLeft = 320;

		if((globals->encodeControlTop > fullHeight) || globals->encodeControlTop < 0)
			globals->encodeControlTop = 240;

		encodeControlhWnd = CreateDialog(encodeControlhInst,MAKEINTRESOURCE(VidomiEncodeControl),NULL, encodeControlDialogHandler);

		globals->translate->translate(text, "VidomiEncodeControl");
		SetWindowText(encodeControlhWnd, text);			

		for(i=0;;i++)
			{
			if(!encodeControlTID[i].id)
				break;
			globals->translate->translate(text, encodeControlTID[i].textId);
			SetWindowText(GetDlgItem(encodeControlhWnd, encodeControlTID[i].id),text);			
			}

		GetWindowRect(encodeControlhWnd, &client);
		MoveWindow(encodeControlhWnd, globals->encodeControlLeft, globals->encodeControlTop, client.right - client.left, client.bottom - client.top, FALSE);

		SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlPriority, CB_RESETCONTENT , 0, 0);

		for(i=0;i<(sizeof(encodeThreadPriorityType)/4);i++)
			{
			globals->translate->translate(text, encodeThreadPriorityType[i]);

			ret = SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlPriority, CB_ADDSTRING , (WPARAM)0 , (LPARAM) text); 									
			SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlPriority, CB_SETITEMDATA  , ret, i);
			if(i==currentPriority)
				SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlPriority, CB_SETCURSEL  , ret, 0);
			}

		SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlSleepPerFrame, CB_RESETCONTENT , 0, 0);

		for(i=0;i<(sizeof(encodeSleepPerFrame)/4);i++)
			{
			sprintf(sleepText,"%d",encodeSleepPerFrame[i]);
			ret = SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlSleepPerFrame, CB_ADDSTRING , (WPARAM)0 , (LPARAM) sleepText); 									
			SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlSleepPerFrame, CB_SETITEMDATA  , ret, encodeSleepPerFrame[i]);
			if(i==currentSleepPerFrame)
				SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlSleepPerFrame, CB_SETCURSEL  , ret, 0);
			}

		SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlShutdown,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

		if(shutdown)
			{
			SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlShutdown,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
			}
		else
			{
			SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlShutdown,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);
			}
		windowsCreated = true;
		}
	visible = true;
	ShowWindow(encodeControlhWnd, SW_SHOW);	
	BringWindowToTop(encodeControlhWnd);
	return(true);
}
// *****************************************************************************
// encodeControlClass::hideOutputWindow()
//
// 
// *****************************************************************************
bool encodeControlClass::hideWindow()
{
	if(windowsCreated)
		{
		//windowsCreated = false;
		ShowWindow(encodeControlhWnd, SW_HIDE);	
		//DestroyWindow(encodeControlhWnd);
		}
	visible = false;

	return(true);
}

// *****************************************************************************
// encodeControlClass::windowVisible()
//
// 
// *****************************************************************************
bool encodeControlClass::windowVisible()
{
	return(visible);
}

// *****************************************************************************
// encodeControlClass::resetWindowState()
//
// 
// *****************************************************************************
void encodeControlClass::resetWindowState()
{
	int ret;
	int i;
	char sleepText[255];
	char text[4096];

	currentPriority = 1;
	running = true;
	
	if(windowsCreated)
		{
		globals->translate->translate(text,"VidomiEncodeControlRunning");
		SetWindowText(GetDlgItem(encodeControlhWnd, VidomiEncodeControlStatusText),text);	
		globals->translate->translate(text,"VidomiEncodeControlPauseConversion");
		SetWindowText(GetDlgItem(encodeControlhWnd, VidomiEncodeControlPause),text);	

		SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlPriority, CB_RESETCONTENT , 0, 0);

		for(i=0;i<(sizeof(encodeThreadPriorityType)/4);i++)
			{
			globals->translate->translate(text, encodeThreadPriorityType[i]);
			ret = SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlPriority, CB_ADDSTRING , (WPARAM)0 , (LPARAM) text); 									
			SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlPriority, CB_SETITEMDATA  , ret, i);
			if(i==currentPriority)
				SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlPriority, CB_SETCURSEL  , ret, 0);
			}

		SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlSleepPerFrame, CB_RESETCONTENT , 0, 0);

		for(i=0;i<(sizeof(encodeSleepPerFrame)/4);i++)
			{
			sprintf(sleepText,"%d",encodeSleepPerFrame[i]);
			ret = SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlSleepPerFrame, CB_ADDSTRING , (WPARAM)0 , (LPARAM) sleepText); 									
			SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlSleepPerFrame, CB_SETITEMDATA  , ret, encodeSleepPerFrame[i]);
			if(i==currentSleepPerFrame)
				SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlSleepPerFrame, CB_SETCURSEL  , ret, 0);
			}

		SendDlgItemMessage(encodeControlhWnd,VidomiEncodeControlShutdown,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);
		}					
}

// *****************************************************************************
// encodeControlClass::jobDialogHandler()
//
//
// *****************************************************************************
BOOL CALLBACK encodeControlClass::encodeControlDialogHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->encodeControl->encodeControlDialogHandler1(hWnd,message,wParam,lParam));
}
// *****************************************************************************
// encodeControlClass::jobDialogHandler1()
//
//
// *****************************************************************************
BOOL encodeControlClass::encodeControlDialogHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	RECT win;
	int state;
	bool onOff;
	unsigned short commandId;
	unsigned short commandEvent;
	char text[4096];

	switch(message)
		{
		case WM_MOVE :
			GetWindowRect(hWnd, &win);
			globals->encodeControlLeft = win.left;
			globals->encodeControlTop  = win.top;    
			return(TRUE);

		case WM_CLOSE:
			hideWindow();
			return(TRUE);

		case WM_COMMAND:
            commandId    = LOWORD(wParam); // Remember, these are...
			commandEvent = HIWORD(wParam); // ...different for Win32!
			//Parse the menu selections:
			switch (commandId) 
				{
				case VidomiEncodeControlStop:
					globals->translate->translate(text,"VidomiEncodeControlStopped");
					SetWindowText(GetDlgItem(hWnd, VidomiEncodeControlStatusText),text);	
					globals->translate->translate(text,"VidomiEncodeControlPauseConversion");
					SetWindowText(GetDlgItem(hWnd, VidomiEncodeControlPause),text);	
					globals->netConversion->abort(-1);
					running = false;
					break;
				case VidomiEncodeControlPause:
					if(running)
						{
						if(globals->netConversion->pause())
							{
							globals->translate->translate(text,"VidomiEncodeControlResumeConversion");
							SetWindowText(GetDlgItem(hWnd, VidomiEncodeControlPause),text);	
							globals->translate->translate(text,"VidomiEncodeControlPaused");
							SetWindowText(GetDlgItem(hWnd, VidomiEncodeControlStatusText),text);	
							}
						else
							{
							globals->translate->translate(text,"VidomiEncodeControlPauseConversion");
							SetWindowText(GetDlgItem(hWnd, VidomiEncodeControlPause),text);	
							globals->translate->translate(text,"VidomiEncodeControlRunning");
							SetWindowText(GetDlgItem(hWnd, VidomiEncodeControlStatusText),text);	
							}
						}
					break;
				case VidomiEncodeControlPriority:
					if(commandEvent==CBN_SELCHANGE)
						{
						if(running)
							{
							state = SendDlgItemMessage(hWnd,VidomiEncodeControlPriority, CB_GETCURSEL  , 0, 0);
							state = SendDlgItemMessage(hWnd,VidomiEncodeControlPriority, CB_GETITEMDATA  , state, 0);
							currentPriority = state;
							globals->netConversion->setThreadPriority(encodeThreadPriorityValue[state]);
							}
						}
					break;
				case VidomiEncodeControlSleepPerFrame:
					if(commandEvent==CBN_SELCHANGE)
						{
						if(running)
							{
							state = SendDlgItemMessage(hWnd,VidomiEncodeControlSleepPerFrame, CB_GETCURSEL  , 0, 0);
							state = SendDlgItemMessage(hWnd,VidomiEncodeControlSleepPerFrame, CB_GETITEMDATA  , state, 0);
							currentSleepPerFrame = state;
							globals->netConversion->setSleepPerFrame(state);
							}
						}
					break;

				case VidomiEncodeControlShutdown:
					if(SendDlgItemMessage(hWnd,VidomiEncodeControlShutdown,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
						onOff = true;
					else
						onOff = false;
					shutdown = onOff;
					globals->netConversion->setShutdown(onOff);
				default:
					break;
				}
			break;
		}

	return (FALSE);
}

