// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
// The start of Ncode
//
// *****************************************************************************
#include "en.h"

GLOBALS * globals;

int PASCAL WinMain(HINSTANCE hInst,HINSTANCE hPrevInst, LPSTR lpszCmd ,int iCmdShow)
{
	MSG msg;                  
	HACCEL accelTable;
	char pathToLanguageFiles[MAX_PATH];
	char pathToSlaveIpFile[MAX_PATH];

	// Now Configure the globals
	configureGlobals(hInst);

	xvidHelper();

	// Now Init the Logging mechanism
	clearLog();	//clear text based log file
	
	EnumWindows(&EnumWindowsProc, 0);
	if(globals->vidomiSwitcherhWnd)
		{
		//we found the switcher 
		globals->vidomiMode = MODE_DISABLED;
		}
	else
		{
		// we failed to find the switcher, so run in stand alone mode
		globals->vidomiMode = MODE_ENCODER;
		}


	logPrintf("Vidomi Encoder Version %.3f Started...\n",globals->version);
	logPrintf("Path to Vidomi is %s\n",globals->pathToExeDir);

	logDumpMemoryStatus("Memory on Startup");
	// Create our Classes (And set defaults in constructors)
	
	globals->translate = new translateClass();
	logPrintf("translateClass created.\n");
	
	// Give us a default language for windows and text
	strcpy(pathToLanguageFiles,globals->pathToExeDir);
	strcat(pathToLanguageFiles,"\\languages\\english.lng");
	if(!globals->translate->setDefaultLanguageFile(pathToLanguageFiles))
		{
		strcpy(pathToLanguageFiles,globals->pathToExeDir);
		strcat(pathToLanguageFiles,"\\..\\..\\installer\\languages\\english.lng");
		globals->translate->setDefaultLanguageFile(pathToLanguageFiles);
		}
	// And now load the language selected

	strcpy(pathToLanguageFiles,globals->pathToExeDir);
	strcat(pathToLanguageFiles,"\\languages\\");
	strcat(pathToLanguageFiles,globals->vidomiLanguage);
	strcat(pathToLanguageFiles,".lng");
	globals->translate->setLanguageFile(pathToLanguageFiles);

	
	globals->window = new windowClass();
	logPrintf("windowClass created.\n");
	globals->netConversion = new netConversionClass();
	logPrintf("netConversionClass created.\n");
	globals->conversion = new conversionClass();
	logPrintf("conversionClass created.\n");
	globals->outputVideo = new outputVideoClass();
	logPrintf("outputVideoClass created.\n");
	globals->scanFile = new scanFileClass();
	logPrintf("scanMpegClass created.\n");
	globals->optionsVideo = new optionsVideoClass();
	logPrintf("optionsVideoClass created.\n");
	globals->optionsAdvancedVideo = new optionsAdvancedVideoClass();
	logPrintf("optionsAdvancedVideoClass created.\n");
	globals->optionsSpecial = new optionsSpecialClass();
	logPrintf("optionsSpecialClass created.\n");
	globals->optionsAudio = new optionsAudioClass();
	logPrintf("optionsAudioClass created.\n");
	globals->optionsGeneral = new optionsGeneralClass();
	logPrintf("optionsGeneralClass created.\n");
	globals->outputStats = new outputStatsClass();
	logPrintf("optionsStatsClass created.\n");
	globals->selectRange = new selectRangeClass();
	logPrintf("selectRangeClass created.\n");
	globals->job = new jobClass();
	logPrintf("jobClass created.\n");
	globals->encodeControl = new encodeControlClass();
	logPrintf("encodeControlClass created.\n");
	globals->optionsSubtitle = new optionsSubtitleClass();
	logPrintf("optionsSubtitleClass created.\n");
	globals->optionsChapters = new optionsChaptersClass();
	logPrintf("optionsChaptersClass created.\n");
	globals->optionsOutputSize = new optionsOutputSizeClass();
	logPrintf("optionsOutputSizeClass created.\n");
	globals->optionsNetwork = new optionsNetworkClass();
	logPrintf("optionsNetworkClass created.\n");
	globals->outputAudio = new outputAudioClass();
	logPrintf("outputAudioClass created.\n");
	globals->jobServer = new jobServerClass();
	logPrintf("jobServerClass created.\n");
	globals->jobHelper = new jobHelperClass();
	logPrintf("jobHelperClass created.\n");
	globals->netHelper = new netHelperClass();
	logPrintf("netHelperClass created.\n");


	// Now Start bringing up the app
	globals->outputVideo->createOutputWindow(hInst);
	logPrintf("Output window created.\n");
	globals->outputStats->createOutputWindow(hInst);
	logPrintf("Stats Output window created.\n");
	globals->window->createWindow(hInst);
	
	globals->job->createWindow(hInst);
	logPrintf("Job window created.\n");
	globals->encodeControl->createWindow(hInst);
	logPrintf("Encode Control window created.\n");

	logPrintf("Loading Slave IP List\n");

	strcpy(pathToSlaveIpFile,globals->pathToExeDir);
	strcat(pathToSlaveIpFile,"\\encodeSlaves.txt");
	globals->netHelper->loadSlaveList(pathToSlaveIpFile);

	// Main message loop:
	accelTable = LoadAccelerators (hInst, MAKEINTRESOURCE(VidomiAccelerators));

	while(GetMessage(&msg, NULL, 0, 0))
		{
		// Only the player has Accelerators at this point
		if (!TranslateAccelerator (globals->hWnd, accelTable, &msg)) 
			{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			}
		}	
	
	DestroyAcceleratorTable(accelTable);

	logPrintf("Shutting Down.\n");

	globals->settings->saveSettingsRegistry();

	delete globals->netConversion;
	delete globals->conversion;
	globals->job->resetJobList();
	delete globals->job;
	delete globals->window;
	delete globals->scanFile;
	delete globals->optionsNetwork;
	delete globals->optionsOutputSize;
	delete globals->optionsSubtitle;
	delete globals->optionsVideo;
	delete globals->optionsAdvancedVideo;
	delete globals->optionsSpecial;
	delete globals->optionsAudio;
	delete globals->selectRange;
	//Clear the pending job list
	delete globals->outputVideo;
	delete globals->outputAudio;
	delete globals->jobServer;
	
	globals->netHelper->freeSlaveList(globals->slaveEncodeMachines);
	delete globals->netHelper;

	delete globals->jobHelper;
	delete globals->outputStats;
	delete globals->translate;
	delete globals->registry;

	delete [] globals->convertConfig->videoFormatOut->lpState;
	delete [] globals->convertConfig->videoFormatOut->lpbiOut;
	delete [] globals->convertConfig->videoFormatOut;

	delete [] globals->convertConfig->videoFormatOutP2->lpState;
	delete [] globals->convertConfig->videoFormatOutP2->lpbiOut;
	delete [] globals->convertConfig->videoFormatOutP2;

	delete [] globals->convertConfig->audioFormatOut;
	delete [] globals->errorText;
	delete [] globals;
	return(FALSE);
}


// *****************************************************************************
//  Configure the Globals and put/get from registry
// *****************************************************************************
void configureGlobals(HINSTANCE hInst)
{
	WAVEFORMATEX audioFormatIn;
	HACMSTREAM audioAcmStream;
	char * commandLine;
	char tempStr[MAX_PATH];
	char * tempPtr;
	char pathToExe[MAX_PATH];
	bool done;
	int i;
	HIC divxNetworksHic;

	// Create Global Var Struct
	globals = new GLOBALS [1];
	memset(globals,0,sizeof(GLOBALS));
	
	globals->hInst = hInst;
	globals->errorText = new char [8192];

	// Now lets have some fun and create the path to the EXE
	commandLine = GetCommandLine();
	strcpy(tempStr,commandLine);
	done=false;

	//walk through the path until we locate .exe
	tempPtr = tempStr;
	while(1)
		{
		if(!(tempPtr = strchr(tempPtr,'.')))
			{
			// if can't find .exe, then use CWD
			GetCurrentDirectory(MAX_PATH, pathToExe);
			done=true;
			break;
			}
		if((*(tempPtr+1) == 'e') && (*(tempPtr+2)=='x') && (*(tempPtr+3) == 'e'))
			{	
			*(tempPtr+4) = 0;
			break;
			}
		if((*(tempPtr+1) == 'E') && (*(tempPtr+2)=='X') && (*(tempPtr+3) == 'E'))
			{	
			*(tempPtr+3) = 0;
			break;
			}
		//Advance to the next Char
		if(tempPtr)
			{
			tempPtr++;
			}
		}

	if(!done)
		{
		// Remove leading quote if any
		if(tempStr[0] == '\"')
			{
			strcpy(pathToExe,&tempStr[1]);
			}
		else
			{
			strcpy(pathToExe,tempStr);
			}
		
		//Now wack off everything after the last "\"
		if(tempPtr = strrchr(pathToExe,'\\'))
			{
			*tempPtr = 0;
			}
		else
			{
			GetCurrentDirectory(MAX_PATH, pathToExe);
			}
		}

	//pathToExe now holds the path to the EXE dir
	strcpy(globals->pathToExeDir,pathToExe);
	// Set Default Values debug vars
	
	//GetCurrentDirectory(MAX_PATH, globals->pathToExeDir);
	strcpy(globals->optionsLogFilename,globals->pathToExeDir);
	strcat(globals->optionsLogFilename, "\\debug.txt");

	strcpy(globals->playFilename, "");
	globals->optionsLogOutput = false;
	globals->optionsUp=false;
	globals->windowTop=320;
	globals->windowLeft=240;
	globals->outputTop=320;
	globals->outputLeft=240;
	globals->statsTop=320;
	globals->statsLeft=500;
	globals->jobTop=320;
	globals->jobLeft=240;
	globals->encodeControlTop=320;
	globals->encodeControlTop=240;
	globals->version=VERSION;
	globals->contactVidomi = true;
	globals->converting = false;
	globals->dualCpuEncodeBuffers=8;

	globals->jobCount = 0;
	globals->jobIndex = 0;

	strcpy(globals->vidomiLanguage, "english");

	for(i=0;i<MAX_JOBS;i++)
		{
		globals->jobList[i].convertConfig = NULL;
		globals->jobList[i].gatherResults = NULL;
		globals->jobList[i].inputValidated = false;
		globals->jobList[i].outputSet = false;
		globals->jobList[i].cropScanDone = false;
		}

	_getcwd(globals->browseSourcePath,MAX_PATH);
	globals->browseSourceDefaultType = 2;
	_getcwd(globals->browseDestPath,MAX_PATH);
	globals->browseDestDefaultType = 2;
	_getcwd(globals->browseSettingsPath,MAX_PATH);
	globals->browseSettingsDefaultType = 2;
	_getcwd(globals->browseJobListPath,MAX_PATH);
	globals->browseJobListDefaultType = 2;

	globals->jobList[0].convertConfig = new CONVERTCONFIG [1];
	globals->jobList[0].gatherResults = new GATHERRESULTS [1];
	memset(globals->jobList[0].gatherResults,0,sizeof(GATHERRESULTS));

	globals->convertConfig = globals->jobList[0].convertConfig;
	globals->gatherResults = globals->jobList[0].gatherResults;

	strcpy(globals->convertConfig->outputFilename,"");
	for(i=0;i<MAX_INPUT_FILES;i++)
		{
		strcpy(globals->convertConfig->inputFilenames[i],"");
		}
	globals->convertConfig->inputFilenameIndex=0;

	divxNetworksHic = ICOpen(mmioFOURCC('v', 'i', 'd', 'c'), mmioFOURCC('d', 'i', 'v', 'x') ,ICMODE_COMPRESS);
	if(divxNetworksHic)
		{
		globals->divxNetworksInstalled = true;
		ICClose(divxNetworksHic);
		}
	else
		{
		globals->divxNetworksInstalled = false;
		}

	configureSmartAVDefaults(globals->convertConfig);

	globals->vidomiMode = MODE_ENCODER;

	// We only support one skin right now
	strcpy(globals->currentSkin,globals->pathToExeDir);
	strcat(globals->currentSkin,"\\skins\\defaultskin.vsk");

	// Set the server to not available and no IP defined
	globals->encodeServerEnabled=false;
	strcpy(globals->encodeServerIPAddress,"");


	globals->registry = new registryClass();
	globals->settings = new settingsClass();

	// Now get the values that are currently set
	// Insert our default values if not found
	globals->settings->loadSettingsRegistry();

	globals->convertConfig->videoFormatOut->hic = ICOpen(globals->convertConfig->videoFormatOut->fccType, globals->convertConfig->videoFormatOut->fccHandler,ICMODE_COMPRESS);

	// Check that the defaults really work.

	if(!globals->convertConfig->videoFormatOut)
		{
		char messTitle[4096];
		char messText[4096];
		globals->translate->translate(messText, "VidomiMessage16");
		globals->translate->translate(messTitle, "VidomiMessage16Title");

		vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);				
		globals->convertConfig->videoFormatOut->hic = 0;
		}
	else
		{
		if(globals->convertConfig->videoFormatOut->hic)
			{
			//globals->convertConfig->videoFormatOut->hic->unused = 0;
			ICClose(globals->convertConfig->videoFormatOut->hic);
			globals->convertConfig->videoFormatOut->hic = 0;
			}
		}
	
	if(!globals->convertConfig->lameFound)
		{
		// Try opening an acm stream that matches the codec selected
		audioFormatIn.nSamplesPerSec=globals->convertConfig->audioFormatOut->wfx.nSamplesPerSec;
		audioFormatIn.nAvgBytesPerSec= audioFormatIn.nSamplesPerSec*2*2;
		if(acmStreamOpen(&audioAcmStream,NULL,&audioFormatIn,(WAVEFORMATEX *)globals->convertConfig->audioFormatOut,NULL,NULL,0,ACM_STREAMOPENF_NONREALTIME))
			{
			char messTitle[4096];
			char messText[4096];
			globals->translate->translate(messText, "VidomiMessage17");
			globals->translate->translate(messTitle, "VidomiMessage17Title");

			vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);				
			}
		else
			{
			acmStreamClose(audioAcmStream,0);
			}
		}

	strcpy(tempStr,globals->pathToExeDir);
	strcat(tempStr,"\\externalcodec.cfg");
	
	// Loadup the external codec settings
	if(!globals->settings->loadCodecCfgFile(tempStr))
		{
		strcpy(tempStr,globals->pathToExeDir);
		strcat(tempStr,"\\..\\..\\installer\\externalcodec.cfg");
		globals->settings->loadCodecCfgFile(tempStr);
		}

}


// *****************************************************************************
//  Configure the globals to have smart defaults
// *****************************************************************************
void configureSmartAVDefaults(CONVERTCONFIG * convertConfig)
{
	int channel;
	int i;

	WAVEFORMATEX audioFormatIn;
	HACMSTREAM audioAcmStream;
	ADPCMWAVEFORMAT * adpcm;
	mp3EncodeClass * mp3Encode;

	convertConfig->hyperCrop=true;
	convertConfig->hyperCropThreshold=15;
	convertConfig->cropTop=0;
	convertConfig->cropBottom=0;
	convertConfig->cropLeft=0;
	convertConfig->cropRight=0;
	convertConfig->scale=false;
	convertConfig->hyperScale=true;
	convertConfig->scaleBy=SCALE_BY_PERCENTAGE;
	convertConfig->scaleType=3;
	convertConfig->scaleWidth=100;
	convertConfig->scaleHeight=100;
	convertConfig->deinterlace=true;
	convertConfig->blend=false;
	convertConfig->blendThreshold=20;
	convertConfig->film=false;
	convertConfig->forceMultiple16 = true;
	convertConfig->processAudio = true;
	convertConfig->maxOutputSize=649;
	convertConfig->resample48KhzAudio = false;
	convertConfig->mp3=true;
	for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
		{
		convertConfig->processAudioSelected[channel]=-1;
		convertConfig->mp3MinBitrate[channel]=128;
		convertConfig->mp3MaxBitrate[channel]=128;
		convertConfig->mp3EncodeMode[channel]=LAME_CBR;
		convertConfig->mp3ChannelType[channel]=LAME_DUALCHANNEL;
		convertConfig->mp3Quality[channel]=LAME_QUALITY_NORMAL;
		convertConfig->mp3OutputFrequency[channel]=0;
		convertConfig->mp3VbrQuality[channel]=4;
		convertConfig->audioDirectStreamCopy[channel]=false;
		strcpy(convertConfig->audioStreamName[channel],"");
		}

	convertConfig->processSubtitles = false;
	for(channel=0;channel<MAX_SUBTITLE_CHANNELS;channel++)
		{
		convertConfig->processSubtitleMode[channel]=-1;
		convertConfig->processSubtitleSelected[channel]=0;
		convertConfig->processSubtitleOnlyEven[channel]=divxSubtitleTypeGraphicFull;
		strcpy(convertConfig->subtitleStreamName[channel],"");
		}

	convertConfig->subtitleColorData[0] = 0xFF000000;
	convertConfig->subtitleColorData[1] = 0x00FFFFFF;
	convertConfig->subtitleColorData[2] = 0x00FF0000;
	convertConfig->subtitleColorData[3] = 0x00808080;

	strcpy(convertConfig->videoStreamName,"");
	convertConfig->lameFound=false;

	convertConfig->vbrNumberOfDiscs = 1;
	convertConfig->vbrReserve = 2;
	convertConfig->vbrAudioBitrate = 128;
	convertConfig->vbrDeletePassOneFiles = true;
	convertConfig->normalizeAudio = NORMALIZE_NONE;
	convertConfig->normalizeLevel = 98;
	convertConfig->encodeMode = ENCODE_MODE_CBR;
	convertConfig->showOutput = false;
	convertConfig->showStats = true;
	convertConfig->vbrCrispness = 100;
	convertConfig->vbrMinKeyFrame = 100;
	convertConfig->vbrMaxKeyFrame = 300;
	convertConfig->vbrMinBitrate = 300;
	convertConfig->vbrMaxBitrate = 6000;
	convertConfig->vbrMinReservoir = 30;
	convertConfig->vbrMaxReservoir = 100;
	convertConfig->vbrQualityBitrate = 1500;
	convertConfig->multiThread = true;
	convertConfig->introAvi = false;
	strcpy(&convertConfig->outputTail[0],".divx");
	convertConfig->scanLength = 5;
	convertConfig->useInfoTxtFile = true;
	convertConfig->lumaProcessing = false;
	convertConfig->lumaLowProcessing = true;
	convertConfig->lumaDeltaProcessing = true;
	convertConfig->lumaLowThresh = 0.20f;
	convertConfig->lumaLowBoost = 50;
	convertConfig->lumaDeltaThresh = 0.02f;
	convertConfig->lumaDeltaBoost = 50;
	convertConfig->startingFilePos = 0;
	convertConfig->startingFileFrame = 0;
	convertConfig->endingFilePos = 0x7fffffffffffffff;
	convertConfig->endingFileFrame = 0;
	convertConfig->useSpecifiedTarget=false;
	convertConfig->specifiedTarget=698;
	convertConfig->aspectRatioOverride=false;
	convertConfig->aspectRatioOverrideValue=1.0f;
	convertConfig->ivtc=true;
	convertConfig->ivtcDeComb = true; //false = blurred fields / true = comb
	convertConfig->ivtcOffset = -1;
	convertConfig->ivtcPolarity = false;
	convertConfig->overwriteExistingFiles=false;
	convertConfig->scanPassScale=true;
	convertConfig->defaultOutputSize=0;	// 0 = FULL SCREEN
	convertConfig->stopAfterPassOne = false;
	convertConfig->mp3LimitToCbr=true;
	convertConfig->mp3LimitToMpeg1=true;
	convertConfig->aspectRatioDontCorrect43=false;
	convertConfig->encodeColorSpace = yv12Frame;
	
	convertConfig->enhancedAvi= true;
	convertConfig->insertChapters = true;
	convertConfig->numChapters =0;
	convertConfig->splitAtKeyframe=false;

	convertConfig->xvidEncodeModeOffset = 0;
	convertConfig->xvidBitrateOffset = 4;
	convertConfig->xvidDesiredSizeOffset = 184;
	convertConfig->xvidStatsFilenameOffset = 220;

	convertConfig->divx4VbrModeOffset=32;
	convertConfig->divx4BitrateOffset=0;
	convertConfig->divx5BitrateOffset=4;
	convertConfig->divx5VbrModeOffset=36;

	for(i=0;i<MAX_RANGES;i++)
		{
		convertConfig->rangelist[i].valid=false;
		convertConfig->rangelist[i].startingFilePos = 0;
		convertConfig->rangelist[i].startingFileFrame = 0;
		convertConfig->rangelist[i].endingFilePos = 0x7fffffffffffffff;
		convertConfig->rangelist[i].endingFileFrame=0;
		}

	// Guarantee that the default state is one valid range
	globals->jobList[globals->jobCount].convertConfig->rangelist[0].startingFilePos = 0;
	globals->jobList[globals->jobCount].convertConfig->rangelist[0].startingFileFrame=0;
	globals->jobList[globals->jobCount].convertConfig->rangelist[0].endingFilePos=0x7fffffffffffffff;
	globals->jobList[globals->jobCount].convertConfig->rangelist[0].endingFileFrame=0;
	globals->jobList[globals->jobCount].convertConfig->rangelist[0].valid=true;

	convertConfig->hyperEncode=true;
	convertConfig->mpegConversion = false;

	strcpy(convertConfig->introAviPrimaryPath,globals->pathToExeDir);
	strcpy(convertConfig->introAviSecondaryPath,globals->pathToExeDir);

	convertConfig->audioFormatOut = (MPEGLAYER3WAVEFORMAT *) new char [AUDIO_STATE_DEFAULT_SIZE];
	convertConfig->videoFormatOut = (COMPVARS * )new char [sizeof(COMPVARS)];

	memset(convertConfig->audioFormatOut,0,AUDIO_STATE_DEFAULT_SIZE);
	memset(convertConfig->videoFormatOut,0,sizeof(COMPVARS));
	// Configure initial settings for both audio and video
	
	// Divex Low Motion
	convertConfig->videoFormatOut->cbSize = sizeof(COMPVARS);
	convertConfig->videoFormatOut->lpbiOut = (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
	convertConfig->videoFormatOut->fccType = 0x63646976;
	convertConfig->videoFormatOut->dwFlags = ICMF_COMPVARS_VALID;
	convertConfig->videoFormatOut->lDataRate = 0x12c; //Favor Divx 
	convertConfig->videoFormatOut->lKey = 0x270f; //favor Divx 
	convertConfig->videoFormatOut->lQ = ICQUALITY_DEFAULT; //0x2710;
	convertConfig->videoFormatOut->lpState = new char [VIDEO_STATE_DEFAULT_SIZE]; // allow for some growth in state size. 
	memset(convertConfig->videoFormatOut->lpState,0,VIDEO_STATE_DEFAULT_SIZE);

	//Setup all the generic stuff
	convertConfig->videoFormatOut->lpbiOut->bmiHeader.biSize = sizeof(BITMAPINFO);
	convertConfig->videoFormatOut->lpbiOut->bmiHeader.biWidth = 720;
	convertConfig->videoFormatOut->lpbiOut->bmiHeader.biHeight = 480;
	convertConfig->videoFormatOut->lpbiOut->bmiHeader.biPlanes = 0x0001;
	convertConfig->videoFormatOut->lpbiOut->bmiHeader.biBitCount = 0x0018;
	convertConfig->videoFormatOut->lpbiOut->bmiHeader.biSizeImage = 720 * 480 * 3;
	convertConfig->videoFormatOut->lpbiOut->bmiHeader.biXPelsPerMeter = 0;
	convertConfig->videoFormatOut->lpbiOut->bmiHeader.biYPelsPerMeter = 0;
	convertConfig->videoFormatOut->lpbiOut->bmiHeader.biClrImportant = 0;
	convertConfig->videoFormatOut->lpbiOut->bmiHeader.biClrUsed = 0;


	convertConfig->videoFormatOut->fccHandler = 0x34766964;
	convertConfig->videoFormatOut->hic = ICOpen(0x63646976, 0x34766964,ICMODE_COMPRESS);
	if(convertConfig->videoFormatOut->hic)
		{
		// We have Divx Low Motion
		convertConfig->videoFormatOut->lpbiOut->bmiHeader.biCompression = 0x34564944;
		}
	else
		{
		// Try Divx Fast Motion
		convertConfig->videoFormatOut->fccHandler  = 0x33766964;
		convertConfig->videoFormatOut->hic = ICOpen(0x63646976, 0x33766964,ICMODE_COMPRESS);
		if(convertConfig->videoFormatOut->hic)
			{
			// We have Divx Fast Motion
			convertConfig->videoFormatOut->lpbiOut->bmiHeader.biCompression = 0x33564944;
			}
		else
			{
			// Try Divx Project Mayo Codec
			convertConfig->videoFormatOut->fccHandler  = 0x78766964;
			convertConfig->videoFormatOut->hic = ICOpen(0x63646976, 0x78766964,ICMODE_COMPRESS);
			if(convertConfig->videoFormatOut->hic)
				{
				// We have Project Mayo
				convertConfig->videoFormatOut->lpbiOut->bmiHeader.biCompression = 0x58564944;
				}
			else
				{
				
				// Try Cinepak as our default

				// Set these important fields to more typical defaults
				convertConfig->videoFormatOut->lDataRate = 10000;
				convertConfig->videoFormatOut->lKey = 30;
				convertConfig->videoFormatOut->lQ = ICQUALITY_DEFAULT; 

				convertConfig->videoFormatOut->fccHandler = 0x64697663;
				convertConfig->videoFormatOut->hic = ICOpen(0x63646976, 0x64697663,ICMODE_COMPRESS);
				if(convertConfig->videoFormatOut->hic)
					{
					// We have Cinepak
					convertConfig->videoFormatOut->lpbiOut->bmiHeader.biCompression = 0x64697663;
					}
				else
					{
					// Set RAW FULL FRAME Output
					convertConfig->videoFormatOut->hic = ICOpen(0x63646976, 0x20424944,ICMODE_COMPRESS);
					convertConfig->videoFormatOut->fccHandler  = 0x20424944;
					convertConfig->videoFormatOut->lpbiOut->bmiHeader.biCompression = 0;
					convertConfig->videoFormatOut->lKey = 0;
					convertConfig->videoFormatOut->lQ = 0;
					}

				}
			}
		}

	// We now have a valid default state
	convertConfig->videoFormatOut->cbState = ICGetStateSize(convertConfig->videoFormatOut->hic);
	ICGetState(convertConfig->videoFormatOut->hic,convertConfig->videoFormatOut->lpState,convertConfig->videoFormatOut->cbState);
	if(convertConfig->videoFormatOut->hic)
		{
		//convertConfig->videoFormatOut->hic->unused = 0;
		ICClose(convertConfig->videoFormatOut->hic);
		convertConfig->videoFormatOut->hic = NULL;
		}

	// Now Configure the default Audio State
	mp3Encode = new mp3EncodeClass();
	

	if(mp3Encode->init(0))
		{
		convertConfig->lameFound=true;
		// We have Lame!!
		mp3Encode->end();
		// Configure raw PCM as the codec default
		convertConfig->audioFormatOut->wfx.wFormatTag=WAVE_FORMAT_PCM;
		convertConfig->audioFormatOut->wfx.nChannels = 2;
		convertConfig->audioFormatOut->wfx.nSamplesPerSec=48000;
		convertConfig->audioFormatOut->wfx.nAvgBytesPerSec= 48000*2*2;
		convertConfig->audioFormatOut->wfx.nBlockAlign=2*2;
		convertConfig->audioFormatOut->wfx.wBitsPerSample=16;
		convertConfig->audioFormatOut->wfx.cbSize=0;
		convertConfig->resample48KhzAudio = false;
		}
	else
		{
		// Try opening an acm MP3 stream
		audioFormatIn.wFormatTag=WAVE_FORMAT_PCM;
		audioFormatIn.nChannels = 2;
		audioFormatIn.nSamplesPerSec=44100;
		audioFormatIn.nAvgBytesPerSec= 44100*2*2;
		audioFormatIn.nBlockAlign=2*2;
		audioFormatIn.wBitsPerSample=16;
		audioFormatIn.cbSize=0;

		convertConfig->audioFormatOut->wfx.wFormatTag = WAVE_FORMAT_MPEGLAYER3;
		convertConfig->audioFormatOut->wfx.nChannels =2;
		convertConfig->audioFormatOut->wfx.nSamplesPerSec = 44100;
		convertConfig->audioFormatOut->wfx.nAvgBytesPerSec = 128*1000/8;
		convertConfig->audioFormatOut->wfx.nBlockAlign = 1;
		convertConfig->audioFormatOut->wfx.wBitsPerSample = 0;
		convertConfig->audioFormatOut->wfx.cbSize = MPEGLAYER3_WFX_EXTRA_BYTES;
		convertConfig->audioFormatOut->wID = MPEGLAYER3_ID_MPEG;
		convertConfig->audioFormatOut->fdwFlags = MPEGLAYER3_FLAG_PADDING_OFF;
		convertConfig->audioFormatOut->nBlockSize = 0x0180;							
		convertConfig->audioFormatOut->nFramesPerBlock = 1;	//0x0100
		convertConfig->audioFormatOut->nCodecDelay = 0x0571;
		
		if(!acmStreamOpen(&audioAcmStream,NULL,&audioFormatIn,(WAVEFORMATEX *)convertConfig->audioFormatOut,NULL,NULL,0,ACM_STREAMOPENF_NONREALTIME))
			{
			// MP3 Encoder Capable of 48Khz @ 128 Kbps Available, so set as our default
			convertConfig->resample48KhzAudio = true;	
			acmStreamClose(audioAcmStream,0);
			}
		else
			{
			// Try 44.1 @ 128 

			// Set the input wav to 44.1Khz 
			audioFormatIn.nSamplesPerSec=44100;
			audioFormatIn.nAvgBytesPerSec= 44100*2*2;

			convertConfig->audioFormatOut->wfx.nSamplesPerSec = 44100;
			convertConfig->audioFormatOut->wfx.nAvgBytesPerSec = 128*1000/8;
			if(!acmStreamOpen(&audioAcmStream,NULL,&audioFormatIn,(WAVEFORMATEX *)convertConfig->audioFormatOut,NULL,NULL,0,ACM_STREAMOPENF_NONREALTIME))
				{
				// MP3 Encoder Capable of 44.1Khz @ 128Kbps Available. 
				convertConfig->resample48KhzAudio = true;
				acmStreamClose(audioAcmStream,0);

				}
			else
				{
				// Try 44.1 @ 128 
				convertConfig->audioFormatOut->wfx.nSamplesPerSec = 44100;
				convertConfig->audioFormatOut->wfx.nAvgBytesPerSec = 56*1000/8;
				if(!acmStreamOpen(&audioAcmStream,NULL,&audioFormatIn,(WAVEFORMATEX *)convertConfig->audioFormatOut,NULL,NULL,0,ACM_STREAMOPENF_NONREALTIME))
					{
					// MP3 Encoder Capable of 44.1Khz @ 56Kbps Available. 
					convertConfig->resample48KhzAudio = true;
					acmStreamClose(audioAcmStream,0);
					}
				else
					{					
					// Make life easy
					adpcm = (ADPCMWAVEFORMAT *) convertConfig->audioFormatOut;
					// Now build up the initial ADPCM state
					adpcm->wfx.wFormatTag=WAVE_FORMAT_ADPCM;
					adpcm->wfx.nSamplesPerSec=44100;
					adpcm->wfx.nAvgBytesPerSec=0xad47;
					adpcm->wfx.wBitsPerSample=4;
					adpcm->wfx.nBlockAlign=0x0800;
					adpcm->wfx.cbSize=32;
					adpcm->wSamplesPerBlock = 0x07f4;
					adpcm->wNumCoef = 0x0007;
					adpcm->aCoef[0].iCoef1 = (short)0x0100;
					adpcm->aCoef[0].iCoef2 = (short)0x0000;
					adpcm->aCoef[1].iCoef1 = (short)0x0200;
					adpcm->aCoef[1].iCoef2 = (short)0xff00;
					adpcm->aCoef[2].iCoef1 = (short)0x0000;
					adpcm->aCoef[2].iCoef2 = (short)0x0000;
					adpcm->aCoef[3].iCoef1 = (short)0x00c0;
					adpcm->aCoef[3].iCoef2 = (short)0x0040;
					adpcm->aCoef[4].iCoef1 = (short)0x00f0;
					adpcm->aCoef[4].iCoef2 = (short)0x0000;
					adpcm->aCoef[5].iCoef1 = (short)0x01cc;
					adpcm->aCoef[5].iCoef2 = (short)0xff30;
					adpcm->aCoef[6].iCoef1 = (short)0x0188;
					adpcm->aCoef[6].iCoef2 = (short)0xff18;
				
					if(!acmStreamOpen(&audioAcmStream,NULL,&audioFormatIn,(WAVEFORMATEX *)convertConfig->audioFormatOut,NULL,NULL,0,ACM_STREAMOPENF_NONREALTIME))
						{
						// ADPCM Encoder Capable of 44.1Khz 
						convertConfig->resample48KhzAudio = true;
						acmStreamClose(audioAcmStream,0);
						}
					else
						{					
						// PCM for You!
						convertConfig->resample48KhzAudio = true;
						convertConfig->audioFormatOut->wfx.wFormatTag=WAVE_FORMAT_PCM;
						convertConfig->audioFormatOut->wfx.nChannels = 2;
						convertConfig->audioFormatOut->wfx.nSamplesPerSec=44100;
						convertConfig->audioFormatOut->wfx.nAvgBytesPerSec= 44100*2*2;
						convertConfig->audioFormatOut->wfx.nBlockAlign=2*2;
						convertConfig->audioFormatOut->wfx.wBitsPerSample=16;
						convertConfig->audioFormatOut->wfx.cbSize=0;					
						}

					}
				}
			}
		}
	delete mp3Encode;

	// Now fill in the second struct using the first we created

	convertConfig->videoFormatOutP2 = (COMPVARS * )new char [sizeof(COMPVARS)];
	memcpy(convertConfig->videoFormatOutP2,	convertConfig->videoFormatOut,sizeof(COMPVARS));
	convertConfig->videoFormatOutP2->lpState = new char [VIDEO_STATE_DEFAULT_SIZE]; // allow for some growth in state size. 
	memcpy(convertConfig->videoFormatOutP2->lpState,convertConfig->videoFormatOut->lpState,VIDEO_STATE_DEFAULT_SIZE);
	convertConfig->videoFormatOutP2->lpbiOut = (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
	memcpy(convertConfig->videoFormatOutP2->lpbiOut,convertConfig->videoFormatOut->lpbiOut,VIDOMI_BITMAP_INFO_SIZE);

}

// *****************************************************************************
//  Find the VidomiSwitcher hwnd
// *****************************************************************************

BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam)
{
char windowText[1024];

	GetWindowText(hwnd,&windowText[0],1024); 
	
	if(!strcmp(&windowText[0],"VidomiSwitcher"))
		{
		//found it
		globals->vidomiSwitcherhWnd = hwnd;
		return(FALSE);
		}
	return(TRUE);
}

// *****************************************************************************
//  vidomiErrorBox
// *****************************************************************************
int vidomiErrorBox(HWND hWnd,char * text, char * caption, unsigned int type)
{
	logPrintf("MessageBox: ");
	logPrintf(text);
	logPrintf("\n");
	return(MessageBox(hWnd,text,caption,type));
}

// *****************************************************************************
//  vidomiErrorBox
// *****************************************************************************
void xvidHelper()
{
	XVIDSTATEUNUSED * xvid;
	unsigned int offset;
	unsigned int base;

	char outputText[1024];

	xvid = new XVIDSTATEUNUSED [1];
	base = (unsigned int) xvid;

	offset = ((unsigned int) &xvid->mode) - base;
	sprintf(outputText,"xvidEncodeModeOffset=%d\n",offset);
	OutputDebugString(outputText);

	offset = ((unsigned int) &xvid->rc_bitrate) - base;
	sprintf(outputText,"xvidBitrateOffset=%d\n",offset);
	OutputDebugString(outputText);

	offset = ((unsigned int) &xvid->desired_size) - base;
	sprintf(outputText,"xvidDesiredSizeOffset=%d\n",offset);
	OutputDebugString(outputText);

	offset = ((unsigned int) &xvid->stats1) - base;
	sprintf(outputText,"xvidStatsfilenameOffset=%d\n",offset);
	OutputDebugString(outputText);

	delete [] xvid;

}

