// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// encoderWindow.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

static TRANSLATEIDTABLE encoderWindowTID[] = {
										{"VidomiMainText1",VidomiMainText1},
										{"VidomiMainText2",VidomiMainText2},
										{"VidomiMainText3",VidomiMainText3},
										{"VidomiMainInputAdd",VidomiMainInputAdd},
										{"VidomiMainClearInputList",VidomiMainClearInputList},
										{"VidomiMainOutputBrowse",VidomiMainOutputBrowse},
										{"VidomiMainStartConversion",VidomiMainStartConversion},
										{"",0},
																			
										};

// *****************************************************************************
// encoderWindowClass::encoderWindowClass()
//
// window class constructor
// *****************************************************************************
encoderWindowClass::encoderWindowClass()
{
	dialoghWnd = NULL;

	converting=false;
	comboStr = new char [256];
	ignoreUpdate = false;
}

// *****************************************************************************
// encoderWindowClass::~encoderWindowClass()
//
// window class destructor
// *****************************************************************************
encoderWindowClass::~encoderWindowClass()
{
	if(dialoghWnd)
		DestroyWindow(dialoghWnd);

}

// *****************************************************************************
// encoderWindowClass::createWindow()
//
//
// *****************************************************************************
HWND encoderWindowClass::createWindow(HINSTANCE hInst)
{
	//HANDLE bitmap;

	
	dialoghWnd = CreateDialog(hInst,MAKEINTRESOURCE(VidomiMain),globals->hWnd, windowDialogHandler);

	if (!dialoghWnd)
		{
		logPrintf("Failed To created Encoder Window\n");
        return(FALSE);
		}

	ShowWindow(dialoghWnd, SW_HIDE);

	configureWindow();

	globals->jobList[globals->jobIndex].outputSet = false;
	
	DragAcceptFiles (dialoghWnd, TRUE); 

	return(dialoghWnd);
}

// *****************************************************************************
// encoderWindowClass::showWindow()
//
// 
// *****************************************************************************
bool encoderWindowClass::showWindow()
{
	ShowWindow(dialoghWnd, SW_SHOW);	

	return(true);
}

// *****************************************************************************
// encoderWindowClass::hideWindow()
//
// 
// *****************************************************************************
bool encoderWindowClass::hideWindow()
{
	ShowWindow(dialoghWnd, SW_HIDE);	

	return(true);
}

// *****************************************************************************
// encoderWindowClass::configureWindow()
//
//
// *****************************************************************************
void encoderWindowClass::configureWindow()
{
	int i;
	char text[4096];

	for(i=0;;i++)
		{
		if(!encoderWindowTID[i].id)
			break;
		globals->translate->translate(text, encoderWindowTID[i].textId);
		SetWindowText(GetDlgItem(dialoghWnd, encoderWindowTID[i].id),text);			
		}

	SendDlgItemMessage(dialoghWnd,VidomiMainInputList, LB_RESETCONTENT  , (WPARAM)0, (LPARAM) 0); 									

	//SetWindowText(GetDlgItem(globals->dialoghWnd, VidomiMainInputName),inputFilename);
	SetWindowText(GetDlgItem(dialoghWnd, VidomiMainOutputName),globals->convertConfig->outputFilename);
}

// *****************************************************************************
// encoderWindowClass::readWindowVars()
//
//
// *****************************************************************************
void encoderWindowClass::readWindowVars(HWND hWnd)
{

	GetWindowText(GetDlgItem(hWnd, VidomiMainOutputName), globals->convertConfig->outputFilename , MAX_PATH);

}



// *****************************************************************************
// encoderWindowClass::windowDialogHandler()
//
//
// *****************************************************************************
BOOL CALLBACK encoderWindowClass::windowDialogHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->window->encoderWindow->windowDialogHandler1(hWnd,message,wParam,lParam));
}

// *****************************************************************************
// encoderWindowClass::windowDialogHandler()
//
//
// *****************************************************************************
bool encoderWindowClass::windowDialogHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	short commandId;
	short commandEvent;
	char filename[MAX_PATH];
	char ending[5];
	ULARGE_INTEGER freeBytesAvailableToCaller;
	ULARGE_INTEGER totalNumberOfBytes;
	ULARGE_INTEGER totalNumberOfFreeBytes;
	char tempName[MAX_PATH];
	unsigned __int64 spaceRequired;
	int ret;
	int i, j;
	char path[MAX_PATH];
	OPENFILENAME ofn;
	char text[4096];

	switch(message)
		{
		case WM_INITDIALOG :
			return(TRUE);


		case WM_COMMAND:
            
			commandId    = LOWORD(wParam); // Remember, these are...
			commandEvent = HIWORD(wParam); // ...different for Win32!
			//Parse the menu selections:
			switch (commandId) 
				{



				case VidomiMainOutputName:
						// set ignore update to Prevent an infinite update loop
					readWindowVars(hWnd);
					globals->jobList[globals->jobIndex].outputSet=true;
					if(globals->jobList[globals->jobIndex].inputValidated)
						EnableWindow(GetDlgItem(dialoghWnd,VidomiMainStartConversion), TRUE);
					globals->job->updateJobWindow();
					break;

				case VidomiMainAudioTrack :
					if(commandEvent==CBN_SELCHANGE)
						{
						ret = SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_GETCURSEL  , 0, 0);
						ret = SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_GETITEMDATA , ret, 0);				
						globals->convertConfig->processAudioSelected[0] = ret;
						}
					break;

				case VidomiMainStartConversion:
					if(converting==false)
						{
						readWindowVars(hWnd);						
						if(globals->jobList[0].outputSet)
							{
							strcpy(tempName,globals->jobList[0].convertConfig->outputFilename);
							ptr = strrchr(tempName,'\\');
							if(ptr)
								{
								*ptr =0;
								if(strcmp(tempName,globals->browseDestPath))
									{
									strcpy(globals->browseDestPath,tempName);
									}
								}
							spaceRequired = (unsigned __int64)(1024 * 1024 * 1024);
							spaceRequired *=2;

							GetDiskFreeSpaceEx(tempName, &freeBytesAvailableToCaller, &totalNumberOfBytes, &totalNumberOfFreeBytes);
							if(freeBytesAvailableToCaller.QuadPart < spaceRequired)
								{
								char messTitle[4096];
								char messText[4096];
								globals->translate->translate(messText, "VidomiMessage18");
								globals->translate->translate(messTitle, "VidomiMessage18Title");


								if(vidomiErrorBox(globals->hWnd,messText, messTitle, MB_ICONWARNING | MB_YESNO | MB_APPLMODAL )==IDNO)
									{
									break;
									}
								}
							}
						converting=true;
						globals->converting = true;

						if(globals->jobList[globals->jobCount].outputSet && globals->jobList[globals->jobCount].inputValidated)
							{
							globals->jobIndex = globals->jobCount;
							globals->job->addToJobList();
							globals->job->setJobIndex(globals->jobIndex);
							}
						globals->job->hideWindow();
						globals->encodeControl->resetWindowState();
						globals->netConversion->startNetConversion();
						
						globals->translate->translate(text, "VidomiMainStopConversion");
						SetWindowText(GetDlgItem(dialoghWnd, VidomiMainStartConversion), text);
						disableEncodeOptionWindows();
						}					
					else
						{						
						globals->netConversion->abort(-1);
						}

					break;

				default:
					break;

				}
			break;

		}

	return (false);

}





