// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// encodeVideo.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

// *****************************************************************************
// encodeVideoClass::encodeVideoClass()
//
// conversion class constructor
// *****************************************************************************
encodeVideoClass::encodeVideoClass()
{
	frameOut = new FRAMEDATA [1];
	memset(frameOut,0,sizeof(FRAMEDATA));
	frameTemp = new FRAMEDATA [1];
	memset(frameTemp,0,sizeof(FRAMEDATA));
	bitmapFormatOut = (BITMAPINFOHEADER *) new char [VIDOMI_BITMAP_INFO_SIZE];

}

// *****************************************************************************
// encodeVideoClass::~encodeVideoClass()
//
// conversion class destructor
// *****************************************************************************
encodeVideoClass::~encodeVideoClass()
{
	delete [] frameOut;
	delete [] frameTemp;
	delete [] bitmapFormatOut;

}

// *****************************************************************************
// encodeVideoClass::init()
//
// *****************************************************************************
bool encodeVideoClass::init(CONVERTCONFIG * settings)
{
	
	xvidEncodeModeOffset = settings->xvidEncodeModeOffset;
	xvidBitrateOffset = settings->xvidBitrateOffset;
	xvidDesiredSizeOffset = settings->xvidDesiredSizeOffset;
	xvidStatsFilenameOffset = settings->xvidStatsFilenameOffset;

	divx4VbrModeOffset = settings->divx4VbrModeOffset;
	divx4BitrateOffset = settings->divx4BitrateOffset;
	divx5BitrateOffset = settings->divx5BitrateOffset;
	divx5VbrModeOffset = settings->divx5VbrModeOffset;

	convertTo24Bpp = false;

	return(true);

}

// *****************************************************************************
// encodeVideoClass::setVideoInputFormat()
//
// width = Width of bitmaps that wll be fed to writeavi
// height = Height of bitmaps that will be fed to writeavi
// bpp = Bits per Pixel. Must be divisible by 8 (IE 24 or 32)
// framerate = framerate
// *****************************************************************************
bool encodeVideoClass::setVideoInputFormat(int width,int height, float framerate, int colorSpace)
{
	memset(&bitmapFormatIn,0,sizeof(BITMAPINFOHEADER));
	bitmapFormatIn.biSize=sizeof(BITMAPINFOHEADER);
	bitmapFormatIn.biWidth=width;
	bitmapFormatIn.biHeight=height;
	bitmapFormatIn.biPlanes=1;
	bitmapFormatIn.biXPelsPerMeter=0;
	bitmapFormatIn.biYPelsPerMeter=0;
	bitmapFormatIn.biClrUsed=0;
	bitmapFormatIn.biClrImportant=0;

	switch(colorSpace)
		{
		
		case rgb32Frame:
			videoBpp = 32;
			bitmapFormatIn.biBitCount=videoBpp;
			bitmapFormatIn.biCompression=BI_RGB;
			bitmapFormatIn.biSizeImage=(width*height)*(videoBpp/8);
			break;

		case yuy2Frame:
			videoBpp = 16;
			bitmapFormatIn.biBitCount=videoBpp;
			bitmapFormatIn.biCompression=mmioFOURCC('Y','U','Y','2');
			bitmapFormatIn.biSizeImage=(width*height)*(videoBpp/8);
			break;
		case yv12Frame:
			videoBpp = 16;
			bitmapFormatIn.biBitCount=videoBpp;
			bitmapFormatIn.biCompression=mmioFOURCC('Y','V','1','2');
			bitmapFormatIn.biPlanes=1;
			bitmapFormatIn.biSizeImage=(width*height) + (((width/2)*(height/2))*2);
			break;
		}


	videoColorSpace = colorSpace;
	videoFramerate = framerate;
	videoWidth = width;
	videoHeight = height;
	videoInputSet = true;

	return (true);

}

// *****************************************************************************
// encodeVideoClass::setVideoOutputFormatCbr()
// videoDefault = the Video codec settings from the registry
// *****************************************************************************
bool encodeVideoClass::setVideoOutputFormatCbr(COMPVARS * videoDefault)
{
	LRESULT lres;

	vbrMode = false;
	divxNetworksMode = false;
	keyframeCodecRestartMode = false;
	xvidMode = false;

	memcpy(&writeVideoSettings, videoDefault, sizeof(COMPVARS));
	writeVideoSettings.lpbiOut = (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
	memcpy(writeVideoSettings.lpbiOut, videoDefault->lpbiOut, VIDOMI_BITMAP_INFO_SIZE);

	writeVideoSettings.lpbiOut->bmiHeader.biWidth = videoWidth;
	writeVideoSettings.lpbiOut->bmiHeader.biHeight = videoHeight;
	writeVideoSettings.lpbiOut->bmiHeader.biSizeImage = videoWidth * videoHeight * (videoBpp/8);
	writeVideoSettings.lpState = new char [videoDefault->cbState];
	memcpy(writeVideoSettings.lpState, videoDefault->lpState, videoDefault->cbState);

	// If compressor open, close it
	if(writeVideoSettings.hic)
		{
		//writeVideoSettings.hic->unused = 0;
		ICClose(writeVideoSettings.hic);
		writeVideoSettings.hic = NULL;
		}

	// Now open it
	if(writeVideoSettings.lpbiOut->bmiHeader.biCompression==BI_RGB)
		{
		uncompressed = true;
		}
	else
		{
		uncompressed = false;
		writeVideoSettings.hic = ICOpen(writeVideoSettings.fccType, writeVideoSettings.fccHandler ,ICMODE_COMPRESS);
		if(!writeVideoSettings.hic)
			{
			return(false);
			}
		}

	// Now test the codec to see if we need to convert the input data to 24BPP
	// A failure here isn't considered a "failure". A sucess is a sign of a 24Bpp conversion being needed
	// This check is needed to make HuffYUV work. 

	if(bitmapFormatIn.biCompression==BI_RGB)
		{
		lres = ICCompressGetFormat(writeVideoSettings.hic,&bitmapFormatIn,bitmapFormatOut);
		if (lres != ICERR_OK)
			{
			bitmapFormatIn.biBitCount=24;
			bitmapFormatIn.biSizeImage=(bitmapFormatIn.biWidth*bitmapFormatIn.biHeight)*(bitmapFormatIn.biBitCount/8);
			lres = ICCompressGetFormat(writeVideoSettings.hic,&bitmapFormatIn,bitmapFormatOut);
			if (lres == ICERR_OK)
				{
				convertTo24Bpp = true;
				}
			}
		}
	return(true);

}

// *****************************************************************************
// encodeVideoClass::setVideoOutputFormatDivxNetworks()
// *****************************************************************************
bool encodeVideoClass::setVideoOutputFormatDivxNetworks(COMPVARS * videoDefault)
{
	vbrMode = false;
	uncompressed = false;
	divxNetworksMode = true;
	xvidMode = false;
	divxNetworksVersion5 = false;

	//	memcpy(&writeVideoSettings, videoDefault, sizeof(COMPVARS));
	memset(&writeVideoSettings, 0, sizeof(COMPVARS));
	writeVideoSettings.cbSize = sizeof(COMPVARS);
	writeVideoSettings.dwFlags = 1;
	writeVideoSettings.lDataRate = 300; //Favor Divx 
	writeVideoSettings.lKey = 9999; //favor Divx 
	writeVideoSettings.fccType = mmioFOURCC('v', 'i', 'd', 'c');
	writeVideoSettings.fccHandler = mmioFOURCC('d', 'i', 'v', 'x');


	writeVideoSettings.lpbiOut = (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
	memcpy(writeVideoSettings.lpbiOut, videoDefault->lpbiOut, VIDOMI_BITMAP_INFO_SIZE);
	writeVideoSettings.lpbiOut->bmiHeader.biSize = sizeof(BITMAPINFO);
	writeVideoSettings.lpbiOut->bmiHeader.biWidth = videoWidth;
	writeVideoSettings.lpbiOut->bmiHeader.biHeight = videoHeight;
	writeVideoSettings.lpbiOut->bmiHeader.biPlanes = 0x0001;
	writeVideoSettings.lpbiOut->bmiHeader.biBitCount = 0x0018;
	writeVideoSettings.lpbiOut->bmiHeader.biSizeImage = videoWidth * videoHeight * (videoBpp/8);
	writeVideoSettings.lpbiOut->bmiHeader.biXPelsPerMeter = 0;
	writeVideoSettings.lpbiOut->bmiHeader.biYPelsPerMeter = 0;
	writeVideoSettings.lpbiOut->bmiHeader.biClrImportant = 0;
	writeVideoSettings.lpbiOut->bmiHeader.biClrUsed = 0;
	writeVideoSettings.lpbiOut->bmiHeader.biCompression = mmioFOURCC('D', 'I', 'V', 'X');
 
	// If compressor open, close it
	if(writeVideoSettings.hic)
		{
		//writeVideoSettings.hic->unused = 0;
		ICClose(writeVideoSettings.hic);
		writeVideoSettings.hic = NULL;
		}

	if(!videoDefault->cbState)
		return(false);

	writeVideoSettings.hic = ICOpen(writeVideoSettings.fccType, writeVideoSettings.fccHandler ,ICMODE_COMPRESS);
	if(!writeVideoSettings.hic)
		return(false);

	writeVideoSettings.cbState = ICGetStateSize(writeVideoSettings.hic);
	
	writeVideoSettings.lpState = new char [VIDEO_STATE_DEFAULT_SIZE];
	memset(writeVideoSettings.lpState,0,VIDEO_STATE_DEFAULT_SIZE);
	memcpy(writeVideoSettings.lpState, videoDefault->lpState, videoDefault->cbState);

	return(true);

}

// *****************************************************************************
// encodeVideoClass::setDivxNetworksLogFile()
// *****************************************************************************
bool encodeVideoClass::setDivxNetworksLogFile(char * logFile)
{

	/*
	DIVXSTATE * ptr;

	if(writeVideoSettings.lpState)
		{
		ptr = (DIVXSTATE *) writeVideoSettings.lpState;

		if(strlen(logFile)<254)
			{
			// If the path is short enough, use it
			strcpy(ptr->logFile,logFile);

			//if(writeVideoSettings.hic)
			//	ICSetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);

			return(true);
			}
		}

	return(false);
	*/

	return(true);
}

// *****************************************************************************
// encodeVideoClass::setDivxNetworksEncodeMode()
// *****************************************************************************
bool encodeVideoClass::setDivxNetworksEncodeMode(int encodeMode, int pass)
{
	switch(encodeMode)
		{
		case ENCODE_MODE_DIVXCBR:
			divxNetworksEncodeMode =  DIVXCODEC_ENCODECBR;
			keyframeCodecRestartMode = false;
			break;
		case ENCODE_MODE_DIVXCBRQUALITY:
			divxNetworksEncodeMode =  DIVXCODEC_ENCODECBRQUALITY;
			keyframeCodecRestartMode= false;
			break;
		case ENCODE_MODE_DIVXVBR:
			divxNetworksEncodeMode = DIVXCODEC_ENCODEVBR + pass;
			keyframeCodecRestartMode = false;
			break;
		case ENCODE_MODE_DIVXVBRPLUS:
			divxNetworksEncodeMode = DIVXCODEC_ENCODECBR;
			keyframeCodecRestartMode = false;
			if(pass==0)
				{
				keyframeCodecRestartMode = false;
				}
			else
				{
				keyframeCodecRestartMode = true;
				}
			break;
		}


	return(true);
}

// *****************************************************************************
// encodeVideoClass::setDivxNetworksBitrate()
// *****************************************************************************
bool encodeVideoClass::setDivxNetworksBitrate(int bitrate)
{

	if(writeVideoSettings.lpState)
		{	

		if(bitrate>6000000)
			bitrate = 6000000;

		divxNetworksEncodeBitrate = bitrate;

		return(true);
		}

	return(false);

}

// *****************************************************************************
// encodeVideoClass::setDivxNetworksKeyframe()
// *****************************************************************************
void encodeVideoClass::setDivxNetworksKeyframe(bool keyframe)
{
	if(keyframe)
		{
		nextKeyFrame = 1;
		}
	else
		{
		nextKeyFrame = 9999;
		}
}

// *****************************************************************************
// encodeVideoClass::updateDivxNetworksBitrate()
// *****************************************************************************
bool encodeVideoClass::updateDivxNetworksBitrate(int bitrate)
{

	if(bitrate>6000000)
		bitrate = 6000000;

	divxNetworksEncodeBitrate = bitrate;
	return(false);

}
// *****************************************************************************
// encodeVideoClass::setVideoOutputFormatVbr()
// *****************************************************************************
bool encodeVideoClass::setVideoOutputFormatVbr(DWORD fourCC)
{
	vbrMode = true;
	uncompressed = false;
	divxNetworksMode = false;
	keyframeCodecRestartMode = false;
	xvidMode = false;

	memset(&writeVideoSettings,0,sizeof(COMPVARS));
	writeVideoSettings.fccType = mmioFOURCC('v', 'i', 'd', 'c');
	writeVideoSettings.fccHandler = fourCC;
	writeVideoSettings.lQ = 10000;
	writeVideoSettings.lDataRate = 1800;
	writeVideoSettings.lKey = 9999;

	writeVideoSettings.hic = ICOpen(writeVideoSettings.fccType, writeVideoSettings.fccHandler ,ICMODE_COMPRESS);
	
	if(!writeVideoSettings.hic)
		{
		return(false);
		}

	return(true);

}
// *****************************************************************************
// encodeVideoClass::startEncoder()
//
// *****************************************************************************
bool encodeVideoClass::startEncoder(bool firstTime)
{
	ICCOMPRESSFRAMES icCompress;
	LRESULT lres;
	DWORD size;
	int * ptr;
	int dataRate;
	long bmhSize;
	int * dnptr;

	if(uncompressed)
		{
		memcpy(bitmapFormatOut,&bitmapFormatIn,VIDOMI_BITMAP_INFO_SIZE);	
		size = bitmapFormatOut->biSizeImage;
		}
	else
		{
		//  query size of bmstruct
		bmhSize=ICCompressGetFormat(writeVideoSettings.hic, &bitmapFormatIn,0);
		if (!bmhSize)
			{
			return(false);
			}
		

		// now get the header
		lres = ICCompressGetFormat(writeVideoSettings.hic,&bitmapFormatIn,bitmapFormatOut);
		if (lres != ICERR_OK)
			{
			return(false);
			}

		// memset(bitmapFormatOut,0,sizeof(BITMAPINFOHEADER));
		//lres = ICCompressGetFormat(writeVideoSettings.hic, &bitmapFormatIn, bitmapFormatOut);
		//if(lres != ICERR_OK)
		//	return(false);
  
		lres = ICGetInfo(writeVideoSettings.hic, &videoCodecInfo, sizeof(ICINFO));
		if(!lres)
			return(false);

		if(vbrMode)
			{
			ICSetState(writeVideoSettings.hic,NULL,0);
			}
		else
			{
			if(xvidMode)
				{
				ICGetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);
				dnptr = (int *) writeVideoSettings.lpState;
				dnptr = (int *) (((char *) dnptr) + xvidBitrateOffset);
				*dnptr = xvidEncodeBitrate;
				}

			if(divxNetworksMode)
				{
				// It would really be nice to know this prior to here..
				if(firstTime)
					{
					if(bitmapFormatOut->biCompression == mmioFOURCC('D', 'X', '5', '0'))
						{
						logPrintf("Using Divx 5 Encode state offsets\n");
						divxNetworksVersion5 = true;
						}
					else
						{
						logPrintf("Using Divx 4 Encode state offsets\n");
						divxNetworksVersion5 = false;
						}
					}
				
				// Hack to try and fix divx 5 crash
				ICGetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);

				if(divxNetworksVersion5)
					{
					dnptr = (int *) writeVideoSettings.lpState;
					dnptr = (int *) (((char *) dnptr) + divx5BitrateOffset);
					*dnptr = divxNetworksEncodeBitrate;
					dnptr = (int *) writeVideoSettings.lpState;
					dnptr = (int *) (((char *) dnptr) + divx5VbrModeOffset);
					*dnptr = divxNetworksEncodeMode;	
					}
				else
					{
					dnptr = (int *) writeVideoSettings.lpState;
					dnptr = (int *) (((char *) dnptr) + divx4BitrateOffset);
					*dnptr = divxNetworksEncodeBitrate;
					dnptr = (int *) writeVideoSettings.lpState;
					dnptr = (int *) (((char *) dnptr) + divx4VbrModeOffset);
					*dnptr = divxNetworksEncodeMode;	
					}
				}

			ICSetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);

			if(divxNetworksMode)
				{
				if(!divxNetworksVersion5)
					{
					ICClose(writeVideoSettings.hic);
					Sleep(300);
					writeVideoSettings.hic = ICOpen(writeVideoSettings.fccType, writeVideoSettings.fccHandler ,ICMODE_COMPRESS);
					if(!writeVideoSettings.hic)
						{
						return(false);
						}
					ICSetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);
					}
				}

			if(xvidMode && keyframeCodecRestartMode)
				{
				ICClose(writeVideoSettings.hic);
				Sleep(300);
				writeVideoSettings.hic = ICOpen(writeVideoSettings.fccType, writeVideoSettings.fccHandler ,ICMODE_COMPRESS);
				if(!writeVideoSettings.hic)
					{
					return(false);
					}
				ICSetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);
				}

			}

		memset(&icCompress, 0, sizeof(ICCOMPRESSFRAMES));

		icCompress.lQuality	= writeVideoSettings.lQ;
		icCompress.lDataRate = writeVideoSettings.lDataRate;
		icCompress.lKeyRate	= writeVideoSettings.lKey;
		icCompress.dwScale		= (unsigned long)(1000000.0f / videoFramerate);
		icCompress.dwRate		= 1000000;

		lres = ICSendMessage(writeVideoSettings.hic, ICM_COMPRESS_FRAMES_INFO, (unsigned long)&icCompress, sizeof(ICCOMPRESSFRAMES));
//		if(lres != ICERR_OK)
//			{
//			return(false);
//			}

		lres = ICCompressBegin(writeVideoSettings.hic, &bitmapFormatIn, bitmapFormatOut);
		if(lres != ICERR_OK)
			{
			return(false);
			}
		// Fetch max size of a compressed frame
		size = ICCompressGetSize(writeVideoSettings.hic,&bitmapFormatIn, bitmapFormatOut);
		}

	if(firstTime)
		{
		frameOut->allocStart = new unsigned char [size]; 
		frameOut->bufferStart = frameOut->allocStart;
		frameOut->data = frameOut->allocStart;
		frameOut->width = videoWidth;
		frameOut->height = videoHeight;

		if(convertTo24Bpp)
			{
			frameTemp->allocStart = new unsigned char [videoWidth*videoHeight*3]; 
			frameTemp->bufferStart = frameTemp->allocStart;
			frameTemp->data = frameTemp->allocStart;
			frameTemp->width = videoWidth;
			frameTemp->height = videoHeight;
			}
		}

	nextKeyFrame = 1;

	if(vbrMode)
		{
		ptr = (int *) writeVideoSettings.hic;
		ptr = (int *) ptr[5];

		divxCrispness = ptr + 14;

		ptr = (int *) ptr[0];
		ptr = (int *) ptr[1];

		divxCurrentCompressionLevel = ptr + 17;
		divxDeltaFrame = ptr + 218;
		divxBitsPerFrame = ptr + 283;
		divxKbps = (double *) (ptr+286);
		divxMaxCompressionLevel = ptr + 292;
		divxMinCompressionLevel = ptr + 293;
		divxBitsRemaining = ptr + 299;
		divxBitsMax = ptr + 300;
		divxBitsMin = ptr + 301;
		divxBitrate = ptr + 302;
		divxKbps2 = (double *) (ptr+304);
		divxLimit = ptr + 306;
		divxBitsPerFrame2 = ptr + 307;

		curDataRate = dataRate = 900*1024;

		*divxCrispness = 100;	
		*divxKbps = dataRate/1024;
		*divxBitrate = dataRate;
		*divxKbps2 = (*divxKbps) * 1.024;
		*divxBitsMax = (dataRate/2)*10;
		*divxBitsMin = dataRate/2;
		*divxLimit = (int)((*divxBitsMax)*0.35);
		*divxBitsPerFrame = (int) ((float)dataRate / videoFramerate);
		*divxBitsPerFrame2 = *divxBitsPerFrame;
		*divxMaxCompressionLevel = 16;
		*divxMinCompressionLevel = 2;
		*divxBitsRemaining = *divxBitsMax;
		*divxCurrentCompressionLevel = 2;
		}

	if(firstTime)
		{
		frameCounter = 0;
		}

	return(true);
}

// *****************************************************************************
// encodeVideoClass::getFormatOut()
//
// *****************************************************************************
BITMAPINFOHEADER * encodeVideoClass::getFormatOut()
{
	return(bitmapFormatOut);
}

// *****************************************************************************
// encodeVideoClass::getFormatOut()
//
// *****************************************************************************
DWORD encodeVideoClass::getFccHandler()
{
	return(writeVideoSettings.fccHandler);
}


// *****************************************************************************
// encodeVideoClass::encoder()
//
// *****************************************************************************
FRAMEDATA * encodeVideoClass::encode(FRAMEDATA * frameIn, DWORD * flags)
{
	int keyframe;
	LRESULT lres;
	FRAMEDATA * encodeFrame;
	

	if(uncompressed)
		{
		*flags = DIVX_KEYFRAME;
		frameCounter++;
		return(frameIn);
		}
	else
		{
		nextKeyFrame--;
		if(!nextKeyFrame)
			{
			keyframe = ICCOMPRESS_KEYFRAME;
			nextKeyFrame = writeVideoSettings.lKey;
			}
		else
			{
			keyframe=0;
			}
		
		// If needed, Do the conversion to support 24 bit codecs
		if(convertTo24Bpp)
			{
			// Byebye performance :o(
			rgb32ToRgb24(frameIn,frameTemp);
			encodeFrame = frameTemp;
			}
		else
			{
			encodeFrame = frameIn;
			}

		lres = ICCompress(writeVideoSettings.hic,keyframe,bitmapFormatOut,frameOut->data, &bitmapFormatIn, encodeFrame->data,
				0,flags,frameCounter,0,writeVideoSettings.lQ,0,NULL);
	
		
		if(*flags==DIVX_KEYFRAME && keyframeCodecRestartMode)
			{
			if(!startEncoder(false))
				return(false);
			
			keyframe = ICCOMPRESS_KEYFRAME;
			nextKeyFrame = writeVideoSettings.lKey;

			lres = ICCompress(writeVideoSettings.hic,keyframe,bitmapFormatOut,frameOut->data, &bitmapFormatIn, encodeFrame->data,
					0,flags,frameCounter,0,writeVideoSettings.lQ,0,NULL);
			}

		frameCounter++;

		if(lres != ICERR_OK)
			return(false);

		// Give the output frame the inputs frame PTS values. Needed for write avi/etc.
		frameOut->pts = frameIn->pts;
		frameOut->hardPts = frameIn->hardPts;
		frameOut->hardPtsFrame = frameIn->hardPtsFrame;

		return(frameOut);

		}

}

// *****************************************************************************
// encodeVideoClass::endEncoder()
//
// *****************************************************************************
bool encodeVideoClass::endEncoder()
{

	if(frameOut->allocStart)
		{
		delete [] frameOut->allocStart;
		memset(frameOut,0,sizeof(FRAMEDATA));
		}

	if(frameTemp->allocStart)
		{
		delete [] frameTemp->allocStart;
		memset(frameTemp,0,sizeof(FRAMEDATA));
		}

	if(writeVideoSettings.hic)
		{
		//writeVideoSettings.hic->unused = 0;
		ICCompressEnd(writeVideoSettings.hic);
		ICClose(writeVideoSettings.hic);
		videoInputSet = false;
		}

	return(true);
}


// *****************************************************************************
// encodeVideoClass::setCrispness()
//
// *****************************************************************************
void encodeVideoClass::setCrispness(int val)
{
	*divxCrispness = val;
}

// *****************************************************************************
// encodeVideoClass::setDataRate()
//
// *****************************************************************************
void encodeVideoClass::setDataRate(int dataRate)
{
	float ratio;

	ratio = ((float)dataRate / (float)curDataRate);

	curDataRate = dataRate;

	*divxKbps = dataRate/1024;
	*divxBitrate = dataRate;
	*divxKbps2 = (*divxKbps) * 1.024;
	*divxBitsMax = (dataRate/2)*10;
	*divxBitsMin = dataRate/2;
	*divxLimit = (int)((*divxBitsMax)*0.35);
	*divxBitsPerFrame = (int) ((float)dataRate / videoFramerate);
	*divxBitsPerFrame2 = *divxBitsPerFrame;
	*divxBitsRemaining = (int) (*divxBitsRemaining * ratio);
}

// *****************************************************************************
// encodeVideoClass::getDataRate()
//
// *****************************************************************************
int encodeVideoClass::getDataRate()
{
	return(curDataRate);
}

// *****************************************************************************
// encodeVideoClass::setCompressionMinMax()
//
// *****************************************************************************
void encodeVideoClass::setCompressionMinMax(int min, int max)
{

	*divxMaxCompressionLevel = max;
	*divxMinCompressionLevel = min;

}

// *****************************************************************************
// encodeVideoClass::setCurrentCompressionLevel()
//
// *****************************************************************************
void encodeVideoClass::setCurrentCompressionLevel(int val)
{
	*divxCurrentCompressionLevel = val;
}

// *****************************************************************************
// encodeVideoClass::getCurrentCompressionLevel()
//
// *****************************************************************************
int encodeVideoClass::getCurrentCompressionLevel(void)
{
	return(*divxCurrentCompressionLevel);
}

// *****************************************************************************
// encodeVideoClass::setBitsRemaining()
//
// *****************************************************************************
void encodeVideoClass::setBitsRemaining(int val)
{
	*divxBitsRemaining = val;
}

// *****************************************************************************
// encodeVideoClass::getBitsRemaining()
//
// *****************************************************************************
int encodeVideoClass::getBitsRemaining(void)
{
	return(*divxBitsRemaining);
}

// *****************************************************************************
// encodeVideoClass::setDeltaFrame()
//
// *****************************************************************************
void encodeVideoClass::setDeltaFrame(int val)
{
	*divxDeltaFrame = val;
}
// *****************************************************************************
// encodeVideoClass::setVideoOutputFormatxvid()
// *****************************************************************************
bool encodeVideoClass::setVideoOutputFormatXvid(COMPVARS * videoDefault)
{
	vbrMode = false;
	uncompressed = false;
	xvidMode = true;
	divxNetworksMode = false;
	keyframeCodecRestartMode = false;

	//	memcpy(&writeVideoSettings, videoDefault, sizeof(COMPVARS));
	memset(&writeVideoSettings, 0, sizeof(COMPVARS));
	writeVideoSettings.cbSize = sizeof(COMPVARS);
	writeVideoSettings.dwFlags = 1;
	writeVideoSettings.lDataRate = 300; //Favor Divx 
	writeVideoSettings.lKey = 9999; //favor Divx 
	writeVideoSettings.fccType = mmioFOURCC('v', 'i', 'd', 'c');
	writeVideoSettings.fccHandler = mmioFOURCC('x', 'v', 'i', 'd');


	writeVideoSettings.lpbiOut = (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
	memcpy(writeVideoSettings.lpbiOut, videoDefault->lpbiOut, VIDOMI_BITMAP_INFO_SIZE);
	writeVideoSettings.lpbiOut->bmiHeader.biSize = sizeof(BITMAPINFO);
	writeVideoSettings.lpbiOut->bmiHeader.biWidth = videoWidth;
	writeVideoSettings.lpbiOut->bmiHeader.biHeight = videoHeight;
	writeVideoSettings.lpbiOut->bmiHeader.biPlanes = 0x0001;
	writeVideoSettings.lpbiOut->bmiHeader.biBitCount = 0x0018;
	writeVideoSettings.lpbiOut->bmiHeader.biSizeImage = videoWidth * videoHeight * (videoBpp/8);
	writeVideoSettings.lpbiOut->bmiHeader.biXPelsPerMeter = 0;
	writeVideoSettings.lpbiOut->bmiHeader.biYPelsPerMeter = 0;
	writeVideoSettings.lpbiOut->bmiHeader.biClrImportant = 0;
	writeVideoSettings.lpbiOut->bmiHeader.biClrUsed = 0;
	writeVideoSettings.lpbiOut->bmiHeader.biCompression = mmioFOURCC('X', 'V', 'I', 'D');
 
	// If compressor open, close it
	if(writeVideoSettings.hic)
		{
		//writeVideoSettings.hic->unused = 0;
		ICClose(writeVideoSettings.hic);
		writeVideoSettings.hic = NULL;
		}

	if(!videoDefault->cbState)
		return(false);

	writeVideoSettings.hic = ICOpen(writeVideoSettings.fccType, writeVideoSettings.fccHandler ,ICMODE_COMPRESS);
	if(!writeVideoSettings.hic)
		return(false);

	// Peter: I changed to query of state size from OS. This is more compatible. 

	writeVideoSettings.cbState = ICGetStateSize(writeVideoSettings.hic); //sizeof(XVIDSTATE);


	writeVideoSettings.lpState = new char [VIDEO_STATE_DEFAULT_SIZE];
	memset(writeVideoSettings.lpState,0,VIDEO_STATE_DEFAULT_SIZE);
	memcpy(writeVideoSettings.lpState, videoDefault->lpState, videoDefault->cbState);

	return(true);

}

// *****************************************************************************
// encodeVideoClass::setxvidLogFile()
// *****************************************************************************
bool encodeVideoClass::setXvidLogFile(char * logFile)
{
	char * ptr;

	if(writeVideoSettings.lpState)
		{
		if(writeVideoSettings.hic)
			ICGetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);

		ptr = (char *) writeVideoSettings.lpState;
		ptr = (char *) (((char *)ptr) + xvidStatsFilenameOffset);
			
		if(strlen(logFile)<254)
			{
			// If the path is short enough, use it
			strcpy(ptr,logFile);

			if(writeVideoSettings.hic)
				ICSetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);

			return(true);
			}
		}
	return(false);
}

// *****************************************************************************
// encodeVideoClass::setxvidEncodeMode()
// *****************************************************************************
bool encodeVideoClass::setXvidEncodeMode(int encodeMode, int pass)
{
	int * ptr;

	if(writeVideoSettings.lpState)
		{	
		
		if(writeVideoSettings.hic)
			ICGetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);

		switch(encodeMode)
			{
			case ENCODE_MODE_XVIDVBR:
				ptr = (int *) writeVideoSettings.lpState;
				ptr = (int *) (((char *)ptr) + xvidEncodeModeOffset);

				if(pass == 0)
					*ptr = DLG_MODE_2PASS_1;
				else
					*ptr = DLG_MODE_2PASS_2_INT;

				keyframeCodecRestartMode = false;
				break;
			case ENCODE_MODE_XVIDHYBRID:
				ptr = (int *) writeVideoSettings.lpState;
				ptr = (int *) (((char *)ptr) + xvidEncodeModeOffset);
				*ptr = DLG_MODE_CBR;
				keyframeCodecRestartMode = true;
				break;
			}

		if(writeVideoSettings.hic)
			ICSetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);

		return(true);
		}
	return(false);
}

// *****************************************************************************
// encodeVideoClass::setxvidBitrate()
// *****************************************************************************
bool encodeVideoClass::setXvidBitrate(int bitrate)
{
	int * ptr;

	if(writeVideoSettings.lpState)
		{	

		//if(writeVideoSettings.hic)
		//	ICGetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);

		ptr = (int *) writeVideoSettings.lpState;
		ptr = (int *) (((char *)ptr) + xvidBitrateOffset);
		
		if(bitrate>10000000)
			bitrate = 10000000;
		
		xvidEncodeBitrate = bitrate;

		*ptr = bitrate;

		if(writeVideoSettings.hic)
			ICSetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);
		return(true);
		}

	return(false);

}

// *****************************************************************************
// encodeVideoClass::setxvidKeyframe()
// *****************************************************************************
void encodeVideoClass::setXvidKeyframe(bool keyframe)
{
	if(keyframe)
		{
		nextKeyFrame = 1;
		}
	else
		{
		nextKeyFrame = 9999;
		}
}

// *****************************************************************************
// encodeVideoClass::updatexvidBitrate()
// *****************************************************************************
bool encodeVideoClass::updateXvidBitrate(int bitrate)
{
	int * ptr;

	if(writeVideoSettings.lpState)
		{	
		if(writeVideoSettings.hic)
			ICGetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);

		ptr = (int *) writeVideoSettings.lpState;
		ptr = (int *) (((char *)ptr) + xvidBitrateOffset);

		if(bitrate>10000000)
			bitrate = 10000000;
		
		xvidEncodeBitrate = bitrate;

		*ptr = bitrate;

		if(writeVideoSettings.hic)
			ICSetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);
		
		return(true);
		}

	return(false);

}

// *****************************************************************************
// encodeVideoClass::setxvidFilesize()
// *****************************************************************************
bool encodeVideoClass::setXvidFilesize(__int64 size)
{
	int * ptr;
	
	if(writeVideoSettings.lpState)
		{	

		if(writeVideoSettings.hic)
			ICGetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);

		ptr = (int *) writeVideoSettings.lpState;
		ptr = (int *) (((char *)ptr) + xvidDesiredSizeOffset);
		
		*ptr = (int) (size >> 10);

		if(writeVideoSettings.hic)
			ICSetState(writeVideoSettings.hic,writeVideoSettings.lpState,writeVideoSettings.cbState);

		return(true);
		}

	return(false);

}

// *****************************************************************************
// postProcessClass::Rgb32ToRgb24()
//
// *****************************************************************************

void encodeVideoClass::rgb32ToRgb24(FRAMEDATA * srcFrame, FRAMEDATA * dstFrame)
{
	unsigned char * src;
	unsigned char * dst;
	unsigned char * src1;
	unsigned char * dst1;
	int w,h;


	dstFrame->bitDepth=24;
	dstFrame->height = srcFrame->height;
	dstFrame->width = srcFrame->width;
	dstFrame->stride = srcFrame->width*3;
	dstFrame->type = srcFrame ->type;

	src = srcFrame->data;
	dst = dstFrame->data;

	for(h=0;h<srcFrame->height;h++)
		{
		src1 = src;
		dst1 = dst;
		for(w=0;w<srcFrame->width;w++)
			{
			*dst1++=*src1++;
			*dst1++=*src1++;
			*dst1++=*src1++;
			*src1++;
			}
		src += srcFrame->stride;
		dst += dstFrame->stride;
		}

}
