
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <float.h>
#include <math.h>
#include "mhead.h"		/* mpeg header structure, decode protos */

#define TRUE 1
#define FALSE 0


typedef struct {
      int (*decode_init) (MPEG_HEAD * h, int framebytes_arg,
			  int reduction_code, int transform_code,
			  int convert_code, int freq_limit);
      void (*decode_info) (DEC_INFO * info);
        IN_OUT(*decode) (unsigned char *bs, short *pcm);
   }
   AUDIO;


static AUDIO audio_table[2][2] =
   {
      {
	 {audio_decode_init, audio_decode_info, audio_decode},
	 {audio_decode8_init, audio_decode8_info, audio_decode8},
      },
      {
	 {i_audio_decode_init, i_audio_decode_info, i_audio_decode},
	 {audio_decode8_init, audio_decode8_info, audio_decode8},
      }
   };

MPEG_HEAD	head;
DEC_INFO	decinfo;
AUDIO		audio_table[2][2];
int			firstPass;
AUDIO		audio;
int			framebytes;
int			bitrate;
char *		inBuffer=0;
int			inHead;
int			inTail;
int			failure;
  
// *****************************************************************************		
// MP3 Decode constants 
// *****************************************************************************		
//decode8 (8Ks output) convert_code:
//   convert_code = 4*bit_code + chan_code
//       bit_code:   1 = 16 bit linear pcm
//                   2 =  8 bit (unsigned) linear pcm
//                   3 = u-law (8 bits unsigned)
//       chan_code:  0 = convert two chan to mono
//                   1 = convert two chan to mono
//                   2 = convert two chan to left chan
//                   3 = convert two chan to right chan
//
//decode (standard decoder) convert_code:
//             0 = two chan output
//             1 = convert two chan to mono
//             2 = convert two chan to left chan
//             3 = convert two chan to right chan
//   or with 8 = 8 bit output 
//          (other bits ignored)
//
//decode (standard decoder) reduction_code:
//             0 = full sample rate output
//             1 = half rate
//             2 = quarter rate
// *****************************************************************************		
#define MP3_CONVERT_CODE 0
#define MP3_REDUCTION_CODE 0
#define MP3_TRANSFORM_CODE 0
#define MP3_FREQ_LIMIT 24000		
#define MP3_INTEGER_DECODE 0		
#define MP3_DECODE8_FLAG 0


// *****************************************************************************
// verifyHeader()
//
// *****************************************************************************
int verifyHeader(unsigned long head)
{
  if ((head & 0xffe00000) != 0xffe00000)
    return FALSE;
  if (!((head >> 17) & 3))
    return FALSE;
  if (((head >> 12) & 0xf) == 0xf)
    return FALSE;
  if (!((head >> 12) & 0xf))
    return FALSE;
  if (((head >> 10) & 0x3) == 0x3)
    return FALSE;
  if (((head >> 19) & 1) == 1 && ((head >> 17) & 3) == 3 && ((head >> 16) & 1) == 1)
    return FALSE;
  if ((head & 0xffff0000) == 0xfffe0000)
    return FALSE;
  
  return TRUE;
}

// *****************************************************************************
// mpegInit()
//
// *****************************************************************************
void mpegInit(void)
{
	inBuffer = malloc(128000);
}


// *****************************************************************************
// mpegInit()
//
// *****************************************************************************
void mpegEnd(void)
{
	if(inBuffer)
		free(inBuffer);
}

// *****************************************************************************
// mpegInit()
//
// *****************************************************************************
int mpegPrepare(char * source, int sourceLength, int * channels, int * samplerate, int * framesize)
{
	unsigned char * ptr;
	unsigned long header;

	failure=FALSE;
	ptr= source;
	
	header = *ptr <<24 | *(ptr+1)<<16 | *(ptr+2)<<8 | *(ptr+3);
	
	if(!verifyHeader(header))
		return(FALSE);
	// setup the function pointers for the appropriate function call
	if(MP3_DECODE8_FLAG &&  MP3_CONVERT_CODE >=4)
		audio = audio_table[MP3_INTEGER_DECODE & 1][0];
	else
		audio = audio_table[MP3_INTEGER_DECODE & 1][0];
	
	framebytes = head_info2(source, sourceLength, &head, &bitrate);

	if (!audio.decode_init(&head, framebytes,
			MP3_REDUCTION_CODE, MP3_TRANSFORM_CODE, MP3_CONVERT_CODE, MP3_FREQ_LIMIT))
		{
		return(FALSE);
		}
	audio.decode_info(&decinfo);

	*channels = decinfo.channels;
	*samplerate = decinfo.samprate;
	*framesize = decinfo.framebytes+1;

	inTail=0;
	inHead = 0;
	return(TRUE);
}


// *****************************************************************************
// mpegInit()
//
// *****************************************************************************
int mpegDecode(char * source, int sourceLength, char *dest)
{
	IN_OUT stat;
	int output;
	int sourceUsed;

	
	memcpy(inBuffer+inTail,source,sourceLength);
	inTail+=sourceLength;

	output = 0;
	sourceUsed=0;
	
	while((inTail-inHead)>=framebytes)
		{
		stat = audio.decode(inBuffer+inHead, (short *) dest);
		if(!stat.out_bytes)
			{
			inHead++;
			failure=TRUE;
			continue;
			}
		else
			{
			failure=FALSE;
			}
		output+=stat.out_bytes;
		dest+=stat.out_bytes;
		inHead+=stat.in_bytes;
		}

	memcpy(inBuffer,inBuffer+inHead,(inTail-inHead)+1);
	inTail -= inHead;
	inHead = 0;

	return(output);

}

