// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// ifo.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

#include "languagecodes.cpp"
// *****************************************************************************
// ifoClass::ifoClass()
//
// ifo class constructor
// *****************************************************************************
ifoClass::ifoClass()
{

}

// *****************************************************************************
// ifoClass::~ifoClass()
//
// ifo class destructor
// *****************************************************************************
ifoClass::~ifoClass()
{

}

// *****************************************************************************
// ifoClass::readIfo()
//
// ifo class destructor
// *****************************************************************************
bool ifoClass::readIfo(char * filename, GATHERRESULTS * fileInfo)
{

	memset(fileInfo,0,sizeof(GATHERRESULTS));

	ifoFile = fopen(filename,"rb");
	if(!ifoFile)
		{
		return(false);
		}

	fseek(ifoFile,0,SEEK_END);
	
	ifoDataSize = ftell(ifoFile);

	fseek(ifoFile,0,SEEK_SET);
	
	ifoData = new unsigned char [ifoDataSize];

	if(!ifoData)
		{
		fclose(ifoFile);
		return(false);
		}

	if(fread(ifoData,1,ifoDataSize,ifoFile)!=ifoDataSize)
		{
		delete [] ifoData;
		fclose(ifoFile);
		return(false);
		}

	fclose(ifoFile);


	readVideoData(fileInfo);

	readAudioData(fileInfo);

	readSubtitleData(fileInfo);

	readChapterData(fileInfo);


	delete [] ifoData;

	return(true);
}

// *****************************************************************************
// ifoClass::readVideoData()
//
// read the Video Format
// *****************************************************************************
void ifoClass::readVideoData(GATHERRESULTS * fileInfo)
{
	int videoByte1;
	int videoByte2;

	fileInfo->averageDatarate=0;
	fileInfo->fileType = SCAN_FILE_TYPE_MPEG;
	fileInfo->videoChromaFormat = CHROMA420;

	// Now Read the VIDEO format Attributes

	ifoPtr = ifoData + IFO_VIDEO_ATTRIBUTES_START;

	videoByte1 = getChar();
	videoByte2 = getChar();

	if(videoByte1&0x30)
		{
		// It's PAL
		switch(videoByte2&0x0c>>2)
			{
			case 0:
				fileInfo->videoWidth = 720;
				fileInfo->videoHeight = 576;
				break;

			case 1:
				fileInfo->videoWidth = 704;
				fileInfo->videoHeight = 576;
				break;
			case 2:
				fileInfo->videoWidth = 352;
				fileInfo->videoHeight = 576;
				break;
			case 3:
				fileInfo->videoWidth = 352;
				fileInfo->videoHeight = 288;
				break;
			}
		fileInfo->videoFramerate=25.0f;

		}
	else
		{
		// It's NTSC
		switch(videoByte2&0x0c>>2)
			{
			case 0:
				fileInfo->videoWidth = 720;
				fileInfo->videoHeight = 480;
				break;

			case 1:
				fileInfo->videoWidth = 704;
				fileInfo->videoHeight = 480;
				break;
			case 2:
				fileInfo->videoWidth = 352;
				fileInfo->videoHeight = 480;
				break;
			case 3:
				fileInfo->videoWidth = 352;
				fileInfo->videoHeight = 240;
				break;
			}

		fileInfo->videoFramerate=29.976f;

		}

	fileInfo->videoAspectRatio = 16.0f/9.0f;
	
}


// *****************************************************************************
// ifoClass::readAudioData()
//
// read the Audio Format
// *****************************************************************************
void ifoClass::readAudioData(GATHERRESULTS * fileInfo)
{
	int audioChannels;
	int ifoAudioChannels;
	int audioByte1;
	int audioByte2;
	int audioByte3;
	int audioByte4;
	int audioByte5;
	int audioByte6;
	int audioByte7;
	int audioByte8;
	bool audioLanguageCode;
	int audioSourceChannels;
	char audioTypeText[256];
	char audioLanguageText[256];
	char audioContentText[256];
	int audioType;
	int i;
	int langIndex;
	int langCode;

	// And Read the Audio Format Attributes

	ifoPtr = ifoData + IFO_AUDIO_ATTRIBUTES_START;

	ifoAudioChannels = getShort();

	audioChannels = 0;

	for(i=0;i<ifoAudioChannels;i++)
		{
		audioByte1 = getChar();
		audioByte2 = getChar();
		audioByte3 = getChar();
		audioByte4 = getChar();
		audioByte5 = getChar();
		audioByte6 = getChar();
		audioByte7 = getChar();
		audioByte8 = getChar();
	
		audioType = (audioByte1&0xe0)>>5;
		if(audioByte1&0x0c)
			audioLanguageCode = true;
		else
			audioLanguageCode = false;

		if((audioType == 0) || (audioType ==2) || (audioType == 3) || (audioType ==4))
			{
			switch(audioType)
				{
				case 0:
					fileInfo->audioType[audioChannels] = audioAc3;
					sprintf(audioTypeText,"AC3");
					break;

				case 2:
				case 3:
					fileInfo->audioType[audioChannels] = audioMpeg;
					sprintf(audioTypeText,"MPEG");
					break;
				case 4:
					fileInfo->audioType[audioChannels] = audioPcm;
					sprintf(audioTypeText,"PCM");
					break;
				}

			strcpy(audioLanguageText,"");

			if(audioLanguageCode)
				{
				langCode = audioByte3;
				langCode = (langCode<<8) | audioByte4;
				for(langIndex=1;;langIndex++)
					{
					if(!langTable[langIndex].langCode)
						break;


					if(langTable[langIndex].langCode == langCode)
						{
						strcpy(audioLanguageText,langTable[langIndex].langName);					
						}
					}
				}
			
			strcpy(audioContentText,"");

			switch(audioByte6)
				{
				case 2:
					strcpy(audioContentText,"For Visually Impared");
					break;
				case 3:
					strcpy(audioContentText,"Directors Comments");
					break;
				case 4:
					strcpy(audioContentText,"Alternate Directors Comments");
					break;
				}
			audioSourceChannels = (audioByte2&0x07)+1;
			
			fileInfo->audioNumber[audioChannels]=i;
			fileInfo->audioRate[audioChannels]=48000;
			fileInfo->audioChannels[audioChannels] = audioSourceChannels;

			fileInfo->audioChannels[audioChannels] = 2;

			if(fileInfo->audioType[audioChannels]==audioMpeg || fileInfo->audioType[audioChannels]==audioPcm)
				{
				if(audioSourceChannels>2)
					fileInfo->audioChannels[audioChannels] = 2;
				else
					fileInfo->audioChannels[audioChannels] = audioSourceChannels;
				}

			sprintf(&fileInfo->audioName[audioChannels][0],"Audio[%d] %s %s(%dCh) %s",i+1,audioLanguageText,audioTypeText, audioSourceChannels,audioContentText);
			strcpy(&fileInfo->audioLanguage[audioChannels][0],audioLanguageText);
			audioChannels++;
			}
		}

	fileInfo->audioFound = audioChannels;

}

// *****************************************************************************
// ifoClass::readSubtitleData()
//
// read the Subtitle Format
// *****************************************************************************
void ifoClass::readSubtitleData(GATHERRESULTS * fileInfo)
{
	int ifoSubtitleChannels;
	int subtitleChannels;
	int subByte1;
	int subByte2;
	int subByte3;
	int subByte4;
	int subByte5;
	int subByte6;
	char subtitleLanguageText[256];
	int i;
	int langIndex;
	int langCode;

	ifoPtr = ifoData + IFO_SUBTITLE_ATTRIBUTES_START;


	ifoSubtitleChannels = getShort();

	subtitleChannels = 0;

	for(i=0;i<ifoSubtitleChannels;i++)
		{
		subByte1 = getChar();
		subByte2 = getChar();
		subByte3 = getChar();
		subByte4 = getChar();
		subByte5 = getChar();
		subByte6 = getChar();

		strcpy(subtitleLanguageText,"");

		if(subByte1&0x03)
			{
			langCode = subByte3;
			langCode = (langCode<<8) | subByte4;
			for(langIndex=1;;langIndex++)
				{
				if(!langTable[langIndex].langCode)
					break;


				if(langTable[langIndex].langCode == langCode)
					{
					strcpy(subtitleLanguageText,langTable[langIndex].langName);					
					}
				}
			}
		sprintf(&fileInfo->subtitleName[subtitleChannels][0],"Subtitle[%d] %s",i+1,subtitleLanguageText); 
		strcpy(&fileInfo->subtitleLanguage[subtitleChannels][0],subtitleLanguageText);

		fileInfo->subtitleNumber[subtitleChannels] = i;

		subtitleChannels++;
		}
	
	fileInfo->subtitleFound = subtitleChannels;


}

// *****************************************************************************
// ifoClass::readChapterData()
//
// read the Chapter Format
// *****************************************************************************
void ifoClass::readChapterData(GATHERRESULTS * fileInfo)
{
	unsigned int pgciPos;
	int pgcPos;
	int pgciProgramChains;
	char pgcPalette[16*4];
	int pgcCellPlaybackInfoPos;
	int pgcProgramMapPos;
	int pgcCellPosInfoPos;
	int i;
	int titleIndex;
	int cell;
	int flags;
	int playTime;
	int firstVobuStart;
	int firstIlvuEnd;
	int lastVobuStart;
	int	lastVobuEnd;
	int vobId;
	int cellId;
	int hours;
	int minutes;
	int seconds;
	int frames;
	int framerate;
	int frameNumber;
	int chapterCounter;

	// Reads the program Chain to determine the chapters

	// This code is setup to handle (well, sort of) more than 1 title...
	titleData[0] = new TITLEDATA [1];
	memset(titleData[0],0,sizeof(TITLEDATA));

	titleIndex=0;

	ifoPtr = ifoData + 0xcc;

	pgciPos = getInt();
	
	pgciPos*=0x800;

	ifoPtr = ifoData + pgciPos;

	pgciProgramChains = getShort();

	// Position us at the first entry in the PGCI

	ifoPtr = ifoData + pgciPos+8+4;

	pgcPos = pgciPos;

	if(pgcPos<0)
		{
		fileInfo->chaptersFound=0;
		}

	pgcPos += getInt();


	// Position us at the first entry in the PGC
	ifoPtr = ifoData + pgcPos+2;

	titleData[titleIndex]->numPrograms = getChar();
	titleData[titleIndex]->numCells = getChar();

	ifoPtr =ifoData + pgcPos + 0xa4;


	// Palette (0, Y, Cr, Cb)
	memcpy(pgcPalette,ifoPtr,4*16);

	// Convert using..
	// R = Y + 1.4022 Cr
	// B = Y + 1.7710 Cb
	// G = 1.7047 Y - 0.1952 B - 0.5647 R

	// Not implemented.

	ifoPtr =ifoData + pgcPos+0xe6;

	pgcProgramMapPos = pgcPos;
	pgcProgramMapPos += getShort();

	ifoPtr =ifoData + pgcProgramMapPos;

	for(i=0;i<titleData[titleIndex]->numPrograms;i++)
		{
		titleData[titleIndex]->programMap[i]=getChar();
		}

	ifoPtr =ifoData + pgcPos+0xea;
	pgcCellPosInfoPos = pgcPos;
	pgcCellPosInfoPos += getShort();

	ifoPtr =ifoData + pgcCellPosInfoPos;

	for(i=0;i<titleData[titleIndex]->numCells;i++)
		{
		vobId =  getShort();
		ifoPtr++;
		cellId = getChar();
		}

	ifoPtr =ifoData + pgcPos+0xe8;

	pgcCellPlaybackInfoPos = pgcPos;
	pgcCellPlaybackInfoPos += getShort();

	// Position us at the start of the cell playback information table
	ifoPtr =ifoData + pgcCellPlaybackInfoPos;

	for(i=0;i<titleData[titleIndex]->numCells;i++)
		{
		flags=getInt();

		titleData[titleIndex]->cellFlags[i] = flags;
		playTime = getInt();
		
		hours = (playTime&0xff000000)>>24;
		minutes = (playTime&0x00ff0000)>>16;
		seconds = (playTime&0x0000ff00)>>8;
		frames = playTime&0x0000003f;

		switch(playTime&0x000000c0)
			{
			case 0x40:
				framerate = 25;
				break;
			
			case 0xc0:
			default:
				framerate = 30;
				break;
			}

		titleData[titleIndex]->cellNumFrames[i] = bcdToHex(hours) * 60 * 60 * framerate;
		titleData[titleIndex]->cellNumFrames[i] += bcdToHex(minutes) * 60 * framerate;
		titleData[titleIndex]->cellNumFrames[i] += bcdToHex(seconds) * framerate;
		titleData[titleIndex]->cellNumFrames[i] += bcdToHex(frames);

		titleData[titleIndex]->cellFramerate[i] = framerate;

		firstVobuStart = getInt();
		firstIlvuEnd = getInt();
		lastVobuStart = getInt();
		lastVobuEnd = getInt();

		titleData[titleIndex]->cellLBAStart[i]=firstVobuStart;
		}


	/* this menthod was found to be unreliable..*/
	// // Create the chapter entries in the fileInfo struct
	//fileInfo->chaptersFound=titleData[titleIndex]->numPrograms;
	//
	// // Subtract off the position info for the first chapter
	//cell = titleData[titleIndex]->programMap[0]-1;
	//startOffset = titleData[titleIndex]->cellLBAStart[cell];
	//
	//	for(i=0;i<titleData[titleIndex]->numPrograms;i++)
	//	{
	//	cell = titleData[titleIndex]->programMap[i]-1;
	//	fileInfo->chapterStartFilePos[i] = (titleData[titleIndex]->cellLBAStart[cell]-startOffset);
	//	// Convert the LBA position to a byte pos.
	//	fileInfo->chapterStartFilePos[i]*=0x800;
	//	}
	// fileInfo->chapterStartFilePos[fileInfo->chaptersFound] = 0x7fffffffffffffff;
	// fileInfo->chaptersFound++;

	frameNumber = 0;
	chapterCounter = 0;
	for(i=0;i<titleData[titleIndex]->numCells;i++)
		{
		cell = titleData[titleIndex]->programMap[chapterCounter]-1;
		if(i==cell)
			{
			fileInfo->chapterStartFramePos[chapterCounter]=frameNumber;
			chapterCounter++;
			}
		frameNumber+= titleData[titleIndex]->cellNumFrames[i];
		}

	fileInfo->chapterStartFramePos[chapterCounter] = 0x7fffffff;

	fileInfo->chaptersFound = chapterCounter;

	delete [] titleData[0];

}


// *****************************************************************************
// ifoClass::getInt()
//
// get an unsigned int from the IFO Data buffer 
// *****************************************************************************
unsigned int ifoClass::getInt()
{
	unsigned int val;

	val = *ifoPtr++;
	val=val<<8;
	val |= *ifoPtr++;
	val=val<<8;
	val |= *ifoPtr++;
	val=val<<8;
	val |= *ifoPtr++;

	return(val);
}

// *****************************************************************************
// ifoClass::getChar()
//
// get an unsigned char from the IFO Data buffer 
// *****************************************************************************
unsigned char ifoClass::getChar()
{

	return (*ifoPtr++);
}

// *****************************************************************************
// ifoClass::getShort()
//
// get an unsigned short from the IFO Data buffer 
// *****************************************************************************
unsigned short ifoClass::getShort()
{
	unsigned short val;

	val = *ifoPtr++;
	val=val<<8;
	val |= *ifoPtr++;

	return (val);
}

// *****************************************************************************
// ifoClass::bcdToHex()
//
// convert small bcd numbers to hex. This will fail on large bcd numbers...
// *****************************************************************************
int ifoClass::bcdToHex(int bcd)
{
	int result;

	result = ((bcd&0xfffffff0) * 10) / 16;

	result += bcd&0x0000000f;

	return(result);
}