// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// job.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

char * vidomiJobListText = "All\0*.*\0Vidomi Job List Files\0*.vidomijoblist;\0\0";

static TRANSLATEIDTABLE jobControlTID[] = {
										{"VidomiJobSelectorClearJobList",VidomiJobSelectorClearJobList},
										{"VidomiJobSelectorDeleteCurrentJob",VidomiJobSelectorDeleteCurrentJob},
										{"VidomiJobSelectorAddNewJob",VidomiJobSelectorAddNewJob},

										{"",0}																			
										};
// *****************************************************************************
// jobClass::jobClass()
//
// conversion class constructor
// *****************************************************************************
jobClass::jobClass()
{
	visible = false;
	windowsCreated = false;
	jobStr = new char [1024];
}

// *****************************************************************************
// jobClass::~jobClass()
//
// conversion class destructor
// *****************************************************************************
jobClass::~jobClass()
{

	if(jobhWnd && windowsCreated)
		DestroyWindow(jobhWnd);

	if(jobStr)
		delete [] jobStr;
}

// *****************************************************************************
// jobClass::createOutputWindow()
//
// conversion class constructor
// *****************************************************************************
bool jobClass::createWindow(HINSTANCE hInst)
{
	jobhInst = hInst;

	return(true);
}

// *****************************************************************************
// jobClass::showJobWindow()
//
// 
// *****************************************************************************
bool jobClass::showWindow()
{
	char text[4096];
	int i;

	if(!windowsCreated)
		{
		RECT client;
		int fullHeight;
		int fullWidth;

		fullHeight = GetSystemMetrics(SM_CYSCREEN);
		fullWidth = GetSystemMetrics(SM_CXSCREEN);

		if((globals->jobLeft > fullWidth) || globals->jobLeft < 0)
			globals->jobLeft = 320;

		if((globals->jobTop > fullHeight) || globals->jobTop < 0)
			globals->jobTop = 240;

		jobhWnd = CreateDialog(jobhInst,MAKEINTRESOURCE(VidomiJobSelector),NULL, jobDialogHandler);


		globals->translate->translate(text, "VidomiJobSelector");
		SetWindowText(jobhWnd, text);			

		for(i=0;;i++)
			{
			if(!jobControlTID[i].id)
				break;
			globals->translate->translate(text, jobControlTID[i].textId);
			SetWindowText(GetDlgItem(jobhWnd, jobControlTID[i].id),text);			
			}

		jobMenu = LoadMenu(jobhInst,MAKEINTRESOURCE(VidomiJobMenu));

		globals->translate->translateMenu(jobMenu);

		SetMenu(jobhWnd,jobMenu);

		GetWindowRect(jobhWnd, &client);
		MoveWindow(jobhWnd, globals->jobLeft, globals->jobTop, client.right - client.left, client.bottom - client.top, FALSE);
		windowsCreated = true;
		}


	if(globals->encodeServerEnabled)
		{
		EnableMenuItem(jobMenu,ViomiJobMenuSubmitJobListToEncodeServer,MF_BYCOMMAND | MF_ENABLED);
		}
	else
		{
		EnableMenuItem(jobMenu,ViomiJobMenuSubmitJobListToEncodeServer,MF_BYCOMMAND | MF_GRAYED | MF_DISABLED);
		}

	updateJobWindow();
	ShowWindow(jobhWnd, SW_SHOW);	
	BringWindowToTop(jobhWnd);
	visible = true;
	return(true);
}

// *****************************************************************************
// jobClass::updateJobWindow()
//
// 
// *****************************************************************************
bool jobClass::updateJobWindow()
{
	char text[4096];

	char jobText[MAX_PATH];
	char jobTextOut[MAX_PATH];
	int i;
	char * ptr;

	if(windowsCreated)
		{
		SendDlgItemMessage(jobhWnd,VidomiJobSelectorJobList, LB_RESETCONTENT  , (WPARAM)0, (LPARAM) 0); 									

		for(i=0;i<=globals->jobCount;i++)
			{
			sprintf(jobText,"%s",globals->jobList[i].convertConfig->outputFilename);
			if(!globals->jobList[i].inputValidated || !globals->jobList[i].outputSet)
				{
				globals->translate->translate(jobText, "VidomiJobIncomplete");		
				globals->translate->translate(text, "VidomiJobJob");		
				sprintf(jobTextOut,"%s %d - %s",text,i+1, jobText);
				}
			else
				{
				ptr = strrchr(jobText,'\\');
				if(ptr)
					{
					ptr++;
					globals->translate->translate(text, "VidomiJobJob");		
					sprintf(jobTextOut,"%s %d - %s",text,i+1, ptr);
					}
				else
					{
					globals->translate->translate(text, "VidomiJobJob");		
					sprintf(jobTextOut,"%s %d - %s",text,i+1, jobText);
					}
				}
			SendDlgItemMessage(jobhWnd,VidomiJobSelectorJobList, LB_INSERTSTRING , (WPARAM)i, 
				(LPARAM) jobTextOut); 									
			}
		}

	return(true);
}
// *****************************************************************************
// jobClass::hideOutputWindow()
//
// 
// *****************************************************************************
bool jobClass::hideWindow()
{
	if(windowsCreated)
		{
		windowsCreated = false;
		ShowWindow(jobhWnd, SW_HIDE);
		DestroyWindow(jobhWnd);
		}
	visible = false;
	return(true);
}

// *****************************************************************************
// jobClass::windowVisible()
//
// 
// *****************************************************************************
bool jobClass::windowVisible()
{
	return(visible);
}

// *****************************************************************************
// jobClass::jobDialogHandler()
//
//
// *****************************************************************************
BOOL CALLBACK jobClass::jobDialogHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->job->jobDialogHandler1(hWnd,message,wParam,lParam));
}
// *****************************************************************************
// jobClass::jobDialogHandler1()
//
//
// *****************************************************************************
BOOL jobClass::jobDialogHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	RECT win;
	int ret;
	unsigned short commandId;
	unsigned short commandEvent;
	OPENFILENAME ofn;
	char filename[MAX_PATH];
	FLATJOBENTRY * flatJobEntry;
	int jobListIndex;
	int status;
	char messTitle[4096];
	char messText[4096];

	switch(message)
		{
		case WM_MOVE :
			GetWindowRect(hWnd, &win);
			globals->jobLeft = win.left;
			globals->jobTop  = win.top;    
			return(TRUE);

		case WM_CLOSE:
			hideWindow();
			return(TRUE);

		case WM_COMMAND:
            commandId    = LOWORD(wParam); // Remember, these are...
			commandEvent = HIWORD(wParam); // ...different for Win32!
			//Parse the menu selections:
			switch (commandId) 
				{
				case VidomiJobSelectorJobList:
					if(commandEvent==CBN_SELCHANGE)
						{
						if(!globals->optionsUp)
							{
							ret = SendDlgItemMessage(hWnd,VidomiJobSelectorJobList, LB_GETCURSEL  , 0, 0);
							if(ret>=0)
								{
								globals->jobIndex = ret;
								setJobIndex(ret);
								}
							}
						else
							{
							SendDlgItemMessage(hWnd,VidomiJobSelectorJobList, LB_SETCURSEL  , globals->jobIndex, 0);
							
							}
						}	
					return(TRUE);

				case ViomiJobMenuClearJobList:
				case VidomiJobSelectorClearJobList:
					resetJobList();
					updateJobWindow();
					break;

				case VidomiJobSelectorAddNewJob:
					addToJobList();
					setJobIndex(globals->jobCount);
				break;
				case VidomiJobSelectorDeleteCurrentJob:
					deleteJobIndex(globals->jobIndex);
					updateJobWindow();
					break;

				case ViomiJobMenuSaveJobList:
					strcpy(filename,"");
					memset(&ofn, 0 , sizeof(OPENFILENAME));
					ofn.lStructSize = sizeof(OPENFILENAME);
					ofn.hwndOwner = hWnd;
					ofn.hInstance = globals->hInst;
					ofn.lpstrFile = filename;
					ofn.nMaxFile = MAX_PATH;
					ofn.lpstrFilter = vidomiJobListText;
					ofn.nFilterIndex = globals->browseJobListDefaultType;
					ofn.lpstrFileTitle = NULL;
					ofn.nMaxFileTitle = 0;
					ofn.lpstrInitialDir = globals->browseJobListPath;
					ofn.Flags = OFN_PATHMUSTEXIST ;
					if(GetSaveFileName(&ofn) != FALSE)
						{
						_getcwd(globals->browseJobListPath,MAX_PATH);
						globals->browseJobListDefaultType = ofn.nFilterIndex;
					
						if(!strstr(filename,".vidomijoblist"))
							{
							strcat(filename,".vidomijoblist");
							}
						saveJobList(filename);
						}
					break;

				case ViomiJobMenuLoadJobList:
					strcpy(filename,"");
					memset(&ofn, 0 , sizeof(OPENFILENAME));
					ofn.lStructSize = sizeof(OPENFILENAME);
					ofn.hwndOwner = hWnd;
					ofn.hInstance = globals->hInst;
					ofn.lpstrFile = filename;
					ofn.nMaxFile = MAX_PATH;
					ofn.lpstrFilter = vidomiJobListText;
					ofn.nFilterIndex = globals->browseJobListDefaultType;
					ofn.lpstrFileTitle = NULL;
					ofn.nMaxFileTitle = 0;
					ofn.lpstrInitialDir = globals->browseJobListPath;
					ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;
					if(GetOpenFileName(&ofn) != FALSE)
						{
						_getcwd(globals->browseJobListPath,MAX_PATH);
						globals->browseJobListDefaultType = ofn.nFilterIndex;
						loadJobList(filename, true);
						globals->window->doneConverting();
						updateJobWindow();
						}
					break;

				case ViomiJobMenuAppendJobList:
					strcpy(filename,"");
					memset(&ofn, 0 , sizeof(OPENFILENAME));
					ofn.lStructSize = sizeof(OPENFILENAME);
					ofn.hwndOwner = hWnd;
					ofn.hInstance = globals->hInst;
					ofn.lpstrFile = filename;
					ofn.nMaxFile = MAX_PATH;
					ofn.lpstrFilter = vidomiJobListText;
					ofn.nFilterIndex = globals->browseJobListDefaultType;
					ofn.lpstrFileTitle = NULL;
					ofn.nMaxFileTitle = 0;
					ofn.lpstrInitialDir = globals->browseJobListPath;
					ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;
					if(GetOpenFileName(&ofn) != FALSE)
						{
						_getcwd(globals->browseJobListPath,MAX_PATH);
						globals->browseJobListDefaultType = ofn.nFilterIndex;
						loadJobList(filename, false);
						globals->window->doneConverting();
						updateJobWindow();
						}
					break;

				case ViomiJobMenuSubmitJobListToEncodeServer:
					// Add the current job if valid
					if(globals->jobList[globals->jobCount].outputSet && globals->jobList[globals->jobCount].inputValidated)
						{
						globals->jobIndex = globals->jobCount;
						globals->job->addToJobList();
						globals->job->setJobIndex(globals->jobIndex);
						}

					if(globals->jobCount==0)
						{
						globals->translate->translate(messText, "VidomiMessage43");
						globals->translate->translate(messTitle, "VidomiMessage43Title");
						vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
						break;
						}

					if(globals->jobServer->connectJobServer(globals->encodeServerIPAddress))
						{
						flatJobEntry = new FLATJOBENTRY [1];

						jobListIndex = 0;
						status = true;
						while(status && (jobListIndex<(globals->jobCount+1)))
							{
							if(globals->jobList[jobListIndex].outputSet && globals->jobList[jobListIndex].inputValidated)
								{
								globals->jobHelper->flattenJobEntry(&globals->jobList[jobListIndex],flatJobEntry);
								status = globals->jobServer->sendJob(flatJobEntry);	
								}
							jobListIndex++;
							}

						delete [] flatJobEntry;

						if(status)
							{
							globals->jobServer->sendClose();
							globals->jobServer->disconnectJobServer();
							globals->translate->translate(messText, "VidomiMessage41");
							globals->translate->translate(messTitle, "VidomiMessage41Title");
							vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK);
							resetJobList();
							updateJobWindow();
							}
						else
							{
							globals->jobServer->disconnectJobServer();
							globals->translate->translate(messText, "VidomiMessage42");
							globals->translate->translate(messTitle, "VidomiMessage42Title");
							vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
							}
						
						}
					else
						{
						// Error out
						char messTitle[4096];
						char messText[4096];
						globals->translate->translate(messText, "VidomiMessage40");
						globals->translate->translate(messTitle, "VidomiMessage40Title");
						vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
						}
					break;
				case ViomiJobMenuGetEncodeServerStatus:
					getEncodeServerStatus();
					break;

				default:
					break;
				}
			break;
		}

	return (FALSE);
}


// *****************************************************************************
// jobClass::saveJobList()
//
// Saves The JobList to Disk
// *****************************************************************************
bool jobClass::saveJobList(char * filename)
{
	FLATJOBENTRY * flatJobEntry;
	JOBFILEHEADER jobFileHeader;
	FILE * output;
	int i;
	int jobsToSave;

	jobsToSave = 0;

	for(i=0;i<(globals->jobCount+1);i++)
		{
		if(globals->jobList[i].inputValidated && globals->jobList[i].outputSet)
			{
			jobsToSave++;
			}
		}

	memset(&jobFileHeader,0,sizeof(jobFileHeader));
	strcpy(jobFileHeader.jobToken,"VidomiJobList");
	jobFileHeader.numJobs = jobsToSave;
	jobFileHeader.sizeOfJobEntry = sizeof(FLATJOBENTRY);

	output = fopen(filename,"wb");
	
	if(!output)
		{
		char messTitle[4096];
		char messText[4096];
		globals->translate->translate(messText, "VidomiMessage39");
		globals->translate->translate(messTitle, "VidomiMessage39Title");
		vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);		
		return(false);
		}

	if(fwrite(&jobFileHeader,sizeof(JOBFILEHEADER),1,output)!=1)
		{
		// Pop up an error message
		fclose(output);	
		char messTitle[4096];
		char messText[4096];
		globals->translate->translate(messText, "VidomiMessage39");
		globals->translate->translate(messTitle, "VidomiMessage39Title");
		vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
		return(false);
		}

	flatJobEntry = new FLATJOBENTRY [1];

	for(i=0;i<(globals->jobCount+1);i++)
		{
		if(globals->jobList[i].inputValidated && globals->jobList[i].outputSet)
			{

			globals->jobHelper->flattenJobEntry(&globals->jobList[i],flatJobEntry);

			if(fwrite(flatJobEntry,sizeof(FLATJOBENTRY),1,output)!=1)
				{	
				// Pop up an error message
				delete [] flatJobEntry;
				fclose(output);
				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage39");
				globals->translate->translate(messTitle, "VidomiMessage39Title");
				vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);			
				return(false);
				}
			}
		}
	fclose(output);	
	
	delete [] flatJobEntry;

	return(true);

}

// *****************************************************************************
// jobClass::saveJobList()
//
// Loads The Joblist From Disk
// *****************************************************************************
bool jobClass::loadJobList(char * filename, bool clearFirst)
{
	JOBFILEHEADER jobFileHeader;
	FLATJOBENTRY * flatJobEntry;
	FILE * input;
	int i;

	input = fopen(filename,"rb");

	if(!input)
		{
		char messTitle[4096];
		char messText[4096];
		globals->translate->translate(messText, "VidomiMessage37");
		globals->translate->translate(messTitle, "VidomiMessage37Title");
		vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
		return(false);
		}

	if(fread(&jobFileHeader,sizeof(JOBFILEHEADER),1,input)!=1)
		{
		fclose(input);

		char messTitle[4096];
		char messText[4096];
		globals->translate->translate(messText, "VidomiMessage37");
		globals->translate->translate(messTitle, "VidomiMessage37Title");
		vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
		return(false);
		}
	if(!((!strcmp(jobFileHeader.jobToken,"VidomiJobList")) && jobFileHeader.sizeOfJobEntry == sizeof(FLATJOBENTRY)))
		{
		// Failed. Header is missing or old version
		char messTitle[4096];
		char messText[4096];
		globals->translate->translate(messText, "VidomiMessage37");
		globals->translate->translate(messTitle, "VidomiMessage37Title");
		vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
		fclose(input);
		return(false);
		}

	// Clear all current jobs

	if(clearFirst)
		{
		resetJobList();
		}
	else
		{
		// Get the current selected Audio Track
		//ret = SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_GETCURSEL  , 0, 0);
		//ret = SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_GETITEMDATA , ret, 0);
		//globals->convertConfig->processAudioSelected[0] = ret;
		}

	flatJobEntry = new FLATJOBENTRY [1];

	for(i=0;i<jobFileHeader.numJobs;i++)
		{
		if(fread(flatJobEntry,sizeof(FLATJOBENTRY),1,input)!=1)
			{
			char messTitle[4096];
			char messText[4096];
			globals->translate->translate(messText, "VidomiMessage38");
			globals->translate->translate(messTitle, "VidomiMessage38Title");
			vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
			break;
			}
		
		globals->jobHelper->unflattenJobEntry(&globals->jobList[globals->jobCount],flatJobEntry);

		if(globals->jobCount == MAX_JOBS)
			{
			char messTitle[4096];
			char messText[4096];
			globals->translate->translate(messText, "VidomiMessage38");
			globals->translate->translate(messTitle, "VidomiMessage38Title");
			vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
			break;
			}
		globals->jobCount++;
		
		makeNewJobEntry();
		}
	fclose(input);

	refresh();
	globals->convertConfig = globals->jobList[globals->jobIndex].convertConfig;
	globals->gatherResults = globals->jobList[globals->jobIndex].gatherResults;

	if(windowsCreated)
		{
		SendDlgItemMessage(jobhWnd,VidomiJobSelectorJobList, LB_SETCURSEL  , globals->jobIndex, 0);
		}

	delete [] flatJobEntry;


	return(true);
}

// *****************************************************************************
// jobClass::addToJobList()
//
// Adds the current conversion to the job List
// *****************************************************************************
bool jobClass::addToJobList()
{
	char * ptr;
	int i;

	
	if(!globals->jobList[globals->jobCount].outputSet || !globals->jobList[globals->jobCount].inputValidated)
		{
		char messTitle[4096];
		char messText[4096];
		globals->translate->translate(messText, "VidomiMessage21");
		globals->translate->translate(messTitle, "VidomiMessage21Title");

		vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
		return(false);
		}
	
	// Get the audio track selected
	//ret = SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_GETCURSEL  , 0, 0);
	//ret = SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_GETITEMDATA , ret, 0);
	
	//globals->convertConfig->processAudioSelected[0] = ret;

	// Scan for the Crop rect
	globals->scanFile->scanCropRect(globals->jobList[globals->jobCount].convertConfig, true);
	globals->scanFile->determineRangeSize(globals->jobList[globals->jobCount].convertConfig);
	globals->jobList[globals->jobCount].cropScanDone =true;
	ptr = strrchr(globals->jobList[globals->jobCount].convertConfig->outputFilename,'\\');
	if(ptr)
		{
		ptr++;
		strcpy(globals->jobList[globals->jobCount].jobName,ptr);
		}
	else
		{
		strcpy(globals->jobList[globals->jobCount].jobName,globals->jobList[globals->jobCount].convertConfig->outputFilename);
		}

	// Remove the tail on the job name we gnerated
	ptr = strrchr(globals->jobList[globals->jobCount].jobName,'.');
	if(ptr)
		{
		*ptr = 0;
		}

	// Verify that no other crop scans are needed
	for(i=0;i<globals->jobCount;i++)
		{
		if(!globals->jobList[i].cropScanDone)
			{
			globals->scanFile->scanCropRect(globals->jobList[i].convertConfig, true);
			globals->scanFile->determineRangeSize(globals->jobList[i].convertConfig);
			globals->jobList[i].cropScanDone =true;
			}
		}

	if(globals->jobCount<MAX_JOBS)
		{
		// Now Duplicate the config to create a new fresh config
		globals->jobCount++;
		makeNewJobEntry();
		}


	return(true);

}

// *****************************************************************************
// jobClass::makeNewJobEntry()
//
// Allocate a clean job struture at the head of the job list
// *****************************************************************************
void jobClass::makeNewJobEntry()
{
	int i;

	globals->jobList[globals->jobCount].convertConfig = new CONVERTCONFIG [1];

	//first copy the configuration
	memcpy(globals->jobList[globals->jobCount].convertConfig,globals->convertConfig,sizeof(CONVERTCONFIG));
	
	// Now replace the filnames that were allocated with new buffers
	strcpy(globals->jobList[globals->jobCount].convertConfig->outputFilename,"");

	for(i=0;i<MAX_INPUT_FILES;i++)
		{
		strcpy(globals->jobList[globals->jobCount].convertConfig->inputFilenames[i],"");
		}

	globals->jobList[globals->jobCount].convertConfig->videoFormatOut = new COMPVARS [1];
	memcpy(globals->jobList[globals->jobCount].convertConfig->videoFormatOut, globals->convertConfig->videoFormatOut, sizeof(COMPVARS));
	globals->jobList[globals->jobCount].convertConfig->videoFormatOut->lpbiOut =  (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
	globals->jobList[globals->jobCount].convertConfig->videoFormatOut->lpState = new char [VIDEO_STATE_DEFAULT_SIZE];
	memcpy(globals->jobList[globals->jobCount].convertConfig->videoFormatOut->lpbiOut, globals->convertConfig->videoFormatOut->lpbiOut, VIDOMI_BITMAP_INFO_SIZE);
	memcpy(globals->jobList[globals->jobCount].convertConfig->videoFormatOut->lpState, globals->convertConfig->videoFormatOut->lpState, globals->convertConfig->videoFormatOut->cbState);
	globals->jobList[globals->jobCount].convertConfig->videoFormatOut->hic = NULL;

	globals->jobList[globals->jobCount].convertConfig->videoFormatOutP2 = new COMPVARS [1];
	memcpy(globals->jobList[globals->jobCount].convertConfig->videoFormatOutP2, globals->convertConfig->videoFormatOutP2, sizeof(COMPVARS));
	globals->jobList[globals->jobCount].convertConfig->videoFormatOutP2->lpbiOut = (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
	globals->jobList[globals->jobCount].convertConfig->videoFormatOutP2->lpState = new char [VIDEO_STATE_DEFAULT_SIZE];
	memcpy(globals->jobList[globals->jobCount].convertConfig->videoFormatOutP2->lpbiOut, globals->convertConfig->videoFormatOutP2->lpbiOut, VIDOMI_BITMAP_INFO_SIZE);
	memcpy(globals->jobList[globals->jobCount].convertConfig->videoFormatOutP2->lpState, globals->convertConfig->videoFormatOutP2->lpState, globals->convertConfig->videoFormatOutP2->cbState);
	globals->jobList[globals->jobCount].convertConfig->videoFormatOutP2->hic = NULL;

	globals->jobList[globals->jobCount].convertConfig->audioFormatOut  = (MPEGLAYER3WAVEFORMAT *) new char [AUDIO_STATE_DEFAULT_SIZE];
	memcpy(globals->jobList[globals->jobCount].convertConfig->audioFormatOut,globals->convertConfig->audioFormatOut, AUDIO_STATE_DEFAULT_SIZE);

	globals->jobList[globals->jobCount].gatherResults = new GATHERRESULTS [1];
	memset(globals->jobList[globals->jobCount].gatherResults,0,sizeof(GATHERRESULTS));

	globals->jobList[globals->jobCount].convertConfig->inputFilenameIndex=0;
	globals->jobList[globals->jobCount].inputValidated = false;
	globals->jobList[globals->jobCount].outputSet = false;
	globals->jobList[globals->jobCount].cropScanDone = false;

	// Set the range selections to not being Valid
	for(i=0;i<MAX_RANGES;i++)
		{
		globals->jobList[globals->jobCount].convertConfig->rangelist[i].valid=false;
		globals->jobList[globals->jobCount].convertConfig->rangelist[i].startingFilePos = 0;
		globals->jobList[globals->jobCount].convertConfig->rangelist[i].startingFileFrame=0;
		globals->jobList[globals->jobCount].convertConfig->rangelist[i].endingFilePos=0x7fffffffffffffff;
		globals->jobList[globals->jobCount].convertConfig->rangelist[i].endingFileFrame=0;
		}
	
	// Set the default range values to the max range

	globals->jobList[globals->jobCount].convertConfig->startingFilePos=0;
	globals->jobList[globals->jobCount].convertConfig->startingFileFrame=0;
	globals->jobList[globals->jobCount].convertConfig->endingFilePos=0x7fffffffffffffff;
	globals->jobList[globals->jobCount].convertConfig->endingFileFrame=0;
	
	// Guarantee that the default state is one valid range
	globals->jobList[globals->jobCount].convertConfig->rangelist[0].startingFilePos = 0;
	globals->jobList[globals->jobCount].convertConfig->rangelist[0].startingFileFrame=0;
	globals->jobList[globals->jobCount].convertConfig->rangelist[0].endingFilePos=0x7fffffffffffffff;
	globals->jobList[globals->jobCount].convertConfig->rangelist[0].endingFileFrame=0;
	globals->jobList[globals->jobCount].convertConfig->rangelist[0].valid=true;

}

// *****************************************************************************
// jobClass::resetJobList()
//
// Clears the current conversion to the job List
// *****************************************************************************
bool jobClass::setJobIndex(int index)
{

	if(index<=globals->jobCount)
		{
		// Get the audio track selected
		//ret = SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_GETCURSEL  , 0, 0);
		//ret = SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_GETITEMDATA , ret, 0);	
		//globals->convertConfig->processAudioSelected[0] = ret;
		
		// Get the current output name
		//GetWindowText(GetDlgItem(globals->window->encoderhWnd, VidomiMainOutputName), globals->convertConfig->outputFilename , MAX_PATH);
		globals->window->skin->getText("outputnametext",globals->convertConfig->outputFilename , MAX_PATH);

		globals->jobIndex = index;
		globals->convertConfig = globals->jobList[globals->jobIndex].convertConfig;
		globals->gatherResults = globals->jobList[globals->jobIndex].gatherResults;

		refresh();
		}
	if(windowsCreated)
		{
		SendDlgItemMessage(jobhWnd,VidomiJobSelectorJobList, LB_SETCURSEL  , globals->jobIndex, 0);
		}
	return(true);
}

// *****************************************************************************
// jobClass::resetJobList()
//
// Clears the current conversion to the job List
// *****************************************************************************
bool jobClass::refresh()
{
	int i;
	bool tempOutputSet;

	tempOutputSet = globals->jobList[globals->jobIndex].outputSet;
	
	globals->window->skin->comboBoxReset("inputnametext");
	globals->window->skin->setSliderPos("inputnameslider", 0);
	//SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainInputList, LB_RESETCONTENT  , (WPARAM)0, (LPARAM) 0); 									
	//SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_RESETCONTENT , 0, 0);

	//EnableWindow(GetDlgItem(globals->window->encoderhWnd,VidomiMainAudioTrack), FALSE);
	if(globals->jobList[globals->jobIndex].inputValidated)
		{
		//if(globals->gatherResults->audioFound)
		//	{
			//EnableWindow(GetDlgItem(globals->window->encoderhWnd,VidomiMainAudioTrack), TRUE);
			// Set the audio pointer back to the one selected and repopulate the audio list
			//for(i=0;i<globals->gatherResults->audioFound;i++)
			//	{
			//	ret = SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_ADDSTRING , (WPARAM)i , (LPARAM) &globals->gatherResults->audioName[i]); 									
			//	SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_SETITEMDATA  , ret, i);
			//	if(i==globals->convertConfig->processAudioSelected[0])
			//		SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_SETCURSEL  , ret, 0);
			//	}
		//	}

		for(i=0;i<globals->convertConfig->inputFilenameIndex;i++)
			{
			globals->window->skin->comboBoxInsertString("inputnametext",globals->convertConfig->inputFilenames[i]);

			//SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainInputList, LB_INSERTSTRING , (WPARAM)i, 
			//	(LPARAM) globals->convertConfig->inputFilenames[i]); 									
			}
		}

	// Setting the output name triggers the "outputset". We set it back with the temp value
	//SetWindowText(GetDlgItem(globals->window->encoderhWnd, VidomiMainOutputName), globals->convertConfig->outputFilename);
	globals->window->skin->setText("outputnametext",globals->convertConfig->outputFilename);
	globals->jobList[globals->jobIndex].outputSet = tempOutputSet;

	if(globals->jobList[globals->jobIndex].outputSet)
		{
		//EnableWindow(GetDlgItem(globals->window->encoderhWnd,VidomiMainStartConversion), TRUE);
		}

	return(true);
}
// *****************************************************************************
// jobClass::deleteJobIndex()
//
// Clears the current conversion to the job List
// *****************************************************************************
bool jobClass::deleteJobIndex(int index)
{
	int i;
	CONVERTCONFIG *	 tempConvertConfig;
	GATHERRESULTS *	 tempGatherResults;
	bool tempInputValidated;
	bool tempOutputSet;
	bool tempCropScanDone;

	if(index>globals->jobCount)
		return(false);

	if(globals->jobCount==0)
		{
		resetJobList();
		return(true);
		}

	for(i=index;i<globals->jobCount;i++)
		{
		tempConvertConfig = globals->jobList[i].convertConfig;
		tempGatherResults = globals->jobList[i].gatherResults;
		tempInputValidated = globals->jobList[i].inputValidated;
		tempOutputSet = globals->jobList[i].outputSet;
		tempCropScanDone = globals->jobList[i].cropScanDone;

		globals->jobList[i].convertConfig = globals->jobList[i+1].convertConfig;
		globals->jobList[i].gatherResults = globals->jobList[i+1].gatherResults;
		globals->jobList[i].inputValidated = globals->jobList[i+1].inputValidated;
		globals->jobList[i].outputSet = globals->jobList[i+1].outputSet;
		globals->jobList[i].cropScanDone = globals->jobList[i+1].cropScanDone;
		globals->jobList[i+1].convertConfig = tempConvertConfig;
		globals->jobList[i+1].gatherResults = tempGatherResults;
		globals->jobList[i+1].inputValidated = tempInputValidated;
		globals->jobList[i+1].outputSet = tempOutputSet;
		globals->jobList[i+1].cropScanDone = tempCropScanDone;
		}
	
	if(globals->jobIndex>0)
		{
		globals->jobIndex = globals->jobIndex-1;
		}

	setJobIndex(globals->jobIndex);

	clearJob(globals->jobCount);
	globals->jobCount--;

	updateJobWindow();
	if(windowsCreated)
		{
		SendDlgItemMessage(jobhWnd,VidomiJobSelectorJobList, LB_SETCURSEL  , globals->jobIndex, 0);
		}
	return(true);
}

// *****************************************************************************
// jobClass::resetJobList()
//
// Clears the current conversion to the job List
// *****************************************************************************
bool jobClass::resetJobList()
{
	int i;

	if(globals->jobCount!=0)
		{
		for(i=1;i<MAX_JOBS;i++)
			{
			clearJob(i);
			}
		}
	
	globals->jobCount = globals->jobIndex = 0;
	globals->jobList[globals->jobIndex].convertConfig->inputFilenameIndex = 0;
	strcpy(globals->jobList[globals->jobIndex].convertConfig->outputFilename,"");

	globals->convertConfig = globals->jobList[globals->jobIndex].convertConfig;
	globals->gatherResults = globals->jobList[globals->jobIndex].gatherResults;

	globals->window->skin->comboBoxReset("inputnametext");
	globals->window->skin->setSliderPos("inputnameslider", 0);
	globals->window->skin->setText("outputnametext",globals->convertConfig->outputFilename);
	globals->jobList[globals->jobIndex].outputSet = false;
	globals->jobList[globals->jobIndex].inputValidated = false;

/*
	SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainInputList, LB_RESETCONTENT  , (WPARAM)0, (LPARAM) 0); 									
	SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_RESETCONTENT , 0, 0);
	SetWindowText(GetDlgItem(globals->window->encoderhWnd, VidomiMainOutputName), globals->convertConfig->outputFilename);
	EnableWindow(GetDlgItem(globals->window->encoderhWnd,VidomiMainAudioTrack), FALSE);
	globals->jobList[globals->jobIndex].outputSet = false;
	globals->jobList[globals->jobIndex].inputValidated = false;
*/
	return(true);
}

// *****************************************************************************
// jobClass::clearJob()
//
// Clears one entry in the job List
// *****************************************************************************
void jobClass::clearJob(int index)
{
	if(globals->jobList[index].convertConfig)
		{
		logPrintf("Freeing jobList[%d].convertConfig\n",index);
		
		if(globals->jobList[index].convertConfig->videoFormatOut)
			{
			logPrintf("Freeing jobList[%d].convertConfig->videoFormatOut\n",index);

			if(globals->jobList[index].convertConfig->videoFormatOut->lpState)
				delete [] globals->jobList[index].convertConfig->videoFormatOut->lpState;
			if(globals->jobList[index].convertConfig->videoFormatOut->lpbiOut)
				delete [] globals->jobList[index].convertConfig->videoFormatOut->lpbiOut;

			delete [] globals->jobList[index].convertConfig->videoFormatOut;
			}
		if(globals->jobList[index].convertConfig->videoFormatOutP2)
			{
			logPrintf("Freeing jobList[%d].convertConfig->videoFormatOutP2\n",index);

			if(globals->jobList[index].convertConfig->videoFormatOutP2->lpState)
				delete [] globals->jobList[index].convertConfig->videoFormatOutP2->lpState;
			if(globals->jobList[index].convertConfig->videoFormatOutP2->lpbiOut)
				delete [] globals->jobList[index].convertConfig->videoFormatOutP2->lpbiOut;

			delete [] globals->jobList[index].convertConfig->videoFormatOutP2;
			}

		if(globals->jobList[index].convertConfig->audioFormatOut)
			{
			logPrintf("Freeing jobList[%d].convertConfig->audioFormatOut\n",index);
			delete [] globals->jobList[index].convertConfig->audioFormatOut;
			}
		delete [] globals->jobList[index].convertConfig;
		globals->jobList[index].convertConfig = NULL;
		}

	if(globals->jobList[index].gatherResults)
		{	
		logPrintf("Freeing jobList[%d].gatherResults\n",index);
		delete [] globals->jobList[index].gatherResults;
		globals->jobList[index].gatherResults = NULL;
		}

	globals->jobList[index].cropScanDone = false;
	globals->jobList[index].inputValidated = false;
	globals->jobList[index].outputSet = false;
}

// *****************************************************************************
// jobClass::encodeStatsDialogHandler()
//
//
// *****************************************************************************
BOOL CALLBACK jobClass::encodeStatsDialogHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->optionsGeneral->optionsDialogHandler1(hWnd,message,wParam,lParam));
}


// *****************************************************************************
// jobClass::encodeStatsDialogHandler()
//
//
// *****************************************************************************
BOOL jobClass::encodeStatsDialogHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
		{
		case WM_CLOSE:
			DestroyWindow(hWnd);
			break;
		}

	return (FALSE);

}

// *****************************************************************************
// jobClass::getEncodeServerStatus()
//
// Clears one entry in the job List
// *****************************************************************************
void jobClass::getEncodeServerStatus()
{
	JOBSERVERSTATS * jobServerStats;
	CONVERSIONSTATS * slaveStats;
	HWND statsHwnd;
	int outputLine;
	int i;
	char messText[4096];

	if(globals->jobServer->connectJobServer(globals->encodeServerIPAddress))
		{
		jobServerStats = globals->jobServer->getStatus();	
		globals->jobServer->sendClose();
		globals->jobServer->disconnectJobServer();

		if(jobServerStats)
			{
			statsHwnd = CreateDialog(globals->hInst,MAKEINTRESOURCE(VidomiEncodeServerStats),globals->hWnd, encodeStatsDialogHandler);
			SendDlgItemMessage(statsHwnd,VidomiEncodeServerStatsOutput, LB_RESETCONTENT  , (WPARAM)0, (LPARAM) 0); 									
			outputLine = 0;

			sprintf(messText,"%d Slaves Available / %d Slaves Running Jobs / %d Jobs In Queue",jobServerStats->numSlavesAvailable,
				jobServerStats->numJobsBeingProcessed, jobServerStats->numJobsInQueue);
			SendDlgItemMessage(statsHwnd,VidomiEncodeServerStatsOutput, LB_INSERTSTRING , (WPARAM)outputLine++, (LPARAM)messText);


			if(jobServerStats->numJobsInQueue)
				{
				SendDlgItemMessage(statsHwnd,VidomiEncodeServerStatsOutput, LB_INSERTSTRING , (WPARAM)outputLine++, (LPARAM)"");
				SendDlgItemMessage(statsHwnd,VidomiEncodeServerStatsOutput, LB_INSERTSTRING , (WPARAM)outputLine++, (LPARAM)"Next 16 Jobs In Queue :");
		
				for(i=0;i<16;i++)
					{
					if(jobServerStats->jobQueue[i].jobNumber!=-1)
						{
						sprintf(messText,"Job Queue %d - %s",i+1, jobServerStats->jobQueue[i].jobName);
						SendDlgItemMessage(statsHwnd,VidomiEncodeServerStatsOutput, LB_INSERTSTRING , (WPARAM)outputLine++, (LPARAM)messText);
						}
					}
				}

			SendDlgItemMessage(statsHwnd,VidomiEncodeServerStatsOutput, LB_INSERTSTRING , (WPARAM)outputLine++, (LPARAM)"");
			SendDlgItemMessage(statsHwnd,VidomiEncodeServerStatsOutput, LB_INSERTSTRING , (WPARAM)outputLine++, (LPARAM)"Slave Status :");

			slaveStats = jobServerStats->stats;

			for(i=0;i<jobServerStats->numStatsThatFollow;i++)
				{
				sprintf(messText,"Slave %d - Pass %d of %d - %.2f%% Completed - %.2f FPS - \"%s\"",i+1,slaveStats->passNumber, slaveStats->totalPasses, slaveStats->percentOfPassComplete, slaveStats->framesPerSecond, slaveStats->jobName);

				SendDlgItemMessage(statsHwnd,VidomiEncodeServerStatsOutput, LB_INSERTSTRING , (WPARAM)outputLine++, (LPARAM)messText);
				UpdateWindow(statsHwnd);
				slaveStats++;
				}

			delete [] jobServerStats;
			}
		}
	else
		{
		// Error out
		char messTitle[4096];
		char messText[4096];
		globals->translate->translate(messText, "VidomiMessage40");
		globals->translate->translate(messTitle, "VidomiMessage40Title");
		vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONERROR);
		}

}
