// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// mp3encode.cpp
// 
//
// *****************************************************************************
#include "en.h"

extern GLOBALS * globals;

// *****************************************************************************
// mp3EncodeClass::mp3EncodeClass()
//
// scanmpeg class constructor
// *****************************************************************************
mp3EncodeClass::mp3EncodeClass()
{
	pathToDll = new char [MAX_PATH];
	dllName = new char [MAX_PATH];

	lameDll = NULL;
	initStream = NULL;
	encodeChunk = NULL;
	deInitStream = NULL;
	closeStream = NULL;

}

// *****************************************************************************
// mp3EncodeClass::~mp3EncodeClass()
//
// scanmpeg class destructor
// *****************************************************************************
mp3EncodeClass::~mp3EncodeClass()
{
	end();

	delete [] pathToDll;
	delete [] dllName;

}

// *****************************************************************************
// mp3EncodeClass::init()
//
// initialize
// *****************************************************************************
bool mp3EncodeClass::init(int instance)
{
	if(instance==0)
		{
		strcpy(dllName,"lame_enc.dll");
		}
	else
		{
		sprintf(dllName,"lame_enc%d.dll",instance);		
		}
	strcpy(pathToDll,globals->pathToExeDir);
	strcat(pathToDll,"\\");
	strcat(pathToDll,dllName);
	lameDll = LoadLibrary(pathToDll);

	if(lameDll==NULL)
		{
		// If failed, make a second attempt at debug env location
		#ifdef _DEBUG
		strcpy(pathToDll,globals->pathToExeDir);
		strcat(pathToDll,"\\..\\lameenc\\");
		strcat(pathToDll,dllName);
		#else
		strcpy(pathToDll,globals->pathToExeDir);
		strcat(pathToDll,"\\..\\lameenc\\");
		strcat(pathToDll,dllName);
		#endif
		lameDll=LoadLibrary(pathToDll);
		if(lameDll==NULL)
			{
			#ifdef _DEBUG
			char messTitle[4096];
			char messText[4096];
			globals->translate->translate(messText, "VidomiMessage22");
			globals->translate->translate(messTitle, "VidomiMessage22Title");
			vidomiErrorBox(NULL,messText,messTitle,MB_OK);
			#endif
			return(false);
			}
		}

	initStream	= (INITSTREAM) GetProcAddress(lameDll, TEXT_INITSTREAM);
	encodeChunk	= (ENCODECHUNK) GetProcAddress(lameDll, TEXT_ENCODECHUNK);
	deInitStream	= (DEINITSTREAM) GetProcAddress(lameDll, TEXT_DEINITSTREAM);
	closeStream	= (CLOSESTREAM) GetProcAddress(lameDll, TEXT_CLOSESTREAM);

	if(!initStream || !encodeChunk || !deInitStream || !closeStream )
		return(false);

	return(true);
}

// *****************************************************************************
// mp3EncodeClass::end()
//
// end
// *****************************************************************************
void mp3EncodeClass::end()
{
	if(lameDll)
		{
		FreeLibrary(lameDll);
		lameDll = NULL;
		}

	initStream = NULL;
	encodeChunk = NULL;
	deInitStream = NULL;
	closeStream = NULL;

}

// *****************************************************************************
// mp3EncodeClass::config()
//
// configure the output
// *****************************************************************************
bool mp3EncodeClass::config(int inputFrequency,int outputFrequency, int inputChannels, 
							int encodeMode, int channelType, int quality, int minBitrate, 
							int maxBitrate, int vbrQuality, unsigned long * sizeOfMp3Buffer, 
							unsigned long * sizeOfWavBuffer)
{
	LAMECONFIG config;
	unsigned int wavBufferSize;
	unsigned int mp3BufferSize;

	memset(&config,0,sizeof(LAMECONFIG));

	config.config = LAME_CONFIG_LAME;
	config.structVersion = 1;
	config.structSize = sizeof(LAMECONFIG);
	config.sampleRate = inputFrequency;
	if(outputFrequency == 0)
		{
		outputFrequency = inputFrequency;
		}
	config.resampleRate = outputFrequency;

	switch(outputFrequency)
		{
		case 22050:
		case 24000:
		case 16000:
			config.mpegVersion = 0; // mpeg 1 = 1 //mpeg 2 =0;
			break;
		case 32000:
		case 44100:
		case 48000:
			config.mpegVersion = 1; // mpeg 1 = 1 //mpeg 2 =0;
			break;
		case 12000:
		case 11025:
		case 8000:
			config.mpegVersion = 0; // mpeg 1 = 1 //mpeg 2 =0;
			break;
		default:
			char messTitle[4096];
			char messText[4096];
			globals->translate->translate(messText, "VidomiMessage23");
			globals->translate->translate(messTitle, "VidomiMessage23Title");

			vidomiErrorBox(NULL,messText, messTitle,MB_OK);
			config.mpegVersion = 1; // mpeg 1 = 1 //mpeg 2 =0;
			break;

		}

	if(inputChannels == 1)
		{
		config.mode = LAME_MODE_MONO;
		}
	else
		{
		config.mode = channelType;
		}

	config.vbrQuality = vbrQuality;
	config.minBitrate = minBitrate;
	config.maxBitrate = maxBitrate;
	config.preset = quality; // 2 = High Quality  , 0 = NORMAL Quality
	config.psycoModel = 0;
	config.emphasis = 0;
	config.original = 1;
	config.noRes = 1;

	switch(encodeMode)
		{
		case LAME_VBR:
			config.enableVbr = 1;
			break;
		
		case LAME_ABR:
			config.dwVbrAbr_bps = minBitrate * 1000;
			config.enableVbr = 1;
		break;

		case LAME_CBR:
		default:
			config.enableVbr = 0;
			break;
		}
	if(initStream(&config, &wavBufferSize, &mp3BufferSize, &lameHandle))
		{
		return(false);
		}
	// need to return mp3BufferSize (size in bytes of the mp3 buffer)
	// need to return wavBufferSize (size in samples of the wav buffer * 2)
	*sizeOfMp3Buffer = mp3BufferSize;
	*sizeOfWavBuffer = wavBufferSize*2;

	return(true);

}

// *****************************************************************************
// mp3EncodeClass::encode()
//
// encode 
// *****************************************************************************
int mp3EncodeClass::encode(unsigned char * wavBuffer, unsigned char * mp3Buffer, int bytesToEncode)
{
	unsigned int bytesWritten;

	bytesWritten=0;
	// Encode samples
	if(encodeChunk(lameHandle, bytesToEncode, (short *)wavBuffer, mp3Buffer, &bytesWritten))
		{
		return(false);
		}
	
	return(bytesWritten);

}

// *****************************************************************************
// mp3EncodeClass::encodeEnd()
//
// We are done encoding, so end the encode process
// *****************************************************************************
int mp3EncodeClass::encodeEnd(unsigned char * mp3Buffer)
{
	unsigned int bytesWritten;

	bytesWritten=0;

	// Deinit the stream
	if(deInitStream(lameHandle, mp3Buffer, &bytesWritten))
		{
		return(false);
		}
	
	return(bytesWritten);

}