/* Copyright (C) 1996, MPEG Software Simulation Group. All Rights Reserved. */

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  The MPEG Software Simulation Group disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 * The MPEG Software Simulation Group does not represent or warrant that the
 * programs furnished hereunder are free of infringement of any third-party
 * patents.
 *
 * Commercial implementations of MPEG-1 and MPEG-2 video, including shareware,
 * are subject to royalty fees to patent holders.  Many of these patents are
 * general enough such that they are unavoidable regardless of implementation
 * design.
 *
 */

/*
 * All modifications (mpeg2decode -> DVD2AVI) are Copyright (C) Chia-chen Kuo - Jan 2001
 */

#include "global.h"
#include "getbit.h"
#include "AC3Dec\ac3.h"

#ifdef TRANSPORTSTREAM
unsigned char *Rdbfr = RawRdbfr;
#endif

/* mp3 decode*/
extern void mpegInit(void);
extern int mpegDecode(char * source, int sourceLength, char *dest);
extern int mpegPrepare(char * source, int sourceLength, int * channels, int * samplerate, int * framesize);

extern int videoFramesOutput;
extern int videoFramesNextPts;
//MAF was unsigned char
static char *FTType[4] = {
	"48KHz", "44.1KHz", "44.1KHz", "44.1KHz"
};

//MAF was unsigned char
static char *AC3ModeDash[8] = {
	"1+1", "1_0", "2_0", "3_0", "2_1", "3_1", "2_2", "3_2"
};

static int Packet_Length, Packet_Header_Length;
static unsigned int AC3_Track, MPA_Track, PCM_Track, Sub_Track;


__forceinline static unsigned int Get_Short(void);

void Initialize_Buffer()
{
#ifdef TRANSPORTSTREAM
	if (TransportStream_Flag)
	{
		BfrSize = BUFFER_SIZE;
		Rdbfr = RawRdbfr;
		Rdptr = Rdbfr + BfrSize;
		Rdmax = Rdptr;

		Fill_Buffer();

		TransportStream_AudContinuity = -1;
		TransportStream_AudReading = 0;

		TransportStream_VidContinuity = -1;
		TransportStream_VidReading = 0;

		TransportStreamSync_Flag = 0;

		NextTransportStreamPacket();

		if(Fault_Flag > CRITICAL_ERROR_LEVEL)
			return;

		CurrentBfr = (*Rdptr << 24) + (*(Rdptr+1) << 16) + (*(Rdptr+2) << 8) + *(Rdptr+3);
		Rdptr += 4;
		Fill_Next();
		BitsLeft = 32;

		return;
	}
	else
	{
		if (SystemStream_Flag)
			BfrSize = SYSTEM_BUFFER_SIZE;
		else
			BfrSize = BUFFER_SIZE;

		Rdbfr = RawRdbfr;
		Rdptr = Rdbfr + BfrSize;
		Rdmax = Rdptr;
	}

#else
	BfrSize = BUFFER_SIZE;
	Rdptr = Rdbfr + BfrSize;
	Rdmax = Rdptr;
#endif

	if (SystemStream_Flag)
		{
		if (Rdptr >= Rdmax)
			Next_Packet();
		CurrentBfr = *Rdptr++ << 24;

		if (Rdptr >= Rdmax)
			Next_Packet();
		CurrentBfr += *Rdptr++ << 16;

		if (Rdptr >= Rdmax)
			Next_Packet();
		CurrentBfr += *Rdptr++ << 8;

		if (Rdptr >= Rdmax)
			Next_Packet();
		CurrentBfr += *Rdptr++;

		Fill_Next();
		}
	else
		{
		Fill_Buffer();

		CurrentBfr = (*Rdptr << 24) + (*(Rdptr+1) << 16) + (*(Rdptr+2) << 8) + *(Rdptr+3);
		Rdptr += 4;

		Fill_Next();
		}

	BitsLeft = 32;
}

void Next_Packet()
{
	static short volume;
	static int i, j, size;
	static unsigned int code, CHUNK_ID, VOBCELL_ID;

	MPA_Track = 0;

	for (;;)
		{
		code = Get_Short();
		code = (code<<16) + Get_Short();

		if(Fault_Flag > CRITICAL_ERROR_LEVEL)
			{
			return;
			}

		// remove system layer byte stuffing
		while ((code & 0xffffff00) != 0x00000100)
			code = (code<<8) + Get_Byte();

		switch (code)
			{
			case PACK_START_CODE:
				Rdptr += 8;
				VOBCELL_ID = 0;
				break;

			case PRIVATE_STREAM_2:
				Packet_Length = Get_Short();

				if (++VOBCELL_ID==2)
					{
					Rdptr += 25;
					VOB_ID = Get_Short();
					CELL_ID = Get_Short();
					Rdptr += Packet_Length - 29;
					}
				else
					{
					Rdptr += Packet_Length;
					}
				break;

			case PRIVATE_STREAM_1:
				Packet_Length = Get_Short();

				Rdptr ++;
				code = Get_Byte();
				Packet_Header_Length = Get_Byte();

				if (code>=0x80)
					{
					code = Get_Byte();
					AudioPTS = (code & 0x0e) << 29;

					AudioPTS |= (Get_Short() & 0xfffe) << 14;
					AudioPTS |= (Get_Short()>>1) & 0x7fff;
					//AudioPTS /= 90;

					Rdptr += Packet_Header_Length-5;
					}
				else
					{
					Rdptr += Packet_Header_Length;
					}

				Packet_Length -= Packet_Header_Length;

				#ifdef TRANSPORTSTREAM
				 if (TransportStream_Flag)			// Transport streams don't have CHUNK_ID's
					CHUNK_ID = SUB_AC3;
			     else
					CHUNK_ID = Get_Byte();
				#else
				  CHUNK_ID = Get_Byte();
				#endif

				if (CHUNK_ID>=SUB_AC3 && CHUNK_ID<SUB_AC3+CHANNEL)
					{
					#ifdef TRANSPORTSTREAM
					  if (TransportStream_Flag)		// Transport streams don't have the 3 AC3 id bytes..
					  {
						  Packet_Length -= 3;
					  }
					  else
					  {
						Rdptr += 3;
						Packet_Length -= 7;
					  }
					#else
						Rdptr += 3;
						Packet_Length -= 7;
					#endif

					AC3_Track = CHUNK_ID-SUB_AC3;

					readAc3Bits();

					Rdptr += Packet_Length;
					}
				else if ((CHUNK_ID>=SUB_PCM && CHUNK_ID<(SUB_PCM+CHANNEL)) && (Format_Flag & FORMAT_LPCM))
					{

					Rdptr += 6; Packet_Length -= 10;
					PCM_Track = CHUNK_ID-SUB_PCM;

					readPcmBits();

					Rdptr += Packet_Length;
					}
				else if ((CHUNK_ID>=SUB_SUB && CHUNK_ID<(SUB_SUB+SUBCHANNELMAX)))
					{
					Sub_Track = CHUNK_ID - SUB_SUB;
					readSubtitleBits();
					}
				else
					{
					Rdptr += Packet_Length-4;
					}
				break;

			case AUDIO_ELEMENTARY_STREAM_7:
				MPA_Track++;
			case AUDIO_ELEMENTARY_STREAM_6:
				MPA_Track++;
			case AUDIO_ELEMENTARY_STREAM_5:
				MPA_Track++;
			case AUDIO_ELEMENTARY_STREAM_4:
				MPA_Track++;
			case AUDIO_ELEMENTARY_STREAM_3:
				MPA_Track++;
			case AUDIO_ELEMENTARY_STREAM_2:
				MPA_Track++;
			case AUDIO_ELEMENTARY_STREAM_1:
				MPA_Track++;
			case AUDIO_ELEMENTARY_STREAM_0:
				Packet_Length = Get_Short();
				code = Get_Byte();

				if ((code & 0xc0)==0x80 && (Format_Flag&FORMAT_MPEG) && mpa[MPA_Track].grabIt)
					{
					readMpegBits();
					if(Fault_Flag >= CRITICAL_ERROR_LEVEL)
						{
						return;
						}
					}
				else
					{
					Rdptr += Packet_Length-1;
					}
				break;


			case VIDEO_ELEMENTARY_STREAM:
				#ifdef TRANSPORTSTREAM
				  if (TransportStream_Flag)
				  {
				    Get_Short(); // This value is always 0 in transport streams
					Packet_Length = (Rdbfr + BfrSize) - Rdptr;
				  }
				  else
				  {
				    Packet_Length = Get_Short();
				  }
				#else
				  Packet_Length = Get_Short();
				#endif

				Rdmax = Rdptr + Packet_Length;

				code = Get_Byte();

				if ((code & 0xc0)==0x80)
				{
					code = Get_Byte();
					Packet_Header_Length = Get_Byte();

					if (code>=0x80 /*&& !Rip_Flag*/)
						{
						code = Get_Byte();
						VideoPTS = (code & 0x0e) << 29;
						VideoPTS |= (Get_Short() & 0xfffe) << 14;
						VideoPTS |= (Get_Short()>>1) & 0x7fff;
						frameData.hardPts=VideoPTS;					
						frameData.hardPtsFrame=videoFramesOutput;	

						Rdptr += Packet_Header_Length-5;
						}
					else
						{
						Rdptr += Packet_Header_Length;
						}
					return;
				}
				else
					{
					Rdptr += Packet_Length-1;
					}
				break;

			default:
				if (code>=SYSTEM_START_CODE)
					{
					Packet_Length = Get_Short();
					Rdptr += Packet_Length;
					}
				break;
			}

		#ifdef TRANSPORTSTREAM
		  if (TransportStream_Flag)		// Return immediately after reading one packet for TS streams
			break;
		#endif

		}
}

void readMpegBits()
{
	static int mpegFirst;
	static int tempChannels,tempSamplerate, tempFramesize;
	static int i, Packet_Header_Length, size;
	static unsigned int code;

	code = Get_Byte();
	Packet_Header_Length = Get_Byte();

	if (code>=0x80)
		{
		code = Get_Byte();

		AudioPTS = (code & 0x0e) << 29;
		AudioPTS |= (Get_Short() & 0xfffe) << 14;
		AudioPTS |= (Get_Short()>>1) & 0x7fff;
		//AudioPTS /= 90;
		Rdptr += Packet_Header_Length-5;
		}
	else
		{
		Rdptr += Packet_Header_Length;
		}
	Packet_Length -= Packet_Header_Length+3;

	while (Rdptr >= (Rdbfr + BfrSize))
		{
		Read = _read(Infile[File_Flag], Rdbfr, BfrSize);

		if (Read < BfrSize)
			Next_File();
		
		if(Fault_Flag >= CRITICAL_ERROR_LEVEL)
			return;

		Rdptr -= BfrSize;
		}

	if (!mpa[MPA_Track].rip && Rip_Flag)
		{
		code = Get_Byte();
		code = (code & 0xff)<<8 | Get_Byte();
		i = 0;

		while (code<0xfff0)
			{
			code = (code & 0xff)<<8 | Get_Byte();
			i++;
			}
		// this is the first time, so copy over the time stamp
		audioData[mpa[MPA_Track].bufferNum].pts = AudioPTS;
		mpegFirst = TRUE;
		Rdptr -= 2; Packet_Length -= i;

		if(mpa[MPA_Track].directStreamCopy)
			{
			audioData[mpa[MPA_Track].bufferNum].type = audioBufferMpegRaw;
			//Do the direct Stream copy work
			while (Packet_Length > 0)
				{
				if (Packet_Length+Rdptr > BfrSize+Rdbfr)
					{
					size = BfrSize+Rdbfr-Rdptr;
					memcpy(audioData[mpa[MPA_Track].bufferNum].data,Rdptr, size);
					audioData[mpa[MPA_Track].bufferNum].data += size;
					audioData[mpa[MPA_Track].bufferNum].bytesUsed +=size;
					
					Packet_Length -= BfrSize+Rdbfr-Rdptr;
					Read = _read(Infile[File_Flag], Rdbfr, BfrSize);
					Rdptr = Rdbfr;
					}
				else
					{
					size = Packet_Length;
					memcpy(audioData[mpa[MPA_Track].bufferNum].data,Rdptr, size);											
					audioData[mpa[MPA_Track].bufferNum].data += size;
					audioData[mpa[MPA_Track].bufferNum].bytesUsed +=size;

					Rdptr += Packet_Length;
					Packet_Length = 0;
					}
				}

			}
		else
			{
			audioData[mpa[MPA_Track].bufferNum].type = audioBufferMpegDecoded;
			while (Packet_Length > 0)
				{
					if (Packet_Length+Rdptr > BfrSize+Rdbfr)
					{
						if(mpegFirst)
							{
							mpegPrepare(Rdptr, BfrSize+Rdbfr-Rdptr, &tempChannels, &tempSamplerate, &tempFramesize);
							mpegFirst = FALSE;
							}
						size = mpegDecode(Rdptr,BfrSize+Rdbfr-Rdptr,audioData[mpa[MPA_Track].bufferNum].data);
						
						audioData[mpa[MPA_Track].bufferNum].data += size;
						audioData[mpa[MPA_Track].bufferNum].bytesUsed +=size;
						
						Packet_Length -= BfrSize+Rdbfr-Rdptr;
						Read = _read(Infile[File_Flag], Rdbfr, BfrSize);
						Rdptr = Rdbfr;
					}
					else
					{
						if(mpegFirst)
							{
							mpegPrepare(Rdptr, Packet_Length, &tempChannels, &tempSamplerate, &tempFramesize);
							mpegFirst = FALSE;
							}
						size = mpegDecode(Rdptr,Packet_Length,audioData[mpa[MPA_Track].bufferNum].data);
						audioData[mpa[MPA_Track].bufferNum].data += size;
						audioData[mpa[MPA_Track].bufferNum].bytesUsed +=size;

						Rdptr += Packet_Length;
						Packet_Length = 0;
					}
				}
			}
		mpa[MPA_Track].rip = 1;					
		}
	else if (mpa[MPA_Track].rip)
		{
		// If the buffer is clean, set the audio PTS for the start of the buffer
		if(!audioData[mpa[MPA_Track].bufferNum].bytesUsed)
			{
			audioData[mpa[MPA_Track].bufferNum].pts = AudioPTS;
			}
		if(mpa[MPA_Track].directStreamCopy)
			{
			//Do the direct Stream copy work
			audioData[mpa[MPA_Track].bufferNum].type = audioBufferMpegRaw;

			while (Packet_Length > 0)
				{
				if (Packet_Length+Rdptr > BfrSize+Rdbfr)
					{
					size = BfrSize+Rdbfr-Rdptr;
					memcpy(audioData[mpa[MPA_Track].bufferNum].data,Rdptr, size);
					audioData[mpa[MPA_Track].bufferNum].data += size;
					audioData[mpa[MPA_Track].bufferNum].bytesUsed +=size;
					
					Packet_Length -= BfrSize+Rdbfr-Rdptr;
					Read = _read(Infile[File_Flag], Rdbfr, BfrSize);
					if(!Read)
						{
						Fault_Flag = CRITICAL_ERROR_LEVEL;
						return;
						}
					Rdptr = Rdbfr;
					}
				else
					{
					size = Packet_Length;
					memcpy(audioData[mpa[MPA_Track].bufferNum].data,Rdptr, size);											
					audioData[mpa[MPA_Track].bufferNum].data += size;
					audioData[mpa[MPA_Track].bufferNum].bytesUsed +=size;
					Rdptr += Packet_Length;
					Packet_Length = 0;
					}
				}

			}
		else
			{
			audioData[mpa[MPA_Track].bufferNum].type = audioBufferMpegDecoded;
			while (Packet_Length > 0)
				{
					if (Packet_Length+Rdptr > BfrSize+Rdbfr)
					{
						size = mpegDecode(Rdptr,BfrSize+Rdbfr-Rdptr,audioData[mpa[MPA_Track].bufferNum].data);
						audioData[mpa[MPA_Track].bufferNum].data += size;
						audioData[mpa[MPA_Track].bufferNum].bytesUsed +=size;

						Packet_Length -= BfrSize+Rdbfr-Rdptr;
						Read = _read(Infile[File_Flag], Rdbfr, BfrSize);
						if(!Read)
							{
							Fault_Flag = CRITICAL_ERROR_LEVEL;
							return;
							}
						Rdptr = Rdbfr;
					}
					else
					{
						size = mpegDecode(Rdptr,Packet_Length,audioData[mpa[MPA_Track].bufferNum].data);
						audioData[mpa[MPA_Track].bufferNum].data += size;
						audioData[mpa[MPA_Track].bufferNum].bytesUsed +=size;
						Rdptr += Packet_Length;
						Packet_Length = 0;
					}
				}
			}
		}
	else
		{
		Rdptr += Packet_Length;
		}

}

void readSubtitleBits()
{
	static unsigned char *ptr;
	static int subSize;

	Packet_Length -=4;

	if (!subtitle[Sub_Track].rip && subtitle[Sub_Track].grabIt)
		{
		memcpy(subtitleData[subtitle[Sub_Track].bufferNum].data,Rdptr,Packet_Length);

		subtitleData[subtitle[Sub_Track].bufferNum].data += Packet_Length;
		subtitleData[subtitle[Sub_Track].bufferNum].bytesUsed += Packet_Length;

		subtitle[Sub_Track].rip = 1;
		}
	else if (subtitle[Sub_Track].rip)
		{
		memcpy(subtitleData[subtitle[Sub_Track].bufferNum].data,Rdptr,Packet_Length);

		subtitleData[subtitle[Sub_Track].bufferNum].data += Packet_Length;
		subtitleData[subtitle[Sub_Track].bufferNum].bytesUsed += Packet_Length;
		}

	if(subtitleData[subtitle[Sub_Track].bufferNum].bytesUsed>2)
		{
		ptr = subtitleData[subtitle[Sub_Track].bufferNum].bufferStart;
		subSize = *ptr++;
		subSize = subSize<<8;
		subSize|= *ptr++;
		if(subtitleData[subtitle[Sub_Track].bufferNum].bytesUsed>=subSize)
			subtitleData[subtitle[Sub_Track].bufferNum].complete=1;
		}

	Rdptr += Packet_Length;				


}

void readPcmBits()
{
	static int j;

	if (!pcm[PCM_Track].rip && Rip_Flag && pcm[PCM_Track].grabIt)
		{
		audioData[pcm[PCM_Track].bufferNum].pts = AudioPTS;
		audioData[pcm[PCM_Track].bufferNum].type = audioBufferPcm;		
		for (j=0; j<Packet_Length; j+=2)
			{
			audioData[pcm[PCM_Track].bufferNum].data[j] = *(Rdptr+j+1);
			audioData[pcm[PCM_Track].bufferNum].data[j+1] = *(Rdptr+j);
			}
		audioData[pcm[PCM_Track].bufferNum].data += Packet_Length;
		audioData[pcm[PCM_Track].bufferNum].bytesUsed += Packet_Length;

		pcm[PCM_Track].rip = 1;
		}
	else if (pcm[PCM_Track].rip)
		{
		if(!audioData[pcm[PCM_Track].bufferNum].bytesUsed)
			audioData[pcm[PCM_Track].bufferNum].pts = AudioPTS;

		audioData[pcm[PCM_Track].bufferNum].type = audioBufferPcm;		

		for (j=0; j<Packet_Length; j+=2)
			{
				audioData[pcm[PCM_Track].bufferNum].data[j] = *(Rdptr+j+1);
				audioData[pcm[PCM_Track].bufferNum].data[j+1] = *(Rdptr+j);
			}

		audioData[pcm[PCM_Track].bufferNum].data += Packet_Length;
		audioData[pcm[PCM_Track].bufferNum].bytesUsed += Packet_Length;							
		}

}

void readAc3Bits()
{
	static int i, size;
	static unsigned int code;

	if (!ac3[AC3_Track].rip && Rip_Flag && ac3[AC3_Track].grabIt)
		{
			code = Get_Byte();
			code = (code & 0xff)<<8 | Get_Byte();
			i = 0;

			while (code!=0xb77)
			{
				code = (code & 0xff)<<8 | Get_Byte();
				i++;
			}

			Get_Short();
			ac3[AC3_Track].rate = (Get_Byte()>>1) & 0x1f;
			Get_Byte();
			ac3[AC3_Track].mode = (Get_Byte()>>5) & 0x07;

			ac3[AC3_Track].rip = 1;
			if(ac3[AC3_Track].directStreamCopy)
				{
				audioData[ac3[AC3_Track].bufferNum].type = audioBufferAc3Raw;	
				size = Packet_Length-i;
				memcpy(audioData[ac3[AC3_Track].bufferNum].data,Rdptr-7,size);
				audioData[ac3[AC3_Track].bufferNum].data+=size;
				}
			else
				{
				InitialAC3(ac3[AC3_Track].bufferNum);

				audioData[ac3[AC3_Track].bufferNum].pts = AudioPTS;
				audioData[ac3[AC3_Track].bufferNum].type = audioBufferAc3Decoded;		

				size = ac3_decode_data(ac3[AC3_Track].bufferNum,Rdptr-7, Packet_Length-i);
				}

			Rdptr -= 4+i;
		}
	else if (ac3[AC3_Track].rip)
		{

		if(!audioData[ac3[AC3_Track].bufferNum].bytesUsed)
			audioData[ac3[AC3_Track].bufferNum].pts = AudioPTS;

		if(ac3[AC3_Track].directStreamCopy)
			{
			audioData[ac3[AC3_Track].bufferNum].type = audioBufferAc3Raw;	
			size = Packet_Length;
			memcpy(audioData[ac3[AC3_Track].bufferNum].data,Rdptr,size);
			audioData[ac3[AC3_Track].bufferNum].data+=size;
			}
		else
			{
			audioData[ac3[AC3_Track].bufferNum].type = audioBufferAc3Decoded;		
			// If the buffer is clean, set the audio PTS for the start of the buffer
			size = ac3_decode_data(ac3[AC3_Track].bufferNum,Rdptr, Packet_Length);
			}
		}

}
unsigned int Get_Bits_All(unsigned int N)
{
	N -= BitsLeft;
	Val = (CurrentBfr << (32 - BitsLeft)) >> (32 - BitsLeft);

	if (N)
		Val = (Val << N) + (NextBfr >> (32 - N));

	CurrentBfr = NextBfr;
	BitsLeft = 32 - N;
	Fill_Next();

	return Val;
}

void Flush_Buffer_All(unsigned int N)
{
	CurrentBfr = NextBfr;
	BitsLeft = BitsLeft + 32 - N;
	Fill_Next();
}

void Fill_Buffer()
{
#ifdef TRANSPORTSTREAM
	if (TransportStream_Flag && Rdbfr != RawRdbfr)
	{
		unsigned int save_CurrentBfr, save_BitsLeft, save_NextBfr, save_Val;

		save_CurrentBfr = CurrentBfr;
		save_BitsLeft = BitsLeft;
		save_NextBfr = NextBfr;
		save_Val = Val;

		NextTransportStreamPacket();
		Read = BfrSize;

		CurrentBfr = save_CurrentBfr;
		BitsLeft = save_BitsLeft;
		NextBfr = save_NextBfr;
		Val = save_Val;
		
		return;
	}
#endif

	Read = _read(Infile[File_Flag], Rdbfr, BfrSize);

	if (Read < BfrSize)
		Next_File();

	if(Fault_Flag >= CRITICAL_ERROR_LEVEL)
		return;

	Rdptr = Rdbfr;

#ifdef TRANSPORTSTREAM
	if (SystemStream_Flag)
		Rdmax -= BfrSize;
	else
	{
		if (!TransportStream_Flag)
			Bitrate_Meter += Read;
	}
#else
	if (SystemStream_Flag)
		Rdmax -= BfrSize;
	else
		Bitrate_Meter += Read;
#endif
}

void Next_File()
{
	int i;

	if (File_Flag < File_Limit-1)
	{
		//If more files, then increment the file index
		File_Flag++;

	}
	else
	{
		//Otherwise Set Fault Flag, and keep index on current file (point at known good data)
		Fault_Flag = 98;

		//Write_Frame(NULL, d2v_current, 0);
		//return;
	}

		//Setup for more reads
	process.run = 0;
	for (i=0; i<File_Flag; i++)
		process.run += process.length[i];

	_lseeki64(Infile[File_Flag], 0, SEEK_SET);
	_read(Infile[File_Flag], Rdbfr + Read, BfrSize - Read);
	
}

static unsigned int Get_Short()
{
	unsigned int i = Get_Byte();
	return (i<<8) | Get_Byte();
}

#ifdef TRANSPORTSTREAM

void DetectTransportStream()
{
	unsigned char sync;

	CleanupTransportStream();

	sync = 0;
	_read(Infile[0], &sync, 1);
	_lseeki64(Infile[0], 0, SEEK_SET);

	TransportStream_Flag = (sync == 0x47);	// Has TS sync byte at beginning of file

	if (TransportStream_Flag)
	{
		SystemStream_Flag = 0;
		InitTransportStream();
		BfrSize = BUFFER_SIZE;
	}
	else
		BfrSize = SYSTEM_BUFFER_SIZE;
}

void InitTransportStream()
{
	if (TransportStreamInit_Flag)
		return;

	TransportStream_AudBuffer = (unsigned char*)malloc(AUDPES_BUFFER_SIZE);
	TransportStream_AudBufferPtr = TransportStream_AudBuffer;
	TransportStream_AudContinuity = -1;
	TransportStream_AudReading = 0;
	TransportStream_VidBuffer = (unsigned char*)malloc(VIDPES_BUFFER_SIZE);
	TransportStream_VidBufferPtr = TransportStream_VidBuffer;
	TransportStream_VidContinuity = -1;
	TransportStream_VidReading = 0;

	TransportStreamInit_Flag = 1;
	TransportStreamSync_Flag = 0;
}

void CleanupTransportStream()
{
	if (!TransportStreamInit_Flag)
		return;

	ReadFromTransportStreamTSFile();

	free(TransportStream_AudBuffer);
	free(TransportStream_VidBuffer);

	TransportStream_Flag = 0;
	TransportStreamInit_Flag = 0;
}

void ReadFromTransportStreamTSFile()
{
	if (Rdbfr == RawRdbfr)
		return;

	if (SystemStream_Flag)
		BfrSize = SYSTEM_BUFFER_SIZE;
	else
		BfrSize = BUFFER_SIZE;

	// Switch to read from transport stream
	Rdbfr = TransportStream_Rdbfr;
	Rdptr = TransportStream_Rdptr;
	Rdmax = TransportStream_Rdmax;
	BitsLeft = TransportStream_BitsLeft;
	CurrentBfr = TransportStream_CurrentBfr;
	NextBfr = TransportStream_NextBfr;
	Val = TransportStream_Val;
	Read = TransportStream_Read;
}

void ReadFromTransportStreamVideoPESBuffer()
{
	if (Rdbfr == TransportStream_VidBuffer)
		return;
	else if (Rdbfr != RawRdbfr)
		ReadFromTransportStreamTSFile();

	// Save current transport stream read state..
	TransportStream_Rdbfr = Rdbfr;
	TransportStream_Rdptr = Rdptr;
	TransportStream_Rdmax = Rdmax;
	TransportStream_BitsLeft = BitsLeft;
	TransportStream_CurrentBfr = CurrentBfr;
	TransportStream_NextBfr = NextBfr;
	TransportStream_Val = Val;
	TransportStream_Read = Read;

	// Switch to read from video stream buffer
	BfrSize = TransportStream_VidBufferPtr - TransportStream_VidBuffer;
	Rdbfr = TransportStream_VidBuffer;
	Rdptr = TransportStream_VidBuffer;
	Rdmax = TransportStream_VidBufferPtr;

	Read = 0;
}

void ReadFromTransportStreamAudioPESBuffer()
{
	if (Rdbfr == TransportStream_AudBuffer)
		return;
	else if (Rdbfr != RawRdbfr)
		ReadFromTransportStreamTSFile();

	// Save current transport stream read state..
	TransportStream_Rdbfr = Rdbfr;
	TransportStream_Rdptr = Rdptr;
	TransportStream_Rdmax = Rdmax;
	TransportStream_BitsLeft = BitsLeft;
	TransportStream_CurrentBfr = CurrentBfr;
	TransportStream_NextBfr = NextBfr;
	TransportStream_Val = Val;
	TransportStream_Read = Read;

	// Switch to read from audio stream buffer
	BfrSize = TransportStream_AudBufferPtr - TransportStream_AudBuffer;
	Rdbfr = TransportStream_AudBuffer;
	Rdptr = TransportStream_AudBuffer;
	Rdmax = TransportStream_AudBufferPtr;

	Read = 0;
}

void SyncTransportStream()
{
	unsigned int sync, error, priority, start, pid, scrambling, adaption, continuity, adaptlen;
	int count = 0;

	ReadFromTransportStreamTSFile();

	for (;;)
	{
		// Give up if we can't find sync in 0x10000 tries..
		if (count > 0x10000)
		{
			Fault_Flag = 98;
			return;
		}

		if (Rdptr >= (Rdbfr + BfrSize))
			Fill_Buffer();

		if (Fault_Flag >= CRITICAL_ERROR_LEVEL)
			break;

		// Get bits..
		CurrentBfr = (*Rdptr << 24) + (*(Rdptr+1) << 16) + (*(Rdptr+2) << 8) + *(Rdptr+3);
		BitsLeft = 32;

		// Make sure we have valid sync byte..
		sync = Get_Bits(8);
		if (sync != 0x47)
		{
			count++;
			Rdptr += 184;
			continue;
		}

		// Get packet header
		error = Get_Bits(1);
		start = Get_Bits(1);
		priority = Get_Bits(1);
		pid = Get_Bits(13);
		scrambling = Get_Bits(2);
		adaption = Get_Bits(2);
		continuity = Get_Bits(4);

		// Check adaption
		if (adaption == 0)
			error = 1;
		else if (adaption & 0x2)
			adaptlen = *Rdptr;
		else
			adaptlen = 0;
		if (adaptlen > 184)
			error = 1;

		// We only care about audio/video packets.. ignore everything else
		if (!error && pid == TransportStream_VidPID && start)
		{
			Rdptr -= 4;
			TransportStreamSync_Flag = 1;
			break;
		}

		Rdptr += 184;

		count++;
	}
}

void NextTransportStreamPacket()
{
	unsigned int sync, error, priority, start, pid, scrambling, adaption, adaptlen;
	int continuity;
	
	ReadFromTransportStreamTSFile();

	for (;;)
	{
		// Must start with stream synced to first valid packet..
		if (!TransportStreamSync_Flag)
			SyncTransportStream();

		if (Fault_Flag >= CRITICAL_ERROR_LEVEL)
			break;

		if (Rdptr >= (Rdbfr + BfrSize))
			Fill_Buffer();

		if (Fault_Flag >= CRITICAL_ERROR_LEVEL)
			break;

		// Get bits..
		CurrentBfr = (*Rdptr << 24) + (*(Rdptr+1) << 16) + (*(Rdptr+2) << 8) + *(Rdptr+3);
		BitsLeft = 32;

		// Make sure we have valid sync byte.. if not, try resyncing stream..
		sync = Get_Bits(8);
		if (sync != 0x47)
		{
			TransportStreamSync_Flag = 0;
			continue;
		}

		// Get packet header
		error = Get_Bits(1);
		start = Get_Bits(1);
		priority = Get_Bits(1);
		pid = Get_Bits(13);
		scrambling = Get_Bits(2);
		adaption = Get_Bits(2);
		continuity = Get_Bits(4);

		// Check adaption
		if (adaption == 0)
			error = 1;
		else if (adaption & 0x2)
			adaptlen = *Rdptr + 1;
		else
			adaptlen = 0;
		if (adaptlen > 184)
			error = 1;

		// We only care about audio/video packets.. ignore everything else
		if (pid == TransportStream_AudPID)
		{
			// DETECT BAD PACKETS!
			if (error || (TransportStream_AudContinuity != -1 &&
				(continuity != ((TransportStream_AudContinuity + 1) & 0xF))))
			{
				// Dump the rest of this PES packet and start again
				TransportStream_AudReading = 0;
				TransportStream_AudBufferPtr = TransportStream_AudBuffer;
				continue;
			}

			if (start)
			{
				if (TransportStream_AudReading)	// Finished previous packet.. return!
				{
					TransportStream_AudReading = 0;
					TransportStream_AudContinuity = -1;
					ReadFromTransportStreamAudioPESBuffer();	// Start reading from this Audio packet
					Next_Packet();
					ReadFromTransportStreamTSFile();
				}

				TransportStream_AudReading = 1;
				TransportStream_AudBufferPtr = TransportStream_AudBuffer;
			}
			
			if (TransportStream_AudReading)
			{
				if ((TransportStream_AudBufferPtr + 184 - adaptlen) - TransportStream_AudBuffer > AUDPES_BUFFER_SIZE)
				{
					Fault_Flag = 98;
					break;
				}
				memcpy(TransportStream_AudBufferPtr, Rdptr + adaptlen, 184 - adaptlen);
				TransportStream_AudBufferPtr += 184 - adaptlen;
				TransportStream_AudContinuity = continuity;
			}
		}
		else if (pid == TransportStream_VidPID)
		{
			// DETECT BAD PACKETS!
			if (error || (TransportStream_VidContinuity != -1 &&
				(continuity != ((TransportStream_VidContinuity + 1) & 0xF))))
			{
				// Dump the rest of this PES packet and start again
				TransportStream_VidReading = 0;
				TransportStream_VidBufferPtr = TransportStream_VidBuffer;
				continue;
			}

			if (start)
			{
				if (TransportStream_VidReading)	// Finished previous packet.. return!
				{
					Rdptr -= 4; // Backup
					TransportStream_VidReading = 0;
					TransportStream_VidContinuity = -1;
					ReadFromTransportStreamVideoPESBuffer();	// Start reading from this Video packet
					Next_Packet();
					if (Rdptr < (Rdbfr + BfrSize))				// We have video data to process.. (exit)
						break;
					ReadFromTransportStreamTSFile();
					Rdptr += 4; // Oops.. we're actually going to go on.. Un-Backup
				}

				TransportStream_VidReading = 1;
				TransportStream_VidBufferPtr = TransportStream_VidBuffer;
			}
			
			if (TransportStream_VidReading)
			{
				if ((TransportStream_VidBufferPtr + 184 - adaptlen) - TransportStream_VidBuffer > VIDPES_BUFFER_SIZE)
				{
					Fault_Flag = 98;
					break;
				}
				memcpy(TransportStream_VidBufferPtr, Rdptr + adaptlen, 184 - adaptlen);
				TransportStream_VidBufferPtr += 184 - adaptlen;
				TransportStream_VidContinuity = continuity;
			}
		}

		Rdptr += 184;
	}
}

#endif

