/* Copyright (C) 1996, MPEG Software Simulation Group. All Rights Reserved. */

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  The MPEG Software Simulation Group disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 * The MPEG Software Simulation Group does not represent or warrant that the
 * programs furnished hereunder are free of infringement of any third-party
 * patents.
 *
 * Commercial implementations of MPEG-1 and MPEG-2 video, including shareware,
 * are subject to royalty fees to patent holders.  Many of these patents are
 * general enough such that they are unavoidable regardless of implementation
 * design.
 *
 */
#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <ddraw.h>
#include <vfw.h>
#include <winreg.h>
#include <direct.h>
#include <io.h>
#include <fcntl.h>
#include "dvd2aviglobals.h"


PAVISTREAM ps, psCompressed;
static int AVI_Init, avi_size, avi_count, frame_count;
static int TFF, RFF, TFB, BFB, playback, Old_Playback, frame_type;
long frame_size;


/* code definition */
#define PICTURE_START_CODE			0x100
#define SLICE_START_CODE_MIN		0x101
#define SLICE_START_CODE_MAX		0x1AF
#define USER_DATA_START_CODE		0x1B2
#define SEQUENCE_HEADER_CODE		0x1B3
#define EXTENSION_START_CODE		0x1B5
#define SEQUENCE_END_CODE			0x1B7
#define GROUP_START_CODE			0x1B8

#define SYSTEM_END_CODE				0x1B9
#define PACK_START_CODE				0x1BA
#define SYSTEM_START_CODE			0x1BB

#define VIDEO_ELEMENTARY_STREAM		0x1E0

#define PRIVATE_STREAM_1			0x1BD
#define PRIVATE_STREAM_2			0x1BF
#define AUDIO_ELEMENTARY_STREAM_0	0x1C0
#define AUDIO_ELEMENTARY_STREAM_1	0x1C1
#define AUDIO_ELEMENTARY_STREAM_2	0x1C2
#define AUDIO_ELEMENTARY_STREAM_3	0x1C3
#define AUDIO_ELEMENTARY_STREAM_4	0x1C4
#define AUDIO_ELEMENTARY_STREAM_5	0x1C5
#define AUDIO_ELEMENTARY_STREAM_6	0x1C6
#define AUDIO_ELEMENTARY_STREAM_7	0x1C7

#define SUB_SUB						0x20
#define SUB_AC3						0x80
#define SUB_PCM						0xA0

/* extension start code IDs */
#define SEQUENCE_EXTENSION_ID					1
#define SEQUENCE_DISPLAY_EXTENSION_ID			2
#define QUANT_MATRIX_EXTENSION_ID				3
#define COPYRIGHT_EXTENSION_ID					4
#define PICTURE_DISPLAY_EXTENSION_ID			7
#define PICTURE_CODING_EXTENSION_ID				8

#define ZIG_ZAG									0
#define MB_WEIGHT								32
#define MB_CLASS4								64

#define I_TYPE			1
#define P_TYPE			2
#define B_TYPE			3

#define MACROBLOCK_INTRA				1
#define MACROBLOCK_PATTERN				2
#define MACROBLOCK_MOTION_BACKWARD		4
#define MACROBLOCK_MOTION_FORWARD		8
#define MACROBLOCK_QUANT				16

#define TOP_FIELD		1
#define BOTTOM_FIELD	2
#define FRAME_PICTURE	3

#define MC_FIELD		1
#define MC_FRAME		2
#define MC_16X8			2
#define MC_DMV			3

#define MV_FIELD		0
#define MV_FRAME		1

#define CHROMA420		1
#define CHROMA422		2
#define CHROMA444		3

#ifdef TRANSPORTSTREAM
#define SYSTEM_BUFFER_SIZE		(2048)
#define TS_PACKET_SIZE			(188)
#define BUFFER_SIZE				(TS_PACKET_SIZE * 128)
#else
#define SYSTEM_BUFFER_SIZE		(2048)
#define BUFFER_SIZE				SYSTEM_BUFFER_SIZE
#endif

unsigned int BfrSize
#ifdef GLOBAL
= SYSTEM_BUFFER_SIZE
#endif
;

#define MAX_FILE_NUMBER			256

#define STORE_RGB32		1
#define STORE_YUY2		2
#define STORE_YV12		3

#define IDCT_MMX		1
#define IDCT_SSEMMX		2
#define	IDCT_FPU		3
#define IDCT_REF		4

#define LOCATE_INIT			0
#define LOCATE_FORWARD		1
#define LOCATE_BACKWARD		-1
#define LOCATE_RIP			2

#define CHANNEL				8
#define SUBCHANNELMAX		32


#define TRACK_NONE		15	// DANGEROUS!
#define TRACK_1			0
#define TRACK_2			1
#define TRACK_3			2
#define TRACK_4			3
#define TRACK_5			4
#define TRACK_6			5
#define TRACK_7			6
#define TRACK_8			7

#define FORMAT_NONE			0x00
#define FORMAT_AC3			0x01
#define FORMAT_MPEG			0x02
#define FORMAT_LPCM			0x04

#define AUDIO_DEMUXALL		0
#define AUDIO_DEMUXONE		1
#define AUDIO_DECODE		2

#define FO_NONE			0
#define FO_FILM			1
#define FO_SWAP			2

#define SRC_NONE		0
#define SRC_SPEED		1
#define SRC_NORMAL		2
#define SRC_QUALITY		3

#define NORM_NONE		0
#define NORM_DYNAMIC	1
#define NORM_MAXIMUM	2

#define TRACK_PITCH		500

#define CRITICAL_ERROR_LEVEL	50

typedef struct {
	int			type;
	int			file;
	__int64		lba;
	int			pf;
	int			trf;
}	D2VData;
D2VData d2v_backward, d2v_forward, d2v_current;

int CH[CHANNEL];
short Sound_Max;

typedef struct {
	FILE					*file;
	char					filename[MAX_PATH];
	int						rip;
	int						size;
	int						delay;
	unsigned int			mode;
	unsigned int			rate;
	int						grabIt;
	int						bufferNum;
	int						directStreamCopy;
}	AC3Stream;
AC3Stream ac3[CHANNEL];

typedef struct {
	FILE					*file;
	int						rip;
	int						delay;
	int						grabIt;
	int						bufferNum;
	int						directStreamCopy;
}	MPAStream;
MPAStream mpa[CHANNEL];

typedef struct {
	FILE					*file;
	char					filename[MAX_PATH];
	int						rip;
	int						size;
	int						delay;
	int						grabIt;
	int						bufferNum;
}	PCMStream;
PCMStream pcm[CHANNEL];

typedef struct {
	int						rip;
	int						grabIt;
	int						bufferNum;
}	SUBStream;
SUBStream subtitle[SUBCHANNELMAX];

struct PROCESS {
	__int64					length[MAX_FILE_NUMBER];
	__int64					total;
	__int64					run;
	__int64					start;
	__int64					end;
	int						trackleft;
	int						trackright;
	int						locate;
	int						startfile;
	__int64					startloc;
	int						endfile;
	__int64					endloc;
	int						file;
	__int64					lba;
	int						leftfile;
	__int64					leftlba;
	int						rightfile;
	__int64					rightlba;
	unsigned int			op;
	unsigned int			ed;
	unsigned int			elapsed;
	unsigned int			remain;
	float					percent;
}	process;

struct CPU {
	BOOL					mmx;
	BOOL					_3dnow;
	BOOL					ssemmx;
	BOOL					ssefpu;
}	cpu;

/* from gui.cpp */

#define INIT_WIDTH		320
#define INIT_HEIGHT		240
#define MASKCOLOR			RGB(0, 6, 0)
#define PRIORITY_HIGH		1
#define PRIORITY_NORMAL		2
#define PRIORITY_LOW		3
int INIT_X, INIT_Y, Priority_Flag, LumDlg_Flag;
HANDLE hThread;
DWORD threadId;
int LumGain;

/* decoder operation control flags */
int AVI_Flag;
int Check_Flag;
int D2V_Flag;
int DDOverlay_Flag;
int Display_Flag;
int Fault_Flag;
int File_Flag;
int File_Limit;
int FO_Flag;
int iDCT_Flag;
int Luminance_Flag;
int Pause_Flag;
int Rip_Flag;
int Scale_Flag;
int Statistics_Flag;
int Stop_Flag;
int Store_Flag;
int SystemStream_Flag;

unsigned int Track_Flag;
int Format_Flag;
int AC3_Flag;
int MPA_Flag;
int Surround_Flag;
int Norm_Flag;
int SRC_Flag;
int DSDown_Flag;
double PreScale_Ratio;

#ifdef TRANSPORTSTREAM

#define AUDPES_BUFFER_SIZE (1024 * 128)
#define VIDPES_BUFFER_SIZE (1024 * 512)

unsigned int TransportStream_Flag;
unsigned int TransportStreamInit_Flag;
unsigned int TransportStreamSync_Flag;
unsigned int TransportStream_VidPID
#ifdef GLOBAL
= 0x11
#endif
;
unsigned int TransportStream_AudPID
#ifdef GLOBAL
= 0x14
#endif
;

unsigned char *TransportStream_AudBuffer;
unsigned char *TransportStream_AudBufferPtr;
int TransportStream_AudContinuity;
int TransportStream_AudReading;

unsigned char *TransportStream_VidBuffer;
unsigned char *TransportStream_VidBufferPtr;
int TransportStream_VidContinuity;
int TransportStream_VidReading;

// Save values for current read buffer
unsigned char *TransportStream_Rdbfr;
unsigned char *TransportStream_Rdptr;
unsigned char *TransportStream_Rdmax;
unsigned int TransportStream_BitsLeft;
unsigned int TransportStream_CurrentBfr;
unsigned int TransportStream_NextBfr;
unsigned int TransportStream_Val;
unsigned int TransportStream_Read;

void DetectTransportStream();
void InitTransportStream();
void CleanupTransportStream();
void ReadFromTransportStreamTSFile();
void ReadFromTransportStreamVideoPESBuffer();
void ReadFromTransportStreamAudioPESBuffer();
void SyncTransportStream();
void NextTransportStreamPacket();

#endif

/* DirectDraw & GDI resources */
LPDIRECTDRAW lpDD;
LPDIRECTDRAW2 lpDD2;
LPDIRECTDRAWSURFACE lpPrimary, lpOverlay;
DDCAPS halcaps;
DDSURFACEDESC ddsd;
DDOVERLAYFX ddofx;
RECT wrect, crect, orect, prect;
HDC hDC;

BITMAPINFOHEADER birgb;
LPBITMAPINFOHEADER lpbirgb
#ifdef GLOBAL
= &birgb
#endif
;

BITMAPINFOHEADER biyuv;
LPBITMAPINFOHEADER lpbiyuv
#ifdef GLOBAL
= &biyuv
#endif
;

/* Global Value */
FILE *D2VFile;
HWND hWnd/*, hDlg, hTrack, hLeftButton, hLeftArrow, hRightArrow, hRightButton*/;
char szInput[MAX_PATH], szOutput[MAX_PATH], szBuffer[MAX_PATH], szBlank[1];

unsigned char *backward_reference_frame[3], *forward_reference_frame[3];
unsigned char *auxframe[3], *current_frame[3];
unsigned char *u422, *v422, *u444, *v444, *rgb32, *yuy2, *lum, *yv12;
__int64 YUVRGB_Scale, LumOffsetMask, LumGainMask;

unsigned int hwidth, hwidthd8, dwidth, qwidth, hheightd2, qheightd2, nwidth, area;
int VideoPTS, AudioPTS;

int Edge_Width, Edge_Height;
unsigned short VOB_ID, CELL_ID;
unsigned int Frame_Number;

int Coded_Picture_Width, Coded_Picture_Height, Chroma_Width, Chroma_Height;
int block_count, Second_Field;
int horizontal_size, vertical_size, mb_width, mb_height;

float frame_rate, Frame_Rate;
int Video_Type, FILM_Purity, NTSC_Purity, Bitrate_Meter;

int Infile[MAX_FILE_NUMBER];
char Infilename[MAX_FILE_NUMBER][MAX_PATH];

int intra_quantizer_matrix[64];
int non_intra_quantizer_matrix[64];
int chroma_intra_quantizer_matrix[64];
int chroma_non_intra_quantizer_matrix[64];

int load_intra_quantizer_matrix;
int load_non_intra_quantizer_matrix;
int load_chroma_intra_quantizer_matrix;
int load_chroma_non_intra_quantizer_matrix;

int q_scale_type;
int alternate_scan;
int quantizer_scale;

short *block[8], *p_block[8];
void *fTempArray, *p_fTempArray;

/* ISO/IEC 13818-2 section 6.2.2.1:  sequence_header() */
int frame_rate_code;
int aspect_ratio_information;

/* ISO/IEC 13818-2 section 6.2.2.3:  sequence_extension() */
int progressive_sequence;
int chroma_format;

/* ISO/IEC 13818-2 section 6.2.3: picture_header() */
int picture_coding_type;
int temporal_reference;

/* ISO/IEC 13818-2 section 6.2.3.1: picture_coding_extension() header */
int f_code[2][2];
int picture_structure;
int frame_pred_frame_dct;
int progressive_frame;
int concealment_motion_vectors;
int intra_dc_precision;
int top_field_first;
int repeat_first_field;
int intra_vlc_format;

/* gethdr.c */
int Get_Hdr(void);
void next_start_code(void);
void sequence_header(void);
int slice_header(void);

/* getpic.c */
void Decode_Picture_Begin(void);
int Decode_Write_Frame(void);
void Decode_Picture_End(void);
void Decode_Picture(void);

// getbit.c
void Initialize_Buffer();

/* idct */
/* idct */
extern void __fastcall MMX_IDCT(short *block);
extern void __fastcall SSEMMX_IDCT(short *block);
//void MMX_IDCT(short *block);

void Initialize_FPU_IDCT(void);
void FPU_IDCT(short *block);
void Initialize_REF_IDCT(void);
void REF_IDCT(short *block);

/* motion.c */
void motion_vectors(int PMV[2][2][2], int dmvector[2], int motion_vertical_field_select[2][2], 
	int s, int motion_vector_count, int mv_format, int h_r_size, int v_r_size, int dmv, int mvscale);
void Dual_Prime_Arithmetic(int DMV[][2], int *dmvector, int mvx, int mvy);

/* mpeg2dec.c */
//DWORD WINAPI MPEG2Dec(PVOID pvoid);
DWORD MPEG2DecInit();

/* norm.c */
void Normalize(FILE *WaveIn, int WaveInPos, char *filename, FILE *WaveOut, int WaveOutPos, int size);

/* store.c */
int Write_Frame_Begin(unsigned char *src[], D2VData d2v, DWORD frame);
int Write_Frame_End(unsigned char *src[], D2VData d2v, DWORD frame, int state);
void RenderYUY2(void);
int DetectVideoType(int frame, int rff);
void AVIKill();

/* wavefs44.c */
void InitialSRC(void);
void Wavefs44(FILE *file, int size, unsigned char *buffer);
void EndSRC(FILE *file);
void Wavefs44File(int delay, int now, int total);
void EndWAV(FILE *file, int size);
void DownWAV(FILE *file);
BOOL CheckWAV(void);
int resample48To44(unsigned char * source, unsigned char * dest, int size);


/* misc.cpp (was gui.cpp)*/
DWORD DDColorMatch(LPDIRECTDRAWSURFACE pdds, COLORREF rgb);
void ThreadKill(void);
void CheckDirectDraw(void);
void CheckCPU(void);
void Recovery(void);

unsigned char WAVHeader[44]
#ifdef GLOBAL
=
{
	0x52, 0x49, 0x46, 0x46, 0x00, 0x00, 0x00, 0x00,
	0x57, 0x41, 0x56, 0x45, 0x66, 0x6d, 0x74, 0x20,
	0x10, 0x00, 0x00, 0x00, 0x01, 0x00, 0x02, 0x00,
	0x80, 0xbb, 0x00, 0x00, 0x00, 0xee, 0x02, 0x00,
	0x04, 0x00, 0x10, 0x00, 0x64, 0x61, 0x74, 0x61,
	0x00, 0x00, 0x00, 0x00
}
#endif
;

char *NormType[3]
#ifdef GLOBAL
=
{
	"", " drc", " max"
}
#endif
;

char *AC3Mode[8]
#ifdef GLOBAL
=
{
	"1+1", "1/0", "2/0", "3/0", "2/1", "3/1", "2/2", "3/2"
}
#endif
;

int AC3Rate[32]
#ifdef GLOBAL
=
{
	32, 40, 48, 56, 64, 80, 96, 112, 128, 160,
	192, 224, 256, 320, 384, 448, 512, 576, 640,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
}
#endif
;

unsigned char AC3Flush[768]
#ifdef GLOBAL
=
{
	0x0b, 0x77, 0xe3, 0x7a, 0x14, 0x40, 0x4b, 0x73, 
	0xf7, 0x2e, 0x0e, 0xff, 0xeb, 0xc7, 0x08, 0x08, 
	0x28, 0x60, 0x64, 0x1a, 0xc7, 0xcf, 0x9f, 0x3e, 
	0x7c, 0xf9, 0xf3, 0xe7, 0xcf, 0x9f, 0x3e, 0x7c, 
	0xf9, 0xf3, 0xef, 0xf9, 0xd5, 0xf3, 0xe7, 0xcf, 
	0x9f, 0x3e, 0x7c, 0xf9, 0xf3, 0xe7, 0xcf, 0x9f, 
	0x3e, 0x7c, 0xf9, 0xf3, 0xe7, 0xcf, 0x9f, 0x3e, 
	0x7c, 0xf9, 0xf3, 0xef, 0xfe, 0x75, 0x7c, 0xf9, 
	0xf3, 0xe7, 0xcf, 0x9f, 0x3e, 0x7c, 0xf9, 0xf3, 
	0xe7, 0xcf, 0x9f, 0x3e, 0x7c, 0xf9, 0xf3, 0xe7, 
	0xcf, 0x9f, 0x3e, 0x7c, 0xfb, 0xca, 0x0f, 0xff, 
	0x3e, 0x7c, 0x80, 0x88, 0x40, 0x03, 0xbc, 0x5d, 
	0x48, 0xbf, 0x8b, 0xfc, 0x05, 0x98, 0xb9, 0xae, 
	0x4f, 0x0b, 0x31, 0x8f, 0xf0, 0xd2, 0xa0, 0x3c, 
	0xb6, 0xeb, 0xb0, 0x90, 0xac, 0xd1, 0x8f, 0x67, 
	0xdf, 0x40, 0xe8, 0x5e, 0x5a, 0x1e, 0x35, 0xa6, 
	0x00, 0x02, 0x7a, 0x00, 0xd6, 0x9e, 0x40, 0xa3, 
	0xad, 0xe5, 0x64, 0x60, 0x7a, 0x24, 0x55, 0xce, 
	0x2b, 0x0c, 0x83, 0x1c, 0xb2, 0x4d, 0x5e, 0x7c, 
	0xfd, 0x97, 0x56, 0x1d, 0xfe, 0x9a, 0xdb, 0x2e, 
	0xa3, 0x07, 0x5d, 0xe1, 0xdd, 0x8c, 0x4c, 0x65, 
	0x9d, 0xda, 0x17, 0xe9, 0xd1, 0xa1, 0x30, 0x9f, 
	0x6b, 0x91, 0x06, 0xb6, 0x59, 0x5b, 0x09, 0x5e, 
	0x8d, 0xad, 0xa9, 0xc6, 0xf5, 0x38, 0x56, 0x20, 
	0x82, 0xac, 0x80, 0x9b, 0x25, 0xb9, 0x97, 0x66, 
	0xcc, 0x10, 0x0b, 0xb3, 0x69, 0x5f, 0x4c, 0xb1, 
	0xea, 0x57, 0xca, 0x8f, 0x41, 0xa8, 0xf5, 0x7f, 
	0x5c, 0x02, 0x3d, 0xb0, 0x2d, 0x15, 0x12, 0x51, 
	0xa2, 0x92, 0xe4, 0x94, 0xac, 0x27, 0x23, 0xa8, 
	0x3c, 0x85, 0x3e, 0xbc, 0x40, 0x5c, 0xa7, 0x02, 
	0xaa, 0x5c, 0xcd, 0xa9, 0x68, 0x36, 0x20, 0xe1, 
	0x6c, 0x98, 0x10, 0xd9, 0xa4, 0x78, 0xd6, 0x99, 
	0x13, 0x88, 0x21, 0x01, 0xe0, 0x00, 0x9d, 0x80, 
	0x0c, 0xc3, 0x70, 0xc0, 0x6d, 0xe1, 0xf3, 0x9d, 
	0x35, 0x3b, 0x8a, 0x7a, 0xce, 0xec, 0x81, 0x96, 
	0x26, 0x90, 0xe5, 0x2a, 0x68, 0x84, 0x40, 0xb4, 
	0x00, 0xe3, 0x20, 0xae, 0xdb, 0x49, 0x10, 0xd7, 
	0x24, 0x12, 0x5d, 0x28, 0x47, 0x5a, 0xd1, 0xde, 
	0xf0, 0xbe, 0xba, 0xf6, 0xcc, 0x19, 0x63, 0xeb, 
	0xc5, 0xc7, 0x99, 0x7a, 0x49, 0xe5, 0x27, 0x1d, 
	0xc4, 0x4d, 0xd9, 0x12, 0xe0, 0xdd, 0xfb, 0x14, 
	0xcb, 0xb1, 0x19, 0xa0, 0x30, 0xa3, 0x80, 0x70, 
	0xfb, 0xd1, 0x7b, 0x02, 0x99, 0x96, 0x56, 0x51, 
	0xf5, 0x4e, 0x9d, 0xda, 0x3b, 0x56, 0x1d, 0x57, 
	0xd7, 0x88, 0xe1, 0x46, 0xb6, 0x02, 0xf5, 0xa2, 
	0xc2, 0xe0, 0x25, 0xe8, 0x2a, 0x3c, 0xbe, 0x92, 
	0x97, 0x74, 0x4b, 0x9e, 0xb6, 0xe3, 0x18, 0x87, 
	0xb4, 0xa5, 0x72, 0x0a, 0x3c, 0x9e, 0x35, 0xa6, 
	0x43, 0xe2, 0x08, 0x40, 0x7e, 0x00, 0x27, 0x60, 
	0x05, 0xb8, 0xf8, 0x6e, 0xc4, 0xee, 0x72, 0xa6, 
	0xed, 0xef, 0xf0, 0x22, 0x97, 0xb0, 0x9f, 0xfc, 
	0xd9, 0x53, 0x41, 0x20, 0xa9, 0xca, 0xf2, 0x99, 
	0x10, 0x0a, 0xc3, 0x61, 0x03, 0x15, 0xa2, 0x73, 
	0x89, 0xfe, 0x8e, 0xeb, 0x9b, 0xa8, 0x87, 0x94, 
	0x3f, 0x26, 0x9b, 0xa8, 0x6b, 0x7b, 0xa9, 0xf3, 
	0x37, 0x9a, 0xd1, 0x9f, 0x8a, 0x87, 0x11, 0x98, 
	0x6b, 0x33, 0x58, 0xd8, 0xd0, 0xc2, 0xb6, 0x7b, 
	0xd2, 0x28, 0x29, 0x5c, 0x76, 0x56, 0xa0, 0x95, 
	0x94, 0x51, 0x2b, 0x12, 0x43, 0x2a, 0xc7, 0xed, 
	0x89, 0xc6, 0x76, 0x12, 0x5f, 0x37, 0x24, 0x8b, 
	0xca, 0x7f, 0xf2, 0x54, 0xb2, 0x83, 0xde, 0x68, 
	0x3e, 0x65, 0xb7, 0xd1, 0x45, 0x18, 0xbd, 0x8a, 
	0xed, 0x9f, 0xce, 0x90, 0x1e, 0xed, 0xd9, 0xeb, 
	0xe0, 0x06, 0x0e, 0x89, 0x37, 0xf7, 0x8d, 0x69, 
	0x80, 0x00, 0x9e, 0x80, 0x2d, 0x2e, 0x0c, 0x76, 
	0x82, 0x3f, 0x12, 0x20, 0xe3, 0xa9, 0xa2, 0x3a, 
	0x5d, 0x69, 0x28, 0x9f, 0x3b, 0x7a, 0x11, 0x2b, 
	0x09, 0x58, 0x07, 0xfa, 0x53, 0xff, 0x78, 0xe8, 
	0xe6, 0xac, 0x10, 0x12, 0x8d, 0x59, 0xf9, 0xd3, 
	0x94, 0x85, 0x21, 0x87, 0xc8, 0x29, 0x73, 0x8b, 
	0x73, 0x83, 0x1b, 0xfa, 0x23, 0x8d, 0xc6, 0x70, 
	0x23, 0xc6, 0x1f, 0x09, 0x40, 0x27, 0x52, 0x61, 
	0xc4, 0xed, 0xcb, 0x95, 0xbd, 0xd5, 0xf7, 0x00, 
	0x76, 0xda, 0xc0, 0xfe, 0xbc, 0x79, 0x54, 0xec, 
	0x1a, 0x2c, 0xdf, 0x64, 0xdc, 0x31, 0xcb, 0xc4, 
	0xce, 0x03, 0xc6, 0xa8, 0x3c, 0x9f, 0x3b, 0xaa, 
	0xb3, 0x7f, 0xd6, 0xe8, 0x7e, 0x21, 0x84, 0xbd, 
	0x69, 0x40, 0xf0, 0x45, 0x80, 0xd9, 0x06, 0x5d, 
	0x10, 0x66, 0xf2, 0x9f, 0xe4, 0x65, 0xa1, 0x29, 
	0xc8, 0x91, 0xfd, 0xd7, 0x08, 0xde, 0x35, 0xa6, 
	0x00, 0x02, 0x78, 0x00, 0x9c, 0xd4, 0x0e, 0xc6, 
	0x86, 0xc9, 0x2d, 0x3a, 0xf6, 0x87, 0x28, 0xae, 
	0xda, 0xbe, 0x71, 0x54, 0x24, 0x1e, 0x77, 0x5b, 
	0xc2, 0x58, 0xaa, 0xf3, 0xa6, 0x1a, 0x7a, 0x4b, 
	0xbe, 0x15, 0x57, 0x95, 0xfb, 0xf9, 0xb1, 0x00, 
	0x4e, 0x76, 0xf8, 0xbb, 0xa5, 0x3d, 0x9c, 0xf8, 
	0xf2, 0xfc, 0x0d, 0xe6, 0x23, 0x64, 0xbb, 0xb4, 
	0x2a, 0x25, 0x16, 0x94, 0xf5, 0xef, 0x8e, 0xb5, 
	0x76, 0x72, 0x93, 0x46, 0x9b, 0x5f, 0x95, 0x79, 
	0x55, 0x64, 0x04, 0x7d, 0x95, 0x32, 0x4f, 0x81, 
	0x49, 0x79, 0xe8, 0xb7, 0x63, 0xe9, 0x3e, 0x4e, 
	0xd1, 0x33, 0xc1, 0x76, 0x85, 0x05, 0xe1, 0x5e, 
	0x6d, 0x10, 0x0e, 0x38, 0x7a, 0x04, 0xb8, 0x33, 
	0x9d, 0x91, 0x4f, 0x7e, 0xc4, 0x68, 0x43, 0x4b, 
	0xe1, 0x37, 0x04, 0xc9, 0xe2, 0xaf, 0x02, 0x27, 
	0xb9, 0x80, 0xad, 0x78, 0xd6, 0x80, 0xea, 0x1e
}
#endif
;

/* default intra quantization matrix */
unsigned char default_intra_quantizer_matrix[64]
#ifdef GLOBAL
=
{
	8, 16, 19, 22, 26, 27, 29, 34,
	16, 16, 22, 24, 27, 29, 34, 37,
	19, 22, 26, 27, 29, 34, 34, 38,
	22, 22, 26, 27, 29, 34, 37, 40,
	22, 26, 27, 29, 32, 35, 40, 48,
	26, 27, 29, 32, 35, 40, 48, 58,
	26, 27, 29, 34, 38, 46, 56, 69,
	27, 29, 35, 38, 46, 56, 69, 83
}
#endif
;

/* zig-zag and alternate scan patterns */
unsigned char scan[2][64]
#ifdef GLOBAL
=
{
	{ // Zig-Zag scan pattern
		0,  1,  8, 16,  9,  2,  3, 10,
	   17, 24, 32, 25, 18, 11,  4,  5,
	   12, 19, 26, 33, 40, 48, 41, 34,
	   27, 20, 13,  6,  7, 14, 21, 28,
	   35, 42, 49, 56, 57, 50, 43, 36,
	   29, 22, 15, 23, 30, 37, 44, 51,
	   58, 59, 52, 45, 38, 31, 39, 46,
	   53, 60, 61, 54, 47, 55, 62, 63
	}
	,
	{ // Alternate scan pattern
		0,  8, 16, 24,  1,  9,  2, 10,
	   17, 25, 32, 40, 48, 56, 57, 49,
	   41, 33, 26, 18,  3, 11, 4,  12,
	   19, 27, 34, 42, 50, 58, 35, 43,
	   51, 59, 20, 28,  5, 13,  6, 14,
	   21, 29, 36, 44, 52, 60, 37, 45,
	   53, 61, 22, 30,  7, 15, 23, 31,
	   38, 46, 54, 62, 39, 47, 55, 63
	}
}
#endif
;

/* non-linear quantization coefficient table */
unsigned char Non_Linear_quantizer_scale[32]
#ifdef GLOBAL
=
{
	0, 1, 2, 3, 4, 5, 6, 7,
	8, 10, 12, 14, 16, 18, 20, 22,
	24, 28, 32, 36, 40, 44, 48, 52,
	56, 64, 72, 80, 88, 96, 104, 112
}
#endif
;

#define ERROR_VALUE	(-1)

typedef struct {
	char run, level, len;
}	DCTtab;

typedef struct {
	char val, len;
}	VLCtab;

/* Table B-10, motion_code, codes 0001 ... 01xx */
VLCtab MVtab0[8]
#ifdef GLOBAL
=
{
	{ERROR_VALUE,0}, {3,3}, {2,2}, {2,2}, {1,1}, {1,1}, {1,1}, {1,1}
}
#endif
;

/* Table B-10, motion_code, codes 0000011 ... 000011x */
VLCtab MVtab1[8]
#ifdef GLOBAL
=
{
	{ERROR_VALUE,0}, {ERROR_VALUE,0}, {ERROR_VALUE,0}, {7,6}, {6,6}, {5,6}, {4,5}, {4,5}
}
#endif
;

/* Table B-10, motion_code, codes 0000001100 ... 000001011x */
VLCtab MVtab2[12]
#ifdef GLOBAL
=
{
	{16,9}, {15,9}, {14,9}, {13,9},
	{12,9}, {11,9}, {10,8}, {10,8},
	{9,8},  {9,8},  {8,8},  {8,8}
}
#endif
;

/* Table B-9, coded_block_pattern, codes 01000 ... 111xx */
VLCtab CBPtab0[32]
#ifdef GLOBAL
=
{
	{ERROR_VALUE,0}, {ERROR_VALUE,0}, {ERROR_VALUE,0}, {ERROR_VALUE,0},
	{ERROR_VALUE,0}, {ERROR_VALUE,0}, {ERROR_VALUE,0}, {ERROR_VALUE,0},
	{62,5}, {2,5},  {61,5}, {1,5},  {56,5}, {52,5}, {44,5}, {28,5},
	{40,5}, {20,5}, {48,5}, {12,5}, {32,4}, {32,4}, {16,4}, {16,4},
	{8,4},  {8,4},  {4,4},  {4,4},  {60,3}, {60,3}, {60,3}, {60,3}
}
#endif
;

/* Table B-9, coded_block_pattern, codes 00000100 ... 001111xx */
VLCtab CBPtab1[64]
#ifdef GLOBAL
=
{
	{ERROR_VALUE,0}, {ERROR_VALUE,0}, {ERROR_VALUE,0}, {ERROR_VALUE,0},
	{58,8}, {54,8}, {46,8}, {30,8},
	{57,8}, {53,8}, {45,8}, {29,8}, {38,8}, {26,8}, {37,8}, {25,8},
	{43,8}, {23,8}, {51,8}, {15,8}, {42,8}, {22,8}, {50,8}, {14,8},
	{41,8}, {21,8}, {49,8}, {13,8}, {35,8}, {19,8}, {11,8}, {7,8},
	{34,7}, {34,7}, {18,7}, {18,7}, {10,7}, {10,7}, {6,7},  {6,7},
	{33,7}, {33,7}, {17,7}, {17,7}, {9,7},  {9,7},  {5,7},  {5,7},
	{63,6}, {63,6}, {63,6}, {63,6}, {3,6},  {3,6},  {3,6},  {3,6},
	{36,6}, {36,6}, {36,6}, {36,6}, {24,6}, {24,6}, {24,6}, {24,6}
}
#endif
;

/* Table B-9, coded_block_pattern, codes 000000001 ... 000000111 */
VLCtab CBPtab2[8]
#ifdef GLOBAL
=
{
	{ERROR_VALUE,0}, {0,9}, {39,9}, {27,9}, {59,9}, {55,9}, {47,9}, {31,9}
}
#endif
;

/* Table B-1, macroblock_address_increment, codes 00010 ... 011xx */
VLCtab MBAtab1[16]
#ifdef GLOBAL
=
{
	{ERROR_VALUE,0}, {ERROR_VALUE,0}, {7,5}, {6,5}, {5,4}, {5,4}, {4,4},
	{4,4}, {3,3}, {3,3}, {3,3}, {3,3}, {2,3}, {2,3}, {2,3}, {2,3}
}
#endif
;

/* Table B-1, macroblock_address_increment, codes 00000011000 ... 0000111xxxx */
VLCtab MBAtab2[104]
#ifdef GLOBAL
=
{
	{33,11}, {32,11}, {31,11}, {30,11}, {29,11}, {28,11}, {27,11}, {26,11},
	{25,11}, {24,11}, {23,11}, {22,11}, {21,10}, {21,10}, {20,10}, {20,10},
	{19,10}, {19,10}, {18,10}, {18,10}, {17,10}, {17,10}, {16,10}, {16,10},
	{15,8},  {15,8},  {15,8},  {15,8},  {15,8},  {15,8},  {15,8},  {15,8},
	{14,8},  {14,8},  {14,8},  {14,8},  {14,8},  {14,8},  {14,8},  {14,8},
	{13,8},  {13,8},  {13,8},  {13,8},  {13,8},  {13,8},  {13,8},  {13,8},
	{12,8},  {12,8},  {12,8},  {12,8},  {12,8},  {12,8},  {12,8},  {12,8},
	{11,8},  {11,8},  {11,8},  {11,8},  {11,8},  {11,8},  {11,8},  {11,8},
	{10,8},  {10,8},  {10,8},  {10,8},  {10,8},  {10,8},  {10,8},  {10,8},
	{9,7},   {9,7},   {9,7},   {9,7},   {9,7},   {9,7},   {9,7},   {9,7},
	{9,7},   {9,7},   {9,7},   {9,7},   {9,7},   {9,7},   {9,7},   {9,7},
	{8,7},   {8,7},   {8,7},   {8,7},   {8,7},   {8,7},   {8,7},   {8,7},
	{8,7},   {8,7},   {8,7},   {8,7},   {8,7},   {8,7},   {8,7},   {8,7}
}
#endif
;

/* Table B-12, dct_dc_size_luminance, codes 00xxx ... 11110 */
VLCtab DClumtab0[32]
#ifdef GLOBAL
=
{
	{1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2},
	{2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2},
	{0, 3}, {0, 3}, {0, 3}, {0, 3}, {3, 3}, {3, 3}, {3, 3}, {3, 3},
	{4, 3}, {4, 3}, {4, 3}, {4, 3}, {5, 4}, {5, 4}, {6, 5}, {ERROR_VALUE, 0}
}
#endif
;

/* Table B-12, dct_dc_size_luminance, codes 111110xxx ... 111111111 */
VLCtab DClumtab1[16]
#ifdef GLOBAL
=
{
	{7, 6}, {7, 6}, {7, 6}, {7, 6}, {7, 6}, {7, 6}, {7, 6}, {7, 6},
	{8, 7}, {8, 7}, {8, 7}, {8, 7}, {9, 8}, {9, 8}, {10,9}, {11,9}
}
#endif
;

/* Table B-13, dct_dc_size_chrominance, codes 00xxx ... 11110 */
VLCtab DCchromtab0[32]
#ifdef GLOBAL
=
{
	{0, 2}, {0, 2}, {0, 2}, {0, 2}, {0, 2}, {0, 2}, {0, 2}, {0, 2},
	{1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2},
	{2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2}, {2, 2},
	{3, 3}, {3, 3}, {3, 3}, {3, 3}, {4, 4}, {4, 4}, {5, 5}, {ERROR_VALUE, 0}
}
#endif
;

/* Table B-13, dct_dc_size_chrominance, codes 111110xxxx ... 1111111111 */
VLCtab DCchromtab1[32]
#ifdef GLOBAL
=
{
	{6, 6}, {6, 6}, {6, 6}, {6, 6}, {6, 6}, {6, 6}, {6, 6}, {6, 6},
	{6, 6}, {6, 6}, {6, 6}, {6, 6}, {6, 6}, {6, 6}, {6, 6}, {6, 6},
	{7, 7}, {7, 7}, {7, 7}, {7, 7}, {7, 7}, {7, 7}, {7, 7}, {7, 7},
	{8, 8}, {8, 8}, {8, 8}, {8, 8}, {9, 9}, {9, 9}, {10,10}, {11,10}
}
#endif
;

/* Table B-14, DCT coefficients table zero,
 * codes 0100 ... 1xxx (used for first (DC) coefficient)
 */
DCTtab DCTtabfirst[12]
#ifdef GLOBAL
=
{
	{0,2,4}, {2,1,4}, {1,1,3}, {1,1,3},
	{0,1,1}, {0,1,1}, {0,1,1}, {0,1,1},
	{0,1,1}, {0,1,1}, {0,1,1}, {0,1,1}
}
#endif
;

/* Table B-14, DCT coefficients table zero,
 * codes 0100 ... 1xxx (used for all other coefficients)
 */
DCTtab DCTtabnext[12]
#ifdef GLOBAL
=
{
	{0,2,4},  {2,1,4},  {1,1,3},  {1,1,3},
	{64,0,2}, {64,0,2}, {64,0,2}, {64,0,2}, /* EOB */
	{0,1,2},  {0,1,2},  {0,1,2},  {0,1,2}
}
#endif
;

/* Table B-14, DCT coefficients table zero,
 * codes 000001xx ... 00111xxx
 */
DCTtab DCTtab0[60]
#ifdef GLOBAL
=
{
	{65,0,6}, {65,0,6}, {65,0,6}, {65,0,6}, /* Escape */
	{2,2,7}, {2,2,7}, {9,1,7}, {9,1,7},
	{0,4,7}, {0,4,7}, {8,1,7}, {8,1,7},
	{7,1,6}, {7,1,6}, {7,1,6}, {7,1,6},
	{6,1,6}, {6,1,6}, {6,1,6}, {6,1,6},
	{1,2,6}, {1,2,6}, {1,2,6}, {1,2,6},
	{5,1,6}, {5,1,6}, {5,1,6}, {5,1,6},
	{13,1,8}, {0,6,8}, {12,1,8}, {11,1,8},
	{3,2,8}, {1,3,8}, {0,5,8}, {10,1,8},
	{0,3,5}, {0,3,5}, {0,3,5}, {0,3,5},
	{0,3,5}, {0,3,5}, {0,3,5}, {0,3,5},
	{4,1,5}, {4,1,5}, {4,1,5}, {4,1,5},
	{4,1,5}, {4,1,5}, {4,1,5}, {4,1,5},
	{3,1,5}, {3,1,5}, {3,1,5}, {3,1,5},
	{3,1,5}, {3,1,5}, {3,1,5}, {3,1,5}
}
#endif
;

/* Table B-15, DCT coefficients table one,
 * codes 000001xx ... 11111111
*/
DCTtab DCTtab0a[252]
#ifdef GLOBAL
=
{
	{65,0,6}, {65,0,6}, {65,0,6}, {65,0,6}, /* Escape */
	{7,1,7}, {7,1,7}, {8,1,7}, {8,1,7},
	{6,1,7}, {6,1,7}, {2,2,7}, {2,2,7},
	{0,7,6}, {0,7,6}, {0,7,6}, {0,7,6},
	{0,6,6}, {0,6,6}, {0,6,6}, {0,6,6},
	{4,1,6}, {4,1,6}, {4,1,6}, {4,1,6},
	{5,1,6}, {5,1,6}, {5,1,6}, {5,1,6},
	{1,5,8}, {11,1,8}, {0,11,8}, {0,10,8},
	{13,1,8}, {12,1,8}, {3,2,8}, {1,4,8},
	{2,1,5}, {2,1,5}, {2,1,5}, {2,1,5},
	{2,1,5}, {2,1,5}, {2,1,5}, {2,1,5},
	{1,2,5}, {1,2,5}, {1,2,5}, {1,2,5},
	{1,2,5}, {1,2,5}, {1,2,5}, {1,2,5},
	{3,1,5}, {3,1,5}, {3,1,5}, {3,1,5},
	{3,1,5}, {3,1,5}, {3,1,5}, {3,1,5},
	{1,1,3}, {1,1,3}, {1,1,3}, {1,1,3},
	{1,1,3}, {1,1,3}, {1,1,3}, {1,1,3},
	{1,1,3}, {1,1,3}, {1,1,3}, {1,1,3},
	{1,1,3}, {1,1,3}, {1,1,3}, {1,1,3},
	{1,1,3}, {1,1,3}, {1,1,3}, {1,1,3},
	{1,1,3}, {1,1,3}, {1,1,3}, {1,1,3},
	{1,1,3}, {1,1,3}, {1,1,3}, {1,1,3},
	{1,1,3}, {1,1,3}, {1,1,3}, {1,1,3},
	{64,0,4}, {64,0,4}, {64,0,4}, {64,0,4}, /* EOB */
	{64,0,4}, {64,0,4}, {64,0,4}, {64,0,4},
	{64,0,4}, {64,0,4}, {64,0,4}, {64,0,4},
	{64,0,4}, {64,0,4}, {64,0,4}, {64,0,4},
	{0,3,4}, {0,3,4}, {0,3,4}, {0,3,4},
	{0,3,4}, {0,3,4}, {0,3,4}, {0,3,4},
	{0,3,4}, {0,3,4}, {0,3,4}, {0,3,4},
	{0,3,4}, {0,3,4}, {0,3,4}, {0,3,4},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,1,2}, {0,1,2}, {0,1,2}, {0,1,2},
	{0,2,3}, {0,2,3}, {0,2,3}, {0,2,3},
	{0,2,3}, {0,2,3}, {0,2,3}, {0,2,3},
	{0,2,3}, {0,2,3}, {0,2,3}, {0,2,3},
	{0,2,3}, {0,2,3}, {0,2,3}, {0,2,3},
	{0,2,3}, {0,2,3}, {0,2,3}, {0,2,3},
	{0,2,3}, {0,2,3}, {0,2,3}, {0,2,3},
	{0,2,3}, {0,2,3}, {0,2,3}, {0,2,3},
	{0,2,3}, {0,2,3}, {0,2,3}, {0,2,3},
	{0,4,5}, {0,4,5}, {0,4,5}, {0,4,5},
	{0,4,5}, {0,4,5}, {0,4,5}, {0,4,5},
	{0,5,5}, {0,5,5}, {0,5,5}, {0,5,5},
	{0,5,5}, {0,5,5}, {0,5,5}, {0,5,5},
	{9,1,7}, {9,1,7}, {1,3,7}, {1,3,7},
	{10,1,7}, {10,1,7}, {0,8,7}, {0,8,7},
	{0,9,7}, {0,9,7}, {0,12,8}, {0,13,8},
	{2,3,8}, {4,2,8}, {0,14,8}, {0,15,8}
}
#endif
;

/* Table B-14, DCT coefficients table zero,
 * codes 0000001000 ... 0000001111
 */
DCTtab DCTtab1[8]
#ifdef GLOBAL
=
{
	{16,1,10}, {5,2,10}, {0,7,10}, {2,3,10},
	{1,4,10}, {15,1,10}, {14,1,10}, {4,2,10}
}
#endif
;

/* Table B-15, DCT coefficients table one,
 * codes 000000100x ... 000000111x
 */
DCTtab DCTtab1a[8]
#ifdef GLOBAL
=
{
	{5,2,9}, {5,2,9}, {14,1,9}, {14,1,9},
	{2,4,10}, {16,1,10}, {15,1,9}, {15,1,9}
}
#endif
;

/* Table B-14/15, DCT coefficients table zero / one,
 * codes 000000010000 ... 000000011111
 */
DCTtab DCTtab2[16]
#ifdef GLOBAL
=
{
	{0,11,12}, {8,2,12}, {4,3,12}, {0,10,12},
	{2,4,12}, {7,2,12}, {21,1,12}, {20,1,12},
	{0,9,12}, {19,1,12}, {18,1,12}, {1,5,12},
	{3,3,12}, {0,8,12}, {6,2,12}, {17,1,12}
}
#endif
;

/* Table B-14/15, DCT coefficients table zero / one,
 * codes 0000000010000 ... 0000000011111
 */
DCTtab DCTtab3[16]
#ifdef GLOBAL
=
{
	{10,2,13}, {9,2,13}, {5,3,13}, {3,4,13},
	{2,5,13}, {1,7,13}, {1,6,13}, {0,15,13},
	{0,14,13}, {0,13,13}, {0,12,13}, {26,1,13},
	{25,1,13}, {24,1,13}, {23,1,13}, {22,1,13}
}
#endif
;

/* Table B-14/15, DCT coefficients table zero / one,
 * codes 00000000010000 ... 00000000011111
 */
DCTtab DCTtab4[16]
#ifdef GLOBAL
=
{
	{0,31,14}, {0,30,14}, {0,29,14}, {0,28,14},
	{0,27,14}, {0,26,14}, {0,25,14}, {0,24,14},
	{0,23,14}, {0,22,14}, {0,21,14}, {0,20,14},
	{0,19,14}, {0,18,14}, {0,17,14}, {0,16,14}
}
#endif
;

/* Table B-14/15, DCT coefficients table zero / one,
 * codes 000000000010000 ... 000000000011111
 */
DCTtab DCTtab5[16]
#ifdef GLOBAL
=
{
	{0,40,15}, {0,39,15}, {0,38,15}, {0,37,15},
	{0,36,15}, {0,35,15}, {0,34,15}, {0,33,15},
	{0,32,15}, {1,14,15}, {1,13,15}, {1,12,15},
	{1,11,15}, {1,10,15}, {1,9,15}, {1,8,15}
}
#endif
;

/* Table B-14/15, DCT coefficients table zero / one,
 * codes 0000000000010000 ... 0000000000011111
 */
DCTtab DCTtab6[16]
#ifdef GLOBAL
=
{
	{1,18,16}, {1,17,16}, {1,16,16}, {1,15,16},
	{6,3,16}, {16,2,16}, {15,2,16}, {14,2,16},
	{13,2,16}, {12,2,16}, {11,2,16}, {31,1,16},
	{30,1,16}, {29,1,16}, {28,1,16}, {27,1,16}
}
#endif
;

/* Table B-3, macroblock_type in P-pictures, codes 001..1xx */
VLCtab PMBtab0[8]
#ifdef GLOBAL
=
{
	{ERROR_VALUE,0},
	{MACROBLOCK_MOTION_FORWARD,3},
	{MACROBLOCK_PATTERN,2}, {MACROBLOCK_PATTERN,2},
	{MACROBLOCK_MOTION_FORWARD|MACROBLOCK_PATTERN,1}, 
	{MACROBLOCK_MOTION_FORWARD|MACROBLOCK_PATTERN,1},
	{MACROBLOCK_MOTION_FORWARD|MACROBLOCK_PATTERN,1}, 
	{MACROBLOCK_MOTION_FORWARD|MACROBLOCK_PATTERN,1}
}
#endif
;

/* Table B-3, macroblock_type in P-pictures, codes 000001..00011x */
VLCtab PMBtab1[8]
#ifdef GLOBAL
=
{
	{ERROR_VALUE,0},
	{MACROBLOCK_QUANT|MACROBLOCK_INTRA,6},
	{MACROBLOCK_QUANT|MACROBLOCK_PATTERN,5}, {MACROBLOCK_QUANT|MACROBLOCK_PATTERN,5},
	{MACROBLOCK_QUANT|MACROBLOCK_MOTION_FORWARD|MACROBLOCK_PATTERN,5}, {MACROBLOCK_QUANT|MACROBLOCK_MOTION_FORWARD|MACROBLOCK_PATTERN,5},
	{MACROBLOCK_INTRA,5}, {MACROBLOCK_INTRA,5}
}
#endif
;

/* Table B-4, macroblock_type in B-pictures, codes 0010..11xx */
VLCtab BMBtab0[16]
#ifdef GLOBAL
=
{
	{ERROR_VALUE,0}, 
	{ERROR_VALUE,0},
	{MACROBLOCK_MOTION_FORWARD,4},
	{MACROBLOCK_MOTION_FORWARD|MACROBLOCK_PATTERN,4},
	{MACROBLOCK_MOTION_BACKWARD,3}, 
	{MACROBLOCK_MOTION_BACKWARD,3},
	{MACROBLOCK_MOTION_BACKWARD|MACROBLOCK_PATTERN,3}, 
	{MACROBLOCK_MOTION_BACKWARD|MACROBLOCK_PATTERN,3},
	{MACROBLOCK_MOTION_FORWARD|MACROBLOCK_MOTION_BACKWARD,2}, 
	{MACROBLOCK_MOTION_FORWARD|MACROBLOCK_MOTION_BACKWARD,2},
	{MACROBLOCK_MOTION_FORWARD|MACROBLOCK_MOTION_BACKWARD,2}, 
	{MACROBLOCK_MOTION_FORWARD|MACROBLOCK_MOTION_BACKWARD,2},
	{MACROBLOCK_MOTION_FORWARD|MACROBLOCK_MOTION_BACKWARD|MACROBLOCK_PATTERN,2},
	{MACROBLOCK_MOTION_FORWARD|MACROBLOCK_MOTION_BACKWARD|MACROBLOCK_PATTERN,2},
	{MACROBLOCK_MOTION_FORWARD|MACROBLOCK_MOTION_BACKWARD|MACROBLOCK_PATTERN,2},
	{MACROBLOCK_MOTION_FORWARD|MACROBLOCK_MOTION_BACKWARD|MACROBLOCK_PATTERN,2}
}
#endif
;

/* Table B-4, macroblock_type in B-pictures, codes 000001..00011x */
VLCtab BMBtab1[8]
#ifdef GLOBAL
=
{
	{ERROR_VALUE,0},
	{MACROBLOCK_QUANT|MACROBLOCK_INTRA,6},
	{MACROBLOCK_QUANT|MACROBLOCK_MOTION_BACKWARD|MACROBLOCK_PATTERN,6},
	{MACROBLOCK_QUANT|MACROBLOCK_MOTION_FORWARD|MACROBLOCK_PATTERN,6},
	{MACROBLOCK_QUANT|MACROBLOCK_MOTION_FORWARD|MACROBLOCK_MOTION_BACKWARD|MACROBLOCK_PATTERN,5},
	{MACROBLOCK_QUANT|MACROBLOCK_MOTION_FORWARD|MACROBLOCK_MOTION_BACKWARD|MACROBLOCK_PATTERN,5},
	{MACROBLOCK_INTRA,5}, 
	{MACROBLOCK_INTRA,5}
}
#endif
;
