/* 
 *	Copyright (C) Chia-chen Kuo - Jan 2001
 *
 *  This file is part of DVD2AVI, a free MPEG-2 decoder
 *	
 *  DVD2AVI is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  DVD2AVI is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */

#include "global.h"
#include "getbit.h"

static int ChromaFormat[4] = {
	0, 6, 8, 12
};

static void GOPBack(void);
static void InitialDecoder();

DWORD MPEG2DecInit()
{
	int i, code;

	Stop_Flag = Rip_Flag = Fault_Flag = 0;
	Frame_Number = Second_Field = 0;
	VOB_ID = CELL_ID = 0;
	Sound_Max = 1; Bitrate_Meter = 0;


	for(i=0; i<CHANNEL; i++)
	{
		ZeroMemory(&mpa[i], sizeof(MPAStream));	
		ZeroMemory(&ac3[i], sizeof(AC3Stream));
		ZeroMemory(&pcm[i], sizeof(PCMStream));
	}

	ZeroMemory(&CH, sizeof(CH));
	
	switch (process.locate)
	{
		case LOCATE_FORWARD:
			process.startfile = process.file;
			process.startloc = (process.lba + 2) * BfrSize;

			process.end = process.total - BfrSize;
			process.endfile = File_Limit - 1;
			process.endloc = (process.length[File_Limit-1]/BfrSize - 1) * BfrSize;
			break;

		case LOCATE_BACKWARD:
			process.startfile = process.file;
			process.startloc = (process.lba - 2) * BfrSize;

			process.end = process.total - BfrSize;
			process.endfile = File_Limit - 1;
			process.endloc = (process.length[File_Limit-1]/BfrSize - 1)*BfrSize;

			GOPBack();
			break;

		case LOCATE_RIP:
			process.startfile = process.leftfile;
			process.startloc = process.leftlba * BfrSize;
			process.endfile = process.rightfile;
			process.endloc = (process.rightlba - 1) * BfrSize;

			process.run = 0;
			for (i=0; i<process.startfile; i++)
				process.run += process.length[i];
			process.start = process.run + process.startloc;

			process.end = 0;
			for (i=0; i<process.endfile; i++)
				process.end += process.length[i];
			process.end += process.endloc;
			break;
	}

	//process.startloc = 25000000;
	// search MPEG-2 Sequence Header
	if (!Check_Flag)
	{
		File_Flag = 0;
		_lseeki64(Infile[0], 0, SEEK_SET);
		Initialize_Buffer();
		

		while (!Check_Flag)
		{
			if(Fault_Flag > CRITICAL_ERROR_LEVEL)
				return (0);
			next_start_code();
			code = Get_Bits(32);

			switch (code)
			{
				case PACK_START_CODE:
					SystemStream_Flag = 1;
					break;

				case SEQUENCE_HEADER_CODE:
					sequence_header();
					InitialDecoder();
					Check_Flag = 1;
					break;
			}
		}
	}

	Frame_Rate = (FO_Flag==FO_FILM) ? frame_rate*4/5 : frame_rate;


	File_Flag = process.startfile;
	_lseeki64(Infile[process.startfile], (process.startloc/BfrSize)*BfrSize, SEEK_SET);
	Initialize_Buffer();


	process.op = 0;

	while (Get_Hdr() && picture_coding_type!=I_TYPE);		// search I

	Rip_Flag = 1;
	process.file = d2v_current.file;		// location
	process.lba = d2v_current.lba;

	Decode_Picture();

	while (Get_Hdr() && picture_coding_type==B_TYPE);		// search I or P

	//Decode_Picture();						//MAF Moved to High Level.

	process.op = timeGetTime();



	return 0;
}




static void GOPBack()
{
	int startfile;
	__int64 startloc, endloc;

	startfile = process.startfile;
	startloc = process.startloc;

	for (;;)
	{
		endloc = startloc + BfrSize;
		startloc -= BfrSize * 16;

		if (startloc > 0)
		{
			process.startfile = startfile;
			process.startloc = startloc;
		}
		else
		{
			process.startloc = 0;
			return;
		}

		_lseeki64(Infile[process.startfile], process.startloc, SEEK_SET);
		Initialize_Buffer();

		while (Get_Hdr() && _telli64(Infile[process.startfile]) <= endloc)
		{
			if (picture_coding_type==I_TYPE)
			{
				process.startfile = d2v_current.file;
				process.startloc = d2v_current.lba * BfrSize;
				return;
			}
		}
	}
}


static void InitialDecoder(void)
{
	int size;

	mb_width = (horizontal_size+15)/16;
	mb_height = progressive_sequence ? (vertical_size+15)/16 : 2*((vertical_size+31)/32);

	Coded_Picture_Width = 16 * mb_width;
	Coded_Picture_Height = 16 * mb_height;

	Chroma_Width = (chroma_format==CHROMA444) ? Coded_Picture_Width : Coded_Picture_Width>>1;
	Chroma_Height = (chroma_format!=CHROMA420) ? Coded_Picture_Height : Coded_Picture_Height>>1;

	block_count = ChromaFormat[chroma_format];

	hheightd2 = Coded_Picture_Height / 2 - 2;
	qheightd2 = Coded_Picture_Height / 4 - 2;
	hwidth = Coded_Picture_Width / 2;
	hwidthd8 = Coded_Picture_Width / 2 - 8;
	dwidth = Coded_Picture_Width * 2;
	qwidth = Coded_Picture_Width * 4;
	nwidth = Coded_Picture_Width * 12;
	area = Coded_Picture_Width * Coded_Picture_Height;

	// -----------------------------------------------------------------------------------------

	size = Coded_Picture_Width * Coded_Picture_Height;
	size += ((Chroma_Width * Chroma_Height)*2);

	// configure the Y buffer pointer (Also Base for V and U planes)

	backward_reference_frame[0] = (unsigned char*)malloc(size);
	forward_reference_frame[0] = (unsigned char*)malloc(size);
	auxframe[0] = (unsigned char*)malloc(size);

	size = Coded_Picture_Width * Coded_Picture_Height;

	// Now configure the V buffer pointer
	backward_reference_frame[2] = backward_reference_frame[0]+size;
	forward_reference_frame[2] = forward_reference_frame[0]+size;
	auxframe[2] = auxframe[0]+size;

	size = Chroma_Width * Chroma_Height;

	// And Last configure the U buffer pointer
	backward_reference_frame[1] = backward_reference_frame[2]+size;
	forward_reference_frame[1] = forward_reference_frame[2]+size;
	auxframe[1] = auxframe[2]+size;

	//for (i=0; i<3; i++)
	//{
	//	if (i==0)
	//		size = Coded_Picture_Width * Coded_Picture_Height;
	//	else
	//		size = Chroma_Width * Chroma_Height;
	//
	//	backward_reference_frame[i] = (unsigned char*)malloc(size);
	//	forward_reference_frame[i] = (unsigned char*)malloc(size);
	//	auxframe[i] = (unsigned char*)malloc(size);
	//}

	u422 = (unsigned char*)malloc(Coded_Picture_Width*Coded_Picture_Height/2);
	v422 = (unsigned char*)malloc(Coded_Picture_Width*Coded_Picture_Height/2);
	u444 = (unsigned char*)malloc(Coded_Picture_Width*Coded_Picture_Height);
	v444 = (unsigned char*)malloc(Coded_Picture_Width*Coded_Picture_Height);

	//lum = (unsigned char*)malloc(Coded_Picture_Width*Coded_Picture_Height);
	
	lum = NULL;

	rgb32 = NULL;
	yuy2 = NULL;
	yv12 = NULL;

	switch(Store_Flag)
		{
		case STORE_YUY2:
			yuy2 = (unsigned char*)malloc(Coded_Picture_Width*Coded_Picture_Height*2);
			break;
		case STORE_RGB32:
			rgb32 = (unsigned char*)malloc(Coded_Picture_Width*Coded_Picture_Height*4);
			break;
		case STORE_YV12:
			yv12 = (unsigned char*)malloc(Coded_Picture_Width*Coded_Picture_Height*2);
			break;
		}
	// -----------------------------------------------------------------------------------------

}

