/* 
 *	Copyright (c) 2000 by WTC
 *
 *  This file is part of WAVEFS44, a free sound sampling rate converter
 *	
 *  WAVEFS44 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  WAVEFS44 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */

#include "global.h"
#include <math.h>

#define M_PI		3.1415926535897932384626433832795
#define ISRM		147
#define OSRM		160
#define WINSIZ		1024		/*16384*/
#define MAXFIRODR	65535
#define ALPHA		10.0

typedef struct {
	short l;
	short r;
}	SmplData;

static int firodr[5] = {0, 4095, 12287, 24575, 32767};
static int lpfcof[5] = {0, 19400, 21200, 21400, 21600};
static unsigned char FQ44100[6] = {0x44, 0xac, 0x00, 0x00, 0x10, 0xb1};

static int sptr, sptrr, eptr, eptrr, ismd, ismr, winpos, iptr, firodrv;
static double hfir[MAXFIRODR+1];
static double suml, sumr, frqc, frqs, regv, wfnc, divisor, hgain;
static SmplData smpld[WINSIZ*2], smplo, *spp;

static double bessel0(double);
int DownKernel(unsigned char * dest);
int resample48To44End(unsigned char * dest);

#define NORMAL_QUALITY 2

__forceinline short SaturateRound(double flt)
{
	int tmp;

	__asm
	{
		fld		[flt]
		fistp	[tmp]
	}

	if (abs(tmp) < 32768)
		return tmp;
	else if (tmp > 32767)
		return 32767;
	else
		return -32768;
}

void InitialSRC()
{
	int i;

	frqs = 44100*OSRM;					// virtual high sampling rate
	frqc = lpfcof[NORMAL_QUALITY];			// cutoff freq.
	firodrv = firodr[NORMAL_QUALITY];			// FIR order = firodrv*2+1

	divisor = bessel0(ALPHA);
	hgain = (2.0*ISRM*frqc)/frqs;
	hfir[0] = hgain;

	for (i=1; i<=firodrv; i++)
	{
		regv = (double)(i*i)/(firodrv*firodrv);
		wfnc = bessel0(sqrt(1.0-regv)*ALPHA)/divisor;
		regv = (2.0*M_PI*frqc*i)/frqs;
		hfir[i] = (hgain*sin(regv)*wfnc)/regv;
	}

	ismd=OSRM/ISRM;
	ismr=OSRM%ISRM;

	sptr = -(firodrv/ISRM);
	sptrr = -(firodrv%ISRM);
	eptr = firodrv/ISRM;
	eptrr = firodrv%ISRM;

	ZeroMemory(smpld, sizeof(smpld));

	winpos = 0;
	iptr = 0;
}


int resample48To44(unsigned char * source, unsigned char * dest, int size)
{
int outBytes;
	
	outBytes =0;

	if (winpos < WINSIZ)
		{
		if ((winpos + (size>>2)) < WINSIZ)
			{
			memcpy(&smpld[winpos], source, size);
			}
		else
			{
			memcpy(&smpld[winpos], source, (WINSIZ-winpos)<<2);
			outBytes += DownKernel(dest);
			memcpy(&smpld[WINSIZ], source+((WINSIZ-winpos)<<2), size-((WINSIZ-winpos)<<2));
			}
		winpos += size>>2;
		}
	else
		{
		if ((winpos + (size>>2)) < (WINSIZ<<1))
			{
			memcpy(&smpld[winpos], source, size);
			winpos += size>>2;
			}
		else
			{
			memcpy(&smpld[winpos], source , ((WINSIZ<<1)-winpos)<<2);
			outBytes += DownKernel(dest);
			memcpy(&smpld[0], source+(((WINSIZ<<1)-winpos)<<2), size-(((WINSIZ<<1)-winpos)<<2));
			winpos += (size>>2) - (WINSIZ<<1);
			}
		}

	
return(outBytes);

}

int resample48To44End(unsigned char * dest)
{
	int i;
	int outBytes;
	
	outBytes = 0;

	if (winpos < WINSIZ)
	{
		for (i=WINSIZ-1; i>=winpos; i--)
		{
			smpld[i].l = 0;
			smpld[i].r = 0;
		}

		outBytes += DownKernel(dest);
	}
	else
	{
		for (i=WINSIZ-1; i>=winpos; i--)
		{
			smpld[WINSIZ+i].l = 0;
			smpld[WINSIZ+i].r = 0;
		}

		outBytes += DownKernel(dest);
	}

	return(outBytes);

}

int DownKernel(unsigned char * dest)
{
	int i, j, l;
	int bytesOut;
	unsigned long * ptrOut;
	unsigned long * ptrIn;

	ptrIn = (unsigned long *) &smplo;
	ptrOut = (unsigned long *) dest;


	iptr += WINSIZ;
	bytesOut=0;

	while (iptr > eptr)
	{
		suml = 0;
		sumr = 0;
		j = -firodrv-sptrr;

		for (i=sptr; i<=eptr; i++, j+=ISRM)
		{
			l = j > 0 ? j : -j;

			spp = &smpld[i & ((WINSIZ<<1)-1)];
			hgain = hfir[l];
			suml += hgain * spp->l;
			sumr += hgain * spp->r;
		}

		smplo.l = SaturateRound(suml);
		smplo.r = SaturateRound(sumr);

		*ptrOut++ = *ptrIn;
		bytesOut+=4;

		sptr += ismd;
		sptrr += ismr;
		if (sptrr>0)
		{
			sptrr -= ISRM;
			sptr++;
		}

		eptr += ismd;
		eptrr += ismr;
		if (eptrr>=ISRM)
		{
			eptrr -= ISRM;
			eptr++;
		}
	}
	return(bytesOut);

}

static double bessel0(double x)
{
	double value = 1.0, step = 1.0, part = x/2;

	while (part > 1.0E-10)
	{
		value += part*part;
		step += 1.0;
		part = (part*x)/(2.0*step);
	}

	return value;
}

