#include <windows.h>
#include <vfw.h>
#include "flask.h"
#include "VBitmap.h"
#include "deinterlacer.h"
#include "resizer.h"
#include "VideoTelecineRemover.h"
#include "yuvresizer.h"

extern "C" bool FPU_enabled;
extern "C" char MMX_enabled;
extern "C" void DIBconvert_24_to_32(void *dest, long dest_pitch, void *src, long src_pitch, long width, long height);

char MMX_enabled=TRUE;
bool FPU_enabled=TRUE;
VBitmap * vSource = NULL;
VBitmap * vDest = NULL;
bool firstResize=false;
VideoTelecineRemover * ivtc = NULL;
VBitmap * ivtcVsource = NULL;
VBitmap * ivtcVdest = NULL;
unsigned char * yuvDest;

// *****************************************************************************
// flaskDLL Entry()
//
//
// *****************************************************************************
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


// *****************************************************************************
// flaskInit()
//
// 
// *****************************************************************************
NCODEFLS_API void flaskInit()
{
	// Hello

}

// *****************************************************************************
// flaskDeinterlace()
//
//  src = pointer to an array of 4 byte pixels (RGBA)
//  dst = pointer to an array where the output of 4 byte pixels (RGBA) will go
//  width = width is pixels
//  height = height in pixels
// *****************************************************************************
NCODEFLS_API void flaskDeinterlace(FRAMEDATA *src, FRAMEDATA * dst)
{

	switch(src->type)
		{
		case rgb32Frame:
			DeInterlace(src, dst);
			break;
		case yv12Frame:
			DeInterlaceYv12(src, dst);
			break;
		}
	
}

// *****************************************************************************
// flaskStartDeinterlacer()
//
//  width = width is pixels (used to create background buffer)
//  height = height in pixels (used to create background buffer)
//  blend = TRUE / FALSE
//  thresHold = 20 is known good blend value.
// *****************************************************************************
NCODEFLS_API int flaskStartDeinterlacer(FRAMEDATA * frame, int blend, int thresHold)
{
	switch(frame->type)
		{
		case rgb32Frame:
			return(StartDeinterlacer(frame->width, frame->height, blend, thresHold));

		case yv12Frame:
			return(StartDeinterlacerYv12(frame->width, frame->height, blend, thresHold));
		}

	return(0);
}

// *****************************************************************************
// flaskStopDeinterlacer()
//
// *****************************************************************************
NCODEFLS_API int flaskStopDeinterlacer(void)
{
	return(StopDeinterlacer());

}

// *****************************************************************************
// flaskStartIVTC()
//
//  src = pointer to source data (32 bits per pixel)
// debug filter on/off
// offset
// invert polarity 
// *****************************************************************************
NCODEFLS_API void flaskStartIVTC(FRAMEDATA * src, bool fDecomb, int iOffset, bool fInvertedPolarity)
{

	// Wrong, but works for now.
	ivtcVsource = new VBitmap(src->data, src->width, src->height, src->bitDepth);
	ivtcVdest = new VBitmap(src->data, src->width, src->height, src->bitDepth);

	ivtc = CreateVideoTelecineRemover(ivtcVsource,false,false,false);

//	delete [] ivtcVsource;
//	delete [] ivtcVdest;
}

// *****************************************************************************
// flaskIVTCProcessIn()
//
// src data
// current frame number
// *****************************************************************************
NCODEFLS_API void flaskIVTCProcessIn(FRAMEDATA * src, long lFrameNum)
{

	// Wrong, but works for now.
//	ivtcVsource = new VBitmap(src->data, src->width, src->height, src->bitDepth);

	ivtcVsource->data = (unsigned long *) src->data;

	ivtc->ProcessIn(ivtcVsource,lFrameNum);
	
}

// *****************************************************************************
// flaskIVTCProcessOut()
//
//  dst = pointer to dest data frame buffer
// *****************************************************************************
NCODEFLS_API long flaskIVTCProcessOut(FRAMEDATA * dst)
{

	// Wrong, but works for now.
//	ivtcVdest = new VBitmap(src->data, src->width, src->height, src->bitDepth);
	ivtcVdest->data = (unsigned long *) dst->data;

	return(ivtc->ProcessOut(ivtcVdest));
	
}

// *****************************************************************************
// flaskStartResizer()
//
//  outputWidth = output width of resulting bitmap
//  output Height = output height of resulting bitmap
//  filterMode = Filter type to use in scale - 0 = none, 1 = bilinear, 2 = table bilinear, 3 = table bicubic
//  width = width is pixels (used to create background buffer)
//  height = height in pixels (used to create background buffer)
//  src = pointer to source data (32 bits per pixel)
// *****************************************************************************
NCODEFLS_API void flaskStartResizer(int outputWidth, int outputHeight, int filterMode, FRAMEDATA * src)
{
	VBitmap * tempSource;
	
	// Wrong, but works for now.
	tempSource = new VBitmap(src->data, src->width, src->height, src->bitDepth);
	tempSource->pitch = src->stride;
	// hack to clean up for dead filter mode in lower layers
	// If it was a filtermode that people would use I'd fix it, but it's not so drop it as a mode.
	if(filterMode>1)
		filterMode++;
	PrepareResizer(outputWidth&0xfffffffe,outputHeight,FALSE,0,filterMode,tempSource);
	
	// Wrong, but works for now.
	delete [] tempSource;
	firstResize = true;

}

// *****************************************************************************
// flaskResize()
//
//  src = pointer to an array of 4 byte pixels (RGBA)
//  dst = pointer to an array where the output of 4 byte pixels (RGBA) will go
//  width = width is pixels
//  height = height in pixels
// *****************************************************************************
NCODEFLS_API void flaskResize(FRAMEDATA * src, FRAMEDATA * dst)
{

	if(firstResize)
		{
		vSource = new VBitmap(src->data, src->width, src->height, src->bitDepth);
		vSource->pitch = src->stride;

		vDest = new VBitmap(dst->data, dst->width, dst->height,dst->bitDepth);
		vDest->pitch = dst->stride;

		firstResize=false;
		}
	else
		{
		vSource->data = (unsigned long *) src->data;
		vDest->data = (unsigned long *) dst->data;
		}

	ResizerRun(vSource, vDest);
}

// *****************************************************************************
// flaskStopResizer()
//
// *****************************************************************************
NCODEFLS_API void flaskStopResizer(void)
{
	UnPrepareResizer();
	if(vSource)
		delete vSource;
	if(vDest)
		delete vDest;

}

// *****************************************************************************
// flaskYuvResize()
//
//  src = pointer toSource Frame
//  dst = pointer to Dest Frame
// *****************************************************************************
NCODEFLS_API void flaskYuvResize(FRAMEDATA * src, FRAMEDATA * dst,int cropLeft, int cropRight, int cropTop, int cropBottom)
{
	unsigned char * ySrc;
	unsigned char * uSrc;
	unsigned char * vSrc;

	unsigned char * yDst;
	unsigned char * uDst;
	unsigned char * vDst;

	if(src->type == yv12Frame)
		{
		ySrc = src->data;
		vSrc = ySrc + (src->width * src->height);
		uSrc = vSrc + ((src->width/2) * (src->height/2));
		
		yDst = dst->data;
		vDst = yDst + (dst->width * dst->height);
		uDst = vDst + ((dst->width/2) * (dst->height/2));

		// Pointer Crop off the top
		ySrc = ySrc + (src->width * cropTop);
		vSrc = vSrc + ((src->width/2) * (cropTop/2));
		uSrc = uSrc + ((src->width/2) * (cropTop/2));

		// Pointer Crop off the Left
		ySrc = ySrc + cropLeft;
		vSrc = vSrc + (cropLeft/2);
		uSrc = uSrc + (cropLeft/2);
	
		src->stride = src->width;
		src->height -= (cropTop + cropBottom);
		src->width -= (cropRight + cropLeft);

		shrink_frame(yDst,uDst,vDst,ySrc,uSrc,vSrc,src,dst,CHROMA420);
		}

}

// *****************************************************************************
// flaskCrop()
//
// *****************************************************************************
NCODEFLS_API void flaskCrop(FRAMEDATA * source, FRAMEDATA * dest, int cropTopOffset, int cropBottomOffset, 
			   int cropLeftOffset, int cropRightOffset)
{
	int h;
	int w;
	unsigned int * dst;
	unsigned int * src;
	int firstline;
	int diff;

	int cropHeight;
	int cropWidth;

	cropHeight = source->height - (cropTopOffset+cropBottomOffset);
	cropWidth = source->width - (cropLeftOffset+cropRightOffset);
	

	h = cropHeight;

	firstline = source->height - cropTopOffset;
	diff      = cropLeftOffset + (source->width -cropWidth - cropLeftOffset);
	dst       = (unsigned int *) (dest->data + (cropWidth * cropHeight-1)*4);
	src       = (unsigned int *) (source->data + ((cropLeftOffset + (firstline - 1)*source->width + cropWidth - 1)*4));

	do
		{
		w   = cropWidth;
		do
			{
			*dst-- = *src--;
			}while(--w);
		src = src - diff;
		}while(--h);

	dest->height=cropHeight;
	dest->width=cropWidth;
	
}

// *****************************************************************************
// flaskDIBconvert_24_to_32()
// pass through to he assembly routines. Keeps interface at a high level.
// *****************************************************************************
NCODEFLS_API void flaskDIBconvert_24_to_32(void *dest, long dest_pitch, void *src, long src_pitch, long width, long height)
{
	DIBconvert_24_to_32(dest, dest_pitch, src, src_pitch, width, height);
}