// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// netConversion.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;


// *****************************************************************************
// netConversionClass::netConversionClass()
//
// netConversion class constructor
// *****************************************************************************
netConversionClass::netConversionClass()
{

	slaveEncodeMachines = NULL;
	netConversionThreadTerminated = true;
	useBuiltinSlave = true;
}

// *****************************************************************************
// netConversionClass::~netConversionClass()
//
// netConversion class destructor
// *****************************************************************************
netConversionClass::~netConversionClass()
{
	int i;

	abortConversion=true;
	forcedAbortConversion=true;

	for(i=0;i<100;i++)
		{
		if(netConversionThreadTerminated)
			{
			break;
			}
		Sleep(100);
		}

	if(slaveEncodeMachines)
		{
		free(slaveEncodeMachines);
		}

}

// *****************************************************************************
// netConversionClass::startNetConversion()
//
// Returns TRUE if success
// *****************************************************************************
bool netConversionClass::startNetConversion()
{
	abortConversion=false;
	forcedAbortConversion=false;
	pauseConversion=false;
	shutdown=false;
	netConversionThreadTerminated = false;
	netThreadHandle = CreateThread(NULL,0,	netConversionThread ,(LPVOID)this , 0 ,&netThreadId);	
	SetThreadPriority(netThreadHandle,THREAD_PRIORITY_NORMAL);

	return(true);
}

// *****************************************************************************
// conversionClass::netConversionThread()
//
// Returns TRUE if success
// *****************************************************************************
DWORD WINAPI netConversionClass::netConversionThread(LPVOID lParam)
{
	netConversionClass * netConv;
	int ret;

	netConv = (netConversionClass *) lParam;
	
	ret = netConv->netConversion();

	logPrintf("Terminating netConversion Write Thread.\n");	

	netConv->netThreadId=0;
	netConv->netConversionThreadTerminated=true;
	ExitThread(ret);   
	return(ret);
		
}

// *****************************************************************************
// netConversionClass::startNetConversion()
//
// Returns TRUE if success
// *****************************************************************************
bool netConversionClass::netConversion()
{
	ENCODESLAVES * slaveList;
	int i;
	
	slaveEncodeMachines = globals->slaveEncodeMachines;
	numSlaveEncodeMachines =  globals->numSlaveEncodeMachines;

	// Create the slave list (For now just this computer)
	slaveList = new ENCODESLAVES [sizeof(ENCODESLAVESINGLE)*numSlaveEncodeMachines];
	memset(slaveList,0,sizeof(ENCODESLAVES)*numSlaveEncodeMachines);

	for(i=0;i<numSlaveEncodeMachines;i++)
		{
		strcpy(slaveList->slave[i].networkName,slaveEncodeMachines->slave[i].machineIp);
		strcpy(slaveList->slave[i].name,slaveEncodeMachines->slave[i].machineName);
		slaveList->slave[i].port = PORT_NUMBER;
		// These are floats, so init to 0.0f
		slaveList->slave[i].conversionStats.percentOfPassComplete = 0.0f;
		slaveList->slave[i].conversionStats.framesPerSecond = 0.0f;
		}

	if(numSlaveEncodeMachines==1 &&!strcmp(slaveList->slave[0].networkName,"0.0.0.0"))
		{
		logPrintf("Using Builting Slave with direct calls\n");
		useBuiltinSlave = true;
		}
	else
		{
		logPrintf("Using TCP/IP Slave\n");
		useBuiltinSlave = false;
		}

	if(!useBuiltinSlave)
		{
		// Create this machines slave
		netSlave = new netSlaveClass();
		netSlave->begin();
		}

	// Create the master communication module
	netMaster = new netMasterClass();

	// Give the master the list of slaves it owns
	netMaster->begin(numSlaveEncodeMachines, slaveList, useBuiltinSlave);

	// Splitup and create the jobs to be processed. 
	createSlaveJobList();

	// If statistics are enabled, then bring up the window
	if(globals->convertConfig->showStats)
		{
		globals->outputStats->showOutputWindow();
		}

	// Provide the stats window with the list of machines that can be monitored

	globals->outputStats->resetMachineSelection();
	for(i=0;i<numSlaveEncodeMachines;i++)
		{
		globals->outputStats->insertMachineName(slaveList->slave[i].name,i);
		}

	encodeStartTime = timeGetTime();
	
	numFailedJobs = 0;

	// Process all jobs on slave machines
	processAllSlaveJobs();


	if(!forcedAbortConversion)
		{
		//Hide our output window
		globals->outputVideo->hideOutputWindow();
		//Tell the wndow we're done.
		globals->window->doneConverting();
		if(shutdown && !abortConversion)
			{
			if(!useBuiltinSlave)
				{
				for(i=0;i<numSlaveEncodeMachines;i++)
					{
					netMaster->sendShutdown(i);
					}
				}

			performShutdown();
			}
		}


	netMaster->end();
	delete netMaster;
	
	if(!useBuiltinSlave)
		{
		netSlave->end();
		delete netSlave;
		}

	releaseSlaveJobList();

	delete [] slaveList;
	return(true);
}

// *****************************************************************************
// netConversionClass::processAllSlaveJobs()
//
// *****************************************************************************
void netConversionClass::processAllSlaveJobs()
{
	int slave;
	int nextJob;
	bool moreWork;
	int jobNumber;
	int status;
	int jobsBeingProcessed;
	int jobsRemaining;
	// Network Conversion Job Loop

	// Prime the connections. 
	netMaster->checkConnections(4);

	for(;;)
		{

		// Open and verify connections to slaves. 
		netMaster->checkConnections(1);
		
		// Poll active encode computers for their status - Hit each slave a maximum of one time over TCP/IP
		for(slave=0;slave<numSlaveEncodeMachines;slave++)
			{
			netMaster->getStatus(slave);
			}	

		// Tally jobs that have completed
		moreWork = false;
		jobsRemaining = 0;
		jobsBeingProcessed = 0;

		for(jobNumber=0;jobNumber<numSlaveJobs;jobNumber++)
			{
			if((!slaveJobList[jobNumber].job->encodeDone) && (slaveJobList[jobNumber].job->slaveAllocated!=-1))
				{
				status = netMaster->getEncodeStatus(slaveJobList[jobNumber].job->slaveAllocated);
				
				// The conversion finished OK
				switch(status)
					{
					case netConversionStatusCommandFailed:
						logPrintf("Slave Machine %d has reported a status of Failed on Job %d\n",slaveJobList[jobNumber].job->slaveAllocated, jobNumber);
						break;						

					case netConversionStatusDone:
						logPrintf("Slave Machine %d Reported Job %d is Done\n",slaveJobList[jobNumber].job->slaveAllocated, jobNumber);
						slaveJobList[jobNumber].job->encodeDone = true;
						slaveJobList[jobNumber].job->slaveAllocated = -1;
						break;

					case netConversionStatusIdle:
						// Idle is an error condition since we allocated a job to this slave.
						logPrintf("Slave %d reported Idle state when it had a job allocated\n",slaveJobList[jobNumber].job->slaveAllocated);
						slaveJobList[jobNumber].job->slaveAllocated = -1;
						slaveJobList[jobNumber].job->encodeDone = false;
						break;

					case netConversionStatusSlaveDied:
						logPrintf("Slave %d stopped responding during the encode.Reassigning job\n",slaveJobList[jobNumber].job->slaveAllocated);
						slaveJobList[jobNumber].job->slaveAllocated = -1;
						slaveJobList[jobNumber].job->encodeDone = false;

						slaveJobList[jobNumber].job->errorRetryCounter++;
						numFailedJobs++;
						if(slaveJobList[jobNumber].job->errorRetryCounter<MAX_NET_SLAVE_DROP_RETRIES)
							{
							// The job needs to be attempted again
							slaveJobList[jobNumber].job->encodeDone = false;
							slaveJobList[jobNumber].job->slaveAllocated = -1;
							}
						else
							{
							// The job has been attempted twice, so mark it as done and carry on.
							// Lets hope we didn't crash two slaves attempting it twice..
							slaveJobList[jobNumber].job->encodeDone = true;
							slaveJobList[jobNumber].job->slaveAllocated = -1;						
							}
						break;

					case netConversionStatusError:
						status = netMaster->getError(slaveJobList[jobNumber].job->slaveAllocated, slaveJobList[jobNumber].job->errorText);
						logPrintf("Slave %d reported Error : %s\n",slaveJobList[jobNumber].job->slaveAllocated, slaveJobList[jobNumber].job->errorText);
						slaveJobList[jobNumber].job->errorRetryCounter++;
						numFailedJobs++;
						if(slaveJobList[jobNumber].job->errorRetryCounter<MAX_NET_ERROR_RETRIES)
							{
							// The job needs to be attempted again
							slaveJobList[jobNumber].job->encodeDone = false;
							slaveJobList[jobNumber].job->slaveAllocated = -1;
							}
						else
							{
							// The job has been attempted twice, so mark it as done and carry on.
							// Lets hope we didn't crash two slaves attempting it twice..
							slaveJobList[jobNumber].job->encodeDone = true;
							slaveJobList[jobNumber].job->slaveAllocated = -1;						
							}
						break;						
					}
				}

			if(!slaveJobList[jobNumber].job->encodeDone)
				{
				moreWork = true;
				}

			// Gather data for Stats display
			if(!slaveJobList[jobNumber].job->encodeDone)
				{
				if(slaveJobList[jobNumber].job->slaveAllocated==-1)
					{
					jobsRemaining++;
					}
				else
					{
					jobsBeingProcessed++;
					}
				}
			}

		//If we're done, then break out of the for Loop
		if(!moreWork)
			{
			break;
			}

		if(abortConversion)
			{
			break;
			}

		// Find a job to be passed out
		nextJob = -1;

		for(jobNumber=0;jobNumber<numSlaveJobs;jobNumber++)
			{
			if(!slaveJobList[jobNumber].job->encodeDone && slaveJobList[jobNumber].job->slaveAllocated==-1)
				{
				nextJob = jobNumber;
				break;
				}
			}

		if(nextJob>=0)
			{
			// Kickoff New Jobs on Machines that are free
			slave = netMaster->findFreeEncodeResource();
			
			if(slave>=0)
				{
				// Found a resource that's free
				if(netMaster->sendJob(slave,&slaveJobList[nextJob].job->flatJob))
					{
					slaveJobList[nextJob].job->slaveAllocated = slave;
					}
				else
					{
					logPrintf("Slave %d failed to accept sendJob command\n",slave);
					}
				}
			}
		// Update Statistics on encode progress
		outputStatistics(jobsBeingProcessed,jobsRemaining);
		//Delay before polling again
		Sleep(1000);
		}

	// Update the statistics display to show 0 jobs remaining
	if(!forcedAbortConversion)
		{
		outputStatistics(0,0);
		}
}

// *****************************************************************************
// netConversionClass::createSlaveJobList()
//
// *****************************************************************************
void netConversionClass::createSlaveJobList()
{
	int jobNumber;
	int i;
	FLATJOBENTRY * flatJob;
	SLAVEJOBLIST * subSlaveJobList;
	int subJobCount;

	numSlaveJobs = 0;

	logPrintf("Creating Slave Job List from %d Jobs\n",globals->jobCount);
	
	flatJob = new FLATJOBENTRY [1];

	for(jobNumber=0;jobNumber<globals->jobCount;jobNumber++)
		{
		globals->jobHelper->flattenJobEntry(&globals->jobList[jobNumber],flatJob);

		subSlaveJobList = globals->netHelper->createSlaveJobList(flatJob,&subJobCount,useBuiltinSlave);
		
		// Grow or create the slavejoblist

		if(numSlaveJobs == 0)
			{
			slaveJobList =  (SLAVEJOBLIST *) malloc(sizeof(SLAVEJOBLIST)*subJobCount);
			}
		else
			{
			slaveJobList =  (SLAVEJOBLIST *) realloc(slaveJobList, sizeof(SLAVEJOBLIST)*(numSlaveJobs+subJobCount));
			}

		// Now copy the sub jobs into our master list
		for(i=0;i<subJobCount;i++)
			{
			memcpy(&slaveJobList->job[numSlaveJobs],&subSlaveJobList->job[i],sizeof(SLAVEJOBLISTENTRY));
			numSlaveJobs++;
			}
		globals->netHelper->freeSlaveJobList(subSlaveJobList);

		}

	logPrintf("Slave Job List has %d Jobs that need processing\n",numSlaveJobs);

}

// *****************************************************************************
// netConversionClass::releaseSlaveJobList()
//
// *****************************************************************************
void netConversionClass::releaseSlaveJobList()
{

	free(slaveJobList);
}

// *****************************************************************************
// netConversionClass::outputSlaveStatistics()
//
// *****************************************************************************
void netConversionClass::outputStatistics(int jobsBeingProcessed, int jobsRemaining)
{
	CONVERSIONSTATS  * conversionStats;
	int monitorMachine;
	DWORD encodeEndTime;
	double cumMs;
	int elapsedMs;
	int status;
	char text[256];
	char statusText[256];

	monitorMachine = globals->outputStats->getMachineSelected();

	conversionStats = netMaster->getEncodeStats(monitorMachine);
	
	status = netMaster->getConnectionStatus(monitorMachine);

	switch(status)
		{
		case NET_NOT_CONNECTED:
			strcpy(statusText,"NETNotConnected");
			break;

		case NET_CONNECTING:
			strcpy(statusText,"NETConnecting");
			break;

		case NET_CONNECTED:
			strcpy(statusText,"NETConnected");
			break;

		case NET_FAILED:
		case NET_SHORTDELAYRETRY:
			strcpy(statusText,"NETShortDelay");
			break;

		case NET_DOWN:
		case NET_LONGDELAYRETRY:
			strcpy(statusText,"NETLongDelay");
			break;

		case NET_PERMANENTLYDOWN:
			strcpy(statusText,"NETPermDown");
			break;
		}
	globals->translate->translate(text, statusText);	
	globals->outputStats->setConnectionStatusText(text);
		
	globals->outputStats->setTotalEncodeJobs(numSlaveJobs);
	globals->outputStats->setFPS(conversionStats->framesPerSecond);
	globals->outputStats->setPass(conversionStats->passNumber, conversionStats->totalPasses);
	globals->outputStats->setPercentComplete(conversionStats->percentOfPassComplete);
	globals->outputStats->setRemainingTime(conversionStats->estimatedRemainingPassTime);
	globals->outputStats->setElapsedPassTime(conversionStats->elapsedPassTime);
	globals->outputStats->setElapsedJobTime(conversionStats->elapsedJobTime);

	globals->outputStats->setJobsBeingProcessed(jobsBeingProcessed);
	globals->outputStats->setJobsRemaining(jobsRemaining);

	globals->outputStats->setJobsCompleted(numSlaveJobs - (jobsRemaining + jobsBeingProcessed));

	globals->outputStats->setRange(conversionStats->rangeNumber, conversionStats->totalRanges);

	encodeEndTime = timeGetTime();

	elapsedMs = encodeEndTime - encodeStartTime;
	cumMs = (double) elapsedMs;
	cumMs /= 1000;

	globals->outputStats->setElapsedTime((int)cumMs);

	globals->outputStats->setJobNameText(conversionStats->jobName);

}

// *****************************************************************************
// netConversionClass::abort()
//
// *****************************************************************************
void netConversionClass::abort(int slave)
{
	int jobNumber;

	abortConversion=true;
	shutdown = false;

	if(slave == -1)
		{
		for(jobNumber=0;jobNumber<numSlaveJobs;jobNumber++)
			{
			if((!slaveJobList[jobNumber].job->encodeDone) && (slaveJobList[jobNumber].job->slaveAllocated!=-1))
				{
				netMaster->sendAbort(slaveJobList[jobNumber].job->slaveAllocated);
				}		
			}
		}
	else
		{
		// Abort the local slave
		globals->conversion->abortConversion=true;
		}
}
// *****************************************************************************
// netConversionClass::abort()
//
// *****************************************************************************
void netConversionClass::setForcedAbortConversion()
{
	abortConversion=true;
	forcedAbortConversion=true;

}
// *****************************************************************************
// netConversionClass::pause()
//
// *****************************************************************************
bool netConversionClass::pause()
{
	int jobNumber;

	pauseConversion = !pauseConversion;

	// Broadcast to all encode Computers
	for(jobNumber=0;jobNumber<numSlaveJobs;jobNumber++)
		{
		if((!slaveJobList[jobNumber].job->encodeDone) && (slaveJobList[jobNumber].job->slaveAllocated!=-1))
			{
			if(pauseConversion)
				{
				netMaster->sendPause(slaveJobList[jobNumber].job->slaveAllocated);
				}
			else
				{
				netMaster->sendResume(slaveJobList[jobNumber].job->slaveAllocated);
				}
			}		
		}

	return(pauseConversion);
}

// *****************************************************************************
// netConversionClass::setThreadPriority()
//
// *****************************************************************************
void netConversionClass::setThreadPriority(int priority)
{
	int jobNumber;

	SetThreadPriority(netThreadHandle,priority);

	for(jobNumber=0;jobNumber<numSlaveJobs;jobNumber++)
		{
		if((!slaveJobList[jobNumber].job->encodeDone) && (slaveJobList[jobNumber].job->slaveAllocated!=-1))
			{
			netMaster->sendThreadPriority(slaveJobList[jobNumber].job->slaveAllocated,priority);
			}		
		}

}

// *****************************************************************************
// netConversionClass::setShutdown()
//
// Returns TRUE if success
// *****************************************************************************
void netConversionClass::setShutdown(bool shutdownState)
{

	shutdown = shutdownState;
		
}

// *****************************************************************************
// netConversionClass::setSleepPerFrame()
//
// *****************************************************************************
void netConversionClass::setSleepPerFrame(int sleep)
{
	int jobNumber;

	// Broadcast to all encode Computers

	for(jobNumber=0;jobNumber<numSlaveJobs;jobNumber++)
		{
		if((!slaveJobList[jobNumber].job->encodeDone) && (slaveJobList[jobNumber].job->slaveAllocated!=-1))
			{
			netMaster->sendSleep(slaveJobList[jobNumber].job->slaveAllocated,sleep);
			}		
		}

}

// *****************************************************************************
// netConversionClass::performShutdown()
//
// *****************************************************************************
void netConversionClass::performShutdown()
{
	HANDLE hToken;
	TOKEN_PRIVILEGES tp;
	LUID luid;
	char messTitle[4096];
	char messText[4096];

	if( OpenProcessToken( GetCurrentProcess(), TOKEN_ALL_ACCESS, &hToken ) ) 
		{
		if( LookupPrivilegeValue( NULL, SE_SHUTDOWN_NAME, &luid ) ) 
			{
			tp.PrivilegeCount = 1;
			tp.Privileges[0].Luid = luid;
			tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
			AdjustTokenPrivileges(hToken, FALSE, &tp, sizeof(TOKEN_PRIVILEGES), (PTOKEN_PRIVILEGES) NULL, (PDWORD) NULL);
			}
		}
	if( ExitWindowsEx(EWX_POWEROFF|EWX_SHUTDOWN|EWX_FORCE, 0xFFFFFFFF)==FALSE ) 
		{
		if( ExitWindowsEx(EWX_SHUTDOWN|EWX_FORCE, 0xFFFFFFFF)==FALSE )
			{
			globals->translate->translate(messText, "VidomiMessage1");
			globals->translate->translate(messTitle, "VidomiMessage1Title");
			vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK);
			}
		}
}

