// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// netSlave.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

// *****************************************************************************
// netSlaveClass::netSlaveClass()
//
// netSlave class constructor
// *****************************************************************************
netSlaveClass::netSlaveClass()
{


}

// *****************************************************************************
// netSlaveClass::~netSlaveClass()
//
// netSlave class destructor
// *****************************************************************************
netSlaveClass::~netSlaveClass()
{


}

// *****************************************************************************
// netSlaveClass::begin()
//
// *****************************************************************************
void netSlaveClass::begin()
{
	// create the slave listen thread
	InitializeCriticalSection(&conversionStatusLock);
	conversionStatus = netConversionStatusIdle;
	
	terminateSlave = false;
	slaveThreadTerminated = false;
	netSlaveThreadHandle = CreateThread(NULL,0,	netSlaveThread ,(LPVOID)this , 0 ,&netSlaveThreadId);	
	//SetThreadPriority(netSlaveThreadHandle,THREAD_PRIORITY_NORMAL);
	SetThreadPriority(netSlaveThreadHandle,THREAD_PRIORITY_IDLE);
	Sleep(1000);
}

// *****************************************************************************
// netSlaveClass::end()
//
// *****************************************************************************
void netSlaveClass::end()
{
	int i;
	// Terminate the slave encode process
	terminateSlave = true;
	for(i=0;i<100;i++)
		{
		if(slaveThreadTerminated)
			break;
		Sleep(100);
		}
}


// *****************************************************************************
// netSlaveClass::netSlaveThread()
//
// *****************************************************************************
DWORD WINAPI netSlaveClass::netSlaveThread(LPVOID lParam)
{
	netSlaveClass * netSlave;
	int ret;

	netSlave = (netSlaveClass *) lParam;
	
	ret = netSlave->netSlave();

	logPrintf("Terminating netSlave Thread.\n");	

	netSlave->netSlaveThreadId=0;
	netSlave->slaveThreadTerminated = true;
	ExitThread(ret);   
	return(ret);
		
}

// *****************************************************************************
// netSlaveClass::startNetConversion()
//
// Returns TRUE if success
// *****************************************************************************
bool netSlaveClass::netSlave()
{
	char commandHeader[NET_COMMAND_HEADER_SIZE];
	DWORD commandToken;
	int commandSize;
	int bytesAvail;
	int err;

	tcp = new tcpClass();

	slaveSocket = tcp->createSocket();
	if(slaveSocket<0)
		{
		delete tcp;
		return(false);
		}

	if(!tcp->bindSocket(slaveSocket,PORT_NUMBER,0))
		{
		tcp->closeSocket(slaveSocket);
		delete tcp;
		return(false);
		}

	if(!tcp->setSocketListen(slaveSocket))
		{
		tcp->closeSocket(slaveSocket);
		delete tcp;
		return(false);
		}

	for(;;)
		{
		rSocket = tcp->acceptConnection(slaveSocket);

		// Create a thread to do the work..
		
		if(rSocket!=-1)
			{
			logPrintf("Slave Received Connection\n");
			// We're connected, so we can begin communicating
			for(;;)
				{
				bytesAvail = tcp->peekBytes(rSocket, &commandHeader[0], NET_COMMAND_HEADER_SIZE);
				err = WSAGetLastError();
				
				// If we ever get anything other than an available number of bytes, then drop the connection
				if((bytesAvail==0) || (bytesAvail<0 && err!=WSAEWOULDBLOCK))
					{
					break;
					}

				if(bytesAvail>=NET_COMMAND_HEADER_SIZE)
					{
					//logPrintf("Have enough bytes for a Slave Command\n");
					// If we have enough bytes for the command header, then fetch the header
					tcp->receiveBytes(rSocket,(char *) &commandToken,4);
					tcp->receiveBytes(rSocket,(char *) &commandSize,4);
					// Parse command and create command threads if necessary
					if(!parseCommand(commandToken, commandSize))
						{
						// If we get a false base, the command fialed to be received for some reason
						break;
						}
					}

				Sleep(50);

				if(terminateSlave)
					{
					break;
					}
				}
			logPrintf("Connection Dropped\n");
			tcp->closeConnection(rSocket);					
			}
		//
		if(terminateSlave)
			break;
		Sleep(100);
		}

	// Terminate and Cleanup whatever encode is in progress
	
	tcp->closeSocket(slaveSocket);

	delete tcp;

	return(true);
}

// *****************************************************************************
// netSlaveClass::parseCommand()
//
// *****************************************************************************
bool netSlaveClass::parseCommand(DWORD command, int size)
{
	switch(command)
		{
		case netStartConversion :
			logPrintf("Executing Start Conversion\n");
			return(startConversion(command, size));
		case netStatus :
			//logPrintf("Executing Get Status\n");
			return(status(command, size));
		case netClose:
			logPrintf("Executing Close\n");
			return(false);
		case netError :
			logPrintf("Executing Get Error\n");
			return(error(command, size));
		case netAbort :
			logPrintf("Executing Abort\n");
			return(abort(command, size));
		case netPause :
			logPrintf("Executing Pause\n");
			return(pause(command, size));
		case netResume :
			logPrintf("Executing Resume\n");
			return(resume(command, size));
		case netSleep :
			logPrintf("Executing Sleep\n");
			return(sleep(command, size));
		case netShutdown :
			logPrintf("Executing Sleep\n");
			return(shutdown(command, size));
		case netThreadPriority :
			logPrintf("Executing ThreadPriority\n");
			return(threadPriority(command, size));
		}
	return(false);
}

// *****************************************************************************
// netSlaveClass::startConversion()
//
// *****************************************************************************
bool netSlaveClass::startConversion(DWORD command, int size)
{
	NETSTARTCONVERSION * inputData;
	NETSTARTCONVERSIONRETURN outputData;

	int status;

	NETSLAVECONVERSIONTHREAD * slaveConversionData;

	EnterCriticalSection(&conversionStatusLock);
	status = conversionStatus;
	if(status != netConversionStatusWorking)
		{
		conversionStatus = netConversionStatusWorking;
		}
	LeaveCriticalSection(&conversionStatusLock);

	inputData = (NETSTARTCONVERSION *) new char [size+NET_COMMAND_HEADER_SIZE];
	
	inputData->command = command;
	inputData->size = size;

	if(!safeReceiveBytes(((char *) inputData)+NET_COMMAND_HEADER_SIZE,size))
		{
		// Comm error or we're terminating
		// Set the conversion status back to what it was when we started this.
		EnterCriticalSection(&conversionStatusLock);
		conversionStatus = status;
		LeaveCriticalSection(&conversionStatusLock);
		delete [] inputData;
		return(false);
		}

	outputData.status = netConversionStatusWorking;

	// If we were already doing an encode, then return command failed
	if(status == netConversionStatusWorking)
		{
		logPrintf("Someone attempted to submit a job to us when we already had a job. Return failure to the source\n");
		outputData.status = netConversionStatusCommandFailed;
		}

	// If the packet sizes don't match, then we have a version mismatch
	if(size !=(sizeof(NETSTARTCONVERSION) - NET_COMMAND_HEADER_SIZE))
		{
		// The command has an incorrect size
		// Comm error or the versions are out of sync
		logPrintf("MAJOR PROBLEM : The slave and the master don't agree on the number of bytes for the conversion data. The master and slave versions are out of sync more than likely. Aborting job start\n");
		// return version mismatch
		outputData.status = netConversionStatusVersionMismatch;
		}

	// return the status
	if(!safeSendBytes((char *) &outputData,sizeof(NETSTARTCONVERSIONRETURN)))
		{
		logPrintf("Start Conversion Send Status failed in safeSendBytes\n");
		// Comm error
		EnterCriticalSection(&conversionStatusLock);
		conversionStatus = status;
		LeaveCriticalSection(&conversionStatusLock);
		delete [] inputData;
		return(false);
		}

	if(outputData.status != netConversionStatusWorking)
		{
		// Set the conversion status back to what it was when we started this.
		EnterCriticalSection(&conversionStatusLock);
		conversionStatus = status;
		LeaveCriticalSection(&conversionStatusLock);
		delete [] inputData;
		return(false);
		}


	// We have the command Data now. Perform the command
	slaveConversionData = new NETSLAVECONVERSIONTHREAD [1];
	slaveConversionData->slave = this;
	slaveConversionData->inputData = inputData;

	// Create the work thread
	conversionThreadHandle = CreateThread(NULL,0,	conversionThread ,(LPVOID)slaveConversionData , 0 ,&conversionThreadId);	
	//SetThreadPriority(conversionThreadHandle,THREAD_PRIORITY_NORMAL);
	SetThreadPriority(conversionThreadHandle,THREAD_PRIORITY_IDLE);

	return(true);
}

// *****************************************************************************
// netSlaveClass::conversionThread()
//
// *****************************************************************************
DWORD WINAPI netSlaveClass::conversionThread(LPVOID lParam)
{
	NETSLAVECONVERSIONTHREAD * 	slaveConversionData;

	bool ret;
	JOBLISTENTRY * jobListEntry;

	slaveConversionData = (NETSLAVECONVERSIONTHREAD *) lParam;
	
	jobListEntry = new JOBLISTENTRY [1];
	globals->jobHelper->createJobEntry(jobListEntry);
	globals->jobHelper->unflattenJobEntry(jobListEntry,&slaveConversionData->inputData->flatJob);

	// Clear the errorText buffer
	strcpy(globals->errorText,"");
	ret = slaveConversionData->slave->conversion(jobListEntry);

	globals->jobHelper->deleteJobEntry(jobListEntry);
	delete [] jobListEntry;
	delete [] slaveConversionData->inputData;
	
	// Give the Slave some time to settle
	Sleep(1000);

	EnterCriticalSection(&slaveConversionData->slave->conversionStatusLock);
	if(ret)
		{
		// The encode completed normally
		slaveConversionData->slave->conversionStatus = netConversionStatusDone;
		}
	else
		{
		// An error occured during the encode
		slaveConversionData->slave->conversionStatus = netConversionStatusError;
		}
	LeaveCriticalSection(&slaveConversionData->slave->conversionStatusLock);

	delete [] slaveConversionData;

	ExitThread(0);   
	return(0);
		
}

// *****************************************************************************
// netSlaveClass::conversion()
//
// Returns TRUE if success
// *****************************************************************************
bool netSlaveClass::conversion(JOBLISTENTRY * jobListEntry)
{
	return(globals->conversion->startSingleConversion(jobListEntry));

}

// *****************************************************************************
// netSlaveClass::status()
//
// *****************************************************************************
bool netSlaveClass::status(DWORD command, int size)
{
	NETSTATUSRETURN outputData;
	if(size !=(sizeof(NETSTATUS) - NET_COMMAND_HEADER_SIZE))
		{
		// The command has an incorrect size
		// Comm error or the versions are out of sync
		return(false);
		}

	// get the current encoder status
	EnterCriticalSection(&conversionStatusLock);
	outputData.status = conversionStatus;
	LeaveCriticalSection(&conversionStatusLock);

	// Call conversion to get the current stats snapshot
	globals->conversion->getConversionStats(&outputData.conversionStats);

	// Return the current conversion status
	if(!safeSendBytes((char *) &outputData,sizeof(NETSTATUSRETURN)))
		{
		logPrintf("Get Status failed in safeSendBytes\n");
		// Comm error or we're terminating
		return(false);
		}

	return(true);
}

// *****************************************************************************
// netSlaveClass::abort()
//
// *****************************************************************************
bool netSlaveClass::abort(DWORD command, int size)
{
	if(size !=(sizeof(NETABORT) - NET_COMMAND_HEADER_SIZE))
		{
		// The command has an incorrect size
		// Comm error or the versions are out of sync
		return(false);
		}

	globals->conversion->abortConversion = true;
	
	return(true);
}

// *****************************************************************************
// netSlaveClass::pause()
//
// *****************************************************************************
bool netSlaveClass::pause(DWORD command, int size)
{
	if(size !=(sizeof(NETABORT) - NET_COMMAND_HEADER_SIZE))
		{
		// The command has an incorrect size
		// Comm error or the versions are out of sync
		return(false);
		}

	globals->conversion->pauseConversion = true;
	
	return(true);
}

// *****************************************************************************
// netSlaveClass::resume()
//
// *****************************************************************************
bool netSlaveClass::resume(DWORD command, int size)
{
	if(size !=(sizeof(NETABORT) - NET_COMMAND_HEADER_SIZE))
		{
		// The command has an incorrect size
		// Comm error or the versions are out of sync
		return(false);
		}

	globals->conversion->pauseConversion = false;
	
	return(true);
}

// *****************************************************************************
// netSlaveClass::sleep()
//
// *****************************************************************************
bool netSlaveClass::sleep(DWORD command, int size)
{
	NETSLEEP inputData;
	if(size !=(sizeof(NETSLEEP) - NET_COMMAND_HEADER_SIZE))
		{
		// The command has an incorrect size
		// Comm error or the versions are out of sync
		return(false);
		}

	if(!safeReceiveBytes(((char *) &inputData)+NET_COMMAND_HEADER_SIZE,size))
		{
		return(false);
		}

	globals->conversion->sleepPerFrame = inputData.sleepValue;
	
	return(true);
}

// *****************************************************************************
// netSlaveClass::sleep()
//
// *****************************************************************************
bool netSlaveClass::shutdown(DWORD command, int size)
{
	NETSHUTDOWN inputData;

	if(size !=(sizeof(NETSHUTDOWN) - NET_COMMAND_HEADER_SIZE))
		{
		// The command has an incorrect size
		// Comm error or the versions are out of sync
		return(false);
		}

	if(!safeReceiveBytes(((char *) &inputData)+NET_COMMAND_HEADER_SIZE,size))
		{
		return(false);
		}

#ifdef VIDOMISLAVE
	if(globals->obeyShutdown)
		{
		performShutdown();
		}
#endif

	return(true);
}
// *****************************************************************************
// netSlaveClass::threadPriority()
//
// *****************************************************************************
bool netSlaveClass::threadPriority(DWORD command, int size)
{
	NETTHREADPRIORITY inputData;
	if(size !=(sizeof(NETTHREADPRIORITY) - NET_COMMAND_HEADER_SIZE))
		{
		// The command has an incorrect size
		// Comm error or the versions are out of sync
		return(false);
		}

	if(!safeReceiveBytes(((char *) &inputData)+NET_COMMAND_HEADER_SIZE,size))
		{
		return(false);
		}
	SetThreadPriority(netSlaveThreadHandle,inputData.threadPriority);
	SetThreadPriority(conversionThreadHandle,inputData.threadPriority);
	SetThreadPriority(globals->conversion->writeThreadHandle,inputData.threadPriority);
	
	return(true);
}
// *****************************************************************************
// netSlaveClass::error()
//
// *****************************************************************************
bool netSlaveClass::error(DWORD command, int size)
{
	NETERRORRETURN outputData;
	if(size !=(sizeof(NETERROR) - NET_COMMAND_HEADER_SIZE))
		{
		// The command has an incorrect size
		// Comm error or the versions are out of sync
		return(false);
		}

	strcpy(outputData.errorText,globals->errorText);

	// Return the current conversion status
	if(!safeSendBytes((char *) &outputData,sizeof(NETERRORRETURN)))
		{
		// Comm error or we're terminating
		return(false);
		}

	return(true);
}

// *****************************************************************************
// netSlaveClass::safeReceiveBytes()
//
// *****************************************************************************
bool netSlaveClass::safeReceiveBytes(char * ptr, int bytesRemaining)
{

	if(!tcp->safeReceiveBytes(rSocket,ptr,bytesRemaining))
		{
		logPrintf("Slave timed out in safeReceiveBytes\n");
		return(false);
		}

	return(true);
}

// *****************************************************************************
// netSlaveClass::safeSendBytes()
//
// *****************************************************************************
bool netSlaveClass::safeSendBytes(char * ptr, int bytesRemaining)
{

	if(!tcp->safeSendBytes(rSocket,ptr,bytesRemaining))
		{
		logPrintf("Slave timed out in safeSendBytes\n");
		return(false);
		}
	
	return(true);


}

// *****************************************************************************
// netSlaveClass::performShutdown()
//
// *****************************************************************************
void netSlaveClass::performShutdown()
{
	HANDLE hToken;
	TOKEN_PRIVILEGES tp;
	LUID luid;

	if( OpenProcessToken( GetCurrentProcess(), TOKEN_ALL_ACCESS, &hToken ) ) 
		{
		if( LookupPrivilegeValue( NULL, SE_SHUTDOWN_NAME, &luid ) ) 
			{
			tp.PrivilegeCount = 1;
			tp.Privileges[0].Luid = luid;
			tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
			AdjustTokenPrivileges(hToken, FALSE, &tp, sizeof(TOKEN_PRIVILEGES), (PTOKEN_PRIVILEGES) NULL, (PDWORD) NULL);
			}
		}
	if( ExitWindowsEx(EWX_POWEROFF|EWX_SHUTDOWN|EWX_FORCE, 0xFFFFFFFF)==FALSE ) 
		{
		ExitWindowsEx(EWX_SHUTDOWN|EWX_FORCE, 0xFFFFFFFF);
		}
}

