// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// optionschapters.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

static TRANSLATEIDTABLE chapterOptionsTID[] = {
										{"VidomiOptionsCancel",VidomiOptionsCancel},
										{"VidomiOptionsOk",VidomiOptionsOk},
										
										{"VidomiOptionsChapterText1",VidomiOptionsChapterText1},
										{"VidomiOptionsChapterText2",VidomiOptionsChapterText2},
										{"VidomiOptionsChapterText3",VidomiOptionsChapterText3},
										{"VidomiOptionsChapterText4",VidomiOptionsChapterText4},
										{"VidomiOptionsChapterText5",VidomiOptionsChapterText5},
										{"",0}

										};

// *****************************************************************************
// optionsChaptersClass::optionsChaptersClass()
//
// conversion class constructor
// *****************************************************************************
optionsChaptersClass::optionsChaptersClass()
{
	convertConfig = new CONVERTCONFIG [1];

}

// *****************************************************************************
// optionsChaptersClass::~optionsChaptersClass()
//
// conversion class destructor
// *****************************************************************************
optionsChaptersClass::~optionsChaptersClass()
{
	delete [] convertConfig;

}

// *****************************************************************************
// optionsChaptersClass::bringUpOptions()
//
// Bring up the options screen
// *****************************************************************************
void optionsChaptersClass::bringUpOptions(HINSTANCE hInst, CONVERTCONFIG * convertConfigIn, GATHERRESULTS * gatherResults)
{
	int i;
	char text[4096];

	optionshWnd = CreateDialog(hInst,MAKEINTRESOURCE(VidomiChapterOptions),globals->hWnd, optionsDialogHandler);
	

	globals->translate->translate(text, "VidomiChapterOptions");
	SetWindowText(optionshWnd, text);			

	for(i=0;;i++)
		{
		if(!chapterOptionsTID[i].id)
			break;
		globals->translate->translate(text, chapterOptionsTID[i].textId);
		SetWindowText(GetDlgItem(optionshWnd, chapterOptionsTID[i].id),text);			
		}


	scanData = gatherResults;

	convertConfigOriginal = convertConfigIn;
	memcpy(convertConfig,convertConfigOriginal,sizeof(CONVERTCONFIG));
	
	currentChapterNumber = 0;

	configureOptionsWindow(optionshWnd);

}

// *****************************************************************************
// encoderVideoClass::optionsDialogHandler()
//
//
// *****************************************************************************
BOOL CALLBACK optionsChaptersClass::optionsDialogHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->optionsChapters->optionsDialogHandler1(hWnd,message,wParam,lParam));
}

// *****************************************************************************
// optionsChaptersClass::windowDialogHandler()
//
//
// *****************************************************************************
BOOL optionsChaptersClass::optionsDialogHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	unsigned short commandId;
	unsigned short commandEvent;
	int ret;
	int numChapters;
	int i;
	char startFrameText[256];

	switch(message)
		{

		case WM_COMMAND:
			commandId   = LOWORD(wParam); // Remember, these are...
			commandEvent = HIWORD(wParam); // ...different for Win32!

			//Parse the menu selections:

			switch (commandId) 
				{
				case VidomiOptionsOk :
					readWindowVars(hWnd);

					numChapters = 0;
					for(i=0;i<MAX_CHAPTERS;i++)
						{
						if((convertConfig->chapterStartFramePos[i]<=0) && (i!=0))
							break;

						strcpy(convertConfigOriginal->chapterName[i], convertConfig->chapterName[i]);
						convertConfigOriginal->chapterStartFramePos[i] = convertConfig->chapterStartFramePos[i];
						numChapters++;
						}
					convertConfigOriginal->numChapters = numChapters;
					convertConfigOriginal->insertChapters = convertConfig->insertChapters;

					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;

				case VidomiOptionsInsertChapters :
					if(SendDlgItemMessage(hWnd,VidomiOptionsInsertChapters,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
						convertConfig->insertChapters = true;
					else
						convertConfig->insertChapters = false;

					enableWindows(hWnd);
					break;


				case VidomiOptionsChapterNumber:
					if(commandEvent==CBN_SELCHANGE)
						{
						GetWindowText(GetDlgItem(hWnd, VidomiOptionsChapterName),convertConfig->chapterName[currentChapterNumber],64);
						convertConfig->chapterName[currentChapterNumber][64]=0;

						GetWindowText(GetDlgItem(hWnd, VidomiOptionsChapterStartFrame), startFrameText , 256);
 						convertConfig->chapterStartFramePos[currentChapterNumber] = atoi(startFrameText);

						//Update the channel number
						ret = SendDlgItemMessage(hWnd,VidomiOptionsChapterNumber, CB_GETCURSEL  , 0, 0);
						currentChapterNumber = SendDlgItemMessage(hWnd,VidomiOptionsChapterNumber, CB_GETITEMDATA , ret, 0);
						
						buildChapterOptions(hWnd);

						}
					break;


				case VidomiOptionsCancel :
					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;
				}
			break;
			

		case WM_CLOSE:
			DestroyWindow(hWnd);
			globals->optionsUp=false;
			break;

		}

	return (FALSE);

}



// *****************************************************************************
// optionsChaptersClass::configureOptionsWindow()
//
//
// *****************************************************************************
void optionsChaptersClass::configureOptionsWindow(HWND hWnd)
{
	char tempStr[256];
	int ret;
	int i;

	SendDlgItemMessage(hWnd,VidomiOptionsChapterNumber, CB_RESETCONTENT , 0, 0);

	for(i=0;i<MAX_CHAPTERS;i++)
		{
		sprintf(tempStr,"%d",i+1);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsChapterNumber, CB_ADDSTRING , (WPARAM)0 , (LPARAM) tempStr); 									
		SendDlgItemMessage(hWnd,VidomiOptionsChapterNumber, CB_SETITEMDATA  , ret, i);
		if(i==currentChapterNumber)
			SendDlgItemMessage(hWnd,VidomiOptionsChapterNumber, CB_SETCURSEL  , ret, 0);
		}

	if(convertConfig->insertChapters)
		SendDlgItemMessage(hWnd,VidomiOptionsInsertChapters,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsInsertChapters,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	buildChapterOptions(hWnd);
	enableWindows(hWnd);

}

// *****************************************************************************
// optionsChaptersClass::readWindowVars()
//
//
// *****************************************************************************
void optionsChaptersClass::readWindowVars(HWND hWnd)
{
	int ret;
	char startFrameText[256];

	ret = SendDlgItemMessage(hWnd,VidomiOptionsChapterNumber, CB_GETCURSEL  , 0, 0);
	currentChapterNumber = SendDlgItemMessage(hWnd,VidomiOptionsChapterNumber, CB_GETITEMDATA , ret, 0);

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsChapterName),convertConfig->chapterName[currentChapterNumber],64);
	convertConfig->chapterName[currentChapterNumber][64]=0;

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsChapterStartFrame), startFrameText , 256);
 	convertConfig->chapterStartFramePos[currentChapterNumber] = atoi(startFrameText);

	if(SendDlgItemMessage(hWnd,VidomiOptionsInsertChapters,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->insertChapters=true;
	else
		convertConfig->insertChapters=false;

}

// *****************************************************************************
// optionsChaptersClass::buildChapterOptions()
//
//
// *****************************************************************************
void optionsChaptersClass::buildChapterOptions(HWND hWnd)
{
	char startFrameText[256];

	SetWindowText(GetDlgItem(hWnd, VidomiOptionsChapterName),convertConfig->chapterName[currentChapterNumber]);
	
	sprintf(startFrameText,"%d",convertConfig->chapterStartFramePos[currentChapterNumber]);

	SetWindowText(GetDlgItem(hWnd, VidomiOptionsChapterStartFrame),startFrameText);

}


// *****************************************************************************
// optionsChaptersClass::enableWindows()
//
//
// *****************************************************************************
void optionsChaptersClass::enableWindows(HWND hWnd)
{
	int state;

	state = convertConfig->insertChapters;

	EnableWindow(GetDlgItem(hWnd,VidomiOptionsChapterNumber), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsChapterText3), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsChapterName), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsChapterText4), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsChapterStartFrame), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsChapterText5), state);
	
}