// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// optionschapters.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

static TRANSLATEIDTABLE chapterOptionsTID[] = {
										{"VidomiOptionsCancel",VidomiOptionsCancel},
										{"VidomiOptionsOk",VidomiOptionsOk},
										{"VidomiNetworkOptionsText1",VidomiNetworkOptionsText1},
										{"VidomiNetworkOptionsText2",VidomiNetworkOptionsText2},
										{"VidomiNetworkOptionsHyperEncodeText1",VidomiNetworkOptionsHyperEncodeText1},
										{"VidomiNetworkOptionsHyperEncodeText2",VidomiNetworkOptionsHyperEncodeText2},
										{"VidomiNetworkOptionsEncodeServerText1",VidomiNetworkOptionsEncodeServerText1},
										{"VidomiNetworkOptionsEncodeServerText2",VidomiNetworkOptionsEncodeServerText2},
									
										{"",0}

										};

// *****************************************************************************
// optionsNetworkClass::optionsNetworkClass()
//
// conversion class constructor
// *****************************************************************************
optionsNetworkClass::optionsNetworkClass()
{
	convertConfig = new CONVERTCONFIG [1];

}

// *****************************************************************************
// optionsNetworkClass::~optionsNetworkClass()
//
// conversion class destructor
// *****************************************************************************
optionsNetworkClass::~optionsNetworkClass()
{
	delete [] convertConfig;

}

// *****************************************************************************
// optionsNetworkClass::bringUpOptions()
//
// Bring up the options screen
// *****************************************************************************
void optionsNetworkClass::bringUpOptions(HINSTANCE hInst, CONVERTCONFIG * convertConfigIn, GATHERRESULTS * gatherResults)
{
	int i;
	char text[4096];

	optionshWnd = CreateDialog(hInst,MAKEINTRESOURCE(VidomiNetworkOptions),globals->hWnd, optionsDialogHandler);
	
	globals->translate->translate(text, "VidomiNetworkOptions");
	SetWindowText(optionshWnd, text);			

	for(i=0;;i++)
		{
		if(!chapterOptionsTID[i].id)
			break;
		globals->translate->translate(text, chapterOptionsTID[i].textId);
		SetWindowText(GetDlgItem(optionshWnd, chapterOptionsTID[i].id),text);			
		}


	scanData = gatherResults;

	convertConfigOriginal = convertConfigIn;
	memcpy(convertConfig,convertConfigOriginal,sizeof(CONVERTCONFIG));
	encodeServerEnabled = globals->encodeServerEnabled;
	strcpy(encodeServerIPAddress,globals->encodeServerIPAddress);

	configureOptionsWindow(optionshWnd);

}

// *****************************************************************************
// encoderVideoClass::optionsDialogHandler()
//
//
// *****************************************************************************
BOOL CALLBACK optionsNetworkClass::optionsDialogHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->optionsNetwork->optionsDialogHandler1(hWnd,message,wParam,lParam));
}

// *****************************************************************************
// optionsNetworkClass::windowDialogHandler()
//
//
// *****************************************************************************
BOOL optionsNetworkClass::optionsDialogHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	unsigned short commandId;
	unsigned short commandEvent;

	switch(message)
		{

		case WM_COMMAND:
			commandId   = LOWORD(wParam); // Remember, these are...
			commandEvent = HIWORD(wParam); // ...different for Win32!

			//Parse the menu selections:

			switch (commandId) 
				{
				case VidomiOptionsOk :
					readWindowVars(hWnd);

					convertConfigOriginal->hyperEncode = convertConfig->hyperEncode;
					globals->encodeServerEnabled = encodeServerEnabled;
					if(encodeServerEnabled)
						{
						strcpy(globals->encodeServerIPAddress,encodeServerIPAddress);
						}
					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;

				case VidomiOptionsCancel :
					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;

				case VidomiNetworkOptionsEncodeServer:
					readWindowVars(hWnd);
					configureOptionsWindow(hWnd);
					break;

				}
			break;
			

		case WM_CLOSE:
			DestroyWindow(hWnd);
			globals->optionsUp=false;
			break;

		}

	return (FALSE);

}



// *****************************************************************************
// optionsNetworkClass::configureOptionsWindow()
//
//
// *****************************************************************************
void optionsNetworkClass::configureOptionsWindow(HWND hWnd)
{
	char machineText[1024];
	int i;
	int ret;
	bool state;

	if(convertConfig->hyperEncode)
		SendDlgItemMessage(hWnd,VidomiNetworkOptionsHyperEncode,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiNetworkOptionsHyperEncode,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);


	SendDlgItemMessage(hWnd,VidomiNetworkOptionsMachineList, CB_RESETCONTENT , 0, 0);

	for(i=0;i<globals->numSlaveEncodeMachines;i++)
		{
		sprintf(machineText,"%d. \"%s\" on %s",i+1,globals->slaveEncodeMachines->slave[i].machineName,globals->slaveEncodeMachines->slave[i].machineIp);

		ret = SendDlgItemMessage(hWnd,VidomiNetworkOptionsMachineList, CB_ADDSTRING , (WPARAM)0 , (LPARAM) machineText); 									
		SendDlgItemMessage(hWnd,VidomiNetworkOptionsMachineList, CB_SETITEMDATA  , ret,i);
		if(i==0)
			SendDlgItemMessage(hWnd,VidomiNetworkOptionsMachineList, CB_SETCURSEL  , ret, 0);	
		}

	if(encodeServerEnabled)
		{
		SendDlgItemMessage(hWnd,VidomiNetworkOptionsEncodeServer,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
		state = true;
		}
	else
		{
		SendDlgItemMessage(hWnd,VidomiNetworkOptionsEncodeServer,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);
		state = false;
		}
	EnableWindow(GetDlgItem(hWnd,VidomiNetworkOptionsEncodeServerText2), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiNetworkOptionsEncodeServerIPAddress), state);	

	SetWindowText(GetDlgItem(hWnd, VidomiNetworkOptionsEncodeServerIPAddress),encodeServerIPAddress);	

}

// *****************************************************************************
// optionsNetworkClass::readWindowVars()
//
//
// *****************************************************************************
void optionsNetworkClass::readWindowVars(HWND hWnd)
{

	if(SendDlgItemMessage(hWnd,VidomiNetworkOptionsHyperEncode,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->hyperEncode=true;
	else
		convertConfig->hyperEncode=false;

	if(SendDlgItemMessage(hWnd,VidomiNetworkOptionsEncodeServer,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		encodeServerEnabled=true;
	else
		encodeServerEnabled=false;

	if(encodeServerEnabled)
		{
		GetWindowText(GetDlgItem(hWnd, VidomiNetworkOptionsEncodeServerIPAddress),encodeServerIPAddress,MAX_PATH);	
		}
}
