// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// optionsOutputSize.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

						
static TRANSLATEIDTABLE outputSizeOptionsTID[] = {
										{"VidomiOptionsVbrReserveText1",VidomiOptionsVbrReserveText1},
										{"VidomiOptionsVbrReserveText2",VidomiOptionsVbrReserveText2},
										{"VidomiOptionsVbrAudioBitrateText1",VidomiOptionsVbrAudioBitrateText1},
										{"VidomiOptionsVbrAudioBitrateText2",VidomiOptionsVbrAudioBitrateText2},
										{"VidomiOptionsVbrNumberOfDiscsText1",VidomiOptionsVbrNumberOfDiscsText1},
										{"VidomiOptionsVbrNumberOfDiscsText2",VidomiOptionsVbrNumberOfDiscsText2},
										{"VidomiVideoOptionsText7",VidomiVideoOptionsText7},
										{"VidomiVideoOptionsText8",VidomiVideoOptionsText8},
										{"VidomiVideoOptionsText9",VidomiVideoOptionsText9},
										{"VidomiOptionsOutputSizeText1",VidomiOptionsOutputSizeText1},
										{"VidomiOptionsOutputSizeText2",VidomiOptionsOutputSizeText2},
										{"VidomiOptionsOutputSizeText3",VidomiOptionsOutputSizeText3},
										{"VidomiOptionsOutputSizeText4",VidomiOptionsOutputSizeText4},
										{"VidomiOptionsOutputSizeText5",VidomiOptionsOutputSizeText5},
										{"VidomiOptionsOutputSizeText6",VidomiOptionsOutputSizeText6},
										{"VidomiOptionsCancel",VidomiOptionsCancel},
										{"VidomiOptionsOk",VidomiOptionsOk},
										{"",0}
																			
										};

// *****************************************************************************
// optionsOutputSizeClass::optionsOutputSizeClass()
//
// class constructor
// *****************************************************************************
optionsOutputSizeClass::optionsOutputSizeClass()
{
	convertConfig = new CONVERTCONFIG [1];

}

// *****************************************************************************
// optionsOutputSizeClass::~optionsOutputSizeClass()
//
// class destructor
// *****************************************************************************
optionsOutputSizeClass::~optionsOutputSizeClass()
{
	delete [] convertConfig;
}

// *****************************************************************************
// optionsOutputSizeClass::bringUpOptions()
//
// Bring up the options screen
// *****************************************************************************
void optionsOutputSizeClass::bringUpOptions(HINSTANCE hInst, CONVERTCONFIG * convertConfigIn, GATHERRESULTS * gatherResults)
{
	char text[4096];
	int i;

	optionshWnd = CreateDialog(hInst,MAKEINTRESOURCE(VidomiOutputSizeOptions),globals->hWnd, optionsDialogHandler);

	globals->translate->translate(text, "VidomiOutputSizeOptions");
	SetWindowText(optionshWnd, text);			

	for(i=0;;i++)
		{
		if(!outputSizeOptionsTID[i].id)
			break;
		globals->translate->translate(text, outputSizeOptionsTID[i].textId);
		SetWindowText(GetDlgItem(optionshWnd, outputSizeOptionsTID[i].id),text);			
		}

	scanData = gatherResults;

	convertConfigOriginal = convertConfigIn;
	memcpy(convertConfig,convertConfigOriginal,sizeof(CONVERTCONFIG));
	
	configureOptionsWindow(optionshWnd);

}

// *****************************************************************************
// encoderVideoClass::optionsDialogHandler()
//
//
// *****************************************************************************
BOOL CALLBACK optionsOutputSizeClass::optionsDialogHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->optionsOutputSize->optionsDialogHandler1(hWnd,message,wParam,lParam));
}
// *****************************************************************************
// optionsOutputSizeClass::optionsDialogHandler1()
//
//
// *****************************************************************************
BOOL optionsOutputSizeClass::optionsDialogHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	unsigned short commandId;
	unsigned short commandEvent;
	switch(message)
		{

		case WM_COMMAND:
			commandId   = LOWORD(wParam); // Remember, these are...
			commandEvent = HIWORD(wParam); // ...different for Win32!

			//Parse the menu selections:

			switch (commandId) 
				{
				case VidomiOptionsOk :
					readWindowVars(hWnd);

					convertConfigOriginal->useSpecifiedTarget = convertConfig->useSpecifiedTarget;
					convertConfigOriginal->specifiedTarget = convertConfig->specifiedTarget;
					convertConfigOriginal->maxOutputSize = convertConfig->maxOutputSize;
					convertConfigOriginal->vbrReserve = convertConfig->vbrReserve;
					convertConfigOriginal->vbrAudioBitrate = convertConfig->vbrAudioBitrate;
					convertConfigOriginal->vbrNumberOfDiscs = convertConfig->vbrNumberOfDiscs;
					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;
				
				case VidomiOptionsOutputSizeRadio1:
				case VidomiOptionsOutputSizeText2:
					if(commandEvent==BN_CLICKED)
						{
						convertConfig->useSpecifiedTarget = false;
						enableWindows(hWnd);
						}
					break;

				case VidomiOptionsOutputSizeRadio2:
				case VidomiOptionsOutputSizeText3:
					if(commandEvent==BN_CLICKED)
						{
						convertConfig->useSpecifiedTarget = true;
						enableWindows(hWnd);
						}
					break;

				case VidomiOptionsCancel :
					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;

				default : 
					break;

				}
			break;
			
		case WM_CLOSE:
			DestroyWindow(hWnd);
			globals->optionsUp=false;
			break;

		}

	return (FALSE);

}

// *****************************************************************************
// optionsOutputSizeClass::configureOptionsWindow()
//
//
// *****************************************************************************
void optionsOutputSizeClass::configureOptionsWindow(HWND hWnd)
{
	char tempStr[256];
	int ret;
	int i;

	sprintf(tempStr,"%d",convertConfig->maxOutputSize);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsMaxOutputSize),tempStr);

	sprintf(tempStr,"%d",convertConfig->vbrReserve);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsVbrReserve),tempStr);

	sprintf(tempStr,"%d",convertConfig->vbrAudioBitrate);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsVbrAudioBitrate),tempStr);

	SendDlgItemMessage(hWnd,VidomiOptionsVbrNumberOfDiscs, CB_RESETCONTENT , 0, 0);

	for(i=1;i<6;i++)
		{
		sprintf(tempStr,"%d",i);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsVbrNumberOfDiscs, CB_ADDSTRING , (WPARAM)0 , (LPARAM) tempStr); 									
		SendDlgItemMessage(hWnd,VidomiOptionsVbrNumberOfDiscs, CB_SETITEMDATA  , ret, i);
		if(i==convertConfig->vbrNumberOfDiscs)
			SendDlgItemMessage(hWnd,VidomiOptionsVbrNumberOfDiscs, CB_SETCURSEL  , ret, 0);
		}

	sprintf(tempStr,"%d",convertConfig->specifiedTarget);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsSpecifiedTarget),tempStr);

	enableWindows(hWnd);

}

// *****************************************************************************
// optionsOutputSizeClass::readWindowVars()
//
//
// *****************************************************************************
void optionsOutputSizeClass::readWindowVars(HWND hWnd)
{
	char tempStr[260];
	int ret;


	GetWindowText(GetDlgItem(hWnd, VidomiOptionsMaxOutputSize), tempStr , 256);
 	convertConfig->maxOutputSize = atoi(tempStr);

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsVbrReserve), tempStr , 256);
 	convertConfig->vbrReserve = atoi(tempStr);

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsVbrAudioBitrate), tempStr , 256);
 	convertConfig->vbrAudioBitrate = atoi(tempStr);

	ret = SendDlgItemMessage(hWnd,VidomiOptionsVbrNumberOfDiscs, CB_GETCURSEL  , 0, 0);
	convertConfig->vbrNumberOfDiscs = SendDlgItemMessage(hWnd,VidomiOptionsVbrNumberOfDiscs, CB_GETITEMDATA , ret, 0);

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsSpecifiedTarget), tempStr , 256);
 	convertConfig->specifiedTarget = atoi(tempStr);

}

// *****************************************************************************
// optionsOutputSizeClass::readWindowVars()
//
//
// *****************************************************************************
void optionsOutputSizeClass::enableWindows(HWND hWnd)
{
	int state;

	if(convertConfig->encodeMode == ENCODE_MODE_VBR ||
		convertConfig->encodeMode == ENCODE_MODE_HYBRID1 ||
		convertConfig->encodeMode == ENCODE_MODE_HYBRID2 ||
		convertConfig->encodeMode == ENCODE_MODE_DIVXVBR ||
		convertConfig->encodeMode == ENCODE_MODE_DIVXVBRPLUS ||
		convertConfig->encodeMode == ENCODE_MODE_XVIDVBR ||
		convertConfig->encodeMode == ENCODE_MODE_XVIDHYBRID)
		state = true;
	else
		state = false;

	EnableWindow(GetDlgItem(hWnd,VidomiOptionsVbrReserve), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsVbrReserveText1), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsVbrReserveText2), state);	


	EnableWindow(GetDlgItem(hWnd,VidomiOptionsOutputSizeText2), state);
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsOutputSizeText3), state);
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsOutputSizeRadio1), state);
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsOutputSizeRadio2), state);

	if(convertConfig->useSpecifiedTarget)
		{
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsOutputSizeText4), state);
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsOutputSizeText5), state);
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsSpecifiedTarget), state);

		EnableWindow(GetDlgItem(hWnd,VidomiOptionsVbrNumberOfDiscs), false);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsVbrNumberOfDiscsText1), false);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsVbrNumberOfDiscsText2), false);

		}
	else
		{
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsVbrNumberOfDiscs), state);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsVbrNumberOfDiscsText1), state);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsVbrNumberOfDiscsText2), state);

		EnableWindow(GetDlgItem(hWnd,VidomiOptionsOutputSizeText4), false);
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsOutputSizeText5), false);
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsSpecifiedTarget), false);
		}
	

	if(convertConfig->processAudio && convertConfig->mp3)
		state = false;
	else
		state = true;

	// Disable when the encode mode isn't able to hit a target size
	if(convertConfig->encodeMode == ENCODE_MODE_CBR || 
		convertConfig->encodeMode == ENCODE_MODE_CBR2PASS ||
		convertConfig->encodeMode == ENCODE_MODE_DIVXCBR ||
		convertConfig->encodeMode == ENCODE_MODE_DIVXCBRQUALITY)
		{
		state = false;
		}

	EnableWindow(GetDlgItem(hWnd,VidomiOptionsVbrAudioBitrate), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsVbrAudioBitrateText1), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsVbrAudioBitrateText2), state);	


	if(convertConfig->useSpecifiedTarget)
		{
		SendDlgItemMessage(hWnd,VidomiOptionsOutputSizeRadio1, BM_SETCHECK  ,BST_UNCHECKED, 0);
		SendDlgItemMessage(hWnd,VidomiOptionsOutputSizeRadio2, BM_SETCHECK  ,BST_CHECKED, 0);
		}
	else
		{
		SendDlgItemMessage(hWnd,VidomiOptionsOutputSizeRadio1, BM_SETCHECK  ,BST_CHECKED, 0);
		SendDlgItemMessage(hWnd,VidomiOptionsOutputSizeRadio2, BM_SETCHECK  ,BST_UNCHECKED, 0);
		}
}


