// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// optionsaudio.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

//int lameBitrates[] = {320,256,224,192,160,128};
static int lameBitrates[] = {320,256,224,192,160,128,112,96,80,64,56,48,40,32,24,16,8};
static int lameMpeg1Bitrates[] = {320,256,224,192,160,128,112,96,80,64,56,48,40,32};
static char * normalizeBy[] = {"NormalizeByNone","NormalizeByOnePass","NormalizeByTwoPass"};
//static char * lameEncMode[] = {"LameEncModeCBR","LameEncModeABR"};
static char * lameEncMode[] = {"LameEncModeCBR","LameEncModeABR","LameEncModeVBR"};
static int lameEncModeValue[] = {LAME_CBR,LAME_ABR,LAME_VBR};
static char * lameChanType[] = {"LameChanTypeStereo","LameChanTypeJointStereo","LameChanTypeDualChannel"};
static char * lameEncQuality[] = {"LameEncQualityNormal"}; //,"Low","High","Voice"};
static int lameOutFreq[] = {0,48000,44100,32000,24000,22050,16000,12000,11025,8000};
static int lameOutFreqMpeg1[] = {0,48000,44100,32000,24000,22050,16000,12000,11025,8000};
static char * vbrQual[] = {"VbrQuality0" , "VbrQuality1","VbrQuality2","VbrQuality3","VbrQuality4","VbrQuality5","VbrQuality6","VbrQuality7","VbrQuality8","VbrQuality9"};

static TRANSLATEIDTABLE audioOptionsTID[] = {
										{"VidomiAudioOptionsText1",VidomiAudioOptionsText1},
										{"VidomiAudioOptionsText2",VidomiAudioOptionsText2},
										{"VidomiOptionsResampleAudioText",VidomiOptionsResampleAudioText},
										{"VidomiOptionsSelectAudioCodec",VidomiOptionsSelectAudioCodec},
										{"VidomiOptionsUseLameText",VidomiOptionsUseLameText},
										{"VidomiOptionsLimitLameToMpeg1Text",VidomiOptionsLimitLameToMpeg1Text},
										{"VidomiOptionsNormalizeLevelText1",VidomiOptionsNormalizeLevelText1},
										{"VidomiOptionsNormalizeLevelText2",VidomiOptionsNormalizeLevelText2},
										{"VidomiAudioOptionsText3",VidomiAudioOptionsText3},
										{"VidomiAudioOptionsText4",VidomiAudioOptionsText4},
										{"VidomiAudioOptionsText5",VidomiAudioOptionsText5},
										{"VidomiAudioOptionsText6",VidomiAudioOptionsText6},
										{"VidomiOptionsLameEncodeModeText",VidomiOptionsLameEncodeModeText},
										{"VidomiOptionsLameMinBitrateText",VidomiOptionsLameMinBitrateText},
										{"VidomiOptionsLameChannelTypeText",VidomiOptionsLameChannelTypeText},
										{"VidomiOptionsLameMaxBitrateText",VidomiOptionsLameMaxBitrateText},
										{"VidomiOptionsLameQualityText",VidomiOptionsLameQualityText},
										{"VidomiOptionsLameOutputFrequencyText",VidomiOptionsLameOutputFrequencyText},
										{"VidomiOptionsLameVbrQualityText",VidomiOptionsLameVbrQualityText},
										{"VidomiAudioOptionsText7",VidomiAudioOptionsText7},
										{"VidomiOptionsCancel",VidomiOptionsCancel},
										{"VidomiOptionsOk",VidomiOptionsOk},

										{"",0}
																			
										};

// *****************************************************************************
// optionsAudioClass::optionsAudioClass()
//
// conversion class constructor
// *****************************************************************************
optionsAudioClass::optionsAudioClass()
{
	convertConfig = new CONVERTCONFIG [1];
}

// *****************************************************************************
// optionsAudioClass::~optionsAudioClass()
//
// conversion class destructor
// *****************************************************************************
optionsAudioClass::~optionsAudioClass()
{
	delete [] convertConfig;
}

// *****************************************************************************
// optionsAudioClass::bringUpOptions()
//
// Bring up the options screen
// *****************************************************************************
void optionsAudioClass::bringUpOptions(HINSTANCE hInst, CONVERTCONFIG * convertConfigIn, GATHERRESULTS * gatherResults)
{
	char text[4096];
	int i;

	optionshWnd = CreateDialog(hInst,MAKEINTRESOURCE(VidomiAudioOptions),globals->hWnd, optionsDialogHandler);
	

	globals->translate->translate(text, "VidomiAudioOptions");
	SetWindowText(optionshWnd, text);			

	for(i=0;;i++)
		{
		if(!audioOptionsTID[i].id)
			break;
		globals->translate->translate(text, audioOptionsTID[i].textId);
		SetWindowText(GetDlgItem(optionshWnd, audioOptionsTID[i].id),text);			
		}

	scanData = gatherResults;

	convertConfigOriginal = convertConfigIn;
	memcpy(convertConfig,convertConfigOriginal,sizeof(CONVERTCONFIG));
	
	convertConfig->audioFormatOut  = (MPEGLAYER3WAVEFORMAT *) new char [AUDIO_STATE_DEFAULT_SIZE];
	memcpy(convertConfig->audioFormatOut,convertConfigOriginal->audioFormatOut,AUDIO_STATE_DEFAULT_SIZE);

	audioFormatChanged=false;
	currentAudioChannel = 0;

	configureOptionsWindow(optionshWnd);


}

// *****************************************************************************
// encoderVideoClass::optionsDialogHandler()
//
//
// *****************************************************************************
BOOL CALLBACK optionsAudioClass::optionsDialogHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->optionsAudio->optionsDialogHandler1(hWnd,message,wParam,lParam));
}

// *****************************************************************************
// optionsAudioClass::windowDialogHandler()
//
//
// *****************************************************************************
BOOL optionsAudioClass::optionsDialogHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	unsigned short commandId;
	unsigned short commandEvent;
	int state;
	int channel;
	int ret;
	int i;
	char tempStr[260];
	int foundHighest;

	switch(message)
		{

		case WM_COMMAND:
			commandId   = LOWORD(wParam); // Remember, these are...
			commandEvent = HIWORD(wParam); // ...different for Win32!

			//Parse the menu selections:

			switch (commandId) 
				{
				case VidomiOptionsOk :
					readWindowVars(hWnd);

					if(audioFormatChanged)
						{
						memcpy(convertConfigOriginal->audioFormatOut,convertConfig->audioFormatOut,AUDIO_STATE_DEFAULT_SIZE);
						}
					convertConfigOriginal->processAudio = convertConfig->processAudio;
					convertConfigOriginal->resample48KhzAudio = convertConfig->resample48KhzAudio;
					convertConfigOriginal->mp3 = convertConfig->mp3;
					convertConfigOriginal->mp3LimitToCbr = convertConfig->mp3LimitToCbr;
					convertConfigOriginal->mp3LimitToMpeg1 = convertConfig->mp3LimitToMpeg1;
					//If the primary track has changed, tell the main window about the change
					if(convertConfigOriginal->processAudioSelected[0] != convertConfig->processAudioSelected[0])
						{
						SendDlgItemMessage(globals->window->encoderhWnd,VidomiMainAudioTrack, CB_SETCURSEL  , convertConfig->processAudioSelected[0], 0);						
						}
						
					for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
						{
						convertConfigOriginal->mp3EncodeMode[channel] = convertConfig->mp3EncodeMode[channel];
						convertConfigOriginal->mp3ChannelType[channel] = convertConfig->mp3ChannelType[channel];
						convertConfigOriginal->mp3Quality[channel] = convertConfig->mp3Quality[channel];
						convertConfigOriginal->mp3MinBitrate[channel] = convertConfig->mp3MinBitrate[channel];
						convertConfigOriginal->mp3MaxBitrate[channel] = convertConfig->mp3MaxBitrate[channel];

						if(convertConfigOriginal->mp3EncodeMode[channel]==LAME_VBR)
							{
							if(convertConfigOriginal->mp3MaxBitrate[channel] <convertConfigOriginal->mp3MinBitrate[channel])
								{
								// Min and MAX are swapped. Make them correct.
								convertConfigOriginal->mp3MinBitrate[channel] = convertConfig->mp3MaxBitrate[channel];
								convertConfigOriginal->mp3MaxBitrate[channel] = convertConfig->mp3MinBitrate[channel];
								}
							}

						convertConfigOriginal->mp3OutputFrequency[channel] = convertConfig->mp3OutputFrequency[channel];
						convertConfigOriginal->mp3VbrQuality[channel] = convertConfig->mp3VbrQuality[channel];
						convertConfigOriginal->processAudioSelected[channel] = convertConfig->processAudioSelected[channel]; 
						strcpy(convertConfigOriginal->audioStreamName[channel],convertConfig->audioStreamName[channel]);
						}
					convertConfigOriginal->normalizeAudio = convertConfig->normalizeAudio;
					convertConfigOriginal->normalizeLevel = convertConfig->normalizeLevel;

					// Compact the audio channels into a group
					foundHighest = 0;
					for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
						{
						if(!foundHighest)
							{
							if(convertConfigOriginal->processAudioSelected[channel]==-1)
								{
								foundHighest = channel;
								}
							}
						else
							{
							if(convertConfigOriginal->processAudioSelected[channel]!=-1)
								{
								convertConfigOriginal->mp3EncodeMode[foundHighest] = convertConfigOriginal->mp3EncodeMode[channel];
								convertConfigOriginal->mp3ChannelType[foundHighest] = convertConfigOriginal->mp3ChannelType[channel];
								convertConfigOriginal->mp3Quality[foundHighest] = convertConfigOriginal->mp3Quality[channel];
								convertConfigOriginal->mp3MinBitrate[foundHighest] = convertConfigOriginal->mp3MinBitrate[channel];
								convertConfigOriginal->mp3MaxBitrate[foundHighest] = convertConfigOriginal->mp3MaxBitrate[channel];
								convertConfigOriginal->mp3OutputFrequency[foundHighest] = convertConfigOriginal->mp3OutputFrequency[channel];
								convertConfigOriginal->mp3VbrQuality[foundHighest] = convertConfigOriginal->mp3VbrQuality[channel];
								convertConfigOriginal->processAudioSelected[foundHighest] = convertConfigOriginal->processAudioSelected[channel]; 
								foundHighest++;
								convertConfigOriginal->processAudioSelected[channel] = -1;
								}
							}
						}
					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;

				case VidomiOptionsProcessAudio :
					if(SendDlgItemMessage(hWnd,VidomiOptionsProcessAudio,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
						convertConfig->processAudio = true;
					else
						convertConfig->processAudio = false;

					enableWindows(hWnd);
					break;

				case VidomiOptionsLimitLameToMpeg1:
					if(convertConfig->mp3)
						{
						if(SendDlgItemMessage(hWnd,VidomiOptionsLimitLameToMpeg1,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
							{
							convertConfig->mp3LimitToMpeg1 = true;
							for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
								{
								if(!(convertConfig->mp3OutputFrequency[channel]==32000 ||
									convertConfig->mp3OutputFrequency[channel]==44100 ||
									convertConfig->mp3OutputFrequency[channel]==48000 ||
									convertConfig->mp3OutputFrequency[channel]==0))
									{
									convertConfig->mp3OutputFrequency[channel]=0;
									}
								}
							}
						else
							{
							convertConfig->mp3LimitToMpeg1 = false;
							}
						buildLameOptions(hWnd);
						
						}
					break;

				case VidomiOptionsUseLame :
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_GETCURSEL  , 0, 0);
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_GETITEMDATA , ret, 0);
					convertConfig->mp3MinBitrate[currentAudioChannel] = ret;
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_GETCURSEL  , 0, 0);
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_GETITEMDATA , ret, 0);
					convertConfig->mp3MaxBitrate[currentAudioChannel] = ret;
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameEncodeMode, CB_GETCURSEL  , 0, 0);
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameEncodeMode, CB_GETITEMDATA , ret, 0);
					convertConfig->mp3EncodeMode[currentAudioChannel] = ret;
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameChannelType, CB_GETCURSEL  , 0, 0);
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameChannelType, CB_GETITEMDATA , ret, 0);
					convertConfig->mp3ChannelType[currentAudioChannel] = ret;
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameQuality, CB_GETCURSEL  , 0, 0);
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameQuality, CB_GETITEMDATA , ret, 0);
					convertConfig->mp3Quality[currentAudioChannel] = ret;
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameOutputFrequency, CB_GETCURSEL  , 0, 0);
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameOutputFrequency, CB_GETITEMDATA , ret, 0);
					convertConfig->mp3OutputFrequency[currentAudioChannel] = ret;
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameVbrQuality, CB_GETCURSEL  , 0, 0);
					ret = SendDlgItemMessage(hWnd,VidomiOptionsLameVbrQuality, CB_GETITEMDATA , ret, 0);
					convertConfig->mp3VbrQuality[currentAudioChannel] = ret;
					GetWindowText(GetDlgItem(hWnd, VidomiOptionsAudioChannelName),convertConfig->audioStreamName[currentAudioChannel],64);

					state = false;
					if(convertConfig->lameFound)
						{
						if(SendDlgItemMessage(hWnd,VidomiOptionsUseLame,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
							{
							state = true;
							convertConfig->mp3 = true;
							}
						else
							{
							convertConfig->mp3 = false;
							}
						}
					enableWindows(hWnd);
					break;

				case VidomiOptionsLameEncodeMode:
					if(commandEvent==CBN_SELCHANGE)
						{
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameEncodeMode, CB_GETCURSEL  , 0, 0);
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameEncodeMode, CB_GETITEMDATA , ret, 0);
						convertConfig->mp3EncodeMode[currentAudioChannel] = ret;
						enableWindows(hWnd);
						}
					break;

				case VidomiOptionsOutputChannel:
					if(commandEvent==CBN_SELCHANGE)
						{
						// Update the current audio selection for the old channel number
						ret = SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_GETCURSEL  , 0, 0);
						ret = SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_GETITEMDATA , ret, 0);
						convertConfig->processAudioSelected[currentAudioChannel] = ret-1;
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_GETCURSEL  , 0, 0);
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_GETITEMDATA , ret, 0);
						convertConfig->mp3MinBitrate[currentAudioChannel] = ret;
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_GETCURSEL  , 0, 0);
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_GETITEMDATA , ret, 0);
						convertConfig->mp3MaxBitrate[currentAudioChannel] = ret;
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameEncodeMode, CB_GETCURSEL  , 0, 0);
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameEncodeMode, CB_GETITEMDATA , ret, 0);
						convertConfig->mp3EncodeMode[currentAudioChannel] = ret;
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameChannelType, CB_GETCURSEL  , 0, 0);
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameChannelType, CB_GETITEMDATA , ret, 0);
						convertConfig->mp3ChannelType[currentAudioChannel] = ret;
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameQuality, CB_GETCURSEL  , 0, 0);
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameQuality, CB_GETITEMDATA , ret, 0);
						convertConfig->mp3Quality[currentAudioChannel] = ret;
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameOutputFrequency, CB_GETCURSEL  , 0, 0);
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameOutputFrequency, CB_GETITEMDATA , ret, 0);
						convertConfig->mp3OutputFrequency[currentAudioChannel] = ret;
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameVbrQuality, CB_GETCURSEL  , 0, 0);
						ret = SendDlgItemMessage(hWnd,VidomiOptionsLameVbrQuality, CB_GETITEMDATA , ret, 0);
						convertConfig->mp3VbrQuality[currentAudioChannel] = ret;
						GetWindowText(GetDlgItem(hWnd, VidomiOptionsAudioChannelName),convertConfig->audioStreamName[currentAudioChannel],64);

						//Update the channel number
						ret = SendDlgItemMessage(hWnd,VidomiOptionsOutputChannel, CB_GETCURSEL  , 0, 0);
						currentAudioChannel = SendDlgItemMessage(hWnd,VidomiOptionsOutputChannel, CB_GETITEMDATA , ret, 0);
						
						if(convertConfig->mp3)
							{
							buildLameOptions(hWnd);
							}
		

						//And rebuild / reselect the audio channel
						SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_RESETCONTENT , 0, 0);
						if(scanData->audioFound)
							{
							for(i=0;i<scanData->audioFound+1;i++)
								{
								if(i==0)
									{
									if(currentAudioChannel!=0)
										{
										strcpy(tempStr,"None");
										ret = SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_ADDSTRING , (WPARAM)0 , (LPARAM) tempStr); 									
										}
									}
								else
									{
									ret = SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_ADDSTRING , (WPARAM)0 , (LPARAM) &globals->gatherResults->audioName[i-1]); 									
									}
								SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_SETITEMDATA  , ret, i);
								if((i-1)==convertConfig->processAudioSelected[currentAudioChannel])
									SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_SETCURSEL  , ret, 0);
								}
							}
						enableWindows(hWnd);
						}
					break;

				case VidomiOptionsCancel :
					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;

				case VidomiOptionsSelectAudioCodec :
					setAudioOutput();
					break;

				}
			break;
			
		case WM_CLOSE:
			DestroyWindow(hWnd);
			globals->optionsUp=false;
			break;

		}

	return (FALSE);

}

// *****************************************************************************
// optionsAudioClass::setAudioInputFormat()
//
// *****************************************************************************
bool optionsAudioClass::setAudioOutput()
{

	
	memset(&audioFormat,0,sizeof (ACMFORMATCHOOSE));

	audioFormat.cbStruct = sizeof(ACMFORMATCHOOSE);
	audioFormat.fdwStyle = ACMFORMATCHOOSE_STYLEF_INITTOWFXSTRUCT;
	audioFormat.hwndOwner = optionshWnd;
	audioFormat.pwfx = (WAVEFORMATEX *) convertConfig->audioFormatOut;
	audioFormat.cbwfx = AUDIO_STATE_DEFAULT_SIZE;
	audioFormat.pszTitle = "Select Audio Encoder";
	audioFormat.szFormatTag[0]=0;
	audioFormat.szFormat[0]=0;
	audioFormat.pszName = NULL;
	audioFormat.cchName = 0;
	audioFormat.fdwEnum = 0;

	audioFormat.pwfxEnum = NULL;			//&audioFormatIn;
	audioFormat.hInstance = NULL;
	audioFormat.pszTemplateName=NULL;
	audioFormat.lCustData=0;
	audioFormat.pfnHook=NULL;

	if(acmFormatChoose(&audioFormat)!=MMSYSERR_NOERROR)
		{
		return(false);
		}

	audioFormatChanged=true;
	return(true);

}


// *****************************************************************************
// optionsAudioClass::configureOptionsWindow()
//
//
// *****************************************************************************
void optionsAudioClass::configureOptionsWindow(HWND hWnd)
{
	char tempStr[256];
	int ret;
	int i;
	char text[4096];

	// Force to true for time being
	convertConfig->mp3LimitToCbr = true;

	if(convertConfig->processAudio)
		SendDlgItemMessage(hWnd,VidomiOptionsProcessAudio,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsProcessAudio,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(convertConfig->resample48KhzAudio)
		SendDlgItemMessage(hWnd,VidomiOptionsResampleAudio,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsResampleAudio,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(convertConfig->lameFound)
		{
		if(convertConfig->mp3)
			{
			SendDlgItemMessage(hWnd,VidomiOptionsUseLame,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
			}
		else
			{
			SendDlgItemMessage(hWnd,VidomiOptionsUseLame,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);
			}

		if(convertConfig->mp3LimitToMpeg1)
			{
			SendDlgItemMessage(hWnd,VidomiOptionsLimitLameToMpeg1,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
			}
		else
			{
			SendDlgItemMessage(hWnd,VidomiOptionsLimitLameToMpeg1,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);
			}
		}

	SendDlgItemMessage(hWnd,VidomiOptionsNormalizeAudio, CB_RESETCONTENT , 0, 0);

	for(i=0;i<(sizeof(normalizeBy)/4);i++)
		{
		globals->translate->translate(text, normalizeBy[i]);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsNormalizeAudio, CB_ADDSTRING , (WPARAM)0 , (LPARAM) text); 									
		SendDlgItemMessage(hWnd,VidomiOptionsNormalizeAudio, CB_SETITEMDATA  , ret, i);
		if(i==convertConfig->normalizeAudio)
			SendDlgItemMessage(hWnd,VidomiOptionsNormalizeAudio, CB_SETCURSEL  , ret, 0);
		}

	SendDlgItemMessage(hWnd,VidomiOptionsOutputChannel, CB_RESETCONTENT , 0, 0);

	for(i=0;i<MAX_AUDIO_CHANNELS;i++)
		{
		sprintf(tempStr,"%d",i+1);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsOutputChannel, CB_ADDSTRING , (WPARAM)0 , (LPARAM) tempStr); 									
		SendDlgItemMessage(hWnd,VidomiOptionsOutputChannel, CB_SETITEMDATA  , ret, i);
		if(i==currentAudioChannel)
			SendDlgItemMessage(hWnd,VidomiOptionsOutputChannel, CB_SETCURSEL  , ret, 0);
		}

	SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_RESETCONTENT , 0, 0);
	if(scanData->audioFound)
		{
		for(i=0;i<scanData->audioFound+1;i++)
			{
			if(i==0)
				{
				if(currentAudioChannel!=0)
					{										
					strcpy(tempStr,"None");
					ret = SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_ADDSTRING , (WPARAM)0 , (LPARAM) tempStr); 									
					}
				}
			else
				{
				ret = SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_ADDSTRING , (WPARAM)0 , (LPARAM) &globals->gatherResults->audioName[i-1]); 									
				}
			SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_SETITEMDATA  , ret, i);
			if((i-1)==convertConfig->processAudioSelected[currentAudioChannel])
				SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_SETCURSEL  , ret, 0);
			}
		}

	buildLameOptions(hWnd);

	sprintf(tempStr,"%d",convertConfig->normalizeLevel);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsNormalizeLevel),tempStr);

	enableWindows(hWnd);
}

// *****************************************************************************
// optionsAudioClass::readWindowVars()
//
//
// *****************************************************************************
void optionsAudioClass::readWindowVars(HWND hWnd)
{
	int ret;
	char tempStr[260];

	if(SendDlgItemMessage(hWnd,VidomiOptionsProcessAudio,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->processAudio=true;
	else
		convertConfig->processAudio=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsResampleAudio,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->resample48KhzAudio=true;
	else
		convertConfig->resample48KhzAudio=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsUseLame,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->mp3=true;
	else
		convertConfig->mp3=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsLimitLameToMpeg1,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->mp3LimitToMpeg1=true;
	else
		convertConfig->mp3LimitToMpeg1=false;

	// Resample must be off when using LAME.
	if(convertConfig->mp3)
		{
		convertConfig->resample48KhzAudio=false;
		}

	ret = SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_GETCURSEL  , 0, 0);
	ret = SendDlgItemMessage(hWnd,VidomiOptionsAudioTrack, CB_GETITEMDATA , ret, 0);
	convertConfig->processAudioSelected[currentAudioChannel] = ret-1;

	if(convertConfig->mp3)
		{
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_GETCURSEL  , 0, 0);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_GETITEMDATA , ret, 0);
		convertConfig->mp3MinBitrate[currentAudioChannel] = ret;
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_GETCURSEL  , 0, 0);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_GETITEMDATA , ret, 0);
		convertConfig->mp3MaxBitrate[currentAudioChannel] = ret;
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameEncodeMode, CB_GETCURSEL  , 0, 0);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameEncodeMode, CB_GETITEMDATA , ret, 0);
		convertConfig->mp3EncodeMode[currentAudioChannel] = ret;
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameChannelType, CB_GETCURSEL  , 0, 0);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameChannelType, CB_GETITEMDATA , ret, 0);
		convertConfig->mp3ChannelType[currentAudioChannel] = ret;
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameQuality, CB_GETCURSEL  , 0, 0);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameQuality, CB_GETITEMDATA , ret, 0);
		convertConfig->mp3Quality[currentAudioChannel] = ret;
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameOutputFrequency, CB_GETCURSEL  , 0, 0);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameOutputFrequency, CB_GETITEMDATA , ret, 0);
		convertConfig->mp3OutputFrequency[currentAudioChannel] = ret;
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameVbrQuality, CB_GETCURSEL  , 0, 0);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameVbrQuality, CB_GETITEMDATA , ret, 0);
		convertConfig->mp3VbrQuality[currentAudioChannel] = ret;
		GetWindowText(GetDlgItem(hWnd, VidomiOptionsAudioChannelName),convertConfig->audioStreamName[currentAudioChannel],64);
		}

	ret = SendDlgItemMessage(hWnd,VidomiOptionsNormalizeAudio, CB_GETCURSEL  , 0, 0);
	convertConfig->normalizeAudio = SendDlgItemMessage(hWnd,VidomiOptionsNormalizeAudio, CB_GETITEMDATA , ret, 0);

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsNormalizeLevel), tempStr , 256);
 	convertConfig->normalizeLevel = atoi(tempStr);
	
}

// *****************************************************************************
// optionsAudioClass::readWindowVars()
//
//
// *****************************************************************************
void optionsAudioClass::buildLameOptions(HWND hWnd)
{
	int i;
	int ret;
	char tempStr[255];
	int state;
	char text[4096];

	//Rebuild and select the lame bitrate
	if(convertConfig->mp3LimitToMpeg1)
		{
		SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_RESETCONTENT , 0, 0);

		for(i=0;i<(sizeof(lameMpeg1Bitrates)/4);i++)
			{
			sprintf(tempStr,"%dKbps",lameMpeg1Bitrates[i]);
			ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_ADDSTRING , (WPARAM)0 , (LPARAM) tempStr); 									
			SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_SETITEMDATA  , ret, lameMpeg1Bitrates[i]);
			if(lameMpeg1Bitrates[i]==convertConfig->mp3MinBitrate[currentAudioChannel])
				SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_SETCURSEL  , ret, 0);
			}

		SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_RESETCONTENT , 0, 0);

		for(i=0;i<(sizeof(lameMpeg1Bitrates)/4);i++)
			{
			sprintf(tempStr,"%dKbps",lameMpeg1Bitrates[i]);
			ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_ADDSTRING , (WPARAM)0 , (LPARAM) tempStr); 									
			SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_SETITEMDATA  , ret, lameMpeg1Bitrates[i]);
			if(lameMpeg1Bitrates[i]==convertConfig->mp3MaxBitrate[currentAudioChannel])
				SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_SETCURSEL  , ret, 0);
			}
		}
	else
		{
		SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_RESETCONTENT , 0, 0);

		for(i=0;i<(sizeof(lameBitrates)/4);i++)
			{
			sprintf(tempStr,"%dKbps",lameBitrates[i]);
			ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_ADDSTRING , (WPARAM)0 , (LPARAM) tempStr); 									
			SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_SETITEMDATA  , ret, lameBitrates[i]);
			if(lameBitrates[i]==convertConfig->mp3MinBitrate[currentAudioChannel])
				SendDlgItemMessage(hWnd,VidomiOptionsLameMinBitrate, CB_SETCURSEL  , ret, 0);
			}
		SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_RESETCONTENT , 0, 0);

		for(i=0;i<(sizeof(lameBitrates)/4);i++)
			{
			sprintf(tempStr,"%dKbps",lameBitrates[i]);
			ret = SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_ADDSTRING , (WPARAM)0 , (LPARAM) tempStr); 									
			SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_SETITEMDATA  , ret, lameBitrates[i]);
			if(lameBitrates[i]==convertConfig->mp3MaxBitrate[currentAudioChannel])
				SendDlgItemMessage(hWnd,VidomiOptionsLameMaxBitrate, CB_SETCURSEL  , ret, 0);
			}
		}

	SendDlgItemMessage(hWnd,VidomiOptionsLameEncodeMode, CB_RESETCONTENT , 0, 0);
	for(i=0;i<(sizeof(lameEncMode)/4);i++)
		{
		globals->translate->translate(text, lameEncMode[i]);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameEncodeMode, CB_ADDSTRING , (WPARAM)0 , (LPARAM) text); 									
		SendDlgItemMessage(hWnd,VidomiOptionsLameEncodeMode, CB_SETITEMDATA  , ret, lameEncModeValue[i]);
		if(lameEncModeValue[i]==convertConfig->mp3EncodeMode[currentAudioChannel])
			SendDlgItemMessage(hWnd,VidomiOptionsLameEncodeMode, CB_SETCURSEL  , ret, 0);
		}
		
		
	SendDlgItemMessage(hWnd,VidomiOptionsLameChannelType, CB_RESETCONTENT , 0, 0);
	for(i=0;i<(sizeof(lameChanType)/4);i++)
		{
		globals->translate->translate(text, lameChanType[i]);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameChannelType, CB_ADDSTRING , (WPARAM)0 , (LPARAM) text); 									
		SendDlgItemMessage(hWnd,VidomiOptionsLameChannelType, CB_SETITEMDATA  , ret, i);
		if(i==convertConfig->mp3ChannelType[currentAudioChannel])
			SendDlgItemMessage(hWnd,VidomiOptionsLameChannelType, CB_SETCURSEL  , ret, 0);
		}

	SendDlgItemMessage(hWnd,VidomiOptionsLameQuality, CB_RESETCONTENT , 0, 0);
	for(i=0;i<(sizeof(lameEncQuality)/4);i++)
		{
		globals->translate->translate(text, lameEncQuality[i]);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameQuality, CB_ADDSTRING , (WPARAM)0 , (LPARAM) text); 									
		SendDlgItemMessage(hWnd,VidomiOptionsLameQuality, CB_SETITEMDATA  , ret, i);
		if(i==convertConfig->mp3Quality[currentAudioChannel])
			SendDlgItemMessage(hWnd,VidomiOptionsLameQuality, CB_SETCURSEL  , ret, 0);
		}

	SendDlgItemMessage(hWnd,VidomiOptionsLameVbrQuality, CB_RESETCONTENT , 0, 0);
	for(i=0;i<(sizeof(vbrQual)/4);i++)
		{
		globals->translate->translate(text, vbrQual[i]);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsLameVbrQuality, CB_ADDSTRING , (WPARAM)0 , (LPARAM) text); 									
		SendDlgItemMessage(hWnd,VidomiOptionsLameVbrQuality, CB_SETITEMDATA  , ret, i);
		if(i==convertConfig->mp3VbrQuality[currentAudioChannel])
			SendDlgItemMessage(hWnd,VidomiOptionsLameVbrQuality, CB_SETCURSEL  , ret, 0);
		}

	SendDlgItemMessage(hWnd,VidomiOptionsLameOutputFrequency, CB_RESETCONTENT , 0, 0);

	for(i=0;i<(sizeof(lameOutFreq)/4);i++)
		{
		if(lameOutFreq[i]==0)
			{
			// Get the translation for "Same As Source"
			// Note that this translate is different than most others
			globals->translate->translate(tempStr, "SameAsSource");
			}
		else
			{
			// Otherwise create the KBps Text
			sprintf(tempStr,"%d Hz",lameOutFreq[i]);
			}

		state = !convertConfig->mp3LimitToMpeg1;

		if(convertConfig->mp3LimitToMpeg1 && (lameOutFreq[i]==48000 ||lameOutFreq[i]==44100||lameOutFreq[i]==32000||lameOutFreq[i]==0))
			{
			state = true;
			}

		if(state)
			{
			ret = SendDlgItemMessage(hWnd,VidomiOptionsLameOutputFrequency, CB_ADDSTRING , (WPARAM)0 , (LPARAM) tempStr); 									
			SendDlgItemMessage(hWnd,VidomiOptionsLameOutputFrequency, CB_SETITEMDATA  , ret, lameOutFreq[i]);
			if(lameOutFreq[i]==convertConfig->mp3OutputFrequency[currentAudioChannel])
				SendDlgItemMessage(hWnd,VidomiOptionsLameOutputFrequency, CB_SETCURSEL  , ret, 0);
			}
		}

	SetWindowText(GetDlgItem(hWnd, VidomiOptionsAudioChannelName),convertConfig->audioStreamName[currentAudioChannel]);

}

// *****************************************************************************
// optionsAudioClass::readWindowVars()
//
//
// *****************************************************************************
void optionsAudioClass::enableWindows(HWND hWnd)
{
	int state;

	state = convertConfig->processAudio;

	EnableWindow(GetDlgItem(hWnd,VidomiOptionsUseLameText), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsNormalizeAudio), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsNormalizeLevel), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsNormalizeLevelText1), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsNormalizeLevelText2), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsUseLame), state);	

	if(state)
		{
		state = false;
		if(convertConfig->lameFound)
			{
			if(convertConfig->mp3)
				{
				state = convertConfig->processAudio;
				}
			}
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsSelectAudioCodec), !state);
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsResampleAudio), !state);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsResampleAudioText), !state);	
	
		}
	else
		{
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsSelectAudioCodec), state);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsResampleAudio), state);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsResampleAudioText), state);	
		}

	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLimitLameToMpeg1), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLimitLameToMpeg1Text), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsOutputChannel), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsAudioTrack), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameMinBitrate), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameMinBitrateText), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameEncodeMode), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameEncodeModeText), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameChannelType), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameChannelTypeText), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameQuality), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameQualityText), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameOutputFrequency), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameOutputFrequencyText), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsAudioChannelName), state);	

	if(convertConfig->mp3EncodeMode[currentAudioChannel] != LAME_VBR)
		{
		state = false;
		}
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameMaxBitrate), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameMaxBitrateText), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameVbrQuality), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsLameVbrQualityText), state);	



}