// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// optionsgeneral.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;
static char * extensionTypes[] = {".avi",".vid",".svid",".divx",".xvid"};
static char * outputSizeText[] = {"VidomiGeneralOptions100Percent","VidomiGeneralOptions75Percent","VidomiGeneralOptions50Percent"};

static TRANSLATEIDTABLE generalOptionsTID[] = {
										{"VidomiOptionsCancel",VidomiOptionsCancel},
										{"VidomiOptionsOk",VidomiOptionsOk},
										{"VidomiGeneralOptionsText1",VidomiGeneralOptionsText1},
										{"VidomiOptionsResetToDefaults",VidomiOptionsResetToDefaults},
										{"VidomiGeneralOptionsText2",VidomiGeneralOptionsText2},
										{"VidomiGeneralOptionsText3",VidomiGeneralOptionsText3},
										{"VidomiGeneralOptionsText4",VidomiGeneralOptionsText4},
										{"VidomiGeneralOptionsText5",VidomiGeneralOptionsText5},
										{"VidomiGeneralOptionsText6",VidomiGeneralOptionsText6},
										{"VidomiGeneralOptionsText7",VidomiGeneralOptionsText7},
										{"VidomiGeneralOptionsText8",VidomiGeneralOptionsText8},
										{"VidomiGeneralOptionsText9",VidomiGeneralOptionsText9},
										{"VidomiGeneralOptionsText10",VidomiGeneralOptionsText10},
										{"VidomiGeneralOptionsText11",VidomiGeneralOptionsText11},
										{"VidomiGeneralOptionsText12",VidomiGeneralOptionsText12},
										{"VidomiGeneralOptionsText13",VidomiGeneralOptionsText13},
										{"VidomiGeneralOptionsText14",VidomiGeneralOptionsText14},
										{"VidomiGeneralOptionsText15",VidomiGeneralOptionsText15},							
										{"VidomiGeneralOptionsText16",VidomiGeneralOptionsText16},							
										{"VidomiGeneralOptionsText17",VidomiGeneralOptionsText17},							
										
										{"",0}

										};

// *****************************************************************************
// optionsGeneralClass::optionsGeneralClass()
//
// conversion class constructor
// *****************************************************************************
optionsGeneralClass::optionsGeneralClass()
{
	convertConfig = new CONVERTCONFIG [1];

}

// *****************************************************************************
// optionsGeneralClass::~optionsGeneralClass()
//
// conversion class destructor
// *****************************************************************************
optionsGeneralClass::~optionsGeneralClass()
{
	delete [] convertConfig;
}

// *****************************************************************************
// optionsGeneralClass::bringUpOptions()
//
// Bring up the options screen
// *****************************************************************************
void optionsGeneralClass::bringUpOptions(HINSTANCE hInst, CONVERTCONFIG * convertConfigIn)
{
	char text[4096];
	int i;

	optionshWnd = CreateDialog(hInst,MAKEINTRESOURCE(VidomiGeneralOptions),globals->hWnd, optionsDialogHandler);
	

	globals->translate->translate(text, "VidomiGeneralOptions");
	SetWindowText(optionshWnd, text);			

	for(i=0;;i++)
		{
		if(!generalOptionsTID[i].id)
			break;
		globals->translate->translate(text, generalOptionsTID[i].textId);
		SetWindowText(GetDlgItem(optionshWnd, generalOptionsTID[i].id),text);			
		}


	convertConfigOriginal = convertConfigIn;
	memcpy(convertConfig,convertConfigOriginal,sizeof(CONVERTCONFIG));

	logOutput = globals->optionsLogOutput;
	contactVidomi = globals->contactVidomi;
	dualCpuEncodeBuffers = globals->dualCpuEncodeBuffers;

	configureOptionsWindow(optionshWnd);

}

// *****************************************************************************
// encoderGeneralClass::optionsDialogHandler()
//
//
// *****************************************************************************
BOOL CALLBACK optionsGeneralClass::optionsDialogHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->optionsGeneral->optionsDialogHandler1(hWnd,message,wParam,lParam));
}


// *****************************************************************************
// optionsGeneralClass::windowDialogHandler()
//
//
// *****************************************************************************
BOOL optionsGeneralClass::optionsDialogHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	unsigned short commandId;
	unsigned short commandEvent;

	switch(message)
		{

		case WM_COMMAND:
			commandId   = LOWORD(wParam); // Remember, these are...
			commandEvent = HIWORD(wParam); // ...different for Win32!

			//Parse the menu selections:

			switch (commandId) 
				{
				case VidomiOptionsOk :
					readWindowVars(hWnd);

					if(dualCpuEncodeBuffers<8)
						dualCpuEncodeBuffers = 8;

					if(dualCpuEncodeBuffers>256)
						dualCpuEncodeBuffers = 256;

					if(globals->optionsLogOutput != logOutput)
						{
						globals->optionsLogOutput = logOutput;
						clearLog();
						}
					globals->contactVidomi = contactVidomi;
					convertConfigOriginal->showOutput = convertConfig->showOutput;
					convertConfigOriginal->showStats = convertConfig->showStats;
					convertConfigOriginal->multiThread = convertConfig->multiThread;
					strcpy(&convertConfigOriginal->outputTail[0],&convertConfig->outputTail[0]);
					convertConfigOriginal->scanLength = convertConfig->scanLength;
					convertConfigOriginal->useInfoTxtFile = convertConfig->useInfoTxtFile;
					convertConfigOriginal->defaultOutputSize = convertConfig->defaultOutputSize;
					convertConfigOriginal->scanPassScale = convertConfig->scanPassScale;
					convertConfigOriginal->overwriteExistingFiles = convertConfig->overwriteExistingFiles;
					globals->dualCpuEncodeBuffers = dualCpuEncodeBuffers;
					convertConfigOriginal->enhancedAvi = convertConfig->enhancedAvi;
					convertConfigOriginal->mpegConversion = convertConfig->mpegConversion;

					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;

				case VidomiOptionsCancel :
					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;


				case VidomiOptionsResetToDefaults:
					// If we created the old structure, then free the memory
					delete [] convertConfigOriginal->videoFormatOut->lpState;
					delete [] convertConfigOriginal->videoFormatOut->lpbiOut;
					delete [] convertConfigOriginal->videoFormatOutP2->lpState;
					delete [] convertConfigOriginal->videoFormatOutP2->lpbiOut;

					configureSmartAVDefaults(convertConfigOriginal);
					memcpy(convertConfig,convertConfigOriginal,sizeof(CONVERTCONFIG));

					logOutput = false;
					
					configureOptionsWindow(hWnd);
			
					if(globals->optionsLogOutput != logOutput)
						{
						globals->optionsLogOutput = logOutput;
						clearLog();
						}

					break;

				default : 
					break;

				}
			break;
			
		case WM_CLOSE:
			DestroyWindow(hWnd);
			globals->optionsUp=false;
			break;

		}

	return (FALSE);

}




// *****************************************************************************
// optionsGeneralClass::configureOptionsWindow()
//
//
// *****************************************************************************
void optionsGeneralClass::configureOptionsWindow(HWND hWnd)
{
	char tempStr[256];
	int i;
	int ret;
	char text[4096];

	if(logOutput)
		SendDlgItemMessage(hWnd,VidomiOptionsOutputDebugfile,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsOutputDebugfile,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(contactVidomi)
		SendDlgItemMessage(hWnd,VidomiOptionsContactVidomi,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsContactVidomi,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(convertConfig->showOutput)
		SendDlgItemMessage(hWnd,VidomiOptionsShowOutput,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsShowOutput,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(convertConfig->showStats)
		SendDlgItemMessage(hWnd,VidomiOptionsShowStats,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsShowStats,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(convertConfig->multiThread)
		SendDlgItemMessage(hWnd,VidomiOptionsMultiThread,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsMultiThread,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	SendDlgItemMessage(hWnd,VidomiOptionsFileExtension, CB_RESETCONTENT , 0, 0);

	for(i=0;i<(sizeof(extensionTypes)/4);i++)
		{
		ret = SendDlgItemMessage(hWnd,VidomiOptionsFileExtension, CB_ADDSTRING , (WPARAM)0 , (LPARAM) extensionTypes[i]); 									
		SendDlgItemMessage(hWnd,VidomiOptionsFileExtension, CB_SETITEMDATA  , ret, i);
		if(!strcmp(extensionTypes[i],&convertConfig->outputTail[0]))
			SendDlgItemMessage(hWnd,VidomiOptionsFileExtension, CB_SETCURSEL  , ret, 0);
		}

	sprintf(&tempStr[0],"%d",convertConfig->scanLength);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsScanLength),&tempStr[0]);

	if(convertConfig->useInfoTxtFile)
		SendDlgItemMessage(hWnd,VidomiOptionsUseInfoTxtFile,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsUseInfoTxtFile,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);


	SendDlgItemMessage(hWnd,VidomiOptionsDefaultOutputSize, CB_RESETCONTENT , 0, 0);

	for(i=0;i<(sizeof(outputSizeText)/4);i++)
		{
		globals->translate->translate(text, outputSizeText[i]);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsDefaultOutputSize, CB_ADDSTRING , (WPARAM)0 , (LPARAM)text); 									
		SendDlgItemMessage(hWnd,VidomiOptionsDefaultOutputSize, CB_SETITEMDATA  , ret, i);
		if(i==convertConfig->defaultOutputSize)
			SendDlgItemMessage(hWnd,VidomiOptionsDefaultOutputSize, CB_SETCURSEL  , ret, 0);
		}

	if(convertConfig->scanPassScale)
		SendDlgItemMessage(hWnd,VidomiOptionsScanPassScale,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsScanPassScale,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(convertConfig->overwriteExistingFiles)
		SendDlgItemMessage(hWnd,VidomiOptionsOverwriteExistingFiles,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsOverwriteExistingFiles,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	sprintf(&tempStr[0],"%d",dualCpuEncodeBuffers);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsDualCpuEncodeBuffers),&tempStr[0]);

	if(convertConfig->enhancedAvi)
		SendDlgItemMessage(hWnd,VidomiOptionsEnhancedAVI,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsEnhancedAVI,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(convertConfig->mpegConversion)
		SendDlgItemMessage(hWnd,VidomiOptionsMpegConversion,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsMpegConversion,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

}

// *****************************************************************************
// optionsGeneralClass::readWindowVars()
//
//
// *****************************************************************************
void optionsGeneralClass::readWindowVars(HWND hWnd)
{
	char tempStr[256];
	int ret;

	if(SendDlgItemMessage(hWnd,VidomiOptionsOutputDebugfile,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		logOutput=true;
	else
		logOutput=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsContactVidomi,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		contactVidomi=true;
	else
		contactVidomi=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsShowOutput,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->showOutput=true;
	else
		convertConfig->showOutput=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsShowStats,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->showStats=true;
	else
		convertConfig->showStats=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsMultiThread,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->multiThread=true;
	else
		convertConfig->multiThread=false;

	ret = SendDlgItemMessage(hWnd,VidomiOptionsFileExtension, CB_GETCURSEL  , 0, 0);
	ret = SendDlgItemMessage(hWnd,VidomiOptionsFileExtension, CB_GETITEMDATA , ret, 0);
	strcpy(&convertConfig->outputTail[0],extensionTypes[ret]);

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsScanLength), &tempStr[0] , 256);
 	convertConfig->scanLength = atoi(&tempStr[0]);

	if(SendDlgItemMessage(hWnd,VidomiOptionsUseInfoTxtFile,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->useInfoTxtFile=true;
	else
		convertConfig->useInfoTxtFile=false;

	ret = SendDlgItemMessage(hWnd,VidomiOptionsDefaultOutputSize, CB_GETCURSEL  , 0, 0);
	ret = SendDlgItemMessage(hWnd,VidomiOptionsDefaultOutputSize, CB_GETITEMDATA , ret, 0);
	convertConfig->defaultOutputSize = ret;

	if(SendDlgItemMessage(hWnd,VidomiOptionsScanPassScale,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->scanPassScale=true;
	else
		convertConfig->scanPassScale=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsOverwriteExistingFiles,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->overwriteExistingFiles=true;
	else
		convertConfig->overwriteExistingFiles=false;

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsDualCpuEncodeBuffers), &tempStr[0] , 256);
 	dualCpuEncodeBuffers = atoi(&tempStr[0]);

	if(SendDlgItemMessage(hWnd,VidomiOptionsEnhancedAVI,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->enhancedAvi=true;
	else
		convertConfig->enhancedAvi=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsMpegConversion,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->mpegConversion=true;
	else
		convertConfig->mpegConversion=false;

	
}