// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// optionssubtitle.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

static TRANSLATEIDTABLE subtitlesOptionsTID[] = {
										{"VidomiOptionsCancel",VidomiOptionsCancel},
										{"VidomiOptionsOk",VidomiOptionsOk},
										{"VidomiSubtitleOptionsText1",VidomiSubtitleOptionsText1},
										{"VidomiSubtitleOptionsText2",VidomiSubtitleOptionsText2},
										{"VidomiSubtitleOptionsText3",VidomiSubtitleOptionsText3},
										{"VidomiOptionsSubtitleChannelType1Text",VidomiOptionsSubtitleChannelType1Text},
										{"VidomiOptionsSubtitleChannelType2Text",VidomiOptionsSubtitleChannelType2Text},
										{"VidomiOptionsSubtitleTrackText",VidomiOptionsSubtitleTrackText},
										{"VidomiOptionsSubtitleOnlyEvenLinesText",VidomiOptionsSubtitleOnlyEvenLinesText},
										{"VidomiOptionsSubtitleChannelType3Text",VidomiOptionsSubtitleChannelType3Text},
										{"VidomiOptionsSubtitleTextFilenameText",VidomiOptionsSubtitleTextFilenameText},
										{"VidomiOptionsSubtitleTextBrowse",VidomiOptionsSubtitleTextBrowse},
										{"VidomiSubtitleOptionsText4",VidomiSubtitleOptionsText4},
										{"VidomiOptionsSubtitleColorBackground",VidomiOptionsSubtitleColorBackground},
										{"VidomiOptionsSubtitleColorFont",VidomiOptionsSubtitleColorFont},
										{"VidomiOptionsSubtitleColorOutline",VidomiOptionsSubtitleColorOutline},
										{"VidomiOptionsSubtitleColorAntialiasing",VidomiOptionsSubtitleColorAntialiasing},
										
										{"",0}

										};

// *****************************************************************************
// optionsSubtitleClass::optionsSubtitleClass()
//
// conversion class constructor
// *****************************************************************************
optionsSubtitleClass::optionsSubtitleClass()
{
	convertConfig = new CONVERTCONFIG [1];
	customColorData = new COLORREF [16];
	memset(customColorData,0,sizeof(COLORREF)*16);
	renderBoxes = false;

}

// *****************************************************************************
// optionsSubtitleClass::~optionsSubtitleClass()
//
// conversion class destructor
// *****************************************************************************
optionsSubtitleClass::~optionsSubtitleClass()
{
	delete [] convertConfig;
	delete [] customColorData;

}

// *****************************************************************************
// optionsSubtitleClass::bringUpOptions()
//
// Bring up the options screen
// *****************************************************************************
void optionsSubtitleClass::bringUpOptions(HINSTANCE hInst, CONVERTCONFIG * convertConfigIn, GATHERRESULTS * gatherResults)
{
	int i;
	char text[4096];

	renderBoxes = false;

	optionshWnd = CreateDialog(hInst,MAKEINTRESOURCE(VidomiSubtitleOptions),globals->hWnd, optionsDialogHandler);
	

	globals->translate->translate(text, "VidomiSubtitleOptions");
	SetWindowText(optionshWnd, text);			

	for(i=0;;i++)
		{
		if(!subtitlesOptionsTID[i].id)
			break;
		globals->translate->translate(text, subtitlesOptionsTID[i].textId);
		SetWindowText(GetDlgItem(optionshWnd, subtitlesOptionsTID[i].id),text);			
		}


	scanData = gatherResults;

	convertConfigOriginal = convertConfigIn;
	memcpy(convertConfig,convertConfigOriginal,sizeof(CONVERTCONFIG));
	
	currentSubtitleChannel = 0;

	alpha[0]=true;

	for(i=1;i<4;i++)
		{
		if(convertConfig->subtitleColorData[i]&0xff000000)
			alpha[i]=true;
		else
			alpha[i]=false;
		}


	configureOptionsWindow(optionshWnd);
	
	renderBoxes = true;
	InvalidateRect(optionshWnd,NULL,FALSE);
	UpdateWindow(optionshWnd);

}

// *****************************************************************************
// encoderVideoClass::optionsDialogHandler()
//
//
// *****************************************************************************
BOOL CALLBACK optionsSubtitleClass::optionsDialogHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->optionsSubtitle->optionsDialogHandler1(hWnd,message,wParam,lParam));
}

// *****************************************************************************
// optionsSubtitleClass::windowDialogHandler()
//
//
// *****************************************************************************
BOOL optionsSubtitleClass::optionsDialogHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	unsigned short commandId;
	unsigned short commandEvent;
	int channel;
	int ret;
	int foundHighest;
	CHOOSECOLOR cc;
	RECT boxPos;
	RECT fill;
	POINT xy;
	LOGBRUSH cb;
	HBRUSH drawBrush;
	HDC hDC;
	int i;
	unsigned int colorVal;

	switch(message)
		{

		case WM_COMMAND:
			commandId   = LOWORD(wParam); // Remember, these are...
			commandEvent = HIWORD(wParam); // ...different for Win32!

			//Parse the menu selections:

			switch (commandId) 
				{
				case VidomiOptionsOk :
					readWindowVars(hWnd);

					for(i=0;i<4;i++)
						{
						colorVal = convertConfig->subtitleColorData[i];
						if(alpha[i])
							{
							colorVal |= 0xff000000;
							}
						else
							{
							colorVal &= 0x00ffffff;
							}
						convertConfigOriginal->subtitleColorData[i] =  colorVal;
						}

					for(channel=0;channel<MAX_SUBTITLE_CHANNELS;channel++)
						{
						convertConfigOriginal->processSubtitleMode[channel] = convertConfig->processSubtitleMode[channel]; 
						convertConfigOriginal->processSubtitleOnlyEven[channel] = convertConfig->processSubtitleOnlyEven[channel]; 
						convertConfigOriginal->processSubtitleSelected[channel] = convertConfig->processSubtitleSelected[channel]; 
						strcpy(convertConfigOriginal->subtitleStreamName[channel],convertConfig->subtitleStreamName[channel]);
						}

					// Compact the audio channels into a group
					foundHighest = 0;
					for(channel=0;channel<MAX_SUBTITLE_CHANNELS;channel++)
						{
						if(!foundHighest)
							{
							if(convertConfigOriginal->processSubtitleMode[channel]==-1)
								{
								foundHighest = channel;
								}
							}
						else
							{
							if(convertConfigOriginal->processSubtitleMode[channel]!=-1)
								{
								convertConfigOriginal->processSubtitleMode[foundHighest] = convertConfigOriginal->processSubtitleMode[channel]; 
								convertConfigOriginal->processSubtitleOnlyEven[foundHighest] = convertConfigOriginal->processSubtitleOnlyEven[channel]; 
								convertConfigOriginal->processSubtitleSelected[foundHighest] = convertConfigOriginal->processSubtitleSelected[channel]; 
								strcpy(convertConfigOriginal->subtitleStreamName[foundHighest],convertConfigOriginal->subtitleStreamName[channel]);
								foundHighest++;
								convertConfigOriginal->processSubtitleMode[channel] = -1;
								}
							}
						}

					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;

				case VidomiOptionsSubtitleChannelType1:
				case VidomiOptionsSubtitleChannelType1Text:
					if(commandEvent==BN_CLICKED)
						{
						GetWindowText(GetDlgItem(hWnd, VidomiOptionsSubtitleChannelName),convertConfig->subtitleStreamName[currentSubtitleChannel],64);
						convertConfig->processSubtitleMode[currentSubtitleChannel] = -1;
						buildSubtitleOptions(hWnd);
						}
					break;

				case VidomiOptionsSubtitleChannelType2:
				case VidomiOptionsSubtitleChannelType2Text:
					if(commandEvent==BN_CLICKED)
						{
						GetWindowText(GetDlgItem(hWnd, VidomiOptionsSubtitleChannelName),convertConfig->subtitleStreamName[currentSubtitleChannel],64);
						convertConfig->processSubtitleMode[currentSubtitleChannel] = SUBTITLE_MODE_GRAPHIC;
						buildSubtitleOptions(hWnd);
						}
					break;

				case VidomiOptionsSubtitleChannelType3:
				case VidomiOptionsSubtitleChannelType3Text:
					if(commandEvent==BN_CLICKED)
						{
						GetWindowText(GetDlgItem(hWnd, VidomiOptionsSubtitleChannelName),convertConfig->subtitleStreamName[currentSubtitleChannel],64);
						convertConfig->processSubtitleMode[currentSubtitleChannel] = SUBTITLE_MODE_TEXT;
						buildSubtitleOptions(hWnd);
						}
					break;

				case VidomiOptionsSubtitleOnlyEvenLines :
					if(SendDlgItemMessage(hWnd,VidomiOptionsSubtitleOnlyEvenLines,BM_GETCHECK , (WPARAM)0, (LPARAM)  0)==BST_CHECKED)
						convertConfig->processSubtitleOnlyEven[currentSubtitleChannel]=true;
					else
						convertConfig->processSubtitleOnlyEven[currentSubtitleChannel]=false;
					break;

				case VidomiOptionsSubtitleChannel:
					if(commandEvent==CBN_SELCHANGE)
						{
						GetWindowText(GetDlgItem(hWnd, VidomiOptionsSubtitleChannelName),convertConfig->subtitleStreamName[currentSubtitleChannel],64);

						ret = SendDlgItemMessage(hWnd,VidomiOptionsSubtitleTrack, CB_GETCURSEL  , 0, 0);
						ret = SendDlgItemMessage(hWnd,VidomiOptionsSubtitleTrack, CB_GETITEMDATA , ret, 0);
						convertConfig->processSubtitleSelected[currentSubtitleChannel] = ret;

						//Update the channel number
						ret = SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannel, CB_GETCURSEL  , 0, 0);
						currentSubtitleChannel = SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannel, CB_GETITEMDATA , ret, 0);
						
						buildSubtitleOptions(hWnd);

						}
					break;

				case VidomiOptionsSubtitleColorBackground:
					cc.lStructSize = sizeof(CHOOSECOLOR);
					cc.hwndOwner = hWnd;
					cc.hInstance = (HWND) globals->hInst;
					cc.rgbResult = (COLORREF) 0x00808080;
					cc.lpCustColors = globals->optionsSubtitle->customColorData;
					cc.Flags = CC_RGBINIT  | CC_ANYCOLOR | CC_FULLOPEN;
					ChooseColor(&cc);
					convertConfig->subtitleColorData[0] = (unsigned int) cc.rgbResult;
					InvalidateRect(optionshWnd,NULL,FALSE);
					UpdateWindow(optionshWnd);
					break;

				case VidomiOptionsSubtitleColorFont:
					cc.lStructSize = sizeof(CHOOSECOLOR);
					cc.hwndOwner = hWnd;
					cc.hInstance = (HWND) globals->hInst;
					cc.rgbResult = (COLORREF) 0x00808080;
					cc.lpCustColors = globals->optionsSubtitle->customColorData;
					cc.Flags = CC_RGBINIT  | CC_ANYCOLOR | CC_FULLOPEN;
					ChooseColor(&cc);
					convertConfig->subtitleColorData[1] = (unsigned int) cc.rgbResult;
					InvalidateRect(optionshWnd,NULL,FALSE);
					UpdateWindow(optionshWnd);
					break;

				case VidomiOptionsSubtitleColorOutline:
					cc.lStructSize = sizeof(CHOOSECOLOR);
					cc.hwndOwner = hWnd;
					cc.hInstance = (HWND) globals->hInst;
					cc.rgbResult = (COLORREF) 0x00808080;
					cc.lpCustColors = globals->optionsSubtitle->customColorData;
					cc.Flags = CC_RGBINIT  | CC_ANYCOLOR | CC_FULLOPEN;
					ChooseColor(&cc);
					convertConfig->subtitleColorData[2] = (unsigned int) cc.rgbResult;
					InvalidateRect(optionshWnd,NULL,FALSE);
					UpdateWindow(optionshWnd);
					break;

				case VidomiOptionsSubtitleColorAntialiasing:
					cc.lStructSize = sizeof(CHOOSECOLOR);
					cc.hwndOwner = hWnd;
					cc.hInstance = (HWND) globals->hInst;
					cc.rgbResult = (COLORREF) 0x00808080;
					cc.lpCustColors = globals->optionsSubtitle->customColorData;
					cc.Flags = CC_RGBINIT  | CC_ANYCOLOR | CC_FULLOPEN;
					ChooseColor(&cc);
					convertConfig->subtitleColorData[3] = (unsigned int) cc.rgbResult;
					InvalidateRect(optionshWnd,NULL,FALSE);
					UpdateWindow(optionshWnd);
					break;

				case VidomiOptionsCancel :
					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;
				}
			break;
			
		case WM_PAINT:

			if(renderBoxes)
				{
				xy.y = 0;
				xy.x = 0;
				ClientToScreen(optionshWnd,&xy);
				
				// Draw Color Box
				GetWindowRect(GetDlgItem(optionshWnd,VidomiOptionsSubtitleColorBackgroundVal), &boxPos);

				fill.top = boxPos.top - xy.y;
				fill.left = boxPos.left - xy.x;
				fill.bottom = boxPos.bottom - xy.y;
				fill.right = boxPos.right - xy.x;
				
				cb.lbStyle = BS_SOLID;
				cb.lbColor = (COLORREF) convertConfig->subtitleColorData[0] &0x00ffffff;
				drawBrush = CreateBrushIndirect(&cb);

				hDC = GetDC(optionshWnd);
				FillRect(hDC,&fill,(HBRUSH) drawBrush);
				FrameRect(hDC,&fill,(HBRUSH) GetStockObject(BLACK_BRUSH));
				ReleaseDC(optionshWnd,hDC);
				DeleteObject(drawBrush);

				// Draw Color Box
				GetWindowRect(GetDlgItem(optionshWnd,VidomiOptionsSubtitleColorFontVal), &boxPos);

				fill.top = boxPos.top - xy.y;
				fill.left = boxPos.left - xy.x;
				fill.bottom = boxPos.bottom - xy.y;
				fill.right = boxPos.right - xy.x;
				
				cb.lbStyle = BS_SOLID;
				cb.lbColor = (COLORREF) convertConfig->subtitleColorData[1]&0x00ffffff;
				drawBrush = CreateBrushIndirect(&cb);

				hDC = GetDC(optionshWnd);
				FillRect(hDC,&fill,(HBRUSH) drawBrush);
				FrameRect(hDC,&fill,(HBRUSH) GetStockObject(BLACK_BRUSH));
				ReleaseDC(optionshWnd,hDC);
				DeleteObject(drawBrush);

				// Draw Color Box
				GetWindowRect(GetDlgItem(optionshWnd,VidomiOptionsSubtitleColorOutlineVal), &boxPos);

				fill.top = boxPos.top - xy.y;
				fill.left = boxPos.left - xy.x;
				fill.bottom = boxPos.bottom - xy.y;
				fill.right = boxPos.right - xy.x;
				
				cb.lbStyle = BS_SOLID;
				cb.lbColor = (COLORREF) convertConfig->subtitleColorData[2]&0x00ffffff;
				drawBrush = CreateBrushIndirect(&cb);

				hDC = GetDC(optionshWnd);
				FillRect(hDC,&fill,(HBRUSH) drawBrush);
				FrameRect(hDC,&fill,(HBRUSH) GetStockObject(BLACK_BRUSH));
				ReleaseDC(optionshWnd,hDC);
				DeleteObject(drawBrush);

				// Draw Color Box
				GetWindowRect(GetDlgItem(optionshWnd,VidomiOptionsSubtitleColorAntialiasVal), &boxPos);

				fill.top = boxPos.top - xy.y;
				fill.left = boxPos.left - xy.x;
				fill.bottom = boxPos.bottom - xy.y;
				fill.right = boxPos.right - xy.x;
				
				cb.lbStyle = BS_SOLID;
				cb.lbColor = (COLORREF) convertConfig->subtitleColorData[3]&0x00ffffff;
				drawBrush = CreateBrushIndirect(&cb);

				hDC = GetDC(optionshWnd);
				FillRect(hDC,&fill,(HBRUSH) drawBrush);
				FrameRect(hDC,&fill,(HBRUSH) GetStockObject(BLACK_BRUSH));
				ReleaseDC(optionshWnd,hDC);
				DeleteObject(drawBrush);
				}
			break;

		case WM_CLOSE:
			DestroyWindow(hWnd);
			globals->optionsUp=false;
			break;

		}

	return (FALSE);

}



// *****************************************************************************
// optionsSubtitleClass::configureOptionsWindow()
//
//
// *****************************************************************************
void optionsSubtitleClass::configureOptionsWindow(HWND hWnd)
{
	char tempStr[256];
	int ret;
	int i;

	SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannel, CB_RESETCONTENT , 0, 0);

	for(i=0;i<MAX_SUBTITLE_CHANNELS;i++)
		{
		sprintf(tempStr,"%d",i+1);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannel, CB_ADDSTRING , (WPARAM)0 , (LPARAM) tempStr); 									
		SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannel, CB_SETITEMDATA  , ret, i);
		if(i==currentSubtitleChannel)
			SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannel, CB_SETCURSEL  , ret, 0);
		}

	buildSubtitleOptions(hWnd);

	SendDlgItemMessage(hWnd,VidomiOptionsSubtitleAlphaBackground,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);

	if(alpha[1])
		SendDlgItemMessage(hWnd,VidomiOptionsSubtitleAlphaFont,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsSubtitleAlphaFont,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(alpha[2])
		SendDlgItemMessage(hWnd,VidomiOptionsSubtitleAlphaOutline,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsSubtitleAlphaOutline,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(alpha[3])
		SendDlgItemMessage(hWnd,VidomiOptionsSubtitleAlphaAntialiasing,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsSubtitleAlphaAntialiasing,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

}

// *****************************************************************************
// optionsSubtitleClass::readWindowVars()
//
//
// *****************************************************************************
void optionsSubtitleClass::readWindowVars(HWND hWnd)
{
	int ret;

	ret = SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannel, CB_GETCURSEL  , 0, 0);
	currentSubtitleChannel = SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannel, CB_GETITEMDATA , ret, 0);

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsSubtitleChannelName),convertConfig->subtitleStreamName[currentSubtitleChannel],64);

	ret = SendDlgItemMessage(hWnd,VidomiOptionsSubtitleTrack, CB_GETCURSEL  , 0, 0);
	ret = SendDlgItemMessage(hWnd,VidomiOptionsSubtitleTrack, CB_GETITEMDATA , ret, 0);
	convertConfig->processSubtitleSelected[currentSubtitleChannel] = ret;


	alpha[0]=true;

	if(SendDlgItemMessage(hWnd,VidomiOptionsSubtitleAlphaFont,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		alpha[1]=true;
	else
		alpha[1]=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsSubtitleAlphaOutline,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		alpha[2]=true;
	else
		alpha[2]=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsSubtitleAlphaAntialiasing,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		alpha[3]=true;
	else
		alpha[3]=false;
	
}

// *****************************************************************************
// optionsSubtitleClass::buildSubtitleOptions()
//
//
// *****************************************************************************
void optionsSubtitleClass::buildSubtitleOptions(HWND hWnd)
{
	int i;
	int ret;
	int state;

	SetWindowText(GetDlgItem(hWnd, VidomiOptionsSubtitleChannelName),convertConfig->subtitleStreamName[currentSubtitleChannel]);

	switch(convertConfig->processSubtitleMode[currentSubtitleChannel])
		{
		case SUBTITLE_MODE_DISABLED:
			// Set radio button
			SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannelType1, BM_SETCHECK  ,BST_CHECKED, 0);
			SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannelType2, BM_SETCHECK  ,BST_UNCHECKED, 0);
			SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannelType3, BM_SETCHECK  ,BST_UNCHECKED, 0);
			state = false;
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTrackText), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTrack), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTrack), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleOnlyEvenLines), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleOnlyEvenLinesText), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTextFilenameText), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTextFilename), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTextBrowse), state);

			break;
		case SUBTITLE_MODE_GRAPHIC:
			// Set radio button
			SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannelType1, BM_SETCHECK  ,BST_UNCHECKED, 0);
			SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannelType2, BM_SETCHECK  ,BST_CHECKED, 0);
			SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannelType3, BM_SETCHECK  ,BST_UNCHECKED, 0);
			state = true;
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTrackText), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTrack), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTrack), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleOnlyEvenLines), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleOnlyEvenLinesText), state);
			state=false;
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTextFilenameText), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTextFilename), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTextBrowse), state);
			break;
		case SUBTITLE_MODE_TEXT:
			// Set radio button
			SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannelType1, BM_SETCHECK  ,BST_UNCHECKED, 0);
			SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannelType2, BM_SETCHECK  ,BST_UNCHECKED, 0);
			SendDlgItemMessage(hWnd,VidomiOptionsSubtitleChannelType3, BM_SETCHECK  ,BST_CHECKED, 0);

			state = false;
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTrackText), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTrack), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTrack), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleOnlyEvenLines), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleOnlyEvenLinesText), state);
			state=true;
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTextFilenameText), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTextFilename), state);
			EnableWindow(GetDlgItem(hWnd,VidomiOptionsSubtitleTextBrowse), state);
			break;
		}

	if(convertConfig->processSubtitleOnlyEven[currentSubtitleChannel])
		{
		SendDlgItemMessage(hWnd,VidomiOptionsSubtitleOnlyEvenLines,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
		}
	else
		{
		SendDlgItemMessage(hWnd,VidomiOptionsSubtitleOnlyEvenLines,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);
		}

	SendDlgItemMessage(hWnd,VidomiOptionsSubtitleTrack, CB_RESETCONTENT , 0, 0);

	if(scanData->subtitleFound)
		{
		for(i=0;i<scanData->subtitleFound;i++)
			{
			ret = SendDlgItemMessage(hWnd,VidomiOptionsSubtitleTrack, CB_ADDSTRING , (WPARAM)0 , (LPARAM) &globals->gatherResults->subtitleName[i]); 									
			SendDlgItemMessage(hWnd,VidomiOptionsSubtitleTrack, CB_SETITEMDATA  , ret, i);
			if(i==convertConfig->processSubtitleSelected[currentSubtitleChannel])
				SendDlgItemMessage(hWnd,VidomiOptionsSubtitleTrack, CB_SETCURSEL  , ret, 0);
			}
		}
	
}


