// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// options.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

static char * scaleType[] = {"ScaleTypeNone","ScaleTypeBilinear","ScaleTypeHQBilinear","ScaleTypeHQBicubic"};
static char * scaleBy[] = {"ScaleByNoScale","ScaleByPixels","ScaleByPercent"};
static char * encodeMode[] = {"EncodeModeNormal","EncodeModeHybrid1","EncodeModeVbr","EncodeModeHybrid2", 
								"EncodeModeDivxNetworks1p", "EncodeModeDivxNetworksQuality", "EncodeModeDivxNetworks2p", 
								"EncodeModeHybrid2TimeBased", "EncodeModeVbrTimeBased","EncodeModeDivxNetworksVidomiVbr",
								"EncodeModeXvidVbr","EncodeModeNormal2pass","EncodeModeXvidHybrid"};
static char * encodeIvtcFilmMode[] = {"EncodeIvtcFilmModeNone","EncodeIvtcFilmModeFilm","EncodeIvtcFilmModeAdaptiveIvtc","EncodeIvtcFilmModeManualIvtc", "EncodeIvtcFilmModeManualIvtcBF"};
static char * encodeColorSpace[] = {"EncodeColorSpaceRgb32","EncodeColorSpaceYV12"};

						
static TRANSLATEIDTABLE videoOptionsTID[] = {
										{"VidomiVideoOptionsText1",VidomiVideoOptionsText1},
										{"VidomiOptionsHyperCropThresholdText",VidomiOptionsHyperCropThresholdText},
										{"VidomiVideoOptionsText2",VidomiVideoOptionsText2},
										{"VidomiVideoOptionsText3",VidomiVideoOptionsText3},
										{"VidomiOptionsCropTopText",VidomiOptionsCropTopText},
										{"VidomiOptionsCropBottomText",VidomiOptionsCropBottomText},
										{"VidomiVideoOptionsText4",VidomiVideoOptionsText4},
										{"VidomiOptionsDeinterlaceText",VidomiOptionsDeinterlaceText},
										{"VidomiOptionsBlendText",VidomiOptionsBlendText},
										{"VidomiOptionsBlendThresholdText",VidomiOptionsBlendThresholdText},
										{"VidomiOptionsIVTCOffsetText",VidomiOptionsIVTCOffsetText},
										{"VidomiOptionsIVTCInvertPolarityText",VidomiOptionsIVTCInvertPolarityText},
										{"VidomiOptionsForceMultiple16Text",VidomiOptionsForceMultiple16Text},
										{"VidomiVideoOptionsText5",VidomiVideoOptionsText5},
										{"VidomiOptionsScaleWidthText",VidomiOptionsScaleWidthText},
										{"VidomiOptionsScaleHeightText",VidomiOptionsScaleHeightText},
										{"VidomiOptionsScaleTypeText",VidomiOptionsScaleTypeText},
										{"VidomiVideoOptionsText6",VidomiVideoOptionsText6},
										{"VidomiVideoOptionsText10",VidomiVideoOptionsText10},
										{"VidomiOptionsCancel",VidomiOptionsCancel},
										{"VidomiOptionsOk",VidomiOptionsOk},
										{"VidomiOptionsSelectVideoCodec",VidomiOptionsSelectVideoCodec},
										{"VidomiOptionsSelectVideoCodec2P2",VidomiOptionsSelectVideoCodec2},
										{"",0}
																			
										};

// *****************************************************************************
// optionsVideoClass::optionsVideoClass()
//
// conversion class constructor
// *****************************************************************************
optionsVideoClass::optionsVideoClass()
{
	convertConfig = new CONVERTCONFIG [1];

}

// *****************************************************************************
// optionsVideoClass::~optionsVideoClass()
//
// conversion class destructor
// *****************************************************************************
optionsVideoClass::~optionsVideoClass()
{
	delete [] convertConfig;
}

// *****************************************************************************
// optionsVideoClass::bringUpOptions()
//
// Bring up the options screen
// *****************************************************************************
void optionsVideoClass::bringUpOptions(HINSTANCE hInst, CONVERTCONFIG * convertConfigIn, GATHERRESULTS * gatherResults)
{
	char text[4096];
	int i;

	optionshWnd = CreateDialog(hInst,MAKEINTRESOURCE(VidomiVideoOptions),globals->hWnd, optionsDialogHandler);

	globals->translate->translate(text, "VidomiVideoOptions");
	SetWindowText(optionshWnd, text);			

	for(i=0;;i++)
		{
		if(!videoOptionsTID[i].id)
			break;
		globals->translate->translate(text, videoOptionsTID[i].textId);
		SetWindowText(GetDlgItem(optionshWnd, videoOptionsTID[i].id),text);			
		}

	scanData = gatherResults;

	convertConfigOriginal = convertConfigIn;
	memcpy(convertConfig,convertConfigOriginal,sizeof(CONVERTCONFIG));
	
	convertConfig->videoFormatOut = new COMPVARS [1];
	memcpy(convertConfig->videoFormatOut, convertConfigOriginal->videoFormatOut, sizeof(COMPVARS));
	convertConfig->videoFormatOut->lpbiOut = NULL;
	convertConfig->videoFormatOut->lpState = NULL;

	convertConfig->videoFormatOutP2 = new COMPVARS [1];
	memcpy(convertConfig->videoFormatOutP2, convertConfigOriginal->videoFormatOutP2, sizeof(COMPVARS));
	convertConfig->videoFormatOutP2->lpbiOut = NULL;
	convertConfig->videoFormatOutP2->lpState = NULL;

	convertConfig->audioFormatOut  = (MPEGLAYER3WAVEFORMAT *) new char [AUDIO_STATE_DEFAULT_SIZE];
	memcpy(convertConfig->audioFormatOut,convertConfigOriginal->audioFormatOut,AUDIO_STATE_DEFAULT_SIZE);

	videoFormatChanged=false;
	videoFormatChangedP2=false;
	configureOptionsWindow(optionshWnd);

}

// *****************************************************************************
// encoderVideoClass::optionsDialogHandler()
//
//
// *****************************************************************************
BOOL CALLBACK optionsVideoClass::optionsDialogHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->optionsVideo->optionsDialogHandler1(hWnd,message,wParam,lParam));
}
// *****************************************************************************
// optionsVideoClass::optionsDialogHandler1()
//
//
// *****************************************************************************
BOOL optionsVideoClass::optionsDialogHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	unsigned short commandId;
	unsigned short commandEvent;
	int ret;
	int state;
	bool valid;
	switch(message)
		{

		case WM_COMMAND:
			commandId   = LOWORD(wParam); // Remember, these are...
			commandEvent = HIWORD(wParam); // ...different for Win32!

			//Parse the menu selections:

			switch (commandId) 
				{
				case VidomiOptionsOk :
					readWindowVars(hWnd);
					valid = true;
					switch(convertConfig->encodeColorSpace)
						{
						case rgb32Frame:
							if((convertConfig->cropLeft&0x01) || (convertConfig->cropRight&0x01))
								{
								// Left and Right crop settings are incorrect
								char messTitle[4096];
								char messText[4096];
								globals->translate->translate(messText, "VidomiMessage24");
								globals->translate->translate(messTitle, "VidomiMessage24Title");

								vidomiErrorBox(globals->hWnd,messText,messTitle,MB_ICONERROR | MB_OK);
								convertConfig->cropLeft=convertConfig->cropLeft&0xfffffffe;
								convertConfig->cropRight=convertConfig->cropRight&0xfffffffe;
								valid = false;		
								}
							if(convertConfig->scaleBy==SCALE_BY_PIXELS)
								{
								if(convertConfig->scaleWidth&0x01)
									{
									// Scale Width must be an even number
									char messTitle[4096];
									char messText[4096];
									globals->translate->translate(messText, "VidomiMessage25");
									globals->translate->translate(messTitle, "VidomiMessage25Title");

									vidomiErrorBox(globals->hWnd,messText,messTitle,MB_ICONERROR | MB_OK);
									convertConfig->scaleWidth = convertConfig->scaleWidth&0xfffffffe;
									valid = false;		
									}
								}
							break;
						
						case yv12Frame:
							if((convertConfig->cropLeft&0x07) || (convertConfig->cropRight&0x07))
								{
								char messTitle[4096];
								char messText[4096];
								globals->translate->translate(messText, "VidomiMessage26");
								globals->translate->translate(messTitle, "VidomiMessage26Title");

								vidomiErrorBox(globals->hWnd,messText,messTitle,MB_ICONERROR | MB_OK);
								convertConfig->cropLeft=(convertConfig->cropLeft+7)&0xfffffff8;
								convertConfig->cropRight=(convertConfig->cropRight+7)&0xfffffff8;
								// Left and Right crop settings are incorrect
								valid = false;		
								}
							if((convertConfig->cropTop&0x07) || (convertConfig->cropBottom&0x07))
								{
								char messTitle[4096];
								char messText[4096];
								globals->translate->translate(messText, "VidomiMessage27");
								globals->translate->translate(messTitle, "VidomiMessage27Title");

								vidomiErrorBox(globals->hWnd,messText,messTitle,MB_ICONERROR | MB_OK);
								convertConfig->cropTop=(convertConfig->cropTop+7)&0xfffffff8;
								convertConfig->cropBottom=(convertConfig->cropBottom+7)&0xfffffff8;
								// Top and Bottom crop settings are incorrect
								valid = false;		
								}

							if(convertConfig->scaleBy==SCALE_BY_PIXELS)
								{
								if(convertConfig->scaleWidth&0x07)
									{
									// Scale Width must be a multiple of 8
									char messTitle[4096];
									char messText[4096];
									globals->translate->translate(messText, "VidomiMessage28");
									globals->translate->translate(messTitle, "VidomiMessage28Title");

									vidomiErrorBox(globals->hWnd,messText,messTitle,MB_ICONERROR | MB_OK);
									convertConfig->scaleWidth = convertConfig->scaleWidth&0xfffffff8;
									valid = false;		
									}
								}
							break;
						}

					if(!valid)
						{
						configureOptionsWindow(hWnd);
						break;
						}

					if(videoFormatChanged)
						{
						// If we created the old structure, then free the memory
						delete [] convertConfigOriginal->videoFormatOut->lpState;
						delete [] convertConfigOriginal->videoFormatOut->lpbiOut;

						memcpy(convertConfigOriginal->videoFormatOut,convertConfig->videoFormatOut, sizeof(COMPVARS));
						convertConfigOriginal->videoFormatOut->lpbiOut = (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
						convertConfigOriginal->videoFormatOut->lpState = new char [VIDEO_STATE_DEFAULT_SIZE]; // allow for some growth in state size. 
						if(convertConfig->videoFormatOut->lpbiOut)
							{
							memcpy(convertConfigOriginal->videoFormatOut->lpbiOut,convertConfig->videoFormatOut->lpbiOut, convertConfig->videoFormatOut->lpbiOut->bmiHeader.biSize);
							memcpy(convertConfigOriginal->videoFormatOut->lpState,convertConfig->videoFormatOut->lpState, convertConfig->videoFormatOut->cbState);
							}

						else
							{
							// User wants uncompressed output. 
							convertConfigOriginal->videoFormatOut->lpbiOut = (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
							convertConfigOriginal->videoFormatOut->lpState = new char [VIDEO_STATE_DEFAULT_SIZE]; // allow for some growth in state size. 
							memset(convertConfigOriginal->videoFormatOut->lpState,0,VIDEO_STATE_DEFAULT_SIZE);
							memset(convertConfigOriginal->videoFormatOut->lpbiOut,0,sizeof(BITMAPINFO));

							convertConfigOriginal->videoFormatOut->lpbiOut->bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
							convertConfigOriginal->videoFormatOut->lpbiOut->bmiHeader.biWidth=720;
							convertConfigOriginal->videoFormatOut->lpbiOut->bmiHeader.biHeight=480;
							convertConfigOriginal->videoFormatOut->lpbiOut->bmiHeader.biPlanes=1;
							convertConfigOriginal->videoFormatOut->lpbiOut->bmiHeader.biBitCount=32;
							convertConfigOriginal->videoFormatOut->lpbiOut->bmiHeader.biCompression=BI_RGB;
							convertConfigOriginal->videoFormatOut->lpbiOut->bmiHeader.biSizeImage=(720*480)*(32/8);
							convertConfigOriginal->videoFormatOut->lpbiOut->bmiHeader.biXPelsPerMeter=0;
							convertConfigOriginal->videoFormatOut->lpbiOut->bmiHeader.biYPelsPerMeter=0;
							convertConfigOriginal->videoFormatOut->lpbiOut->bmiHeader.biClrUsed=0;
							convertConfigOriginal->videoFormatOut->lpbiOut->bmiHeader.biClrImportant=0;
							}

						// Close the OS handle
						if(convertConfig->videoFormatOut->hic)
							{
							//convertConfig->videoFormatOut->hic->unused = 0;
							ICClose(convertConfig->videoFormatOut->hic);
							convertConfigOriginal->videoFormatOut->hic=NULL;						
							}
						}

					if(videoFormatChangedP2)
						{

						// Now update the second codec settings

						// If we created the old structure, then free the memory
						delete [] convertConfigOriginal->videoFormatOutP2->lpState;
						delete [] convertConfigOriginal->videoFormatOutP2->lpbiOut;

						memcpy(convertConfigOriginal->videoFormatOutP2,convertConfig->videoFormatOutP2, sizeof(COMPVARS));
						convertConfigOriginal->videoFormatOutP2->lpbiOut = (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
						convertConfigOriginal->videoFormatOutP2->lpState = new char [VIDEO_STATE_DEFAULT_SIZE]; // allow for some growth in state size. 
						if(convertConfig->videoFormatOutP2->lpbiOut)
							{
							memcpy(convertConfigOriginal->videoFormatOutP2->lpbiOut,convertConfig->videoFormatOutP2->lpbiOut, convertConfig->videoFormatOutP2->lpbiOut->bmiHeader.biSize);
							memcpy(convertConfigOriginal->videoFormatOutP2->lpState,convertConfig->videoFormatOutP2->lpState, convertConfig->videoFormatOutP2->cbState);
							}

						else
							{
							// User wants uncompressed output. 
							convertConfigOriginal->videoFormatOutP2->lpbiOut = (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
							convertConfigOriginal->videoFormatOutP2->lpState = new char [VIDEO_STATE_DEFAULT_SIZE]; // allow for some growth in state size. 
							memset(convertConfigOriginal->videoFormatOutP2->lpState,0,VIDEO_STATE_DEFAULT_SIZE);
							memset(convertConfigOriginal->videoFormatOutP2->lpbiOut,0,sizeof(BITMAPINFO));

							convertConfigOriginal->videoFormatOutP2->lpbiOut->bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
							convertConfigOriginal->videoFormatOutP2->lpbiOut->bmiHeader.biWidth=720;
							convertConfigOriginal->videoFormatOutP2->lpbiOut->bmiHeader.biHeight=480;
							convertConfigOriginal->videoFormatOutP2->lpbiOut->bmiHeader.biPlanes=1;
							convertConfigOriginal->videoFormatOutP2->lpbiOut->bmiHeader.biBitCount=32;
							convertConfigOriginal->videoFormatOutP2->lpbiOut->bmiHeader.biCompression=BI_RGB;
							convertConfigOriginal->videoFormatOutP2->lpbiOut->bmiHeader.biSizeImage=(720*480)*(32/8);
							convertConfigOriginal->videoFormatOutP2->lpbiOut->bmiHeader.biXPelsPerMeter=0;
							convertConfigOriginal->videoFormatOutP2->lpbiOut->bmiHeader.biYPelsPerMeter=0;
							convertConfigOriginal->videoFormatOutP2->lpbiOut->bmiHeader.biClrUsed=0;
							convertConfigOriginal->videoFormatOutP2->lpbiOut->bmiHeader.biClrImportant=0;
							}

						// Close the OS handle
						if(convertConfig->videoFormatOutP2->hic)
							{
							//convertConfig->videoFormatOut->hic->unused = 0;
							ICClose(convertConfig->videoFormatOutP2->hic);
							convertConfigOriginal->videoFormatOutP2->hic=NULL;						
							}

						}
					convertConfigOriginal->hyperCrop = convertConfig->hyperCrop;
					convertConfigOriginal->hyperCropThreshold = convertConfig->hyperCropThreshold;
					convertConfigOriginal->cropTop = convertConfig->cropTop;
					convertConfigOriginal->cropBottom = convertConfig->cropBottom;
					convertConfigOriginal->cropLeft = convertConfig->cropLeft;			
					convertConfigOriginal->cropRight = convertConfig->cropRight;		
					convertConfigOriginal->hyperScale = convertConfig->hyperScale;
					convertConfigOriginal->scale = convertConfig->scale;
					convertConfigOriginal->scaleBy = convertConfig->scaleBy;
					convertConfigOriginal->scaleType = convertConfig->scaleType;
					convertConfigOriginal->scaleWidth = convertConfig->scaleWidth;
					convertConfigOriginal->scaleHeight = convertConfig->scaleHeight;
					convertConfigOriginal->deinterlace = convertConfig->deinterlace;
					convertConfigOriginal->blend= convertConfig->blend;
					convertConfigOriginal->blendThreshold = convertConfig->blendThreshold;
					convertConfigOriginal->film = convertConfig->film;
					convertConfigOriginal->ivtc = convertConfig->ivtc;
					convertConfigOriginal->ivtcOffset = convertConfig->ivtcOffset;
					convertConfigOriginal->ivtcDeComb = convertConfig->ivtcDeComb;
					convertConfigOriginal->forceMultiple16 = convertConfig->forceMultiple16;
					convertConfigOriginal->ivtcFilmMode = convertConfig->ivtcFilmMode;

					convertConfigOriginal->encodeMode = convertConfig->encodeMode;
					convertConfigOriginal->vbrCrispness = convertConfig->vbrCrispness;
					convertConfigOriginal->encodeColorSpace = convertConfig->encodeColorSpace;

					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;

				case VidomiOptionsIVTCFilm:
					if(commandEvent==CBN_SELCHANGE)
						{
						ret = SendDlgItemMessage(hWnd,VidomiOptionsIVTCFilm, CB_GETCURSEL  , 0, 0);
						convertConfig->ivtcFilmMode = SendDlgItemMessage(hWnd,VidomiOptionsIVTCFilm, CB_GETITEMDATA , ret, 0);
						enableWindows(hWnd);					
						}
					break;


				case VidomiOptionsEncodeMode:
					if(commandEvent==CBN_SELCHANGE)
						{
						ret = SendDlgItemMessage(hWnd,VidomiOptionsEncodeMode, CB_GETCURSEL  , 0, 0);
						convertConfig->encodeMode = SendDlgItemMessage(hWnd,VidomiOptionsEncodeMode, CB_GETITEMDATA , ret, 0);
						enableWindows(hWnd);
						}
					break;

				case VidomiOptionsHyperCrop:
					if(SendDlgItemMessage(hWnd,VidomiOptionsHyperCrop,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
						convertConfig->hyperCrop = true;
					else
						convertConfig->hyperCrop = false;
					enableWindows(hWnd);
					break;
				case VidomiOptionsHyperScale:					
				case VidomiOptionsScaleBy:

					if(SendDlgItemMessage(hWnd,VidomiOptionsHyperScale,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
						convertConfig->hyperScale = true;
					else
						convertConfig->hyperScale = false;						
					
					state = SendDlgItemMessage(hWnd,VidomiOptionsScaleBy, CB_GETCURSEL  , 0, 0);
					convertConfig->scaleBy = SendDlgItemMessage(hWnd,VidomiOptionsScaleBy, CB_GETCURSEL  , state, 0);

					enableWindows(hWnd);

					break;

				case VidomiOptionsEncodeColorSpace:
					if(commandEvent==CBN_SELCHANGE)
						{
						ret = SendDlgItemMessage(hWnd,VidomiOptionsEncodeColorSpace, CB_GETCURSEL  , 0, 0);
						convertConfig->encodeColorSpace = SendDlgItemMessage(hWnd,VidomiOptionsEncodeColorSpace, CB_GETITEMDATA , ret, 0);
						enableWindows(hWnd);
						}
					break;

				case VidomiOptionsCancel :
					if(videoFormatChanged)
						{
						//convertConfig->videoFormatOut->hic->unused = 0;
						ICClose(convertConfig->videoFormatOut->hic);
						}
					if(videoFormatChangedP2)
						{
						ICClose(convertConfig->videoFormatOutP2->hic);					
						}
					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;

				case VidomiOptionsSelectVideoCodec :
					setVideoOutput();
					break;

				case VidomiOptionsSelectVideoCodec2:
					setVideoOutputP2();
					break;

				default : 
					break;

				}
			break;
			
		case WM_CLOSE:
			DestroyWindow(hWnd);
			globals->optionsUp=false;
			break;

		}

	return (FALSE);

}
// *****************************************************************************
// optionsVideoClass::setVideoOutputP2()
//
// returns TRUE if user selects compressor and presses OK. False on error
// *****************************************************************************
bool optionsVideoClass::setVideoOutputP2()
{
	int width;
	int height;

	width = 720;
	height = 480;
	memset(&bitmapFormatP2,0,sizeof(BITMAPINFOHEADER));
	bitmapFormatP2.biSize=sizeof(BITMAPINFOHEADER);
	bitmapFormatP2.biWidth=width;
	bitmapFormatP2.biHeight=height;
	bitmapFormatP2.biPlanes=1;
	bitmapFormatP2.biBitCount=24;
	bitmapFormatP2.biCompression=BI_RGB;
	bitmapFormatP2.biSizeImage=(width*height)*(32/8);
	bitmapFormatP2.biXPelsPerMeter=0;
	bitmapFormatP2.biYPelsPerMeter=0;
	bitmapFormatP2.biClrUsed=0;
	bitmapFormatP2.biClrImportant=0;

	if(convertConfig->videoFormatOutP2->hic!=NULL)
		{
		ICClose(convertConfig->videoFormatOutP2->hic);
		convertConfig->videoFormatOutP2->hic = NULL;
		}

	convertConfig->videoFormatOutP2->dwFlags =  ICMF_COMPVARS_VALID; 
	if(ICCompressorChoose(globals->hWnd,0,&bitmapFormatP2,NULL,convertConfig->videoFormatOutP2,"Select the Second Pass Video Output Codec"))
		{
		// hit OK
		videoFormatChangedP2 = true;
		return(true);
		}
	else
		{
		// hit cancel
		return(false);
		}

}

// *****************************************************************************
// optionsVideoClass::setVideoOutput()
//
// returns TRUE if user selects compressor and presses OK. False on error
// *****************************************************************************
bool optionsVideoClass::setVideoOutput()
{
	int width;
	int height;
	HIC divxNetworksHic;
	int * ptr;
	HIC xvidHic;
	int * xptr;
	char * xcptr;
	bool divxNetworksVersion5;
	long bmhSize;
	BITMAPINFOHEADER * bmfh;
	LRESULT lres;

	divxNetworksHic=NULL;
	xvidHic=NULL;


	width = 720;
	height = 480;
	memset(&bitmapFormat,0,sizeof(BITMAPINFOHEADER));
	bitmapFormat.biSize=sizeof(BITMAPINFOHEADER);
	bitmapFormat.biWidth=width;
	bitmapFormat.biHeight=height;
	bitmapFormat.biPlanes=1;
	bitmapFormat.biSizeImage=(width*height)*(32/8);
	bitmapFormat.biXPelsPerMeter=0;
	bitmapFormat.biYPelsPerMeter=0;
	bitmapFormat.biClrUsed=0;
	bitmapFormat.biClrImportant=0;

	if(convertConfig->encodeColorSpace==rgb32Frame)
		{
		bitmapFormat.biBitCount=24;
		bitmapFormat.biCompression=BI_RGB;
		}
	else
		{
		bitmapFormat.biBitCount=32;
		bitmapFormat.biCompression=mmioFOURCC('Y','V','1','2');
		}


	if(convertConfig->videoFormatOut->hic!=NULL)
		{
		ICClose(convertConfig->videoFormatOut->hic);
		convertConfig->videoFormatOut->hic = NULL;
		}

	if((convertConfig->encodeMode == ENCODE_MODE_XVIDVBR) || (convertConfig->encodeMode == ENCODE_MODE_XVIDHYBRID))
		{

		xvidHic = ICOpen(mmioFOURCC('v', 'i', 'd', 'c'), mmioFOURCC('x', 'v', 'i', 'd') ,ICMODE_COMPRESS);
		if(xvidHic)
			{
			convertConfig->videoFormatOut->cbState = ICGetStateSize(xvidHic);
			convertConfig->videoFormatOut->lpState = new unsigned char [convertConfig->videoFormatOut->cbState];
			memset(convertConfig->videoFormatOut->lpState,0,convertConfig->videoFormatOut->cbState);
			ICGetState(xvidHic,convertConfig->videoFormatOut->lpState,convertConfig->videoFormatOut->cbState);
			if(convertConfig->videoFormatOut->lpState)
				{
				xptr = (int *) convertConfig->videoFormatOut->lpState;
				xptr = (int *) (((char *)xptr) + convertConfig->xvidEncodeModeOffset);

				if(convertConfig->encodeMode == ENCODE_MODE_XVIDVBR)
					{
					*xptr = DLG_MODE_2PASS_1;
					}
				else
					{
					*xptr = DLG_MODE_CBR;
					}

				if(convertConfig->encodeMode == ENCODE_MODE_XVIDHYBRID)
					{
					xptr = (int *) convertConfig->videoFormatOut->lpState;
					xptr = (int *) (((char *)xptr) + convertConfig->xvidBitrateOffset);
					*xptr = 10000000;
					}
				xcptr = (char *) convertConfig->videoFormatOut->lpState;
				xcptr = (char *) (((char *)xcptr) + convertConfig->xvidStatsFilenameOffset);

				strcpy(xcptr,"Ignore-Set by Vidomi");
				ICSetState(xvidHic,convertConfig->videoFormatOut->lpState,convertConfig->videoFormatOut->cbState);
				}
		

			ICConfigure(xvidHic,globals->hWnd);
			convertConfig->videoFormatOut->cbState = ICGetStateSize(xvidHic);
			ICGetState(xvidHic,convertConfig->videoFormatOut->lpState,convertConfig->videoFormatOut->cbState);
			ICClose(xvidHic);
			videoFormatChanged = true;				
			}
		return(true);
		}

	if(convertConfig->encodeMode == ENCODE_MODE_CBR || convertConfig->encodeMode == ENCODE_MODE_CBR2PASS)
		{
		//memset(convertConfig->videoFormatOut,0,sizeof(COMPVARS));
		//convertConfig->videoFormatOut->cbSize=sizeof(COMPVARS);
		convertConfig->videoFormatOut->dwFlags =  ICMF_COMPVARS_VALID;
		if(ICCompressorChoose(globals->hWnd,0,&bitmapFormat,NULL,convertConfig->videoFormatOut,"Select the Video Output Codec"))
			{

			videoFormatChanged = true;
			return(true);
			}
		else
			{
			// hit cancel
			return(false);
			}

		}
	else
		{
		// It's a DIVX 4/5 codec set

		// Create an instance of our divx settings class
		// If we have a codec open, then close it
		memset(convertConfig->videoFormatOut, 0, sizeof(COMPVARS));
		convertConfig->videoFormatOut->cbSize = sizeof(COMPVARS);
		convertConfig->videoFormatOut->dwFlags = 1;
		convertConfig->videoFormatOut->lDataRate = 300; //Favor Divx 
		convertConfig->videoFormatOut->lKey = 9999; //favor Divx 
		convertConfig->videoFormatOut->fccType = mmioFOURCC('v', 'i', 'd', 'c');
		convertConfig->videoFormatOut->fccHandler = mmioFOURCC('d', 'i', 'v', 'x');

		// Set registry values for DIVX 4

		divxNetworksHic = ICOpen(mmioFOURCC('v', 'i', 'd', 'c'), mmioFOURCC('d', 'i', 'v', 'x') ,ICMODE_COMPRESS);

		if(divxNetworksHic)
			{
			convertConfig->videoFormatOut->lpbiOut = (BITMAPINFO *) new char [VIDOMI_BITMAP_INFO_SIZE];
			convertConfig->videoFormatOut->lpbiOut->bmiHeader.biSize = sizeof(BITMAPINFO);
			convertConfig->videoFormatOut->lpbiOut->bmiHeader.biWidth = 720;
			convertConfig->videoFormatOut->lpbiOut->bmiHeader.biHeight = 480;
			convertConfig->videoFormatOut->lpbiOut->bmiHeader.biPlanes = 0x0001;
			convertConfig->videoFormatOut->lpbiOut->bmiHeader.biBitCount = 0x0018;
			convertConfig->videoFormatOut->lpbiOut->bmiHeader.biSizeImage = 720 * 480 * 3;
			convertConfig->videoFormatOut->lpbiOut->bmiHeader.biXPelsPerMeter = 0;
			convertConfig->videoFormatOut->lpbiOut->bmiHeader.biYPelsPerMeter = 0;
			convertConfig->videoFormatOut->lpbiOut->bmiHeader.biClrImportant = 0;
			convertConfig->videoFormatOut->lpbiOut->bmiHeader.biClrUsed = 0;
			convertConfig->videoFormatOut->lpbiOut->bmiHeader.biCompression = mmioFOURCC('D', 'I', 'V', 'X');


			convertConfig->videoFormatOut->cbState = ICGetStateSize(divxNetworksHic);
			convertConfig->videoFormatOut->lpState = new unsigned char [convertConfig->videoFormatOut->cbState];
			memset(convertConfig->videoFormatOut->lpState,0,convertConfig->videoFormatOut->cbState);
			
			ICGetState(divxNetworksHic,convertConfig->videoFormatOut->lpState,convertConfig->videoFormatOut->cbState);

			//  query size of bmstruct
			bmhSize=ICCompressGetFormat(divxNetworksHic, &bitmapFormat,0);

			bmfh = ( BITMAPINFOHEADER *) new char [bmhSize];
			lres = ICCompressGetFormat(divxNetworksHic,&bitmapFormat,bmfh);

			if (lres != ICERR_OK)
				{
				delete [] bmfh;
				ICClose(divxNetworksHic);
				return(false);
				}

			if(bmfh->biCompression == mmioFOURCC('D', 'X', '5', '0'))
				{
				logPrintf("Using Divx 5 Encode state offsets for Video Options Screen\n");
				divxNetworksVersion5 = true;
				}
			else
				{
				logPrintf("Using Divx 4 Encode state offsets for Video Options Screen\n");
				divxNetworksVersion5 = false;
				}
			
			
			if(convertConfig->videoFormatOut->lpState)
				{
				if(divxNetworksVersion5)
					{
					ptr = (int *) convertConfig->videoFormatOut->lpState;
					ptr = (int *) (((char *)ptr) + convertConfig->divx5VbrModeOffset);
					}
				else
					{
					ptr = (int *) convertConfig->videoFormatOut->lpState;
					ptr = (int *) (((char *)ptr) + convertConfig->divx4VbrModeOffset);
					}

				switch(convertConfig->encodeMode)
					{
					case ENCODE_MODE_DIVXCBR:
						*ptr = DIVXCODEC_ENCODECBR;
						break;
					case ENCODE_MODE_DIVXCBRQUALITY:
						*ptr = DIVXCODEC_ENCODECBRQUALITY;
						break;
					case ENCODE_MODE_DIVXVBR:
						*ptr = DIVXCODEC_ENCODEVBR;
						break;
					case ENCODE_MODE_DIVXVBRPLUS:
						*ptr = DIVXCODEC_ENCODECBR;
						break;
					}

				ICSetState(divxNetworksHic,convertConfig->videoFormatOut->lpState,convertConfig->videoFormatOut->cbState);
				}			
	

			delete [] bmfh;

			ICConfigure(divxNetworksHic,globals->hWnd);
			convertConfig->videoFormatOut->cbState = ICGetStateSize(divxNetworksHic);
			ICGetState(divxNetworksHic,convertConfig->videoFormatOut->lpState,convertConfig->videoFormatOut->cbState);
			ICClose(divxNetworksHic);
			videoFormatChanged = true;
			}
		
		return(true);
		}
		
}

// *****************************************************************************
// optionsVideoClass::configureOptionsWindow()
//
//
// *****************************************************************************
void optionsVideoClass::configureOptionsWindow(HWND hWnd)
{
	char tempStr[256];
	int ret;
	int i;
	int val;
	char text[4096];

	switch(convertConfig->ivtcFilmMode)
		{
		case 0:
			convertConfig->film=false;
			convertConfig->ivtc=false;
			break;
		case 1:
			convertConfig->film=true;
			convertConfig->ivtc=false;
			break;
		case 2:
			convertConfig->deinterlace=false;
			convertConfig->film=true;
			convertConfig->ivtc=false;
			convertConfig->ivtcOffset=-1;
			convertConfig->ivtcDeComb=true;
			break;
		case 3:
			convertConfig->film=false;
			convertConfig->ivtc=true;
			convertConfig->ivtcDeComb=true;
			break;
		case 4:
			convertConfig->film=false;
			convertConfig->ivtc=true;
			convertConfig->ivtcDeComb=false;
			break;
		}

	sprintf(tempStr,"%d",convertConfig->hyperCropThreshold);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsHyperCropThreshold),tempStr);
	sprintf(tempStr,"%d",convertConfig->cropTop);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsCropTop),tempStr);
	sprintf(tempStr,"%d",convertConfig->cropBottom);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsCropBottom),tempStr);
	sprintf(tempStr,"%d",convertConfig->cropLeft);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsCropLeft),tempStr);
	sprintf(tempStr,"%d",convertConfig->cropRight);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsCropRight),tempStr);
	sprintf(tempStr,"%d",convertConfig->scaleWidth);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsScaleWidth),tempStr);
	sprintf(tempStr,"%d",convertConfig->scaleHeight);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsScaleHeight),tempStr);
	sprintf(tempStr,"%d",convertConfig->blendThreshold);
	SetWindowText(GetDlgItem(hWnd, VidomiOptionsBlendThreshold),tempStr);

	if(convertConfig->deinterlace)
		SendDlgItemMessage(hWnd,VidomiOptionsDeinterlace,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsDeinterlace,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(convertConfig->hyperScale)
		SendDlgItemMessage(hWnd,VidomiOptionsHyperScale,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsHyperScale,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(convertConfig->hyperCrop)
		SendDlgItemMessage(hWnd,VidomiOptionsHyperCrop,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsHyperCrop,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(convertConfig->blend)
		SendDlgItemMessage(hWnd,VidomiOptionsBlend,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsBlend,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(convertConfig->film)
		SendDlgItemMessage(hWnd,VidomiOptionsFilm,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsFilm,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(convertConfig->forceMultiple16)
		SendDlgItemMessage(hWnd,VidomiOptionsForceMultiple16,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiOptionsForceMultiple16,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);


	SendDlgItemMessage(hWnd,VidomiOptionsScaleType, CB_RESETCONTENT , 0, 0);

	for(i=0;i<(sizeof(scaleType)/4);i++)
		{
		globals->translate->translate(text, scaleType[i]);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsScaleType, CB_ADDSTRING , (WPARAM)0 , (LPARAM) text); 									
		SendDlgItemMessage(hWnd,VidomiOptionsScaleType, CB_SETITEMDATA  , ret, i);
		if(i==convertConfig->scaleType)
			SendDlgItemMessage(hWnd,VidomiOptionsScaleType, CB_SETCURSEL  , ret, 0);
		}


	for(i=0;i<(sizeof(scaleBy)/4);i++)
		{
		globals->translate->translate(text, scaleBy[i]);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsScaleBy, CB_ADDSTRING , (WPARAM)0 , (LPARAM) text); 									
		SendDlgItemMessage(hWnd,VidomiOptionsScaleBy, CB_SETITEMDATA  , ret, i);
		if(i==convertConfig->scaleBy)
			SendDlgItemMessage(hWnd,VidomiOptionsScaleBy, CB_SETCURSEL  , ret, 0);
		}


	// Build the mode list. Done this way for future expansion
	SendDlgItemMessage(hWnd,VidomiOptionsEncodeMode, CB_RESETCONTENT , 0, 0);

	for(i=0;i<(sizeof(encodeMode)/4);i++)
		{
		val = i;

		if(i == ENCODE_MODE_DIVXCBR || 
			i == ENCODE_MODE_DIVXCBRQUALITY ||
			i == ENCODE_MODE_DIVXVBR ||
			i == ENCODE_MODE_DIVXVBRPLUS)
			{
			if(!globals->divxNetworksInstalled)
				{
				val = -1;
				}
			}

		if(val!=-1)
			{
			globals->translate->translate(text, encodeMode[val]);

			ret = SendDlgItemMessage(hWnd,VidomiOptionsEncodeMode, CB_ADDSTRING , (WPARAM)0 , (LPARAM) text); 									
			SendDlgItemMessage(hWnd,VidomiOptionsEncodeMode, CB_SETITEMDATA  , ret, i);
			if(i==convertConfig->encodeMode)
				SendDlgItemMessage(hWnd,VidomiOptionsEncodeMode, CB_SETCURSEL  , ret, 0);
			}
		}


	SendDlgItemMessage(hWnd,VidomiOptionsIVTCOffset, CB_RESETCONTENT , 0, 0);

	if(convertConfig->ivtcOffset<0)
		convertConfig->ivtcOffset=0;
	for(i=0;i<5;i++)
		{
		sprintf(tempStr,"%d",i);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsIVTCOffset, CB_ADDSTRING , (WPARAM)0 , (LPARAM) tempStr); 									
		SendDlgItemMessage(hWnd,VidomiOptionsIVTCOffset, CB_SETITEMDATA  , ret, i);
		if(i==convertConfig->ivtcOffset)
			SendDlgItemMessage(hWnd,VidomiOptionsIVTCOffset, CB_SETCURSEL  , ret, 0);
		}


	// Color space is handled a bit differently than other text options
	SendDlgItemMessage(hWnd,VidomiOptionsEncodeColorSpace, CB_RESETCONTENT , 0, 0);

	globals->translate->translate(text, encodeColorSpace[0]);

	ret = SendDlgItemMessage(hWnd,VidomiOptionsEncodeColorSpace, CB_ADDSTRING , (WPARAM)0 , (LPARAM) text); 									
	SendDlgItemMessage(hWnd,VidomiOptionsEncodeColorSpace, CB_SETITEMDATA  , ret, rgb32Frame);
	if(convertConfig->encodeColorSpace==rgb32Frame)
		SendDlgItemMessage(hWnd,VidomiOptionsEncodeColorSpace, CB_SETCURSEL  , ret, 0);

	if(globals->gatherResults->fileType ==  SCAN_FILE_TYPE_MPEG && 
		globals->gatherResults->videoChromaFormat == CHROMA420)
		{
		globals->translate->translate(text, encodeColorSpace[1]);
		ret = SendDlgItemMessage(hWnd,VidomiOptionsEncodeColorSpace, CB_ADDSTRING , (WPARAM)0 , (LPARAM) text); 									
		SendDlgItemMessage(hWnd,VidomiOptionsEncodeColorSpace, CB_SETITEMDATA  , ret, yv12Frame);
		if(convertConfig->encodeColorSpace==yv12Frame)
			SendDlgItemMessage(hWnd,VidomiOptionsEncodeColorSpace, CB_SETCURSEL  , ret, 0);
		}

	enableWindows(hWnd);

}

// *****************************************************************************
// optionsVideoClass::readWindowVars()
//
//
// *****************************************************************************
void optionsVideoClass::readWindowVars(HWND hWnd)
{
	char tempStr[260];
	int ret;

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsHyperCropThreshold), tempStr , 256);
 	convertConfig->hyperCropThreshold = atoi(tempStr);

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsCropTop), tempStr , 256);
 	convertConfig->cropTop = atoi(tempStr);

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsCropBottom), tempStr , 256);
 	convertConfig->cropBottom = atoi(tempStr);

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsCropLeft), tempStr , 256);
 	convertConfig->cropLeft = atoi(tempStr);
	
	GetWindowText(GetDlgItem(hWnd, VidomiOptionsCropRight), tempStr , 256);
 	convertConfig->cropRight = atoi(tempStr);

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsBlendThreshold), tempStr , 256);
 	convertConfig->blendThreshold = atoi(tempStr);

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsScaleWidth), tempStr , 256);
 	convertConfig->scaleWidth = atoi(tempStr);

	GetWindowText(GetDlgItem(hWnd, VidomiOptionsScaleHeight), tempStr , 256);
 	convertConfig->scaleHeight = atoi(tempStr);

	if(SendDlgItemMessage(hWnd,VidomiOptionsBlend,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->blend=true;
	else
		convertConfig->blend=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsDeinterlace,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->deinterlace=true;
	else
		convertConfig->deinterlace=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsHyperCrop,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->hyperCrop=true;
	else
		convertConfig->hyperCrop=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsHyperScale,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->hyperScale=true;
	else
		convertConfig->hyperScale=false;

	if(SendDlgItemMessage(hWnd,VidomiOptionsForceMultiple16,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->forceMultiple16=true;
	else
		convertConfig->forceMultiple16=false;

	ret = SendDlgItemMessage(hWnd,VidomiOptionsScaleType, CB_GETCURSEL  , 0, 0);
	convertConfig->scaleType = SendDlgItemMessage(hWnd,VidomiOptionsScaleType, CB_GETITEMDATA , ret, 0);

	ret = SendDlgItemMessage(hWnd,VidomiOptionsScaleBy, CB_GETCURSEL  , 0, 0);
	convertConfig->scaleBy = SendDlgItemMessage(hWnd,VidomiOptionsScaleBy, CB_GETITEMDATA , ret, 0);
	
	if(convertConfig->scaleBy>0)
		convertConfig->scale = true;
	else
		convertConfig->scale = false;

	ret = SendDlgItemMessage(hWnd,VidomiOptionsEncodeMode, CB_GETCURSEL  , 0, 0);
	convertConfig->encodeMode = SendDlgItemMessage(hWnd,VidomiOptionsEncodeMode, CB_GETITEMDATA , ret, 0);

	if(SendDlgItemMessage(hWnd,VidomiOptionsIVTCInvertPolarity,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		convertConfig->ivtcPolarity=true;
	else
		convertConfig->ivtcPolarity=false;

	ret = SendDlgItemMessage(hWnd,VidomiOptionsIVTCFilm, CB_GETCURSEL  , 0, 0);
	convertConfig->ivtcFilmMode = SendDlgItemMessage(hWnd,VidomiOptionsIVTCFilm, CB_GETITEMDATA , ret, 0);

	ret = SendDlgItemMessage(hWnd,VidomiOptionsIVTCOffset, CB_GETCURSEL  , 0, 0);
	convertConfig->ivtcOffset = SendDlgItemMessage(hWnd,VidomiOptionsIVTCOffset, CB_GETITEMDATA , ret, 0);

	// If YV12 mode, then turn off deinterlace and IVTC
	if(convertConfig->encodeColorSpace==yv12Frame)
		{
		//convertConfig->deinterlace = false;
		convertConfig->ivtc = false;
		}
}

// *****************************************************************************
// optionsVideoClass::enableWindows()
//
//
// *****************************************************************************
void optionsVideoClass::enableWindows(HWND hWnd)
{
	int state;
	int ret;
	int i;
	int end;
	char text[4096];

	switch(convertConfig->ivtcFilmMode)
		{
		case 0:
			convertConfig->film=false;
			convertConfig->ivtc=false;
			break;
		case 1:
			convertConfig->film=true;
			convertConfig->ivtc=false;
			break;
		case 2:
			convertConfig->deinterlace=false;
			convertConfig->film=true;
			convertConfig->ivtc=false;
			convertConfig->ivtcOffset=-1;
			convertConfig->ivtcDeComb=true;
			break;
		case 3:
			convertConfig->film=false;
			convertConfig->ivtc=true;
			convertConfig->ivtcDeComb=true;
			break;
		case 4:
			convertConfig->film=false;
			convertConfig->ivtc=true;
			convertConfig->ivtcDeComb=false;
			break;
		}

	//Configure the hyper crop settings
	if(convertConfig->hyperCrop)
		state = false;
	else
		state = true;

	EnableWindow(GetDlgItem(hWnd,VidomiOptionsCropBottom), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsCropBottomText), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsCropTop), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsCropTopText), state);	
	
	//Configure the hyper scale settings
	if(convertConfig->scaleBy==0)
		state = false;
	else
		state = true;

	EnableWindow(GetDlgItem(hWnd,VidomiOptionsForceMultiple16), state);
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsForceMultiple16Text), state);
	
	state = convertConfig->hyperScale;

	if(convertConfig->scaleBy==0)
		{
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleHeight), FALSE);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleHeightText), FALSE);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleWidth), FALSE);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleWidthText), FALSE);	
		}
	if(convertConfig->scaleBy==SCALE_BY_PIXELS)
		{
		// "Height (Pixels)"
		globals->translate->translate(text, "HeightPixels");
		SetWindowText(GetDlgItem(hWnd, VidomiOptionsScaleHeightText),text);			

		// "Width (Pixels)"
		globals->translate->translate(text, "WidthPixels");
		SetWindowText(GetDlgItem(hWnd, VidomiOptionsScaleWidthText),text);			

		EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleHeight), !state);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleHeightText), !state);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleWidth), TRUE);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleWidthText), TRUE);	
		}
	if(convertConfig->scaleBy==SCALE_BY_PERCENTAGE)
		{
		// "% of Height"
		globals->translate->translate(text, "HeightPercent");
		SetWindowText(GetDlgItem(hWnd, VidomiOptionsScaleHeightText),text);			

		// "% of Width"
		globals->translate->translate(text, "WidthPercent");
		SetWindowText(GetDlgItem(hWnd, VidomiOptionsScaleWidthText),text);			

		EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleHeight), !state);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleHeightText), !state);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleWidth), TRUE);	
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleWidthText), TRUE);	
		}


//	if(convertConfig->encodeColorSpace==yv12Frame)
//		{
//		state = false;
//		}

	state = true;

	EnableWindow(GetDlgItem(hWnd,VidomiOptionsDeinterlace), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsDeinterlaceText), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsBlend), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsBlendText), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsBlendThreshold), state);	
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsBlendThresholdText), state);	


	if(convertConfig->encodeColorSpace==yv12Frame)
		state = false;
	else
		state = true;

	EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleType), state);		
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsScaleTypeText), state);		


	
	switch(convertConfig->encodeMode)
		{
		case ENCODE_MODE_CBR2PASS:
			// Select Video Codec Pass 1 Defaults
			globals->translate->translate(text, "VidomiOptionsSelectVideoCodec2P1");
			break;

		case ENCODE_MODE_CBR:
		case ENCODE_MODE_HYBRID1:
		case ENCODE_MODE_HYBRID2:
		case ENCODE_MODE_VBR:
			// Select Video Codec Defaults
			globals->translate->translate(text, "VidomiOptionsSelectVideoCodec1");
			break;
		case ENCODE_MODE_XVIDVBR:
		case ENCODE_MODE_XVIDHYBRID:
			globals->translate->translate(text, "VidomiOptionsSelectVideoCodec3");
			break;
		default:
			// Set Divx 4 Codec Parameters
			globals->translate->translate(text, "VidomiOptionsSelectVideoCodec2");
			break;
		}

	SetWindowText(GetDlgItem(hWnd, VidomiOptionsSelectVideoCodec),text);			


	if(convertConfig->encodeMode == ENCODE_MODE_CBR || 
		convertConfig->encodeMode == ENCODE_MODE_CBR2PASS || 
		convertConfig->encodeMode == ENCODE_MODE_DIVXCBR ||
		convertConfig->encodeMode == ENCODE_MODE_DIVXCBRQUALITY ||
		convertConfig->encodeMode == ENCODE_MODE_DIVXVBR ||
		convertConfig->encodeMode == ENCODE_MODE_DIVXVBRPLUS ||
		convertConfig->encodeMode == ENCODE_MODE_XVIDVBR || 
		convertConfig->encodeMode == ENCODE_MODE_XVIDHYBRID)
		{
		state = true;
		}
	else
		{
		state = false;
		}

	EnableWindow(GetDlgItem(hWnd,VidomiOptionsSelectVideoCodec), state);	
	
	// Now enable the second codec set button if we're in two pass codec mode

	if(convertConfig->encodeMode == ENCODE_MODE_CBR2PASS)
		{
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsSelectVideoCodec2), TRUE);		
		}
	else
		{
		EnableWindow(GetDlgItem(hWnd,VidomiOptionsSelectVideoCodec2), FALSE);	
		}

	if(convertConfig->ivtcFilmMode>=3)
		state = true;
	else
		state = false;

	if(convertConfig->encodeColorSpace==yv12Frame)
		{
		state = false;
		}

	EnableWindow(GetDlgItem(hWnd,VidomiOptionsIVTCOffset), state);
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsIVTCInvertPolarity), state);
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsIVTCOffsetText), state);		
	EnableWindow(GetDlgItem(hWnd,VidomiOptionsIVTCInvertPolarityText), state);		

	// If YV12 mode, then limit to FILM and normal.
	end = sizeof(encodeIvtcFilmMode)/4;
	if(convertConfig->encodeColorSpace==yv12Frame)
		{
		end = 2;
		}

	if(convertConfig->ivtcFilmMode>end)
		{
		convertConfig->ivtcFilmMode = 0;
		}

	SendDlgItemMessage(hWnd,VidomiOptionsIVTCFilm, CB_RESETCONTENT , 0, 0);

	for(i=0;i<end;i++)
		{
		globals->translate->translate(text, encodeIvtcFilmMode[i]);

		ret = SendDlgItemMessage(hWnd,VidomiOptionsIVTCFilm, CB_ADDSTRING , (WPARAM)0 , (LPARAM) text); 									
		SendDlgItemMessage(hWnd,VidomiOptionsIVTCFilm, CB_SETITEMDATA  , ret, i);
		if(i==convertConfig->ivtcFilmMode)
			SendDlgItemMessage(hWnd,VidomiOptionsIVTCFilm, CB_SETCURSEL  , ret, 0);
		}

}


