// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// outputStats.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

static TRANSLATEIDTABLE outputStatisticsTID[] = {
										{"VidomiStatisticsText2",VidomiStatisticsText2},
										{"VidomiStatisticsText3",VidomiStatisticsText3},
										{"VidomiStatisticsText4",VidomiStatisticsText4},
										{"VidomiStatisticsText5",VidomiStatisticsText5},
										{"VidomiStatisticsText6",VidomiStatisticsText6},
										{"VidomiStatisticsText7",VidomiStatisticsText7},
										{"VidomiStatisticsText9",VidomiStatisticsText9},
										{"VidomiStatisticsText10",VidomiStatisticsText10},
										{"VidomiStatisticsText11",VidomiStatisticsText11},
										{"VidomiStatisticsText12",VidomiStatisticsText12},
										{"VidomiStatisticsText13",VidomiStatisticsText13},
										{"VidomiStatisticsText14",VidomiStatisticsText14},
										{"VidomiStatisticsText15",VidomiStatisticsText15},
										{"VidomiStatisticsText16",VidomiStatisticsText16},
										{"VidomiStatisticsText17",VidomiStatisticsText17},
										{"VidomiStatisticsText18",VidomiStatisticsText18},
										{"",0}									
										};

// *****************************************************************************
// outputStatsClass::outputStatsClass()
//
// conversion class constructor
// *****************************************************************************
outputStatsClass::outputStatsClass()
{
	visible = false;
	statsStr = new char [1024];
	windowsCreated = false;
}

// *****************************************************************************
// outputStatsClass::~outputStatsClass()
//
// conversion class destructor
// *****************************************************************************
outputStatsClass::~outputStatsClass()
{
	if(statshWnd && windowsCreated)
		DestroyWindow(statshWnd);

	if(statsStr)
		delete [] statsStr;
}

// *****************************************************************************
// outputStatsClass::createOutputWindow()
//
// conversion class constructor
// *****************************************************************************
bool outputStatsClass::createOutputWindow(HINSTANCE hInst)
{
	RECT client;
	int fullHeight;
	int fullWidth;
	char text[4096];
	int i;

	statshInst = hInst;

	if(!windowsCreated)
		{
		fullHeight = GetSystemMetrics(SM_CYSCREEN);
		fullWidth = GetSystemMetrics(SM_CXSCREEN);

		if((globals->statsLeft > fullWidth) || globals->statsLeft < 0)
			globals->statsLeft = 320;

		if((globals->statsTop > fullHeight) || globals->statsTop < 0)
			globals->statsTop = 240;

		statshWnd = CreateDialog(statshInst,MAKEINTRESOURCE(VidomiStatistics),NULL, statsDialogHandler);


		globals->translate->translate(text, "VidomiStatistics");
		SetWindowText(statshWnd, text);			

		for(i=0;;i++)
			{
			if(!outputStatisticsTID[i].id)
				break;
			globals->translate->translate(text, outputStatisticsTID[i].textId);
			SetWindowText(GetDlgItem(statshWnd, outputStatisticsTID[i].id),text);			
			}

		GetWindowRect(statshWnd, &client);
		MoveWindow(statshWnd, globals->statsLeft, globals->statsTop, client.right - client.left, client.bottom - client.top, FALSE);
		windowsCreated = true;
		}


	return(true);
}

// *****************************************************************************
// outputStatsClass::showOutputWindow()
//
// 
// *****************************************************************************
bool outputStatsClass::showOutputWindow()
{
	ShowWindow(statshWnd, SW_SHOW);	
	BringWindowToTop(statshWnd);
	visible = true;
	return(true);
}

// *****************************************************************************
// outputStatsClass::hideOutputWindow()
//
// 
// *****************************************************************************
bool outputStatsClass::hideOutputWindow()
{
	if(windowsCreated)
		{
		//windowsCreated = false;
		ShowWindow(statshWnd, SW_HIDE);
		//DestroyWindow(statshWnd);
		}
	visible = false;
	return(true);
}

// *****************************************************************************
// outputStatsClass::windowVisible()
//
// 
// *****************************************************************************
bool outputStatsClass::windowVisible()
{
	return(visible);
}

// *****************************************************************************
// outputStatsClass::statsDialogHandler()
//
//
// *****************************************************************************
BOOL CALLBACK outputStatsClass::statsDialogHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->outputStats->statsDialogHandler1(hWnd,message,wParam,lParam));
}
// *****************************************************************************
// outputStatsClass::statsDialogHandler1()
//
//
// *****************************************************************************
BOOL outputStatsClass::statsDialogHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	RECT win;

	switch(message)
		{
		case WM_MOVE :
			GetWindowRect(hWnd, &win);
			globals->statsLeft = win.left;
			globals->statsTop  = win.top;    
			return(TRUE);

		case WM_CLOSE:
			hideOutputWindow();
			return(TRUE);
		}

	return (FALSE);
}

// *****************************************************************************
// outputStatsClass::setFPS()
//
// 
// *****************************************************************************
void outputStatsClass::setFPS(float fps)
{

	if(visible && windowsCreated)
		{
		sprintf(statsStr,"%.2f",fps);
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsFPSText),statsStr);
		}
}

// *****************************************************************************
// outputStatsClass::setPass()
//
// 
// *****************************************************************************
void outputStatsClass::setPass(int currentPass, int totalPasses)
{
	char text[256];
	char text1[256];
	char statsStrW[256];

	if(visible && windowsCreated)
		{
		globals->translate->translate(text, "VidomiStatisticsText8");
		strcpy(text1,"%d ");
		strcat(text1,text);
		strcat(text1," %d");
		sprintf(statsStrW,text1,currentPass,totalPasses);
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsPassNumberText),statsStrW);
		}
}

// *****************************************************************************
// outputStatsClass::setPercentComplete()
//
// 
// *****************************************************************************
void outputStatsClass::setPercentComplete(float percent)
{
	if(visible && windowsCreated)
		{
		sprintf(statsStr,"%.2f%%",percent);
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsPercentCompleteText),statsStr);
		}
}

// *****************************************************************************
// outputStatsClass::setElapsedTime()
//
// 
// *****************************************************************************
void outputStatsClass::setElapsedPassTime(int secs)
{
	int hours;
	int minutes;
	int seconds;

	if(visible && windowsCreated)
		{
		hours = secs / 3600;
		minutes = (secs - (hours * 3600) ) / 60;
		seconds = secs - ((hours * 3600) + (minutes * 60));
		sprintf(statsStr,"%.2d:%.2d:%.2d", hours, minutes, seconds);
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsElapsedTimeText),statsStr);
		}
}

// *****************************************************************************
// outputStatsClass::setElapsedTime()
//
// 
// *****************************************************************************
void outputStatsClass::setElapsedJobTime(int secs)
{
	int hours;
	int minutes;
	int seconds;

	if(visible && windowsCreated)
		{
		hours = secs / 3600;
		minutes = (secs - (hours * 3600) ) / 60;
		seconds = secs - ((hours * 3600) + (minutes * 60));
		sprintf(statsStr,"%.2d:%.2d:%.2d", hours, minutes, seconds);
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsTotalElapsedTimeText),statsStr);
		}
}

// *****************************************************************************
// outputStatsClass::setElapsedTime()
//
// 
// *****************************************************************************
void outputStatsClass::setElapsedTime(int secs)
{
	int hours;
	int minutes;
	int seconds;

	if(visible && windowsCreated)
		{
		hours = secs / 3600;
		minutes = (secs - (hours * 3600) ) / 60;
		seconds = secs - ((hours * 3600) + (minutes * 60));
		sprintf(statsStr,"%.2d:%.2d:%.2d", hours, minutes, seconds);
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsElapsedEncodeTimeText),statsStr);
		}
}

// *****************************************************************************
// outputStatsClass::setElapsedTime()
//
// 
// *****************************************************************************
void outputStatsClass::setRemainingTime(int secs)
{
	int hours;
	int minutes;
	int seconds;

	if(visible && windowsCreated)
		{
		hours = secs / 3600;
		minutes = (secs - (hours * 3600) ) / 60;
		seconds = secs - ((hours * 3600) + (minutes * 60));
		sprintf(statsStr,"%.2d:%.2d:%.2d", hours, minutes, seconds);
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsRemainingTimeText),statsStr);
		}
}

// *****************************************************************************
// outputStatsClass::setTotalEncodeJobs()
//
// 
// *****************************************************************************
void outputStatsClass::setTotalEncodeJobs(int count)
{

	if(visible && windowsCreated)
		{
		sprintf(statsStr,"%d", count);
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsTotalEncodeJobsNumberText),statsStr);
		}
}

// *****************************************************************************
// outputStatsClass::setJobsBeingProcessed()
//
// 
// *****************************************************************************
void outputStatsClass::setJobsBeingProcessed(int count)
{

	if(visible && windowsCreated)
		{
		sprintf(statsStr,"%d", count);
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsJobsBeingProcessedNumberText),statsStr);
		}
}

// *****************************************************************************
// outputStatsClass::setJobsRemaining()
//
// 
// *****************************************************************************
void outputStatsClass::setJobsRemaining(int count)
{

	if(visible && windowsCreated)
		{
		sprintf(statsStr,"%d", count);
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsJobsRemainingNumberText),statsStr);
		}
}

// *****************************************************************************
// outputStatsClass::setJobsCompleted()
//
// 
// *****************************************************************************
void outputStatsClass::setJobsCompleted(int count)
{

	if(visible && windowsCreated)
		{
		sprintf(statsStr,"%d", count);
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsJobsCompletedNumberText),statsStr);
		}
}

// *****************************************************************************
// outputStatsClass::setPass()
//
// 
// *****************************************************************************
void outputStatsClass::setRange(int currentRange, int totalNumRanges)
{
	char text[256];
	char text1[256];
	char statsStrW[256];

	if(visible && windowsCreated)
		{
		globals->translate->translate(text, "VidomiStatisticsText8");	//of
		strcpy(text1,"%d ");
		strcat(text1,text);
		strcat(text1," %d");
		sprintf(statsStrW,text1,currentRange,totalNumRanges);
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsRangeNumberText),statsStrW);
		}
}

// *****************************************************************************
// outputStatsClass::setJobNameText()
//
// 
// *****************************************************************************
void outputStatsClass::setJobNameText(char * jobText)
{

	if(visible && windowsCreated)
		{
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsEncodeJobNameText),jobText);
		}
}

// *****************************************************************************
// outputStatsClass::setJobNameText()
//
// 
// *****************************************************************************
void outputStatsClass::setConnectionStatusText(char * connectionText)
{

	if(visible && windowsCreated)
		{
		SetWindowText(GetDlgItem(statshWnd, VidomiStatsConnectionStatusText),connectionText);
		}
}
// *****************************************************************************
// outputStatsClass::resetMachineSelection()
//
// 
// *****************************************************************************
void outputStatsClass::resetMachineSelection()
{
	if(windowsCreated)
		{

		SendDlgItemMessage(statshWnd,VidomiStatsMachineSelected, CB_RESETCONTENT , 0, 0);

		}

}

// *****************************************************************************
// outputStatsClass::resetMachineSelection()
//
// 
// *****************************************************************************
void outputStatsClass::insertMachineName(char * name, int index)
{
	int ret;

	if(windowsCreated)
		{
		ret = SendDlgItemMessage(statshWnd,VidomiStatsMachineSelected, CB_ADDSTRING , (WPARAM)0 , (LPARAM) name); 									
		SendDlgItemMessage(statshWnd,VidomiStatsMachineSelected, CB_SETITEMDATA  , ret, index);
		if(index==0)
			SendDlgItemMessage(statshWnd,VidomiStatsMachineSelected, CB_SETCURSEL  , ret, 0);
		}

}

// *****************************************************************************
// outputStatsClass::resetMachineSelection()
//
// 
// *****************************************************************************
int outputStatsClass::getMachineSelected()
{
	int ret;

	ret = 0;

	if(windowsCreated)
		{
		ret = SendDlgItemMessage(statshWnd,VidomiStatsMachineSelected, CB_GETCURSEL  , 0, 0);
		ret = SendDlgItemMessage(statshWnd,VidomiStatsMachineSelected, CB_GETITEMDATA , ret, 0);
		
		}

	return(ret);
}