// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// outputvideo.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

// *****************************************************************************
// outputVideoClass::outputVideoClass()
//
// conversion class constructor
// *****************************************************************************
outputVideoClass::outputVideoClass()
{
	videoInputSet = false;
	visible = false;
	fullscreen = false;

	videoWidth = 32;
	videoHeight = 32;
	outputWidth = 32;
	outputHeight = 32;
	panelWidth = 0;
	panelHeight = 0;
	aspectRatio = 720.0f/480.0f;
	videoColorSpace = rgb32Frame;

	hdd = DrawDibOpen();
	InitializeCriticalSection(&fullscreenLock);
	

}

// *****************************************************************************
// outputVideoClass::~outputVideoClass()
//
// conversion class destructor
// *****************************************************************************
outputVideoClass::~outputVideoClass()
{
	if(hdd)
		DrawDibClose(hdd);

	if(outputhWnd)
		DestroyWindow(outputhWnd);

}

// *****************************************************************************
// outputVideoClass::createOutputWindow()
//
// conversion class constructor
// *****************************************************************************
bool outputVideoClass::createOutputWindow(HINSTANCE hInst)
{
	RECT client;
	int fullHeight;
	int fullWidth;

	fullHeight = GetSystemMetrics(SM_CYSCREEN);
	fullWidth = GetSystemMetrics(SM_CXSCREEN);

	if((globals->outputLeft > fullWidth) || globals->outputLeft < 0)
		globals->outputLeft = 320;

	if((globals->outputTop > fullHeight) || globals->outputTop < 0)
		globals->outputTop = 240;

	outputhInst = hInst;

	registerWindowClass(hInst);
	
	outputhWnd = CreateWindow("VidomiOutputClass", "Vidomi Output", WS_OVERLAPPED | WS_SIZEBOX | WS_SYSMENU | WS_MINIMIZEBOX, 
							0, 0, 
							320, 240, NULL, NULL, hInst, NULL );

	outputhWnd1 = CreateWindow("VidomiOutputFullClass", "Vidomi Output" , WS_POPUP, 
							0, 0, 
							320, 240, NULL, NULL, outputhInst, NULL );

	if(!outputhWnd || !outputhWnd1)
		return(false);

	GetClientRect(outputhWnd, &client);
	leftBorder = 320 - client.right;
	topBorder = 240 - client.bottom;

	MoveWindow(outputhWnd, globals->outputLeft, globals->outputTop, leftBorder+320, topBorder+240, TRUE);

	return(true);
}

// *****************************************************************************
// outputVideoClass::showOutputWindow()
//
// 
// *****************************************************************************
bool outputVideoClass::showOutputWindow()
{

	ShowWindow(outputhWnd, SW_SHOW);
	BringWindowToTop(outputhWnd);	
	visible = true;
	return(true);
}

// *****************************************************************************
// outputVideoClass::hideOutputWindow()
//
// 
// *****************************************************************************
bool outputVideoClass::hideOutputWindow()
{
	ShowWindow(outputhWnd, SW_HIDE);	
	visible = false;
	return(true);
}

// *****************************************************************************
// outputVideoClass::windowVisible()
//
// 
// *****************************************************************************
bool outputVideoClass::windowVisible()
{
	return(visible);
}

// *****************************************************************************
// outputVideoClass::registerWindowClass()
//
//
// *****************************************************************************
bool outputVideoClass::registerWindowClass(HINSTANCE hInst)
{
	//Main Window
	windCls.style         = CS_HREDRAW | CS_VREDRAW | CS_BYTEALIGNCLIENT;
    windCls.lpfnWndProc   = (WNDPROC)windowMessageHandler;
    windCls.cbClsExtra    = 0;
    windCls.cbWndExtra    = 0;
    windCls.hInstance     = hInst;          
    windCls.hIcon         = LoadIcon(hInst, MAKEINTRESOURCE(VidomiIcon));
    windCls.hCursor       = LoadCursor(NULL, IDC_ARROW);
    windCls.hbrBackground = HBRUSH(0);
    windCls.lpszMenuName  = NULL;
	windCls.lpszClassName = "VidomiOutputClass";
	
    if (!RegisterClass( &windCls))
		{
		logPrintf("Failed To create Output Window Class\n");
        return false;
		}

	// Now create the full screen version
    windCls.lpfnWndProc   = (WNDPROC)windowMessageHandlerFullscreen;
	windCls.lpszClassName = "VidomiOutputFullClass";
   if (!RegisterClass( &windCls))
		{
		logPrintf("Failed To create Output Window Class\n");
        return false;
		}

	return true;
}

// *****************************************************************************
// windowClass::windowMessageHandler()
//
//
// *****************************************************************************
BOOL CALLBACK outputVideoClass::windowMessageHandlerFullscreen(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->outputVideo->windowMessageHandlerFullscreen1(hWnd,message,wParam,lParam));
}

// *****************************************************************************
// windowClass::windowMessageHandler()
//
//
// *****************************************************************************
BOOL outputVideoClass::windowMessageHandlerFullscreen1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return (DefWindowProc(hWnd, message, wParam, lParam));

}
// *****************************************************************************
// windowClass::windowMessageHandler()
//
//
// *****************************************************************************
BOOL CALLBACK outputVideoClass::windowMessageHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	return(globals->outputVideo->windowMessageHandler1(hWnd,message,wParam,lParam));
}

// *****************************************************************************
// windowClass::windowMessageHandler()
//
//
// *****************************************************************************
BOOL outputVideoClass::windowMessageHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	RECT * ptr;
	PAINTSTRUCT paint;
	HDC hDC;
	RECT win;

	switch(message)
		{

		case WM_MOVE :
			GetWindowRect(hWnd, &win);
			globals->outputLeft = win.left;
			globals->outputTop  = win.top;    
			return(TRUE);
	
		case WM_CLOSE:
			hideOutputWindow();
			return(TRUE);

		case WM_SIZING :
			ptr = (RECT *) lParam;
			if(resize)
				{
				outputWidth = ptr->right - ptr->left - leftBorder - panelWidth;
				outputHeight = ptr->bottom - ptr->top - topBorder - panelHeight;

				if(outputHeight <128)
					outputHeight = 128;
				if(outputWidth <128)
					outputWidth = 128;
				}
			ptr->right = ptr->left + outputWidth + leftBorder + panelWidth;
			ptr->bottom = ptr->top + outputHeight + topBorder + panelHeight;
			return(TRUE);
		case WM_PAINT :
			hDC = BeginPaint(hWnd,&paint);
			EndPaint(hWnd,&paint);
			needsRepaint=true;
			return(TRUE);

		default:
			return (DefWindowProc(hWnd, message, wParam, lParam));
		}

	return (FALSE);

}

// *****************************************************************************
// outputVideoClass::setVideoInputFormat()
//
// width = Width of bitmaps that wll be fed to writeavi
// height = Height of bitmaps that will be fed to writeavi
// bpp = Bits per Pixel. Must be divisible by 8 (IE 24 or 32)
// framerate = framerate
// *****************************************************************************
bool outputVideoClass::setVideoInputFormat(int width,int height,int colorSpace)
{

	memset(&bitmapInfo,0,sizeof(BITMAPINFO));
	bitmapInfo.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	bitmapInfo.bmiHeader.biWidth=width;
	bitmapInfo.bmiHeader.biHeight=height;
	bitmapInfo.bmiHeader.biPlanes=1;
	bitmapInfo.bmiHeader.biBitCount=32;
	bitmapInfo.bmiHeader.biCompression=BI_RGB;
	bitmapInfo.bmiHeader.biSizeImage=(width*height)*(32/8);
	bitmapInfo.bmiHeader.biXPelsPerMeter=0;
	bitmapInfo.bmiHeader.biYPelsPerMeter=0;
	bitmapInfo.bmiHeader.biClrUsed=0;
	bitmapInfo.bmiHeader.biClrImportant=0;

	videoBpp = 32;
	videoWidth = width;
	videoHeight = height;
	videoColorSpace = colorSpace;
	outputWidth = width;
	outputHeight = height;
	videoInputSet = true;
	outputOffsetX = 0;
	outputOffsetY = 0;
	
	setOutputWindowText("Vidomi Output");
	visible = false;
	fullscreen = false;
	needsRepaint=true;
	resize = true;

	return (true);

}

// *****************************************************************************
// outputVideoClass::displayFrame()
//
// frame = The Frame to be displayed to the screen
// *****************************************************************************
bool outputVideoClass::displayFrame(FRAMEDATA * frame)
{
	HDC hDC;
	FRAMEDATA tempFrame;
	unsigned char * yStart;
	unsigned char * uStart;
	unsigned char * vStart;
	unsigned char * y;
	unsigned char * u;
	unsigned char * v;
	unsigned char * ptr;
	int xPos;
	int yPos;
	int bb;
	int gg;
	int rr;
	int destWidth;
	int destHeight;

	if(!videoInputSet)
		return(false);

	if(!visible)
		return(true);

	destWidth = outputWidth;
	destHeight = outputHeight;

	if(videoColorSpace == yv12Frame)
		{
		tempFrame.data = new unsigned char [frame->width * frame->height * 4];
		yStart = frame->data;
		vStart = frame->data + (frame->width * frame->height);
		uStart = vStart + ((frame->width/2) * (frame->height/2));

		ptr = tempFrame.data;

		for(yPos=0;yPos<frame->height;yPos++)
			{
			y = yStart + (frame->height - yPos) * frame->width;
			u = uStart + ((frame->height - yPos)/2) * (frame->width/2);
			v = vStart + ((frame->height - yPos)/2) * (frame->width/2);
			for(xPos=0;xPos<(frame->width/2);xPos++)
				{

				rr = (int)((*y-128) + (1.140 * (*v-128)));
				gg = (int)((*y-128) - (0.394 * (*u-128)) - (0.581 * (*v-128)));
				bb = (int)((*y-128) + (2.032 * (*u-128)));

				rr+=128;
				gg+=128;
				bb+=128;

				if(rr<0)
					rr =0;
				if(rr>255)
					rr =255;
				if(gg<0)
					gg = 0;
				if(gg>255)
					gg = 255;
				if(bb<0)
					bb = 0;
				if(bb>255)
					bb = 255;

				*ptr++ = (unsigned char) bb; 
				*ptr++ = (unsigned char) gg;
				*ptr++ = (unsigned char) rr;
				*ptr++ = 0;
				
				*y++;


				rr = (int)((*y-128) + (1.140 * (*v-128)));
				gg = (int)((*y-128) - (0.394 * (*u-128)) - (0.581 * (*v-128)));
				bb = (int)((*y-128) + (2.032 * (*u-128)));

				rr+=128;
				gg+=128;
				bb+=128;

				if(rr<0)
					rr =0;
				if(rr>255)
					rr =255;
				if(gg<0)
					gg = 0;
				if(gg>255)
					gg = 255;
				if(bb<0)
					bb = 0;
				if(bb>255)
					bb = 255;

				*ptr++ = (unsigned char) bb; 
				*ptr++ = (unsigned char) gg;
				*ptr++ = (unsigned char) rr; 
				*ptr++ = 0;

				*y++;
				*v++;
				*u++;

				}
			}

		hDC = GetDC(outputhWnd);

		if (frame->width > outputWidth)
			destHeight = frame->height * destWidth / frame->width;

		StretchDIBits(hDC, 
			0, 0, destWidth, destHeight, 
			0, 0, frame->width, frame->height,
			tempFrame.data, &bitmapInfo, DIB_RGB_COLORS, SRCCOPY);
		
		delete [] tempFrame.data;

		}
	else
		{
		hDC = GetDC(outputhWnd);

		if (frame->width > outputWidth)
			destHeight = frame->height * destWidth / frame->width;

		StretchDIBits(hDC, 
			0, 0, destWidth, destHeight, 
			0, 0, frame->width, frame->height,
			frame->data, &bitmapInfo, DIB_RGB_COLORS, SRCCOPY);
		}

	ReleaseDC(outputhWnd,hDC);
	needsRepaint=false;
	return(true);
}

// *****************************************************************************
// outputVideoClass::displayAviFrame()
//
// frame = The Frame to be displayed to the screen
// *****************************************************************************
bool outputVideoClass::displayAviFrame(FRAMEDATA * frame)
{
	HDC hDC;
	int destWidth;
	int destHeight;
	int destOffsetX;
	int destOffsetY;
	BITMAPINFOHEADER * bi;
	HWND draw;
	int ret;
	if(!videoInputSet)
		return(false);

	if(!visible)
		return(true);

	bi = (BITMAPINFOHEADER *)frame->bufferStart;
	
	EnterCriticalSection(&fullscreenLock);
	if(!fullscreen)
		draw = outputhWnd;
	else
		draw = outputhWnd1;

	destWidth = outputWidth;
	destHeight = outputHeight;
	destOffsetX = outputOffsetX;
	destOffsetY = outputOffsetY;
	LeaveCriticalSection(&fullscreenLock);

	if (destWidth < bi->biWidth)
		{
		destHeight = bi->biHeight * destWidth / bi->biWidth;
		}

	if (destHeight < bi->biHeight)
		{
		destWidth = bi->biWidth * destHeight / bi->biHeight;
		}

	hDC = GetDC(draw);

	ret = DrawDibDraw(hdd,hDC,destOffsetX,destOffsetY,destWidth,destHeight,bi,frame->data,0,0,bi->biWidth,bi->biHeight,0);
 	ReleaseDC(draw,hDC);
	needsRepaint=false;

	return(true);
}


// *****************************************************************************
// outputVideoClass::setOutputWindowSize()
//
// 
// *****************************************************************************
bool outputVideoClass::setOutputWindowSize(int width,int height, int extraWidth, int extraHeight)
{
	RECT cur;

	videoWidth = width;
	videoHeight = height;
	outputWidth = width;
	outputHeight = height;
	panelWidth = extraWidth;
	panelHeight = extraHeight;

	aspectRatio = (double)height / (double)width;
	invAspectRatio = (double)width / (double)height;
	
	GetWindowRect(outputhWnd, &cur);
	MoveWindow(outputhWnd, globals->outputLeft, globals->outputTop, leftBorder+videoWidth+ extraWidth, topBorder+videoHeight+extraHeight, TRUE);
	GetWindowRect(outputhWnd, &outputState);
	return(true);
}

// *****************************************************************************
// outputVideoClass::setOutputWindowSize()
//
// 
// *****************************************************************************
bool outputVideoClass::windowRepaint()
{
	return(needsRepaint);
}


// *****************************************************************************
// outputVideoClass::getScreenState()
//
// 
// *****************************************************************************
bool outputVideoClass::getScreenState()
{
	bool screen;
	EnterCriticalSection(&fullscreenLock);
	screen = fullscreen;
	LeaveCriticalSection(&fullscreenLock);

	return(screen);
}

// *****************************************************************************
// outputVideoClass::setOutputWindowText()
//
// 
// *****************************************************************************
bool outputVideoClass::setOutputWindowText(char * tempStr)
{

	if(visible)
		{
		SetWindowText(outputhWnd,tempStr); 
		}
	return(true);
}


// *****************************************************************************
// outputVideoClass::setOutputWindowText()
//
// 
// *****************************************************************************
bool outputVideoClass::setResize(bool onOff)
{
	
	resize= onOff;

	return(true);
}