// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// postprocess.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

// *****************************************************************************
// postProcessClass::postProcessClass()
//
// conversion class constructor
// *****************************************************************************
postProcessClass::postProcessClass()
{
	pathToDll = new char [MAX_PATH];

	outputWidth=720;
	outputHeight=480;

	flaskDll = NULL;
	flaskInit = NULL;
	flaskDeinterlace = NULL;
	flaskStartDeinterlacer = NULL;
	flaskStopDeinterlacer = NULL;
	flaskStartResizer = NULL;
	flaskStopResizer = NULL;
	flaskResize	= NULL;
	flaskCrop = NULL;
	flaskDIBconvert_24_to_32 = NULL;
	flaskStartIVTC = NULL;
	flaskIVTCProcessIn = NULL;
	flaskIVTCProcessOut = NULL;
	flaskYuvResize = NULL;
	processBuffer[0] = NULL;
	processBuffer[1] = NULL; 
	renderSubtitles = NULL;

	deInterlaceInit = false;
	resizerInit = false;

	ivtc = false;

}

// *****************************************************************************
// postProcessClass::~postProcessClass()
//
// conversion class destructor
// *****************************************************************************
postProcessClass::~postProcessClass()
{
	end();
	delete [] pathToDll;
}

// *****************************************************************************
// postProcessClass::init()
//
// *****************************************************************************
bool postProcessClass::init()
{
	strcpy(pathToDll,globals->pathToExeDir);
	strcat(pathToDll,"\\ncodefls.dll");

	flaskDll = LoadLibrary(pathToDll);

	if(flaskDll==NULL)
		{
		// If failed, make a second attempt at debug env location
		#ifdef _DEBUG
		strcpy(pathToDll,globals->pathToExeDir);
		strcat(pathToDll,"\\..\\ncodefls\\Debug\\ncodefls.dll");
		#else
		strcpy(pathToDll,globals->pathToExeDir);
		strcat(pathToDll,"\\..\\ncodefls\\Release\\ncodefls.dll");
		#endif
		flaskDll=LoadLibrary(pathToDll);
		if(flaskDll==NULL)
			return(false);
		}

	flaskInit = (FLASKINIT) GetProcAddress(flaskDll, TEXT_FLASKINIT);
	flaskDeinterlace = (FLASKDEINTERLACE) GetProcAddress(flaskDll, TEXT_FLASKDEINTERLACE);
	flaskStartDeinterlacer = (FLASKSTARTDEINTERLACER) GetProcAddress(flaskDll, TEXT_FLASKSTARTDEINTERLACER);
	flaskStopDeinterlacer = (FLASKSTOPDEINTERLACER) GetProcAddress(flaskDll, TEXT_FLASKSTOPDEINTERLACER);
	flaskStartResizer = (FLASKSTARTRESIZER) GetProcAddress(flaskDll, TEXT_FLASKSTARTRESIZER);
	flaskStopResizer = (FLASKSTOPRESIZER) GetProcAddress(flaskDll, TEXT_FLASKSTOPRESIZER);
	flaskResize	= (FLASKRESIZE) GetProcAddress(flaskDll, TEXT_FLASKRESIZE);
	flaskCrop = (FLASKCROP) GetProcAddress(flaskDll, TEXT_FLASKCROP);
	flaskDIBconvert_24_to_32 = (FLASKDIBCONVERT_24_TO_32) GetProcAddress(flaskDll, TEXT_FLASKDIBCONVERT_24_TO_32);
	flaskStartIVTC = (FLASKSTARTIVTC) GetProcAddress(flaskDll, TEXT_FLASKSTARTIVTC);
	flaskIVTCProcessIn = (FLASKIVTCPROCESSIN) GetProcAddress(flaskDll, TEXT_FLASKIVTCPROCESSIN);
	flaskIVTCProcessOut = (FLASKIVTCPROCESSOUT) GetProcAddress(flaskDll, TEXT_FLASKIVTCPROCESSOUT);
	flaskYuvResize	= (FLASKYUVRESIZE) GetProcAddress(flaskDll, TEXT_FLASKYUVRESIZE);

	if(!flaskDll || !flaskInit || !flaskDeinterlace || !flaskStartDeinterlacer || !flaskStopDeinterlacer ||
		!flaskStartResizer || !flaskStopResizer || !flaskResize || !flaskCrop || !flaskDIBconvert_24_to_32 ||
		!flaskIVTCProcessOut || !flaskIVTCProcessIn || !flaskStartIVTC || !flaskYuvResize)
		{
		FreeLibrary(flaskDll);
		flaskDll = NULL;
		return(false);
		}

	renderSubtitles = new renderSubtitlesClass();
	renderSubtitles->init();
	return(true);
}

// *****************************************************************************
// postProcessClass::end()
//
// *****************************************************************************
void postProcessClass::end()
{
	if(flaskDll)
		{
		if(deInterlaceInit)
			{
			flaskStopDeinterlacer();
			deInterlaceInit = false;
			}
		if(resizerInit)
			{
			flaskStopResizer();
			resizerInit = false;
			}
		FreeLibrary(flaskDll);
		}
	if(processBuffer[0])
		{
		if(processBuffer[0]->allocStart)
			{
			delete [] processBuffer[0]->allocStart;
			processBuffer[0]->allocStart = NULL;
			}
		delete [] processBuffer[0];
		}
	if(processBuffer[1])
		{
		if(processBuffer[1]->allocStart)
			{
			delete [] processBuffer[1]->allocStart;
			processBuffer[1]->allocStart = NULL;
			}
		delete [] processBuffer[1];
		}

	if(renderSubtitles)
		{
		delete renderSubtitles;
		}
}

// *****************************************************************************
// postProcessClass::setDeinterlace()
//
// on = TRUE for deinterlace is ENABLED
// blend = TRUE for BLEND
// threshold = 0-? 20 is a known working value
// *****************************************************************************
void postProcessClass::setDeinterlace(bool setOn, bool setBlend, int setThreshold)
{
	deinterlace = setOn;
	blend = setBlend;
	threshold = setThreshold;
}

// *****************************************************************************
// postProcessClass::setDeinterlace()
//
// on = TRUE for deinterlace is ENABLED
// blend = TRUE for BLEND
// threshold = 0-? 20 is a known working value
// *****************************************************************************
void postProcessClass::setIvtc(bool setOn, bool setDeComb, int setOffset, bool setPolarity)
{
	ivtc = setOn;
	ivtcDeComb = setDeComb;
	ivtcOffset = setOffset;
	ivtcPolarity = setPolarity;
}


// *****************************************************************************
// postProcessClass::setHyperScale()
//
// on = TRUE for hyperScale
// width = aspect ratio adjusted width to use
// height = aspect ratio adjusted height to use
// *****************************************************************************
void postProcessClass::setHyperScale(bool setOn, double aspect, bool aspectOverride)
{
	hyperScale = setOn;
	aspectRatio = aspect;
	aspectOverrideDone = aspectOverride;
}

// *****************************************************************************
// postProcessClass::forceMultiple16()
//
// on = TRUE for deinterlace is ENABLED
// blend = TRUE for BLEND
// threshold = 0-? 20 is a known working value
// *****************************************************************************
void postProcessClass::setForceMultiple16(bool setOn)
{
	forceMultiple16 = setOn;
}

// *****************************************************************************
// postProcessClass::setOutputDimentions()
//
// on = TRUE for scale is enabled
// width = output width
// height = output height
// *****************************************************************************
void postProcessClass::setOutputDimentions(bool setOn,int type, int by, int width, int height)
{
	scale = setOn;
	if(scale)
		{
		scaleWidth = width;
		scaleHeight = height;
		scaleType = type;
		scaleBy = by;
		}
	else
		{
		scaleWidth = 0;
		scaleHeight = 0;
		}
}

// *****************************************************************************
// postProcessClass::setCrop()
//
// on = TRUE for scale is enabled
// width = output width
// height = output height
// *****************************************************************************
void postProcessClass::setCrop(int top, int bottom, int left, int right)
{
	cropTop = top;
	cropBottom = bottom;
	cropLeft = left;
	cropRight = right;

}

// *****************************************************************************
// postProcessClass::getOutputDimentions()
//
// on = TRUE for scale is enabled
// width = output width
// height = output height
// *****************************************************************************
void postProcessClass::getOutputDimentions(int * width, int * height)
{

	*width = outputWidth;
	*height = outputHeight;
}

// *****************************************************************************
// postProcessClass::getOutputFramerate()
//
// *****************************************************************************
float postProcessClass::getOutputFramerate()
{

	return(outputFramerate);
}

// *****************************************************************************
// postProcessClass::begin()
//
// width = input frame width
// height = input frame height
// *****************************************************************************
bool postProcessClass::begin(int width, int height, float framerate, int colorSpace, bool dualCpu, CONVERTCONFIG * settings)
{
	// Finish the initalization process
	double percentage;
	int correctedWidth;
	int correctedHeight;
	double diff;
	int i;
	int stride;

	inputWidth = width;
	inputHeight = height;
	inputFramerate = framerate;
	inputColorSpace = colorSpace;
	dualMode = dualCpu;

	ivtcFrameCount =1;

	if(ivtc)
		{
		outputFramerate = framerate * 0.8f;
		}
	else
		{
		outputFramerate = framerate; 
		}

	// If scaling, scale output is used.
	if(hyperScale)
		{
		correctedHeight = height;
		correctedWidth = (int)((double)height * aspectRatio);	

		if(!aspectOverrideDone)
			{
			// If the aspect ratio is 4:3
			if(settings->aspectRatioDontCorrect43)
				{
				diff = (aspectRatio  - fabs(4.0f / 3.0f));
				if(diff < 0.001f && diff > -0.001f)
					{
					// and our input AR is 1.5, then let let the source slide as being correct
					// This is to detect 720x480 source (AR 1.5) with an AR of 1.33 in the file
					diff = (fabs((double)width / (double)height) - fabs(720.0f / 480.0f));
					if(diff <0.001f && diff > -0.001f)
						{
						correctedHeight = height;
						correctedWidth = width;
						}
					}
				}
			}
		}
	else
		{
		correctedHeight = height;
		correctedWidth = width; 			
		}

	correctedWidth-=(cropLeft+cropRight);
	correctedHeight-=(cropTop+cropBottom);

	scaleOutputWidth = correctedWidth;
	scaleOutputHeight = correctedHeight;
	
	//Correct for a scale that's going up

	if(scaleOutputWidth>inputWidth)
		{
		percentage =  (double)inputWidth / (double)scaleOutputWidth;
		scaleOutputWidth = (int)(scaleOutputWidth * percentage);
		scaleOutputHeight = (int)(scaleOutputHeight * percentage);
		}

	if(scaleOutputHeight>inputHeight)
		{
		percentage =  (double)inputHeight / (double)scaleOutputHeight;
		scaleOutputWidth = (int)(scaleOutputWidth * percentage);
		scaleOutputHeight = (int)(scaleOutputHeight * percentage);
		}


	if(scale)
		{
		switch(scaleBy)
			{
			case SCALE_BY_PERCENTAGE :
				if(hyperScale)
					{
					scaleHeight = scaleWidth;
					}				
				if(scaleWidth>100)
					scaleWidth=100;
				if(scaleHeight>100)
					scaleHeight=100;
				if(scaleWidth<1)
					scaleWidth=100;
				if(scaleHeight<1)
					scaleHeight=100;
				scaleOutputWidth = (int)(((double) scaleOutputWidth / 100.0f) * (double) scaleWidth);
				scaleOutputHeight = (int)(((double) scaleOutputHeight / 100.0f) * (double) scaleHeight);
				break;
			case SCALE_BY_PIXELS :
				if(hyperScale)
					{
					if(scaleWidth>inputWidth)
						scaleWidth=inputWidth;
					if(scaleHeight>inputHeight)
						scaleHeight=inputHeight;
					percentage = (double) scaleWidth / (double)scaleOutputWidth; 
					scaleOutputWidth = scaleWidth;
					scaleOutputHeight = (int)((double) scaleOutputHeight * percentage);
					}
				else				
					{
					if(scaleWidth>inputWidth)
						scaleWidth=inputWidth;
					if(scaleHeight>inputHeight)
						scaleHeight=inputHeight;
					scaleOutputWidth = scaleWidth;
					scaleOutputHeight = scaleHeight;
					}
				break;
			default :
				break;
			}

		//Force scale output to a multiple of 16
		if(forceMultiple16)
			{
			// Force to multiple of 16
			scaleOutputHeight = ((scaleOutputHeight) / 16) * 16;
			scaleOutputWidth = ((scaleOutputWidth) / 16) * 16;
			}

		switch(inputColorSpace)
			{
			case rgb32Frame:
				scaleOutputWidth&=0xfffffffe;
				scaleOutputHeight&=0xfffffffe;
				break;
			case yv12Frame:
				scaleOutputWidth&=0xfffffff8;
				scaleOutputHeight&=0xfffffff8;
				break;				
			}
		}


	outputWidth = scaleOutputWidth;
	outputHeight = scaleOutputHeight;

	// Output is always 32 BPP.
	switch(inputColorSpace)
		{
		case rgb32Frame:
			outputBpp = 32;
			stride = inputWidth*(outputBpp/8);
			break;
		case yuy2Frame:
			outputBpp = 16;
			stride = inputWidth*(outputBpp/8);
			break;
		case yv12Frame:
			outputBpp = 16;
			stride = inputWidth;
			break;
		}

	for(i=0;i<2;i++)
		{
		processBuffer[i] = new FRAMEDATA [1];
		processBuffer[i]->allocStart = new unsigned char [(inputWidth*inputHeight*(outputBpp/8))+16];
		processBuffer[i]->bufferStart = (unsigned char *)((unsigned long)processBuffer[i]->allocStart+15 &0xfffffff0);
		processBuffer[i]->bufferSize = inputWidth*inputHeight*(outputBpp/8);
		processBuffer[i]->data = processBuffer[i]->bufferStart;
		processBuffer[i]->width = inputWidth;
		processBuffer[i]->stride = stride;	
		processBuffer[i]->height = inputHeight;
		processBuffer[i]->bitDepth = outputBpp;
		processBuffer[i]->type=inputColorSpace;
		}

	switch(inputColorSpace)
		{
		case rgb32Frame:
			if(ivtc)
				{
				flaskStartIVTC(processBuffer[0],ivtcDeComb,ivtcOffset,ivtcPolarity);
		
				flaskStartDeinterlacer(processBuffer[0], blend, threshold);
				deInterlaceInit = true;
				}
			else
				{
				processBuffer[0]->width = width-(cropLeft+cropRight);
				processBuffer[0]->height = height-(cropTop+cropBottom);
				processBuffer[0]->stride = processBuffer[0]->width*(outputBpp/8);;	

				flaskStartDeinterlacer(processBuffer[0], blend, threshold);
				deInterlaceInit = true;
				}

			processBuffer[0]->width = width-(cropLeft+cropRight);
			processBuffer[0]->height = height-(cropTop+cropBottom);
			processBuffer[0]->stride = processBuffer[0]->width*(outputBpp/8);;	
			
			flaskStartResizer(scaleOutputWidth, scaleOutputHeight, scaleType, processBuffer[0]);
			resizerInit = true;
			break;

		case yuy2Frame:
			//Doing YUY2 Post Process Init
			outputWidth = inputWidth;
			outputHeight = inputHeight;
			outputFramerate = inputFramerate;
			break;

		case yv12Frame:

			flaskStartDeinterlacer(processBuffer[0], blend, threshold);
			deInterlaceInit = true;

			//Doing YV12 Post Process Init
			if((cropLeft | cropRight | cropTop | cropBottom))
				{	
				scale = true;
				}

			outputFramerate = inputFramerate;
			if(((inputWidth-(cropLeft+cropRight)) <scaleOutputWidth) || 
				((inputHeight-(cropTop+cropBottom)) <scaleOutputHeight))
				{
				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage30");
				globals->translate->translate(messTitle, "VidomiMessage30Title");

				vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK); 
				return(false);
				}
			break;

		}


	// Only clear the error accumulator on the first AVI creteVideoStream
	videoErrorAccum = 0.0f;

	return(true);
}


// *****************************************************************************
// postProcessClass::processFrame()
//
// Returns pointer to Frame to Write to Disk
// *****************************************************************************
FRAMEDATA * postProcessClass::processFrame(FRAMEDATA * inputFrame)
{
	switch(inputColorSpace)
		{
		case rgb32Frame:
			return(processFrameRgb32(inputFrame));
		case yuy2Frame:
			return(processFrameYuy2(inputFrame));
		case yv12Frame:
			return(processFrameYv12(inputFrame));
		default:
			return(0);
		}
}

// *****************************************************************************
// postProcessClass::processFrameYuy2()
//
// Returns pointer to Frame to Write to Disk
// *****************************************************************************
FRAMEDATA * postProcessClass::processFrameYuy2(FRAMEDATA * inputFrame)
{
	int frameOut;
	int frameIn;
	int tempPts;
	int	temp;

	processBuffer[2] = inputFrame;
	frameOut = 0;	//frame out is buffer 0
	frameIn = 2;	//frame In is our fake ptr.
	
	// Reset Buffers to starting values
	processBuffer[0]->bitDepth=inputFrame->bitDepth;
	processBuffer[0]->height=inputFrame->height;
	processBuffer[0]->width=inputFrame->width;
	processBuffer[0]->stride=inputFrame->width*(inputFrame->bitDepth/8);
	processBuffer[0]->data = processBuffer[0]->bufferStart;
	processBuffer[1]->bitDepth=inputFrame->bitDepth;
	processBuffer[1]->height=inputFrame->height;
	processBuffer[1]->width=inputFrame->width;
	processBuffer[1]->stride=inputFrame->width*(inputFrame->bitDepth/8);
	processBuffer[1]->data = processBuffer[1]->bufferStart;


	memcpy(processBuffer[frameOut]->bufferStart,processBuffer[frameIn]->bufferStart,processBuffer[frameIn]->bufferSize);
	frameOut=!frameOut;
	frameIn=!frameOut;

	//Carry the time code with the data
	processBuffer[frameIn]->pts = inputFrame->pts;
	processBuffer[frameIn]->hardPts = inputFrame->hardPts;
	processBuffer[frameIn]->hardPtsFrame = inputFrame->hardPtsFrame;


	// We need to keep the clock counting, so add to the clock based on our framerate
	tempPts = (int) (90000.0f / inputFramerate); 

	// And accumulate the error so that we stay in perfect sync
	videoErrorAccum += ((90000.0f / inputFramerate) - tempPts); 
	if(videoErrorAccum>1)
		{
		temp = (int) videoErrorAccum;
		tempPts += temp;
		videoErrorAccum -=tempPts;
		}

	inputFrame->pts += (int) tempPts;


	return(processBuffer[frameIn]);

}

// *****************************************************************************
// postProcessClass::processFrameYv12()
//
// Returns pointer to Frame to Write to Disk
// *****************************************************************************
FRAMEDATA * postProcessClass::processFrameYv12(FRAMEDATA * inputFrame)
{
	int frameOut;
	int frameIn;
	int tempPts;
	int	temp;
	int size;

	processBuffer[2] = inputFrame;
	frameOut = 0;	//frame out is buffer 0
	frameIn = 2;	//frame In is our fake ptr.
	
	// Reset Buffers to starting values
	processBuffer[0]->bitDepth=inputFrame->bitDepth;
	processBuffer[0]->height=inputFrame->height;
	processBuffer[0]->width=inputFrame->width;
	processBuffer[0]->stride=inputFrame->width;
	processBuffer[0]->data = processBuffer[0]->bufferStart;

	processBuffer[1]->bitDepth=inputFrame->bitDepth;
	processBuffer[1]->height=inputFrame->height;
	processBuffer[1]->width=inputFrame->width;
	processBuffer[1]->stride=inputFrame->width;
	processBuffer[1]->data = processBuffer[1]->bufferStart;

	if(deinterlace)
		{
		flaskDeinterlace(processBuffer[frameIn],processBuffer[frameOut]);
		frameOut=!frameOut;
		frameIn=!frameOut;
		}

	if(scale)
		{
		processBuffer[frameOut]->width=scaleOutputWidth;
		processBuffer[frameOut]->height=scaleOutputHeight;
		flaskYuvResize(processBuffer[frameIn],processBuffer[frameOut],cropLeft,cropRight,cropTop,cropBottom);
		frameOut=!frameOut;
		frameIn=!frameOut;
		}
		
	if(!scale && !deinterlace && dualMode)
		{
		// This will only happen if no crop or scale is being done on dual CPU's.
		// On Dual CPU's the data must be copied out of the buffer for some operation
		size = processBuffer[frameIn]->width * processBuffer[frameIn]->height;
		size += (processBuffer[frameIn]->width/2) * (processBuffer[frameIn]->height/2);
		size += (processBuffer[frameIn]->width/2) * (processBuffer[frameIn]->height/2);
		memcpy(processBuffer[frameOut]->bufferStart,processBuffer[frameIn]->bufferStart,size);
		frameOut=!frameOut;
		frameIn=!frameOut;
		}

	//Carry the time code with the data
	processBuffer[frameIn]->pts = inputFrame->pts;
	processBuffer[frameIn]->hardPts = inputFrame->hardPts;
	processBuffer[frameIn]->hardPtsFrame = inputFrame->hardPtsFrame;


	// We need to keep the clock counting, so add to the clock based on our framerate
	tempPts = (int) (90000.0f / inputFramerate); 

	// And accumulate the error so that we stay in perfect sync
	videoErrorAccum += ((90000.0f / inputFramerate) - tempPts); 
	if(videoErrorAccum>1)
		{
		temp = (int) videoErrorAccum;
		tempPts += temp;
		videoErrorAccum -=tempPts;
		}

	inputFrame->pts += (int) tempPts;


	return(processBuffer[frameIn]);

}
// *****************************************************************************
// postProcessClass::processFrameRgb32()
//
// Returns pointer to Frame to Write to Disk
// *****************************************************************************
FRAMEDATA * postProcessClass::processFrameRgb32(FRAMEDATA * inputFrame)
{
	int frameOut;
	int frameIn;
	int tempPts;
	int	temp;
	long ivtcFrameCountNew;

	// The trick: We set the third ptr to be the inputFrame
	// after the first swap we rewrite the frameIn to be the inverse of frameout.
	// poof ping pong buffers with a mystry third buffer
	processBuffer[2] = inputFrame;
	frameOut = 0;	//frame out is buffer 0
	frameIn = 2;	//frame In is our fake ptr.
	
	// Reset Buffers to starting values
	processBuffer[0]->bitDepth=inputFrame->bitDepth;
	processBuffer[0]->height=inputFrame->height;
	processBuffer[0]->width=inputFrame->width;
	processBuffer[0]->stride=inputFrame->width*(inputFrame->bitDepth/8);
	processBuffer[0]->data = processBuffer[0]->bufferStart;
	processBuffer[1]->bitDepth=inputFrame->bitDepth;
	processBuffer[1]->height=inputFrame->height;
	processBuffer[1]->width=inputFrame->width;
	processBuffer[1]->stride=inputFrame->width*(inputFrame->bitDepth/8);
	processBuffer[1]->data = processBuffer[1]->bufferStart;

	if(ivtc)
		{

		if(deinterlace)
			{
			flaskDeinterlace(processBuffer[frameIn],processBuffer[frameOut]);
			frameOut=!frameOut;
			frameIn=!frameOut;
			}

		ivtcFrameCountNew = flaskIVTCProcessOut(processBuffer[frameOut]);
		flaskIVTCProcessIn(processBuffer[frameIn],ivtcFrameCount);
		ivtcFrameCount++;
		if(ivtcFrameCountNew<0)
			return(NULL);
		frameOut=!frameOut;
		frameIn=!frameOut;


		if(!deinterlace && !scale)
			{
			// Do old school crop (slow)
			if((cropTop | cropBottom | cropLeft | cropRight)==0)
				{
				if(dualMode)
					{
					// On Dual CPU's the data must be copied out of the buffer for some operation
					// Since no operations were done, we do a copy
					memcpy(processBuffer[frameOut]->bufferStart,processBuffer[frameIn]->bufferStart,processBuffer[frameIn]->bufferSize);
					frameOut=!frameOut;
					frameIn=!frameOut;
					}
				}
			else
				{
				if(flaskDll)
					{
					flaskCrop(processBuffer[frameIn],processBuffer[frameOut], 
								cropTop, cropBottom, cropLeft, cropRight);

					frameOut=!frameOut;
					frameIn=!frameOut;

					processBuffer[frameOut]->bitDepth=outputBpp;
					processBuffer[frameOut]->height=inputHeight-(cropTop+cropBottom);
					processBuffer[frameOut]->width=inputWidth-(cropLeft+cropRight);
					processBuffer[frameOut]->stride=processBuffer[frameOut]->width*(outputBpp/8);
					processBuffer[frameOut]->data = processBuffer[frameOut]->bufferStart;
					processBuffer[frameIn]->bitDepth=outputBpp;
					processBuffer[frameIn]->height=inputHeight-(cropTop+cropBottom);
					processBuffer[frameIn]->width=inputWidth-(cropLeft+cropRight);
					processBuffer[frameIn]->stride=processBuffer[frameIn]->width*(outputBpp/8);
					processBuffer[frameIn]->data = processBuffer[frameIn]->bufferStart;
					}
				}
			}

		if(scale)
			{
			//then it's on us to pointer crop
			processBuffer[frameIn]->data += cropBottom * (inputWidth * (outputBpp/8));
			processBuffer[frameIn]->data += cropLeft * (outputBpp/8);
			processBuffer[frameIn]->width=inputWidth-(cropRight+cropLeft);
			processBuffer[frameIn]->height=inputHeight-(cropTop+cropBottom);

			processBuffer[frameOut]->stride=scaleOutputWidth*(outputBpp/8);
			processBuffer[frameOut]->width=scaleOutputWidth;
			processBuffer[frameOut]->height=scaleOutputHeight;
			flaskResize(processBuffer[frameIn],processBuffer[frameOut]);
			frameOut=!frameOut;
			frameIn=!frameOut;
			}
		}
	else
		{
		
		if(deinterlace)
			{
			//pointer crop
			processBuffer[frameIn]->data += cropBottom * (inputWidth * (outputBpp/8));
			processBuffer[frameIn]->data += cropLeft * (outputBpp/8);
			processBuffer[frameIn]->width=inputWidth-(cropRight+cropLeft);
			processBuffer[frameIn]->height=inputHeight-(cropTop+cropBottom);
			processBuffer[frameOut]->bitDepth=outputBpp;
			processBuffer[frameOut]->height=inputHeight-(cropTop+cropBottom);
			processBuffer[frameOut]->width=inputWidth-(cropLeft+cropRight);
			processBuffer[frameOut]->stride=processBuffer[frameOut]->width*(outputBpp/8);
			processBuffer[frameOut]->data = processBuffer[frameOut]->bufferStart;

			if(0)
				{
				duplicateEvenLines(processBuffer[frameIn],processBuffer[frameOut]);
				}
			else
				{
				flaskDeinterlace(processBuffer[frameIn],processBuffer[frameOut]);
				}
			frameOut=!frameOut;
			frameIn=!frameOut;
			}

		if(!deinterlace && !scale)
			{
			if((cropTop | cropBottom | cropLeft | cropRight)==0)
				{
				if(dualMode)
					{
					// On Dual CPU's the data must be copied out of the buffer for some operation
					// Since no operations were done, we do a copy
					memcpy(processBuffer[frameOut]->bufferStart,processBuffer[frameIn]->bufferStart,processBuffer[frameIn]->bufferSize);
					frameOut=!frameOut;
					frameIn=!frameOut;
					}
				}
			else
				{
				// Do old school crop (slow)
				if(flaskDll)
					{
					flaskCrop(processBuffer[frameIn],processBuffer[frameOut], 
								cropTop, cropBottom, cropLeft, cropRight);

					frameOut=!frameOut;
					frameIn=!frameOut;

					processBuffer[frameOut]->bitDepth=32;
					processBuffer[frameOut]->height=inputHeight-(cropTop+cropBottom);
					processBuffer[frameOut]->width=inputWidth-(cropLeft+cropRight);
					processBuffer[frameOut]->stride=processBuffer[frameOut]->width*(outputBpp/8);
					processBuffer[frameOut]->data = processBuffer[frameOut]->bufferStart;
					processBuffer[frameIn]->bitDepth=32;
					processBuffer[frameIn]->height=inputHeight-(cropTop+cropBottom);
					processBuffer[frameIn]->width=inputWidth-(cropLeft+cropRight);
					processBuffer[frameIn]->stride=processBuffer[frameIn]->width*(outputBpp/8);
					processBuffer[frameIn]->data = processBuffer[frameIn]->bufferStart;
					}
				}
			}

		if(scale)
			{
			if(!deinterlace)
				{
				//then it's on us to pointer crop
				processBuffer[frameIn]->data += cropBottom * (inputWidth * (outputBpp/8));
				processBuffer[frameIn]->data += cropLeft * (outputBpp/8);
				processBuffer[frameIn]->width=inputWidth-(cropRight+cropLeft);
				processBuffer[frameIn]->height=inputHeight-(cropTop+cropBottom);
				}

			processBuffer[frameOut]->stride=scaleOutputWidth*(outputBpp/8);
			processBuffer[frameOut]->width=scaleOutputWidth;
			processBuffer[frameOut]->height=scaleOutputHeight;
			flaskResize(processBuffer[frameIn],processBuffer[frameOut]);
			frameOut=!frameOut;
			frameIn=!frameOut;
			}

		}


	//Carry the time code with the data
	processBuffer[frameIn]->pts = inputFrame->pts;
	processBuffer[frameIn]->hardPts = inputFrame->hardPts;
	processBuffer[frameIn]->hardPtsFrame = inputFrame->hardPtsFrame;


	// We need to keep the clock counting, so add to the clock based on our framerate
	tempPts = (int) (90000.0f / inputFramerate); 

	// And accumulate the error so that we stay in perfect sync
	videoErrorAccum += ((90000.0f / inputFramerate) - tempPts); 
	if(videoErrorAccum>1)
		{
		temp = (int) videoErrorAccum;
		tempPts += temp;
		videoErrorAccum -=tempPts;
		}

	inputFrame->pts += (int) tempPts;


	return(processBuffer[frameIn]);
}

// *****************************************************************************
// postProcessClass::Rgb24ToRgb32()
//
// *****************************************************************************

void postProcessClass::rgb24ToRgb32(FRAMEDATA * srcFrame, FRAMEDATA * dstFrame)
{
	// Use Virtual Dub's convert to do the conversion

	dstFrame->bitDepth=32;
	dstFrame->height = srcFrame->height;
	dstFrame->width = srcFrame->width;
	dstFrame->stride = srcFrame->width*4;
	dstFrame->type = srcFrame ->type;

	flaskDIBconvert_24_to_32(dstFrame->data, dstFrame->stride, srcFrame->data, srcFrame->stride, srcFrame->width, srcFrame->height);

}


// *****************************************************************************
// postProcessClass::duplicateEvenLines()
//
// *****************************************************************************

void postProcessClass::duplicateEvenLines(FRAMEDATA * srcFrame, FRAMEDATA * dstFrame)
{
	unsigned char * ptr1;
	unsigned char * ptr2;
	int i;

	ptr1 = srcFrame->data;
	ptr2 = dstFrame->data;

	dstFrame->bitDepth=32;
	dstFrame->height = srcFrame->height;
	dstFrame->width = srcFrame->width;
	dstFrame->stride = srcFrame->width*4;

	for(i=0;i<(srcFrame->height/2);i++)
		{
		// duplicate even lines to even and odd lines in output buffer
		memcpy(ptr2,ptr1,srcFrame->width*4);
		ptr2+=dstFrame->stride;
		memcpy(ptr2,ptr1,srcFrame->width*4);
		ptr1+=srcFrame->stride;
		ptr1+=srcFrame->stride;
		ptr2+=dstFrame->stride;
		}

}



// *****************************************************************************
// postProcessClass::Rgb32ToRgb24()
//
// *****************************************************************************

void postProcessClass::rgb32ToRgb24(FRAMEDATA * srcFrame, FRAMEDATA * dstFrame)
{
	unsigned char * src;
	unsigned char * dst;
	unsigned char * src1;
	unsigned char * dst1;
	int w,h;


	dstFrame->bitDepth=24;
	dstFrame->height = srcFrame->height;
	dstFrame->width = srcFrame->width;
	dstFrame->stride = srcFrame->width*3;
	dstFrame->type = srcFrame ->type;

	src = srcFrame->data;
	dst = dstFrame->data;

	for(h=0;h<srcFrame->height;h++)
		{
		src1 = src;
		dst1 = dst;
		for(w=0;w<srcFrame->width;w++)
			{
			*dst1++=*src1++;
			*dst1++=*src1++;
			*dst1++=*src1++;
			*src1++;
			}
		src += srcFrame->stride;
		dst += dstFrame->stride;
		}

}


// *****************************************************************************
// postProcessClass::Rgb32ToRgb24()
//
// *****************************************************************************

void postProcessClass::bgr24ToRgb32(FRAMEDATA * srcFrame, FRAMEDATA * dstFrame)
{
	unsigned char * src;
	unsigned char * dst;
	unsigned char * src1;
	unsigned char * dst1;
	int w,h;
	unsigned char r,g,b;

	dstFrame->bitDepth=24;
	dstFrame->height = srcFrame->height;
	dstFrame->width = srcFrame->width;
	dstFrame->stride = srcFrame->width*4;
	dstFrame->type = rgb32Frame;

	
	src = srcFrame->data;
	dst = dstFrame->data;

	for(h=0;h<srcFrame->height;h++)
		{
		src1 = src;
		dst1 = dst;
		for(w=0;w<srcFrame->width;w++)
			{
			b = *src1++;
			g = *src1++;
			r = *src1++;
			*dst1++=g;
			*dst1++=r;
			*dst1++=b;
			*dst1++=0;
			}
		src += srcFrame->stride;
		dst += dstFrame->stride;
		}

}



