// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
// postprocess class
// *****************************************************************************
#ifndef postprocess_h
#define postprocess_h

#define SCALE_BY_PIXELS 1
#define SCALE_BY_PERCENTAGE 2

class postProcessClass 
{

	public:
									postProcessClass();
									~postProcessClass();
			bool					init();
			bool					begin(int width, int height, float framerate, int colorSpace, bool dualCpu, CONVERTCONFIG * settings);
			void					setDeinterlace(bool setOn, bool setBlend, int setThreshold);
			void					setHyperScale(bool setOn, double aspectRatio, bool aspectOverride);
			void					setOutputDimentions(bool setOn,int type, int by, int width, int height);
			void					setCrop(int top, int bottom, int left, int right);
			void					getOutputDimentions(int * outputWidth, int * outputHeight);
			void					setForceMultiple16(bool setOn);
			FRAMEDATA *				processFrame(FRAMEDATA * inputFrame);
			FRAMEDATA *				processFrameYuy2(FRAMEDATA * inputFrame);
			FRAMEDATA *				processFrameRgb32(FRAMEDATA * inputFrame);
			FRAMEDATA *				processFrameYv12(FRAMEDATA * inputFrame);


			void					end();
			void					rgb24ToRgb32(FRAMEDATA * srcFrame, FRAMEDATA * dstFrame);
			void					rgb32ToRgb24(FRAMEDATA * srcFrame, FRAMEDATA * dstFrame);
			void					bgr24ToRgb32(FRAMEDATA * srcFrame, FRAMEDATA * dstFrame);
			void					duplicateEvenLines(FRAMEDATA * srcFrame, FRAMEDATA * dstFrame);
			float					getOutputFramerate();
			void					setIvtc(bool setOn, bool setDeComb, int setOffset, bool setPolarity);

	private:
			FRAMEDATA *				processBuffer[3];
			FRAMEDATA *				subtitleFrame;
			int						inputWidth;
			int						inputHeight;
			float					inputFramerate;
			int						inputColorSpace;
			int						outputWidth;
			int						outputHeight;
			float					outputFramerate;
			int						outputBpp;
			bool					blend;
			int						threshold;
			bool					deinterlace;
			bool					scale;
			int						scaleType;
			int						scaleBy;
			int						scaleWidth;
			int						scaleHeight;
			bool					hyperScale;
			int						hyperScaleWidth;
			int						hyperScaleHeight;
			bool					crop;
			int						cropTop;
			int						cropBottom;
			int						cropLeft;
			int						cropRight;
			HINSTANCE				flaskDll;
			FLASKINIT				flaskInit;
			FLASKDEINTERLACE		flaskDeinterlace;
			FLASKSTARTDEINTERLACER	flaskStartDeinterlacer;
			FLASKSTOPDEINTERLACER	flaskStopDeinterlacer;
			FLASKSTARTRESIZER		flaskStartResizer;
			FLASKSTOPRESIZER		flaskStopResizer;
			FLASKRESIZE				flaskResize;
			FLASKCROP				flaskCrop;
			FLASKDIBCONVERT_24_TO_32 flaskDIBconvert_24_to_32;
			FLASKSTARTIVTC			flaskStartIVTC;
			FLASKIVTCPROCESSIN		flaskIVTCProcessIn;
			FLASKIVTCPROCESSOUT		flaskIVTCProcessOut;
			FLASKYUVRESIZE			flaskYuvResize;
			char *					pathToDll;
			bool					forceMultiple16;
			int						scaleOutputWidth;
			int						scaleOutputHeight;
			double					videoErrorAccum;
			double					aspectRatio;
			bool					aspectOverrideDone;
			bool					ivtc;
			bool					ivtcDeComb;
			int						ivtcOffset;
			bool					ivtcPolarity;
			long					ivtcFrameCount;
			bool					deInterlaceInit;
			bool					resizerInit;
			bool					dualMode;
			renderSubtitlesClass *	renderSubtitles;
			int						subtitleFrameCounter;

};



#endif