// *****************************************************************************
//
//
// readavi.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;
char * aviGuid = {"00020000-0000-0000-C000-000000000046"};
HIC videoHic;

// *****************************************************************************
// readAviClass::readAviClass()
//
// conversion class constructor
// *****************************************************************************
readAviClass::readAviClass()
{
	int i;

	audioIn = NULL;
	audioOut = NULL;
	videoHic = NULL;

	frameIn = NULL;

	notDone = true;
	aviInitialized = false;
	inputFileIndex =0;
	for(i=0;i<16;i++)
		{
		inputFiles[i] = new char [MAX_PATH];
		}
	audio = NULL;
	frame = NULL;
	divx = NULL;
}

// *****************************************************************************
// readAviClass::~readAviClass()
//
// conversion class destructor
// *****************************************************************************
readAviClass::~readAviClass()
{

}

// *****************************************************************************
// readAviClass::init()
//
// *****************************************************************************
bool readAviClass::init()
{
	int channel;

	readFileIndex = 0;
	aviReadMode = AVI_READ_MODE_PLAYER_NONE;
	divx = new divxClass();
	notDone = true;

	for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
		{
		audioFormatIn[channel] = (MPEGLAYER3WAVEFORMAT *) new char [AUDIO_FORMAT_BUFFER_SIZE];
		}

	audioAcmStream = NULL;

	return(true);
}

// *****************************************************************************
// readAviClass::end()
//
// *****************************************************************************
void readAviClass::end()
{
	int i;
	int channel;

	if(videoHic)
		{
		ICDecompressEnd(videoHic);
		//videoHic->unused = 0;
		ICClose(videoHic);
		}

	if(audio)
		{
		delete [] audio->bufferStart;
		delete [] audio;
		}

	if(audioAcmStream)
		{
		acmStreamUnprepareHeader(audioAcmStream,&audioAcmHeader,0);
		acmStreamClose(audioAcmStream,0);
		}

	if(frame)
		{
		delete [] frame->allocStart;
	//	delete [] frameIn->windowsBitmapInfo;
		delete [] frame;
		}

	if(audioIn)
		{
		delete [] audioIn->bufferStart;
		delete [] audioIn;
		}

	if(audioOut)
		{	
		delete [] audioOut->bufferStart;
		delete [] audioOut;
		}

	if(frameIn)
		{
		delete [] frameIn->allocStart;
	//	delete [] frameIn->windowsBitmapInfo;
		delete [] frameIn;
		}

	for(i=0;i<16;i++)
		{
		*inputFiles[i] = 0;
		}

	for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
		{
		delete [] audioFormatIn[channel];
		}

	divx->end();
	delete divx;

}

// *****************************************************************************
// readAviClass::clearInput()
//
// Clear all files that have been set as input files. 
// *****************************************************************************
void readAviClass::clearInput()
{
	int i;

	inputFileIndex =0;

	for(i=0;i<16;i++)
		{
		*inputFiles[i] = 0;
		}
}


// *****************************************************************************
// readAviClass::addInput()
//
// inputFilename = path and name of input File
// *****************************************************************************
void readAviClass::addInput(char * inputFilename)
{
	strcpy(inputFiles[inputFileIndex],inputFilename);
	inputFileIndex++;
}

// *****************************************************************************
// readAviClass::begin()
//
// *****************************************************************************
bool readAviClass::begin(int mode)
{
	int ret;
	MMRESULT retmm;
	DWORD icRet;
	bool divxCodec;
	bool fakeoutCodec;
	int channel;
	int audioChannelWanted;
	int videoChannelWanted;
	DIVXSTREAMHEADER myVideoHeader;

	switch(mode)
		{
		case yv12Frame:
			aviReadMode = AVI_READ_MODE_PLAYER_YV12;
			break;
		case rgb32Frame:
		default:
			aviReadMode = AVI_READ_MODE_PLAYER_RGB32;
			break;
		}
	frameCount = 0;
	hasAudio = false;
	hasVideo = false;
	fakeoutCodec = false;
	decodeFirstTime	= true;

	audioChannelWanted = 0;
	videoChannelWanted = 0;

	divx->init();

	// Setup the Overlay output
	if(ret=divx->openRead(inputFiles[readFileIndex]))
		{
		
		logPrintf("Divx->openRead Failed with %d - %s\n",ret,divx->getErrorString(ret));
		return(false);
		}

	numVideoChannels = divx->getVideoStreamCount();
	numAudioChannels = divx->getAudioStreamCount();

	if(!numVideoChannels)
		{
		divx->end();
		logPrintf("No Video Streams found in source.\n");
		return(false);
		}

	if(numVideoChannels>MAX_VIDEO_CHANNELS)
		numVideoChannels = MAX_VIDEO_CHANNELS;

	if(numAudioChannels>MAX_AUDIO_CHANNELS)
		numAudioChannels = MAX_AUDIO_CHANNELS;

	if(audioChannelWanted<numAudioChannels)
		{
		currentAudioChannel = audioChannelWanted;
		}
	else
		{
		currentAudioChannel = 0;
		}

	if(videoChannelWanted<numVideoChannels)
		{
		currentVideoChannel = videoChannelWanted;
		}
	else
		{
		currentVideoChannel = 0;
		}

	for(channel=0;channel<numVideoChannels;channel++)
		{
		divx->getVideoStreamInfo(channel,&divxVideoStreamInfo[channel]);	
		height = divxVideoStreamInfo[channel].height;
		width = divxVideoStreamInfo[channel].width;
		framerate = (float) divxVideoStreamInfo[channel].framerate;
		msPerFrame = 1000.0f / framerate;
		hasVideo=true;
		}

	// Create the buffer
	frameIn = new FRAMEDATA [1];
	memset(frameIn,0,sizeof(FRAMEDATA));

	frameIn->bufferSize = (width*height*4)+16;
	frameIn->allocStart = (unsigned char *) new unsigned char [frameIn->bufferSize];
	frameIn->bufferStart = frameIn->allocStart;
	frameIn->width = width;
	frameIn->stride=width*4;
	frameIn->height = height;
	frameIn->bitDepth =32;
	frameIn->type=rgb32Frame;
	frameIn->hardPts=0;
	frameIn->data = frameIn->bufferStart;
	frameIn->windowsBitmapInfo = (BITMAPINFO *) new unsigned char [VIDOMI_BITMAP_INFO_SIZE];

	videoFormatIn = frameIn->windowsBitmapInfo;
	memset(videoFormatIn,0,VIDOMI_BITMAP_INFO_SIZE);

	
	divx->getVideoStreamFormat(currentVideoChannel,videoFormatIn);

	for(channel=0;channel<numAudioChannels;channel++)
		{
		divx->getAudioStreamFormat(channel,audioFormatIn[channel]);

		sampleRate[channel] = audioFormatIn[channel]->wfx.nSamplesPerSec;
		channels[channel] = audioFormatIn[channel]->wfx.nChannels; // note - we assume 16 bit samples in

		if(channels[channel]!=1 || channels[channel]!=2)
			channels[channel]=2;

		audioFormatOut[channel].wFormatTag=WAVE_FORMAT_PCM;

		// Start of audio decode negotiation

		ret = acmFormatSuggest(NULL, (WAVEFORMATEX *)audioFormatIn[channel], 	(WAVEFORMATEX *)&audioFormatOut[channel], sizeof(WAVEFORMATEX), ACM_FORMATSUGGESTF_WFORMATTAG);
		
		audioFormatOut[channel].nChannels = channels[channel];
		audioFormatOut[channel].nSamplesPerSec=sampleRate[channel];
		audioFormatOut[channel].nBlockAlign=channels[channel]*2;
		audioFormatOut[channel].nAvgBytesPerSec= sampleRate[channel]*channels[channel]*2;
		audioFormatOut[channel].wBitsPerSample=16;
		audioFormatOut[channel].cbSize=0;

		if(audioFormatIn[channel]->wfx.wBitsPerSample==0)
			bytesPerSample[channel]=1;
		else
			bytesPerSample[channel]=audioFormatIn[channel]->wfx.wBitsPerSample/8;
		
		hasAudio = true;
		}

	// Start of video decode negotiation

	frame = new FRAMEDATA [1];
	memset(frame,0,sizeof(FRAMEDATA));
	frame->bufferSize = (width*height*4) + 16;
	frame->allocStart = (unsigned char *) new unsigned char [frame->bufferSize];
	frame->bufferStart = frame->allocStart;
	frame->width = width;
	frame->height = height;
	frame->stride=width*4;
	frame->bitDepth =32;
	frame->type=aviReadMode;
	frame->hardPts=0;
	frame->data = frame->bufferStart;
	frame->windowsBitmapInfo = (BITMAPINFO *) new unsigned char [VIDOMI_BITMAP_INFO_SIZE];
	
	videoFormatOut = frame->windowsBitmapInfo;
	memset(videoFormatOut,0,VIDOMI_BITMAP_INFO_SIZE);

	videoFormatOut->bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	videoFormatOut->bmiHeader.biWidth=width;
	videoFormatOut->bmiHeader.biHeight=height;
	videoFormatOut->bmiHeader.biPlanes=1;
	videoFormatOut->bmiHeader.biXPelsPerMeter=0;
	videoFormatOut->bmiHeader.biYPelsPerMeter=0;
	videoFormatOut->bmiHeader.biClrUsed=0;
	videoFormatOut->bmiHeader.biClrImportant=0;
	
	if(videoFormatIn->bmiHeader.biCompression==BI_RGB)
		{
		// If it's uncompressed, then assume it's a 24bit DIB
		if(videoFormatOut->bmiHeader.biBitCount==24)
			{
			frameIn->stride = frameIn->width*3;
			frameIn->bitDepth=24;
			videoFormatOut->bmiHeader.biBitCount=32;
			}
		if(videoFormatOut->bmiHeader.biBitCount==32)
			{
			frameIn->stride = frameIn->width*4;
			frameIn->bitDepth=32;
			videoFormatOut->bmiHeader.biBitCount=32;
			}
		}
	else
		{
		if(videoFormatIn->bmiHeader.biCompression == mmioFOURCC('M', 'P', '4', '3'))
			{
			videoFormatIn->bmiHeader.biCompression = mmioFOURCC('D', 'I', 'V', '3');
			}


		divxCodec=false;
		if(videoFormatIn->bmiHeader.biCompression == mmioFOURCC('D', 'I', 'V', '3') ||	
			videoFormatIn->bmiHeader.biCompression == mmioFOURCC('D', 'I', 'V', '4'))
			{
			divxCodec=true;
			}

		if(videoFormatIn->bmiHeader.biCompression == mmioFOURCC('D', 'I', 'V', 'X'))
			{
			//globals->playerConfig->sharedConfig.invertPlayback = !globals->playerConfig->sharedConfig.invertPlayback;
			}

		switch(aviReadMode)
			{

			case AVI_READ_MODE_PLAYER_YV12 :
				if(divxCodec)
					{
					//If it's divx, we cheat and ask for 16 BIT RGB at first
					videoFormatOut->bmiHeader.biCompression=BI_RGB;
					videoFormatOut->bmiHeader.biBitCount=16;
					fakeoutCodec =true;
					}
				else
					{
					videoFormatOut->bmiHeader.biCompression = mmioFOURCC('Y', 'V', '1', '2');
					videoFormatOut->bmiHeader.biBitCount=16;
					}
				break;

			case AVI_READ_MODE_PLAYER_RGB32 :
			default :
				videoFormatOut->bmiHeader.biCompression=BI_RGB;
				videoFormatOut->bmiHeader.biBitCount=32;
				break;
			}
		
		videoFormatOut->bmiHeader.biSizeImage=(width*height)*(videoFormatOut->bmiHeader.biBitCount/8);
		videoHic = ICOpen(mmioFOURCC('V', 'I', 'D', 'C'),videoFormatIn->bmiHeader.biCompression ,ICMODE_FASTDECOMPRESS);

		if(!videoHic)
			{
			divx->getVideoStreamHeader(currentVideoChannel,&myVideoHeader);
			
			videoHic = ICOpen(mmioFOURCC('V', 'I', 'D', 'C'),myVideoHeader.fccHandler , ICMODE_DECOMPRESS);

			if(!videoHic)
				{
				videoHic = ICLocate(mmioFOURCC('V', 'I', 'D', 'C'),videoFormatIn->bmiHeader.biCompression,
						(BITMAPINFOHEADER *) videoFormatIn,(BITMAPINFOHEADER *)videoFormatOut,ICMODE_DECOMPRESS);
				}
			}

		if(videoHic)
			{		
			// If it's not divx then do the queries
			if(!fakeoutCodec)
				{
				// First try our favored mode
				if(icRet = ICDecompressQuery(videoHic,videoFormatIn,videoFormatOut)!=ICERR_OK)
					{
					// We failed to get the one we wanted, so try all combinations in order of preference. 
					videoFormatOut->bmiHeader.biCompression=BI_RGB;
					videoFormatOut->bmiHeader.biBitCount=32;
					videoFormatOut->bmiHeader.biSizeImage=(width*height)*(videoFormatOut->bmiHeader.biBitCount/8);
					if(icRet = ICDecompressQuery(videoHic,videoFormatIn,videoFormatOut)!=ICERR_OK)
						{
						videoFormatOut->bmiHeader.biBitCount=24;
						videoFormatOut->bmiHeader.biSizeImage=(width*height)*(videoFormatOut->bmiHeader.biBitCount/8);
						if(icRet = ICDecompressQuery(videoHic,videoFormatIn,videoFormatOut)!=ICERR_OK)
							{
							hasVideo=false;
							}
						}
					}
				}
			}
		else
			{
			hasVideo=false;
			}


		if(hasVideo)
			{
			icRet =	ICDecompressBegin(videoHic,videoFormatIn,videoFormatOut);

			if(fakeoutCodec)
				{
				// ignore failure error with divx
				switch(aviReadMode)
					{
					case AVI_READ_MODE_PLAYER_YV12 :
						videoFormatOut->bmiHeader.biCompression = mmioFOURCC('Y', 'V', '1', '2');
						break;
					default :
						break;
					}
				}
			else
				{
				//icRet =	ICDecompressBegin(videoHic,videoFormatIn,videoFormatOut);
		
				if(icRet !=ICERR_OK)
					hasVideo=false;
				}
			}
		}

	if(hasVideo==false)
		{
		//AVI File is bogus.
		divx->end();
		delete [] frame->allocStart;
		delete [] frame;
		delete [] frameIn->allocStart;
		delete [] frameIn;
		return(false);
		}

	if(hasAudio)
		{
		samplesPerFrame[channel] = (double) sampleRate[channel] / framerate;

		if(retmm = acmStreamOpen(&audioAcmStream,NULL,(WAVEFORMATEX *) audioFormatIn[currentAudioChannel],&audioFormatOut[currentAudioChannel],NULL,NULL,0,ACM_STREAMOPENF_NONREALTIME))
			{
			logPrintf("acmStreamOpen Failed wih result %d\n",(int)retmm);
			hasAudio = false;					
			}
		}

	if(hasAudio)
		{

		// Determine buffer sizes
		sourceAtom = 4096;

		acmStreamSize(audioAcmStream,sourceAtom,&destAtom,ACM_STREAMSIZEF_SOURCE );

		audioIn = new AUDIODATA [1];
		audioOut = new AUDIODATA [1];

		audioIn->bufferSize = sourceAtom;
		audioOut->bufferSize = destAtom;

		audioIn->bufferStart = (unsigned char *) new unsigned char [audioIn->bufferSize];
		audioIn->data = audioIn->bufferStart;
		audioIn->bytesUsed = 0;

		audioOut->bufferStart = (unsigned char *) new unsigned char [audioOut->bufferSize];
		audioOut->data = audioOut->bufferStart;
		audioOut->bytesUsed = 0;


		audioAcmHeader.cbStruct = sizeof(ACMSTREAMHEADER);
		audioAcmHeader.fdwStatus = 0;
		audioAcmHeader.dwUser =0;
		audioAcmHeader.pbSrc = (unsigned char *) audioIn->bufferStart;
		audioAcmHeader.cbSrcLength  = audioIn->bufferSize;
		audioAcmHeader.cbSrcLengthUsed  = 0;
		audioAcmHeader.dwSrcUser = 0;
		audioAcmHeader.pbDst = (unsigned char *)audioOut->bufferStart;
		audioAcmHeader.cbDstLength  = audioOut->bufferSize;
		audioAcmHeader.cbDstLengthUsed  = 0;
		audioAcmHeader.dwDstUser = 0;
				

		if(retmm = acmStreamPrepareHeader(audioAcmStream,&audioAcmHeader,0))
			{
			logPrintf("acmStreamPrepareHeader Failed %d\n");
			acmStreamClose(audioAcmStream,0);
			hasAudio = false;
			}
		audioAcmHeader.cbSrcLengthUsed  = 0;
		audioAcmHeader.cbDstLengthUsed  = 0;
		}


	startingFrame = 0;
	endingFrame = divx->getVideoStreamFrameCount(currentVideoChannel);
	endingFrame--;
	frameCount = startingFrame;

	if(hasAudio)
		{
		startingSample = 0; 
		endingSample = divx->getAudioStreamByteCount(currentAudioChannel);

		audio = new AUDIODATA [1];

		audio->bufferStart = new unsigned char [AVIREADAUDIO_BUFFER_SIZE];
		audio->bufferSize = AVIREADAUDIO_BUFFER_SIZE;
		audio->data = audio->bufferStart;
		audio->bytesUsed = 0;
		audio->headUsed = 0;
		audio->pts = 0;
		audio->type = audioBufferPcm;
		audioSamplePos = startingSample;
		audioBytesToSkip = 0;
		}
	else
		{
		// Create a dummy buffer for everyone to consume
		audio = new AUDIODATA [1];
		audio->bufferStart = new unsigned char [AVIREADAUDIO_BUFFER_SIZE];
		audio->bufferSize = AVIREADAUDIO_BUFFER_SIZE;
		audio->data = audio->bufferStart;
		audio->bytesUsed = 0;
		audio->headUsed = 0;
		audio->pts = 0;
		audio->type = audioBufferPcm;
		audioSamplePos = 0;
		audioBytesToSkip = 0;
		}

	seekFramePos(startingFrame);

	return(true);	
}

// *****************************************************************************
// readAviClass::setIdct()
//
// *****************************************************************************
void readAviClass::setIdct(int idctType)
{
	// This is not used for AVI reading
}

// *****************************************************************************
// readAviClass::setFilm()
//
// *****************************************************************************
void readAviClass::setFilm(bool film)
{
	// This is not used for AVI reading
}

// *****************************************************************************
// readAviClass::setAudio()
//
// *****************************************************************************
void readAviClass::setAudio(bool audio, int channelIndex, int audioType, int audioTrack, int streamCopy)
{
	//ignored currently
}


// *****************************************************************************
// readAviClass::checkEnd()
//
// *****************************************************************************
bool readAviClass::checkEnd()
{
	if(notDone)
		return(false);
	else
		return(true);
}

// *****************************************************************************
// readAviClass::getDimentions()
//
// width = pointer to var that will hold the width
// height = pointer to var that will hold the height
// *****************************************************************************
void readAviClass::getDimentions(int * inWidth, int * inHeight, double * aspectRatio)
{
	*inWidth = width;
	*inHeight = height;
	*aspectRatio = (double)width / (double)height;
}

// *****************************************************************************
// readAviClass::getFrameType()
//
// *****************************************************************************
int readAviClass::getFrameType(void)
{

	return(0);
}

// *****************************************************************************
// readAviClass::getDimentions()
//
// width = pointer to var that will hold the width
// height = pointer to var that will hold the height
// *****************************************************************************
void readAviClass::getAudioOutput(int channel, int * outChannels, int * outSampleRate)
{
	if(hasAudio)
		{
		*outChannels = channels[channel];
		*outSampleRate = sampleRate[channel];
		}
	else
		{
		*outChannels = 0;
		*outSampleRate = 0;
		}

}

// *****************************************************************************
// readAviClass::getFramerate()
//
// *****************************************************************************
float readAviClass::getFramerate()
{
	return(framerate);
}

// *****************************************************************************
// readAviClass::setDecodeMode()
//
// 0 = Encoder mode
// 1 = 32 Bit Player mode
// 2 = 16 bit Player Mode
// *****************************************************************************
void readAviClass::setDecodeMode(int mode)
{
	aviReadMode = mode;

}


// *****************************************************************************
// readAviClass::decodeFrame()
//
// *****************************************************************************
int readAviClass::decodeFrameDecompress(FRAMEDATA * source,DWORD decompType, long size)
{
	DWORD icRet;

	videoFormatIn->bmiHeader.biSizeImage = size;

	if(icRet = ICDecompress(videoHic,decompType,&videoFormatIn->bmiHeader,source->data,&videoFormatOut->bmiHeader,frame->data)!=ICERR_OK)
		{
		notDone=false;
		}

	return(notDone);
}

// *****************************************************************************
// readAviClass::decodeFrame()
//
// *****************************************************************************
bool readAviClass::decodeFrameRead(FRAMEDATA * dest)
{
	frameCount++;

	return(divx->getVideoStreamNextFrame(currentVideoChannel, dest->data));

}

// *****************************************************************************
// readAviClass::decodeFrameSize()
//
// *****************************************************************************
void readAviClass::decodeFrameSize(unsigned int * size)
{

	*size = divx->getVideoStreamNextFrameSize(currentVideoChannel);

}
// *****************************************************************************
// readAviClass::decodeFrame()
//
// *****************************************************************************
int readAviClass::decodeFrame()
{
	unsigned int size;

	decodeFrameSize(&size);

	if(videoFormatIn->bmiHeader.biCompression != BI_RGB)
		{
		if(decodeFrameRead(frameIn))
			return(decodeFrameDecompress(frameIn, ICDECOMPRESS_NOTKEYFRAME, size));
		else
			return(false);
		}
	else
		{
		if(decodeFrameRead(frameIn))
			{
			if(frameIn->bitDepth==24)
				{
				rgb24ToRgb32(frameIn,frame);
				return(true);
				}
			if(frameIn->bitDepth==32)
				{
				memcpy(frame->bufferStart,frameIn->bufferStart,frameIn->bufferSize);
				return(true);
				}
			return(false);
			}
		else
			{
			return(false);
			}
		}
}

// *****************************************************************************
// readAviClass::decodeFrame()
//
// *****************************************************************************
int readAviClass::skipFrame()
{
	unsigned int size;

	decodeFrameSize(&size);

	if(decodeFrameRead(frameIn))
		{	
		return(decodeFrameDecompress(frameIn, ICDECOMPRESS_HURRYUP, size));
		}
	else
		{
		notDone = false;
		return(false);
		}

}

// *****************************************************************************
// readAviClass::getFrameData()
//
// *****************************************************************************
FRAMEDATA * readAviClass::getFrameData()
{
	return(frame);
}

// *****************************************************************************
// readAviClass::getAudioData()
//
// *****************************************************************************
AUDIODATA * readAviClass::getAudioData(int channelIndex)
{
	long bytesRead;
	long copyLength;
	double frameMsPos;
	double audioMsPos;

	if(hasAudio)
		{
		if(audioSamplePos>=endingSample)
			{
			if((audio->bytesUsed+1024)<audio->bufferSize)
				{
				memset(audio->data,0,1024);
				audio->bytesUsed +=1024;
				audio->data+=1024;
				audioSamplePos = endingSample;
				}
			return(audio);
			}
		// If buffer has data to be consumed, just return
		if(!audio->bytesUsed)
			{
			//Determine if it's time to feed more audio to the encoder
			frameMsPos = msPerFrame;
			frameMsPos *= (frameCount - startingFrame);

			audioMsPos = 1000.0f / (double) audioFormatIn[currentAudioChannel]->wfx.nAvgBytesPerSec;
			audioMsPos *= audioSamplePos;
			if(frameMsPos<audioMsPos)
				return(audio);


			// Otherwise we need to decode some data
			bytesRead = divx->getAudioStreamBytes(currentAudioChannel,audioIn->data, audioIn->bufferSize);
			if(bytesRead)
				{
				if(!acmStreamConvert(audioAcmStream,&audioAcmHeader,ACM_STREAMCONVERTF_BLOCKALIGN))
					{
					if(audioAcmHeader.cbDstLengthUsed)
						{
						copyLength = audioAcmHeader.cbDstLengthUsed-audioBytesToSkip;
						memcpy(audio->data,audioOut->bufferStart + audioBytesToSkip,copyLength);		
						audio->bytesUsed +=copyLength;
						audio->data+= copyLength;
						audioSamplePos+=bytesRead;	//samplesRead;
						audioBytesToSkip = 0;
						}
					}
				}
			}
		}
	return(audio);
}

// *****************************************************************************
// readAviClass::getLength()
//
// get the length in frames
// *****************************************************************************
__int64 readAviClass::getLength()
{
	return(endingFrame);
}

// *****************************************************************************
// readAviClass::getNextKeyframe()
//
// Seek to a position in the stream
// *****************************************************************************
long readAviClass::getNextKeyframe(void)
{
	int nextKey;
	nextKey = divx->getVideoStreamNextKeyFrame(currentVideoChannel,frameCount);

// If we're going off the end, then just provide out current frame as the last one
	if((nextKey == -1) || (nextKey >=endingFrame))
		return(frameCount-1);

	return(nextKey);
}

// *****************************************************************************
// readAviClass::getNextKeyframe()
//
// Seek to a position in the stream
// *****************************************************************************
long readAviClass::getClosestKeyframe(int seekFrame)
{
	int prevKey;
	int currentKey;
	int currentFrame;

	currentFrame = seekFrame;
	if(currentFrame<=startingFrame)
		currentFrame = startingFrame;

	prevKey =0;
	for(;;)
		{
		currentKey =  divx->getVideoStreamNextKeyFrame(currentVideoChannel,prevKey+1);
		// If we're off the end, then just provide the last frame as the last key
		if(currentKey ==-1)
			currentKey = endingFrame;
		if(currentKey>=currentFrame)
			{
			if((currentKey - currentFrame) < (currentFrame - prevKey))
				{
				return(currentKey);
				}
			else
				{
				return(prevKey);
				}
			}
		if(currentKey == prevKey)
			return(currentKey);
		prevKey = currentKey;
		}
}

// *****************************************************************************
// readAviClass::getPrevKeyframe()
//
// Seek to a position in the stream
// *****************************************************************************
long readAviClass::getPrevKeyframe(void)
{
	int prevKey;
	int currentKey;
	int currentFrame;

	currentFrame = frameCount-1;
	if(currentFrame<=startingFrame)
		currentFrame = startingFrame;

	prevKey =0;
	for(;;)
		{
		currentKey =  divx->getVideoStreamNextKeyFrame(currentVideoChannel,prevKey+1);
			
		// If we're off the end, then just provide the last frame as the last key
		if(currentKey ==-1)
			currentKey = endingFrame;
		if(currentKey>=currentFrame)
			return(prevKey);
		if(currentKey == prevKey)
			return(currentKey);
		prevKey = currentKey;
		}
}

// *****************************************************************************
// readAviClass::seekFramePos()
//
// Seek to a position in the stream
// *****************************************************************************
bool readAviClass::seekFramePos(long frame)
{
	double frameTime;
//	double bitrateTime;
	double samplesPerMs;
	double pos;
	unsigned int audioBytePos;
	unsigned int alignedBytePos;

	frameCount = frame;

	divx->seekVideoStream(currentVideoChannel,frame);
	
	if(hasAudio)
		{
		frameTime = msPerFrame;
		frameTime *= (frameCount - startingFrame);
		samplesPerMs = audioFormatIn[currentAudioChannel]->wfx.nAvgBytesPerSec / 1000;			
		pos = samplesPerMs * frameTime;
		audioBytePos = (unsigned int)(pos * (double) bytesPerSample[currentAudioChannel]);		
		alignedBytePos = audioBytePos / audioFormatIn[currentAudioChannel]->wfx.nBlockAlign;
		alignedBytePos *=audioFormatIn[currentAudioChannel]->wfx.nBlockAlign;
		
		audioSamplePos = alignedBytePos;
		audioBytesToSkip = audioBytePos - alignedBytePos;

		divx->seekAudioStream(currentAudioChannel,audioSamplePos);

		}
	return(true);
}

// *****************************************************************************
// readAviClass::getCurPos()
//
// get the length of the entire set of files
// *****************************************************************************
__int64 readAviClass::getCurPos()
{
	return(frameCount);
}
// *****************************************************************************
// readAviClass::seekPos()
//
// Seek to a position in the stream (used for MPEG files)
// *****************************************************************************
bool readAviClass::seekPos(__int64 seekPos)
{
	seekFramePos((int)seekPos);
	return(true);
}

// *****************************************************************************
// readAviClass::getFramePos()
//
// Get our position in the stream
// *****************************************************************************
long readAviClass::getFramePos()
{
	if(frameCount-1>startingFrame)
		return(frameCount-1);
	else
		return(startingFrame);
}

// *****************************************************************************
// readAviClass::setSubtitle()
//
// *****************************************************************************
void readAviClass::setSubtitle(bool subEnabled,int channelIndex, int subtitleTrack)
{
	
}

// *****************************************************************************
// readAviClass::getSubtitleData()
//
// *****************************************************************************
SUBTITLEDATA * readAviClass::getSubtitleData(int channelIndex)
{
	return(NULL);
}

// *****************************************************************************
// readAviClass::Rgb32ToRgb24()
//
// *****************************************************************************

void readAviClass::rgb24ToRgb32(FRAMEDATA * srcFrame, FRAMEDATA * dstFrame)
{
	unsigned char * src;
	unsigned char * dst;
	unsigned char * src1;
	unsigned char * dst1;
	int w,h;
	unsigned char r,g,b;

	dstFrame->bitDepth=32;
	dstFrame->height = srcFrame->height;
	dstFrame->width = srcFrame->width;
	dstFrame->stride = srcFrame->width*4;
	dstFrame->type = rgb32Frame;

	
	src = srcFrame->data;
	dst = dstFrame->data;

	for(h=0;h<srcFrame->height;h++)
		{
		src1 = src;
		dst1 = dst;
		for(w=0;w<srcFrame->width;w++)
			{
			r = *src1++;
			g = *src1++;
			b = *src1++;
			*dst1++=r;
			*dst1++=g;
			*dst1++=b;
			*dst1++=0;
			}
		src += srcFrame->stride;
		dst += dstFrame->stride;
		}

}

