// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// registry.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

// *****************************************************************************
// registry::registry()
//
// conversion class constructor
// *****************************************************************************
registryClass::registryClass()
{

}

// *****************************************************************************
// registry::~registry()
//
// conversion class destructor
// *****************************************************************************
registryClass::~registryClass()
{

}

// *****************************************************************************
// registryClass::openCreateReg()
//
// *****************************************************************************
int registryClass::openCreateReg(HKEY * tempKey, char * programName)
{
	DWORD disp;

	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, programName, NULL , KEY_ALL_ACCESS , tempKey)==ERROR_SUCCESS)
		{
		disp = TRUE;
		}
	else
		{
		//Open Failed, so Create the key
		RegCreateKeyEx( HKEY_LOCAL_MACHINE , programName, 
			NULL , NULL, REG_OPTION_NON_VOLATILE , KEY_ALL_ACCESS , NULL , tempKey , &disp );
		if(disp == REG_CREATED_NEW_KEY)
			disp = TRUE;
		else
			disp = FALSE;
		}
	return(disp);
}

// *****************************************************************************
// registryClass::setRegVal()
//
// This function sets a registry value.
// *****************************************************************************
void registryClass::setRegVal(HKEY tempKey , char * keyStr , DWORD val )
{
	DWORD tempVal;
	
	tempVal = val;
	RegSetValueEx(tempKey , keyStr , NULL , REG_DWORD , (BYTE *)&tempVal , 4);
		
}

// *****************************************************************************
// registryClass::setRegFloat0to1()
//
// This function sets a registry value.
// *****************************************************************************
void registryClass::setRegFloat0to1(HKEY tempKey , char * keyStr , float val )
{
	DWORD tempVal;
	
	tempVal = (DWORD) (val * 1000000000);
	RegSetValueEx(tempKey , keyStr , NULL , REG_DWORD , (BYTE *)&tempVal , 4);
		
}

// *****************************************************************************
// registryClass::setRegVal()
//
// This function sets a registry value.
// *****************************************************************************
void registryClass::setRegBool(HKEY tempKey , char * keyStr , DWORD val )
{
	DWORD tempVal;
	
	tempVal = val;
	RegSetValueEx(tempKey , keyStr , NULL , REG_DWORD , (BYTE *)&tempVal , 4);
		
}

// *****************************************************************************
// registryClass::setRegValN()
//
// This function sets a registry value.
// *****************************************************************************
void registryClass::setRegValN(HKEY tempKey , char * keyStr , unsigned char * addr,  int size)
{
	RegSetValueEx(tempKey , keyStr , NULL , REG_DWORD , addr , size);		
}

// *****************************************************************************
// registryClass::setRegStr()
//
// This function sets a registry String
// *****************************************************************************
void registryClass::setRegStr(HKEY tempKey , char * keyStr , char * valStr )
{
	
	RegSetValueEx(tempKey , keyStr , NULL , REG_SZ , (BYTE *)valStr , strlen(valStr)+1);
		
}

// *****************************************************************************
// registryClass::getSetRegVal()
//
// *****************************************************************************
DWORD registryClass::getSetRegVal(HKEY tempKey , char * keyStr, int * var)
{
	DWORD tempVal;
	DWORD tempLen=4;

	if(RegQueryValueEx(tempKey , keyStr , NULL , NULL, (BYTE *)&tempVal , &tempLen )!=ERROR_SUCCESS)
		{
		tempVal = *var;
		RegSetValueEx(tempKey , keyStr , NULL , REG_DWORD , (BYTE *)&tempVal , 4);
		}
	*var = tempVal;
	return(tempVal);
}

// *****************************************************************************
// registryClass::getSetRegVal()
//
// *****************************************************************************
float registryClass::getSetRegFloat0to1(HKEY tempKey , char * keyStr, float * var)
{
	DWORD tempVal;
	DWORD tempLen=4;
	
	if(RegQueryValueEx(tempKey , keyStr , NULL , NULL, (BYTE *)&tempVal , &tempLen )!=ERROR_SUCCESS)
		{
		tempVal = (DWORD) (*var * 1000000000);
		RegSetValueEx(tempKey , keyStr , NULL , REG_DWORD , (BYTE *)&tempVal , 4);
		}
	*var = (float) ((double)tempVal / (double)1000000000);
	return(*var);
}


// *****************************************************************************
// registryClass::getSetRegVal()
//
// *****************************************************************************
DWORD registryClass::getSetRegBool(HKEY tempKey , char * keyStr, bool * var)
{
	DWORD tempVal;
	DWORD tempLen=4;

	if(RegQueryValueEx(tempKey , keyStr , NULL , NULL, (BYTE *)&tempVal , &tempLen )!=ERROR_SUCCESS)
		{
		tempVal = *var;
		RegSetValueEx(tempKey , keyStr , NULL , REG_DWORD , (BYTE *)&tempVal , 4);
		}
	if(tempVal)
		*var = true;
	else
		*var = false;

	return(tempVal);
}

// *****************************************************************************
// registryClass::getSetRegValN()
//
// *****************************************************************************
void registryClass::getSetRegValN(HKEY tempKey , char * keyStr, unsigned char * addr, int size)
{
	DWORD tempLen;

	tempLen = size;

	if(RegQueryValueEx(tempKey , keyStr , NULL , NULL, (BYTE *)addr , &tempLen )!=ERROR_SUCCESS)
		{
		RegSetValueEx(tempKey , keyStr , NULL , REG_DWORD , (BYTE *)addr , size);
		}
}

// *****************************************************************************
// registryClass::getSetRegStr()
//
// *****************************************************************************
int registryClass::getSetRegStr(HKEY tempKey , char * keyStr, char * tempString, int tempStringLen)
{
	static DWORD tempLen;

	tempLen = tempStringLen;
	
	if(RegQueryValueEx(tempKey , keyStr , NULL , NULL, (BYTE *) tempString , &tempLen )!=ERROR_SUCCESS)
		{
		RegSetValueEx(tempKey , keyStr , NULL , REG_SZ , (BYTE *)tempString , strlen(tempString)+1);	
		}
	return TRUE;	
}

// *****************************************************************************
// registryClass::closeReg()
//
// *****************************************************************************
void registryClass::closeReg(HKEY tempKey)
{
	RegCloseKey( tempKey);
}