// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// scanFile.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

double framerateTable[] = 	{
			0.0f,
		  	23.976,
		  	24.0,
			25.0,
			29.970,
			30.0,
			50.0,
			50.94,
			60.0,
			-1.0,		
			-1.0,
			-1.0,
			-1.0,
			-1.0,
			-1.0,
			-1.0 };

int sampleTable [] = { 48000,44100,32000,0};
int mp3SampleTable[] = {22050, 24000, 16000, 0,
						  0,     0,     0, 0,
						22050, 24000, 16000, 0,
						44100, 48000, 32000, 0 };

int ac3Channels[] = {2,1,2,3,3,4,4,5};
double aspectTable[] = { 0.0f, 1.0f, (4.0f / 3.0f), (16.0f / 9.0f), (2.21f / 1.0f)};

// *****************************************************************************
// scanFileClass::scanFileClass()
//
// scanmpeg class constructor
// *****************************************************************************
scanFileClass::scanFileClass()
{
	readBuffer = new unsigned char [SCANMPEG_BUFFER_SIZE];
	shredBuffer = new unsigned char [SHREDMPEG_BUFFER_SIZE];
}

// *****************************************************************************
// scanFileClass::~scanFileClass()
//
// scanmpeg class destructor
// *****************************************************************************
scanFileClass::~scanFileClass()
{
	delete [] readBuffer;
	delete [] shredBuffer;
}
// *****************************************************************************
// scanFileClass::scanFile()
//
// scans an mpeg file for useful info. 
// *****************************************************************************
bool scanFileClass::scanFile(char * filename, int scanSize, bool useInfoFile, GATHERRESULTS * fileInfo)
{
	char * tempPtr;

	// Check file extension, then call apropriate routine
	tempPtr = strrchr(filename,'.');

	if(!_strnicmp(tempPtr+1,"avi",3))
		{
		fileInfo->fileType = SCAN_FILE_TYPE_AVI;
		return(scanAvi(filename, fileInfo));
		}
	else
		{
		// Assume it's some sort of mpg input
		fileInfo->fileType = SCAN_FILE_TYPE_MPEG;
		return(scanMpeg(filename, scanSize, useInfoFile, fileInfo));
		}

}
// *****************************************************************************
// scanFileClass::scanFile()
//
// scans an mpeg file for useful info. 
// *****************************************************************************
bool scanFileClass::scanAvi(char * filename, GATHERRESULTS * fileInfo)
{
	avReadClass * avRead;
	int channels;
	int sampleRate;
	int i;

	avRead = new readAviClass();

	if(!avRead->init())
		{
		avRead->end();
		delete avRead;
		return(false);
		}
	avRead->clearInput();
	avRead->addInput(filename);
	avRead->setIdct(1);
	avRead->setAudio(true,0,0,0,false);

	if(!avRead->begin(rgb32Frame))
		{
		delete avRead;
		return(false);
		}
	
	
	// see if we failed on the open of the avi file. 
	if(avRead->checkEnd())
		{
		avRead->end();
		delete avRead;
		return(false);		
		}

	avRead->decodeFrame();

	avRead->getDimentions(&fileInfo->videoWidth,&fileInfo->videoHeight,&fileInfo->videoAspectRatio);

	fileInfo->videoFramerate = avRead->getFramerate();
	fileInfo->videoChromaFormat = CHROMARGB32;

	for(i=0;i<16;i++)
		{
		fileInfo->audioRate[i]= 0;
		fileInfo->audioChannels[i]= 0;
		fileInfo->audioName[i][0]= 0;
		fileInfo->audioNumber[i]= 0;
		fileInfo->subtitleName[i][0]= 0;
		fileInfo->subtitleNumber[i]= 0;
		}

	fileInfo->chaptersFound=0;
	fileInfo->audioFound = 0;
	fileInfo->subtitleFound = 0;
	avRead->getAudioOutput(0, &channels, &sampleRate);

	//sadly we only support these encode rates currently. force the reader to ignore invalid audio streams
	if(sampleRate==22050 || sampleRate==44100 || sampleRate==48000)
		{
		if(channels)
			{
			fileInfo->audioRate[0] = sampleRate;
			fileInfo->audioChannels[0] = channels;
			fileInfo->audioType[0]=audioPcm; 
			fileInfo->audioNumber[0]=0;

			if(channels==2)
				{
				sprintf(&fileInfo->audioName[0][0],"AVI Audio (%d Channels)",channels);
				}
			else
				{
				sprintf(&fileInfo->audioName[0][0],"AVI Audio (Mono)");
				}
			fileInfo->audioFound++;
			}	
		}
	avRead->end();
	delete avRead;
	
	return(true);

}
// *****************************************************************************
// scanFileClass::scanFile()
//
// scans an mpeg file for useful info. 
// *****************************************************************************
bool scanFileClass::scanMpeg(char * filename, int scanSize, bool useInfoFile, GATHERRESULTS * fileInfo)
{
	unsigned long code;
	int width;
	int height;
	double aspectRatio;
	int aspectRatioCode;
	int frameRateCode;
	int chromaFormat;
	float frameRate;
	int horizontalSizeExtension;
	int verticalSizeExtension;
	int frameRateExtensionN;
	int frameRateExtensionD;
	bool sequenceHeaderFound;
	int buffPos;
	unsigned char * ptr;
	long shredPos;
	long shredEnd;
	long shredSize;
	int packetCode;
	int packetLength;
	int streamType;
	int i;
	int search;
	int mpegId;
	__int64 bigFileSize;
	long validStart;
	int mpegMode;
	int audioIndex;
	int audioSubIndex;
	unsigned char mpegHeader[4];
	mp3DecodeClass * mp3Decode;
	int mp3FrameSize;
	char * infoFilename;
	char * infoPtr;
	char * infoPtr1;
	bool foundInfoFile;
	FILE * infoFile;
	char * infoTextLine;
	char * infoChannelName;
	bool noInfoFound;
	int subtitleIndex;
	bool isTransportStream;

	mp3Decode = new mp3DecodeClass();
	if(!mp3Decode->init())
		{
		delete mp3Decode;
		mp3Decode = NULL;
		}

	foundInfoFile = false;
	infoFilename = new char [MAX_PATH];
	infoTextLine = new char [1024];
	infoChannelName = new char [1024];
	
	if(useInfoFile)
		{
		strcpy(infoFilename,filename);
		infoPtr = strrchr(infoFilename,'_');
		if(!infoPtr)
			{
			// If failed to find "_" try "."
			infoPtr = strrchr(infoFilename,'.');
			}
		if(infoPtr)
			{
			*infoPtr = 0;
			strcat(infoFilename,"_INFO.txt");
			infoFile = fopen(infoFilename,"r");
			if(infoFile)
				{
				// Found an INFO file
				fclose(infoFile);
				foundInfoFile = true;
				}
			}
		}
			

	inFile = _open(filename,_O_SEQUENTIAL | _O_BINARY );
	if(inFile==-1)
		{
		logPrintf("Failed to open Mpeg File for Scanning\n");
		return(false);
		}

	if(_lseeki64( inFile, 0, SEEK_END )==-1)
		{
		logPrintf("Failed to seek to end of Mpeg File for Scanning\n");
		_close(inFile);
		return(false);
		}

	bigFileSize = _telli64( inFile );

	
	if(bigFileSize >(scanSize*1024*1024))
		inFileSize = (scanSize*1024*1024);
	else
		inFileSize = (long)bigFileSize;
		

	_lseeki64(inFile,0,SEEK_SET);

	shredPos = 0;
	shredEnd = (scanSize*1024*1024);
	if(inFileSize<shredEnd)
		shredEnd = inFileSize;
	
	bufferStart = 0;
	bufferEnd = -1;
	sequenceHeaderFound = false;

	chromaFormat = CHROMA420;

	for(i=0;i<16;i++)
		{
		audioAc3Found[i]=0;
		audioAc3SourceChannels[i]=0;
		audioAc3Rate[i]=0;
		audioAc3Channels[i]=0;
		audioMpegFound[i]=0;
		audioMpegRate[i]=0;
		audioMpegChannels[i]=0;
		audioPcmFound[i]=0;
		audioPcmRate[i]=0;
		audioPcmChannels[i]=0;
		subtitleFound[i]=0;
		}

	validStart = 0;

	_read(inFile,shredBuffer,SHREDMPEG_BUFFER_SIZE);
	ptr = shredBuffer;
	isTransportStream = (*ptr == 0x47);

	while(shredPos<shredEnd)
		{
		_lseek(inFile,shredPos,SEEK_SET);
		shredSize = SHREDMPEG_BUFFER_SIZE;
		if(shredPos+shredSize > shredEnd)
			shredSize = shredEnd - shredPos;

		_read(inFile,shredBuffer,shredSize);
		ptr = shredBuffer;
		for(buffPos=0;buffPos<shredSize;buffPos++)
			{
			code = code<<8 | *ptr++;
			if(code>>8 == 0x00000001)
				{
				// Parse the code
				streamType =0;
				switch(code)
					{
					case PACK_START:
						validStart = shredPos + buffPos;
						break;

					case SEQUENCE_HEADER :
						setPos(shredPos + buffPos+1);
						sequenceHeaderFound = true;
						width = getBits(12);
						height = getBits(12);
						aspectRatioCode = getBits(4);
						aspectRatio = aspectTable[aspectRatioCode];
						frameRateCode = getBits(4);
						frameRate = (float) framerateTable[frameRateCode];
						break;

					case SEQUENCE_EXTENSION :
						setPos(shredPos + buffPos+1);
						flushBits(8);
						flushBits(1);
						flushBits(2);			//chromaFormat = getBits(2);
						horizontalSizeExtension = getBits(2);
						verticalSizeExtension = getBits(2);
						flushBits(12);
						flushBits(1);
						flushBits(8);
						flushBits(1);
						frameRateExtensionN = getBits(2);
						frameRateExtensionD = getBits(5);
						
						frameRate = (float) framerateTable[frameRateCode] * (frameRateExtensionN + 1) / (frameRateExtensionD +1 );
						width = (horizontalSizeExtension<<12) | (width&0x0fff);
						height = (verticalSizeExtension<<12) | (height&0x0fff);
						break;

					case PRIVATE_STREAM :
						//Consider any streams found invalid if a packstart wasn't found within 32 bytes of us
						if(!isTransportStream && (((shredPos + buffPos) - validStart) >32))
							{
							break;								
							}
						setPos(shredPos + buffPos+1);
						packetLength = getBits(16);
						flushBits(16);
						packetLength = getBits(8);
						flushBits(8*packetLength);
						if (isTransportStream)
							streamType = 0x80;
						else
							streamType = getBits(8);

						if(streamType>=0x80 && streamType < 0x8f)
							{
							streamType-=0x80;
							if(audioAc3Found[streamType])
								break;

							search=0;
							//It's and AC3 audio chunk
							packetCode=0;
							do{
								packetCode = packetCode<<8 | getBits(8);
								search++;
								}while(((packetCode&0x0000ffff)!=0xb77) && (search<256));
							if(search>=256)
								break;
							flushBits(16);
							audioAc3Rate[streamType] = sampleTable[(getBits(8)>>6)&0x03];
							if((audioAc3Rate[streamType]!=44100) && (audioAc3Rate[streamType]!=48000) && 
								(audioAc3Rate[streamType]!=32000) )
								break;
							flushBits(8);
							audioAc3SourceChannels[streamType] = ac3Channels[getBits(3)];
							audioAc3Found[streamType]=1;
							audioAc3Channels[streamType]=2;
							break;
							}
						if(streamType>=0xa0 && streamType <0xaf)
							{
							// It's a PCM chunk so assume it's the following specs
							streamType-=0xa0;
							audioPcmFound[streamType]=1;
							audioPcmRate[streamType]=48000;
							audioPcmChannels[streamType]=2;
							break;
							}
						if(streamType>=0x20 && streamType <0x2f)
							{
							// It's a subtitle chunk
							streamType-=0x20;
							subtitleFound[streamType]=1;
							break;
							}
						break;
					
					case MPEG_STREAM7 :
						streamType++;
					case MPEG_STREAM6 :
						streamType++;
					case MPEG_STREAM5 :
						streamType++;
					case MPEG_STREAM4 :
						streamType++;
					case MPEG_STREAM3 :
						streamType++;
					case MPEG_STREAM2 :
						streamType++;
					case MPEG_STREAM1 :
						streamType++;
					case MPEG_STREAM0 :
						//Consider any streams found invalid if a packstart wasn't found within 32 bytes of us
						// NOTE: This code had to be removed due to the WINTV-PVR hardware not outputing 
						// a pack start before they output an audio frame.
						//if(((shredPos + buffPos) - validStart) >32)
						//	{
						//	break;								
						//	}
						if(audioMpegFound[streamType])
							break;
						setPos(shredPos + buffPos+1);
						packetLength = getBits(16);
						packetCode = getBits(8)&0xc0;
						flushBits(8);
						packetLength = getBits(8);
						flushBits(8*packetLength);
							
						packetCode=0;
						search=0;
						do{
							packetCode = packetCode<<8 | getBits(8);
							search++;
							}while(((packetCode&0x0000ffe0)!=0x0000ffe0) && (search<32));
						if(search>=32)
							break;

						if(mp3Decode)
							{
							mpegHeader[0] = (packetCode&0x0000ff00)>>8;
							mpegHeader[1] = packetCode&0x000000ff;
							mpegHeader[2] = (unsigned char) getBits(8);
							mpegHeader[3] = (unsigned char) getBits(8);
							// Now ask the MPEG decoder to tell us what the stream has
							if(mp3Decode->prepare(&mpegHeader[0],4,&audioMpegChannels[streamType],&audioMpegRate[streamType],&mp3FrameSize))
								audioMpegFound[streamType]=1;
							}
						else
							{
							// Old style method without Opensource call
							mpegId = (packetCode&0x18)>>3;
							
							audioMpegRate[streamType] = mp3SampleTable[(mpegId*4) + ((getBits(8)&0x0c)>>2)];
							if((audioMpegRate[streamType]!=44100) && (audioMpegRate[streamType]!=48000) && 
								(audioMpegRate[streamType]!=32000) )
								break;
							flushBits(8);
							mpegMode = getBits(8);
							mpegMode = (mpegMode&0x000000c0)>>6;
							audioMpegFound[streamType]=1;
							if(mpegMode==MPEG_MODE_MONO)
								audioMpegChannels[streamType]=1;
							else
								audioMpegChannels[streamType]=2;
							}
						break;

					default:
						break;
					}

				}

			}
		shredPos+=shredSize;

		}


	//determine if we found everything we need
	if(!sequenceHeaderFound)
		{
		//Not a valid MPEG file (more than likely)
		return(false);
		}	
	
	fileInfo->videoWidth = width;
	fileInfo->videoHeight = height;
	fileInfo->videoFramerate = frameRate;
	fileInfo->videoAspectRatio = aspectRatio;
	fileInfo->videoChromaFormat = chromaFormat;

	audioIndex=0;
	audioSubIndex=0;
	for(i=0;i<16;i++)
		{
		if(audioAc3Found[i])
			{
			fileInfo->audioRate[audioIndex] = audioAc3Rate[i];
			fileInfo->audioChannels[audioIndex] = audioAc3Channels[i];
			noInfoFound = true;
			if(foundInfoFile)
				{
				sprintf(infoChannelName,"[0x%X]",0x80+i);
				infoFile = fopen(infoFilename,"r");
				while(!feof(infoFile))
					{
					fgets(infoTextLine,1024,infoFile);
					if(strstr(infoTextLine,"Stream"))
						{
						infoPtr = strstr(infoTextLine,infoChannelName);
						if(infoPtr)
							{
							//We found the name
							infoPtr+= strlen(infoChannelName);
							infoPtr1 = strstr(infoPtr,"]");
							if(infoPtr1)
								{
								*infoPtr1 = 0;
								sprintf(&fileInfo->audioName[audioIndex][0],"AC3[%d] ",
								audioSubIndex+1);
								strcat(&fileInfo->audioName[audioIndex][0],infoPtr);
								noInfoFound=false;
								break;
								}
							}
						}
					}
				fclose(infoFile);
				}
			if(noInfoFound)
				{
				if(audioAc3SourceChannels[i]>1)
					{
					sprintf(&fileInfo->audioName[audioIndex][0],"AC3 Audio Track %d (%d Channels)",
						audioSubIndex+1,audioAc3SourceChannels[i]);
					}
				else
					{
					sprintf(&fileInfo->audioName[audioIndex][0],"AC3 Audio Track %d (Mono)", audioSubIndex+1);
					}
				}
			fileInfo->audioType[audioIndex]=audioAc3;
			fileInfo->audioNumber[audioIndex]=i;
			audioIndex++;
			audioSubIndex++;
			} 
		}

	audioSubIndex=0;
	for(i=0;i<16;i++)
		{
		if(audioMpegFound[i])
			{
			fileInfo->audioRate[audioIndex] = audioMpegRate[i];
			fileInfo->audioChannels[audioIndex] = audioMpegChannels[i];

			noInfoFound = true;
			if(foundInfoFile)
				{
				sprintf(infoChannelName,"[0x%X]",0xc0+i);
				infoFile = fopen(infoFilename,"r");
				while(!feof(infoFile))
					{
					fgets(infoTextLine,1024,infoFile);
					if(strstr(infoTextLine,"Stream"))
						{
						infoPtr = strstr(infoTextLine,infoChannelName);
						if(infoPtr)
							{
							//We found the name
							infoPtr+= strlen(infoChannelName);
							infoPtr1 = strstr(infoPtr,"]");
							if(infoPtr1)
								{
								*infoPtr1 = 0;
								sprintf(&fileInfo->audioName[audioIndex][0],"MPEG[%d] ",
								audioSubIndex+1);
								strcat(&fileInfo->audioName[audioIndex][0],infoPtr);
								noInfoFound=false;
								break;
								}
							}
						}
					}
				fclose(infoFile);
				}

			if(noInfoFound)
				{
				if(audioMpegChannels[i]>1)
					{
					sprintf(&fileInfo->audioName[audioIndex][0],"MPEG Audio Track %d (%d Channels)",audioSubIndex+1,audioMpegChannels[i]);
					}
				else
					{
					sprintf(&fileInfo->audioName[audioIndex][0],"MPEG Audio Track %d (Mono)",audioSubIndex+1);
					}
				}
			fileInfo->audioType[audioIndex]=audioMpeg; 
			fileInfo->audioNumber[audioIndex]=i;
			audioIndex++;
			audioSubIndex++;
			}
		}



	audioSubIndex=0;
	for(i=0;i<16;i++)
		{
		if(audioPcmFound[i])
			{
			fileInfo->audioRate[audioIndex] = audioPcmRate[i];
			fileInfo->audioChannels[audioIndex] = audioPcmChannels[i];
			noInfoFound = true;

			if(foundInfoFile)
				{
				sprintf(infoChannelName,"[0x%X]",0xa0+i);
				infoFile = fopen(infoFilename,"r");
				while(!feof(infoFile))
					{
					fgets(infoTextLine,1024,infoFile);
					if(strstr(infoTextLine,"Stream"))
						{
						infoPtr = strstr(infoTextLine,infoChannelName);
						if(infoPtr)
							{
							//We found the name
							infoPtr+= strlen(infoChannelName);
							infoPtr1 = strstr(infoPtr,"]");
							if(infoPtr1)
								{
								*infoPtr1 = 0;
								sprintf(&fileInfo->audioName[audioIndex][0],"PCM[%d] ",
								audioSubIndex+1);
								strcat(&fileInfo->audioName[audioIndex][0],infoPtr);
								noInfoFound=false;
								break;
								}
							}
						}
					}
				fclose(infoFile);
				}

			if(noInfoFound)
				{
				if(audioPcmChannels[i]>1)
					{
					sprintf(&fileInfo->audioName[audioIndex][0],"PCM Audio Track %d (%d Channels)",audioSubIndex+1,audioPcmChannels[i]);
					}
				else
					{
					sprintf(&fileInfo->audioName[audioIndex][0],"PCM Audio Track %d (Mono)",audioSubIndex+1);
					}
				}
			fileInfo->audioType[audioIndex]=audioPcm; 
			fileInfo->audioNumber[audioIndex]=i;
			audioIndex++;
			audioSubIndex++;
			}
		}

	fileInfo->audioFound=audioIndex;

	subtitleIndex=0;
	for(i=0;i<16;i++)
		{
		if(subtitleFound[i])
			{
			noInfoFound = true;

			if(foundInfoFile)
				{
				sprintf(infoChannelName,"[0x%X]",0x20+i);
				infoFile = fopen(infoFilename,"r");
				while(!feof(infoFile))
					{
					fgets(infoTextLine,1024,infoFile);
					if(strstr(infoTextLine,"Stream"))
						{
						infoPtr = strstr(infoTextLine,infoChannelName);
						if(infoPtr)
							{
							//We found the name
							infoPtr+= strlen(infoChannelName);
							infoPtr1 = strstr(infoPtr,"]");
							if(infoPtr1)
								{
								*infoPtr1 = 0;
								sprintf(&fileInfo->subtitleName[subtitleIndex][0],"Subtitle[%d] ",
								subtitleIndex+1);
								strcat(&fileInfo->subtitleName[subtitleIndex][0],infoPtr);
								noInfoFound=false;
								break;
								}
							}
						}
					}
				fclose(infoFile);
				}

			if(noInfoFound)
				{
				sprintf(&fileInfo->subtitleName[subtitleIndex][0],"Subtitle Track %d",subtitleIndex+1);
				}
			fileInfo->subtitleNumber[subtitleIndex]=i;
			subtitleIndex++;
			}
		}

	fileInfo->subtitleFound = subtitleIndex;

	fileInfo->averageDatarate = 0;

	if(foundInfoFile)
		{
		infoFile = fopen(infoFilename,"r");
		while(!feof(infoFile))
			{
			fgets(infoTextLine,1024,infoFile);
			if(strstr(infoTextLine,"AvgDataRate (B/s):"))
				{
				infoPtr = strstr(infoTextLine,":");
				if(infoPtr)
					{
					//We found the name
					infoPtr++;
					fileInfo->averageDatarate = atoi(infoPtr);
					break;
					}
				else
					{
					break;
					}
				}
			}
		fclose(infoFile);
		}

	fileInfo->chaptersFound=0;

	_close(inFile);

	if(mp3Decode)
		{
		delete mp3Decode;
		}

	if(infoFilename)
		{
		delete [] infoFilename;
		}
	if(infoTextLine)
		{
		delete [] infoTextLine;
		}

	if(infoChannelName)
		{
		delete [] infoChannelName;
		}

	return(true);
}

// *****************************************************************************
// readMpegClass::flushBits()
//
// flush N bits
// *****************************************************************************
void scanFileClass::flushBits(int numBits)
{
	getBits(numBits);
}

// *****************************************************************************
// readMpegClass::getBits()
//
// get up to 32 Bits from the file
// *****************************************************************************
unsigned long scanFileClass::getBits(int numBits)
{
	int giveBits;
	unsigned long outputValue;

	outputValue=0;
	if(!reachedEnd)
		{
		while(numBits)
			{
			giveBits = numBits;
			if(numBits >= bitsRemaining)
				giveBits = bitsRemaining;
			outputValue = (outputValue << giveBits) | 
				(readBuffer[bufferPos]&(0xff>>(8-bitsRemaining)))>>(bitsRemaining-giveBits);
			bitsRemaining = bitsRemaining - giveBits;
			numBits = numBits - giveBits;
			if(!bitsRemaining)
				{
				bufferPos++;
				bitsRemaining=8;
				if((bufferPos + bufferStart)>bufferEnd)
					setPos(bufferPos + bufferStart);
				if(reachedEnd)
					return(outputValue);
				}
			}
		}
	return(outputValue);

}

// *****************************************************************************
// readMpegClass::alignFrame()
//
// align to the beginning of a frame header
// *****************************************************************************
void scanFileClass::alignFrame(void)
{
	long nextBits;

	if(bitsRemaining!=8)
		getBits(bitsRemaining);

	nextBits =0;

	while(!reachedEnd)
		{
		nextBits = peekBits(24);
		if(nextBits!=0x000001)
			{
			getBits(8);					//keep moving forward in stream until w find a valid frame start
			}
		else
			{
			break;
			}
		}
}



// *****************************************************************************
// readMpegClass::peekBits()
//
// get up to 32 Bits from the file
// *****************************************************************************
unsigned long scanFileClass::peekBits(int numBits)
{
	int giveBits;
	unsigned long outputValue;
	long tempBufferPos;
	long tempBitsRemaining;

	outputValue=0;
	
	tempBufferPos = bufferPos;
	tempBitsRemaining = bitsRemaining;

	if(!reachedEnd)
		{
		while(numBits)
			{
			giveBits = numBits;
			if(numBits >= tempBitsRemaining)
				giveBits = tempBitsRemaining;
			outputValue = outputValue << giveBits | 
				(readBuffer[tempBufferPos]&(0xffffffff>>(32-tempBitsRemaining)))>>(tempBitsRemaining>>giveBits);
			tempBitsRemaining = tempBitsRemaining - giveBits;
			numBits = numBits - giveBits;
			if(!tempBitsRemaining)
				{
				// Peek is allowed to go past the end for up to 32 bits (it's max peek size). 
				// this is to prevent thrashing of the buffer
				tempBufferPos++;
				tempBitsRemaining=8;
				}
			}
		}
	return(outputValue);

}

// *****************************************************************************
// readMpegClass::setPos()
//
// get up to 32 Bits from the file
// *****************************************************************************
bool scanFileClass::setPos(long pos)
{
	long readSize;

	if(pos < bufferStart || pos > bufferEnd)
		{
		if(pos>=inFileSize)
			{
			reachedEnd=true;
			return(false);
			}

		// we need to fill the buffer with new data
		_lseek(inFile,pos,SEEK_SET);
		
		readSize = inFileSize - pos;
		if(readSize > SCANMPEG_BUFFER_SIZE)
			readSize = SCANMPEG_BUFFER_SIZE;

		_read(inFile,readBuffer,readSize);
		
		bufferStart = pos;
		bufferEnd = pos + (readSize-8);		// Provide a peek ahead slack for the peekBits function

		bitsRemaining = 8;
		bufferPos = 0;
		reachedEnd=false;
		}
	else
		{
		// move the pointer in memory
		bufferPos = pos - bufferStart;
		bitsRemaining = 8;
		reachedEnd=false;
		}
	return(true);
}

// *****************************************************************************
// scanFileClass::scanFile()
//
// scans an mpeg file for useful info. 
// *****************************************************************************
bool scanFileClass::scanCropRect(CONVERTCONFIG * settings, bool prompt)
{
	avReadClass * avRead;
	postProcessClass * postProcess;
	FRAMEDATA * frame;
	int width;
	int height;
	float framerate;
	__int64 filePos;
	__int64 fileLen;
	FRAMEDATA * endingFrame;
	unsigned char * ptr;
	unsigned char * ptrw;	
	int h;
	int w;
	int i;
	unsigned char r,g,b,a;
	int hasDataV[2048];
	int top;
	int bot;
	int cropIndex;
	int topCrop[10];
	int botCrop[10];
	__int64 cropFilePos[10];
	int hightCount;
	int hightIndex;
	int curCount;
	char * tempPtr;
	int mode;
	double aspectRatio;


	// Check file extension, then call apropriate routine

	logPrintf("Starting Crop Rect Scan\n");

	tempPtr = strrchr(settings->inputFilenames[0],'.');

	if(!_strnicmp(tempPtr+1,"avi",3))
		{
		avRead = new readAviClass();
		mode=1;
		}
	else
		{
		avRead = new readMpegClass();
		mode=0;
		}

	if(!avRead->init())
		{
		return(false);
		}

	postProcess = new postProcessClass();
	if(!postProcess->init())
		{
		return(false);
		}


	avRead->clearInput();
	for(i=0;i<settings->inputFilenameIndex;i++)
		avRead->addInput(settings->inputFilenames[i]);
	avRead->setIdct(1);
	avRead->setAudio(false,0,0,0,false);

	// then tell the decoder we're ready to begin
	avRead->begin(rgb32Frame);
	// see if we failed on the open of the mpeg file. 
	if(avRead->checkEnd())
		{
		avRead->end();
		delete avRead;
		return(false);
		}

	avRead->decodeFrame();
	avRead->getDimentions(&width,&height,&aspectRatio);
	framerate = avRead->getFramerate();

	if(globals->convertConfig->hyperCrop)
		{
		postProcess->setHyperScale(false,0, false);
		postProcess->setCrop(0,0,0,0);
		postProcess->setDeinterlace(false,0,0);
		postProcess->setOutputDimentions(false,0,0,0,0);
		postProcess->begin(width,height,framerate,rgb32Frame,false,settings);

		//Set input frame size
		globals->outputVideo->setVideoInputFormat(width,height,rgb32Frame);
		//set output window size
		globals->outputVideo->setOutputWindowSize(width,height,0,0);
		//display the output window
		globals->outputVideo->showOutputWindow();

		fileLen = avRead->getLength();
		filePos = fileLen / 11;

		for(i=0;i<height;i++)
			{
			hasDataV[i]=0;
			}
		cropIndex=0;
		while(filePos<fileLen)
			{
			if(mode==0)
				{
				avRead->seekPos(filePos);
				}
			else
				{
				avRead->seekFramePos(avRead->getNextKeyframe());
				}
			avRead->decodeFrame();
		
			if(avRead->checkEnd())
				{
				logPrintf("Abnormal end of stream. Partial Mpeg or critical error hit\n");
				break;
				}	

			frame = avRead->getFrameData();
			endingFrame = postProcess->processFrame(frame);

			ptr = endingFrame->bufferStart;

			// Do the frame scan for crop rect vertical (height)
			for(h=0;h<height;h++)
				{
				ptrw = ptr;
				for(w=0;w<width;w++)
					{
					b = *ptrw++;
					g = *ptrw++;
					r = *ptrw++;
					a = *ptrw++;
					if((r >settings->hyperCropThreshold) && (g > settings->hyperCropThreshold) && (b > settings->hyperCropThreshold) )
						{
						hasDataV[h] = 1;
						break;
						}
					}
				ptr +=width*4;			
				}

			// Find the first non Zero line
			bot =0;
			for(i=0;i<height;i++)
				{
				if(hasDataV[i]!=0)
					break;
				bot++;
				}

			top=0;

			for(i=height-1;i>=0;i--)
				{
				if(hasDataV[i]!=0)
					break;
				top++;
				}

			if(bot!=height)
				{
				ptr = endingFrame->bufferStart + (bot* (width * 4));
				for(i=0;i<width;i++)
					{
					*ptr++ = 0x00;
					*ptr++ = 0x00;
					*ptr++ = 0xff;
					*ptr++ = 0x00;
					}
				}
			if(top!=height)
				{
				ptr = endingFrame->bufferStart + (((height-1)-top) * (width * 4));
				for(i=0;i<width;i++)
					{
					*ptr++ = 0x00;
					*ptr++ = 0x00;
					*ptr++ = 0xff;
					*ptr++ = 0x00;
					}
				}

			globals->outputVideo->displayFrame(endingFrame);

			// Save the vertical results
			if((bot==height && top==height) || (bot==0 && top==0))
				{
				botCrop[cropIndex]=0;
				topCrop[cropIndex]=0;
				}
			else
				{
				botCrop[cropIndex]=bot;
				topCrop[cropIndex]=top;
				}
			//save the file pos we looked at
			cropFilePos[cropIndex] = filePos;
			cropIndex++;
			filePos += (fileLen / 11);
			}

		// Now analyze the results

		hightCount=0;
		hightIndex=0;
		for(h=0;h<10;h++)
			{
			curCount=0;
			for(w=0;w<10;w++)
				{
				if((botCrop[w] == botCrop[h]) &&
					(topCrop[w] == topCrop[h]) )
					{
					curCount++;
					}
				}	
				if(curCount>hightCount)
					{
					hightCount=curCount;
					hightIndex = h;
					}
			}

		if(hightCount<5)
			{
			if(prompt)
				{
				//We have low Confidence in the Results.
				//Build a frame to display and ask about
				filePos = (fileLen / 5);
				avRead->seekPos(filePos);
				avRead->decodeFrame();
				frame = avRead->getFrameData();
				endingFrame = postProcess->processFrame(frame);

				//Draw The red Lines

				if(top!=height)
					{
					ptr = endingFrame->bufferStart + (bot * (width * 4));
					for(i=0;i<width;i++)
						{
						*ptr++ = 0x00;
						*ptr++ = 0x00;
						*ptr++ = 0xff;
						*ptr++ = 0x00;
						}
					}
				if(bot!=height)
					{
					ptr = endingFrame->bufferStart + (((height-1)-top) * (width * 4));
					for(i=0;i<width;i++)
						{
						*ptr++ = 0x00;
						*ptr++ = 0x00;
						*ptr++ = 0xff;
						*ptr++ = 0x00;
						}
					}

				char messTitle[4096];
				char messText[4096];
				globals->translate->translate(messText, "VidomiMessage31");
				globals->translate->translate(messTitle, "VidomiMessage31Title");

				if(vidomiErrorBox(globals->hWnd,messText,messTitle,MB_YESNO | MB_ICONQUESTION)==IDYES)
					{
					//yes
					settings->cropTop = topCrop[hightIndex];
					settings->cropBottom = botCrop[hightIndex];
					if(settings->encodeColorSpace == yv12Frame)
						{
						settings->cropTop = (settings->cropTop+7) & 0xfffffff8;
						settings->cropBottom = (settings->cropBottom+7) & 0xfffffff8;
						}
					}
				else
					{
					//no
					char messTitle[4096];
					char messText[4096];
					globals->translate->translate(messText, "VidomiMessage32");
					globals->translate->translate(messTitle, "VidomiMessage32Title");

					vidomiErrorBox(globals->hWnd,messText,messTitle,MB_OK | MB_ICONINFORMATION);
					settings->cropTop = 0;
					settings->cropBottom=0;
					}
				}
			else
				{
				settings->cropTop = topCrop[hightIndex];
				settings->cropBottom = botCrop[hightIndex];
				if(settings->encodeColorSpace == yv12Frame)
					{
					settings->cropTop = (settings->cropTop+7) & 0xfffffff8;
					settings->cropBottom = (settings->cropBottom+7) & 0xfffffff8;
					}
				}
			}
		else
			{
			//We're probably OK in going with these values
			settings->cropTop = topCrop[hightIndex];
			settings->cropBottom = botCrop[hightIndex];
			if(settings->encodeColorSpace == yv12Frame)
				{
				settings->cropTop = (settings->cropTop+7) & 0xfffffff8;
				settings->cropBottom = (settings->cropBottom+7) & 0xfffffff8;
				}
			}

		//Hide our output window
		globals->outputVideo->hideOutputWindow();
		}

	logPrintf("Cleaning up after Crop Rect Scan\n");
	delete postProcess;
	avRead->end();
	delete avRead;
	logPrintf("Ended Crop Rect Scan\n");

	return(true);
}

// *****************************************************************************
// scanFileClass::determineRangeSize()
//
// scans an mpeg file for useful info. 
// *****************************************************************************
bool scanFileClass::determineRangeSize(CONVERTCONFIG * convertConfig)
{
	int numRanges;
	avReadClass * avRead;
	int fileNum;
	int i;

	numRanges = 0;

	for(i=0;i<MAX_RANGES;i++)
		{
		if(convertConfig->rangelist[i].valid)
			{
			numRanges++;
			}
		}
	if(numRanges==0)
		{
		convertConfig->rangelist[0].endingFilePos = 0x7fffffffffffffff;
		convertConfig->rangelist[0].endingFileFrame =0;
		convertConfig->rangelist[0].startingFilePos = 0;
		convertConfig->rangelist[0].startingFileFrame =0;
		convertConfig->rangelist[0].valid = true;
		numRanges =1;
		}
	convertConfig->numRanges = numRanges;

	if((convertConfig->numRanges == 1) && convertConfig->rangelist[0].endingFilePos == 0x7fffffffffffffff)
		{
		// If end range isn't known yet, then open the mpeg and fetch it

		avRead = avRead = new readMpegClass();	
		if(!avRead->init())
			{
			delete avRead;
			return(false);
			}

		// Build the file Size
		avRead->clearInput();
		for(fileNum=0;fileNum<convertConfig->inputFilenameIndex;fileNum++)
			{
			avRead->addInput(convertConfig->inputFilenames[fileNum]);
			}
		if(!avRead->begin(rgb32Frame))
			{
			// Failed, so act like we can't break it up
			delete avRead;
			return(false);
			}
		convertConfig->rangelist[0].endingFilePos = avRead->getLength();

		avRead->end();
		delete avRead;
		}

	return(true);
}