// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// settings.cpp
// 
//
// *****************************************************************************
#include "en.h"

extern GLOBALS * globals;

// *****************************************************************************
// settingsClass::settingsClass()
//
// settings class constructor
// *****************************************************************************
settingsClass::settingsClass()
{
	tempString = new char [32000];
	readString = new char [32000];
	writeStr = new char [MAX_PATH];
}

// *****************************************************************************
// settingsClass::~settingsClass()
//
// settings class destructor
// *****************************************************************************
settingsClass::~settingsClass()
{
	delete [] tempString;
	delete [] readString;
	delete [] writeStr;
}

// *****************************************************************************
// settingsClass::saveSettingsFile()
// *****************************************************************************
bool settingsClass::saveSettingsFile(char * filename)
{
	settingsFile = fopen(filename,"w+");
	if(!settingsFile)
		{
		return(false);
		}

	fileMode = true;

	saveSettings();

	fclose(settingsFile);

	return(true);

}

// *****************************************************************************
// settingsClass::saveSettingsRegistry()
// *****************************************************************************
bool settingsClass::saveSettingsRegistry()
{
	DWORD regReturn;

	regReturn = globals->registry->openCreateReg(&settingsReg,REGISTRY_PROGRAM_NAME);
	if(!regReturn)
		{
		return(false);
		}

	fileMode = false;

	saveSettings();

	globals->registry->closeReg(settingsReg);

	return(true);

}


// *****************************************************************************
// settingsClass::loadSettingsFile()
// *****************************************************************************
bool settingsClass::loadSettingsFile(char * filename)
{
	settingsFile = fopen(filename,"r");
	if(!settingsFile)
		{
		return(false);
		}

	fileMode = true;

	loadSettings();

	fclose(settingsFile);
	
	// Send the loaded Codec settings to the codec..Divx 4 requirement 
	globals->convertConfig->videoFormatOut->hic = ICOpen(globals->convertConfig->videoFormatOut->fccType, globals->convertConfig->videoFormatOut->fccHandler ,ICMODE_COMPRESS);
	ICSetState(globals->convertConfig->videoFormatOut->hic,globals->convertConfig->videoFormatOut->lpState,globals->convertConfig->videoFormatOut->cbState);
	if(globals->convertConfig->videoFormatOut->hic)
		{
		//globals->convertConfig->videoFormatOut->hic->unused = 0;
		ICClose(globals->convertConfig->videoFormatOut->hic);
		globals->convertConfig->videoFormatOut->hic = NULL;
		}
	return(true);

}

// *****************************************************************************
// settingsClass::loadCodecCfgFile()
// *****************************************************************************
bool settingsClass::loadCodecCfgFile(char * filename)
{
	settingsFile = fopen(filename,"r");
	if(!settingsFile)
		{
		return(false);
		}

	fileMode = true;

	// Xvid Offsets
	getInt("xvidEncodeModeOffset" , &globals->convertConfig->xvidEncodeModeOffset);
	getInt("xvidBitrateOffset" , &globals->convertConfig->xvidBitrateOffset);
	getInt("xvidDesiredSizeOffset" , &globals->convertConfig->xvidDesiredSizeOffset);
	getInt("xvidStatsfilenameOffset" , &globals->convertConfig->xvidStatsFilenameOffset);

	// Divx Networks Offsets
	getInt("divx4VbrModeOffset" , &globals->convertConfig->divx4VbrModeOffset);
	getInt("divx4BitrateOffset" , &globals->convertConfig->divx4BitrateOffset);
	getInt("divx5BitrateOffset" , &globals->convertConfig->divx5BitrateOffset);
	getInt("divx5VbrModeOffset" , &globals->convertConfig->divx5VbrModeOffset);

	fclose(settingsFile);
	
	return(true);

}

// *****************************************************************************
// settingsClass::loadSettingsRegistry()
// *****************************************************************************
bool settingsClass::loadSettingsRegistry()
{
	DWORD regReturn;

	regReturn = globals->registry->openCreateReg(&settingsReg,REGISTRY_PROGRAM_NAME);
	if(!regReturn)
		{
		return(false);
		}

	fileMode = false;

	loadSettings();

	globals->registry->closeReg(settingsReg);

	return(true);

}

// *****************************************************************************
// settingsClass::loadSettings()
// *****************************************************************************
void settingsClass::loadSettings()
{
	int channel;
	int i;

	if(!fileMode)
		{
		// These vars aren't saved in the text file
		getInt("optionsLogOutput" , &globals->optionsLogOutput);
		getInt("statsLeft" , &globals->statsLeft);
		getInt("statsTop" , &globals->statsTop);
		getInt("jobLeft" , &globals->jobLeft);
		getInt("jobTop" , &globals->jobTop);
		getInt("encodeControlLeft" , &globals->encodeControlLeft);
		getInt("encodeControlTop" , &globals->encodeControlTop);
		getBool("contactVidomi" , &globals->contactVidomi);
		getBool("showOutput" , &globals->convertConfig->showOutput);
		getBool("showStats" , &globals->convertConfig->showStats);
		getBool("multiThread" , &globals->convertConfig->multiThread);
		getStr("outputTail" , &globals->convertConfig->outputTail[0],6); 
		getStr("browseSourcePath" , globals->browseSourcePath,MAX_PATH); 
		getInt("browseSourceDefaultType" , &globals->browseSourceDefaultType);
		getStr("browseDestPath" , globals->browseDestPath,MAX_PATH); 
		getInt("browseDestDefaultType" , &globals->browseDestDefaultType);
		getStr("browseSettingsPath" , globals->browseSettingsPath,MAX_PATH); 
		getInt("browseSettingsDefaultType" , &globals->browseSettingsDefaultType);
		getBool("useInfoTxtFile" , &globals->convertConfig->useInfoTxtFile);
		getInt("scanLength" , &globals->convertConfig->scanLength);
		getStr("vidomiLanguage" , globals->vidomiLanguage,MAX_PATH); 
		getStr("browseJobListPath" , globals->browseJobListPath,MAX_PATH); 
		getInt("browseJobListDefaultType" , &globals->browseJobListDefaultType);

		getBool("encodeServerEnabled" , &globals->encodeServerEnabled);
		getStr("encodeServerIPAddress" , globals->encodeServerIPAddress,MAX_PATH); 		
		}
	// Pass 1
	getInt("videoFormatSelected" , (int *)&globals->convertConfig->videoFormatOut->fccHandler);
	getValN("videoFormatBitmapOut" , (unsigned char *)globals->convertConfig->videoFormatOut->lpbiOut,VIDOMI_BITMAP_INFO_SIZE);
	getInt("videoFormatlKey" , (int *)&globals->convertConfig->videoFormatOut->lKey);
	getInt("videoFormatlQ" , (int *)&globals->convertConfig->videoFormatOut->lQ);
	getInt("videoFormatStateSize" , (int *)&globals->convertConfig->videoFormatOut->cbState);
	getValN("videoFormatState" , (unsigned char *) globals->convertConfig->videoFormatOut->lpState, VIDEO_STATE_DEFAULT_SIZE); 

	//Pass 2
	getInt("videoFormatP2Selected" , (int *)&globals->convertConfig->videoFormatOutP2->fccHandler);
	getValN("videoFormatP2BitmapOut" , (unsigned char *)globals->convertConfig->videoFormatOutP2->lpbiOut,VIDOMI_BITMAP_INFO_SIZE);
	getInt("videoFormatP2lKey" , (int *)&globals->convertConfig->videoFormatOutP2->lKey);
	getInt("videoFormatP2lQ" , (int *)&globals->convertConfig->videoFormatOutP2->lQ);
	getInt("videoFormatP2StateSize" , (int *)&globals->convertConfig->videoFormatOutP2->cbState);
	getValN("videoFormatP2State" , (unsigned char *) globals->convertConfig->videoFormatOutP2->lpState, VIDEO_STATE_DEFAULT_SIZE); 

	getValN("audioFormatState" , (unsigned char *) globals->convertConfig->audioFormatOut, AUDIO_STATE_DEFAULT_SIZE);
	getBool("mp3" , &globals->convertConfig->mp3);
	for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
		{
		sprintf(writeStr,"mp3EncodeMode%d",channel);
		getInt(writeStr , &globals->convertConfig->mp3EncodeMode[channel]);
		sprintf(writeStr,"mp3ChannelType%d",channel);
		getInt(writeStr , &globals->convertConfig->mp3ChannelType[channel]);
		sprintf(writeStr,"mp3Quality%d",channel);
		getInt(writeStr , &globals->convertConfig->mp3Quality[channel]);
		sprintf(writeStr,"mp3MinBitrate%d",channel);
		getInt(writeStr , &globals->convertConfig->mp3MinBitrate[channel]);
		sprintf(writeStr,"mp3MaxBitrate%d",channel);
		getInt(writeStr , &globals->convertConfig->mp3MaxBitrate[channel]);
		sprintf(writeStr,"mp3Outputfrequency%d",channel);
		getInt(writeStr , &globals->convertConfig->mp3OutputFrequency[channel]);
		sprintf(writeStr,"mp3VbrQuality%d",channel);
		getInt(writeStr , &globals->convertConfig->mp3VbrQuality[channel]);
		sprintf(writeStr,"audioDirectStreamCopy%d",channel);
		getBool(writeStr , &globals->convertConfig->audioDirectStreamCopy[channel]);
		}

	for(channel=0;channel<MAX_SUBTITLE_CHANNELS;channel++)
		{
		sprintf(writeStr,"processSubtitleOnlyEven%d",channel);
		getBool(writeStr , &globals->convertConfig->processSubtitleOnlyEven[channel]);
		}

	getInt("maxOutputSize" , &globals->convertConfig->maxOutputSize);
	getBool("deInterlace" , &globals->convertConfig->deinterlace);
	getBool("blend" , &globals->convertConfig->blend);
	getInt("blendThreshold" , &globals->convertConfig->blendThreshold);
	getInt("scaleType" , &globals->convertConfig->scaleType);
	getInt("scaleWidth" , &globals->convertConfig->scaleWidth);
	getInt("scaleHeight" , &globals->convertConfig->scaleHeight);
	getInt("scaleBy" , &globals->convertConfig->scaleBy);
	getBool("scale" , &globals->convertConfig->scale);
	getBool("resample48KhzAudio" , &globals->convertConfig->resample48KhzAudio);
	getInt("hyperCropThreshold" , &globals->convertConfig->hyperCropThreshold);
	getBool("forceMultiple16" , &globals->convertConfig->forceMultiple16);
	getInt("vbrNumberOfDiscs" , &globals->convertConfig->vbrNumberOfDiscs);
	getInt("vbrReserve" , &globals->convertConfig->vbrReserve);
	getInt("vbrAudioBitrate" , &globals->convertConfig->vbrAudioBitrate);
	getInt("normalizeLevel" , &globals->convertConfig->normalizeLevel);
	getInt("normalizeAudio" , &globals->convertConfig->normalizeAudio);
	getInt("encodeMode" , &globals->convertConfig->encodeMode);
	getInt("vbrCrispness" , &globals->convertConfig->vbrCrispness);
	getInt("vbrMinKeyFrame" , &globals->convertConfig->vbrMinKeyFrame);
	getInt("vbrMaxKeyFrame" , &globals->convertConfig->vbrMaxKeyFrame);
	getInt("vbrMinBitrate" , &globals->convertConfig->vbrMinBitrate);
	getInt("vbrMaxBitrate" , &globals->convertConfig->vbrMaxBitrate);
	getInt("vbrMinReservoir" , &globals->convertConfig->vbrMinReservoir);
	getInt("vbrMaxReservoir" , &globals->convertConfig->vbrMaxReservoir);
	getBool("processAudio" , &globals->convertConfig->processAudio);
	getBool("lumaProcessing" , &globals->convertConfig->lumaProcessing);
	getBool("lumaLowProcessing" , &globals->convertConfig->lumaLowProcessing);
	getBool("lumaDeltaProcessing" , &globals->convertConfig->lumaDeltaProcessing);
	getFloat0to1("lumaLowThresh" , &globals->convertConfig->lumaLowThresh);
	getInt("lumaLowBoost" , &globals->convertConfig->lumaLowBoost);
	getFloat0to1("lumaDeltaThresh" , &globals->convertConfig->lumaDeltaThresh);
	getInt("lumaDeltaBoost" , &globals->convertConfig->lumaDeltaBoost);
	getBool("vbrDeletePassOneFiles" , &globals->convertConfig->vbrDeletePassOneFiles);
	getBool("hyperCrop" , &globals->convertConfig->hyperCrop);
	getBool("hyperScale" , &globals->convertConfig->hyperScale);
	getInt("cropLeft" , &globals->convertConfig->cropLeft);
	getInt("cropRight" , &globals->convertConfig->cropRight);
	getInt("cropTop" , &globals->convertConfig->cropTop);
	getInt("cropBottom" , &globals->convertConfig->cropBottom);
	getBool("introAvi" , &globals->convertConfig->introAvi);
	getBool("useSpecifiedTarget" , &globals->convertConfig->useSpecifiedTarget);
	getInt("specifiedTarget" , &globals->convertConfig->specifiedTarget);
	getStr("introAviPrimaryPath" , globals->convertConfig->introAviPrimaryPath,MAX_PATH);
	getStr("introAviSecondaryPath" , globals->convertConfig->introAviSecondaryPath,MAX_PATH);
	getInt("ivtcFilmMode" , &globals->convertConfig->ivtcFilmMode);
	getInt("ivtcOffset" , &globals->convertConfig->ivtcOffset);
	getBool("ivtcPolarity" , &globals->convertConfig->ivtcPolarity);
	getBool("overwriteExistingFiles" , &globals->convertConfig->overwriteExistingFiles);
	getBool("scanPassScale" , &globals->convertConfig->scanPassScale);
	getInt("defaultOutputSize" , &globals->convertConfig->defaultOutputSize);
	getBool("stopAfterPassOne" , &globals->convertConfig->stopAfterPassOne);
	getBool("mp3LimitToCbr" , &globals->convertConfig->mp3LimitToCbr);
	getBool("mp3LimitToMpeg1" , &globals->convertConfig->mp3LimitToMpeg1);
	getBool("aspectRatioDontCorrect43" , &globals->convertConfig->aspectRatioDontCorrect43);
	getInt("dualCpuEncodeBuffers" , &globals->dualCpuEncodeBuffers);
	getInt("vbrQualityBitrate" , &globals->convertConfig->vbrQualityBitrate);
	getInt("encodeColorSpace" , &globals->convertConfig->encodeColorSpace);
	getBool("enhancedAvi" , &globals->convertConfig->enhancedAvi);
	getBool("insertChapters" , &globals->convertConfig->insertChapters);
	getBool("hyperEncode" , &globals->convertConfig->hyperEncode);
	getBool("mpegConversion" , &globals->convertConfig->mpegConversion);
	
	for(i=0;i<MAX_SUBTITLE_CHANNELS;i++)
		{
		sprintf(writeStr,"subtitleStreamName%d",i);
		getStr(writeStr , globals->convertConfig->subtitleStreamName[i],MAX_PATH); 
		}

	for(i=0;i<MAX_AUDIO_CHANNELS;i++)
		{
		sprintf(writeStr,"audioStreamName%d",i);
		getStr(writeStr , globals->convertConfig->audioStreamName[i],MAX_PATH); 
		}

	if(fileMode)
		{
		for(i=0;i<MAX_CHAPTERS;i++)
			{
			sprintf(writeStr,"chapterName%d",i);
			getValN(writeStr , (unsigned char *)globals->convertConfig->chapterName[i],64*2); 
			}
		}

	for(i=0;i<4;i++)
		{
		sprintf(writeStr,"subtitleColorData%d",i);
		getInt(writeStr , (int *) &globals->convertConfig->subtitleColorData[i]);
		}

	if(fileMode)
		{
		for (channel=0;channel<32;channel++) 
			{
			sprintf(writeStr,"rangeStartingFilePos%d",channel);
			getValN(writeStr , (unsigned char *)&globals->convertConfig->rangelist[channel].startingFilePos,8);
			sprintf(writeStr,"rangeStartingFileFrame%d",channel);
			getInt(writeStr , &globals->convertConfig->rangelist[channel].startingFileFrame);
			sprintf(writeStr,"rangeEndingFilePos%d",channel);
			getValN(writeStr , (unsigned char *)&globals->convertConfig->rangelist[channel].endingFilePos,8);
			sprintf(writeStr,"rangeEndingFileFrame%d",channel);
			getInt(writeStr ,&globals->convertConfig->rangelist[channel].endingFileFrame);
			sprintf(writeStr,"rangeValid%d",channel);
			getInt(writeStr , &globals->convertConfig->rangelist[channel].valid);	
			}
		}
}

// *****************************************************************************
// settingsClass::saveSettings()
// *****************************************************************************
void settingsClass::saveSettings()
{
	int channel;
	int i;

	if(!fileMode)
		{
		// These vars aren't saved in the text file
		saveInt("statsLeft" , globals->statsLeft);
		saveInt("statsTop" , globals->statsTop);
		saveInt("jobLeft" , globals->jobLeft);
		saveInt("jobTop" , globals->jobTop);
		saveInt("encodeControlLeft" , globals->encodeControlLeft);
		saveInt("encodeControlTop" , globals->encodeControlTop);
		if(globals->vidomiMode == MODE_ENCODER)
			{
			saveBool("contactVidomi" , globals->contactVidomi);
			saveInt("optionsLogOutput" , globals->optionsLogOutput);
			}
		saveBool("showOutput" , globals->convertConfig->showOutput);
		saveBool("showStats" , globals->convertConfig->showStats);
		saveBool("multiThread" , globals->convertConfig->multiThread);
		saveStr("outputTail" , &globals->convertConfig->outputTail[0]);
		saveStr("browseSourcePath" , globals->browseSourcePath);
		saveInt("browseSourceDefaultType" , globals->browseSourceDefaultType);
		saveStr("browseDestPath" , globals->browseDestPath);
		saveInt("browseDestDefaultType" , globals->browseDestDefaultType);
		saveStr("browseSettingsPath" , globals->browseSettingsPath);
		saveInt("browseSettingsDefaultType" , globals->browseSettingsDefaultType);
		saveBool("useInfoTxtFile" , globals->convertConfig->useInfoTxtFile);
		saveInt("scanLength" , globals->convertConfig->scanLength);
		saveStr("browseJobListPath" , globals->browseJobListPath);
		saveInt("browseJobListDefaultType" , globals->browseJobListDefaultType);

		saveBool("encodeServerEnabled" , globals->encodeServerEnabled);
		saveStr("encodeServerIPAddress" , globals->encodeServerIPAddress);
		}

	// Pass 1
	saveInt("videoFormatSelected" , globals->convertConfig->videoFormatOut->fccHandler);
	saveValN("videoFormatBitmapOut" , (unsigned char *)globals->convertConfig->videoFormatOut->lpbiOut, VIDOMI_BITMAP_INFO_SIZE);
	saveInt("videoFormatlKey" , globals->convertConfig->videoFormatOut->lKey);
	saveInt("videoFormatlQ" , globals->convertConfig->videoFormatOut->lQ);
	saveInt("videoFormatStateSize" , globals->convertConfig->videoFormatOut->cbState);
	saveValN("videoFormatState" , (unsigned char *)globals->convertConfig->videoFormatOut->lpState, globals->convertConfig->videoFormatOut->cbState);

	// Pass 2
	saveInt("videoFormatP2Selected" , globals->convertConfig->videoFormatOutP2->fccHandler);
	saveValN("videoFormatP2BitmapOut" , (unsigned char *)globals->convertConfig->videoFormatOutP2->lpbiOut, VIDOMI_BITMAP_INFO_SIZE);
	saveInt("videoFormatP2lKey" , globals->convertConfig->videoFormatOutP2->lKey);
	saveInt("videoFormatP2lQ" , globals->convertConfig->videoFormatOutP2->lQ);
	saveInt("videoFormatP2StateSize" , globals->convertConfig->videoFormatOutP2->cbState);
	saveValN("videoFormatP2State" , (unsigned char *)globals->convertConfig->videoFormatOutP2->lpState, globals->convertConfig->videoFormatOut->cbState);

	saveValN("audioFormatState" , (unsigned char *)globals->convertConfig->audioFormatOut, sizeof(WAVEFORMATEX) + globals->convertConfig->audioFormatOut->wfx.cbSize);
	saveBool("mp3" , globals->convertConfig->mp3);

	for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
		{
		sprintf(writeStr,"mp3MinBitrate%d",channel);
		saveInt(writeStr , globals->convertConfig->mp3MinBitrate[channel]);
		sprintf(writeStr,"mp3MaxBitrate%d",channel);
		saveInt(writeStr , globals->convertConfig->mp3MaxBitrate[channel]);
		sprintf(writeStr,"mp3EncodeMode%d",channel);
		saveInt(writeStr , globals->convertConfig->mp3EncodeMode[channel]);
		sprintf(writeStr,"mp3ChannelType%d",channel);
		saveInt(writeStr , globals->convertConfig->mp3ChannelType[channel]);
		sprintf(writeStr,"mp3Quality%d",channel);
		saveInt(writeStr , globals->convertConfig->mp3Quality[channel]);
		sprintf(writeStr,"mp3OutputFrequency%d",channel);
		saveInt(writeStr , globals->convertConfig->mp3OutputFrequency[channel]);
		sprintf(writeStr,"mp3VbrQuality%d",channel);
		saveInt(writeStr , globals->convertConfig->mp3VbrQuality[channel]);
		sprintf(writeStr,"audioDirectStreamCopy%d",channel);
		saveBool(writeStr , globals->convertConfig->audioDirectStreamCopy[channel]);
		}

	for(channel=0;channel<MAX_SUBTITLE_CHANNELS;channel++)
		{
		sprintf(writeStr,"processSubtitleOnlyEven%d",channel);
		saveBool(writeStr , globals->convertConfig->processSubtitleOnlyEven[channel]);	
		}

	saveInt("maxOutputSize" , globals->convertConfig->maxOutputSize);
	saveBool("deInterlace" , globals->convertConfig->deinterlace);
	saveBool("blend" , globals->convertConfig->blend);
	saveInt("blendThreshold" , globals->convertConfig->blendThreshold);
	saveInt("scaleType" , globals->convertConfig->scaleType);
	saveInt("scaleWidth" , globals->convertConfig->scaleWidth);
	saveInt("scaleHeight" , globals->convertConfig->scaleHeight);
	saveInt("scaleBy" , globals->convertConfig->scaleBy);
	saveBool("scale" , globals->convertConfig->scale);
	saveBool("resample48KhzAudio" , globals->convertConfig->resample48KhzAudio);
	saveInt("hyperCropThreshold" , globals->convertConfig->hyperCropThreshold);
	saveBool("forceMultiple16" , globals->convertConfig->forceMultiple16);
	saveInt("vbrNumberOfDiscs" , globals->convertConfig->vbrNumberOfDiscs);
	saveInt("vbrReserve" , globals->convertConfig->vbrReserve);
	saveInt("vbrAudioBitrate" , globals->convertConfig->vbrAudioBitrate);
	saveInt("normalizeLevel" , globals->convertConfig->normalizeLevel);
	saveInt("normalizeAudio" , globals->convertConfig->normalizeAudio);
	saveInt("encodeMode" , globals->convertConfig->encodeMode);
	saveBool("film" , globals->convertConfig->film);
	saveInt("vbrCrispness" , globals->convertConfig->vbrCrispness);
	saveInt("vbrMinKeyFrame" , globals->convertConfig->vbrMinKeyFrame);
	saveInt("vbrMaxKeyFrame" , globals->convertConfig->vbrMaxKeyFrame);
	saveInt("vbrMinBitrate" , globals->convertConfig->vbrMinBitrate);
	saveInt("vbrMaxBitrate" , globals->convertConfig->vbrMaxBitrate);
	saveInt("vbrMinReservoir" , globals->convertConfig->vbrMinReservoir);
	saveInt("vbrMaxReservoir" , globals->convertConfig->vbrMaxReservoir);
	saveBool("processAudio" , globals->convertConfig->processAudio);
	saveBool("lumaProcessing" , globals->convertConfig->lumaProcessing);
	saveBool("lumaLowProcessing" , globals->convertConfig->lumaLowProcessing);
	saveBool("lumaDeltaProcessing" , globals->convertConfig->lumaDeltaProcessing);
	saveFloat0to1("lumaLowThresh" , globals->convertConfig->lumaLowThresh);
	saveInt("lumaLowBoost" , globals->convertConfig->lumaLowBoost);
	saveFloat0to1("lumaDeltaThresh" , globals->convertConfig->lumaDeltaThresh);
	saveInt("lumaDeltaBoost" , globals->convertConfig->lumaDeltaBoost);
	saveBool("vbrDeletePassOneFiles" , globals->convertConfig->vbrDeletePassOneFiles);
	saveBool("hyperCrop" , globals->convertConfig->hyperCrop);
	saveBool("hyperScale" , globals->convertConfig->hyperScale);
	saveInt("cropLeft" , globals->convertConfig->cropLeft);
	saveInt("cropRight" , globals->convertConfig->cropRight);
	saveInt("cropTop" , globals->convertConfig->cropTop);
	saveInt("cropBottom" , globals->convertConfig->cropBottom);
	saveBool("introAvi" , globals->convertConfig->introAvi);
	saveBool("useSpecifiedTarget" , globals->convertConfig->useSpecifiedTarget);
	saveInt("specifiedTarget" , globals->convertConfig->specifiedTarget);
	saveStr("introAviPrimaryPath" , globals->convertConfig->introAviPrimaryPath);
	saveStr("introAviSecondaryPath" , globals->convertConfig->introAviSecondaryPath);
	saveInt("ivtcFilmMode" , globals->convertConfig->ivtcFilmMode);
	saveInt("ivtcOffset" , globals->convertConfig->ivtcOffset);
	saveBool("ivtcPolarity" , globals->convertConfig->ivtcPolarity);
	saveBool("overwriteExistingFiles" , globals->convertConfig->overwriteExistingFiles);
	saveBool("scanPassScale" , globals->convertConfig->scanPassScale);
	saveInt("defaultOutputSize" , globals->convertConfig->defaultOutputSize);
	saveBool("stopAfterPassOne" , globals->convertConfig->stopAfterPassOne);
	saveBool("mp3LimitToCbr" , globals->convertConfig->mp3LimitToCbr);
	saveBool("mp3LimitToMpeg1" , globals->convertConfig->mp3LimitToMpeg1);
	saveBool("aspectRatioDontCorrect43" , globals->convertConfig->aspectRatioDontCorrect43);
	saveInt("dualCpuEncodeBuffers" , globals->dualCpuEncodeBuffers);
	saveInt("vbrQualityBitrate" , globals->convertConfig->vbrQualityBitrate);
	saveInt("encodeColorSpace" , globals->convertConfig->encodeColorSpace);
	saveBool("enhancedAvi" , globals->convertConfig->enhancedAvi);
	saveBool("insertChapters" , globals->convertConfig->insertChapters);
	saveBool("hyperEncode" , globals->convertConfig->hyperEncode);
	saveBool("mpegConversion" , globals->convertConfig->mpegConversion);


	for(i=0;i<MAX_SUBTITLE_CHANNELS;i++)
		{
		sprintf(writeStr,"subtitleStreamName%d",i);
		saveStr(writeStr , globals->convertConfig->subtitleStreamName[i]);
		}

	for(i=0;i<MAX_AUDIO_CHANNELS;i++)
		{
		sprintf(writeStr,"audioStreamName%d",i);
		saveStr(writeStr , globals->convertConfig->audioStreamName[i]);
		}

	if(fileMode)
		{
		for(i=0;i<MAX_CHAPTERS;i++)
			{
			sprintf(writeStr,"chapterName%d",i);
			saveValN(writeStr , (unsigned char *)globals->convertConfig->chapterName[i], 64*2);
			}
		}

	for(i=0;i<4;i++)
		{
		sprintf(writeStr,"subtitleColorData%d",i);
		saveInt(writeStr , globals->convertConfig->subtitleColorData[i]);	
		}

	if(fileMode)
		{
		for (channel=0;channel<MAX_RANGES;channel++) 
			{
			sprintf(writeStr,"rangeStartingFilePos%d",channel);
			saveValN(writeStr , (unsigned char *)&globals->convertConfig->rangelist[channel].startingFilePos,8);
			sprintf(writeStr,"rangeStartingFileFrame%d",channel);
			saveInt(writeStr , globals->convertConfig->rangelist[channel].startingFileFrame);
			sprintf(writeStr,"rangeEndingFilePos%d",channel);
			saveValN(writeStr , (unsigned char *)&globals->convertConfig->rangelist[channel].endingFilePos,8);
			sprintf(writeStr,"rangeEndingFileFrame%d",channel);
			saveInt(writeStr ,globals->convertConfig->rangelist[channel].endingFileFrame);
			sprintf(writeStr,"rangeValid%d",channel);
			saveInt(writeStr , globals->convertConfig->rangelist[channel].valid);	
			}
		}
}

// *****************************************************************************
// settingsClass::getBool()
// *****************************************************************************
void settingsClass::getBool( char * keyStr , bool * var)
{
	char * ptr;
	int value;

	if(fileMode)	
		{
		fseek(settingsFile,0,SEEK_SET);
		while(!feof(settingsFile))
			{
			fgets(readString,32000,settingsFile);
			ptr = strchr(readString,'=');
			if(ptr)
				{
				*ptr=0;
				}
			if(!strcmp(readString,keyStr))
				{
				ptr++;
				value = atoi(ptr);
				//This is done to quiet VC++
				if(value)
					*var = true;
				else
					*var = false;
				break;
				}
			}
		}
	else
		{
		globals->registry->getSetRegBool(settingsReg,keyStr,var);
		}
}

// *****************************************************************************
// settingsClass::getInt()
// *****************************************************************************
void settingsClass::getInt( char * keyStr , int * var)
{
	char * ptr;
	int value;

	if(fileMode)	
		{
		fseek(settingsFile,0,SEEK_SET);
		while(!feof(settingsFile))
			{
			fgets(readString,32000,settingsFile);
			ptr = strchr(readString,'=');
			if(ptr)
				{
				*ptr=0;
				}
			if(!strcmp(readString,keyStr))
				{
				ptr++;
				value = atoi(ptr);
				*var = value;
				break;
				}
			}
		}
	else
		{
		globals->registry->getSetRegVal(settingsReg,keyStr,var);
		}
}


// *****************************************************************************
// settingsClass::getInt()
// *****************************************************************************
void settingsClass::getValN( char * keyStr , unsigned char * addr,  int size)
{
	char * ptr;
	char * ptr1;
	unsigned char * outPtr;;
	int count;
	int i;
	int lineNumber;
	int bytesRead;
	int totalRead;

	if(fileMode)	
		{
		totalRead=0;
		outPtr = addr;
		for(lineNumber=1;lineNumber<1024;lineNumber++)
			{
			bytesRead=0;
			sprintf(tempString,"%s%d",keyStr,lineNumber);	
			fseek(settingsFile,0,SEEK_SET);
			while(!feof(settingsFile))
				{
				fgets(readString,32000,settingsFile);
				ptr = strchr(readString,'=');
				if(ptr)
					{
					*ptr=0;
					}
				if(!strcmp(readString,tempString))
					{
					ptr++;
					ptr1 = strchr(ptr,' ');
					if(ptr1)
						{
						*ptr1 = 0;
						count = atoi(ptr);
						ptr1++;
						ptr = ptr1;
						}
					else
						{
						break;
						}
					for(i=0;i<count;i++)
						{
						ptr1 = strchr(ptr,' ');
						if(ptr1)
							{
							*ptr1 = 0;
							*outPtr++ = atoi(ptr);
							bytesRead++;
							totalRead++;
							ptr1++;
							ptr = ptr1;
							if(totalRead>=size)
								{
								break;
								}
							}
						else
							{
							break;
							}
						}
					break;
					}
				}
			//If we failed to find any then bail out
			if(!bytesRead)
				break;
			if(totalRead>=size)
				{
				break;
				}

			}
		
		}
	else
		{
		globals->registry->getSetRegValN(settingsReg,keyStr, addr,  size);
		}
}

// *****************************************************************************
// settingsClass::getFloat()
// *****************************************************************************
void settingsClass::getFloat0to1( char * keyStr , float * var)
{
	char * ptr;
	float value;

	if(fileMode)	
		{
		fseek(settingsFile,0,SEEK_SET);
		while(!feof(settingsFile))
			{
			fgets(readString,32000,settingsFile);
			ptr = strchr(readString,'=');
			if(ptr)
				{
				*ptr=0;
				}
			if(!strcmp(readString,keyStr))
				{
				ptr++;
				value = (float)atof(ptr);
				*var = value;
				break;
				}
			}
		}
	else
		{
		globals->registry->getSetRegFloat0to1(settingsReg,keyStr,var);
		}
}

// *****************************************************************************
// settingsClass::getInt()
// *****************************************************************************
void settingsClass::getStr( char * keyStr , char * string, int stringLength)
{
	char * ptr;

	if(fileMode)	
		{
		fseek(settingsFile,0,SEEK_SET);
		while(!feof(settingsFile))
			{
			fgets(readString,32000,settingsFile);
			ptr = strchr(readString,'=');
			if(ptr)
				{
				*ptr=0;
				}
			if(!strcmp(readString,keyStr))
				{
				ptr++;
				if(strlen(ptr)<(unsigned int)stringLength)
					{
					strncpy(string,ptr,strlen(ptr)-1);
					}
				break;
				}
			}
		}
	else
		{
		globals->registry->getSetRegStr(settingsReg,keyStr,string,stringLength);
		}
}

// *****************************************************************************
// settingsClass::saveBool()
// *****************************************************************************
void settingsClass::saveBool( char * keyStr , DWORD val)
{
	if(fileMode)	
		{
		sprintf(tempString,"%s=%d\n",keyStr,val);
		fputs(tempString,settingsFile);
		}
	else
		{
		globals->registry->setRegBool(settingsReg,keyStr,val);
		}
}

// *****************************************************************************
// settingsClass::saveInt()
// *****************************************************************************
void settingsClass::saveInt( char * keyStr , int val)
{
	if(fileMode)	
		{
		sprintf(tempString,"%s=%d\n",keyStr,val);
		fputs(tempString,settingsFile);
		}
	else
		{
		globals->registry->setRegVal(settingsReg,keyStr,val);
		}
}

// *****************************************************************************
// settingsClass::saveInt()
// *****************************************************************************
void settingsClass::saveValN( char * keyStr , unsigned char * addr, int size)
{
	int i;
	unsigned char * ptr;
	int lineNumber;
	int tempSize;
	int	lineSize;
	if(fileMode)	
		{		
		ptr = addr;
		lineNumber =1;
		tempSize = size;
		for(;;)
			{
			if(size>16)
				{
				lineSize = 16;
				}
			else
				{
				lineSize = size;
				}
			sprintf(tempString,"%s%d=%d ",keyStr,lineNumber,lineSize);
			fputs(tempString,settingsFile);
			for(i=0;i<lineSize;i++)
				{
				size--;
				sprintf(tempString,"%d ",*ptr++);
				fputs(tempString,settingsFile);
				}
			sprintf(tempString,"\n");
			fputs(tempString,settingsFile);
			lineNumber++;
			if(!size)
				break;
			}
		
		}
	else
		{
		globals->registry->setRegValN(settingsReg,keyStr,addr,size);
		}
}
// *****************************************************************************
// settingsClass::saveInt()
// *****************************************************************************
void settingsClass::saveFloat0to1( char * keyStr , float val)
{
	if(fileMode)	
		{
		sprintf(tempString,"%s=%f\n",keyStr,val);
		fputs(tempString,settingsFile);
		}
	else
		{
		globals->registry->setRegFloat0to1(settingsReg,keyStr,val);
		}
}

// *****************************************************************************
// settingsClass::saveInt()
// *****************************************************************************
void settingsClass::saveStr( char * keyStr , char * val)
{
	if(fileMode)	
		{
		sprintf(tempString,"%s=%s\n",keyStr,val);
		fputs(tempString,settingsFile);
		}
	else
		{
		globals->registry->setRegStr(settingsReg,keyStr,val);
		}
}