// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
// Slave Encoder
//
// *****************************************************************************
#include "../en.h"

GLOBALS * globals;

int PASCAL WinMain(HINSTANCE hInst,HINSTANCE hPrevInst, LPSTR lpszCmd ,int iCmdShow)
{
	MSG msg;                  
	char pathToLanguageFiles[MAX_PATH];
	HKEY regKey;
	DWORD regReturn;

	// Now Configure the globals
	configureGlobals(hInst);

	// Now Init the Logging mechanism
	clearLog();	//clear text based log file
	
	logPrintf("Vidomi Slave Encoder Version %.3f Started...\n",globals->version);
	logPrintf("Path to Vidomi is %s\n",globals->pathToExeDir);

	logDumpMemoryStatus("Memory on Startup");
	// Create our Classes (And set defaults in constructors)
	
	globals->translate = new translateClass();
	logPrintf("translateClass created.\n");
	
	// Give us a default language for windows and text
	strcpy(pathToLanguageFiles,globals->pathToExeDir);
	strcat(pathToLanguageFiles,"\\languages\\english.lng");
	if(!globals->translate->setDefaultLanguageFile(pathToLanguageFiles))
		{
		strcpy(pathToLanguageFiles,globals->pathToExeDir);
		strcat(pathToLanguageFiles,"\\..\\..\\..\\installer\\languages\\english.lng");
		globals->translate->setDefaultLanguageFile(pathToLanguageFiles);
		}
	// And now load the language selected

	strcpy(pathToLanguageFiles,globals->pathToExeDir);
	strcat(pathToLanguageFiles,"\\languages\\");
	strcat(pathToLanguageFiles,globals->vidomiLanguage);
	strcat(pathToLanguageFiles,".lng");
	globals->translate->setLanguageFile(pathToLanguageFiles);

	globals->slaveWindow = new slaveWindowClass();
	globals->slaveWindow->createWindow(hInst);
	logPrintf("slaveWindowClass created.\n");

	globals->conversion = new conversionClass();
	logPrintf("conversionClass created.\n");

	globals->jobHelper = new jobHelperClass();
	logPrintf("jobHelperClass created.\n");

	globals->netSlave = new netSlaveClass();
	logPrintf("netSlaveClass created.\n");

	globals->slaveOptions = new slaveOptionsClass();
	logPrintf("slaveOptionsClass created.\n");

	globals->netSlave->begin();
	logPrintf("netSlave Started Listening..\n");

	// Now Start bringing up the app
	logPrintf("Entering message loop.\n");

	while(GetMessage(&msg, NULL, 0, 0))
		{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
		}	
	
	logPrintf("Shutting Down.\n");

	regReturn = globals->registry->openCreateReg(&regKey,REGISTRY_PROGRAM_NAME);
	
	if(regReturn)
		{
		globals->registry->setRegVal(regKey, "optionsLogOutput" , globals->optionsLogOutput);
		globals->registry->setRegBool(regKey, "obeyShutdown" , globals->obeyShutdown);
		}
	globals->registry->closeReg(regKey);

	globals->netSlave->end();
	delete globals->netSlave;
	delete globals->conversion;

	delete globals->slaveOptions;
	delete globals->slaveWindow;
	delete globals->translate;
	delete globals->registry;
	delete [] globals->errorText;
	delete [] globals;

	return(FALSE);
}


// *****************************************************************************
//  Configure the Globals and put/get from registry
// *****************************************************************************
void configureGlobals(HINSTANCE hInst)
{
	char * commandLine;
	char tempStr[MAX_PATH];
	char * tempPtr;
	char pathToExe[MAX_PATH];
	bool done;
	HKEY regKey;
	DWORD regReturn;

	// Create Global Var Struct
	globals = new GLOBALS [1];
	memset(globals,0,sizeof(GLOBALS));
	
	globals->hWnd = NULL;
	globals->hInst = hInst;
	globals->errorText = new char [8192];

	// Now lets have some fun and create the path to the EXE
	commandLine = GetCommandLine();
	strcpy(tempStr,commandLine);
	done=false;

	//walk through the path until we locate .exe
	tempPtr = tempStr;
	while(1)
		{
		if(!(tempPtr = strchr(tempPtr,'.')))
			{
			// if can't find .exe, then use CWD
			GetCurrentDirectory(MAX_PATH, pathToExe);
			done=true;
			break;
			}
		if((*(tempPtr+1) == 'e') && (*(tempPtr+2)=='x') && (*(tempPtr+3) == 'e'))
			{	
			*(tempPtr+4) = 0;
			break;
			}
		if((*(tempPtr+1) == 'E') && (*(tempPtr+2)=='X') && (*(tempPtr+3) == 'E'))
			{	
			*(tempPtr+3) = 0;
			break;
			}
		//Advance to the next Char
		if(tempPtr)
			{
			tempPtr++;
			}
		}

	if(!done)
		{
		// Remove leading quote if any
		if(tempStr[0] == '\"')
			{
			strcpy(pathToExe,&tempStr[1]);
			}
		else
			{
			strcpy(pathToExe,tempStr);
			}
		
		//Now wack off everything after the last "\"
		if(tempPtr = strrchr(pathToExe,'\\'))
			{
			*tempPtr = 0;
			}
		else
			{
			GetCurrentDirectory(MAX_PATH, pathToExe);
			}
		}

	//pathToExe now holds the path to the EXE dir
	strcpy(globals->pathToExeDir,pathToExe);
	// Set Default Values debug vars
	
	//GetCurrentDirectory(MAX_PATH, globals->pathToExeDir);
	strcpy(globals->optionsLogFilename,globals->pathToExeDir);
	strcat(globals->optionsLogFilename, "\\slavedebug.txt");

	globals->optionsLogOutput = false;

	globals->version=VERSION;
	globals->dualCpuEncodeBuffers=8;
	globals->hideSlaveIcon = false;
	globals->obeyShutdown = false;
	globals->optionsUp = false;

	strcpy(globals->vidomiLanguage, "english");

	globals->registry = new registryClass();

	regReturn = globals->registry->openCreateReg(&regKey,REGISTRY_PROGRAM_NAME);
	if(regReturn)
		{
		globals->registry->getSetRegVal(regKey, "optionsLogOutput" , &globals->optionsLogOutput);
		globals->registry->getSetRegStr(regKey, "vidomiLanguage" , globals->vidomiLanguage, MAX_PATH);
		globals->registry->getSetRegBool(regKey, "hideSlaveIcon" , &globals->hideSlaveIcon);
		globals->registry->getSetRegBool(regKey, "obeyShutdown" , &globals->obeyShutdown);
		}

	globals->registry->closeReg(regKey);

	// Clear the error Text String
	strcpy(globals->errorText,"");

}

// *****************************************************************************
//  Configure the Globals and put/get from registry
// *****************************************************************************
int vidomiErrorBox(HWND hWnd,char * text, char * caption, unsigned int type)
{
	strcat(globals->errorText,text);
	strcat(globals->errorText,"\n");
	return(false);
}
