// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// slaveOptions.cpp
// 
//
// *****************************************************************************

#include "../en.h"

extern GLOBALS * globals;

static TRANSLATEIDTABLE slaveOptionsTID[] = {
										{"VidomiOptionsCancel",VidomiOptionsCancel},
										{"VidomiOptionsOk",VidomiOptionsOk},
										
										{"VidomiSlaveOptionsText1",VidomiSlaveOptionsText1},
										{"VidomiSlaveOptionsDebugText",VidomiSlaveOptionsDebugText},
										{"VidomiSlaveOptionsDisplayIconText",VidomiSlaveOptionsDisplayIconText},
										{"VidomiSlaveOptionsObeyShutdownText",VidomiSlaveOptionsObeyShutdownText},
										{"",0}

										};

// *****************************************************************************
// slaveOptionsClass::slaveOptionsClass()
//
// conversion class constructor
// *****************************************************************************
slaveOptionsClass::slaveOptionsClass()
{

}

// *****************************************************************************
// slaveOptionsClass::~slaveOptionsClass()
//
// conversion class destructor
// *****************************************************************************
slaveOptionsClass::~slaveOptionsClass()
{

}

// *****************************************************************************
// slaveOptionsClass::bringUpOptions()
//
// Bring up the options screen
// *****************************************************************************
void slaveOptionsClass::bringUpOptions(HINSTANCE hInst)
{
	int i;
	char text[4096];

	optionshWnd = CreateDialog(hInst,MAKEINTRESOURCE(VidomiSlaveOptions),globals->hWnd, optionsDialogHandler);
	

	globals->translate->translate(text, "VidomiSlaveOptions");
	SetWindowText(optionshWnd, text);			

	for(i=0;;i++)
		{
		if(!slaveOptionsTID[i].id)
			break;
		globals->translate->translate(text, slaveOptionsTID[i].textId);
		SetWindowText(GetDlgItem(optionshWnd, slaveOptionsTID[i].id),text);			
		}

	logOutput = globals->optionsLogOutput;
	hideIcon = globals->hideSlaveIcon;
	obeyShutdown = globals->obeyShutdown;

	configureOptionsWindow(optionshWnd);

}

// *****************************************************************************
// encoderVideoClass::optionsDialogHandler()
//
//
// *****************************************************************************
BOOL CALLBACK slaveOptionsClass::optionsDialogHandler(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	return(globals->slaveOptions->optionsDialogHandler1(hWnd,message,wParam,lParam));
}

// *****************************************************************************
// slaveOptionsClass::windowDialogHandler()
//
//
// *****************************************************************************
BOOL slaveOptionsClass::optionsDialogHandler1(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	unsigned short commandId;
	unsigned short commandEvent;
	HKEY regKey;
	DWORD regReturn;

	switch(message)
		{

		case WM_COMMAND:
			commandId   = LOWORD(wParam); // Remember, these are...
			commandEvent = HIWORD(wParam); // ...different for Win32!

			//Parse the menu selections:

			switch (commandId) 
				{
				case VidomiOptionsOk :
					readWindowVars(hWnd);
					
					globals->optionsLogOutput = logOutput;
					globals->hideSlaveIcon = hideIcon;
					globals->obeyShutdown = obeyShutdown;

					if(globals->hideSlaveIcon)
						{
						globals->slaveWindow->deleteTaskBarIcon();

						// The Slave Icon Registry value is written right away.
						regReturn = globals->registry->openCreateReg(&regKey,REGISTRY_PROGRAM_NAME);
	
						if(regReturn)
							{
							globals->registry->setRegBool(regKey, "hideSlaveIcon" , globals->hideSlaveIcon);
							}

						globals->registry->closeReg(regKey);
						}
					
					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;


				case VidomiOptionsCancel :
					DestroyWindow(hWnd);
					globals->optionsUp=false;
					break;
				}
			break;
			

		case WM_CLOSE:
			DestroyWindow(hWnd);
			globals->optionsUp=false;
			break;

		}

	return (FALSE);

}



// *****************************************************************************
// slaveOptionsClass::configureOptionsWindow()
//
//
// *****************************************************************************
void slaveOptionsClass::configureOptionsWindow(HWND hWnd)
{

	if(logOutput)
		SendDlgItemMessage(hWnd,VidomiSlaveOptionsDebug,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiSlaveOptionsDebug,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(hideIcon)
		SendDlgItemMessage(hWnd,VidomiSlaveOptionsDisplayIcon,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiSlaveOptionsDisplayIcon,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);

	if(obeyShutdown)
		SendDlgItemMessage(hWnd,VidomiSlaveOptionsObeyShutdown,BM_SETCHECK , (WPARAM)BST_CHECKED, (LPARAM)  0);
	else
		SendDlgItemMessage(hWnd,VidomiSlaveOptionsObeyShutdown,BM_SETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0);
	
}

// *****************************************************************************
// slaveOptionsClass::readWindowVars()
//
//
// *****************************************************************************
void slaveOptionsClass::readWindowVars(HWND hWnd)
{

	if(SendDlgItemMessage(hWnd,VidomiSlaveOptionsDebug,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		logOutput=true;
	else
		logOutput=false;

	if(SendDlgItemMessage(hWnd,VidomiSlaveOptionsDisplayIcon,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		hideIcon=true;
	else
		hideIcon=false;

	if(SendDlgItemMessage(hWnd,VidomiSlaveOptionsObeyShutdown,BM_GETCHECK , (WPARAM)BST_UNCHECKED, (LPARAM)  0)==BST_CHECKED)
		obeyShutdown=true;
	else
		obeyShutdown=false;

}

