// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// subtitles.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

// *****************************************************************************
// subtitlesClass::subtitlesClass()
//
// subtitles class constructor
// *****************************************************************************
subtitlesClass::subtitlesClass()
{
}

// *****************************************************************************
// subtitlesClass::~subtitlesClass()
//
// subtitles class destructor
// *****************************************************************************
subtitlesClass::~subtitlesClass()
{
}

// *****************************************************************************
// subtitlesClass::init()
//
// *****************************************************************************
void subtitlesClass::init(float inFramerate,float outFramerate)
{
	int i;
	for(i=0;i<MAX_SUBTITLE_CHANNELS;i++)
		{
		subtitleTimer[i]=0;
		subtitleType[i] = divxSubtitleTypeGraphicFull;
		subtitlePreprocess[i] = true;
		}
	
	inputFramerate = inFramerate;
	outputFramerate = outFramerate; 
	rateChange =  outputFramerate / inputFramerate;
	
}

// *****************************************************************************
// subtitlesClass::setChannelMode()
//
// *****************************************************************************
void subtitlesClass::channelConfigure(int channelIndex,int subMode, bool half, char * filename)
{


	switch(subMode)
		{
		case SUBTITLE_MODE_GRAPHIC:
			if(half)
				subtitleType[channelIndex] = divxSubtitleTypeGraphicHalf;
			else
				subtitleType[channelIndex] = divxSubtitleTypeGraphicFull;
			break;
		case SUBTITLE_MODE_TEXT:
			subtitleType[channelIndex] = divxSubtitleTypeText;
			strcpy(subtitleFilename[channelIndex],filename);
			break;
		}

	subtitlePreprocess[channelIndex] = true;


}

// *****************************************************************************
// subtitlesClass::end()
//
// *****************************************************************************
void subtitlesClass::end()
{

}

// *****************************************************************************
// subtitlesClass::renderSubtitle()
//
// *****************************************************************************
bool subtitlesClass::processSubtitleData(int channelIndex,CONVERTCONFIG * settings,SUBTITLEDATA * sub)
{
	if(parseSubtitleControl(channelIndex,settings, sub))
		{
		if(subtitleTimer[channelIndex])
			return(true);
		else
			return(false);
		}
	else	
		{
		return(false);
		}
}

// *****************************************************************************
// subtitlesClass::renderSubtitle()
//
// *****************************************************************************
bool subtitlesClass::parseSubtitleControl(int channelIndex, CONVERTCONFIG * settings, SUBTITLEDATA * sub)
{
	unsigned char * ptr;
	unsigned short offset;
	unsigned char * controlStart;
	unsigned short controlEndOffset;
	unsigned char * dataStart;
	unsigned short dataSize;
	bool done;
	int i;
	int a;
	int r;
	int g;
	int b;

	SUBTITLEFRAMEDATA subtitleFrame;

	ptr = sub->bufferStart;

	ptr+=2;
	offset = (*ptr++)<<8;
	offset |= *ptr++;

	dataSize = (*ptr++)<<8;
	dataSize |= *ptr++;
	dataStart = ptr;

	ptr += (offset-4);

	controlEndOffset = (*ptr++)<<8;
	controlEndOffset |= *ptr++;

	controlStart = ptr;

	done = false;
	
	// Just in case this subtitle chunk is bad
	subtitleTimer[channelIndex] = 0;

	do{
		switch(*ptr++)
			{
			case 0x00:
				// Ignored
				break;

			case 0x01:
				// Ignored - Start Displaying?
				break;
			case 0x03:
				//Palette Information
				subtitlePalette[channelIndex][0] = *ptr>>4;
				subtitlePalette[channelIndex][1] = (*ptr++)&0x0f;
				subtitlePalette[channelIndex][2] = *ptr>>4;
				subtitlePalette[channelIndex][3] = (*ptr++)&0x0f;
				break;
			case 0x04:
				//Alpha Information
				subtitleAlpha[channelIndex][3] = *ptr>>4;
				subtitleAlpha[channelIndex][2] = (*ptr++)&0x0f;
				subtitleAlpha[channelIndex][1] = *ptr>>4;
				subtitleAlpha[channelIndex][0] = (*ptr++)&0x0f;
				break;
			case 0x05:
				// Subtitle rect
				subtitleLeftPixel[channelIndex] = (*ptr++)<<4;
				subtitleLeftPixel[channelIndex] |= ((*ptr)&0xf0)>>4;
				subtitleRightPixel[channelIndex] = ((*ptr++)&0x0f)<<8;
				subtitleRightPixel[channelIndex] |= *ptr++;

				subtitleTopPixel[channelIndex] = (*ptr++)<<4;
				subtitleTopPixel[channelIndex] |= ((*ptr)&0xf0)>>4;
				subtitleBottomPixel[channelIndex] = ((*ptr++)&0x0f)<<8;
				subtitleBottomPixel[channelIndex] |= *ptr++;
				break;
			case 0x06:
				// Buffer Start positions
				subtitleEvenLineStart[channelIndex] = (*ptr++)<<8;
				subtitleEvenLineStart[channelIndex] |= *ptr++;
				subtitleOddLineStart[channelIndex] = (*ptr++)<<8;
				subtitleOddLineStart[channelIndex] |= *ptr++;
				subtitleEvenLineStart[channelIndex]-=2;
				break;

			case 0xff:
				// process the end sequence
				subtitleTimer[channelIndex] = (*ptr++)<<8;
				subtitleTimer[channelIndex] |= *ptr++;
				subtitleTimer[channelIndex] = (int)((subtitleTimer[channelIndex]/2) * rateChange);
				done = true;
				break;
			default:
				// Unknown code. Error out.
				subtitleTimer[channelIndex] = 0;
				done = true;
				break;
			}
		}while(!done);

	sub->header.duration = subtitleTimer[channelIndex];
	sub->header.top = subtitleTopPixel[channelIndex];
	sub->header.bot = subtitleBottomPixel[channelIndex];
	sub->header.left = subtitleLeftPixel[channelIndex];
	sub->header.right = subtitleRightPixel[channelIndex];
	sub->header.evenStartPos = subtitleEvenLineStart[channelIndex];
	sub->header.oddStartPos = subtitleOddLineStart[channelIndex];
	

	if(subtitlePreprocess[channelIndex])
		{
		// Determine the color mapping
		memset(&subtitleFrame,0,sizeof(SUBTITLEFRAMEDATA));
		subtitleFrame.allocStart = new unsigned char [720*576*4];
		subtitleFrame.bufferSize = 720*576*4;
		subtitleFrame.bufferStart = subtitleFrame.allocStart;
		subtitleFrame.data = subtitleFrame.allocStart;

		memset(subtitleFrame.data,0,720*576*4);

		if(!preprocessSubtitleData(channelIndex, sub, &subtitleFrame))
			{
			delete [] subtitleFrame.allocStart;
			return(false);
			}
	
		delete [] subtitleFrame.allocStart;
		subtitlePreprocess[channelIndex]=false;
		}

	// This data is regenerated to support dual CPU's..

	sub->format.mode = subtitleType[channelIndex];
	for(i=0;i<4;i++)
		{
		a = (settings->subtitleColorData[i] &0xff000000)>>24;
		b = (settings->subtitleColorData[i] &0x00ff0000)>>16;
		g = (settings->subtitleColorData[i] &0x0000ff00)>>8;
		r = settings->subtitleColorData[i] &0x000000ff;

		//BG color is always transparent
		if(i==0)
			{
			a=0xff;
			}
		sub->format.palette[i] = (a<<24) | (r<<16) | (g<<8) | b;

		}

	sub->format.remap[0] = subtitleRemap[channelIndex][0];
	sub->format.remap[1] = subtitleRemap[channelIndex][1];			
	sub->format.remap[2] = subtitleRemap[channelIndex][2];
	sub->format.remap[3] = subtitleRemap[channelIndex][3];

	// This is done last so that the preprocess does it's work on the full subtitle res.

	if(subtitleType[channelIndex]==divxSubtitleTypeGraphicFull)
		{
		//sub->bytesUsed = offset-4;
		sub->header.oddStartPos = subtitleOddLineStart[channelIndex];
		}
	else
		{
		sub->bytesUsed = subtitleOddLineStart[channelIndex];
		sub->header.oddStartPos = NULL;
		}

	return(true);
}



// *****************************************************************************
// subtitlesClass::renderSubtitle()
//
// *****************************************************************************
bool subtitlesClass::preprocessSubtitleData(int channelIndex, SUBTITLEDATA * sub, SUBTITLEFRAMEDATA * subtitleFrame)
{
	// Render the subtitle to the subtitle frame buffer
	unsigned int * ptr; 
	int freq[4];
	int index[4];
	int x;
	int y;
	int val;
	int bgColor;
	int aaColor;
	int outlineColor;
	int fontColor;
	int tempVal;
	int i;

	colorTable[0] = 0x0;
	colorTable[1] = 0x1;
	colorTable[2] = 0x2;
	colorTable[3] = 0x3;

	renderSubtitle(sub, subtitleFrame);

	if(!subtitleFrame->height)
		{
		return(false);
		}

	ptr =  (unsigned int *) subtitleFrame->data;

	freq[0] = 0;
	freq[1] = 0;
	freq[2] = 0;
	freq[3] = 0;

	for(i=0;i<(subtitleFrame->height*subtitleFrame->width);i++)
		{
		freq[*ptr++&0x00000003]++;
		}
	index[0]=0;
	index[1]=1;
	index[2]=2;
	index[3]=3;

	for(y=0;y<3;y++)
		for(x=y+1;x<4;x++)
		{
		if(freq[y] > freq[x])
			{
			val = freq[x];
			freq[x] = freq[y];
			freq[y] = val;
			val = index[x];
			index[x] = index[y];
			index[y] = val;
			}
		}

	//Not a valid subtitle unless we get three color values 
	if(freq[0]==0 && freq[1]==0)
		{
		return(false);
		}

	bgColor = 0;		//index[3];
	aaColor = index[0];
	
	colorTable[0] = 0x0;
	colorTable[1] = 0x1;
	colorTable[2] = 0x2;
	colorTable[3] = 0x3;

	// Now we remove the AA color and rerender. The first color we find will should be the outline color

	ptr =  (unsigned int *) subtitleFrame->data;

	val=bgColor;

	for(i=0;i<(subtitleFrame->height * subtitleFrame->width);i++)
		{
		tempVal = *ptr++&0x00000003;
		if((tempVal != bgColor) && (tempVal !=aaColor))
			{
			val = tempVal;
			break;
			}
		}

	outlineColor = val;

	for(i=0;i<4;i++)
		{
		colorTable[i] = 0x00000000;
		}

	colorTable[bgColor] = 0xffffffff;
	colorTable[aaColor] = 0xffffffff;
	colorTable[outlineColor] = 0xffffffff;

	fontColor = outlineColor;

	for(i=0;i<4;i++)
		{
		if(!colorTable[i])
			{
			fontColor = i;
			break;
			}
		}

	subtitleRemap[channelIndex][0] = bgColor;
	subtitleRemap[channelIndex][1] = fontColor;
	subtitleRemap[channelIndex][2] = outlineColor;
	subtitleRemap[channelIndex][3] = aaColor;

	return(true);
}

// *****************************************************************************
// subtitlesClass::renderSubtitle()
//
// *****************************************************************************
bool subtitlesClass::renderSubtitle(SUBTITLEDATA * sub, SUBTITLEFRAMEDATA * subtitleFrame)
{
	unsigned char * ptr;
	int size;
	bool renderOdd;
	unsigned char * tempPtr;
	int tempSize;

	renderSubtitlePalette[0] = 0;
	renderSubtitlePalette[1] = 1;
	renderSubtitlePalette[2] = 2;
	renderSubtitlePalette[3] = 3;

	ptr = subtitleFrame->bufferStart;
	current = (unsigned int *) ptr;

	subtitleWidth = sub->header.right - sub->header.left;
	subtitleHeight =sub->header.bot - sub->header.top; 

	dataPtr[0] = sub->bufferStart;
	dataPtr[0] += sub->header.evenStartPos;
	dataNibble[0] = false;
	dataPtr[1] = sub->bufferStart;
	dataPtr[1] +=sub->header.oddStartPos;
	dataNibble[1] = false;

	size = (*dataPtr[0]++)<<8;
	size |= *dataPtr[0]++;

	subtitleField = 0;
	
	stride = subtitleWidth;

	outputPtr = current;

	dataSize[0] = dataPtr[1] - dataPtr[0];
	dataSize[1] = (dataPtr[0] + size) - dataPtr[1];
	renderOdd = true;

	lineCounter = 0;

	for(;;)
		{
		tempPtr = dataPtr[0];
		tempSize = dataSize[0];
		renderLineRgb32();
		if(renderOdd)
			{
			subtitleField ^= 0x01;
			renderLineRgb32();
			subtitleField ^= 0x01;
			}
		else
			{
			dataPtr[0] = tempPtr;
			dataSize[0] = tempSize;
			renderLineRgb32();
			}

		if(dataSize[0]<=0)
			break;
		}

	subtitleFrame->width = subtitleWidth;
	subtitleFrame->height = lineCounter;
	return(true);
}


// *****************************************************************************
// subtitlesClass::renderLineRgb32()
// 
// *****************************************************************************
void subtitlesClass::renderLineRgb32()
{
	int	token;
	int run;
	int color;
	int xPos;
	bool outputData;
	unsigned int outputColor;

	xPos = 0;

	if(lineCounter)
		{
		outputData = true;
		}
	else
		{
		outputData =false;
		}

	do{
		token = getBits();
		if(token<0x04)
			{
			token = (token<<4) | getBits();
			if(token<0x10)
				{
				token = (token<<4) | getBits();
				if(token<0x40)
					{
					token = (token<<4) | getBits();
					if(!(token&0xfff0))
						{
						color = renderSubtitlePalette[token&0x03];
						token = -1;
						}
					}
				}
			}
		if(token>=0)
			{
			run = token>>2;
			color = (int) renderSubtitlePalette[token&0x03];
			}
		else
			{
			// run length is the remainder of the current line.
			run = subtitleWidth - xPos;

			// Not sure if this is correct, but it prevents a crash
			if(run<0)
				run = 0;
			if(run>subtitleWidth)
				run = subtitleWidth;
			}

		// Render..
		xPos +=run;

		outputColor = colorTable[color];

		if(!(outputColor&0xff000000))
			{
			// Run is the output color run length
			while(run--)
				{
				*outputPtr++ = outputColor;  
				}
			outputData = true;
			}
		else
			{
			while(run--)
				{
				*outputPtr++ = 0xff000000;
				}
			}

		if(xPos > subtitleWidth)
			token = -1;

		}while(token>=0);

	if(outputData)
		{
		lineCounter++;
		current+=stride;
		}

	outputPtr = current;
	if(dataNibble[subtitleField])
		{
		getBits();
		}

}
// *****************************************************************************
// subtitlesClass::getBits()
// 
// *****************************************************************************
int subtitlesClass::getBits()
{
	if(!dataSize[subtitleField])
		return(0);

	if(!dataNibble[subtitleField])
		{
		dataNibble[subtitleField] = !dataNibble[subtitleField];
		return(*dataPtr[subtitleField]>>4);
		}
	else
		{
		dataSize[subtitleField]--;
		dataNibble[subtitleField] = !dataNibble[subtitleField];
		return((*dataPtr[subtitleField]++)&0x0f);
		}
}
