// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// writeavi.cpp
// 
//
// *****************************************************************************

#include "en.h"

extern GLOBALS * globals;

char * writeAviGuid = {"00020000-0000-0000-C000-000000000046"};

// *****************************************************************************
// writeAviClass::writeAviClass()
//
// conversion class constructor
// *****************************************************************************
writeAviClass::writeAviClass()
{

	int channel;
	int i;

	videoInputSet = false;

	savedFormatOut = (BITMAPINFOHEADER *) new char [VIDOMI_BITMAP_INFO_SIZE];

	for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
		{
		audioInputSet[channel] = false;
		audioFormatOut[channel] = NULL;
		audioChannelActive[channel] = false;
		audioSamplesSentIn[channel] = 0;
		audioWriteSamplesIn[channel] = false;
		}

	for(channel=0;channel<MAX_SUBTITLE_CHANNELS;channel++)
		{
		subtitleChannelActive[channel] = false;
		}

	for(i=0;i<MAX_CHAPTERS;i++)
		{
		chapterName[i] = new char [65];
		*chapterName[i]=0;
		}

	// Hard coded buffer size
	bufferSize = (48000 * 2 * 2) / 10;

	for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
		{
		audioIn[channel] = new AUDIODATA [1];
		audioOut[channel] = new AUDIODATA [1];
		audioIn[channel]->bufferStart = (unsigned char *) new unsigned char [bufferSize];
		audioIn[channel]->bufferSize = bufferSize;
		audioIn[channel]->data = audioIn[channel]->bufferStart;
		audioIn[channel]->bytesUsed = 0;

		audioOut[channel]->bufferStart = (unsigned char *) new unsigned char [bufferSize];
		audioOut[channel]->bufferSize = bufferSize;
		audioOut[channel]->data = audioOut[channel]->bufferStart;
		audioOut[channel]->bytesUsed = 0;

		}
	maxFileSize = 3980;
	maxFileSize*=1024*1024;			//1980
	hasAudioStream=false;
	hasVideoStream=false;
	hasSubtitleStream=false;
	regenStreamOverride=false;
	for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
		{
		syncAudioVideo[channel]=true;
		mp3Encode[channel] = new mp3EncodeClass();
		}

}

// *****************************************************************************
// writeAviClass::~writeAviClass()
//
// conversion class destructor
// *****************************************************************************
writeAviClass::~writeAviClass()
{
	int channel;
	int i;

	for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
		{
		if(audioFormatOut[channel])
			delete [] audioFormatOut[channel];

		if(audioIn[channel])
			{
			delete [] audioIn[channel]->bufferStart;
			delete [] audioIn[channel];
			}

		if(audioOut[channel])
			{	
			delete [] audioOut[channel]->bufferStart;
			delete [] audioOut[channel];
			}
		delete mp3Encode[channel];
		}

	for(i=0;i<MAX_CHAPTERS;i++)
		{
		delete [] chapterName[i];
		}

	delete [] savedFormatOut;

	delete divx;
}

// *****************************************************************************
// writeAviClass::init()
//
// *****************************************************************************
bool writeAviClass::init()
{
	int channel;

	for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
		{
		mp3Encode[channel]->init(channel);
		}
	// set lame use as denied.
	
	divx = new divxClass();

	return(true);

}
// *****************************************************************************
// writeAviClass::setVideoInputFormat()
//
// width = Width of bitmaps that wll be fed to writeavi
// height = Height of bitmaps that will be fed to writeavi
// bpp = Bits per Pixel. Must be divisible by 8 (IE 24 or 32)
// framerate = framerate
// film = is this a 29.97 source being forced to a 23.97 source ?
// *****************************************************************************
bool writeAviClass::setVideoInputFormat(int width,int height, float framerate, bool film)
{
	videoFramerate = framerate;
	videoBpp = 32;
	videoWidth = width;
	videoHeight = height;
	videoFilm = film;
	videoInputSet = true;

	return (true);

}

// *****************************************************************************
// writeAviClass::setVideoOutputFormat()
//
// handler = The 4CC handler of the output stream
// *****************************************************************************
bool writeAviClass::setVideoOutputFormat(DWORD handler)
{
	fccHandler = handler;

	return(true);
}
// *****************************************************************************
// writeAviClass::setOutputSize()
//
// returns TRUE if user selects compressor and presses OK. False on error
// *****************************************************************************
void writeAviClass::setOutputSize(unsigned long size)
{
	maxFileSize=size;
}

// *****************************************************************************
// writeAviClass::setUseLame()
//
// Set the Use of Lame on / off
// *****************************************************************************
void writeAviClass::setUseMp3(int channelIndex, bool onOff, int outputFrequency, 
							  int encodeMode, int channelType, int quality, int minBitrate, 
							  int maxBitrate, int vbrQuality)
{
	mp3[channelIndex] = onOff;
	mp3EncodeMode[channelIndex] = encodeMode;
	mp3ChannelType[channelIndex] = channelType;
	mp3Quality[channelIndex] = quality;
	mp3MinBitrate[channelIndex] = minBitrate;
	mp3MaxBitrate[channelIndex] = maxBitrate;
	mp3OutputFrequency[channelIndex] = outputFrequency;
	mp3VbrQuality[channelIndex] = vbrQuality;

	// VBR not allowed hack

	//mp3EncodeMode[channelIndex] = LAME_CBR; //encodeMode; //LAME_CBR;
	//mp3MaxBitrate[channelIndex] = minBitrate;	//Hack for 44.1Khz A/V sync problem

}

// *****************************************************************************
// writeAviClass::gettotalBytesWritten()
//
// *****************************************************************************
__int64 writeAviClass::getTotalBytesWritten()
{
	return(totalBytesWritten + totalFileBytesWritten);
}

// *****************************************************************************
// writeAviClass::resetCapturePtsFlag()
//
// *****************************************************************************
void writeAviClass::resetPtsFlags(int channelIndex, bool atSplit)
{
	if(atSplit)
		{
		syncAudioVideo[channelIndex] = true;
		firstSync[channelIndex] = true;
		captureFirstVideoPts = true;
		}
	else
		{
		captureFirstVideoPts = true;
		syncAudioVideo[channelIndex] = true;
		firstSync[channelIndex] = true;
		}

}

// *****************************************************************************
// writeAviClass::setAudioOutputFormat()
//
// *****************************************************************************
bool writeAviClass::setAudioOutputFormat(int channelIndex, int sampleRate, int channels, MPEGLAYER3WAVEFORMAT * audioDefault)
{

	audioOutBufferSize[channelIndex]=AUDIO_STATE_DEFAULT_SIZE;
	audioFormatOut[channelIndex]  = (WAVEFORMATEX *) new char [audioOutBufferSize[channelIndex]];

	if(!mp3[channelIndex])
		{
		memcpy(audioFormatOut[channelIndex],audioDefault,AUDIO_STATE_DEFAULT_SIZE);
		}
	return(true);
}

// *****************************************************************************
// writeAviClass::setAudioInputFormat()
//
// *****************************************************************************
bool writeAviClass::setAudioInputFormat(int channelIndex, int sampleRate, int channels)
{
	unsigned long mp3BufferSize;
	unsigned long wavBufferSize;
	int outFreq;
/*
	int i;
	mp3DecodeClass * mp3Decode;
	int mpegChannels;
	int mpegFrameSize;
	int mpegSamplerate;
	bool validLameSettings;
	int bytesEncoded;
*/
	memset(&audioFormatIn[channelIndex],0,sizeof(WAVEFORMATEX));	
	memset(&audioAcmHeader[channelIndex],0,sizeof (ACMSTREAMHEADER));

	//note that we assume 16 bits per sample.
	audioFormatIn[channelIndex].wFormatTag=WAVE_FORMAT_PCM;
	audioFormatIn[channelIndex].nChannels = channels;
	audioFormatIn[channelIndex].nSamplesPerSec=sampleRate;
	audioFormatIn[channelIndex].nAvgBytesPerSec= sampleRate*channels*2;
	audioFormatIn[channelIndex].nBlockAlign=channels*2;
	audioFormatIn[channelIndex].wBitsPerSample=16;
	audioFormatIn[channelIndex].cbSize=0;

	audioBytesPerSample = 2 * channels;

	audioAcmHeader[channelIndex].cbStruct = sizeof(ACMSTREAMHEADER);
	audioAcmHeader[channelIndex].fdwStatus = 0;
	audioAcmHeader[channelIndex].dwUser =0;
	audioAcmHeader[channelIndex].pbSrc = (unsigned char *) audioIn[channelIndex]->bufferStart;
	audioAcmHeader[channelIndex].cbSrcLength  = audioIn[channelIndex]->bufferSize;
	audioAcmHeader[channelIndex].cbSrcLengthUsed  = 0;
	audioAcmHeader[channelIndex].dwSrcUser = 0;
	audioAcmHeader[channelIndex].pbDst = (unsigned char *)audioOut[channelIndex]->bufferStart;
	audioAcmHeader[channelIndex].cbDstLength  = audioOut[channelIndex]->bufferSize;
	audioAcmHeader[channelIndex].cbDstLengthUsed  = 0;
	audioAcmHeader[channelIndex].dwDstUser = 0;

	if(mp3[channelIndex])
		{
		if(mp3Encode[channelIndex]->init(channelIndex))
			{


			mp3Encode[channelIndex]->config(sampleRate,mp3OutputFrequency[channelIndex], 
											channels, mp3EncodeMode[channelIndex],
											mp3ChannelType[channelIndex],mp3Quality[channelIndex],
											mp3MinBitrate[channelIndex] , mp3MaxBitrate[channelIndex],
											mp3VbrQuality[channelIndex],
											&mp3BufferSize, &wavBufferSize);


			if(audioIn[channelIndex]->bufferSize!=(int)wavBufferSize)
				{
				delete [] audioIn[channelIndex]->bufferStart;
				audioIn[channelIndex]->bufferStart = new unsigned char [wavBufferSize];
				audioIn[channelIndex]->bytesUsed =0;
				audioIn[channelIndex]->bufferSize = wavBufferSize;
				audioIn[channelIndex]->data = audioIn[channelIndex]->bufferStart;
				}

			if(audioOut[channelIndex]->bufferSize!=(int)mp3BufferSize)
				{
				delete [] audioOut[channelIndex]->bufferStart;
				audioOut[channelIndex]->bufferStart = new unsigned char [mp3BufferSize];
				audioOut[channelIndex]->bytesUsed =0;
				audioOut[channelIndex]->bufferSize = mp3BufferSize;
				audioOut[channelIndex]->data = audioOut[channelIndex]->bufferStart;
				}

			if(mp3OutputFrequency[channelIndex]==0)
				{
				outFreq = sampleRate;
				}
			else
				{
				outFreq = mp3OutputFrequency[channelIndex];
				}

			//Build up the Header structure for windows
			//mp3Out[channelIndex].wfx.wFormatTag = WAVE_FORMAT_MPEGLAYER3;
			//mp3Out[channelIndex].wfx.nChannels =channels;
			//mp3Out[channelIndex].wfx.nSamplesPerSec = outFreq;
			//mp3Out[channelIndex].wfx.nAvgBytesPerSec = (mp3MinBitrate[channelIndex]*1000)/8;
			//mp3Out[channelIndex].wfx.nBlockAlign = 1;
			//mp3Out[channelIndex].wfx.wBitsPerSample = 0;
			//mp3Out[channelIndex].wfx.cbSize = MPEGLAYER3_WFX_EXTRA_BYTES;
			//mp3Out[channelIndex].wID = MPEGLAYER3_ID_MPEG;
			//mp3Out[channelIndex].fdwFlags = MPEGLAYER3_FLAG_PADDING_OFF;
			//mp3Out[channelIndex].nBlockSize = (unsigned short) mp3BufferSize;							
			//mp3Out[channelIndex].nFramesPerBlock = 1;	//0x0100
			//mp3Out[channelIndex].nCodecDelay = 1393;					

			mp3Out[channelIndex].wfx.wFormatTag = WAVE_FORMAT_MPEGLAYER3;
			mp3Out[channelIndex].wfx.nChannels =channels;
			mp3Out[channelIndex].wfx.nSamplesPerSec = outFreq;
			mp3Out[channelIndex].wfx.nBlockAlign = 1;
			mp3Out[channelIndex].wfx.wBitsPerSample = 0;
			mp3Out[channelIndex].wfx.cbSize = MPEGLAYER3_WFX_EXTRA_BYTES;	//12
			mp3Out[channelIndex].wID = MPEGLAYER3_ID_MPEG;	//1
			mp3Out[channelIndex].fdwFlags = MPEGLAYER3_FLAG_PADDING_OFF; //2
			mp3Out[channelIndex].nBlockSize = (unsigned short) mp3BufferSize;
			mp3Out[channelIndex].nFramesPerBlock = 1;
			mp3Out[channelIndex].nCodecDelay = 1393;					

			mp3Out[channelIndex].wfx.nAvgBytesPerSec = (mp3MinBitrate[channelIndex]*1000)/8;

			logPrintf("Average bytes per Second = %d\n",mp3Out[channelIndex].wfx.nAvgBytesPerSec);

			memcpy(audioFormatOut[channelIndex],&mp3Out[channelIndex],sizeof(MPEGLAYER3WAVEFORMAT));


			}
		else
			{
			// Lame not found, so disable it's use
			mp3[channelIndex] = false; 
			}
		}
	else
		{
		if(audioFormatOut[channelIndex]->wFormatTag == WAVE_FORMAT_PCM)
			{
			audioFormatOut[channelIndex]->wFormatTag = audioFormatIn[channelIndex].wFormatTag;
			audioFormatOut[channelIndex]->nChannels = audioFormatIn[channelIndex].nChannels;
			audioFormatOut[channelIndex]->nSamplesPerSec = audioFormatIn[channelIndex].nSamplesPerSec;
			audioFormatOut[channelIndex]->nAvgBytesPerSec = audioFormatIn[channelIndex].nAvgBytesPerSec;
			audioFormatOut[channelIndex]->nBlockAlign = audioFormatIn[channelIndex].nBlockAlign;
			audioFormatOut[channelIndex]->wBitsPerSample = audioFormatIn[channelIndex].wBitsPerSample;
			audioFormatOut[channelIndex]->cbSize = audioFormatIn[channelIndex].cbSize;
			}
		}
	audioSampleRate[channelIndex]=sampleRate;
	audioChannels[channelIndex] = channels;
	audioInputSet[channelIndex] = true;

	return (true);

}

// *****************************************************************************
// writeAviClass::createAudioStream()
//
// Codec must have been selected, input format setup, and AVI file created
// *****************************************************************************
bool writeAviClass::createAudioStream(int channelIndex, char * streamName)
{
	int ret;
	MMRESULT retmm;

	totalAudioBytesWritten[channelIndex] = 0;

	if(!regenStreamOverride)
		{
		totalSamplesWritten[channelIndex] = 0;
		audioSamplesSentIn[channelIndex] = 0;
		firstSync[channelIndex] = true;
		memset(&streamAudioFormat[channelIndex],0,sizeof(AVISTREAMINFO));
		if(!mp3[channelIndex])
			{
			streamAudioFormat[channelIndex].fccType = streamtypeAUDIO;
			streamAudioFormat[channelIndex].fccHandler = audioFormatOut[channelIndex]->nBlockAlign;
			streamAudioFormat[channelIndex].dwScale = 1;//audioFormatOut->nAvgBytesPerSec;
			streamAudioFormat[channelIndex].dwInitialFrames = 0; 
			streamAudioFormat[channelIndex].dwRate = audioFormatOut[channelIndex]->nAvgBytesPerSec;	
			streamAudioFormat[channelIndex].dwSuggestedBufferSize = audioIn[channelIndex]->bufferSize;
			streamAudioFormat[channelIndex].dwQuality = -1;
			streamAudioFormat[channelIndex].dwSampleSize = audioFormatOut[channelIndex]->nBlockAlign;
			}
		else
			{
			streamAudioFormat[channelIndex].fccType = streamtypeAUDIO;
			streamAudioFormat[channelIndex].dwScale = 1;//audioFormatOut->nAvgBytesPerSec;
			streamAudioFormat[channelIndex].dwInitialFrames = 0; 
			streamAudioFormat[channelIndex].dwRate = audioFormatOut[channelIndex]->nAvgBytesPerSec;	
			//streamAudioFormat[channelIndex].dwSuggestedBufferSize = audioIn[channelIndex]->bufferSize;
			streamAudioFormat[channelIndex].dwQuality = -1;
			// This next line must not be 0 to keep compat. with virtualdub
			streamAudioFormat[channelIndex].dwSampleSize = audioFormatOut[channelIndex]->nBlockAlign;

			}
		if(streamName)
			{
			strcpy(streamAudioFormat[channelIndex].szName,streamName);
			}
		}
	
	ret = divx->createAudioStream(&streamAudioFormat[channelIndex],(MPEGLAYER3WAVEFORMAT *)audioFormatOut[channelIndex],sizeof(WAVEFORMATEX) + audioFormatOut[channelIndex]->cbSize);

	streamAudioInterface[channelIndex] = ret;

	if(!regenStreamOverride)
		{
		//Setup the acm encoder  

		if(!mp3[channelIndex])
			{
			if(audioFormatOut[channelIndex]->wFormatTag == WAVE_FORMAT_PCM)
				{
				audioAcmStream[channelIndex] = NULL;
				}
			else
				{	
				if(retmm = acmStreamOpen(&audioAcmStream[channelIndex],NULL,&audioFormatIn[channelIndex],audioFormatOut[channelIndex],NULL,NULL,0,ACM_STREAMOPENF_NONREALTIME))
					{
					switch(retmm)
						{
						case ACMERR_NOTPOSSIBLE : 
							logPrintf("acmStreamOpen Failed with ACMERR_NOTPOSSIBLE\n");
							break;
						case MMSYSERR_INVALFLAG :
							logPrintf("acmStreamOpen Failed with MMSYSERR_INVALIDFLAG\n");
							break;
						case MMSYSERR_INVALHANDLE :
							logPrintf("acmStreamOpen Failed with MMSYSERR_INVALIDHANDLE\n");
							break;
						case MMSYSERR_INVALPARAM :
							logPrintf("acmStreamOpen Failed with MMSYSERR_INVALPARAM\n");
							break;
						case MMSYSERR_NOMEM :
							logPrintf("acmStreamOpen Failed with MMSYSERR_NOMEM\n");
							break;
						default :
						logPrintf("acmStreamOpen Failed wih result %d\n",(int)retmm);
							
						}
					return(false);
					}
				if(retmm = acmStreamPrepareHeader(audioAcmStream[channelIndex],&audioAcmHeader[channelIndex],0))
					{
					logPrintf("acmStreamPrepareHeader Failed %d\n");

					return(false);
					}
				}
			}
		}

	capturedTotalSamplesWritten[channelIndex] = totalSamplesWritten[channelIndex];
	totalSamplesWritten[channelIndex] = 0;

	hasAudioStream = true;
	syncAudioVideo[channelIndex] = true;

	audioChannelActive[channelIndex] = true;

	return(true);

}


// *****************************************************************************
// writeAviClass::createFile()
//
// filename = Null terminated filename to create
// *****************************************************************************
bool writeAviClass::createFile(char * filename, char * tail, int reserveSize)
{
	char file[MAX_PATH];

	if(!regenStreamOverride)
		{
		fileNumber=1;
		strcpy(rootFilename,filename);
		strcpy(rootFilenameTail,tail);
		totalBytesWritten=0;
		headerSize = reserveSize;
		}


	// Set chapter count to none
	chapterCount = 0;

	strcpy(file,filename);
	strcat(file,rootFilenameTail);

	divx->init();
	if(divx->createFile(file,false,false,headerSize))
		{
		logPrintf("Failed To Create the Divx File %s\n",file);
		return(false);
		}

	totalFileBytesWritten=headerSize + 0x98; //overhead for any AVI
	
	return(true);

}

// *****************************************************************************
// writeAviClass::createVideoStream()
//
// Codec must have been selected, input format setup, and AVI file created
// *****************************************************************************
bool writeAviClass::createVideoStream(BITMAPINFOHEADER * bitmapFormatOut, char * streamName)
{
	int ret;

	totalVideoBytesWritten = 0;

	if(!regenStreamOverride)
		{
		totalFrameCount = 0;
		frameCounter = 0;

		// Save the bitmap format for later use
		memcpy(savedFormatOut,bitmapFormatOut,bitmapFormatOut->biSize);

		// Setup the stram video struct
		memset(&streamVideoFormat,0,sizeof(AVISTREAMINFO));
		streamVideoFormat.fccType = streamtypeVIDEO;
		streamVideoFormat.fccHandler = fccHandler;
		streamVideoFormat.dwScale = (unsigned long)(1000000.0f / videoFramerate);
		streamVideoFormat.dwRate = 1000000; //(unsigned long) (videoFramerate * 1000000.0f);
		streamVideoFormat.dwSuggestedBufferSize = videoWidth * videoHeight * 4 ;
		streamVideoFormat.dwQuality = -1;
		if(streamName)
			{
			strcpy(streamVideoFormat.szName,streamName);
			}
		}

	ret = divx->createVideoStream(&streamVideoFormat,savedFormatOut,savedFormatOut->biSize);

	streamVideoInterface = ret;

	capturedFrameCounter = frameCounter;
	frameCounter=0;
	hasVideoStream = true;
	captureFirstVideoPts = true;


	return(true);

}

// *****************************************************************************
// writeAviClass::createSubtitleStream()
//
// AVI file must have been created already
// *****************************************************************************
bool writeAviClass::createSubtitleStream(int channelIndex, char * streamName)
{
	int ret;

	if(!regenStreamOverride)
		{
		memset(&streamSubtitleFormat[channelIndex],0,sizeof(DIVXSTREAMINFO));
		
		streamSubtitleFormat[channelIndex].fccType = divxStreamtypeSUBTITLE;

		if(streamName)
			{
			strcpy(streamSubtitleFormat[channelIndex].szName,streamName);
			}
		}

	ret = divx->createSubtitleStream(&streamSubtitleFormat[channelIndex]);

	streamSubtitleInterface[channelIndex] = ret;
	subtitleChannelActive[channelIndex]=true;
	hasSubtitleStream=true;
	
	return(true);
}


// *****************************************************************************
// writeAviClass::writeVideoData()
//
// filename = Null terminated filename to create
// framerate = framerate of AVI multiplied by 1000
// *****************************************************************************
bool writeAviClass::writeVideoData(FRAMEDATA * frame, BITMAPINFOHEADER * bitmapFormatOut, DWORD flags, bool * splitAvi,bool performSplit, bool cutChange)
{
	long bytesWritten;

	bytesWritten=0;
	
	bytesWritten = divx->writeVideoStreamFrame(streamVideoInterface,flags,frame->data,bitmapFormatOut->biSizeImage);
	
	if(bytesWritten==-1)
		{
		logPrintf("writeVideoStreamFrame Failed to write any bytes\n");
		return(false);
		}

	if(captureFirstVideoPts)
		{
		firstVideoPts = frame->pts;
		captureFirstVideoPts = false;
		}

	if(cutChange)
		{
		firstVideoPts=frame->hardPts;    
		capturedFrameCounter = frameCounter;
		}

	totalFrameCount++;
	frameCounter++;

	totalFileBytesWritten+=bytesWritten+24; //24 is overhead for any write. 16 bytes of idx and 8 bytes of chunk id and size
	
	totalVideoBytesWritten+=bytesWritten;


	if(totalFileBytesWritten>maxFileSize)
		{
		*splitAvi = true;
		}

	if(!performSplit)
		{
		return(true);
		}

	if(totalFileBytesWritten>maxFileSize)
		{
		logPrintf("Video Write Splitting AVI output\n");
	
		if(!split())
			{
			return(false);
			}

		}

	return(true);
}

// *****************************************************************************
// writeAviClass::writeAudioData()
//
// audio = point to AUDIODATA structure
// *****************************************************************************
int writeAviClass::writeAudioData(int channelIndex, AUDIODATA * audio, FRAMEDATA * frame, bool * splitAvi, bool performSplit, bool cutChange)
{
	long bytesWritten;
	long samplesWritten;
	long bytesWrittenTemp;
	int bytesToCopy;
	int pos;
	int bytesEncoded;
	int tempPts;
	float sampMult;
	int audioFramesOutput;
	int	calculatedSamplesOut;
	int cutAudioVideoOffset;

	audioFramesOutput = 1;

	if(audio->type == audioBufferMpegRaw || audio->type == audioBufferAc3Raw)
		{
		// User wants the raw data inserted into the target file
		bytesWritten = audio->bytesUsed;
		firstSync[channelIndex] = false;
		syncAudioVideo[channelIndex] = false;
		// Do something with the data
		audio->bytesUsed = 0;
		audio->data = audio->bufferStart;
		}
	else
		{
		if(cutChange && !syncAudioVideo[channelIndex]) 
			{
			// Compute the number of missing/excess samples written out
			sampMult = ((float)(audioSampleRate[channelIndex]))/90000.0f;
			calculatedSamplesOut = (int) (((double)capturedFrameCounter / videoFramerate) * (double)audioSampleRate[channelIndex] );
			audioVideoOffset[channelIndex] =  calculatedSamplesOut - totalSamplesWritten[channelIndex];
			audioVideoOffset[channelIndex] = audioVideoOffset[channelIndex] * 2 * audioChannels[channelIndex];

			// Compute The cuts A/V offset
			tempPts = firstVideoPts - audio->pts;			
			cutAudioVideoOffset = (int) ((float) tempPts * sampMult);
			cutAudioVideoOffset = cutAudioVideoOffset * 2 * audioChannels[channelIndex];
			cutAudioVideoOffset -= audioVideoOffset[channelIndex];


			audioVideoOffset[channelIndex] = cutAudioVideoOffset;
			}
		else
			{
			// User wants the decoded data encoded
			if(syncAudioVideo[channelIndex])
				{
				if(firstSync[channelIndex])
					{
					// If it's the first sync, then we use the PTS stamps in the MPEG stream
					if(captureFirstVideoPts)
						{
						firstVideoPts = frame->pts;
						captureFirstVideoPts = false;
						}

					sampMult = ((float)(audioSampleRate[channelIndex]))/90000.0f;
					tempPts = firstVideoPts - audio->pts;
					
					audioVideoOffset[channelIndex] = (int) ((float) tempPts * sampMult);
					audioVideoOffset[channelIndex] = audioVideoOffset[channelIndex] * 2 * audioChannels[channelIndex];
					startingAudioVideoOffset[channelIndex] = audioVideoOffset[channelIndex];
					firstSync[channelIndex] = false;
					}
				else
					{
					// On all other splits we calculate what should be correct
					logPrintf("***Audio Sync routine - capturedFrameCounter = %d / capturedTotalSamplesWritten = %d\n",capturedFrameCounter, capturedTotalSamplesWritten[channelIndex]);
					calculatedSamplesOut = (int) (((double)capturedFrameCounter / videoFramerate) * (double)audioSampleRate[channelIndex] );
					audioVideoOffset[channelIndex] =  calculatedSamplesOut - capturedTotalSamplesWritten[channelIndex];
					audioVideoOffset[channelIndex] = audioVideoOffset[channelIndex] * 2 * audioChannels[channelIndex];			
					logPrintf("***Audio Sync routine - audioVideoOffset = %d\n",audioVideoOffset[channelIndex]);
					}
				syncAudioVideo[channelIndex] = false;
				}
			}

		bytesWritten=0;
		samplesWritten=0;
		pos=0;

		while(audio->bytesUsed)
			{
			// Special case handling for AC3/MPEG Delay value
			if(audioVideoOffset[channelIndex]!=0)
				{
				// This is the first time audio has been written to the AVI file
				if(audioVideoOffset[channelIndex]<0)
					{
					// Audio is ahead, insert silence
					if(-audioVideoOffset[channelIndex] > (audioIn[channelIndex]->bufferSize - audioIn[channelIndex]->bytesUsed))
						bytesToCopy = audioIn[channelIndex]->bufferSize - audioIn[channelIndex]->bytesUsed;
					else
						bytesToCopy = -audioVideoOffset[channelIndex];

					memset(audioIn[channelIndex]->data,0,bytesToCopy);
					audioVideoOffset[channelIndex]+=bytesToCopy;
					audioIn[channelIndex]->data += bytesToCopy;
					audioIn[channelIndex]->bytesUsed +=bytesToCopy;

					totalSamplesWritten[channelIndex] += bytesToCopy / (audioChannels[channelIndex] * 2);

					}
				else
					{
					// Video is ahead, skip data in the input buffer to keep sync
					if(audioVideoOffset[channelIndex] >audio->bytesUsed)
						bytesToCopy = audio->bytesUsed;
					else
						bytesToCopy = audioVideoOffset[channelIndex];
					audio->data -= bytesToCopy;
					audio->bytesUsed -= bytesToCopy;
					audioVideoOffset[channelIndex] -= bytesToCopy;
					pos+=bytesToCopy;
					}
				}
			else
				{
				

				// Copy BLAH bytes to a temp buffer
				if(audio->bytesUsed >(audioIn[channelIndex]->bufferSize - audioIn[channelIndex]->bytesUsed))
					bytesToCopy = audioIn[channelIndex]->bufferSize - audioIn[channelIndex]->bytesUsed;
				else
					bytesToCopy = audio->bytesUsed;

				memcpy(audioIn[channelIndex]->data, audio->bufferStart+pos, bytesToCopy);
				audio->bytesUsed -= bytesToCopy;
				audio->data -= bytesToCopy;
				audioIn[channelIndex]->data += bytesToCopy;
				audioIn[channelIndex]->bytesUsed +=bytesToCopy;

				totalSamplesWritten[channelIndex] += bytesToCopy / (audioChannels[channelIndex] * 2);

				pos+=bytesToCopy;
				}
			// Encode those bytes
			if(audioIn[channelIndex]->bytesUsed == audioIn[channelIndex]->bufferSize)
				{

				if(!mp3[channelIndex])
					{
					if(audioFormatOut[channelIndex]->wFormatTag == WAVE_FORMAT_PCM)
						{
						memcpy(audioOut[channelIndex]->bufferStart, audioIn[channelIndex]->bufferStart,audioIn[channelIndex]->bytesUsed);
						bytesEncoded = audioIn[channelIndex]->bytesUsed;
						audioSamplesSentIn[channelIndex]+=audioIn[channelIndex]->bytesUsed / audioBytesPerSample;
						}
					else
						{
						audioSamplesSentIn[channelIndex]+=audioIn[channelIndex]->bytesUsed / audioBytesPerSample;

						if(acmStreamConvert(audioAcmStream[channelIndex],&audioAcmHeader[channelIndex],ACM_STREAMCONVERTF_BLOCKALIGN))
							{
							logPrintf("acmStreamConvert (Audio) Failed\n");
							return(false);
							}
						bytesEncoded = audioAcmHeader[channelIndex].cbDstLengthUsed;
						}
					if(bytesEncoded)
						{
						if(audioWriteSamplesIn[channelIndex])
							{
							bytesWrittenTemp = divx->writeAudioStreamBytesEnhanced(streamAudioInterface[channelIndex],audioOut[channelIndex]->bufferStart,bytesEncoded,audioSamplesSentIn[channelIndex]);
							}
						else
							{
							bytesWrittenTemp = divx->writeAudioStreamBytes(streamAudioInterface[channelIndex],audioOut[channelIndex]->bufferStart,bytesEncoded);
							}

						if(!bytesWrittenTemp)
							{
							logPrintf("writeAudioStreamBytes failed to write any bytes\n");
							return(false);
							}
						audioFramesOutput++;
						totalAudioBytesWritten[channelIndex]+=bytesWrittenTemp;
						bytesWritten+=bytesWrittenTemp+24; //24 is overhead for any write. 16 bytes of idx and 8 bytes of chunk id and size
						audioSamplesSentIn[channelIndex]=0;
						}

					}
				else
					{
					audioSamplesSentIn[channelIndex]+=audioIn[channelIndex]->bytesUsed / audioBytesPerSample;

					bytesEncoded =  mp3Encode[channelIndex]->encode(audioIn[channelIndex]->bufferStart,audioOut[channelIndex]->data,audioIn[channelIndex]->bytesUsed>>1);

					if(bytesEncoded)
						{
						if(audioWriteSamplesIn[channelIndex])
							{
							bytesWrittenTemp = divx->writeAudioStreamBytesEnhanced(streamAudioInterface[channelIndex],audioOut[channelIndex]->data,bytesEncoded,audioSamplesSentIn[channelIndex]);
							}
						else
							{						
							bytesWrittenTemp = divx->writeAudioStreamBytes(streamAudioInterface[channelIndex],audioOut[channelIndex]->data,bytesEncoded);
							}

						if(!bytesWrittenTemp)
							{
							logPrintf("writeAudioStreamBytes failed to write any bytes\n");
							return(false);
							}
						audioFramesOutput++;
						totalAudioBytesWritten[channelIndex]+=bytesWrittenTemp;
						bytesWritten+=bytesWrittenTemp+24; //24 is overhead for any write. 16 bytes of idx and 8 bytes of chunk id and size
						audioSamplesSentIn[channelIndex]=0;
						}
					}
				audioIn[channelIndex]->bytesUsed = 0;
				audioIn[channelIndex]->data = audioIn[channelIndex]->bufferStart;
				}
			}
		}
	
	totalFileBytesWritten+=bytesWritten;

	// Set the split flag when we're over the media disk size
	if(totalFileBytesWritten>maxFileSize)
		{
		*splitAvi = true;
		}

	// We only split after all audio tracks have been written and we're told we can do the split
	if(!performSplit)
		{
		return(audioFramesOutput);
		}

	if(totalFileBytesWritten>maxFileSize)
		{
		logPrintf("Audio Write Splitting AVI output\n");
		
		if(!split())
			{
			return(false);
			}
		
		audioFramesOutput=1;

		}
	return(audioFramesOutput);
}

// *****************************************************************************
// writeAviClass::split()
//
// *****************************************************************************
bool writeAviClass::split()
{
	int channel;
	int bytesEncoded;
	long bytesWrittenTemp;
	unsigned long mp3BufferSize;
	unsigned long wavBufferSize;

	//Close off the MP3 frame
	if(hasAudioStream)
		{
		for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
			{
			if(mp3[channel] && audioChannelActive[channel])
				{
				bytesEncoded =  mp3Encode[channel]->encodeEnd(audioOut[channel]->data);
				
				audioOut[channel]->data+=bytesEncoded;
				bytesEncoded = audioOut[channel]->data - audioOut[channel]->bufferStart;

				if(bytesEncoded)
					{
					if(audioWriteSamplesIn[channel])
						{
						bytesWrittenTemp = divx->writeAudioStreamBytesEnhanced(streamAudioInterface[channel],audioOut[channel]->bufferStart,bytesEncoded,0);
						}
					else
						{
						bytesWrittenTemp = divx->writeAudioStreamBytes(streamAudioInterface[channel],audioOut[channel]->bufferStart,bytesEncoded);							
						}

					if(!bytesWrittenTemp)
						{
						logPrintf("writeAudioStreamBytes failed to write any bytes\n");
						return(false);
						}
					}

				// And now restart the mp3 stream
				mp3Encode[channel]->end();
				delete mp3Encode[channel];
				mp3Encode[channel] = new mp3EncodeClass();
				mp3Encode[channel]->init(channel);

				mp3Encode[channel]->config(audioSampleRate[channel],mp3OutputFrequency[channel],
											audioChannels[channel], mp3EncodeMode[channel], 
											mp3ChannelType[channel], mp3Quality[channel], 
											mp3MinBitrate[channel],mp3MaxBitrate[channel] , 
											mp3VbrQuality[channel],
											&mp3BufferSize, &wavBufferSize);
				}
			}
		}
	regenStreamOverride=true;

	closeVideoStream();
	if(hasAudioStream)
		{
		for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
			{
			if(audioChannelActive[channel])
				{
				closeAudioStream(channel);
				}
			}
		}

	if(hasSubtitleStream)
		{
		for(channel=0;channel<MAX_SUBTITLE_CHANNELS;channel++)
			{
			if(subtitleChannelActive[channel])
				{
				closeSubtitleStream(channel);
				}
			}
		}

	closeFile();
	
	Sleep(1000);

	totalBytesWritten +=totalFileBytesWritten;

	sprintf(tempFilename,"%s%d",rootFilename,fileNumber);
	fileNumber++;

	if(!createFile(tempFilename,rootFilenameTail,headerSize))
		{
		logPrintf("Failed to Create new AVI File %s\n",tempFilename);
		return(false);
		}
	if(hasVideoStream)
		{
		if(!createVideoStream(savedFormatOut, NULL))
			{
			logPrintf("Failed to Create new Video Stream\n");
			return(false);
			}
		}
	if(hasAudioStream)
		{
		for(channel=0;channel<MAX_AUDIO_CHANNELS;channel++)
			{
			if(audioChannelActive[channel])
				{
				if(!createAudioStream(channel,NULL))
					{
					logPrintf("Failed to Create new Audio Stream\n");
					return(false);
					}
				audioIn[channel]->bytesUsed = 0;
				audioIn[channel]->data = audioIn[channel]->bufferStart;
				}
			}
		}

	if(hasSubtitleStream)
		{
		for(channel=0;channel<MAX_SUBTITLE_CHANNELS;channel++)
			{
			if(subtitleChannelActive[channel])
				{
				if(!createSubtitleStream(channel,NULL))
					{
					logPrintf("Failed to Create new Subtitle Stream\n");
					return(false);
					}
				}
			}
		}


	regenStreamOverride=false;

	return(true);
}

// *****************************************************************************
// writeAviClass::writeSubtitleData()
//
// sub = point to SUBTITLEDATA structure
// *****************************************************************************
int writeAviClass::writeSubtitleData(int channelIndex, SUBTITLEDATA * sub)
{
	int bytesWritten;


	divx->setSubtitleStreamFormat(streamSubtitleInterface[channelIndex],&sub->format);

	bytesWritten = divx->writeSubtitleStreamData(streamSubtitleInterface[channelIndex],&sub->header, sub->bufferStart, sub->bytesUsed);

	if(!bytesWritten)
		{
		logPrintf("writeSubtitleStreamData Failed to write any bytes\n");
		return(false);
		}

	totalFileBytesWritten+=bytesWritten+24;

 // Write the subtitle data to disk
	return(bytesWritten);

}

// *****************************************************************************
// writeAviClass::insertChapterStart()
//
// 
// *****************************************************************************
void writeAviClass::insertChapterStart(char * chapName)
{
	chapterStartPos[chapterCount] = frameCounter;
	strcpy(chapterName[chapterCount],chapName);
	chapterCount++;

}

// *****************************************************************************
// writeAviClass::closeFile()
//
// *****************************************************************************
bool writeAviClass::closeFile()
{

	divx->closeFile();

	divx->end();

	return(true);
}

// *****************************************************************************
// writeAviClass::closeVideoStream()
//
// *****************************************************************************
bool writeAviClass::closeVideoStream()
{
	DIVXSTREAMCHAPTERS * chapterData;
	DIVXSTREAMCHAPTERENTRY * chapterEntry;
	int chapterDataSize;
	int i;

	if(chapterCount)
		{
		// First build the chapter structure
		chapterData = (DIVXSTREAMCHAPTERS *) new char [MAX_CHAPTERS*128];
		
		memset(chapterData,0,MAX_CHAPTERS*128);

		chapterDataSize = sizeof(DIVXSTREAMCHAPTERS); 
		chapterDataSize -= sizeof(DIVXSTREAMCHAPTERENTRY);

		chapterEntry = chapterData->chapterEntry;
		
		chapterData->mode = 0;
		chapterData->numChapters = chapterCount;

		for(i=0;i<chapterCount;i++)
			{
			chapterEntry->chapterStartPos = chapterStartPos[i];	
			strcpy(chapterEntry->chapterName,chapterName[i]);
			chapterEntry->entrySize = strlen(chapterName[i]);
			chapterEntry->entrySize += sizeof(DIVXSTREAMCHAPTERENTRY);
			
			// Make entry size an even number
			chapterEntry->entrySize = (chapterEntry->entrySize+1)&0xfffffffe;
			chapterDataSize += chapterEntry->entrySize;
			chapterEntry = (DIVXSTREAMCHAPTERENTRY *)(((char *)chapterEntry) + chapterEntry->entrySize);
			}
		// If we have chapter data, then tell the divx write code about them
		divx->setVideoStreamChapters(streamVideoInterface,chapterData,chapterDataSize);
		delete [] chapterData;
		}

	if(!regenStreamOverride)
		{
		videoInputSet = false;
		}

	logPrintf("Close of Video Stream\n");
	logPrintf("Number of Frames Written %d\n",frameCounter);

	return(true);
}

// *****************************************************************************
// writeAviClass::closeAudioStream()
//
// *****************************************************************************
bool writeAviClass::closeAudioStream(int channelIndex)
{
	int numFrames;
	int bytesPerSecond; 

	if(mp3[channelIndex])
		{
		// if LAME VBR audio, then fixup the header data to conform to nandub style VBR audio
		if((mp3EncodeMode[channelIndex]==LAME_VBR) || (mp3EncodeMode[channelIndex]==LAME_ABR))
			{
			numFrames = totalAudioBytesWritten[channelIndex] / 1152;
			bytesPerSecond = (int)((double) totalAudioBytesWritten[channelIndex] / ((double)frameCounter / videoFramerate));
			streamAudioFormat[channelIndex].dwLength = numFrames;
			streamAudioFormat[channelIndex].dwScale = 1152;
			streamAudioFormat[channelIndex].dwRate = mp3Out[channelIndex].wfx.nSamplesPerSec;
			streamAudioFormat[channelIndex].dwSampleSize = 0;

			mp3Out[channelIndex].wfx.nBlockAlign = 1152;
			mp3Out[channelIndex].nBlockSize = totalAudioBytesWritten[channelIndex] / numFrames;
			mp3Out[channelIndex].nCodecDelay = 0;
			mp3Out[channelIndex].wfx.nAvgBytesPerSec = bytesPerSecond; 
			divx->audioStreamFixup(streamAudioInterface[channelIndex],&streamAudioFormat[channelIndex],&mp3Out[channelIndex],sizeof(MPEGLAYER3WAVEFORMAT));
			}
		}

	if(!regenStreamOverride)
		{
		if(!mp3[channelIndex])
			{
			if(audioAcmStream[channelIndex])
				{
				acmStreamUnprepareHeader(audioAcmStream[channelIndex],&audioAcmHeader[channelIndex],0);
				acmStreamClose(audioAcmStream[channelIndex],0);
				}
			}
		else
			{
			mp3Encode[channelIndex]->end();
			}
		audioInputSet[channelIndex]=false;
		audioChannelActive[channelIndex]=false;
		}
	
	logPrintf("Close of Audio Channel %d\n",channelIndex);
	logPrintf("Total Audio Bytes Written to channel %d - %d\n",channelIndex, totalAudioBytesWritten[channelIndex]);
	logPrintf("Total Audio Samples Written to channel %d - %d\n",channelIndex, totalSamplesWritten[channelIndex]);
	logPrintf("Number of Frames Written %d\n",frameCounter);
	logPrintf("Corrected Average bytes per Second = %d\n",(int)((double)totalAudioBytesWritten[channelIndex] / videoFramerate));


	return(true);

}

// *****************************************************************************
// writeAviClass::closeSubtitleStream()
//
// *****************************************************************************
bool writeAviClass::closeSubtitleStream(int channelIndex)
{
	logPrintf("Close of Subtitle Channel %d\n",channelIndex);	
	if(!regenStreamOverride)
		{
		subtitleChannelActive[channelIndex]=false;
		}

	return(true);

}