// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
// writeavi class
// *****************************************************************************
#ifndef writeavi_h
#define writeavi_h

extern bool lameInit();
extern void lameEnd();
extern bool lameConfig(int inputFrequency, int bitrate, unsigned long * sizeOfMp3Buffer, unsigned long * sizeOfWavBuffer);
extern int lameEncode(unsigned char * wavBuffer, unsigned char * mp3Buffer, int bytesToEncode);
extern int lameEncodeEnd(unsigned char * wavBuffer, unsigned char * mp3Buffer, int bytesToEncode);

class writeAviClass 
{

	public:
									writeAviClass();
									~writeAviClass();
			bool					setVideoInputFormat(int width,int height, float framerate, bool film);
			bool					setFilm(bool onOff);
			bool					setAudioInputFormat(int channelIndex, int sampleRate, int channels);
			bool					createFile(char * filename, char * tail, int headerReserve);
			bool					createVideoStream(BITMAPINFOHEADER * bitmapFormatOut, char * streamName);
			bool					writeVideoData(FRAMEDATA * frame, BITMAPINFOHEADER * bitmapFormatOut, DWORD flags, bool * splitAvi,bool performSplit, bool cutChange);
			bool					closeFile();
			bool					closeVideoStream();
			bool					createAudioStream(int channelIndex, char * streamName);
			int						writeAudioData(int channelIndex, AUDIODATA * audio, FRAMEDATA * frame, bool * splitAvi, bool performSplit, bool cutChange);
			bool					closeAudioStream(int channelIndex);
			void					setOutputSize(unsigned long size);
			void					setUseMp3(int channelIndex, bool onOff, int outputFrequency, int encodeMode, int channelType, int quality, int minBitrate, int maxBitrate, int vbrQuality);
			bool					setVideoOutputFormat(DWORD handler);
			bool					setAudioOutputFormat(int channelIndex,int sampleRate, int channels, MPEGLAYER3WAVEFORMAT * audioDefault);
			bool					init();
			__int64 				getTotalBytesWritten();
			void					resetPtsFlags(int channelIndex, bool atSplit);
			int						writeSubtitleData(int channelIndex, SUBTITLEDATA * sub);
			bool					createSubtitleStream(int channelIndex, char * streamName);
			bool					closeSubtitleStream(int channelIndex);
			void					insertChapterStart(char * chapName);
			bool					split();



	private:
			int						frameCounter;
			int						totalFrameCount;
			float					videoFramerate;
			bool					videoFilm;
			int						videoWidth;
			int						videoHeight;
			int						videoBpp;
			bool					videoInputSet;
			bool					audioInputSet[MAX_AUDIO_CHANNELS];
			int						audioOutBufferSize[MAX_AUDIO_CHANNELS];
			int						audioSampleRate[MAX_AUDIO_CHANNELS];
			int						audioChannels[MAX_AUDIO_CHANNELS];
			AUDIODATA *				audioOut[MAX_AUDIO_CHANNELS];
			AUDIODATA *				audioIn[MAX_AUDIO_CHANNELS];
			int						bufferSize;
			unsigned long			totalFileBytesWritten;
			unsigned long			maxFileSize;
			char					rootFilename[MAX_PATH];
			char					tempFilename[MAX_PATH];
			char					rootFilenameTail[6];
			bool					hasAudioStream;
			bool					hasVideoStream;
			bool					hasSubtitleStream;
			bool					regenStreamOverride;
			int						fileNumber;
			bool					mp3[MAX_AUDIO_CHANNELS];
			int						mp3MinBitrate[MAX_AUDIO_CHANNELS];
			int						mp3MaxBitrate[MAX_AUDIO_CHANNELS];
			int						mp3EncodeMode[MAX_AUDIO_CHANNELS];
			int						mp3ChannelType[MAX_AUDIO_CHANNELS];
			int						mp3Quality[MAX_AUDIO_CHANNELS];
			int						mp3OutputFrequency[MAX_AUDIO_CHANNELS];
			int						mp3VbrQuality[MAX_AUDIO_CHANNELS];
			bool					syncAudioVideo[MAX_AUDIO_CHANNELS];
			mp3EncodeClass *		mp3Encode[MAX_AUDIO_CHANNELS];
			int						audioVideoOffset[MAX_AUDIO_CHANNELS];
			int						startingAudioVideoOffset[MAX_AUDIO_CHANNELS];
			bool					captureFirstVideoPts;
			int						firstVideoPts;
			int						totalAudioBytesWritten[MAX_AUDIO_CHANNELS];

			int						totalVideoBytesWritten;
			__int64					totalBytesWritten;

			MPEGLAYER3WAVEFORMAT	mp3Out[MAX_AUDIO_CHANNELS];
			WAVEFORMATEX			audioFormatIn[MAX_AUDIO_CHANNELS];
			WAVEFORMATEX *			audioFormatOut[MAX_AUDIO_CHANNELS];
			DIVXSTREAMINFO			streamAudioFormat[MAX_AUDIO_CHANNELS];
			int						streamAudioInterface[MAX_AUDIO_CHANNELS];
			HACMSTREAM				audioAcmStream[MAX_AUDIO_CHANNELS];
			ACMSTREAMHEADER			audioAcmHeader[MAX_AUDIO_CHANNELS];

			int						streamVideoInterface;
			DIVXSTREAMINFO			streamVideoFormat;
			DWORD					fccHandler;
			BITMAPINFOHEADER *		savedFormatOut;
			int						totalSamplesWritten[MAX_AUDIO_CHANNELS];
			int						capturedTotalSamplesWritten[MAX_AUDIO_CHANNELS];
			bool					firstSync[MAX_AUDIO_CHANNELS];
			int						capturedFrameCounter;
			bool					audioChannelActive[MAX_AUDIO_CHANNELS];
			bool					subtitleChannelActive[MAX_SUBTITLE_CHANNELS];
			divxClass *				divx;
			DIVXSTREAMINFO			streamSubtitleFormat[MAX_SUBTITLE_CHANNELS];
			int						streamSubtitleInterface[MAX_SUBTITLE_CHANNELS];
			int						chapterCount;
			int						chapterStartPos[MAX_CHAPTERS];
			char *					chapterName[MAX_CHAPTERS];
			int						headerSize;
			int						audioBytesPerSample;

			int						audioSamplesSentIn[MAX_AUDIO_CHANNELS];
			bool					audioWriteSamplesIn[MAX_AUDIO_CHANNELS];

};


#endif