// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
// Encoder Server
//
// *****************************************************************************
#include "es.h"

GLOBALS * globals;

int PASCAL WinMain(HINSTANCE hInst,HINSTANCE hPrevInst, LPSTR lpszCmd ,int iCmdShow)
{
	MSG msg;                  
	char pathToSlaveIpFile[MAX_PATH];

	configureGlobals(hInst);
	
	clearLog();

	globals->window = new windowClass();
	globals->listen = new listenClass();
	globals->visualLog = new visualLogClass();
	globals->netHelper = new netHelperClass();
	globals->serverLoop = new serverLoopClass();
	
	globals->window->createWindow(hInst);

	globals->visualLog->logPrintf("Vidomi Encode Server %.03f Started\n",VERSION);

	globals->visualLog->logPrintf("Loading \"encodeslaves.txt\" File\n");
	strcpy(pathToSlaveIpFile,globals->pathToExeDir);
	strcat(pathToSlaveIpFile,"\\encodeSlaves.txt");
	globals->netHelper->loadSlaveList(pathToSlaveIpFile);
	if(globals->numSlaveEncodeMachines==1 && (!strcmp(globals->slaveEncodeMachines->slave[0].machineIp,"0.0.0.0")))
		{
		globals->visualLog->logPrintf("Failed To Load \"encodeslaves.txt\"\n");
		globals->visualLog->logPrintf("Aborting Starup\n");
		while(GetMessage(&msg, NULL, 0, 0))
			{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			}	
		}
	else
		{
		globals->visualLog->logPrintf("Found %d Slaves in File\n",globals->numSlaveEncodeMachines);
		globals->visualLog->logPrintf("Starting to Listen For Encode Jobs\n");

		globals->slaveEncodeStats = new CONVERSIONSTATS [globals->numSlaveEncodeMachines];

		globals->serverLoop->begin();
		globals->listen->begin();

		while(GetMessage(&msg, NULL, 0, 0))
			{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			}	

		globals->listen->end();
		globals->serverLoop->end();

		delete [] globals->slaveEncodeStats;
		}

	delete globals->serverLoop;
	delete globals->listen;
	delete globals->window;
	delete globals->visualLog;
	delete globals->netHelper;

	free(globals->slaveJobList);
	delete [] globals;

	return(FALSE);
}

// *****************************************************************************
//  Configure the Globals and put/get from registry
// *****************************************************************************
void configureGlobals(HINSTANCE hInst)
{
	char * commandLine;
	char tempStr[MAX_PATH];
	char * tempPtr;
	char pathToExe[MAX_PATH];
	bool done;
	int i;

	globals = new GLOBALS [1];
	memset(globals,0,sizeof(GLOBALS));

	globals->hInst = hInst;

	// Now lets have some fun and create the path to the EXE
	commandLine = GetCommandLine();
	strcpy(tempStr,commandLine);
	done=false;

	//walk through the path until we locate .exe
	tempPtr = tempStr;
	while(1)
		{
		if(!(tempPtr = strchr(tempPtr,'.')))
			{
			// if can't find .exe, then use CWD
			GetCurrentDirectory(MAX_PATH, pathToExe);
			done=true;
			break;
			}
		if((*(tempPtr+1) == 'e') && (*(tempPtr+2)=='x') && (*(tempPtr+3) == 'e'))
			{	
			*(tempPtr+4) = 0;
			break;
			}
		if((*(tempPtr+1) == 'E') && (*(tempPtr+2)=='X') && (*(tempPtr+3) == 'E'))
			{	
			*(tempPtr+3) = 0;
			break;
			}
		//Advance to the next Char
		if(tempPtr)
			{
			tempPtr++;
			}
		}

	if(!done)
		{
		// Remove leading quote if any
		if(tempStr[0] == '\"')
			{
			strcpy(pathToExe,&tempStr[1]);
			}
		else
			{
			strcpy(pathToExe,tempStr);
			}
		
		//Now wack off everything after the last "\"
		if(tempPtr = strrchr(pathToExe,'\\'))
			{
			*tempPtr = 0;
			}
		else
			{
			GetCurrentDirectory(MAX_PATH, pathToExe);
			}
		}

	//pathToExe now holds the path to the EXE dir
	strcpy(globals->pathToExeDir,pathToExe);
	// Set Default Values debug vars
	
	//GetCurrentDirectory(MAX_PATH, globals->pathToExeDir);
	strcpy(globals->optionsLogFilename,globals->pathToExeDir);
	strcat(globals->optionsLogFilename, "\\encodeserverdebug.txt");

	globals->optionsLogOutput = FALSE;
	globals->slaveJobListSize = 256;
	globals->slaveJobList = (SLAVEJOBLIST *) malloc(globals->slaveJobListSize * sizeof(SLAVEJOBLIST));
	memset(globals->slaveJobList,0,globals->slaveJobListSize * sizeof(SLAVEJOBLIST));
	
	for(i=0;i<globals->slaveJobListSize;i++)
		{
		// make the job list appear to be empty
		globals->slaveJobList[i].job->encodeDone = true;
		globals->slaveJobList[i].job->slaveAllocated = -1;
		}

	InitializeCriticalSection(&globals->slaveJobListLock);

	globals->slaveEncodeStats = NULL;
	InitializeCriticalSection(&globals->slaveEncodeStatsLock);

	globals->slaveJobCounter = 0;


}