// *****************************************************************************
//	Vidomi - Video Encoder
//
//	Copyright (C) 2001 Vidomi, Inc.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

// *****************************************************************************
// *****************************************************************************
//
//
// netMaster.cpp
// 
//
// *****************************************************************************

#include "es.h"

extern GLOBALS * globals;

// *****************************************************************************
// netMasterClass::netMasterClass()
//
// netMaster class constructor
// *****************************************************************************
netMasterClass::netMasterClass()
{

	InitializeCriticalSection(&conversionStatusLock);
	fakeSlaveThreadTerminated =true;
}

// *****************************************************************************
// netMasterClass::~netMasterClass()
//
// netMaster class destructor
// *****************************************************************************
netMasterClass::~netMasterClass()
{


}

// *****************************************************************************
// netMasterClass::begin()
//
// *****************************************************************************
void netMasterClass::begin(int myNumEncodeSlaves , ENCODESLAVES * myEncodeSlaves, bool myUseBuiltinSlave)
{
	int i;
	numEncodeSlaves = myNumEncodeSlaves;
	encodeSlaves = myEncodeSlaves;
	conversionStatus = netConversionStatusIdle;

	useBuiltinSlave = myUseBuiltinSlave;
	fakeSlaveThreadTerminated =true;

	if(!useBuiltinSlave)
		{	
		tcp = new tcpClass();
		for(i=0;i<numEncodeSlaves;i++)
			{
			encodeSlaves->slave[i].connectionStatus = NET_NOT_CONNECTED;
			}
		}
#ifdef USEBUILTINSLAVE
	else
		{
		encodeSlaves->slave[0].connectionStatus = NET_CONNECTED;	
		}
#endif
}

// *****************************************************************************
// netMasterClass::end()
//
// *****************************************************************************
void netMasterClass::end()
{
	int i;

	if(!useBuiltinSlave)
		{	
		// Close all the slave connections
		for(i=0;i<numEncodeSlaves;i++)
			{
			if(encodeSlaves->slave[i].commSocket>=0)
				{
				if(encodeSlaves->slave[i].connectionStatus == NET_CONNECTED)
					{
					sendClose(i);
					}
				tcp->closeSocket(encodeSlaves->slave[i].commSocket);
				}
			}
		delete tcp;
		}
#ifdef USEBUILTINSLAVE
	else
		{
		for(i=0;i<100;i++)
			{
			if(fakeSlaveThreadTerminated)
				break;
			Sleep(100);
			}
		}
#endif
}

// *****************************************************************************
// netMasterClass::checkConnections()
//
// *****************************************************************************
void netMasterClass::checkConnections(int numChances)
{
	int i;
	int chances;

	if(!useBuiltinSlave)
		{	
		// Find the first available Free Resource
		for(chances=0;chances<numChances;chances++)
			{
			for(i=0;i<numEncodeSlaves;i++)
				{
				if(encodeSlaves->slave[i].connectionStatus != NET_CONNECTED)
					{
					if(testConnection(i))
						{
						getStatus(i);
						break;
						}
					}
				}
			if(numChances>1)
				{
				// Sleep 250ms of a second between retrys
				Sleep(250);
				}
			}
		}
}
// *****************************************************************************
// netMasterClass::findFreeEncodeResource()
//
// *****************************************************************************
int netMasterClass::findFreeEncodeResource()
{
	int i;
	int status;

	// Find the first available Free Resource
	for(i=0;i<numEncodeSlaves;i++)
		{
		if(encodeSlaves->slave[i].connectionStatus == NET_CONNECTED)
			{
			EnterCriticalSection(&conversionStatusLock);
			status = encodeSlaves->slave[i].encodeStatus;
			LeaveCriticalSection(&conversionStatusLock);
			if(status != netConversionStatusWorking)
				{
				return(i);
				}
			}
		}

	return(-1);
}

// *****************************************************************************
// netMasterClass::getEncodeStatus()
//
// *****************************************************************************
int netMasterClass::getEncodeStatus(int slaveNumber)
{

	return(encodeSlaves->slave[slaveNumber].encodeStatus);
	
}

// *****************************************************************************
// netMasterClass::getConnectionStatus()
//
// *****************************************************************************
int netMasterClass::getConnectionStatus(int slaveNumber)
{

	return(encodeSlaves->slave[slaveNumber].connectionStatus);
	
}
// *****************************************************************************
// netMasterClass::getEncodeStats()
//
// *****************************************************************************
CONVERSIONSTATS * netMasterClass::getEncodeStats(int slaveNumber)
{

	return(&encodeSlaves->slave[slaveNumber].conversionStats);
	
}

// *****************************************************************************
// netMasterClass::testConnection()
//
// *****************************************************************************
bool netMasterClass::testConnection(int slaveNumber)
{
	int status;
	fd_set writeFds;
	fd_set readFds;
	fd_set errorFds;
	timeval statusTimer;
	DWORD curTime;

#ifdef USEBUILTINSLAVE
	// Early out if we're using a fake slave
	if(useBuiltinSlave)
		{	
		encodeSlaves->slave[slaveNumber].connectionStatus = NET_CONNECTED;
		return(true);
		}
#endif

	if(slaveNumber<numEncodeSlaves)
		{
		switch(encodeSlaves->slave[slaveNumber].connectionStatus)
			{
			case NET_NOT_CONNECTED:
				logPrintf("Master is attempting to connect to Slave %d\n",slaveNumber);

				encodeSlaves->slave[slaveNumber].commSocket = tcp->createSocket();

				if(encodeSlaves->slave[slaveNumber].commSocket<0)
					{
					encodeSlaves->slave[slaveNumber].connectionStatus = NET_FAILED;
					encodeSlaves->slave[slaveNumber].connectionErrorTime = timeGetTime(); 
					return(false);
					}
				encodeSlaves->slave[slaveNumber].connectionStartTime = timeGetTime();

				status = tcp->connectAddress(encodeSlaves->slave[slaveNumber].commSocket,
											 encodeSlaves->slave[slaveNumber].port,
											 encodeSlaves->slave[slaveNumber].networkName);

				if(!status)
					{
					encodeSlaves->slave[slaveNumber].connectionStatus = NET_CONNECTED;
					return(false);
					}

				encodeSlaves->slave[slaveNumber].connectionStatus = NET_CONNECTING;
				return(false);

			case NET_CONNECTING:
				logPrintf("Master is waiting for slave %d to connect\n",slaveNumber);
				memset(&statusTimer,0,sizeof(timeval));
				FD_ZERO(&readFds);
				FD_ZERO(&writeFds);
				FD_ZERO(&errorFds);
		        FD_SET(encodeSlaves->slave[slaveNumber].commSocket,&writeFds);
		        FD_SET(encodeSlaves->slave[slaveNumber].commSocket,&readFds);
		        FD_SET(encodeSlaves->slave[slaveNumber].commSocket,&errorFds);

				if(select(0,&readFds,&writeFds,&errorFds,&statusTimer)<0)
					{
					logPrintf("Connection to Slave %d Failed.\n",slaveNumber);					
					encodeSlaves->slave[slaveNumber].connectionStatus = NET_FAILED;
					encodeSlaves->slave[slaveNumber].connectionErrorTime = timeGetTime(); 
					return(false);
					}
				if(FD_ISSET(encodeSlaves->slave[slaveNumber].commSocket,&writeFds))
					{
					logPrintf("Master connected to Slave %d\n",slaveNumber);
					encodeSlaves->slave[slaveNumber].connectionStatus = NET_CONNECTED;
					return(true);
					}
				curTime = timeGetTime();
	
				if((curTime - encodeSlaves->slave[slaveNumber].connectionStartTime)>10000)
					{
					logPrintf("Connection to Slave %d never happened.\n",slaveNumber);					
					encodeSlaves->slave[slaveNumber].connectionStatus = NET_DOWN;
					encodeSlaves->slave[slaveNumber].connectionErrorTime = timeGetTime(); 
					}
				return(false);

			case NET_CONNECTED:
				// Had checks for connection still being valid, but those didn't work..
				return(true);

			case NET_FAILED:
				logPrintf("Slave %d is in a Failed State\n", slaveNumber);
				tcp->closeSocket(encodeSlaves->slave[slaveNumber].commSocket);
				encodeSlaves->slave[slaveNumber].connectionStatus =NET_SHORTDELAYRETRY;	
				// Whait a short while and retry
				return(false);

			case NET_DOWN:
				logPrintf("Slave %d is in a network down State\n", slaveNumber);
				tcp->closeSocket(encodeSlaves->slave[slaveNumber].commSocket);
				encodeSlaves->slave[slaveNumber].connectionStatus =NET_LONGDELAYRETRY;	
				return(false);

			case NET_SHORTDELAYRETRY:
				curTime = timeGetTime();
				if((curTime - encodeSlaves->slave[slaveNumber].connectionErrorTime)>10000)
					{
					encodeSlaves->slave[slaveNumber].connectionStatus =NET_NOT_CONNECTED;
					}						
				return(false);

			case NET_LONGDELAYRETRY:
				/*
				if(encodeSlaves->slave[slaveNumber].attemptedLongRetrys==3)
					{
					// No more retrys for you!
					encodeSlaves->slave[slaveNumber].connectionStatus =NET_PERMANENTLYDOWN;
					return(false);
					}
				*/
				curTime = timeGetTime();
				if((curTime - encodeSlaves->slave[slaveNumber].connectionErrorTime)>60000)
					{
					encodeSlaves->slave[slaveNumber].connectionStatus =NET_NOT_CONNECTED;
					encodeSlaves->slave[slaveNumber].attemptedLongRetrys++;
					}						
				return(false);

			case NET_PERMANENTLYDOWN:
				return(false);
			}
		}
	return(false);

}

// *****************************************************************************
// netMasterClass::sendJob()
//
// *****************************************************************************
bool netMasterClass::sendJob(int slaveNumber, CONVERTCONFIG * settings, GATHERRESULTS * scan)
{
	NETSTARTCONVERSION * outputData;

#ifdef USEBUILTINSLAVE
	// Early out if we're using a fake slave
	if(useBuiltinSlave)
		{
		return(startFakeSlaveConversion(settings, scan));
		}
#endif
	
	outputData = new NETSTARTCONVERSION [1];
	outputData->command = netStartConversion; 
	outputData->size = sizeof(NETSTARTCONVERSION)-NET_COMMAND_HEADER_SIZE;
	memcpy(&outputData->settings,settings,sizeof(CONVERTCONFIG));
	memcpy(&outputData->scan,scan,sizeof(GATHERRESULTS));

	// If Codec is open - Close it
	if(settings->videoFormatOut->hic)
		{		
		ICClose(settings->videoFormatOut->hic);
		settings->videoFormatOut->hic = NULL;
		}

	if(settings->videoFormatOutP2->hic)
		{		
		ICClose(settings->videoFormatOutP2->hic);
		settings->videoFormatOutP2->hic = NULL;
		}

	// Now flatten the video codec settings
	memcpy(&outputData->settingsVideoFormatOut,settings->videoFormatOut,sizeof(COMPVARS));
	memcpy(&outputData->settingsVideoFormatOutLpbiOut,settings->videoFormatOut->lpbiOut, sizeof(BITMAPINFO));
	memcpy(&outputData->settingsVideoFormatOutLpState,settings->videoFormatOut->lpState, settings->videoFormatOut->cbState);

	// Now flatten the second pass video codec settings
	memcpy(&outputData->settingsVideoFormatOutP2,settings->videoFormatOutP2,sizeof(COMPVARS));
	memcpy(&outputData->settingsVideoFormatOutP2LpbiOut,settings->videoFormatOutP2->lpbiOut, sizeof(BITMAPINFO));
	memcpy(&outputData->settingsVideoFormatOutP2LpState,settings->videoFormatOutP2->lpState, settings->videoFormatOutP2->cbState);

	// And flatten the audio codec settings
	memcpy(&outputData->settingsAudioFormatOut,settings->audioFormatOut,AUDIO_STATE_DEFAULT_SIZE);

	if(testConnection(slaveNumber))
		{
		// Connection is known good now
		logPrintf("Sending Job to Slave %d\n", slaveNumber);

		if(safeSendBytes(slaveNumber, (char *) outputData, sizeof(NETSTARTCONVERSION)))
			{
			// Command was sent OK
			logPrintf("Finished Sending Job to Slave %d\n", slaveNumber);
			delete [] outputData;
			return(true);
			}

		}
	
	delete [] outputData;
	return(false);
}

// *****************************************************************************
// netMasterClass::getStatus()
//
// *****************************************************************************
int netMasterClass::getStatus(int slaveNumber)
{
	NETSTATUS * outputData;
	NETSTATUSRETURN inputData;

#ifdef USEBUILTINSLAVE
	// Early out if we're using a fake slave
	if(useBuiltinSlave)
		{	
		globals->conversion->getConversionStats(&encodeSlaves->slave[slaveNumber].conversionStats);
		EnterCriticalSection(&conversionStatusLock);
		encodeSlaves->slave[slaveNumber].encodeStatus = conversionStatus;
		LeaveCriticalSection(&conversionStatusLock);
		return(true);
		}
#endif
	outputData = new NETSTATUS [1];
	outputData->command = netStatus; 
	outputData->size = sizeof(NETSTATUS)-NET_COMMAND_HEADER_SIZE;

	
	if(testConnection(slaveNumber))
		{
		// Connection is known good now
		//logPrintf("Getting Status from Slave %d\n", slaveNumber);

		if(safeSendBytes(slaveNumber, (char *) outputData, sizeof(NETSTATUS)))
			{
			if(safeReceiveBytes(slaveNumber, (char *) &inputData, sizeof(NETSTATUSRETURN)))
				{
				memcpy(&encodeSlaves->slave[slaveNumber].conversionStats,&inputData.conversionStats,sizeof(CONVERSIONSTATS));
				encodeSlaves->slave[slaveNumber].encodeStatus = inputData.status;
				delete [] outputData;
				//logPrintf("Finished Getting Status from Slave %d\n", slaveNumber);

				return(encodeSlaves->slave[slaveNumber].encodeStatus);
				}
			}

		}
	return(-1);

}

// *****************************************************************************
// netMasterClass::getAbort()
//
// *****************************************************************************
int netMasterClass::sendAbort(int slaveNumber)
{
	NETABORT * outputData;


#ifdef USEBUILTINSLAVE
	// Early out if we're using a fake slave
	if(useBuiltinSlave)
		{	
		globals->conversion->abortConversion = true;
		return(true);
		}
#endif
	outputData = new NETABORT [1];
	outputData->command = netAbort; 
	outputData->size = sizeof(NETABORT)-NET_COMMAND_HEADER_SIZE;
	

	if(testConnection(slaveNumber))
		{
		// Connection is known good now
		logPrintf("Sending Abort To Slave %d\n", slaveNumber);

		if(safeSendBytes(slaveNumber, (char *) outputData, sizeof(NETABORT)))
			{
			logPrintf("Finished Sending Abort To Slave %d\n", slaveNumber);
			return(true);
			}

		}
	return(false);

}

// *****************************************************************************
// netMasterClass::sendPause()
//
// *****************************************************************************
int netMasterClass::sendPause(int slaveNumber)
{
	NETPAUSE * outputData;

#ifdef USEBUILTINSLAVE
	// Early out if we're using a fake slave
	if(useBuiltinSlave)
		{	
		globals->conversion->pauseConversion = true;
		return(true);
		}
#endif

	outputData = new NETPAUSE [1];
	outputData->command = netPause; 
	outputData->size = sizeof(NETPAUSE)-NET_COMMAND_HEADER_SIZE;
	
	if(testConnection(slaveNumber))
		{
		// Connection is known good now
		logPrintf("Sending Pause to Slave %d\n", slaveNumber);

		if(safeSendBytes(slaveNumber, (char *) outputData, sizeof(NETPAUSE)))
			{
			logPrintf("Finished Sending Pause to Slave %d\n", slaveNumber);
			return(true);
			}

		}
	return(false);

}

// *****************************************************************************
// netMasterClass::sendResume()
//
// *****************************************************************************
int netMasterClass::sendResume(int slaveNumber)
{
	NETRESUME * outputData;

#ifdef USEBUILTINSLAVE
	// Early out if we're using a fake slave
	if(useBuiltinSlave)
		{	
		globals->conversion->pauseConversion = false;
		return(true);
		}
#endif

	outputData = new NETRESUME [1];
	outputData->command = netResume; 
	outputData->size = sizeof(NETRESUME)-NET_COMMAND_HEADER_SIZE;
	
	if(testConnection(slaveNumber))
		{
		// Connection is known good now
		logPrintf("Sending Resume to Slave %d\n", slaveNumber);

		if(safeSendBytes(slaveNumber, (char *) outputData, sizeof(NETRESUME)))
			{
			logPrintf("Finished Sending Resume to Slave %d\n", slaveNumber);
			return(true);
			}

		}
	return(false);

}

// *****************************************************************************
// netMasterClass::sendSleep()
//
// *****************************************************************************
int netMasterClass::sendSleep(int slaveNumber, int sleep)
{
	NETSLEEP * outputData;

#ifdef USEBUILTINSLAVE
	// Early out if we're using a fake slave
	if(useBuiltinSlave)
		{	
		globals->conversion->sleepPerFrame = sleep;
		return(true);
		}
#endif

	outputData = new NETSLEEP [1];
	outputData->command = netSleep; 
	outputData->size = sizeof(NETSLEEP)-NET_COMMAND_HEADER_SIZE;
	outputData->sleepValue = sleep;

	if(testConnection(slaveNumber))
		{
		// Connection is known good now
		logPrintf("Sending Sleep to Slave %d\n", slaveNumber);

		if(safeSendBytes(slaveNumber, (char *) outputData, sizeof(NETSLEEP)))
			{
			logPrintf("Finished Sending Sleep to Slave %d\n", slaveNumber);
			return(true);
			}

		}
	return(false);
}

// *****************************************************************************
// netMasterClass::sendThreadPriority()
//
// *****************************************************************************
int netMasterClass::sendThreadPriority(int slaveNumber, int threadPriority)
{
	NETTHREADPRIORITY * outputData;

#ifdef USEBUILTINSLAVE
	// Early out if we're using a fake slave
	if(useBuiltinSlave)
		{	
		SetThreadPriority(fakeSlaveConversionThreadHandle,threadPriority);
		SetThreadPriority(globals->conversion->writeThreadHandle,threadPriority);
		return(true);
		}
#endif

	outputData = new NETTHREADPRIORITY [1];
	outputData->command = netThreadPriority; 
	outputData->size = sizeof(NETTHREADPRIORITY)-NET_COMMAND_HEADER_SIZE;
	outputData->threadPriority = threadPriority;

	if(testConnection(slaveNumber))
		{
		// Connection is known good now
		logPrintf("Sending Set Thread Priority to Slave %d\n", slaveNumber);

		if(safeSendBytes(slaveNumber, (char *) outputData, sizeof(NETTHREADPRIORITY)))
			{
			logPrintf("Finished Sending Set Thread Priority to Slave %d\n", slaveNumber);
			return(true);
			}

		}
	return(false);
}

// *****************************************************************************
// netMasterClass::getError()
//
// *****************************************************************************
int netMasterClass::getError(int slaveNumber, char * errorBuffer )
{
	NETERROR * outputData;
	NETERRORRETURN inputData;

#ifdef USEBUILTINSLAVE
	// Early out if we're using a fake slave
	if(useBuiltinSlave)
		{	
		strcpy(errorBuffer, globals->errorText);
		return(true);
		}
#endif

	outputData = new NETERROR [1];
	outputData->command = netError; 
	outputData->size = sizeof(NETERROR)-NET_COMMAND_HEADER_SIZE;
	
	if(testConnection(slaveNumber))
		{
		// Connection is known good now
		logPrintf("Getting Error From Slave %d\n", slaveNumber);

		if(safeSendBytes(slaveNumber, (char *) outputData, sizeof(NETERROR)))
			{
			if(safeReceiveBytes(slaveNumber, (char *) &inputData, sizeof(NETERRORRETURN)))
				{
				strcpy(errorBuffer, inputData.errorText);
				delete [] outputData;
				logPrintf("Finished Getting Error From Slave %d\n", slaveNumber);
				return(true);
				}
			}

		}
	return(false);

}
// *****************************************************************************
// netMasterClass::sendClose()
//
// *****************************************************************************
int netMasterClass::sendClose(int slaveNumber)
{
	NETCLOSE * outputData;

#ifdef USEBUILTINSLAVE
	// Early out if we're using a fake slave
	if(useBuiltinSlave)
		{	
		return(true);
		}
#endif

	outputData = new NETCLOSE [1];
	outputData->command = netClose; 
	outputData->size = sizeof(NETCLOSE)-NET_COMMAND_HEADER_SIZE;
	
	if(testConnection(slaveNumber))
		{
		// Connection is known good now
		logPrintf("Sending Close To Slave %d\n", slaveNumber);

		if(safeSendBytes(slaveNumber, (char *) outputData, sizeof(NETCLOSE)))
			{
			delete [] outputData;
			return(true);
			}

		}

	return(false);
}
// *****************************************************************************
// netMasterClass::safeSendBytes()
//
// *****************************************************************************
bool netMasterClass::safeSendBytes(int slaveNumber, char * ptr, int bytesRemaining)
{

	if(!tcp->safeSendBytes(encodeSlaves->slave[slaveNumber].commSocket,ptr,bytesRemaining))
		{
		encodeSlaves->slave[slaveNumber].connectionStatus = NET_FAILED;
		encodeSlaves->slave[slaveNumber].connectionErrorTime = timeGetTime(); 
		// close socket and retry connection
		logPrintf("Master timed out communicating with Slave %d in safeSendBytes\n",slaveNumber);	
		return(false);
		}
	
	return(true);

}

// *****************************************************************************
// netMasterClass::safeReceiveBytes()
//
// *****************************************************************************
bool netMasterClass::safeReceiveBytes(int slaveNumber, char * ptr, int bytesRemaining)
{

	if(!tcp->safeReceiveBytes(encodeSlaves->slave[slaveNumber].commSocket,ptr,bytesRemaining))
		{
		encodeSlaves->slave[slaveNumber].connectionStatus = NET_FAILED;
		encodeSlaves->slave[slaveNumber].connectionErrorTime = timeGetTime(); 
		logPrintf("Master timed out communicating with Slave %d in safeReceiveBytes\n",slaveNumber);	
		return(false);
		}

	return(true);

}
#ifdef USEBUILTINSLAVE

// *****************************************************************************
//  netMasterClass::startConversion()
//
// *****************************************************************************
bool netMasterClass::startFakeSlaveConversion(CONVERTCONFIG * settings, GATHERRESULTS * scan)
{
	FAKESLAVECONVERSIONTHREAD * fakeSlaveConversionData;
	int status;

	fakeSlaveThreadTerminated = false;
	// We have the command Data now. Perform the command
	fakeSlaveConversionData = new FAKESLAVECONVERSIONTHREAD [1];
	fakeSlaveConversionData->master = this;
	fakeSlaveConversionData->settings = settings;
	fakeSlaveConversionData->scan = scan;

	EnterCriticalSection(&conversionStatusLock);
	status = conversionStatus;
	if(status != netConversionStatusWorking)
		{
		conversionStatus = netConversionStatusWorking;
		}
	LeaveCriticalSection(&conversionStatusLock);

	// If the status wasn't idle, then the command is bogus - ignore it
	if(status == netConversionStatusWorking)
		{
		return(false);
		}


	// Create the work thread
	fakeSlaveConversionThreadHandle = CreateThread(NULL,0,	fakeSlaveConversionThread ,(LPVOID)fakeSlaveConversionData , 0 ,&fakeSlaveConversionThreadId);	
	SetThreadPriority(fakeSlaveConversionThreadHandle,THREAD_PRIORITY_NORMAL);


	return(true);
}

// *****************************************************************************
//  netMasterClass::conversionThread()
//
// *****************************************************************************
DWORD WINAPI netMasterClass::fakeSlaveConversionThread(LPVOID lParam)
{
	FAKESLAVECONVERSIONTHREAD * fakeSlaveConversionData;
	bool ret;

	fakeSlaveConversionData = (FAKESLAVECONVERSIONTHREAD *) lParam;
	
	// Clear the errorText buffer
	strcpy(globals->errorText,"");
	ret = fakeSlaveConversionData->master->fakeSlaveConversion(fakeSlaveConversionData->settings, fakeSlaveConversionData->scan);

	// Give the Slave some time to settle
	Sleep(1000);

	EnterCriticalSection(&fakeSlaveConversionData->master->conversionStatusLock);
	if(ret)
		{
		// The encode completed normally
		fakeSlaveConversionData->master->conversionStatus = netConversionStatusDone;
		}
	else
		{
		// An error occured during the encode
		fakeSlaveConversionData->master->conversionStatus = netConversionStatusError;
		}
	LeaveCriticalSection(&fakeSlaveConversionData->master->conversionStatusLock);

	fakeSlaveConversionData->master->fakeSlaveThreadTerminated = true;

	delete [] fakeSlaveConversionData;

	ExitThread(0);   
	return(0);
		
}

// *****************************************************************************
// netSlaveClass::conversion()
//
// Returns TRUE if success
// *****************************************************************************
bool netMasterClass::fakeSlaveConversion(CONVERTCONFIG * settings, GATHERRESULTS * scan)
{
	return(globals->conversion->startSingleConversion(settings, scan));

}
#endif
