// *****************************************************************************
//
//
// serverLoop.cpp
// 
//
// *****************************************************************************

#include "es.h"

extern GLOBALS * globals;

// *****************************************************************************
// serverLoopClass::serverLoopClass()
//
// serverLoop class constructor
// *****************************************************************************
serverLoopClass::serverLoopClass()
{

	serverLoopThreadHandle = NULL;
}

// *****************************************************************************
// serverLoopClass::~serverLoopClass()
//
// serverLoop class destructor
// *****************************************************************************
serverLoopClass::~serverLoopClass()
{

}

// *****************************************************************************
// serverLoopClass::begin()
//
// *****************************************************************************
bool serverLoopClass::begin()
{
	terminateServerLoop = false;
	serverLoopThreadTerminated = false;
	serverLoopThreadHandle = CreateThread(NULL,0,	serverLoopThread ,(LPVOID)this , 0 ,&serverLoopThreadId);	
	SetThreadPriority(serverLoopThreadHandle,THREAD_PRIORITY_NORMAL);

	return(true);
}

// *****************************************************************************
// serverLoopClass::end()
//
//
// *****************************************************************************
void serverLoopClass::end()
{
	int i;

	// end the server loop
	terminateServerLoop = true;
	for(i=0;i<100;i++)
		{
		if(serverLoopThreadTerminated)
			break;
		Sleep(100);
		}
	
}

// *****************************************************************************
// serverLoopClass::serverLoopThread()
//
// Returns TRUE if success
// *****************************************************************************

DWORD WINAPI serverLoopClass::serverLoopThread(LPVOID lParam)
{
	serverLoopClass * server;
	int ret;

	server = (serverLoopClass *) lParam;
	
	ret = server->serverLoop();

	server->serverLoopThreadTerminated = true;

	ExitThread(ret);   
	return(ret);
}



// *****************************************************************************
// serverLoopClass::serverLoop()
//
//
// *****************************************************************************
bool serverLoopClass::serverLoop()
{
	ENCODESLAVES * slaveList;
	int i;

	// Create the slave Machine List

	slaveList = new ENCODESLAVES [sizeof(ENCODESLAVESINGLE)*globals->numSlaveEncodeMachines];
	memset(slaveList,0,sizeof(ENCODESLAVES)*globals->numSlaveEncodeMachines);

	for(i=0;i<globals->numSlaveEncodeMachines;i++)
		{
		strcpy(slaveList->slave[i].networkName,globals->slaveEncodeMachines->slave[i].machineIp);
		strcpy(slaveList->slave[i].name,globals->slaveEncodeMachines->slave[i].machineName);
		slaveList->slave[i].port = PORT_NUMBER;
		// These are floats, so init to 0.0f
		slaveList->slave[i].conversionStats.percentOfPassComplete = 0.0f;
		slaveList->slave[i].conversionStats.framesPerSecond = 0.0f;
		}

	// Create the master communication module
	netMaster = new netMasterClass();

	// Give the master the list of slaves it owns
	netMaster->begin(globals->numSlaveEncodeMachines, slaveList, false);

	// Prime the connections. 
	netMaster->checkConnections(4);
	statusOutputTimer = 1;
	uptimeStartTime = timeGetTime();

	for(;;)
		{
		serverLoopWork();
	
		if(terminateServerLoop)
			{
			break;
			}
		}

	netMaster->end();
	delete netMaster;

	delete [] slaveList;

	return(true);
}

// *****************************************************************************
// serverLoopClass::serverLoopWork()
//
//
// *****************************************************************************
void serverLoopClass::serverLoopWork()
{
	int slave;
	int jobNumber;
	int status;
	int nextJobJobNumber;
	int nextJob;
	int jobsBeingProcessed;
	int slavesAvailable;
	int jobsInQueue;
	int hours;
	int minutes;
	int seconds;
	int	secs;
	DWORD uptimeCurTime;
	CONVERSIONSTATS * slaveStats;
	CONVERSIONSTATS * statPtr;

	// Open and verify connections to slaves. 
	netMaster->checkConnections(1);
	
	jobsBeingProcessed = 0;
	slavesAvailable = 0;
	jobsInQueue = 0;;

	// Poll active encode computers for their status - Hit each slave a maximum of one time over TCP/IP
	for(slave=0;slave<globals->numSlaveEncodeMachines;slave++)
		{
		netMaster->getStatus(slave);
		status = netMaster->getEncodeStatus(slave);
		if(status == netConversionStatusWorking)
			{
			jobsBeingProcessed++;
			}
		status = netMaster->getConnectionStatus(slave);
		if(status == NET_CONNECTED)
			{
			slavesAvailable++;
			}
		}
	

	// Make a single pass over the job list
	for(jobNumber=0;jobNumber<globals->slaveJobListSize;jobNumber++)
		{
		EnterCriticalSection(&globals->slaveJobListLock);
		
		// Count the number of jobs that are not being processed
		if((!globals->slaveJobList[jobNumber].job->encodeDone) && (globals->slaveJobList[jobNumber].job->slaveAllocated==-1))
			{
			jobsInQueue++;
			}

		// Tally jobs that have completed

		if((!globals->slaveJobList[jobNumber].job->encodeDone) && (globals->slaveJobList[jobNumber].job->slaveAllocated!=-1))
			{
			
			status = netMaster->getEncodeStatus(globals->slaveJobList[jobNumber].job->slaveAllocated);
			
			// The conversion finished OK
			switch(status)
				{
				case netConversionStatusCommandFailed:
					globals->visualLog->logPrintf("Slave Machine %d has reported a status of Failed on Job %d\n",globals->slaveJobList[jobNumber].job->slaveAllocated, jobNumber);
					break;						

				case netConversionStatusDone:
					globals->visualLog->logPrintf("Slave Machine %d Reported Job %d is Done\n",globals->slaveJobList[jobNumber].job->slaveAllocated, jobNumber);
					globals->slaveJobList[jobNumber].job->encodeDone = true;
					globals->slaveJobList[jobNumber].job->slaveAllocated = -1;
					break;

				case netConversionStatusIdle:
					// Idle is an error condition since we allocated a job to this slave.
					globals->visualLog->logPrintf("Slave %d reported Idle state when it had a job allocated\n",globals->slaveJobList[jobNumber].job->slaveAllocated);
					globals->slaveJobList[jobNumber].job->slaveAllocated = -1;
					globals->slaveJobList[jobNumber].job->encodeDone = false;
					break;

				case netConversionStatusSlaveDied:
					// Idle is an error condition since we allocated a job to this slave.
					globals->visualLog->logPrintf("Slave %d stopped responding during the encode.Reassigning job\n",globals->slaveJobList[jobNumber].job->slaveAllocated);
					globals->slaveJobList[jobNumber].job->errorRetryCounter++;
					if(globals->slaveJobList[jobNumber].job->errorRetryCounter<MAX_NET_SLAVE_DROP_RETRIES)
						{
						// The job needs to be attempted again
						globals->slaveJobList[jobNumber].job->encodeDone = false;
						globals->slaveJobList[jobNumber].job->slaveAllocated = -1;
						}
					else
						{
						// The job has been attempted twice, so mark it as done and carry on.
						// Lets hope we didn't crash two slaves attempting it twice..
						globals->slaveJobList[jobNumber].job->encodeDone = true;
						globals->slaveJobList[jobNumber].job->slaveAllocated = -1;						
						}
					break;

				case netConversionStatusError:
					status = netMaster->getError(globals->slaveJobList[jobNumber].job->slaveAllocated, globals->slaveJobList[jobNumber].job->errorText);
					globals->visualLog->logPrintf("Slave %d reported Error : %s\n",globals->slaveJobList[jobNumber].job->slaveAllocated, globals->slaveJobList[jobNumber].job->errorText);
					globals->slaveJobList[jobNumber].job->errorRetryCounter++;
					if(globals->slaveJobList[jobNumber].job->errorRetryCounter<MAX_NET_ERROR_RETRIES)
						{
						// The job needs to be attempted again
						globals->slaveJobList[jobNumber].job->encodeDone = false;
						globals->slaveJobList[jobNumber].job->slaveAllocated = -1;
						}
					else
						{
						// The job has been attempted twice, so mark it as done and carry on.
						// Lets hope we didn't crash two slaves attempting it twice..
						globals->slaveJobList[jobNumber].job->encodeDone = true;
						globals->slaveJobList[jobNumber].job->slaveAllocated = -1;						
						}
					break;						
				}
				
			}
		LeaveCriticalSection(&globals->slaveJobListLock);
		}

	// Find the lowest Job that needs to be started

	EnterCriticalSection(&globals->slaveJobListLock);

	nextJob = -1;
	nextJobJobNumber = 0x7fffffff;
	for(jobNumber=0;jobNumber<globals->slaveJobListSize;jobNumber++)
		{
		if(!globals->slaveJobList[jobNumber].job->encodeDone && globals->slaveJobList[jobNumber].job->slaveAllocated==-1)
			{
			if(globals->slaveJobList[jobNumber].job->jobNumber < nextJobJobNumber)
				{
				// Is this the lowest ? 
				nextJob = jobNumber;
				nextJobJobNumber = globals->slaveJobList[jobNumber].job->jobNumber;
				}
			}
		}

	LeaveCriticalSection(&globals->slaveJobListLock);

	// And start the job 

	if(nextJob>=0)
		{
		EnterCriticalSection(&globals->slaveJobListLock);

		// Kickoff New Jobs on Machines that are free
		slave = netMaster->findFreeEncodeResource();
		
		if(slave>=0)
			{
			// Found a resource that's free
			if(netMaster->sendJob(slave,&globals->slaveJobList[nextJob].job->flatJob))
				{
				globals->slaveJobList[nextJob].job->slaveAllocated = slave;
				globals->visualLog->logPrintf("Started Job \"%s\"on Slave %d\n",&globals->slaveJobList[nextJob].job->flatJob.jobName,slave);
				}
			else
				{
				globals->visualLog->logPrintf("Slave %d failed to accept sendJob command\n",slave);
				}
			}
		LeaveCriticalSection(&globals->slaveJobListLock);
		}

	// And delay before doing it all over again
	statusOutputTimer--;
	if(!statusOutputTimer)
		{
		uptimeCurTime = timeGetTime();
		secs = (uptimeCurTime - uptimeStartTime)/1000;
		statusOutputTimer = SERVER_LOOP_STATUS_TIMER;
		hours = secs / 3600;
		minutes = (secs - (hours * 3600) ) / 60;
		seconds = secs - ((hours * 3600) + (minutes * 60));

		globals->visualLog->logPrintf("[%.2d:%.2d:%.2d] : %d Slaves Available / %d Slaves Running Jobs / %d Jobs In Queue\n",
			hours, minutes, seconds, slavesAvailable, jobsBeingProcessed, jobsInQueue);
		}

	statPtr = globals->slaveEncodeStats;

	// and last Build up the stats for anyone that asks
	EnterCriticalSection(&globals->slaveEncodeStatsLock);
	for(slave=0;slave<globals->numSlaveEncodeMachines;slave++)
		{
		slaveStats = netMaster->getEncodeStats(slave);
		memcpy(statPtr,slaveStats,sizeof(CONVERSIONSTATS));
		statPtr++;
		}
	globals->jobsBeingProcessed = jobsBeingProcessed;
	globals->slavesAvailable = slavesAvailable;
	globals->jobsInQueue = jobsInQueue;
	LeaveCriticalSection(&globals->slaveEncodeStatsLock);
	Sleep(1000);
}