// *****************************************************************************
//
//
// visualLog.cpp
// 
//
// *****************************************************************************

#include "es.h"

extern GLOBALS * globals;

// *****************************************************************************
// visualLogClass::visualLogClass()
//
// visualLog class constructor
// *****************************************************************************
visualLogClass::visualLogClass()
{
	int i;

	for(i=0;i<MAX_VISUAL_LOG_LINE;i++)
		{
		textScreen[i] = NULL;
		}
	curLine = 0;
	InitializeCriticalSection(&screenLock);
	font = CreateFont(17, 0, 0, 0, FW_NORMAL , FALSE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, "MS Sans Serif");

}

// *****************************************************************************
// visualLogClass::~visualLogClass()
//
// visualLog class destructor
// *****************************************************************************
visualLogClass::~visualLogClass()
{
	int i;

	for(i=0;i<MAX_VISUAL_LOG_LINE;i++)
		{
		if(textScreen[i])
			{
			delete [] textScreen[i];
			}
		}

}

// *****************************************************************************
// visualLogClass::logPrintf()
//
// visualLog printf
// *****************************************************************************
void visualLogClass::logPrintf(char *format, ...)
{
	char stringOut[2048];
	int i;
	char * ptr;
	va_list va_arg;

	va_start(va_arg, format);

	vsprintf(stringOut, format, va_arg);

	va_end(va_arg);

	// Strip off any CR/LF 
	ptr = strrchr(stringOut,0x0a);
	if(ptr)
		{
		*ptr = 0;
		}
	ptr = strrchr(stringOut,0x0d);
	if(ptr)
		{
		*ptr = 0;
		}

	EnterCriticalSection(&screenLock);
	if(curLine==MAX_VISUAL_LOG_LINE)
		{
		delete [] textScreen[0];

		for(i=1;i<MAX_VISUAL_LOG_LINE;i++)
			{
			textScreen[i-1] = textScreen[i];
			}
		textScreen[i-1] = NULL;
		curLine--;
		}

	textScreen[curLine] = new char [strlen(stringOut)+1];
	strcpy(textScreen[curLine], stringOut);	

	curLine++;
	LeaveCriticalSection(&screenLock);
	InvalidateRect(globals->hWnd,NULL,FALSE);
	UpdateWindow(globals->hWnd);

}

// *****************************************************************************
// visualLogClass::paintWindow()
//
// visualLog printf
// *****************************************************************************
void visualLogClass::paintWindow(HDC hDC, RECT * paintWindow)
{
	SIZE size;
	int vertLines;
	int outPos;
	int yPos;
	RECT window;
	
	GetClientRect(globals->hWnd,&window);

	FillRect(hDC,&window,(HBRUSH) GetStockObject(BLACK_BRUSH));
	SelectObject(hDC,font);

	GetTextExtentPoint32(hDC,"ABC",3,&size);		
	
	SetBkMode(hDC, TRANSPARENT);
	SetTextColor(hDC, 0x00ffffff);
	
	vertLines = (window.bottom - window.top) / size.cy;
	
	EnterCriticalSection(&screenLock);
	if(curLine >vertLines)
		{
		outPos = curLine - vertLines;		
		}
	else
		{
		outPos = 0;
		}

	yPos = 0;
	
	for(;outPos<curLine;outPos++)
		{
		ExtTextOut(hDC,0,yPos,ETO_CLIPPED,&window,textScreen[outPos],strlen(textScreen[outPos]),NULL);
		yPos+=size.cy+1;
		}
	LeaveCriticalSection(&screenLock);

}